/**
 * This is the desktop. The html code of the page is organized in three parts. On top there is a top menu,
 * in the middle there is the desktop itself where the windows can be moved around, on the bottom there is 
 * a statusbar.  
 */
 
 
// constants


/**
 * The event listened to for a button being added to the status bar. When the status bar has changed height 
 * we need to change the height of the desktop. 
 */
var STATUSBAR_BUTTON_ADDED_EVENT = "statusbarButtonAdded";


/**
 * The event listened to for a button removed from the status bar. When the status bar has changed height 
 * we need to change the height of the desktop. 
 */
var STATUSBAR_BUTTON_REMOVED_EVENT = "statusbarButtonRemoved";


// construction



/**
 * The single instance of the desktop.
 */
Desktop.instanceObject = new Desktop();


/**
 * Constructs a instance of this class. 
 */
function Desktop() {

	
	/**
	 * The html code of top menu. 
	 */
	this.topMenuHtmlCode;
	
	
	/**
	 * The status bar object used. 
	 */
	this.statusbar;
	
	
	/**
	 * The html code of the desktop itself. 
	 */
	this.desktopHtmlCode;
	
	
	/**
	 * The event listener used to listen to events.
	 */
	this.eventProcessor = new Listener(this);
	this.eventProcessor.addEvent(STATUSBAR_BUTTON_ADDED_EVENT, processStatusButtonAddedEvent);
	this.eventProcessor.addEvent(STATUSBAR_BUTTON_REMOVED_EVENT, processStatusButtonRemovedEvent);
}


/**
 * Gets the single instance of the desktop in use. 
 */
Desktop.instance = function() {
	return this.instanceObject;
}


// services


/**
 * Set the html code of the top menu used.  
 * @param menu The top menu to set. 
 * @throws Error if menu is undefined or null.
 */
Desktop.prototype.setTopMenu = function(menu) {
	assertExists(menu, "The menu to set does not exist");
	this.topMenuHtmlCode = menu;
}


/**
 * Set the status bar used. 
 * @param statusbar The status bar object used. 
 * @throws Error if statusbar is undefined or null.
 */
Desktop.prototype.setStatusbar = function(statusbar) {
	assertExists(statusbar, "The status bar to set is undefined or null");
	this.statusbar = statusbar;
}


/**
 * Set the html code of the desktop itself. 
 * @param desktop The desktop code to set. 
 * @throws Error if desktop is undefined or null.
 */
Desktop.prototype.setDesktop = function(desktop) {
	assertExists(desktop, "The desktop to set is undefined or null");
	this.desktopHtmlCode = desktop;
}


/**
 * Sets the height of the desktop according to the height of the window, top menu and statusbar. 
 */
Desktop.prototype.setDesktopHeight = function() {
	var windowHeight = isIE() ? window.document.documentElement.offsetHeight : getWindowHeight(window);
	var height = windowHeight - this.topMenuHtmlCode.offsetHeight - Statusbar.instance().getHeight();
	if (height > 0 && height != parseInt(this.desktopHtmlCode.style.height)) {
		this.desktopHtmlCode.style.height = height + "px";
	}
	if (isSafari() && windowHeight != getWindowHeight(window)) {
		// it looks like safari may need some time to update the height of the window, therefore we recalculate the
		// height of the window to check it has not changed, if it has reset the height of the desktop
		var height = getWindowHeight(window) - this.topMenuHtmlCode.offsetHeight - Statusbar.instance().getHeight();
		if (height > 0 && height != parseInt(this.desktopHtmlCode.style.height)) {
			this.desktopHtmlCode.style.height = height + "px";
		}
	}
}


/**
 * Gets the html code of the desktop.
 * @return The html code of the desktop.
 */
Desktop.prototype.getDesktopCode = function() {
	return this.desktopHtmlCode;
}


// event processing


/**
 * Processes a status bar button added event. 
 * @param event The event to process. 
 */
function processStatusButtonAddedEvent(event) {
	this.setDesktopHeight();
}


/**
 * Processes a status bar button removed event. 
 * @param event The event to process. 
 */
function processStatusButtonRemovedEvent(event) {
	this.setDesktopHeight();
}
