/** 
 * Classes can listen events through a instance of this class.
 */ 
 
/**
 * Constructs the event listener.
 * @param object Optional argument, The object to call the handler functions on when a event is fired. 
 */
function Listener(object) {
	this.object = object;
	getEventServer().addListener(this);
	
	// this array holds the events to process
	this.eventsToListenTo = new Array();
}


// servicces


/**
 * Add a event to listen to.
 * @param event The event name.
 * @param functionToCall The function to call for the event. 
 * @throws Error if event is null,  undefined or empty.
 * @throws Error if function is null or undefined. 
 */
Listener.prototype.addEvent = function(event, functionToCall) {
	assertNotEmpty(event, "Event name is undefined, null or empty: " + event);
	assertDefined(functionToCall, "The function to call is undefined or null");
	
	this.eventsToListenTo[event] = functionToCall;
}	


/**
 * Removes a event to listen to.
 * @param event The evnet name.
 * @throws Error if event is null, undefined or empty.
 */
Listener.prototype.removeEvent = function(event){
	assertNotEmpty(event, "Event name is undefined, null or empty: " + event);
	this.eventsToListenTo[event] = null;
	this.eventsToListenTo = removeNullEntries(this.eventsToListenTo);
}


/**
 * Stop listening to events. 
 */
Listener.prototype.stopListening = function() {
	getEventServer().removeListener(this);
}

/**
 * Starts listening again after the listening has been stopped.
 */
Listener.prototype.startListening = function() {
	if (!getEventServer().isListener(this)) {
		getEventServer().addListener(this);
	}
}


/**
 * Change the object to call the event handler fucntions on.
 * @param object The object to call the event handler fucntions on. 
 * @throws Error if object is null or undefined. 
 */
Listener.prototype.changeObject = function(object) {
	assertDefined(object, "Object is undefined or empty");
	
	this.object = object;
}


/**
 * Returns true if the listener is listening to a given event, false otherwise.
 * @param event The event to check.
 */
Listener.prototype.listeningToEvent = function(event) {
	assertExists(event, "The event to check does not exist");
	
	return exists(this.eventsToListenTo[event.getEvent()]);
}


// event processing


/**
 * Called by the event server to notify about events.
 * @param event The event that has happened.
 * @throws Error if the event is undefined.
 */
Listener.prototype.update = function(event) {
	assertDefined(event, "The event is undefined");
	// process the event if a event processor for the event exists
	var eventProcessorFunction = this.eventsToListenTo[event.getEvent()];
	if (!exists(eventProcessorFunction)) {
		return;
	}
	
	if (exists(this.object)) {
		eventProcessorFunction.call(this.object, event);	
	}	
	else {
		eventProcessorFunction.call(null, event);	
	}
}