/**
 * This class shows action details.
 */
 
 
// constants


/**
 * The base url used to edit a alarm action.
 */
var EDIT_ACTION_BASE_URL = "/?resource=AlarmManager&mode=set&command=alarmAction&action=edit&sid=" + getSid() + "&";


/**
 * The base url used to edit a library alarm action.
 */
var EDIT_LIBRARY_EMAIL_ACTION_BASE_URL = "/?resource=AlarmManager&mode=set&command=libraryAction&action=edit&sid=" + getSid() + "&";


// construction


/**
 * The class.
 * @param setActionBaseUrl The base url used to set a email alarm action. 
 * @param editActionBaseUrl The base url used to edit a email alarm action.
 * @param addToLibraryUrl The base url used to add a action to the action library.  
 * @param htmlCode The html code containing the email detail section.
 * @throws Error if setActionBaseUrl is undefined, null or empty. 
 * @throws Error if editActionBaseUrl is undefined, null or empty. 
 * @throws Error if htmlCode is null or undefined. 
 */
function ActionDetail(setActionBaseUrl, editActionBaseUrl, addToLibraryBaseUrl, htmlCode) {
	assertNotEmpty(setActionBaseUrl, "The set action base url is undefined, null or empty", setActionBaseUrl);
	assertNotEmpty(editActionBaseUrl, "The edit action base url is undefined, null or empty", editActionBaseUrl);
	assertExists(htmlCode, "The html code to set is null or undefined");
	this.htmlCode = htmlCode;
	this.setActionBaseUrl = setActionBaseUrl;
	this.editActionBaseUrl = editActionBaseUrl;
	this.addToLibraryBaseUrl = addToLibraryBaseUrl;
	this.actionId = getInput(this.htmlCode, "actionId");
}


// services


/**
 * Sets a alarm action based on the values entered in the alarm actions window. 
 */
ActionDetail.prototype.setAlarmAction = function() {
	var url = this.createSetAlarmActionUrl();
	if (!isNull(url)) {
		loadXML(url, "alarmActionSet", this);
	}
}


/**
 * Send a edit alarm request based on the value in the alarm actions window. 
 */
ActionDetail.prototype.editAlarmAction = function() {
	var url = this.createEditAlarmActionUrl();
	if (!isNull(url)) {
		loadXML(url, "alarmActionEdited", this);
	}
}


/**
 * Adds the values in the add/edit alarm actions window to the alarm action library.
 */
ActionDetail.prototype.addToLibrary = function() {
	var url = this.createAddToLibraryUrl();
	if (!isNull(url)) {
		loadXML(url, "addedToLibrary", this);
	}
}


/**
 * Adds the values in a give data set to the alarm action library.
 * @param dataSet The data set whos value to add to the library.
 * @throws Error if dataSet is undefined null or empty.
 */
ActionDetail.prototype.addToLibraryDataSet = function(dataSet) {
	assertExists(dataSet, "The dataSet to add to the library is undefined null or empty");
	var url = this.createAddToLibraryUrlDataSet(dataSet);
	if (!isNull(url)) {
		loadXML(url, "addedToLibrary", this);
	}
}


/**
 * Set a action id.  
 * @param id The id to set. 
 * @throws Error if id is undefined, null or empty. 
 */
ActionDetail.prototype.setActionId = function(id) {
	assertNotEmpty(id, "The id to set is undefined, null or empty");
	this.actionId.value = id;
}


/**
 * Shows the edit alarm action button.
 */
ActionDetail.prototype.showEditActionButton = function() {
	var setButton = getElementWithAttribute(this.getHtmlCode(), "setAlarmAction", "name", "input");
	setClass(setButton, "inactive");
	var cloneButton = getElementWithAttribute(this.getHtmlCode(), "cloneAlarmAction", "name", "input");
	setClass(cloneButton, "active");
	var editButton = getElementWithAttribute(this.getHtmlCode(), "editAlarmAction", "name", "input");
	setClass(editButton, "active");
	var addToLibraryButton = getElementWithAttribute(this.getHtmlCode(), "addToLibraryButton", "name", "input");
	if (exists(addToLibraryButton)) {
		setClass(addToLibraryButton, "active");
	}
}


/**
 * Show set alarm action button.
 */
ActionDetail.prototype.showSetActionButton = function() {
	var setButton = getElementWithAttribute(this.getHtmlCode(), "setAlarmAction", "name", "input");
	setClass(setButton, "active");
	var cloneButton = getElementWithAttribute(this.getHtmlCode(), "cloneAlarmAction", "name", "input");
	setClass(cloneButton, "inactive");
	var editButton = getElementWithAttribute(this.getHtmlCode(), "editAlarmAction", "name", "input");
	setClass(editButton, "inactive");
	var addToLibraryButton = getElementWithAttribute(this.getHtmlCode(), "addToLibraryButton", "name", "input");
	if (exists(addToLibraryButton)) {
		setClass(addToLibraryButton, "inactive");
	}
}


/**
 * Enter has been pressed in one of the alarm action input fields. 
 */
ActionDetail.prototype.enterPressedAlarmAction = function() {
	var editButton = getElementWithAttribute(this.getHtmlCode(), "editAlarmAction", "name", "input");
	if ("active" == getClassName(editButton)) {
		this.editAlarmAction();
	}
	else {
		this.setAlarmAction();
	}
}


/**
 * Shows the correct options in the variable selector according to a given table.
 * @param table The table to show the options for. 
 */
ActionDetail.prototype.showVariableOptions = function(table) {
	var selector = getElementWithAttribute(this.getHtmlCode(), "variableSelector", "name", "select");
	deleteOptions(selector);
	selector.add(new Option(DEFAULT_VARIABLE_SELECT_OPTION, "0"), undefined);
	var variables = tableVariables[table];
	
	for (var i = 0; i < variables.length; i++) {
		// the default option has value 0
		selector.add(new Option(variables[i], i + 1), undefined);
	}
}


/**
 * Gets the value of the alarm id field. 
 * @return The value of the alarm id field, a empty string if no alarm id set.  
 */
ActionDetail.prototype.getAlarmId = function() {
	var alarmId = getInput(this.getHtmlCode(), "alarmId"); 
	return exists(alarmId) ? alarmId.value : ""; 
}


// implementation


/**
 * Creates the url to set a alarm action based on the values set in the alarm action window. 
 */
ActionDetail.prototype.createSetAlarmActionUrl = function() {
	try {
		return this.setActionBaseUrl + this.createActionParameterUrl();
	}
	catch (e) {
		this.setMessage(undefined, e.message);
		return null;	
	}
}


/**
 * Creates the url to edit a alarm action based on the values set in the alarm action window. 
 */
ActionDetail.prototype.createEditAlarmActionUrl = function() {
	try {
		var actionId = getInput(this.htmlCode, "actionId");
		return this.editActionBaseUrl + this.createActionParameterUrl() + "actionId=" + actionId.value;
	}
	catch (e) {
		this.setMessage(undefined, e.message);
		return null;	
	}
}


/**
 * Creates the url to add a action to the action library using the data in the action detail fields. 
 */
ActionDetail.prototype.createAddToLibraryUrl = function() {
	try {
		var actionId = getInput(this.getHtmlCode(), "actionId");
		return this.addToLibraryBaseUrl + this.createActionParameterUrl();
	}
	catch (e) {
		this.setMessage(undefined, e.message);
		return null;	
	}
}


/**
 * Creates the add to library url for a given data set. 
 * @param dataSet The data set to create the data for.
 * @throws Error if dataSet us undefined or null. 
 */
ActionDetail.prototype.createAddToLibraryUrlDataSet = function(dataSet) {
	assertExists(dataSet, "The data set to create the add to library url for is undefined or null");
	try {
		var actionId = getInput(this.getHtmlCode(), "actionId");
		return this.addToLibraryBaseUrl + this.createActionParameterUrlDataSet(dataSet);
	}
	catch (e) {
		this.setMessage(undefined, e.message);
		return null;	
	}
}


/**
 * A action has ben set/edited or added to the action library, parse the response from the
 * server and show any message.
 * @param response The response from the server.  
 */
ActionDetail.prototype.parseReply = function(response) {
	if (containsErrorMessage(response)) {
		this.setMessage(undefined, getErrorMessage(response));
	}	
	else if (containsSuccessMessage(response)) {
		this.setMessage(getSuccessMessage(response), undefined);
	}
}

/**
 * Callback method for editing a alarm action. 
 * @param response The response from the server. 
 */
ActionDetail.prototype.alarmActionEdited = function(response) {
	this.parseReply(response);
	getEventServer().notifyListeners(new Event(ACTION_EDITED, this));
}


/**
 * Callback method for setting a alarm action.
 * @param response The response from the server.
 */
ActionDetail.prototype.alarmActionSet = function(response) {
	this.parseReply(response);
	getEventServer().notifyListeners(new Event(ACTION_SET, this));
}


/**
 * Callback method for adding a alarm action to the action library.
 * @param response The response from the server. 
 */
ActionDetail.prototype.addedToLibrary = function(response) {
	this.parseReply(response);
	getEventServer().notifyListeners(new Event(ACTION_ADDED_TO_LIBRARY, null));
}


/**
 * Set the alarm action mesage field.
 * @param successMessage The success message to set. 
 * @param errorMessage The error message to set. 
 */
ActionDetail.prototype.setMessage = function(sucessMessage, errorMessage) {
	var messageContainer = getElementWithAttribute(this.getHtmlCode(), "alarmActionmessage", "name", "div");
	if (isDefined(sucessMessage) && !isEmpty(sucessMessage)) {
		setClass(messageContainer, "messageContainer ok");
		messageContainer.innerHTML = "<nobr>" + sucessMessage + "</nobr>";
	}
	else if (isDefined(errorMessage) && !isEmpty(errorMessage)) {
		setClass(messageContainer, "messageContainer error");
		messageContainer.innerHTML = "<nobr>" + errorMessage + "</nobr>";
	}
}


/**
 * Clears the alarm action message.
 */
ActionDetail.prototype.clearAlarmActionMessage = function() {
	getElementWithAttribute(this.getHtmlCode(), "alarmActionmessage", "name", "div").innerHTML = "";
}



/**
 * Gets the html code of this detail section. 
 * Returns the html code containing this action detail section. 
 */
ActionDetail.prototype.getHtmlCode = function() {
	return this.htmlCode;
}
