/**
 * This is the alarm action table. 
 */
 

// constants 


/**
 * The base url used to delete action. The action id, alarm id and the sid need to be appended 
 * to this url.
 */
var DELETE_ACTION_BASE_URL = "/?resource=AlarmManager&mode=set&command=alarmAction&action=delete&"


/**
 * The base url used to delete a library aciton. The action id and the sid need to be appended to this url.
 */
var DELETE_LIBRARY_ACTION_BASE_URL = "/?resource=AlarmManager&mode=set&command=libraryAction&action=delete&";


// construction


/**
 * Factory method to create new alarm action table objects.
 * @param id The id of the table to update.
 * @param url The url to use to update the table.
 * @param parentElement The element the table is to be inserted in. 
 * @param windowId The id of the window the table is in. 
 * @param Optional time parameter. The time between each update.
 * If this is not specified the default value is used.
 * @returns A new alarm action table object.
 * @throws Error if id is undefined, null or empty.
 * @throws Error if parentElement is undefined, or null.
 * @throws Error if windowId is undefined, null or empty.
 */
function createActionTable(id, url, parentElement, windowId, time) {
	assertNotEmpty(id, "Id for the table is undefined, empty or null: " + id);
	assertDefined(parentElement, "The element to insert the table into is undefined or null");
	assertNotEmpty(windowId, "windowId to set for the table is undefined empty or null: " + windowId);
	
	var subInstance = new ActionTable();
	var superInstance = createMyTable(id, url, parentElement, windowId, time);
	var instance = createSubClass(superInstance, subInstance);
	instance["supercreateDataTable"] = superInstance["supercreateDataTable"];
	instance["superupdateTableRow"] = superInstance["superupdateTableRow"];
	
	TableManager.instance().removeTable(TableManager.instance().getTable(id));
	TableManager.instance().registerNewTable(instance);

	// let the event processor call event handler functions as a method
	// of the mytable object instance
	superInstance.getEventProcessor().changeObject(instance);
	
	// overwrite the repeater to call the load function on the MyTable object	
	instance.repeater = new Repeater(load, instance, time);
	instance.eventListener.addEvent(WINDOW_SHOWN_EVENT, processAlarmActionWindowShownEvent);

	return instance;
}


/**
 * Subclasses mytable.js to add behaviour needed in the alarmactions table.
 */
function ActionTable() {
	// when the mouse is moved over table rows the details of the action row are shown in the
	// action detail section of the window. When the user moves the mouse out of the table
	// the values of the action detail section are restored as they where before the user moved
	// the mouse over the rows. This indicates whether the mouse is over the table. 
	this.mouseOver = false;

	// if this is set to true details of the action row the mouse is over are shown
	this.showDetails = true;
	
	this.alarmId = "";
	
	// when a row is clicked it is marked and it details are shown in the alarm detail section of the window
	// this is the value of the rows id field
	this.markedId = -1;
	
	// the class used to mark html rows
	this.markClass = "";
}


// services


/**
 * Set the class used to mark html rows. 
 * @param className The name of the class to mark html rows. 
 * @throws Error if className is undefined, null or empty.
 */
ActionTable.prototype.setMarkClass = function(className) {
	assertNotEmpty(className, "The class name to set is undefined, null or empty");
	this.markClass = className;
}
 
 
/**
 * Starts the loading of alarm actions. 
 * @param id The id of the alarm to load the actions for. 
 */
ActionTable.prototype.startLoading = function(id) {
	if (hasParameter(this.url, "alarmId", "&")) {
		this.url = replaceParameterValue(this.url, "alarmId", id, "&");
	}
	this.restartUpdate();
}


/**
 * Deletes a action on a right click on the delete menu. 
 */
ActionTable.prototype.deleteActionRightClick = function() {
	if (lastEntryFieldName != null && lastTableObject != null && lastTdNode != null) {
		var row = this.getRowFromTd(lastTdNode);
		var rowIndex = this.getRowIndex(row[0]);
		this.deleteEntry(this.getRowId(this.rows[rowIndex]));
	}	
}


// overridden from mytable.js


/**
 * Creates the url needed to delete a entry.
 * @param value The value for the id field of the row to delete. 
 * @return The url to delete the entry.
 */
ActionTable.prototype.createDeleteEntryUrl = function(value) {
	var id = getParameterValue(value, this.getIdFields()[0], "&");
	var alarmId;
	if (this.window.getId().equalsIgnoreCase("alarmactionswindow")) {
		var window = WindowManager.instance().getWindow("alarmactionswindow");
		alarmId = getInput(window.getHtmlCode(), "alarmId").value;
	}
	return this.deleteActionUrl(id, alarmId);
}


/**
 * Create delete action url.
 * @param id The id of the action to delete.
 * @param alarmId The id of the alarm the action to delete is for. 
 */
ActionTable.prototype.deleteActionUrl = function(id, alarmId) {
	if (this.id.indexOf("Library") > -1) {
		return DELETE_LIBRARY_ACTION_BASE_URL + "sid=" + getSid() + "&actionId=" + id;
	}
	return DELETE_ACTION_BASE_URL + "sid=" + getSid() + "&actionId=" + id + "&alarmId=" + (exists(alarmId) ? alarmId : "");
}



/**
 * A entry has been deleted, if the marked entry has been deleted unmark it. 
 * @param deletedRow The row that has been deleted. 
 */
ActionTable.prototype.entryDeleted = function(deletedRow) {
	if (getFieldFromRow(this.getIdFields()[0], deletedRow).getValue() == this.markedId) {
		this.unMarkRow(deletedRow);
		getEventServer().notifyListeners(new Event(MARKED_ACTION_DELETED, this));
	}
	else {
		getEventServer().notifyListeners(new Event(ACTION_DELETED, deletedRow));
	}
}


// overridden from table.js


/**
 * Updates a html row of the table according to the data in the specified data row.
 * This is overridden to update the delete link as the order of the row changes,
 * this need to be done or the delete links my delete the wrong rows.
 * @param dataRow The array containing the data to insert into the html row.
 * @param htmlRow The html row that is to be update with the data in dataRow.
 */
ActionTable.prototype.updateTableRow = function(dataRow, htmlRow) {
	var argumentsToSuper = new Array();
	argumentsToSuper[0] = dataRow;
	argumentsToSuper[1] = htmlRow;
	this.callSuper("updateTableRow", argumentsToSuper);
	
	// update the rows delete link, so the link deletes the correct entry
	var deleteLink = getElementWithAttribute(htmlRow, "deleteLink", "name", "a");
	if (exists(deleteLink)) {
		this.setDeleteLinkOnClick(deleteLink, dataRow);
	}
	
	// udpate the css class of the html row, the position of the marked row can have changed
	// when the table is updated so we might need to update the rows css class
	var index = -1;
	for (var i = 0; i < this.rows.length; i++) {
		if (dataRow == this.rows[i]) {
			index = i;
		}
	}
	this.addRowClasses(htmlRow, index);
}


/**
 * Adds the mouseover class for a row.
 * @param row The html row to set the color for.
 */
ActionTable.prototype.addRowMouseOverClass = function(row) {
	if (!isDefined(this.mouseOverClass)) {
		return;
	}

	row.counter = 0;
	var table = this;
	row.onmouseover = function(event) {
		if (table.mouseOver == false) {
			getEventServer().notifyListeners(new Event(MOUSE_MOVED_OVER_ACTION_TABLE, table));
		}
		this.counter = this.counter + 1;
		setClass(this, table.mouseOverClass);
		var row = table.htmlRow(event);
		if (row != table.rowMouseOver) {
			table.rowMouseOver = row;
			table.mouseOver = true;
			var eventData = new Array();
			eventData["row"] = table.rows[table.getHtmlRowIndex(table.htmlRow(event))];
			eventData["source"] = table;
			getEventServer().notifyListeners(new Event(MOUSE_MOVED_OVER_ACTION_ROW, eventData));	
		}
	};
	
	row.onmouseout = function(event) {
		this.counter = 0;
		if (exists(table.htmlRow(event)) && table.markedId != table.getIdFieldValue(table.htmlRow(event))) {
			setClass(this, table.getRowClass(table.getHtmlRowIndex(this)));
		}
	};
	
	row.onclick = function(event) {
		if (exists(table.htmlRow(event)) && table.markedId == table.getIdFieldValue(table.htmlRow(event))) {
			table.unMarkRow(table.getMarkedRow());
			table.showDetails = true;
			getEventServer().notifyListeners(new Event(ACTION_DESELECTED, table));
		}
		else if (exists(table.htmlRow(event))) {
			if (table.markedId != -1) {
				table.unMarkRow(table.getMarkedRow());
			}
			table.markRow(table.htmlRow(event));
			table.showDetails = false;
			var eventData = new Array();
			eventData["row"] = table.rows[table.getHtmlRowIndex(table.htmlRow(event))];
			eventData["source"] = table;
			getEventServer().notifyListeners(new Event(ACTION_SELECTED, eventData));
		}
	};
}


/**
 * Creates the data table. This is overridden from mytable to add some mouse events to the table. 
 */
ActionTable.prototype.createDataTable = function() {
	this.callSuper("createDataTable");
	
	var table = this.getTableCode();
	var tableObject = this;
	table.onmouseout = function(event) {
		event = exists(event) ? event : window.event;
		var target = (window.event) ? event.srcElement : event.target;
		var related = relatedTarget(event);
		
		var td = findTdNode(related);
		if (td != null && tableObject.getRowFromTd(td) != null) {
			// this onmouseout event occurs due to onmouseout events of the rows and cell inside the table
			return;
		}
		// the mouse has been moved outside of the table
		getEventServer().notifyListeners(new Event(MOUSE_MOVED_OUT_ACTION_TABLE, tableObject));
		tableObject.mouseOver = false;
		tableObject.showDetails = true;
		tableObject.rowMouseOver = null;
	}
}


/**
 * Set the container containing the data table to a given height. This needs to be done
 * so the add entry section can be right under the data table.
 * Overridden to not do this since there is no add entry section in the action tables. 
 * @param height The height to set in pixels. 
 */
ActionTable.prototype.setTableContainerHeight = function(height) {
	// in the super class mytable the height of the table container 
	// is adjusted depending on how many rows are in the table. This is done
	// because the add entry table should always be right under the data table.
	// in the alarm action tables there is no add entry table.
}



/**
 * Activates the no access div of this window. This is overridden to also hide the menu.
 */
ActionTable.prototype.activateNoAccess = function() {
	this.callSuper("activateNoAccess");
	// hide the menu
	var menu = getElementWithAttribute(this.window.getHtmlCode(), "tabsSection", "name", "div");
	setClass(menu, "inactive");
}



/**
 * Hides the no access for this window.
 */
ActionTable.prototype.hideNoAccess = function() {
	if (!isDefined(this.noAccess)) {
		return;
	}
	
	var noAccesHeight = this.noAccess.offsetHeight;
	setClass(this.noAccess, "inactive");
	this.noAccessShown = false;
	
	// adjust the size of the window
	this.window.htmlCode.style.height = (parseInt(this.window.htmlCode.style.height) - noAccesHeight) + "px";
	this.window.htmlCode.style.height = (parseInt(this.window.htmlCode.style.height) + this.originalTableHeight) + "px";
	
	var widthDiff = this.originalTableWidth - parseInt(this.window.htmlCode.style.width);
	this.window.htmlCode.style.width = (parseInt(this.window.htmlCode.style.width) + widthDiff) + "px";
	if (isDefined(this.originalClass) && !isNull(this.originalClass) && !isEmpty(this.originalClass)) {
		setClass(this.getTableContainerCode(), this.originalClass);
	}
	else {
		setClass(this.getTableContainerCode(), "active");
	}
	// show the menu
	var menu = getElementWithAttribute(this.window.getHtmlCode(), "tabsSection", "name", "div");
	setClass(menu, "subMenu");
}


// event processing


/**
 * The right click menu choice "Edit alarm action" has been clicked. Mark and show the clicked actions details.
 */
ActionTable.prototype.editAction = function() {
	if (lastEntryFieldName != null && lastTableObject != null && lastTdNode != null) {
		var htmlRow = this.getHtmlRowFromTd(lastTdNode);
		if (this.markedId == this.getIdFieldValue(htmlRow)) {
			this.unMarkRow(this.getMarkedRow());
			this.showDetails = true;
			getEventServer().notifyListeners(new Event(ACTION_DESELECTED, this));
		}
		else {
			this.markRow(htmlRow);
			this.showDetails = false;
			var eventData = new Array();
			eventData["row"] = this.rows[this.getHtmlRowIndex(htmlRow)];
			eventData["source"] = this;
			getEventServer().notifyListeners(new Event(ACTION_SELECTED, eventData));
		}
	}	
}


/**
 * Processes a alarm action window. 
 * @param event The event to process. 
 */
function processAlarmActionWindowShownEvent(event) {
	// do nothing here, usually we start updating tables when their window is being shown, 
	// use the startLoading function with a alarm id to start updating this table
}


// implementation


/**
 * Marks a specified row. 
 * @param row The html row to mark. 
 */
ActionTable.prototype.markRow = function(row) {
	if (this.markedId != -1 && exists(this.getMarkedRow())) {
		this.unMarkRow(this.getMarkedRow());
	}
	this.markedId = this.getIdFieldValue(row);
	setClass(row, this.markClass);
}
 

/**
 * Unmarks a row.
 * @param row The row to unmark. 
 */
ActionTable.prototype.unMarkRow = function(row) {
	this.markedId = -1;
	if (exists(this.getRowClass(this.getHtmlRowIndex(row)))) {
		setClass(row, this.getRowClass(this.getHtmlRowIndex(row)));
	}
}


/**
 * Gets the html ow a mouse event is for. 
 * @param event The event to get the row for. 
 * @return The html row the event is for. 
 */
ActionTable.prototype.htmlRow = function(event) {
	event = exists(event) ? event : window.event;
	var srcElement = isIE() || isOpera() ? event.srcElement : event.target;
	return this.getHtmlRowFromTd(findTdNode(srcElement));
}


/**
 * Get the marked row. 
 * @return The html code of the current marked row, null if no row is marked. 
 */
ActionTable.prototype.getMarkedRow = function() {
	if (this.markedId == -1) {
		return null;
	}
	var row = this.getRowWithFieldValue(this.getIdFields()[0], this.markedId); 
	var index = this.getRowIndex(row[0]);
	return this.getTableCode().rows[index];
}


/**
 * Gets the value of a action rows id field value. 
 * @param row The html row to get the id field value for. 
 * @return The value of the id field value of the row the event is for. 
 */ 
ActionTable.prototype.getIdFieldValue = function(row) {
	var rowIndex = this.getHtmlRowIndex(row);
	return getFieldFromRow(this.getIdFields()[0], this.rows[rowIndex]).getValue();
}



/**
 * Gets the class to set for a row with a given index. 
 * This is overridden to return the marked class for marked rows. 
 * @param index The index of the row. 
 * @return The class to set, null if no class to set. 
 */
ActionTable.prototype.getRowClass = function(index) {
	if (index == -1 || !isDefined(this.rowClass) || this.getTableCode().rows[index].counter > 0) {
		return null;
	}
	var idFieldValue = getFieldFromRow(this.getIdFields()[0], this.rows[index]).getValue();
	if (index != -1 && idFieldValue == this.markedId) {
		// the row is marked 
		return this.markClass;
	}
	if (isDefined(this.rowClass) && !this.alternateRowClassOn) {
		return this.rowClass;
	}
	if (index % 2 == 0) {
		return this.rowClass;
 	}
 	else {
	 	return this.alternateClass;
 	}
}
