/**
 * This file provides utility functions regarding alarm actions.
 */ 


/**
 * The add alarm actions window title.
 */
var ADD_ALARM_ACTIONS_TABLE = "Add alarm actions";


/**
 * The edit alarm actions window title.
 */
var EDIT_ALARM_ACTIONS_TABLE = "Edit alarm actions";


/**
 * The title of the alarm action library window.
 */ 
var LIBRARY_ALARM_ACTIONS = "Alarm action library";


/**
 * The default option in the variable selector.
 */
var DEFAULT_VARIABLE_SELECT_OPTION = "Select a variable to insert";


/**
 * The default value in the to field.
 */
var DEFAULT_TO_VALUE = "[Enter a to address]";


/**
 * The default value in the subject field.
 */
var DEFAULT_SUBJECT_VALUE = "[Enter a subject]";


/**
 * The height to set for the table cell containing the actions when the program actions are shown in the library window.
 */
var PROGRAM_ACTIONS_CELL_HEIGHT_LIBRARY = 160;


/**
 * The height to set for the table cell containing the actions when the program actions are shown.
 */
var PROGRAM_ACTIONS_CELL_HEIGHT = 180;


/**
 * The height to set for the table cell containing the actions when the email actions are shown in the library window.
 */
var EMAIL_ACTIONS_CELL_HEIGHT_LIBRARY = 215;


/**
 * The height to set for the table cell containing the actions when the email actions are shown.
 */
var EMAIL_ACTIONS_CELL_HEIGHT = 242;


/**
 * The variables to insert. 
 */
var variables = new Array();
variables["Insert pid"] = "%%pid%%";
variables["Insert ip address"] = "%%ip%%";
variables["Insert src ip address"] = "%%srcIp%%";
variables["Insert dst ip address"] = "%%dstIp%%";
variables["Insert mac address"] = "%%mac%%";
variables["Insert src mac address"] = "%%srcMac%%";
variables["Insert dst mac address"] = "%%dstMac%%";
variables["Insert entry"] = "%%entry%%";
variables["Insert trigger value"] = "%%triggerValue%%";
variables["Insert trigger time"] = "%%triggerTime%%";


/**
 * The name of the variable buttons to show for each table.
 */
var tableVariables = new Array();
tableVariables["protocol"] =  ["Insert pid", "Insert trigger time", "Insert trigger value"];
tableVariables["nlhost"] =  ["Insert ip address", "Insert trigger time", "Insert trigger value"];
tableVariables["nlmatrix"] =  ["Insert src ip address", "Insert dst ip address", "Insert trigger time", "Insert trigger value"];
tableVariables["alhost"] =  ["Insert pid", "Insert ip address", "Insert trigger time", "Insert trigger value"];
tableVariables["almatrix"] =  ["Insert pid", "Insert src ip address", "Insert dst ip address", "Insert trigger time", "Insert trigger value"];
tableVariables["host"] =  ["Insert mac address", "Insert trigger time", "Insert trigger value"];
tableVariables["matrix"] =  ["Insert src mac address", "Insert dst mac address", "Insert trigger time", "Insert trigger value"];
tableVariables["network"] =  ["Insert entry", "Insert trigger time", "Insert trigger value"];
tableVariables["summary"] =  ["Insert ip address", "Insert trigger time", "Insert trigger value"];


/**
 * The error message sendt by the server if a alarm action was set or edited when no
 * email server is configured yet.
 */
var EMAIL_CONFIG_NOT_SET_MESSAGE = "E-mail configuration not set";


// construction



/**
 * Factory method to create new alarm action window objects.
 * @returns A new probeWindow object.
 * @param id The id of the table in the window. This is needed for catching events from the table.
 * @throws Error if id is undefined, null or empty.
 * @throws Error if actionDetail is null.
 */
function createActionWindow(id, tableId) {
	assertNotEmpty(id, "Id to set for the window is undefined, null or empty: " + id);
	
	var subInstance = new ActionWindow();
	var superInstance = createProbeWindow(id, tableId);
	subInstance = createSubClass(superInstance, subInstance);
	subInstance["superhide"] = superInstance["superhide"];
		
	WindowManager.instance().removeWindow(superInstance);
	WindowManager.instance().registerNewWindow(subInstance);
	
	// let the event processor call event handler functions as a method
	// of the mytable object instance
	superInstance.getEventProcessor().changeObject(subInstance);
	subInstance.eventListener.addEvent("tableUpdated", processUpdatedEventActionWindow);
	subInstance.eventListener.addEvent(ACTION_SET, processActionSetEvent);
	subInstance.eventListener.addEvent(ACTION_EDITED, processActionEditedEvent);
	subInstance.eventListener.addEvent(ACTION_ADDED_TO_LIBRARY, processActionAddedToLibraryEvent);
	subInstance.eventListener.addEvent(MOUSE_MOVED_OVER_ACTION_TABLE, processMouseMovedOverTableEvent);
	subInstance.eventListener.addEvent(MOUSE_MOVED_OVER_ACTION_ROW, processMouseMovedOverActionEvent);
	subInstance.eventListener.addEvent(MOUSE_MOVED_OUT_ACTION_TABLE, processMouseMovedOutTableEvent);
	subInstance.eventListener.addEvent(ACTION_SELECTED, processActionSelectedEvent);
	subInstance.eventListener.addEvent(ACTION_DESELECTED, processActionDeselectedEvent);
	subInstance.eventListener.addEvent(MARKED_ACTION_DELETED, processActionDeselectedEvent);
	subInstance.eventListener.addEvent("noAccessHidden", processNoAccessHiddenActionWindow);
	subInstance.eventListener.addEvent("noAccessShown", processNoAccessShownActionWindow);
	
	subInstance.eventListener.addEvent("tableResized", processActionTableResizedEvent);
	
	// the windows drag helper is initialized in the window constructor, we need to change the drag helpers
	// window refernce to be this window not the super instance of Window
	subInstance.dragHelper.window = subInstance;
	
	return subInstance;
}


/**
 * The alarm action window class.
 * @throws Error if actionDetail is null.
 */
function ActionWindow() {

	
	/**
	 * set to true if a action has been selected
	 */
	this.actionSelected = false;
	
	
	/**
	 * set to true if the mouse is over a action row and this rows details are shown
	 */
	this.actionDetailsShown = false;
	
	
	/**
	 * the id of the email action table in this window
	 */
	this.emailTableId = "";
	
	
	/**
	 * the id of the program action table in this window
	 */
	this.programTableId = "";
	
	
	/**
	 * Ff this is set to true details of the action row the mouse is over are shown
	 */
	this.showDetails = true;
	
	
	/**
	 * The temporarily storage for email action detail data. When a mouse is moved over a email action row, the
	 * detail for that action is shown, when the mouse is moved out of the table again the original email action
	 * data is set. Here the original email action data can be stored. 
	 */
	this.mouseOverEmailStorage = new EmailActionDataSet();
	
	
	/**
	 * The temporarily storage for email action details data when a action is selected. 
	 */
	this.selectedEmailStorage = new EmailActionDataSet();


	/**
	 * The temporarily storage for program action detail data. When a mouse is moved over a program action row, the
	 * detail for that action is shown, when the mouse is moved out of the table again the original program action
	 * data is set. Here the original program action data can be stored. 
	 */ 
	this.mouseOverProgramStorage = new ProgramActionDataSet();
	
	
	/**
	 * The temporarily storage for program action details data when a action is selected. 
	 */
	this.selectedProgramStorage = new ProgramActionDataSet();
}


// services


/**
 * Shows the add alarm action window.
 */
function showAddAlarmActionLibraryWindow() {
	var window = WindowManager.instance().getWindow("alarmactionslibrarywindow");
	WindowManager.instance().activateWindow(window.getId());
	if (window.emailActionsShown()) {
		window.getEmailTable().restartUpdate();
		window.getEmailTable().updateHtmlTable();
		window.getEmailDetailSection().clearAlarmActionFields();
		window.getEmailDetailSection().showSetActionButton();
	}
	else {
		window.getProgramTable().restartUpdate();
		window.getProgramTable().updateHtmlTable();
		window.getProgramDetailSection().clearAlarmActionFields();
		window.getProgramDetailSection().showSetActionButton();
	}
}

 
/**
 * Shows the add alarm action window. 
 */
function showAddAlarmActionWindow() {
	var window = WindowManager.instance().getWindow("alarmactionswindow");
	window.setWindowTitle(ADD_ALARM_ACTIONS_TABLE);
	WindowManager.instance().activateWindow(window.getId());
	// this inits the email table if it has not been done yet
	window.getEmailTable();
	if (window.emailActionsShown()) {
		showAddEmailActionWindow();
	}
	else {
		showAddProgramActionWindow();
	}
}


/**
 * Shows the add email action window. 
 */
function showAddEmailActionWindow() {
	var window = WindowManager.instance().getWindow("alarmactionswindow");
	window.getEmailDetailSection().showVariableOptions(selectedTableName());
	window.getEmailTable().cancelUpdate();
	window.getEmailTable().clear();
	window.getEmailTable().clearDataTable();
	var id = getInput(window.getHtmlCode(), "alarmId");
	if (exists(id) && !isEmpty(id.value)) {
		window.getEmailTable().startLoading(id.value);
	}
	window.getEmailDetailSection().clearAlarmActionFields();
	window.getEmailDetailSection().populateActionChooser();
	window.getEmailDetailSection().showSetActionButton();
}

 
/**
 * Shows the add program action window. 
 */
function showAddProgramActionWindow() {
	var window = WindowManager.instance().getWindow("alarmactionswindow");
	window.getProgramDetailSection().showVariableOptions(selectedTableName());
	window.getProgramTable().cancelUpdate();
	window.getProgramTable().clear();
	window.getProgramTable().clearDataTable();
	var id = getInput(window.getHtmlCode(), "alarmId");		
	if (exists(id) && !isEmpty(id.value)) {
		window.getProgramTable().startLoading(id.value);
	}
	window.getProgramDetailSection().clearAlarmActionFields();
	window.getProgramDetailSection().populateActionChooser();
	window.getProgramDetailSection().showSetActionButton();
}


/**
 * Shows the edit alarm action window. 
 */
function showEditAlarmActionWindow() {
	var window = WindowManager.instance().getWindow("alarmactionswindow");
	WindowManager.instance().activateWindow(window.getId());
	window.setWindowTitle(EDIT_ALARM_ACTIONS_TABLE);
	if (window.emailActionsShown()) {
		showEditEmailActionWindow();
	}
	else {
		showEditProgramActionWindow();
	}
}


/**
 * Shows the edit email action window. 
 */
function showEditEmailActionWindow() {
	var window = WindowManager.instance().getWindow("alarmactionswindow");
	window.getEmailDetailSection().showVariableOptions(selectedTableName());
	window.getEmailTable().clear();
	window.getEmailTable().clearDataTable();
	window.getEmailDetailSection().clearAlarmActionFields();
	window.getEmailDetailSection().populateActionChooser();
	getElementWithAttribute(window.getEmailDetailSection().getHtmlCode(), "libraryActionChooser", "name", "select").selectedIndex = 0;
	window.getEmailDetailSection().showSetActionButton();
	// set the alarm id to load actions for	
	var id = getInput(alarmDetailWindow().getHtmlCode(), "alarmId");	
	window.getEmailTable().startLoading(id.value);
	getInput(window.getEmailDetailSection().getHtmlCode(), "alarmId").value = id.value;
	getInput(window.getProgramDetailSection().getHtmlCode(), "alarmId").value = id.value;
}


/**
 * Shows the edit program action window. 
 */
function showEditProgramActionWindow() {
	var window = WindowManager.instance().getWindow("alarmactionswindow");
	window.getProgramDetailSection().showVariableOptions(selectedTableName());
	window.getProgramTable().clear();
	window.getProgramTable().clearDataTable();
	window.getProgramDetailSection().clearAlarmActionFields();
	window.getProgramDetailSection().populateActionChooser();
	getElementWithAttribute(window.getProgramDetailSection().getHtmlCode(), "libraryActionChooser", "name", "select").selectedIndex = 0;
	window.getProgramDetailSection().showSetActionButton();
	// set the alarm id to load actions for	
	var id = getInput(alarmDetailWindow().getHtmlCode(), "alarmId");	
	window.getProgramTable().startLoading(id.value);
	getInput(window.getProgramDetailSection().getHtmlCode(), "alarmId").value = id.value;
	getInput(window.getEmailDetailSection().getHtmlCode(), "alarmId").value = id.value;
}


/**
 * Sets the email action detail panel in this window. 
 * @param emailSection The email section to set.
 * @throws Error if emailSection is null or undefined. 
 */
ActionWindow.prototype.setEmailDetailSection = function(emailSection) {
	assertExists(emailSection, "The email section to set is null");
	this.emailDetailSection = emailSection;
}


/**
 * Gets the email action detail section from this window. 
 * @return The email action detail section set. 
 */
ActionWindow.prototype.getEmailDetailSection = function() {
	return this.emailDetailSection;
}


/**
 * Sets the id of the email action table. 
 * @param id The id to set. 
 * @throws Error if id is undefined, null or empty.
 */
ActionWindow.prototype.setEmailTableId = function(id) {
	assertNotEmpty(id, "The id to set for the email table is undefined, null or empty");
	this.emailTableId = id;
}


/**
 * Gets the id of the email action table set for this window.
 * @return The email table id set for this window, a empty string if none set.  
 */
ActionWindow.prototype.getEmailTableId = function() {
	return this.emailTableId;
}


/**
 * Get the email table in this window.
 * @return The email table object in this window. 
 */
ActionWindow.prototype.getEmailTable = function(id) {
	return TableManager.instance().getTable(this.emailTableId);
}	


/**
 * Sets the program action detail panel in this window.
 * @param programSection The program section to set. 
 * @throws Error if programSection is null or undefined.
 */
ActionWindow.prototype.setProgramDetailSection = function(programSection) {
	assertExists(programSection, "The program section to set is null");
	this.programDetailSection = programSection;
}

 
 /**
  * Gets the program action detail section from this window. 
  * @return The program action detail section set. 
  */
ActionWindow.prototype.getProgramDetailSection = function() {
	return this.programDetailSection;
}
 

/**
 * Sets the id of the program action table in this window. 
 * @param id The program action id to set.
 * @throws Error if id is undefined, null or empty.  
 */
ActionWindow.prototype.setProgramTableId = function(id) {
	assertNotEmpty(id, "The id to set for the program table is undefined, null or empty");
	this.programTableId = id;
}


/**
 * Get the program action table id set for this table. 
 * @return The program action table id set for this table, a empty string if none set. 
 */
ActionWindow.prototype.getProgramTableId = function() {
	return this.programTableId;
}


/**
 * Gets the program action table in this window. 
 * @return The program action table object in this window.
 */
ActionWindow.prototype.getProgramTable = function() {
	return TableManager.instance().getTable(this.programTableId);
}


/**
 * Shows the email action table and the email action details section. 
 */
ActionWindow.prototype.showEmailActions = function() {
	if (this.emailActionsShown()) {
		// the program actions are already shown
		return;
	}
	this.showEmailActionTable();
	this.showEmailActionDetailSection();
	this.activateEmailLink();
	this.getEmailDetailSection().populateActionChooser();
	if (this.id != "alarmactionslibrarywindow" && !isEmpty(selectedTableName())) {
		// set the correct variables for the table
		this.getProgramDetailSection().showVariableOptions(selectedTableName());
	}
	// the program action library table is being updated while the program actions are showing to populate the
	// library chooser, stop it from updating
	if (!WindowManager.instance().getWindow("alarmactionslibrarywindow").isShown()) {
		TableManager.instance().getTable("programAlarmActionLibraryTable").cancelUpdate();
	}
}


/**
 * Shows the program action table and the program action details section. 
 */
ActionWindow.prototype.showProgramActions = function() {
	if (!this.emailActionsShown()) {
		// the program actions are already shown
		return;
	}
	this.showProgramActionTable();
	this.showProgramActionDetailSection();
	this.activateProgramLink();
	this.getProgramDetailSection().populateActionChooser();
	if (this.id != "alarmactionslibrarywindow" && !isEmpty(selectedTableName())) {
		// set the correct variables for the table
		this.getProgramDetailSection().showVariableOptions(selectedTableName());
	}
	// the email action library table is being updated while the email actions are showing to populate the
	// library chooser, stop it from updating if hte libraru window is not shown
	if (!WindowManager.instance().getWindow("alarmactionslibrarywindow").isShown()) {
		TableManager.instance().getTable("emailAlarmActionLibraryTable").cancelUpdate();
	}
}
 
  
/**
 * Set the alarm id field based on a alarm xml.
 * @param alarmXml The alarm xml.
 */
ActionWindow.prototype.setAlarmActionId = function(alarmXml) {
	var id = getTagNodeValue("id", alarmXml);
	if (!isEmpty(id)) {
		getInput(this.getEmailDetailSection().getHtmlCode(), "alarmId").value = id;
		getInput(this.getProgramDetailSection().getHtmlCode(), "alarmId").value = id;
	}
}


/**
 * The setup button has been clicked. Show the setup of the active table (program or email table).
 * @param columns The number of colums the fields to choose should be displayed in.
 * @param event The onclick event that triggered the calling of this function.
 */
ActionWindow.prototype.showSetup = function(columns, event) {
	if (this.emailActionsShown()) {
		TableManager.instance().getTable(this.emailTableId).showSetup(columns, event);
	}
	else {
		TableManager.instance().getTable(this.programTableId).showSetup(columns, event);
	}
}


/**
 * Copies a action from the alarm action library into the add/edit alarm action window.
 */	
ActionWindow.prototype.addFromLibrary = function() {
	if (this.emailActionsShown()) {
		this.getEmailDetailSection().addFromLibrary();
	}
	else {
		this.getProgramDetailSection().addFromLibrary();
	}
}


// overridden from probeWindow.js


/**
 * Expand the chart area. This is overridden to expand the alarm detail section instead.
 */ 
ActionWindow.prototype.expandChartSection = function() {
	this.showAlarmDetailSection();
	this.showColapseButton();
	this.colapsedChart = false;
	this.showLegend();
	if (exists(this.madeNotResizable) && this.madeNotResizable == true) {
		this.makeResizable();
	}
}



/**
 * Shows this window alarm detail section.
 */
ActionWindow.prototype.showAlarmDetailSection = function() {
	// In IE we need to set the row invisible or else one pixel of the row will still be
	// visible, while in Safari setting the row invisible has no effect
	var chartRow = getElementWithAttribute(this.htmlCode, "chartRow", "name", "tr");
	var chartCell = getElementWithAttribute(this.htmlCode, "chartarea", "id", "td");
	var container = this.container;

	if ("inactive" == chartRow.getAttribute("class") || "inactive" == chartRow.getAttribute("className")) {
		// because of problems in IE we need to set the whole row invisible, not 
		// only the cell representing the container, or else 1 pixel of the row will still be visible
		setClass(chartRow, "active");
		setClass(chartCell, "active");

		// update the height of the window
		var windowHeight = parseInt(this.htmlCode.style.height);
		this.htmlCode.style.height = windowHeight + this.expandedContainerHeight + "px";
		this.height = parseInt(this.htmlCode.style.height);
	}
}


/**
 * Colapses the chart area of a window. This is overridden to colapse the alarm action 
 * detail section instead.
 */
ActionWindow.prototype.colapseChartSection = function() {
	this.hideActionDetailSection();
	this.showExpandButton();
	this.colapsedChart = true;
	
	if (this.colapsedContent) {
		// the content is also colapsed the window is to be unresizable
		this.makeNotResizable();
		this.madeNotResizable = true;
	}
	
	this.hideLegend();
}


/**
 * Hide the alarm detail section.
 */
ActionWindow.prototype.hideActionDetailSection = function() {
	this.expandedContainerHeight = this.container.offsetHeight;	
	// because of problems in IE we need to set the whole row invisible, not 
	// only the cell representing the container, or else 1 pixel of the row will still be visible,
	// while in Safari setting the row invisible has no effect
	var chartRow = getElementWithAttribute(this.htmlCode, "chartRow", "name", "tr");
	if (!exists(chartRow)) {
		return;
	}
	var chartCell = getElementWithAttribute(this.htmlCode, "chartarea", "id", "td");
	if ("inactive" == chartRow.getAttribute("class") || "inactive" == chartRow.getAttribute("className")) {
		// the chart section is already hidden
		return;
	} 
	
	setClass(chartRow, "inactive");
	setClass(chartCell, "inactive");
	
	var windowHeight = parseInt(this.htmlCode.style.height);
	this.htmlCode.style.height = windowHeight - this.expandedContainerHeight + "px";
	this.height = parseInt(this.htmlCode.style.height);
}	



/**
 * Hides a window. This is overridden to stop possible updating of action library tables. 
 */
ActionWindow.prototype.hide = function() {
	this.callSuper("hide");
	
	// when a window is hidden safari does not fire the mouseout event on the close button
	// so the mouseover effect of the button is still visible the next time the window is made visible
	// set the correct class for the closebutton manually
	var closeButton = getElementWithAttribute(this.getHtmlCode(), "closeButton", "id", "div");
	if (exists(closeButton)) {
		setClass(closeButton, "closeButton WindowButton");
	}
	
	// stop the updating of the program and email action library tables if the library window is not shown
	if (!WindowManager.instance().getWindow("alarmactionslibrarywindow").isShown()) {
		TableManager.instance().getTable("emailAlarmActionLibraryTable").cancelUpdate();
		TableManager.instance().getTable("programAlarmActionLibraryTable").cancelUpdate();
	}
}


/**
 * Hides this windows chart section. The chart section
 * is the chart selector and the space for the charts.
 * This is overridden to calculate the height of the action detail sections correct.
 */
ActionWindow.prototype.hideChartSection = function() {
	// because of problems in IE we need to set the whole row invisible, not 
	// only the cell representing the container, or else 1 pixel of the row will still be visible,
	// while in Safari setting the row invisible has no effect
	var chartRow = getElementWithAttribute(this.htmlCode, "chartRow", "name", "tr");
	if (!exists(chartRow)) {
		return;
	}
	var chartCell = getElementWithAttribute(this.htmlCode, "chartarea", "id", "td");
	if ("inactive" == chartRow.getAttribute("class") || "inactive" == chartRow.getAttribute("className")) {
		// the chart section is already hidden
		return;
	} 
	
	setClass(chartRow, "inactive");
	setClass(chartCell, "inactive");
	var container = this.container;

	// update the size of the window
	this.expandedContainerHeight = this.container.offsetHeight;	
	var windowHeight = parseInt(this.htmlCode.style.height);
	this.htmlCode.style.height = windowHeight - this.expandedContainerHeight + "px";
	this.height = parseInt(this.htmlCode.style.height);
	
	// the container and the active chart are not to be resized 
	// when they are not shown
	if (!isNull(this.container)) {
		this.removeResizableElement(this.container);
	}
	if (!isNull(this.activeChart)) {
		this.removeResizableElement(this.activeChart.htmlCode);
	}
}	


// event processing


/**
 * The right click menu choice "Add to library" has been clicked. Add the clicked action to the library.
 * This forwards the event to the table currently being shown.
 */
ActionWindow.prototype.addToLibrary = function() {
	if (this.emailActionsShown()) {
		var row = lastTableObject.getRowFromTd(lastTdNode);
		var dataSet = new EmailActionDataSet();
		dataSet.to = getFieldFromRow("to", row).getValue();
		dataSet.subject = getFieldFromRow("subject", row).getValue();
		dataSet.content = getFieldFromRow("content", row).getValue();
		dataSet.max = getFieldFromRow("maxEmails", row).getValue();
		dataSet.maxTimeSelected = getFieldFromRow("maxEmailTime", row).getValue();
		this.getEmailDetailSection().addToLibraryDataSet(dataSet);
	}
	else {
		var dataSet = new ProgramActionDataSet();
		var row = lastTableObject.getRowFromTd(lastTdNode);
		dataSet.program = getFieldFromRow("programName", row).getValue();
		dataSet.parameters = getFieldFromRow("parameters", row).getValue();
		dataSet.description = getFieldFromRow("description", row).getValue(); 
		this.getProgramDetailSection().addToLibraryDataSet(dataSet);
	}
}


/**
 * The right click menu choice "Edit alarm action" has been clicked. Mark and show the clicked actions details.
 * This forwards the event to the table currently being shown. 
 */
ActionWindow.prototype.editAction = function() {
	if (this.emailActionsShown()) {
		TableManager.instance().getTable(this.emailTableId).editAction();
	}
	else {
		TableManager.instance().getTable(this.programTableId).editAction();
	}
}


/**
 * Deletes a action on a right click on the delete menu. This forwards the event 
 * to the table currently being shown. 
 */
ActionWindow.prototype.deleteActionRightClick = function() {
	if (this.emailActionsShown()) {
		TableManager.instance().getTable(this.emailTableId).deleteActionRightClick();
	}
	else {
		TableManager.instance().getTable(this.programTableId).deleteActionRightClick();
	}
}


/**
 * Processes a table resized event. 
 * @param event The event to process. 
 */
function processActionTableResizedEvent(event) {
	var source = event.getData();
	if (source != this.getEmailDetailSection() && source != this.getProgramDetailSection()) {
		// only process events for this window
		return;
	}
	if (this.getEmailTableId() != event.getData()["id"] && this.getProgramTableId() != event.getData()["id"]) {
		return;
	} 

	var width = event.getData()["width"];
	var height = event.getData()["height"];
	var resizedHeight = this.resizeWindowHeight(height);
	var resizedWidth = this.resizeWindowWidth(width);
	this.resizeElements(height, width, false);
	this.resized();
}


/**
 * Processes a updated event from the table.
 * @param event The event to process. 
 */
function processUpdatedEventActionWindow(event) {
	if (event.getData()["id"].indexOf("ActionLibraryTable") == -1 || this.id != "alarmactionswindow") {
		// only events from the library are processed in the add/edit alarm window
		// we need to do this to update the dropdown box with actions from the library
		return;
	}
	if (this.emailActionsShown()) {
		this.getEmailDetailSection().populateActionChooser();
	}
	else {
		this.getProgramDetailSection().populateActionChooser();
	}
}


/**
 * Processes a action set event. 
 * @param event The event to process. 
 */
function processActionSetEvent(event) {
	var source = event.getData();
	if (source != this.getEmailDetailSection() && source != this.getProgramDetailSection()) {
		// only process events for this window
		return;
	}
	
	var source = event.getData();
	if (source != this.getEmailDetailSection() && source != this.getProgramDetailSection()) {
		// only process events for this window
		return;
	}
	if (this.emailActionsShown()) {
		this.getEmailTable().restartUpdate();
	}
	else {
		this.getProgramTable().restartUpdate();
	}
}


/**
 * Processes a action edited event. 
 * @param event The event to process. 
 */
function processActionEditedEvent(event) {
	var source = event.getData();
	if (source != this.getEmailDetailSection() && source != this.getProgramDetailSection()) {
		// only process events for this window
		return;
	}
	
	if (this.emailActionsShown()) {
		TableManager.instance().getTable(this.emailTableId).restartUpdate();
	}
	else {
		TableManager.instance().getTable(this.programTableId).restartUpdate();
	}
}


/**
 * Processes a action added to library event. 
 * @param event The event to process.  
 */
function processActionAddedToLibraryEvent(event) {
	// TODO: implement this
}


/**
 * Process a mouse moved over the table event. 
 * @param event The event to process. 
 */ 
function processMouseMovedOverTableEvent(event) {
	var source = event.getData();
	if (source != this.getEmailTable() && source != this.getProgramTable()) {
		// only process events for this window
		return;
	}
	this.getEmailDetailSection().clearAlarmActionMessage();
	this.getProgramDetailSection().clearAlarmActionMessage();
}


/**
 * Process a mouse moved out of the table event. 
 * @param event The event to process. 
 */
function processMouseMovedOutTableEvent(event) {
	var source = event.getData();
	if (source != this.getEmailTable() && source != this.getProgramTable()) {
		// only process events for this window
		return;
	}
	if (this.emailActionsShown()) {
		this.restoreEmailActionDetails(this.mouseOverEmailStorage);
	}
	if (!this.emailActionsShown()) {
		this.restoreProgramActionDetails(this.mouseOverProgramStorage);
	}
	this.showDetails = true;
	this.actionDetailsShown = false;
}


/**
 * Process a mouse moved over a action row event. When the mouse is moved over
 * one of the action rows the rows details are to be shown in the action detail section.
 * @param event The event to process.
 */
function processMouseMovedOverActionEvent(event) {
	var source = event.getData()["source"];
	if (source != this.getEmailTable() && source != this.getProgramTable()) {
		// only process events for this window
		return;
	}
	if (this.actionDetailsShown == false && this.actionSelected == false) {
		this.saveActionDetails();
	}
	// set the action details of the row
	if (this.showDetails == true) {
		this.actionDetailsShown = true;
		this.showActionDetail(event.getData()["row"]);
	}
}	
	

/**
 * Processes a action selected event. 
 * @param event The event to process. 
 */
function processActionSelectedEvent(event) {
	var source = event.getData()["source"];
	if (source != this.getEmailTable() && source != this.getProgramTable()) {
		// only process events for this window
		return;
	}
	this.showDetails = false;
	this.saveActionDetailsSelect();
	var row = event.getData()["row"];
	this.showActionDetail(row);
	this.actionSelected = true;
	// when the mouse is moved out of the table the data in the mouse over storage is restored
	// now a row has been click to be edited when the mouse is moved out of the table do not  
	// change the action detail data
	if (this.emailActionsShown()) {
		this.mouseOverEmailStorage.to = getFieldFromRow("to", row).getValue();
		this.mouseOverEmailStorage.subject = URLDecode(getFieldFromRow("subject", row).getValue());
		this.mouseOverEmailStorage.max = getFieldFromRow("maxEmails", row).getValue();
		this.mouseOverEmailStorage.maxTimeSelected = getFieldFromRow("maxEmailTime", row).getValue();
		this.mouseOverEmailStorage.content = URLDecode(getFieldFromRow("content", row).getValue());
	}
	else {
		this.mouseOverProgramStorage.program = URLDecode(getFieldFromRow("programName", row).getValue());
		var parameters = URLDecode(getFieldFromRow("parameters", row).getValue());
		this.mouseOverProgramStorage.parameters = isEmpty(parameters) ? DEFAULT_PARAMETERS_FIELD_VALUE : parameters;
		var description = URLDecode(getFieldFromRow("description", row).getValue());
		this.mouseOverProgramStorage.description = isEmpty(description) ? DEFAULT_DESCRIPTION_FIELD_VALUE : description;
	}
}


/**
 * Processes a action deselecteed event.
 * @param event The event to process. 
 */
function processActionDeselectedEvent(event) {
	var source = event.getData();
	if (source != this.getEmailTable() && source != this.getProgramTable()) {
		// only process events for this window
		return;
	}
	this.actionSelected = false;
	this.showDetails = true;
	if (this.emailActionsShown()) {
		this.getEmailDetailSection().showSetActionButton();
	}
	else {
		this.getProgramDetailSection().showSetActionButton();
	}
	this.restoreActionDetailsDeselected();
	
	// action has been deselected let the data been restored when mouse is moved out of the table
	// be the data present before action selected
	this.mouseOverEmailStorage.to = this.selectedEmailStorage.to;
	this.mouseOverEmailStorage.subject = this.selectedEmailStorage.subject;
	this.mouseOverEmailStorage.max = this.selectedEmailStorage.max;
	this.mouseOverEmailStorage.maxTimeSelected = this.selectedEmailStorage.maxTimeSelected;
	this.mouseOverEmailStorage.content = this.selectedEmailStorage.content;
	this.mouseOverProgramStorage.program = this.selectedProgramStorage.program;
	this.mouseOverProgramStorage.parameters = this.selectedProgramStorage.parameters;
	this.mouseOverProgramStorage.description = this.selectedProgramStorage.description;
}


/**
 * Processes a no access hidden event from a table. Overridden to deal with both the
 * email and program action table.  
 * @param event The event to process. 
 */
function processNoAccessHiddenActionWindow(event) {
	if (!this.getEmailTableId().equalsIgnoreCase(event.getData()) && !this.getProgramTableId().equalsIgnoreCase(event.getData())) {
		return;
	}

	this.showChartSelector();
	this.showChartSection();
	this.noAccessShown = false;
}


/**
 * Processes a no acces shown event from a table. Overridden to deal with both the 
 * email and program action table.  
 * @param event The event to process.
 */
function processNoAccessShownActionWindow(event) {
	if (!this.getEmailTableId().equalsIgnoreCase(event.getData()) && !this.getProgramTableId().equalsIgnoreCase(event.getData())) {
		return;
	}
	
	this.hideChartSection();
 	this.hideChartSelector();
	this.noAccessShown = true;
}


// implementation


/**
 * Checks if the email actions are currently showing. 
 * @return True if the email actions are showind, false otherwise. 
 */
ActionWindow.prototype.emailActionsShown = function() {
	var emailAlarmTable = getElementWithAttribute(this.getHtmlCode(), "emailAlarmTable", "name", "div");
	if (getClassName(emailAlarmTable) == "active") {
		return true;
	}
	return false;
}


/**
 * Shows the email action table, the program action table is hidden. 
 */
ActionWindow.prototype.showEmailActionTable = function() {
	var emailAlarmTable = getElementWithAttribute(this.getHtmlCode(), "emailAlarmTable", "name", "div");
	var programAlarmTable = getElementWithAttribute(this.getHtmlCode(), "programAlarmTable", "name", "div");
	setClass(programAlarmTable, "inactive");
	setClass(emailAlarmTable, "active");
	this.getProgramTable().cancelUpdate();
	this.getEmailTable().cancelUpdate();
	this.getEmailTable().startLoading(this.getEmailDetailSection().getAlarmId());
	
	if (emailAlarmTable.offsetWidth != parseInt(this.getEmailTable().getTableContainerCode().style.width)) {
		this.getProgramTable().resizeTable(0, (emailAlarmTable.offsetWidth - parseInt(this.getEmailTable().getTableContainerCode().style.width)));
	}
}


/**
 * Shows the email action detail section, the program action detail section is hidden.
 */
ActionWindow.prototype.showEmailActionDetailSection = function() {
	var emailActionDetail = getElementWithAttribute(this.getHtmlCode(), "emailActionDetail", "name", "table");
	var programActionDetail = getElementWithAttribute(this.getHtmlCode(), "programActionDetail", "name", "table");
	setClass(programActionDetail, "inactive");
	setClass(emailActionDetail, "actionDetail");
	// reset the height of the table cell the actions are in
	var cell = getElementWithAttribute(this.htmlCode, "chartarea", "id", "td");
	cell.style.height = this.getEmailActionCellHeight() + "px";
	this.resizeWindowHeight(this.getEmailActionCellHeight() - this.getProgramActionCellHeight());
	this.resized();
}


/**
 * Shows the program action table, the email action table is hidden.  
 */
ActionWindow.prototype.showProgramActionTable = function() {
	var emailAlarmTable = getElementWithAttribute(this.getHtmlCode(), "emailAlarmTable", "name", "div");
	var programAlarmTable = getElementWithAttribute(this.getHtmlCode(), "programAlarmTable", "name", "div");
	setClass(emailAlarmTable, "inactive");
	setClass(programAlarmTable, "active");
	this.getProgramTable().cancelUpdate();
	this.getProgramTable().startLoading(this.getProgramDetailSection().getAlarmId());
	this.getEmailTable().cancelUpdate();
	
	if (programAlarmTable.offsetWidth != parseInt(this.getProgramTable().getTableContainerCode().style.width)) {
		this.getProgramTable().resizeTable(0, (programAlarmTable.offsetWidth - parseInt(this.getProgramTable().getTableContainerCode().style.width)));
	}
}


/**
 * Shows the program action detail section, the email program action detail is hidden.
 */
ActionWindow.prototype.showProgramActionDetailSection = function() {
	var emailActionDetail = getElementWithAttribute(this.getHtmlCode(), "emailActionDetail", "name", "table");
	var programActionDetail = getElementWithAttribute(this.getHtmlCode(), "programActionDetail", "name", "table");
	setClass(emailActionDetail, "inactive");
	setClass(programActionDetail, "actionDetail");
	this.getProgramDetailSection().loadPrograms();
	// reset the height of the table cell the actions are in
	var cell = getElementWithAttribute(this.htmlCode, "chartarea", "id", "td");
	cell.style.height = this.getProgramActionCellHeight() + "px";
	this.resizeWindowHeight(this.getProgramActionCellHeight() - this.getEmailActionCellHeight());
	this.resized();
}


/**
 * Gets the height to set for the cell containing the action details when the program action details
 * are shown. The height depends on if the window is the add/edit action window or the action library window, 
 * because the add/edit window has a dropdown box to select actions from the library.
 * @return The height to set. 
 */
ActionWindow.prototype.getProgramActionCellHeight = function() {
	if (LIBRARY_ALARM_ACTIONS == this.getWindowTitle()) {
		return PROGRAM_ACTIONS_CELL_HEIGHT_LIBRARY;
	}
	return PROGRAM_ACTIONS_CELL_HEIGHT;	
}


/**
 * Gets the height to set for the cell containing the action details when the email action details 
 * are shown. The height depends on whether the window is the add/edit action window of the action library window,
 * because the add/edit window has a dropdown box to select actions from the library.
 */
ActionWindow.prototype.getEmailActionCellHeight = function() {
	if (LIBRARY_ALARM_ACTIONS == this.getWindowTitle()) {
		return EMAIL_ACTIONS_CELL_HEIGHT_LIBRARY;
	}
	return EMAIL_ACTIONS_CELL_HEIGHT;
	
}


/**
 * Activates the email link and deactivates the program action link.
 */
ActionWindow.prototype.activateEmailLink = function() {
	var emailActionLink = getElementWithAttribute(this.getHtmlCode(), "emailActionLink", "name", "a");
	setClass(emailActionLink, "activeLink");
	var programActionLink = getElementWithAttribute(this.getHtmlCode(), "programActionLink", "name", "a");
	setClass(programActionLink, "inActiveLink");
}


/**
 * Activates the program link and deactivates the email action link.
 */
ActionWindow.prototype.activateProgramLink = function() {
	var programActionLink = getElementWithAttribute(this.getHtmlCode(), "programActionLink", "name", "a");
	setClass(programActionLink, "activeLink");
	var emailActionLink = getElementWithAttribute(this.getHtmlCode(), "emailActionLink", "name", "a");
	setClass(emailActionLink, "inActiveLink");
}


/**
 * Restores saved action details when a row has been deselected.
 */
ActionWindow.prototype.restoreActionDetailsDeselected = function() {
	if (this.emailActionsShown()) {
		this.restoreEmailActionDetails(this.selectedEmailStorage);
	}
	else {
		this.restoreProgramActionDetails(this.selectedProgramStorage);
	}
}


/**
 * A action is to be edited, save the original action details so they can be restored 
 * when the action is deselected.
 */
ActionWindow.prototype.saveActionDetailsSelect = function() {
	if (this.emailActionsShown()) {
		this.saveEmailActionDetailsSelect();
	}
	else {
		this.saveProgramActionDetailsSelect();
	}
}


/**
 * A email action is to be edited, save the original email action details so they can be restored 
 * when the action is deselected.
 */
ActionWindow.prototype.saveEmailActionDetailsSelect = function() {
	this.selectedEmailStorage.to = this.mouseOverEmailStorage.to;
	this.selectedEmailStorage.subject = this.mouseOverEmailStorage.subject;
	this.selectedEmailStorage.max = this.mouseOverEmailStorage.max;
	this.selectedEmailStorage.maxTimeSelected = this.mouseOverEmailStorage.maxTimeSelected;
	this.selectedEmailStorage.content = this.mouseOverEmailStorage.content;
	this.getEmailDetailSection().showEditActionButton();
}


/**
 * A program action is to be edited, save the original program action details so they can be restored 
 * when the action is deselected.
 */
ActionWindow.prototype.saveProgramActionDetailsSelect = function() {
	this.selectedProgramStorage.program = this.mouseOverProgramStorage.program;
	this.selectedProgramStorage.parameters = this.mouseOverProgramStorage.parameters;
	this.selectedProgramStorage.description = this.mouseOverProgramStorage.description;
	this.getProgramDetailSection().showEditActionButton();
}


/**
 * Shows the action details from a table row.  
 * @param row The table row containing the action to show. 
 */
ActionWindow.prototype.showActionDetail = function(row) {
	if (this.emailActionsShown()) {
		this.showEmailActionDetail(row);
	}
	else {
		this.showProgramActionDetail(row);
	}
}


/**
 * Shows the action details for a email table row. 
 * @param row The table row containing the action to show.
 */
ActionWindow.prototype.showEmailActionDetail = function(row) {
	this.getEmailDetailSection().setTo(getFieldFromRow("to", row).getValue());
	this.getEmailDetailSection().setMax(getFieldFromRow("maxEmails", row).getValue());
	this.getEmailDetailSection().setMaxPerTime(getFieldFromRow("maxEmailTime", row).getValue());
	this.getEmailDetailSection().setSubject(URLDecode(getFieldFromRow("subject", row).getValue()));
	this.getEmailDetailSection().setContent(URLDecode(getFieldFromRow("content", row).getValue()));
	this.getEmailDetailSection().setActionId(getFieldFromRow("id", row).getValue());
}


/**
 * Shows the program action details for a program table row.
 * @param row The table row containing the action to show. 
 */
ActionWindow.prototype.showProgramActionDetail = function(row) {
	this.getProgramDetailSection().setProgram(URLDecode(getFieldFromRow("programName", row).getValue()));
	var description = URLDecode(getFieldFromRow("description", row).getValue());
	this.getProgramDetailSection().setDescription(isEmpty(description) ? DEFAULT_DESCRIPTION_FIELD_VALUE : description);
	var parameters = URLDecode(getFieldFromRow("parameters", row).getValue());
	this.getProgramDetailSection().setParameters(isEmpty(parameters) ? DEFAULT_PARAMETERS_FIELD_VALUE : parameters);
	this.getProgramDetailSection().setActionId(getFieldFromRow("id", row).getValue());
}


/**
 * Save the action details. 
 */
ActionWindow.prototype.saveActionDetails = function() {
	if (this.emailActionsShown()) {
		this.saveEmailActionDetails();
	}
	else {
		this.saveProgramActionDetails();
	}
}


/**
 * Save the program action details. 
 */
ActionWindow.prototype.saveProgramActionDetails = function() {
	this.mouseOverProgramStorage.program = this.getProgramDetailSection().getProgram();
	this.mouseOverProgramStorage.parameters = this.getProgramDetailSection().getParameters();
	this.mouseOverProgramStorage.description = this.getProgramDetailSection().getDescription();
}


/**
 * Save the email action details. 
 */
ActionWindow.prototype.saveEmailActionDetails = function() {
	this.mouseOverEmailStorage.to = this.getEmailDetailSection().getTo();
	this.mouseOverEmailStorage.subject = this.getEmailDetailSection().getSubject();
	this.mouseOverEmailStorage.max = this.getEmailDetailSection().getMax();
	this.mouseOverEmailStorage.maxTimeSelected = this.getEmailDetailSection().getMaxPerTime();
	this.mouseOverEmailStorage.content = this.getEmailDetailSection().getContent();
}


/**
 * Restores temporarily stored program action details. 
 * @param storage The temporarily storage to restore the action from. 
 */
ActionWindow.prototype.restoreProgramActionDetails = function(storage) {
	this.getProgramDetailSection().setProgram(storage.program);
	this.getProgramDetailSection().setParameters(storage.parameters);
	this.getProgramDetailSection().setDescription(storage.description);
}



/**
 * Restores temporarily stored email action details. 
 * @param storage The temporarily storage to restore the action from. 
 */
ActionWindow.prototype.restoreEmailActionDetails = function(storage) {
	this.getEmailDetailSection().setTo(storage.to);
	this.getEmailDetailSection().setSubject(storage.subject); 
	this.getEmailDetailSection().setMax(storage.max);
	this.getEmailDetailSection().setMaxPerTime(storage.maxTimeSelected); 
	this.getEmailDetailSection().setContent(storage.content);
}
 