/**
 * This class shows email action details, and lets the user add/edit email actions. 
 */
 
 
// constants


/**
 * The base url used to set a email alarm action.
 */
var SET_EMAIL_ACTION_BASE_URL = "/?resource=AlarmManager&mode=set&command=emailAlarmAction&action=set&sid=" + getSid() + "&";


/**
 * The base url to set a library email alarm action.
 */
var SET_LIBRARY_EMAIL_ACTION_BASE_URL = "/?resource=AlarmManager&mode=set&command=libraryEmailAction&action=set&sid=" + getSid() + "&";



/**
 * The class used to represent email action detail data. All the email action fields are represented
 * by a variabel in this class.
 */
function EmailActionDataSet() {
	var to;
	var subject;
	var max;
	var maxTimeSelected;
	var content;
}


// construction
 
 
/**
 * Factory method to create a email action detail object.
 * @param setActionBaseUrl The base url used to set a email alarm action. 
 * @param editActionBaseUrl The base url used to edit a email alarm action.
 * @param addToLibraryUrl The base url used to add a action to the action library.  
 * @param htmlCode The html code containing the email detail section.
 * @throws Error if setActionBaseUrl is undefined, null or empty. 
 * @throws Error if editActionBaseUrl is undefined, null or empty. 
 * @throws Error if htmlCode is null or undefined.
 */
function createEmailActionDetail(setActionBaseUrl, editActionBaseUrl, addToLibraryBaseUrl, htmlCode) {
	assertNotEmpty(setActionBaseUrl, "The set action base url is undefined, null or empty", setActionBaseUrl);
	assertNotEmpty(editActionBaseUrl, "The edit action base url is undefined, null or empty", editActionBaseUrl);
	assertExists(htmlCode, "The html code to set is null or undefined");
	var subInstance = new EmailActionDetail(htmlCode);
	var superInstance = new ActionDetail(setActionBaseUrl, editActionBaseUrl, addToLibraryBaseUrl, htmlCode);
	subInstance = createSubClass(superInstance, subInstance);
	
	return subInstance;
}


/**
 * The email action detail class. 
 * @param htmlCode The html code containing the email detail section.
 * @throws Error if htmlCode is null or undefined.
 */
function EmailActionDetail(htmlCode) {
	assertExists(htmlCode, "The html code to set is null or undefined");
	this.htmlCode = htmlCode;
	this.to = getInput(this.htmlCode, "toAddress");
	this.max = getInput(this.htmlCode, "max");
	this.maxPerTime = getElementWithAttribute(htmlCode, "maxPerTime", "name", "select");
	this.subject = getInput(this.htmlCode, "subject");
	this.content = getElementWithAttribute(this.htmlCode, "content", "name", "textarea")
}


// services


/**
 * Get the value currenty set in the to field. 
 */
EmailActionDetail.prototype.getTo = function() {
	return this.to.value;
}


/**
 * Set the to field to a given value. 
 * @param to The value to set. 
 * @throws Error if to is null or undefined. 
 */
EmailActionDetail.prototype.setTo = function(to) {
	assertExists(to, "To address to set is undefined or null");
	this.to.value = to;
}


/**
 * Gets the current max value set. 
 */
EmailActionDetail.prototype.getMax = function() {
	return this.max.value;
}

/**
 * Sets the current value of the max field. 
 * @param max The max value to set. 
 * @throws Error if max is undefined or null.
 */
EmailActionDetail.prototype.setMax = function(max) {
	assertExists(max, "The max value to set is undefined or null");
	this.max.value = max;
}


/**
 * Gets the selected value of the max per time selector. 
 */
EmailActionDetail.prototype.getMaxPerTime = function() {
	return this.maxPerTime.options[this.maxPerTime.selectedIndex].value;
}


/**
 * Select a given index in the max per time select.
 * @param value The value to select.  
 * @throws Error if value is undefined, null or empty.   
 */
EmailActionDetail.prototype.setMaxPerTime = function(value) {
	assertNotEmpty(value, "The value to select is undefined, null or empty");
	selectInSelector(this.maxPerTime, value);
}


/**
 * Get the value currently set in the subject field. 
 */
EmailActionDetail.prototype.getSubject = function() {
	return this.subject.value;
}
 
 
/**
 * Sets the current value of the subject field. 
 * @param subject The subject to set. 
 * @throws Error if subject is undefined or null.
 */
EmailActionDetail.prototype.setSubject = function(subject) {
	assertExists(subject, "The subject to set is undefined or null");
	this.subject.value = subject;
}
 

/**
 * Get the value currently set in the content field. 
 */
EmailActionDetail.prototype.getContent = function() {
	return this.content.value;
}


/**
 * Set the value in the content field.
 * @param content The content to set. 
 * @throws Error if content is undefined or null. 
 */
EmailActionDetail.prototype.setContent = function(content) {
	assertExists(content, "The content to set is undefined or null");
	this.content.value = content;
}


/**
 * The to field has been focused. 
 */
EmailActionDetail.prototype.toFocused = function() {
	var toAddress = getInput(this.getHtmlCode(), "toAddress");
	if (DEFAULT_TO_VALUE == toAddress.value) {
		toAddress.value = "";
	}
	this.clearAlarmActionMessage();
}


/**
 * The subject field has been focused.
 */
EmailActionDetail.prototype.subjectFocused = function() {
	var subject = getInput(this.getHtmlCode(), "subject");
	if (DEFAULT_SUBJECT_VALUE == subject.value) {
		subject.value = "";
	}
	this.clearAlarmActionMessage();
}


/**
 * A variable has been selected, insert it into the content.
 */
EmailActionDetail.prototype.variableSelected = function() {
	var variableSelector = getElementWithAttribute(this.getHtmlCode(), "variableSelector", "name", "select");
	var selectedOption = variableSelector.options[variableSelector.selectedIndex].text;
	if (DEFAULT_VARIABLE_SELECT_OPTION == selectedOption) {
		// the default option was selected, nothing to insert
		return;
	}
	var content = getElementWithAttribute(this.getHtmlCode(), "content", "name", "textarea");
	content.value += variables[selectedOption] + " ";
	content.focus();
}



/**
 * Copies a action from the alarm action library into the add/edit alarm action window.
 */	
EmailActionDetail.prototype.addFromLibrary = function() {
	var actionChooser = getElementWithAttribute(this.getHtmlCode(), "libraryActionChooser", "name", "select");
	var actionId = actionChooser.options[actionChooser.selectedIndex].value;
	if (actionId == -1) {
		return;
	}
	
	var action = TableManager.instance().getTable("emailAlarmActionLibraryTable").getRowWithFieldValue("id", actionId);
	getInput(this.getHtmlCode(), "toAddress").value = getFieldFromRow("to", action).getValue();
	getInput(this.getHtmlCode(), "subject").value = URLDecode(getFieldFromRow("subject", action).getValue());
	getElementWithAttribute(this.getHtmlCode(), "content", "name", "textarea").value = URLDecode(getFieldFromRow("content", action).getValue());
	getInput(this.getHtmlCode(), "max").value = getFieldFromRow("maxEmails", action).getValue();
	var maxTime = getFieldFromRow("maxEmailTime", action).getValue();
	selectInSelector(getElementWithAttribute(this.getHtmlCode(), "maxPerTime", "name", "select"), maxTime);
}



/**
 * Populates the action chooser with actions from the action library.
 */
EmailActionDetail.prototype.populateActionChooser = function() {
	var table = TableManager.instance().getTable("emailAlarmActionLibraryTable");
	var actionChooser = getElementWithAttribute(this.getHtmlCode(), "libraryActionChooser", "name", "select");
	if (!exists(actionChooser)) {
		return;
	}
	if (!table.isUpdating()) {
		table.restartUpdate();
	}
	var data = table.getData();
	var selectedIndex = actionChooser.selectedIndex;
	var selectedText = actionChooser.options[actionChooser.selectedIndex].text;
	deleteOptions(actionChooser);
	actionChooser.add(new Option("Choose a action from the library", "-1"), undefined);
	for (var i = 0; i < data.length; i++) {
		var id = getFieldFromRow("id", data[i]);
		var to = getFieldFromRow("to", data[i]);
		var subject = getFieldFromRow("subject", data[i]);
		actionChooser.add(new Option(to.getValue() + " / " + URLDecode(subject.getValue()), id.getValue()), undefined);
	}
	// select the option selected before the new options are inserted, if the selected option has been
	// deleted select the default option
	selectedIndex = getOptionIndex(selectedText, actionChooser) ;
	actionChooser.selectedIndex = (selectedIndex > actionChooser.options.length || selectedIndex == -1) ? 0 : selectedIndex;
}



/**
 * Clears the add alarm action fields.
 */
EmailActionDetail.prototype.clearAlarmActionFields = function() {
	var librarySelect = getElementWithAttribute(this.getHtmlCode(), "libraryActionChooser", "name", "select");
	if (exists(librarySelect)) {
		librarySelect.selectedIndex = 0;
	}	
	getInput(this.getHtmlCode(), "toAddress").value = DEFAULT_TO_VALUE;
	getInput(this.getHtmlCode(), "subject").value = DEFAULT_SUBJECT_VALUE;
	getElementWithAttribute(this.getHtmlCode(), "content", "name", "textarea").value = "";
	getInput(this.getHtmlCode(), "actionId").value = "";
	getInput(this.getHtmlCode(), "max").value = "";
	var maxTime = getElementWithAttribute(this.getHtmlCode(), "maxPerTime", "name", "select");
	maxTime.selectedIndex = 0;
	this.clearAlarmActionMessage();
}


/**
 * Returns the part of a url with the email actions parameters set, based on the values set in the
 * alarm action detail window. 
 */
EmailActionDetail.prototype.createActionParameterUrl = function() {
	var content = getElementWithAttribute(this.getHtmlCode(), "content", "name", "textarea");
	var alarmId = getInput(this.getHtmlCode(), "alarmId");
	return this.toUrl() + this.subjectUrl() + "content=" + encodeURIComponent(content.value) + (alarmId == null ? "" : "&alarmId=" + alarmId.value) + "&" + this.maxPerTimeUrl();
}


/**
 * Returns the part of a url with the email actions parameters set, based on the values 
 * in the given email action data set. 
 * @param dataSet The data set to take the data from. 
 * @throws Error if the data set is undefined null or empty.
 */
EmailActionDetail.prototype.createActionParameterUrlDataSet = function(dataSet) {
	assertExists(dataSet, "The data set to create the parameter url for is undefined or null");
	return "to=" + dataSet.to + "&subject=" + encodeURIComponent(dataSet.subject) + "&content=" + encodeURIComponent(dataSet.content) + "&max=" + dataSet.max + "&maxTime=" + dataSet.maxTimeSelected + "&";
}


/**
 * Create the max per time url.
 * @return The max per time part of a set alarm action url.
 */
EmailActionDetail.prototype.maxPerTimeUrl = function() {
	var max = getInput(this.getHtmlCode(), "max");
	if (isEmpty(max.value)) {
		throw new Error ("No max value set");
	}	
	if (!isNumberString(max.value)) {
		throw new Error ("Invalid max value");
	}
	var maxPerTime = getElementWithAttribute(this.getHtmlCode(), "maxPerTime", "name", "select");
	var maxTime = maxPerTime.options[maxPerTime.selectedIndex].value;
	if ("-1".equalsIgnoreCase(maxTime)) {
		throw new Error ("No max time selected");
	}
	return "max=" + max.value + "&maxTime=" + maxTime + "&";
}

 
/**
 * Creates the subject part of a set alarm action url.
 * @return The subject part of a set alarm action url.
 */
EmailActionDetail.prototype.subjectUrl = function() {
	var subject = getInput(this.getHtmlCode(), "subject");
	if (isEmpty(subject.value) || DEFAULT_SUBJECT_VALUE == subject.value) {
		throw new Error ("No subject set");
	}
	return "subject=" + encodeURIComponent(subject.value) + "&";
}


/**
 * Create the to address part of a set alarm action url.
 * @return The to part of a set alarm action url.
 */
EmailActionDetail.prototype.toUrl = function() {
	var to = getInput(this.getHtmlCode(), "toAddress");
	if (isEmpty(to.value) || DEFAULT_TO_VALUE == to.value) {
		throw new Error("No to address set");
	}
	var tokenizer = new StringTokenizer(to.value, ",");
	var addresses = "";
	while(true) {
		if (!tokenizer.hasMoreTokens()) {
			break;
		}
		var token = tokenizer.nextToken();
		if (!isEmpty(token)) {
			addresses += isEmpty(addresses) ? "" : ",";
			addresses += trim(token);
		}
	}
	return "to=" + addresses + "&";
}


/**
 * Set the email config not set error message.
 */
EmailActionDetail.prototype.setEmailConfigNotSetMessage = function() {
	this.clearAlarmActionMessage();
	var messageContainer = getElementWithAttribute(this.getHtmlCode(), "alarmActionmessage", "name", "div");
	setClass(messageContainer, "messageContainer error");
	var link = document.createElement("a");
	messageContainer.appendChild(link);
	link.appendChild(document.createTextNode(EMAIL_CONFIG_NOT_SET_MESSAGE + ". Click to set e-mail configuration"));
	link.style.cursor = "pointer";
	setClass(link, "error");
	link.onclick = function() {
		if (exists(top.frames["Setup"])) {
			// the frame has been loaded already, activate the tab and show the email config window
			top.tabSection.activateTab("Setup");
			top.frames["Setup"].showEmailWindow();
		}
		else {
			// the frame has not been loaded yet
			top.tabSection.activateTab("Setup");
			top.showWindow = "emailConfiguration";
		}
		top.returnTab = currentTabName;
	}
}


// implementation


/**
 * A action has ben set/edited or added to the action library, parse the response from the
 * server and show any message. This is overridden to show the email config not set link
 * when necessary.
 * @param response The response from the server.  
 */
EmailActionDetail.prototype.parseReply = function(response) {
	var argumentsToSuper = new Array();
	argumentsToSuper[0] = response;
	this.callSuper("parseReply", argumentsToSuper);
	
	if (containsErrorMessage(response) && EMAIL_CONFIG_NOT_SET_MESSAGE == getErrorMessage(response)) {
		this.setEmailConfigNotSetMessage();
	}
}