/**
 * Shows the email actions. 
 */


// constants


/**
 * The base url used to load email alarm actions. The session id and the alarm id need to be
 * appended to this url.
 */
var LOAD_EMAIL_ACTION_BASE_URL = "/?resource=AlarmManager&mode=get&command=emailAlarmAction&";
 

/**
 * The base url used to load library email alarm actions. The session id needs to be appended to this url.
 */
var LOAD_LIBRARY_EMAIL_ACTION_BASE_URL = "/?resource=AlarmManager&mode=get&command=libraryEmailAction&";


// construction


/**
 * Factory method to create new email alarm action table objects.
 * @param id The id of the table to update.
 * @param url The url to use to update the table.
 * @param parentElement The element the table is to be inserted in. 
 * @param windowId The id of the window the table is in. 
 * @param Optional time parameter. The time between each update.
 * If this is not specified the default value is used.
 * @returns A new alarm action table object.
 * @throws Error if id is undefined, null or empty.
 * @throws Error if parentElement is undefined, or null.
 * @throws Error if windowId is undefined, null or empty.
 */
function createEmailActionTable(id, url, parentElement, windowId, time) {
	assertNotEmpty(id, "Id for the table is undefined, empty or null: " + id);
	assertDefined(parentElement, "The element to insert the table into is undefined or null");
	assertNotEmpty(windowId, "windowId to set for the table is undefined empty or null: " + windowId);
	
	var subInstance = new EmailActionsTable();
	var superInstance = createActionTable(id, url, parentElement, windowId, time);
	var instance = createSubClass(superInstance, subInstance);
	instance["supercreateDataTable"] = superInstance["supercreateDataTable"];
	instance["superupdateTableRow"] = superInstance["superupdateTableRow"];
	instance["supersetData"] = superInstance["supersetData"];
	
	TableManager.instance().removeTable(TableManager.instance().getTable(id));
	TableManager.instance().registerNewTable(instance);

	// let the event processor call event handler functions as a method
	// of the mytable object instance
	superInstance.getEventProcessor().changeObject(instance);
	
	// overwrite the repeater to call the load function on the MyTable object	
	instance.repeater = new Repeater(load, instance, time);
	instance.eventListener.addEvent(WINDOW_SHOWN_EVENT, processAlarmActionWindowShownEvent);

	return instance;
}



/**
 * Subclasses actiontabls.js to add behaviour needed in the email alarmactions table.
 */
function EmailActionsTable() {
	
}


// services


/**
 * Set data into the table based on a xmlDocument. To 
 * show the data on screen call paint(). This is overridden to add the to field.
 * The to field might contain multiple recipient mail addresses. 
 * @param xmlDoc The data in xml format.
 * @param tagName The name of the tag the data for each row is in.
 * @param throws Error if xmlDoc is undefined or null.
 * @param throws Error if tagName is undefined, null or empty.
 */
EmailActionsTable.prototype.setData = function (xmlDoc, tagName) {
	assertDefined(xmlDoc, "XmlDoc is undefined or null");
	assertNotEmpty(tagName, "TagName is undefined, null or empty");
	var rows = xmlDoc.getElementsByTagName(tagName);
	
	var len = rows.length;
	for (var i = 0; i < len; i++) {
		var row = this.parseXmlRow(rows[i]);
		row[row.length] = this.createToField(row)
		this.rows[this.rowCounter] = row;
		this.rowCounter++;
	}
	
	if (!this.sortable) {
		return;
	}
	
	// sort the table
	if (exists(this.columnToSort)) {
		this.sort(this.columnToSort);
	}
	if (this.reversed) {
		this.reverseColumn(this.columnToSort);
	}
}



/**
 * Adds a action to the table. The action to add must be a table row from another table. This
 * is used to copy actions from the add/edit action window to the action library.
 * @param action The action to add as a table row. 
 */
EmailActionsTable.prototype.addAction = function(action) {
	WindowManager.instance().activateWindow(this.window.getId());
	this.storeAddActionFieldData();
	getInput(this.window.getHtmlCode(), "toAddress").value = getFieldFromRow("to", action).getValue();
	getInput(this.window.getHtmlCode(), "subject").value = getFieldFromRow("subject", action).getValue();
	getElementWithAttribute(this.window.getHtmlCode(), "content", "name", "textarea").value = getFieldFromRow("content", action).getValue();
	this.window.setAlarmAction();
	this.restoreAddActionFields();
}


// implementation


/**
 * Creates the to field, based on the recipients set. 
 * @row The table row to create the to field for. 
 * @return The created to field.
 */
EmailActionsTable.prototype.createToField = function(row) {
	// get all the recipients fields
	var recipients = "";
	for (var i = 0; i < row.length; i++) {
		if ("recipient".equalsIgnoreCase(row[i].getName())) {
			recipients += isEmpty(recipients) ? "" : ",";
			recipients += row[i].getValue();
		}
	}
	return new Field("to", recipients);
}


/**
 * Inserts a field value into a given html node. 
 * @param field The field to insert the value for.
 * @param node The node to insert the value into.
 */
EmailActionsTable.prototype.insertFieldValue = function(field, node, cell) {
	if (!this.isHeader(field) && field.getName().indexOf("subject") > -1) {
		node.appendChild(document.createTextNode(URLDecode(field.getValue())));
	}
	else {
		node.appendChild(document.createTextNode(field.getValue()));
	}
}