/**
 * This class shows program action details, and lets the user add/edit email actions. 
 */
 
 
// constants


/**
 * The base url used to set a email alarm action.
 */
var SET_PROGRAM_ACTION_BASE_URL = "/?resource=AlarmManager&mode=set&command=programAlarmAction&action=set&sid=" + getSid() + "&";


/**
 * The base url to set a library email alarm action.
 */
var SET_LIBRARY_PROGRAM_ACTION_BASE_URL = "/?resource=AlarmManager&mode=set&command=libraryProgramAction&action=set&sid=" + getSid() + "&";


/**
 * The request used to load program names. The session id need to be added to 
 * this a valid request. 
 */
var PROGRAM_NAME_REQUEST = "/?resource=AlarmManager&mode=get&command=programNames&sid=";
		
		
/**
 * The default value in the program select. 
 */
var DEFAULT_PROGRAM_SELECT = "Select a program";


/**
 * The default value in the description field. 
 */
var DEFAULT_DESCRIPTION_FIELD_VALUE = "[Enter a description]";


/**
 * The default value in the parameters field. 
 */
var DEFAULT_PARAMETERS_FIELD_VALUE = "[Enter the parameters]";



/**
 * This class is used to represent program actino detail data. All the program action fields are represented
 * by a variabel in this class.
 */
function ProgramActionDataSet() {
	var program;
	var parameters;
	var description;
}


// construction
 
 
/**
 * Factory method to create a program action detail object.
 * @param setActionBaseUrl The base url used to set a email alarm action. 
 * @param editActionBaseUrl The base url used to edit a email alarm action.
 * @param addToLibraryUrl The base url used to add a action to the action library.  
 * @param htmlCode The html code containing the email detail section.
 * @throws Error if setActionBaseUrl is undefined, null or empty. 
 * @throws Error if editActionBaseUrl is undefined, null or empty. 
 * @throws Error if htmlCode is null or undefined.
 */
function createProgramActionDetail(setActionBaseUrl, editActionBaseUrl, addToLibraryBaseUrl, htmlCode) {
	assertNotEmpty(setActionBaseUrl, "The set action base url is undefined, null or empty", setActionBaseUrl);
	assertNotEmpty(editActionBaseUrl, "The edit action base url is undefined, null or empty", editActionBaseUrl);
	assertExists(htmlCode, "The html code to set is null or undefined");
	var subInstance = new ProgramActionDetail(htmlCode);
	var superInstance = new ActionDetail(setActionBaseUrl, editActionBaseUrl, addToLibraryBaseUrl, htmlCode);
	subInstance = createSubClass(superInstance, subInstance);
	
	return subInstance;
}


/**
 * The email action detail class. 
 * @param htmlCode The html code containing the email detail section.
 * @throws Error if htmlCode is null or undefined.
 */
function ProgramActionDetail(htmlCode) {
	assertExists(htmlCode, "The html code to set is null or undefined");
	this.htmlCode = htmlCode;
	this.programSelect = getElementWithAttribute(this.htmlCode, "programSelect", "name", "select");
	this.parameters = getInput(this.htmlCode, "parameters");
	this.description = getInput(this.htmlCode, "description");
}


// services


/**
 * Returns the selected program.
 */
ProgramActionDetail.prototype.getProgram = function() {
	return this.programSelect.options[this.programSelect.selectedIndex].value;
}


/**
 * Selects a program name. 
 * @param name The program name to select. 
 * @throws Error if name is undefined, null or empty. 
 */
ProgramActionDetail.prototype.setProgram = function(name) {
	assertNotEmpty(name, "The program name to select is undefined, null or empty");
	selectInSelector(this.programSelect, name);
}


/**
 * Gets the current set parameters. 
 */
ProgramActionDetail.prototype.getParameters = function() {
	return this.parameters.value;
}


/**
 * Sets the parameters to a given value. 
 * @param parameters The parameters to set. 
 * @throws Error if parameters is undefined or null. 
 */
ProgramActionDetail.prototype.setParameters = function(parameters) {
	assertExists(parameters, "The parameters to set is undefined or null");
	this.parameters.value = parameters;
}


/**
 * Gets the current set description. 
 */
ProgramActionDetail.prototype.getDescription = function(description) {
	return this.description.value;
}


/**
 * Sets the description field to a given value. 
 * @description The description to set.
 * @throws Error if description is undefined or null.  
 */
ProgramActionDetail.prototype.setDescription = function(description) {
	assertExists(description, "The description to set is undefined or null");
	this.description.value = description;
}


/**
 * Loads the program names. 
 */
ProgramActionDetail.prototype.loadPrograms = function() {
	loadXML(PROGRAM_NAME_REQUEST + getSid(), "programsLoaded", this);
}


/**
 * Callback method for loading of the program names. 
 * @param response The response from the server. 
 */
ProgramActionDetail.prototype.programsLoaded = function(response) {
	var programs = getTagNodeValues("program", response);
	var selectedValue = this.programSelect.options[this.programSelect.selectedIndex].value;
	deleteOptions(this.programSelect);
	this.programSelect.add(new Option(DEFAULT_PROGRAM_SELECT, DEFAULT_PROGRAM_SELECT), undefined);
	for (var i = 0; exists(programs) && i < programs.length; i++) {
		this.programSelect.add(new Option(URLDecode(programs[i]), URLDecode(programs[i])), undefined);
	}
	selectInSelector(this.programSelect, selectedValue);
}


/**
 * The parameters field has been focused, clear the default message if it has not been changed. 
 */
ProgramActionDetail.prototype.parametersFocused = function() {
	if (this.parameters.value == DEFAULT_PARAMETERS_FIELD_VALUE) { 
		this.parameters.value = "";
	}
	this.clearAlarmActionMessage();
}


/**
 * The description field has been focusd, clear the default message if it has not been changed. 
 */
ProgramActionDetail.prototype.descriptionFocused = function() {
	if (this.description.value == DEFAULT_DESCRIPTION_FIELD_VALUE) { 
		this.description.value = "";
	}
	this.clearAlarmActionMessage();
}


/**
 * A variable has been selected, insert it into the parameter field.
 */
ProgramActionDetail.prototype.variableSelected = function() {
	var variableSelector = getElementWithAttribute(this.getHtmlCode(), "variableSelector", "name", "select");
	var selectedOption = variableSelector.options[variableSelector.selectedIndex].text;
	if (DEFAULT_VARIABLE_SELECT_OPTION == selectedOption) {
		// the default option was selected, nothing to insert
		return;
	}
	if (DEFAULT_PARAMETERS_FIELD_VALUE == this.parameters.value) {
		this.parameters.value = "";
	}
	this.parameters.value += variables[selectedOption] + " ";
	this.parameters.focus();
}


/**
 * Returns the part of a url with the program actions parameters set, based on the values set in the
 * alarm action detail window. 
 */
ProgramActionDetail.prototype.createActionParameterUrl = function() {
	var alarmId = getInput(this.getHtmlCode(), "alarmId");
	return this.programUrl() + this.parametersUrl() +  this.descriptionUrl() + (alarmId == null ? "" : "alarmId=" + alarmId.value)  + "&";
}


/**
 * Returns the part of a url with the program actions parameters set, based on the value in a given dataset.
 * @param dataSet The data set to use to create the url.
 * @throws Error if dataSet is undefined or null.
 */
ProgramActionDetail.prototype.createActionParameterUrlDataSet = function(dataSet) {
	assertExists(dataSet, "The data set to create the parameter url for is undefined or null");
	return "programName=" + encodeURIComponent(dataSet.program) + "&parameters=" + encodeURIComponent(dataSet.parameters) + "&description=" + encodeURIComponent(dataSet.description) + "&"; 
}

 
/**
 * Copies a action from the alarm action library into the add/edit alarm action window.
 */	
ProgramActionDetail.prototype.addFromLibrary = function() {
	var actionChooser = getElementWithAttribute(this.getHtmlCode(), "libraryActionChooser", "name", "select");
	var actionId = actionChooser.options[actionChooser.selectedIndex].value;
	if (actionId == -1) {
		return;
	}
	
	var action = TableManager.instance().getTable("programAlarmActionLibraryTable").getRowWithFieldValue("id", actionId);
	selectInSelector(this.programSelect, URLDecode(getFieldFromRow("programName", action).getValue()));
	var parameters = URLDecode(getFieldFromRow("parameters", action).getValue())
	this.parameters.value = isEmpty(parameters) ? DEFAULT_PARAMETERS_FIELD_VALUE : parameters;
	var description = URLDecode(getFieldFromRow("description", action).getValue());
	this.description.value = isEmpty(description) ? DEFAULT_DESCRIPTION_FIELD_VALUE : description;
}


/**
 * Populates the action chooser with actions from the action library.
 */
ProgramActionDetail.prototype.populateActionChooser = function() {
	var table = TableManager.instance().getTable("programAlarmActionLibraryTable");
	var actionChooser = getElementWithAttribute(this.getHtmlCode(), "libraryActionChooser", "name", "select");
	if (!exists(actionChooser)) {
		return;
	}
	if (!table.isUpdating()) {
		table.restartUpdate();
	}
	
	var data = table.getData();
	var selectedIndex = actionChooser.selectedIndex;
	var selectedText = actionChooser.options[actionChooser.selectedIndex].text;
	deleteOptions(actionChooser);
	actionChooser.add(new Option("Choose a action from the library", "-1"), undefined);
	for (var i = 0; i < data.length; i++) {
		var id = getFieldFromRow("id", data[i]);
		var programName = getFieldFromRow("programName", data[i]);
		actionChooser.add(new Option(URLDecode(programName.getValue()), id.getValue()), undefined);
	}
	// select the option selected before the new options are inserted, if the selected option has been
	// deleted select the default option
	selectedIndex = getOptionIndex(selectedText, actionChooser) ;
	actionChooser.selectedIndex = (selectedIndex > actionChooser.options.length || selectedIndex == -1) ? 0 : selectedIndex;
}



/**
 * Clears the add alarm action fields.
 */
ProgramActionDetail.prototype.clearAlarmActionFields = function() {
	var librarySelect = getElementWithAttribute(this.getHtmlCode(), "libraryActionChooser", "name", "select");
	if (exists(librarySelect)) {
		librarySelect.selectedIndex = 0;
	}	
	this.programSelect.selectedIndex = 0;
	this.parameters.value = DEFAULT_PARAMETERS_FIELD_VALUE;
	this.description.value = DEFAULT_DESCRIPTION_FIELD_VALUE;
	getInput(this.getHtmlCode(), "actionId").value = "";
	this.clearAlarmActionMessage();
}


// implementation


/**
 * Create the parameter part of a program action url.
 * @return The parameter part of a program action url.
 */
ProgramActionDetail.prototype.parametersUrl = function() {
	var parameters = "";
	if (this.parameters.value != DEFAULT_PARAMETERS_FIELD_VALUE) { 
		parameters = this.parameters.value;
	}
	return "parameters=" + encodeURIComponent(parameters) + "&";
}


/**
 * Create the description part of a program action url.
 * @return The description part of a program action url. 
 */
ProgramActionDetail.prototype.descriptionUrl = function() {
	var description = "";
	if (this.description.value != DEFAULT_DESCRIPTION_FIELD_VALUE) {
		description = this.description.value;
	}
	return "description=" + encodeURIComponent(description) + "&";
}


/**
 * Create the program name part of a set program action url.
 * @return The progam name part of a set program action url.
 */
ProgramActionDetail.prototype.programUrl = function() {
	if (this.programSelect.options[this.programSelect.selectedIndex].text == DEFAULT_PROGRAM_SELECT) {
		throw new Error("No program selected");
	}	
	return "programName=" + encodeURIComponent(this.programSelect.options[this.programSelect.selectedIndex].text) + "&";
}
