

/**
 * This defines the different units of measure for an alarm.
 */
 
	
// defined units
	
	
	/**
	 * Bytes for the entire sampling time.
	 */
	var BYTES = new AlarmUnit("Bytes", false, 1);
	
	
	/**
	 * Kilobytes for the entire sampling time.
	 */
	var KILO_BYTES = new AlarmUnit("KB", false, 1024);
	
	
	/**
	 * Megabytes for the entire sampling time.
	 */
	var MEGA_BYTES = new AlarmUnit("MB", false, KILO_BYTES.byteCount * 1024);
	
	
	/**
	 * Gigabytes for the entire sampling time.
	 */
	var GIGA_BYTES = new AlarmUnit("GB", false, MEGA_BYTES.byteCount * 1024);
	
	
	/**
	 * Bits per second.
	 */
	var BITS = new AlarmUnit("bit/s", true, 1/8);
	
	
	/**
	 * Kilobits per second.
	 */
	var KILO_BITS = new AlarmUnit("Kbit/s", true, BITS.byteCount * 1000);
	
	
	/**
	 * Megabits per second.
	 */
	var MEGA_BITS = new AlarmUnit("Mbit/s", true, KILO_BITS.byteCount * 1000);
	
	
	/**
	 * Gigabits per second.
	 */
	var GIGA_BITS = new AlarmUnit("Gbit/s", true, MEGA_BITS.byteCount * 1000);
	

	/**
	 * The number of packets.
	 */
	var PACKETS = new AlarmUnit("Packets", false);
	
	
	/**
	 * The number of packets per second.
	 */
	var PACKETS_SECOND = new AlarmUnit("Packets/sec", true);
	
	
	/**
	 * The number of entries (protocols, hosts, conversations, etc.).
	 */
	var ENTRIES = new AlarmUnit("Entries", false);
	
	
	/**
	 * The number of entries per second.
	 */
	var ENTRIES_SECOND = new AlarmUnit("Entries/sec", true);


// construction


/**
 * Creates a new unit with the specified label.
 * @param label The label for this unit type.
 * @param perSecond True if this is a per-second type of unit.
 * @param byteCount The number of bytes for this unit of measure.
 */
function AlarmUnit(label, perSecond, byteCount) {
	this.label = label;
	this.perSecond = perSecond;
	if (exists(byteCount)) {
		this.byteCount = byteCount;
	}
}



// services
	

/**
 * Calculates the correct value of this unit based on the number of bytes given.
 * @param bytes The bytes from which to calculate.
 */
AlarmUnit.prototype.calculate = function(bytes) {
	if (this == PACKETS && this == PACKETS_SECOND && this == ENTRIES && this == ENTRIES_SECOND) {
		throw new Error("Unit not bytes");
	}
	return bytes / this.byteCount;
}


/**
 * Calculates a value of this unit in bytes.
 * @param unitValue The value in this unit to calculate into bytes.
 * @return unitValue in bytes. 
 */
AlarmUnit.prototype.toBytes = function(unitValue) {
	if (this == PACKETS && this == PACKETS_SECOND && this == ENTRIES && this == ENTRIES_SECOND) {
		throw new Error("Unit not bytes");
	}
	return unitValue * this.byteCount;
}


/**
 * Returns true if this unit is a per-second type.
 */
AlarmUnit.prototype.isPerSecond = function() {
	return this.perSecond;
}


/**
 * Returns true if this unit is a byte-type unit.
 */
AlarmUnit.prototype.isByteType = function() {
	return this == BYTES || this == KILO_BYTES || this == MEGA_BYTES || this == GIGA_BYTES || this == BITS || this == KILO_BITS || this == MEGA_BITS || this == GIGA_BITS;
}


/**
 * Returns the unit label.
 */
AlarmUnit.prototype. toString = function() {
	return this.label;
}


/**
 * Returns the correct instance based on the unit label.
 * @param label The unit label.
 */
AlarmUnit.getInstance = function(label) {
	if (BYTES.label.equalsIgnoreCase(label)) {
		return BYTES;
	}
	if (KILO_BYTES.label.equalsIgnoreCase(label)) {
		return KILO_BYTES;
	}
	if (MEGA_BYTES.label.equalsIgnoreCase(label)) {
		return MEGA_BYTES;
	}
	if (GIGA_BYTES.label.equalsIgnoreCase(label)) {
		return GIGA_BYTES;
	}
	if (BITS.label.equalsIgnoreCase(label)) {
		return BITS;
	}
	if (KILO_BITS.label.equalsIgnoreCase(label)) {
		return KILO_BITS;
	}
	if (MEGA_BITS.label.equalsIgnoreCase(label)) {
		return MEGA_BITS;
	}
	if (GIGA_BITS.label.equalsIgnoreCase(label)) {
		return GIGA_BITS;
	}
	if (PACKETS.label.equalsIgnoreCase(label)) {
		return PACKETS;
	}
	if (PACKETS_SECOND.label.equalsIgnoreCase(label)) {
		return PACKETS_SECOND;
	}
	if (ENTRIES.label.equalsIgnoreCase(label)) {
		return ENTRIES;
	}
	if (ENTRIES_SECOND.label.equalsIgnoreCase(label)) {
		return ENTRIES_SECOND;
	}
	return null;
}