

/**
 * The base url used to delete a alarm.
 */
var DELETE_TRIGGERED_ALARM_BASE_URL = "/?resource=AlarmManager&mode=set&command=deleteTriggeredAlarm&sid=" + getSid();


// construction


/**
 * Factory method to create new triggeredAlarmTable objects.
 * @param id The id of the table to update.
 * @param url The url to load the data from.
 * @param parentElement The element the table is to be inserted in. 
 * @param windowId The id of the window the table is in. 
 * @param Optional time parameter. The time between each update.
 * @returns A new probeTable object.
 * If this is not specified the default value is used.
 * @throws Error if id is undefined, null or empty.
 * @throws Error if url is undefined, null or empty.
 * @throws Error if parentElement is undefined, or null.
 * @throws Error if windowId is undefined, null or empty.
 */
function createTriggeredAlarmTable(id, url, parentElement, windowId, time) {
	assertNotEmpty(id, "Id for the table is undefined, empty or null: " + id);
	assertNotEmpty(url, "Url to set for the table is undefined empty or null: " + url);
	assertDefined(parentElement, "The element to insert the table into is undefined or null");
	assertNotEmpty(windowId, "windowId to set for the table is undefined empty or null: " + windowId);

	var subInstance = new TriggeredAlarmTable();
	var superInstance = createAlarmTable(id, url, parentElement, windowId, time);
	var instance = createSubClass(superInstance, subInstance);
	
	TableManager.instance().removeTable(TableManager.instance().getTable(id));
	TableManager.instance().registerNewTable(instance);

	// let the event processor call event handler functions as a method
	// of the mytable object instance
	superInstance.getEventProcessor().changeObject(instance);

	// overwrite the repeater to call the load function on the AlarmTable object	
	instance.repeater = new Repeater(load, instance, time);

	return instance;
}



/**
 * Subclasses alarmtable.js to add specifc behaviour for triggered alarms.
 */
function TriggeredAlarmTable() {
}


// implementation


/**
 * Creates the url needed to delete a alarm.
 * @param value The value for the id field of the row to delete. 
 * @return The url to delete the entry.
 */
TriggeredAlarmTable.prototype.createDeleteEntryUrl = function(value) {
	var id = getParameterValue(value, "id", "&");
	var url = DELETE_TRIGGERED_ALARM_BASE_URL + "&id=" + id + "&"; 
	var srcIp = getFieldFromRow("srcIp", this.rows[this.getIdRowNumber(value)]);
	if (exists(srcIp)) {
		url += "srcIp=" + srcIp.getValue() + "&";
	}
	return url;
}


/**
 * Adds the row class to a row. Overridden to set colors for the rows depending
 * on their severity.
 * @param row The html row to set the class for.
 * @param index The index of the row in the table.
 */
TriggeredAlarmTable.prototype.addRowClass = function(row, index) {
	var severityField = getFieldFromRow("severity", this.rows[index]);
	if (this.mouseOverClass == getClassName(row) && row.counter > 0) {
		// the class set for the row is the mouseover class and the mouse
		// is still over the row, do not change the class
		return;
	}
	
	if ("Critical".equalsIgnoreCase(severityNames[severityField.getValue()])) {
		setClass(row, "critical");
	}
	else if ("Serious".equalsIgnoreCase(severityNames[severityField.getValue()])) {
		setClass(row, "serious");
	}
	else if ("Warning".equalsIgnoreCase(severityNames[severityField.getValue()])) {
		setClass(row, "warning");
	}
	else if ("Information".equalsIgnoreCase(severityNames[severityField.getValue()])) {
		setClass(row, "information");
	}
}


/**
 * Creates the text to show in a alarm tooltip.
 * @param alarm Array containing the alarm fields. 
 * @return The text to shown in the tooltip for the alarm.  
 */
TriggeredAlarmTable.prototype.createAlarmTooltipText = function(alarm) {
	var label = "Alarm type: ";
	if (getFieldFromRow("absoluteAlarm", alarm).getValue() == "true") {
		label += "absolute";
	}
	else {
		label += "delta";
	}
	label += "<br>Trigger type: " + getFieldFromRow("triggerType", alarm).getValue();
	label += "<br>Upper trigger: " + getFieldFromRow("upperTriggerValue", alarm).getValue() + " " + getFieldFromRow("upperUnit", alarm).getValue();
	label += "<br>Lower trigger: " + getFieldFromRow("lowerTriggerValue", alarm).getValue() + " " + getFieldFromRow("lowerUnit", alarm).getValue();
	label += "<br>Visibility: " + visibilityNames[getFieldFromRow("visibility", alarm).getValue()];
	return label;
}


/**
 * Inserts a severity name into a given html node. The severity number 
 * is replaced with its name.
 * @param field The field to insert the value for.
 * @param node The node to insert the value into.
 */
TriggeredAlarmTable.prototype.insertSeverityName = function(field, node, cell) {
	node.appendChild(document.createTextNode(severityNames[field.getValue()]));
}
