

// construction


/**
 * Factory method to create new alarmTable objects.
 * @param id The id of the table to update.
 * @param url The url to load the data from.
 * @param parentElement The element the table is to be inserted in. 
 * @param windowId The id of the window the table is in. 
 * @param Optional time parameter. The time between each update.
 * @returns A new probeTable object.
 * If this is not specified the default value is used.
 * @throws Error if id is undefined, null or empty.
 * @throws Error if url is undefined or null.
 * @throws Error if parentElement is undefined, or null.
 * @throws Error if windowId is undefined, null or empty.
 */
function createTriggerValuesTable(id, url, parentElement, windowId, time) {
	assertNotEmpty(id, "Id for the table is undefined, empty or null: " + id);
	assertExists(url, "Url to set for the table is undefined or empty: " + url);
	assertDefined(parentElement, "The element to insert the table into is undefined or null");
	assertNotEmpty(windowId, "windowId to set for the table is undefined empty or null: " + windowId);

	var subInstance = new TriggerValuesTable();
	var superInstance = createProbeTable(id, url, parentElement, windowId, time);
	var instance = createSubClass(superInstance, subInstance);
	instance["supersetData"] = superInstance["supersetData"];
	
	TableManager.instance().removeTable(TableManager.instance().getTable(id));
	TableManager.instance().registerNewTable(instance);

	// let the event processor call event handler functions as a method
	// of the alarmTable object instance
	superInstance.getEventProcessor().changeObject(instance);
	instance.eventListener.addEvent("resized", processAlarmValueTableResizedEvent);

	return instance;
}


/**
 * Subclasses table.js to add specifc behaviour for alarm tables.
 */
function TriggerValuesTable() {
}



// event processing


/**
 * Process a resize event.
 * @param event The resize event to process.
 */
function processAlarmValueTableResizedEvent(event) {
	if (event.getData()["id"] != this.windowName) {
		// the table only cares about events from its window, none of the others
		return;
	}
	if (this.noAccessShown) {
		return;
	}
	
	var resizedWidth = event.getData()["width"];
	var resizedHeight = event.getData()["height"];
	this.resizeTable(resizedHeight, resizedWidth);
}


// implementation



/**
 * Set data into the table based on a triggered alarm row. To 
 * show the data on screen call paint().
 * @param alarm The alarm row to insert. 
 * @param throws Error if alarm is undefined or null.
 */
TriggerValuesTable.prototype.setData = function (alarm) {
	assertExists(alarm, "The triggered alarm to insert is undefined or null");
	this.parseAlarm(alarm);
	
	if (!this.sortable) {
		return;
	}
	
	// sort the table
	if (exists(this.columnToSort)) {
		this.sort(this.columnToSort);
	}
	if (this.reversed) {
		this.reverseColumn(this.columnToSort);
	}
}


/**
 * Parses the trigger time and values of a triggered alarm. 
 * @param alarm The alarm to parse. 
 */
TriggerValuesTable.prototype.parseAlarm = function(alarm) {
	var triggerValueTokenizer = new StringTokenizer(getFieldFromRow("triggerValue", alarm).getValue(), ",");
	var triggerTimeTokenizer = new StringTokenizer(getFieldFromRow("triggerTime", alarm).getValue(), ",");
	while(true) {
		if (!triggerValueTokenizer.hasMoreTokens()) {
			break;
		}
		var row = new Array();
		row[0] = new Field("triggerTime", triggerTimeTokenizer.nextToken());
		row[1] = new Field("triggerValue", triggerValueTokenizer.nextToken());
		this.rows[this.rowCounter] = row;
		this.rowCounter++;
	}
}


/**
 * Inserts a field value into a given html node. 
 * @param field The field to insert the value for.
 * @param node The node to insert the value into.
 */
TriggerValuesTable.prototype.insertFieldValue = function(field, node, cell) {
	if (!this.isHeader(field) && field.getName().indexOf("triggerTime") > -1) {
		this.insertTriggerTime(field, node, cell);
	}
	else {
		node.appendChild(document.createTextNode(field.getValue()));
	}
}


/**
 * Inserts the formatted trigger time into a given html node. 
  *@param field The field to insert the value for.
 * @param node The node to insert the value into.
 */
TriggerValuesTable.prototype.insertTriggerTime = function(field, node, cell) {
	var date = new Date(parseInt(field.getValue()));
	var dateString = "";
	var day = date.getDate() < 10 ? "0" + date.getDate() : date.getDate();	
	// the months start at 0
	var month = (date.getMonth() + 1) < 10 ? "0" + (date.getMonth() + 1) : (date.getMonth() + 1);
	// the getYear methods returns the number of years since 1900
	var year = (date.getYear() + 1900) + "";
	year = year.substring(2);
	var minutes = date.getMinutes() < 10 ? "0" + date.getMinutes() : date.getMinutes();
	var seconds = date.getSeconds() < 10 ? "0" + date.getSeconds() : date.getSeconds();
	dateString = day + "." + month + "." + year + " " +  date.getHours() + ":" + minutes + ":" + seconds;
	node.appendChild(document.createTextNode(dateString));
}
