/**
 * This class manages and updates a chart. 
 */


/**
 * A chart of this type represents the data in the table, and will be resized
 * when the topN changes. Charts of this type are also not closable in the windows.
 */
Chart.TOPN_CHART = 0;


/**
 * A chart of this type will not be resized when the topN of the table changes.
 */
Chart.INTERVAL_CHART = 1;


/**
 * This is a chart container. A chart container contains a line chart and a gauge chart side by side. 
 */
Chart.CHART_CONTAINER = 2;


/**
 * This is a line chart container. A line chart container can contain multiple line charts.  
 */
Chart.LINE_CHART_CONTAINER = 3;


/**
 * The default height set for topn charts, my charts, and interval charts. 
 */
Chart.DEFAULT_HEIGHT = 155;


/**
 * The default height set for chart containers. 
 */
Chart.CHART_CONTAINER_DEFAULT_HEIGHT = 200;


/**
 * Ths is the max number of updates that can be done on a chart before at least one update completes.
 * If chart are updated faster than they are loaded by the browser no new charts are shown, 
 * because a new update starts before the old one is finished. When this max counter is reached
 * the chart is not updated any more until the last update has completed.
 */
Chart.MAX_UPDATE_COUNTER = 5;


/**
 * The base url to get the next interval chart data. The resource (Statistics.protocol)
 * and the sid have to be added to the url.
 */
Chart.INTERVAL_CHART_DATA_BASE_URL = "mode=getdata&command=intervalchartdata&";


/**
 * The url to get the next interval time for a chart. The resource (Statistics.protocol) 
 * and the sid have to be added to the url. 
 */
Chart.NEXT_INTERVAL_BASE_URL = "mode=getdata&command=nextinterval&";


/**
 * Parameters used in the line charts.
 */
Chart.LINECHART_PARAMETERS = "chart=line&width=792&height=0&background=%23ffffff|%23cccccc&3dmodeon=true&antialias=true&3ddepth=5&valueLinesOn=true&stackedOn=true&sampleColors=%231e7096f0|%232492c7f0&alternateChartBackground=%23e7e7e7&servletMode=true&";


/**
 * Mapping between javascript and java tables names. 
 */
var TABLE_NAME_MAP = new Array();
TABLE_NAME_MAP["summaryTable"] = "summary";
TABLE_NAME_MAP["topNProtocolsTable"] = "protocol";
TABLE_NAME_MAP["myProtocolsTable"] = "protocol";
TABLE_NAME_MAP["topNHostsTable"] = "nlhost";
TABLE_NAME_MAP["myHostsTable"] = "nlhost";
TABLE_NAME_MAP["topNHostListenersTable"] = "nlhost";
TABLE_NAME_MAP["topNHostTalkersTable"] = "nlhost";
TABLE_NAME_MAP["topNConversationsTable"] = "nlmatrix";
TABLE_NAME_MAP["myConversationsTable"] = "nlmatrix";
TABLE_NAME_MAP["myProtocolsPerHostTable"] = "alhost";


/**
 * This array contains the base links for each table.
 */ 
var baseUrl = new Array();
baseUrl["summaryTable"] = "/?mode=getdata&command=intervalchart&resource=Statistics.summary&sid=";
baseUrl["topNProtocolsTable"] = "/?mode=getdata&command=intervalchart&resource=Statistics.protocol&sid=";
baseUrl["myProtocolsTable"] = "/?mode=getdata&command=intervalchart&resource=Statistics.protocol&sid=";
baseUrl["topNHostsTable"] = "/?resource=Statistics.nlhost&command=intervalchart&mode=getdata&sid=";
baseUrl["myHostsTable"] = "/?resource=Statistics.nlhost&command=intervalchart&mode=getdata&sid=";
baseUrl["topNHostListenersTable"] = "/?resource=Statistics.nlhost&command=intervalchart&mode=getdata&sid=";
baseUrl["topNHostTalkersTable"] = "/?resource=Statistics.nlhost&command=intervalchart&mode=getdata&sid=";
baseUrl["topNConversationsTable"] = "/?resource=Statistics.nlmatrix&command=intervalchart&mode=getdata&sid=";
baseUrl["myConversationsTable"] = "/?resource=Statistics.nlmatrix&command=intervalchart&mode=getdata&sid=";
baseUrl["myProtocolsPerHostTable"] = "/?resource=Statistics.alhost&command=intervalchart&mode=getdata&sid=";
baseUrl["protocolDetailTable"] = "/?mode=getdata&command=intervalchart&resource=Statistics.protocol&sid=";
baseUrl["protocolTopHostsTable"] = "/?resource=Statistics.alhost&command=intervalchart&mode=getdata&sid=";
baseUrl["protocolTopConversationsTable"] = "/?resource=Statistics.almatrix&command=intervalchart&mode=getdata&sid=";
baseUrl["nlhostDetailTable"] = "/?resource=Statistics.nlhost&command=intervalchart&mode=getdata&sid=";
baseUrl["nlhostTopProtocolsTable"] = "/?resource=Statistics.alhost&command=intervalchart&mode=getdata&sid=";
baseUrl["nlhostTopConversationsTable"] = "/?resource=Statistics.nlmatrix&command=intervalchart&mode=getdata&sid=";
baseUrl["topNNetworksTable"] = "/?mode=getdata&command=intervalchart&resource=Statistics.summary&sid=";


/**
 * Creates a chart from a block of xml data. 
 * @param chartXml The block of xml to create a chart of. 
 * @param window The window the charts are in. 
 * @return The chart created.
 */
function createChartFromXml(chartXml, window) {
	var type = getTagNodeValue("type", chartXml);
	var chart;
	if (!isNumberString(type)) {
		// invalid value for chart type
		return;
	}
	else if (Chart.TOPN_CHART == parseInt(type)) {
		chart = window.getTopNChart();
		var rowCount = getTagNodeValue("rowCount", chartXml);
		chart.rowCount = exists(rowCount) && isNumberString(rowCount) ? rowCount : chart.rowCount;
	}
	else {
		chart = createChartXml(chartXml, window);
	}
	
	if (isDefined(chart)) {
		chart.setXmlLayout(chartXml);
	}
	return chart;
}


/**
 * Constructs a chart to show in the windows.
 * @param chartCode The html code for the chart.
 * @param type The type of the chart (Chart.TOPN_CHART, Chart.INTERVAL_CHART);
 * @param title The title of this chart.
 * @param window The window the chart is in.
 * @throws Error if chartCode is undefined or null.
 * @throws Error if type is not one of the valid types.
 * @throws Error if title is undefined, null or empty.
 * @throws Error if window is undefined or null.
 */
function Chart(chartCode, type, title, window) {
	assertExists(chartCode, "The charts html code is undefined or null");
	assertValidType(type, "Invalid graf type: " + type);
	assertNotEmpty(title, "Title for chart is undefined, null or empty: " + title);
	assertExists(window, "The window the chart is in is undefined or null");

	this.htmlCode = chartCode;
	this.imageCode = this.htmlCode.getElementsByTagName("img")[0];
	assertExists(this.imageCode, "The charts html code does not contain a image tag");
	this.type = type;
	this.title = title;
	this.window = window;
	// used for silent chart loading
	this.loadingImage;
	this.udpateCounter;
	// set to true if the chart has loaded for the first time
	this.firstLoad = false;
	// set to true when the image is currently being loaded
	this.loaded = false;
	// true if the loading animation is shown instead of the chart
	this.loadingShown = false;
	
	// if this is true the chart is detacehd
	this.detached = false;
	
	// when the summary tab is deactivated the updating of charts is stopped until
	// the summary tab is activated again. If this is true the chart was active
	// when the tab was deactivated and is to be activated again when the summary tab
	// is activated. 
	this.activeTabDeactivated = false;
	
	// create the repeater object used for repeatedly calling the updateChart method
	this.repeater = new Repeater(updateChart, this, 0);
	
	// the source ip address for the entry this chart is for
	this.src = "0";
	
	// the destination ip address for the entry this chart if for
	this.dst = "0";
	
	// the protocol address for the entry this chart is for
	this.protocol = "0";
	
	// the field this chart is for
	this.field = -1;
	
	// set to true if the line chart data is being loaded
	this.loadingLineChartData = false;
	
	// set to true if the chart is currently loading
	this.loading = false;
	
	// the last time the chart data was requested
	this.dataTime = 0;
	
	// the time in seconds between each interval
	this.intervalTime = 0;
	
	// the unit used in the sample values
	this.sampleValuesUnit = "bit/s";
	
	// the time until the next interval is ready in seconds
	this.timeToNextInterval = 0;
	
	// the loading animation used while loading
	this.loadingAnimation;
	
	ChartManager.instance().registerNewChart(this);
}


/**
 * Returns true if a type is valid, false otherwise.
 * @param type The type to check.
 * @param message The error message to throw when the type is not valid.
 */
function assertValidType(type, message) {
	if (Chart.TOPN_CHART == type) {
		return;
	}
	
	if (Chart.INTERVAL_CHART == type) {
		return;
	}
	
	if (Chart.CHART_CONTAINER == type) {
		return;
	}
	
	if (Chart.LINE_CHART_CONTAINER == type) {
		return;
	}
	throw new Error(message);	
}


/**
 * Returns true if a chart container contains a chart with a given id.
 * @param id the id of one of the charts in the chart container. 
 */
Chart.prototype.containsChart = function(id) {
	var charts = this.getHtmlImages();
	for (var i = 0; i < charts.length; i++) {
		if (id == charts[i].getAttribute("id")) {
			return true;
		}
	}
	return false;
}


// services


/**
 * Returns the id of the charts chart tag.
 */
Chart.prototype.getId = function() {
	return this.htmlCode.getAttribute("name");
}


/**
 * Returns the chart title. If the title contains a separator character (:),
 * only the first part is returned.
 */
Chart.prototype.getTitle = function() {
	if (this.title.indexOf(":") > -1) {
		return this.title.substring(0, this.title.indexOf(":"));
	}
	return this.title;
}


/**
 * Returns the type of the chart.
 */
Chart.prototype.getType = function() {
	return this.type;
}


/**
 * Get the height of the chart.
 */
Chart.prototype.getHeight = function() {
	return parseInt(this.getHtmlCode().style.height);
}


/**
 * Get the width of the chart.
 */
Chart.prototype.getWidth = function() {
	return parseInt(this.getHtmlCode().style.width);
}


/**
 * Returns true if this chart is closable, false if not.
 */
Chart.prototype.isClosable = function() {
	if (Chart.TOPN_CHART == this.type || Chart.LINE_CHART_CONTAINER == this.type) {
		return false;
	}
	return true;	
}


/**
 * Returns the html code of this chart. *When the loading screen is shown for the chart
 * this returns the html code of the loading screen. 
 */
Chart.prototype.getHtmlCode = function() {
	return this.htmlCode;
}


/**
 * Returns the html code of this charts image tag.
 */
Chart.prototype.getImageCode = function() {
	return this.imageCode;
}


/**
 * Sets the title of this chart. 
 * @param title The title to set. 
 * @throws Error if title is undefined null or empty.
 */
Chart.prototype.setTitle = function(title) {
	assertNotEmpty(title, "Title to set for the chart is undefined, null or empty: " + title);
	this.title = title;
}


/**
 * Set the height of the chart. 
 * @param height The height to set. 
 * @throws Error if height is undefined, not a number or negative. 
 */
Chart.prototype.setHeight = function(height) {
	assertNotNegative(height, "Height to set for the chart is undefined, not a number or negative: " + height);
	this.getHtmlCode().style.height = parseInt(height) + "px";
}


/**
 * Set the width of the chart. 
 * @param width The width to set. 
 * @throws Error if width is undefined, not a number or negative.  
 */
Chart.prototype.setWidth = function (width) {
	assertNotNegative(width, "The width to set is undefined, not a number or negative: " + width);
	this.getHtmlCode().style.width = parseInt(width) + "px";
}


/**
 * Sets a layout for the chart based on a block of xml. 
 * @param xmlLayout The block of xml containing the layout to set. 
 */
Chart.prototype.setXmlLayout = function(xmlLayout) {
	var title = URLDecode(getTagNodeValue("title", xmlLayout));
	if (!isEmpty(title)) {
		this.setTitle(title);
	}
	
	var status = getTagNodeValue("chartStatus", xmlLayout);
	if ("detached" == status) {
		// the chart is detached, do not set the width, the width is for the window
		return;
	}
	
	var height = getTagNodeValue("height", xmlLayout);
	if (!isEmpty(height) && isNumberString(height) && !isNegative(height)) {
		this.setHeight(parseInt(height));
	}
	
	var width = getTagNodeValue("width", xmlLayout);
	if (!isEmpty(width) && isNumberString(width) && !isNegative(width)) {
		this.setWidth(parseInt(width));
	}
}


/**
 * Set the src ip address of the entry this chart is for.
 * @param address The address to set as a string(10.0.2.33).
 * @throws Error if address is undefined or null.
 */
Chart.prototype.setSrcAddress = function(address) {
	assertNotEmpty(address, "Source ip address to set for the chart is undefined, null or empty");
	this.src = address;
}


/**
 * Returns the src ip address of the entry this chart if for, 0 if not set.
 */
Chart.prototype.getSrcAddress = function() {
	return this.src;
}


/**
 * Set the dst ip address of the entry this chart is for.
 * @param address The address to set as a string(10.0.2.33).
 * @throws Error if address is undefined or null.
 */
Chart.prototype.setDstAddress = function(address) {
	assertNotEmpty(address, "Destination ip address to set for the chart is undefined, null or empty");
	
	this.dst = address;
}


/**
 * Returns the dst ip address of the entry this chart if for, 0 if not set.
 */
Chart.prototype.getDstAddress = function() {
	return this.dst;
}


/**
 * Set the protocol of the entry this chart is for.
 * @param protocol The protocol to set as a string(1.2048.6.80).
 * @throws Error if protocol is undefined or null.
 */
Chart.prototype.setProtocol = function(protocol) {
	assertNotEmpty(protocol, "Protocol to set for the chart is undefined, null or empty");
	
	this.protocol = protocol;
}



/**
 * Returns the protocol of the entry this chart if for, 0 if not set.
 */
Chart.prototype.getProtocol = function() {
	return this.protocol;
}


/**
 * Sets the field the chart is for. 
 * @param field The field to set.
 * @throws Error if field is not a number. 
 */
Chart.prototype.setField = function(field) {
	assertIsNumber(field, "The field to set is not a number");
	this.field = field;
}


/**
 * Gets the field set for this chart. 
 */
Chart.prototype.getField = function() {
	return this.field;
}


/**
 * Returns this chart as xml.
 * @return This chart as a xml string. 
 */
Chart.prototype.toXml = function() {
	var xmlString = 
		"<chart>" +
			"<chartId>" + this.getId() + "</chartId>" +
			"<type>" + this.getType() + "</type>" +
			"<table>" + this.tableId + "</table>" +
			"<title>" + encodeURIComponent(this.getTitle()) + "</title>";
			
	if (this.getField() > -1) {
		xmlString += "<field>" + this.getField() + "</field>";
	}
	if (isDefined(this.src) && !isNull(this.src)) {
		xmlString += "<src>" + this.src + "</src>";
	}
	if (isDefined(this.dst) && !isNull(this.dst)) {
		xmlString += "<dst>" + this.dst + "</dst>";
	}
	if (isDefined(this.protocol) && !isNull(this.protocol)) {
		xmlString +=  "<protocol>" + this.protocol + "</protocol>";
	}
	if (exists(this.rowCount)) {
		xmlString += "<rowCount>" + this.rowCount + "</rowCount>";
	}
	if (this.detached) {
		xmlString += "<chartStatus>detached</chartStatus>";
		xmlString += "<height>" + this.window.getHeight() + "</height>";
		xmlString += "<width>" + this.window.getWidth() + "</width>";
		xmlString += "<xPosition>" + this.window.getXPosition() + "</xPosition>";
		xmlString += "<yPosition>" + this.window.getYPosition() + "</yPosition>";
		xmlString += this.window.statusToXml();
	}
	else {
		xmlString += "<height>" + this.getHeight() + "</height>";
	}
			
			
	xmlString += "</chart>";
	return xmlString;
}


// event processing


/**
 * Processes a chart attached event. 
 * @param event The event to process. 
 */
function processChartAttachedEvent(event) {
	if (this != event.getData()["chart"]) {
		// only process events for this chart
		return;
	}
	
	this.window = event.getData()["window"];
	this.detached = false;
}


/**
 * Processes a chart detached event. 
 * @param event The event to process. 
 */
function processChartDetachedEvent(event) {
	if (this != event.getData()["chart"]) {
		// only process events for this chart
		return;
	}
	
	this.window = event.getData()["window"];
	this.detached = true;
}


/**
 * Processes a chart closed event. 
 * @param event The event to process. 
 */
function processChartClosedEvent(event) {
	if (this != event.getData()) {
		return;
	}
	
	// the chart has been shown start updating
	this.cancelUpdate();
	this.eventListener.stopListening();
	
}


/**
 * Process a chart hidden event. 
 * @param event The event to process. 
 */
function processChartHiddenEvent(event) {
	if (this != event.getData()) {
		return;
	}
	
	// the chart has been shown start updating
	this.cancelUpdate();
}


/**
 * Process a chart shown event. 
 * @param event The event to process. 
 */
function processChartShownEvent(event) {
	if (this != event.getData()) {
		return;
	}
	
	this.window.setContainerSize(this.getHeight());
	this.fitIntoContainer(this.window.container, false);
	
	// the chart has been shown start updating
	this.startUpdate();
}


/**
 * Processes a tab activated event. 
 * @param event The event to process. 
 */
function processTabActivatedEventChart(event) {
	if (!currentTabName.equalsIgnoreCase(event.getData())) {
		return;
	}
	if (!this.activeTabDeactivated) {
		return;
	}
	
	this.startUpdate();
	this.activeTabDeactivated = false;
}


/**
 * Processes a tab deactivated event.
 * @param event The event to process.
 */
function processTabDeactivatedEventChart(event) {
	if (!currentTabName.equalsIgnoreCase(event.getData())) {
		return;
	}
	
	if (!this.isUpdating()) {
		return;
	}
	
	this.cancelUpdate();
	this.activeTabDeactivated = true;
}


/**
 * Processes a window hidden event. 
 * @param event The event to process. 
 */
function processWindowHiddenChart(event) {
	if (event.getData() != this.window.getId()) {
		// the chart only cares about events from its window, none of the others
		return;
	}
	this.cancelUpdate();
}


/**
 * Processes a window shown event. 
 * @param event The event to process. 
 */
function processWindowShownChart(event) {
	if (event.getData() != this.window.getId()) {
		// the chart only cares about events from its window, none of the others
		return;
	}
	if (!this.window.isActiveChartId(this.getId())) {
		// only the active chart is to be updated
		return;
	}
	
	this.startUpdate();
}


/**
 * Processes a event probe name change event.
 * @param event The event to process.
 */
function processProbeNameChangedEvent(event) {
	// replace the old probename in the title with the new one
	var oldName = event.getData()["oldProbeName"];
	var newName = event.getData()["newProbeName"];
	if (oldName == newName || isEmpty(oldName) || isEmpty(newName)) {
		return;
	}
	if ("summarywindow" != this.window.getId()) {
		return;
	}
	
	this.title = replace(this.title, oldName, newName);
	this.htmlCode.setAttribute("name", replace(this.htmlCode.getAttribute("name"), oldName, newName));
}


/**
 * Fires a chart resized event. 
 * @param deltaHeight The delta height the chart has been resized with.
 * @param deltaWidth The delta width the chart has been resized with.
 */
Chart.prototype.fireChartResizedEvent = function(deltaHeight, deltaWidth) {
	var eventData = new Array();
	eventData["id"] = this.getId();
	eventData["deltaHeight"] = deltaHeight;
	eventData["deltaWidth"] = deltaWidth;
	getEventServer().notifyListeners(new Event("chartResized", eventData));
}


// updating


/**
 * Returns true if the chart is beeing updated periodically, false 
 * if the chart is not being updated. 
 */
Chart.prototype.isUpdating = function() {
	return this.repeater.isRepeating();
}


/**
 * Starts the updating of this chart.
 */
Chart.prototype.startUpdate = function() {
	if (!this.isUpdating() && ChartManager.instance().getLoadChartStatus(this.window.getId())) {
		this.repeater.start();
	}
}


/**
 * Restarts the updating of this chart. 
 */
Chart.prototype.restartUpdate = function() {
	this.cancelUpdate();
	this.startUpdate();
}


/**
 * Cancels the updating of this chart.
 */
Chart.prototype.cancelUpdate = function() {
	this.repeater.stop();
}


/**
 * Mark this chart for getting a new timestamp the next time it is updated. 
 */
Chart.prototype.markForTimestamp = function() {
	this.markedForTimestamp = true;
}


/**
 * Called repeatedly to update the chart. 
 */ 
function updateChart() {
	this.updateChart();
}


/**
 * Checks if the chart is currently being loaded. 
 * @return True if the chart is being loaded, false otherwise. 
 */
Chart.prototype.isLoading = function() {
	return this.loading;
}


/**
 * Checks if the loading animation is shown instead of the chart image. The loading
 * animation is shown the first time the chart is loaded. 
 */
Chart.prototype.isLoadingAnimationShown = function() {
	return this.loadingShown;
}


/**
 * Updates this chart.
 */
Chart.prototype.updateChart = function() {
	if (!ChartManager.instance().getLoadChartStatus(this.window.getId())) {
		return;
	}
	this.setUrl(this.htmlCode, this.createUrl());
}


/**
 * Get the loading image. The loading image is used for silent image loading in Internet Explorer. 
 * If the chart is not currently loading this will return null.
 * @return The loading image, null if the chart is not loading. 
 */
Chart.prototype.getLoadingImage = function() {
	return this.loadingImage;
}


/**
 * Sends a request for the time to the next interval from the server.
 */
Chart.prototype.requestTimeToNextInterval = function() {
	if (this.updating == true) {
		loadXML(this.createTimeUntilNextIntervalUrl(), "timeToNextIntervalCallBack", this);
	}
}


/**
 * Sets a new url for a chart.
 * @param image the image tag representing the chart to set the url for.
 * @param newUrl The url to set for the image.
 */
Chart.prototype.setUrl = function(image, newUrl) {
	if (isEmpty(newUrl) || !ChartManager.instance().getLoadChartStatus(this.window.getId())) {
		return;
	}
	if (Chart.MAX_UPDATE_COUNTER == this.udpateCounter) {
		// no more images are loaded until the last finishes
		return;
	}
	
	this.loading = true;
	this.increaseUpdateCounter(image);
	this.initLoadingImage(image);
	if ((isFirefox() || isSafari()) && this.isChartCreatorAppletLoaded()) {
		this.loadingImage.src = this.chartCreatorApplet().createChart(newUrl);
	}
	else {
		this.loadingImage.src = this.addForwardIpParameter(newUrl);
	}
	
	if (this.getType() == Chart.INTERVAL_CHART && !this.firstLoad && !this.loadingShown) {
		this.showLoading();
	}
}


/**
 * This function is called every time an image is being loaded.
 */
function chartUpdated() {
	if (!isDefined(this.id) || isNull(this.id)) {
		// normally this function should be called in the scope of the image, that way we can
		// access the id of the image and get the corresponding chart object, but in safari
		// this function is called in the global scope. In safari we can therefore not use silent image loading
		return;	
	}
	
	var loadedChart = getChartInstance(this.id);
	if (isNull(loadedChart) || !isDefined(loadedChart)) {
		// can happen if the image is closed while updating
		return;
	}
	loadedChart.chartUpdated(this);
}


/**
 * This function is called every time an image is being loaded.
 */
Chart.prototype.chartUpdated = function() {
	if (!isDefined(this.loadingImage) || isNull(this.loadingImage)) {
		// can happen if the image is closed while updating
		return;
	}

	this.loading = false;
	var imageUpdated = this.loadingImage;
	imageUpdated.onload = "";
	setClass(imageUpdated, getClassName(this.imageCode));
	this.replaceImage(imageUpdated);
	this.loadingImage = null;
	this.decreaseUpdateCounter(this);
	deallocateImage(this.loadingImage);

	if (this.loadingShown) {
		this.hideLoading();
	}

	this.firstLoad = true;
}


/**
 * Initializes the loading image used for silent image loading.
 * @param image The image to create the loading image for.
 */
Chart.prototype.initLoadingImage = function(image) {
	deallocateImage(this.loadingImage);
	this.loadingImage = new Image();
	this.loadingImage.id = this.getId();
	this.loadingImage.onload = chartUpdated;
}


/**
 * Replaces the html code of a chart image tag with a given image.
 * @param image The image to replace this images htmlCode with.
 */
Chart.prototype.replaceImage = function(image) {
	if (!isDefined(this.htmlCode.parentNode) || isNull(this.htmlCode.parentNode)) {
		// the image has no parent, can happen if the image is closed
		// while updating
		return;	
	}
	
	if (this.loadingShown) {
		this.htmlCode.replaceChild(image, this.imageCode);
		deallocateImage(this.imageCode);
		this.imageCode = image;
	}
	else {
		this.htmlCode.replaceChild(image, this.imageCode);
		deallocateImage(this.imageCode);
		this.imageCode = image;
	}
}


/**
 * Returns the chart object with a given id, null if no chart with
 * the id exists and no chart container contains a chart with the given id.
 * @param id The id to get the chart for.
 */
function getChartInstance(id) {
	var chart = ChartManager.instance().getChart(id);
	if (!isNull(chart)) {
		return chart;
	}

	chart = ChartManager.instance().getChartContainer(id);
	return chart;
}


/**
 * Updates the chart when a new interval is present.
 * @param timeToNextIntervalXml The loaded xml data.
 */
Chart.prototype.timeToNextIntervalCallBack = function(timeToNextIntervalXml) {
	// parse the result
	var nextIntervalNode = timeToNextIntervalXml.getElementsByTagName("timeToNextInterval")[0];
	if (!exists(nextIntervalNode)) {
		// might be because user has no access to resuorce
		return;
	}
	var timeToNextIntervalString = nextIntervalNode.firstChild.nodeValue;
	this.timeToNextInterval = isNumberString(timeToNextIntervalString) ? parseInt(timeToNextIntervalString) : -1;
}


/**
 * Sets the time to next interval from loaded chart data. 
 * @param chartData The loaded chart data string.
 */
Chart.prototype.timeToNextIntervalData = function(chartData) {
	var timeToNextInterval = getParameterValue(chartData, "next", "&");
	timeToNextInterval = isNumberString(timeToNextInterval) ? parseInt(timeToNextInterval) : -1;
	// if interval data is disabled the time to next interval is 0, use the interval time instead
	var enabled = getParameterValue(chartData, "enabled", "&");
	var intervalTime = getParameterValue(chartData, "interval", "&");
	var intervalTime = isEmpty(intervalTime) ? 0 : parseInt(intervalTime);
	timeToNextInterval = "false" == enabled ? intervalTime : timeToNextInterval;
	this.timeToNextInterval = timeToNextInterval;
}


/**
 * Checks if a string contains the access denied response from the server. 
 * @param string The string to check.
 * @return True if string contains the access denied response, false otherwise.
 * @throws Error if string is undefined, null or empty.
 */
Chart.prototype.isAccessDeniedResponse = function(string) {
	assertNotEmpty(string, "The string to check for access denied is undefined, null or empty");
	if (string.indexOf("<title>Access denied</title>") > -1) {
		return true;
	}
	return false;
}	

// url


/**
 * Creates a url for retrieving the time until next interval for the chart.  
 * @return The url to request the time until next interval. 
 */
Chart.prototype.createTimeUntilNextIntervalUrl = function () {
	var resource = getParameterValue(this.getBaseUrl(), "resource", "&");
	var url = "/?resource=" + resource + "&" + Chart.NEXT_INTERVAL_BASE_URL + "sid=" + getSid() + "&";
	return url;
}


/**
 * Requests the data needed to create a line chart url.
 */
Chart.prototype.requestLineChartData = function() {
	if (this.updating == true && !this.loadingLineChartData) {
		this.loadingLineChartData = true;
		getResponse(this.createLineChartDataUrl(), "lineChartDataCallBack", this);
	}
}


/**
 * Creates the url to request the line chart data. 
 * @return The url to request the chart data. 
 */
Chart.prototype.createLineChartDataUrl = function() {
	var resource = getParameterValue(this.getBaseUrl(), "resource", "&");
	var url = "/?resource=" + resource + "&" + Chart.INTERVAL_CHART_DATA_BASE_URL + "sid=" + getSid(); 
	url += "&field=" + this.getField();
	url += "&src=" + this.getSrcAddress(); 
	url += "&dst=" + this.getDstAddress();
	url += "&protocol=" + this.getProtocol();
	return url + "&time=" + this.dataTime + "&";
}


/**
 * The callback method for the loading of the chart data. Creates the new url and sets it.
 * This is to be implemented by subclasses. 
 * @param chartData The chart data loaded.
 */
Chart.prototype.lineChartDataCallBack = function(chartData) {
	// to be implemented by subclasses
}


/**
 * Creates the entry part of a chart url.
 * @return The entry part of the url.
 */
Chart.prototype.createEntryUrl = function() {
	var entryUrl = "protocol=" + this.getProtocol() + "&";
	entryUrl += "src=" + this.getSrcAddress() + "&";
	entryUrl += "dst=" + this.getDstAddress() + "&";
	return entryUrl + "field=" + this.getField() + "&"
}


/**
 * Creates a new url for a chart that is to be updated.
 * @returns The newly created url.
 */
Chart.prototype.createUrl = function() {
	// to be overridden by sub classes
}


/**
 * Adds a parameter to the chart url.
 * @param name The name of the parameter to add. 
 * @param request The request string containing the parameter to add and its value.  
 * @param url The url to add the parameter to.
 * @return The url with the parameter and the value added.
 */
Chart.prototype.addParameterToUrl = function(name, request, url) {
	if (isEmpty(name) || isEmpty(request)) {
		return url;
	}
	var value = URLDecode(getParameterValue(request, name, "&"));
	return isEmpty(value) ? url : (url + name + "=" + value + "&");
}


/**
 * Get the base url for a table.
 * @param id The id of the table to get the link for.
 * @return The base url for the table, undefined if no base url for the table is specified.
 */
Chart.prototype.getBaseUrl = function(){
	return baseUrl[this.tableId] + getSid() + "&";
}


// implementation



/**
 * Gets the html code of the two images in the container.
 * @return A array containing the discovered image tags in the container. 
 */
Chart.prototype.getHtmlImages = function() {
	var	images = this.htmlCode.getElementsByTagName("img");
	return images;
}


/**
 * Parses the line chart data.
 * @param chartData The loaded chart data. 
 */
Chart.prototype.parseLineChartData = function(chartData) {
	if (this.intervalTime != this.intervalTimeChartData(chartData) || this.isIntervalDataDisabled(chartData)) {
		// if the interval time is changed or the interval data has been
		// disabled, clear the interval data
		this.sampleValues = "";
	}
	
	this.intervalTime = this.intervalTimeChartData(chartData);
	this.setSampleValueChartData(chartData);
	var dataTimeParameter = getParameterValue(chartData, "lastIntervalTime", "&");
	this.dataTime = isNumberString(dataTimeParameter) ? parseInt(dataTimeParameter) : this.dataTime;
}


/**
 * Gets the interval time from a request a chart data response. 
 * @param chartDataResponse The chart data response to get the interval time from. 
 * @return The interval time from the response, -1 if no interval time in the response. 
 */
Chart.prototype.intervalTimeChartData = function(chartDataResponse) {
	var intervalTime = getParameterValue(chartDataResponse, "interval", "&");
	return isEmpty(intervalTime) ? -1 : parseInt(intervalTime);
}


/**
 * Sets the line chart sample values from a line chart data response.
 * @param chartDataResponse The response to the get the sample values from. 
 */
Chart.prototype.setSampleValueChartData = function(chartDataResponse) {
	this.recalculateSampleValues(this.sampleValuesUnit, trim(getParameterValue(chartDataResponse, "rangeLabelPostfix", "&")));
	this.sampleValuesUnit = trim(getParameterValue(chartDataResponse, "rangeLabelPostfix", "&"));
	this.sampleValues = this.addSampleValues(chartDataResponse, this.sampleValues);	
}


/**
 * Recalculates the sample values for a new unit if needed. 
 * @param unit The unit the sample values data currently is in.
 * @param newUnit The new unit for the sample values data. 
 */
Chart.prototype.recalculateSampleValues = function(unit, newUnit) {
	if (!this.sampleValuesUnit.equalsIgnoreCase(newUnit) && !isEmpty(this.sampleValues)) {
		this.sampleValues = this.changeThroughputUnit(unit, newUnit, this.sampleValues);
	}
}


/**
 * Checks whether interval data has been disabled based on a chart data response.
 * @param chartDataResponse The response to the chart data request.
 * @return True if data has been disabled, false otherwise. 
 */
Chart.prototype.isIntervalDataDisabled = function(chartDataResponse) {
	var enabled = getParameterValue(chartDataResponse, "enabled", "&");
	if ("false" == enabled) {
		return true;
	}
	return false;
}


/**
 * Changes the sample throughput values unit.
 * @param originalunit The original unit.
 * @param newUnit The new unit to calculate the sample values into. 
 * @param sampleValue The sample value to change. 
 * @return The sample values in the new unit. 
 */
Chart.prototype.changeThroughputUnit = function(originalUnit, newUnit, sampleValues) {
	var newSampleValues = "";
	var tokenizer = new StringTokenizer(sampleValues, ",");
	while(true) {
		if (!tokenizer.hasMoreTokens()) {
			break;
		}
		
		var oldValue = parseFloat(tokenizer.nextToken());
		var newValue = isNaN(oldValue) ? "" : changeThroughputValueUnit(oldValue, originalUnit, newUnit);
		newSampleValues += newValue;
		if (tokenizer.hasMoreTokens()) {
			newSampleValues += ",";
		}
	}
	return newSampleValues;
}


/**
 * Adds sample values to the sample values of this chart. 
 * @param chartData The chart data loaded from the server. 
 * @param sampleValues The sample values from chartData are added to these already existing values. 
 * @return The sample values from chartData added to sampleValues
 */
Chart.prototype.addSampleValues = function(chartData, sampleValues) {
	var newSampleValues = getParameterValue(chartData, "sampleValues", "&")
	sampleValues = !exists(sampleValues) ? "" : sampleValues;
	sampleValues += isEmpty(sampleValues) ? "" : (isEmpty(newSampleValues) ? "" : ","); 
	sampleValues += newSampleValues;
	if (isEmpty(sampleValues)) {
		return "";
	}
	var intervalCount = getParameterValue(chartData, "total", "&");
	intervalCount = isEmpty(intervalCount) ? 0 : parseInt(intervalCount);
	return this.trimSampleValues(intervalCount, sampleValues);
}


/**
 * Trims the sample values to a given sample count. If neccessary the first sample
 * values are skipped. 
 * @param count Trim down the sample values to contain max count values.
 * @param sampleValues The sample value to trim. 
 * @return The trimmed sample values. 
 */
Chart.prototype.trimSampleValues = function(count, sampleValues) {
	var tokenizer = new StringTokenizer(sampleValues, ",");
	if (count >= tokenizer.countTokens()) {
		return sampleValues;
	}
	
	var diff = tokenizer.countTokens() - count;
	for (var i = 0; i < diff; i++) {
		tokenizer.nextToken();
	}
	var newSampleValues = "";
	for (var i = 0; i < count; i++) {
		newSampleValues += tokenizer.nextToken();
		if (i < count - 1) {
			newSampleValues += ",";
		}
	}
	return newSampleValues;
}


/**
 * Fits the chart into a container.
 * @param container The container to fit the chart into.
 * @param reload If this is true the chart is reloaded if it does not fin into the container.
 * @throws Error if contianer is undefined or null.
 */
Chart.prototype.fitIntoContainer = function(container, reload) {
	assertDefined(container, "The container to fit the chart into is undefined or null");
	if (this.fitsIntoContainer(container)) {
		return;
	}
	this.adjustSizeToContainer(container);
	
	if (reload) {
		this.updateChart();
	}
}


/**
 * Sets the size of the chart according to a container.
 * @param container The container to fit into.
 */
Chart.prototype.adjustSizeToContainer = function(container) {
	var containerHeight = parseInt(container.style.height);
	var containerWidth = parseInt(container.style.width);
	var chartHeight = parseInt(this.htmlCode.style.height);
	var chartWidth = parseInt(this.htmlCode.style.width);
	if (chartHeight != containerHeight) {
		this.htmlCode.style.height = containerHeight + "px";
	}
	if (chartWidth != containerWidth) {
		this.htmlCode.style.width = containerWidth  + "px";
	}
}


/**
 * Returns true if the url of a chart needs to be changed in order to fit into the container.
 * @param container The container to fit the chart into.
 */
Chart.prototype.fitsIntoContainer = function(container) {
	var containerHeight = parseInt(container.style.height);
	var containerWidth = parseInt(container.style.width);
	var chartHeight = parseInt(this.htmlCode.style.height);
	var chartWidth = parseInt(this.htmlCode.style.width);

	if ((chartHeight == containerHeight) && (chartWidth == containerWidth)) {
		return true;
	}
	return false;
}


/**
 * Creates the entry part of a chart request based on a embedded request. 
 * @param embeddedRequest The embedded request to create the entry part of the url from.
 * @return The entry part (src, dst, protocol, field) of a chart url from a embedded request.
 */
Chart.prototype.createEntry = function(embeddedRequest) {
	var entryUrl = "&protocol=" + getParameterValue(embeddedRequest, "protocol", "&") + "&";
	entryUrl += "src=" + getParameterValue(embeddedRequest, "src", "&") + "&";
	entryUrl += "dst=" + getParameterValue(embeddedRequest, "dst", "&") + "&";
	return entryUrl + "field=" + getParameterValue(embeddedRequest, "field", "&") + "&";
}


/** 
 * Checks if the image needs a timestamp added, in order to automatically be updated.
 */
function needTimestamp(url) {
	if ((isOpera() || (navigator['appName'] != "Microsoft Internet Explorer") && url.indexOf("?") > -1)
		|| (navigator['appName'] == "Microsoft Internet Explorer" && navigator['appVersion'].indexOf("Windows NT 6") > -1 )
		|| navigator['appVersion'].indexOf("MSIE 7.0") > -1) {
			return true;
	}
	return false;
}


/**
 * Updates a chart. A new timestamp is added to the images url.
 * @return The url of the image with a new timestamp added.
 */
Chart.prototype.updateChartTimestamp = function(url) {
	var ts = new Date().valueOf();
	// replace the time stamp
	if (hasParameter(url, "ts", "&")) {
		url = replaceParameterValue(url, "ts", ts, "&");
	}
	// add the time stamp
	else {
		url = url + "&ts=" + ts;
	}
	return url;
}


/**
 * Increased a charts updateCounter by one.
 */
Chart.prototype.increaseUpdateCounter = function() {
	if (!isDefined(this.udpateCounter)) {
		this.udpateCounter = 0;
	}
	this.udpateCounter = this.udpateCounter + 1;
}


/**
 * Decrease a charts updateCounter by one.
 */ 
Chart.prototype.decreaseUpdateCounter = function() {
	if (Chart.MAX_UPDATE_COUNTER == this.udpateCounter) {
		this.udpateCounter = 0;
	}
	else {
		this.udpateCounter = this.udpateCounter - 1;
	}
}


/**
 * Deallocates a image to prevent memory leak.
 * @param image The image to deallocate.
 */
function deallocateImage(image) {
	if (isDefined(image) && !isNull(image)) {
		image.onload = "";
		image.src = "";
		delete image;
	}
}


/**
 * Hides the loading animation and shows the chart.
 */
Chart.prototype.hideLoading = function() {
	this.htmlCode.removeChild(this.loadingAnimation.getHtmlCode());
	if (this.window.activeChart.title == this.title) {
		setClass(this.imageCode, "chart");
	}
	this.loadingShown = false;
}


/**
 * Shows a loading animation instead of the chart.
 */
Chart.prototype.showLoading = function() {
	if (this.loadingShown) {
		return;
	}	
	
	this.loadingShown = true;
	setClass(this.imageCode, "inactive");
	this.createLoadingScreen();
	this.loadingAnimation.show();
}


/**
 * Create loading screen.
 */
Chart.prototype.createLoadingScreen = function() {
	this.loadingAnimation = new LoadingAnimation(this.htmlCode, "100%", "100%", "Loading chart...");
	this.loadingAnimation.setClass("active");
	this.loadingAnimation.setLoadingImageSrc("images/loading.gif");
}


/**
 * Checks if the chart creator applet is loaded. If java is enabled or no java plugin installed it might
 * be initalized and loaded. 
 * @return True if the applet is loaded, false otherwise. 
 */
Chart.prototype.isChartCreatorAppletLoaded = function() {
	var applet = this.chartCreatorApplet();
	if (!exists(applet) || !exists(applet.createChart)) {
		return false;
	}
	return true;
}


/**
 * Gets the chart creator applet.
 * @return The chart creator applet. 
 */
Chart.prototype.chartCreatorApplet = function() {
	return document.applets["ChartCreator"];
}


/**
 * Adds the forward ip parameter if necessary. 
 * @param request the request to add the parameter to.
 * @return The request with the forward ip parameter added if neccessary.  
 */
Chart.prototype.addForwardIpParameter = function(request) {
	return setForwardIpParameter(request);
}

