/**
 * This class manages and updates a line chart and a gauge chart side by side. 
 */


/**
 * The part of a gauge url containing the gauge charts sector colors.
 */
ChartContainer.GAUGE_SECTOR_COLORS = "%23f39e9e,%23eb8d8d,%23e27d7d,%23d96d6c,%23ce5657,%23c54646,%23bc3535,%23b32525,%23ab1415"


/**
 * The color to use in the gauge charts current pointer. 
 */
ChartContainer.GAUGE_CURRENT_COLOR = "%230066FF";


/**
 * The color to use in the gauge charts average pointer. 
 */
ChartContainer.GAUGE_AVERAGE_POINTER = "%2367c100"; 


/**
 * The base url used for the gauge charts, the column number has to be added to the end of this
 * request to complete the request, also the sid has to be added to the request.
 */
ChartContainer.GAUGE_CHART_BASE_URL = "/?resource=Statistics.summary&mode=getdata&command=throughputgaugechart&chart=gauge&width=230&height=200&border=2,gray&border_1=5,lightGray&lowerRange=0&startAngle=45&endAngle=315&rangeLabelsOn=true&rangeLabelStyle=outside&currentColor=" + ChartContainer.GAUGE_CURRENT_COLOR + "&averageColor=" + ChartContainer.GAUGE_AVERAGE_POINTER + "&sectorColors=" + ChartContainer.GAUGE_SECTOR_COLORS + "&background=%23ffffff|%23cccccc&servletMode=true&";


/**
 * The base url to get the next interval chart data. The resource (Statistics.protocol)
 * and the sid have to be added to the url.
 */
ChartContainer.GAUGE_CHART_DATA_BASE_URL = "mode=getdata&command=throughputgaugechartdata&";


/**
 * Create a instance of this class. A chart container is a subclass
 * of Chart.
 * @param chartCode The chart containers html code
 * @param title The title of this chart.
 * @param window The window the chart is in.
 * @param tableId The id of the table this chart is for.
 * @return A instance of this class.
 * @throws Error if chartCode is undefined or null.
 * @throws Error if title is undefined, null or empty.
 * @throws Error if window is undefined or null.
 * @throws Error if tableId is undefined, null or empty. 
 */
function createChartContainer(chartCode, title, window, tableId) {
	assertExists(chartCode, "The charts html code is undefined or null");
	assertNotEmpty(title, "Title for chart is undefined, null or empty: " + title);
	assertExists(window, "The window the chart container is in is undefined");
	assertNotEmpty(tableId, "TableId for chart is undefined, null or empty: " + tableId);

	var superInstance = new Chart(chartCode, Chart.CHART_CONTAINER, title, window);
	var subInstance = new ChartContainer(chartCode, tableId);
	subInstance = createSubClass(superInstance, subInstance);
	
	ChartManager.instance().removeChart(superInstance);
	ChartManager.instance().registerNewChart(subInstance);
	
	return subInstance;
}


// construction


/**
 * Constructs a chart container. A chart container contains multiple charts.
 * @param container The container containing the charts. 
 * @param tableId The id of the table this chart is for.
 * @throws Error if tableId is undefined, null or empty.
 */
function ChartContainer(container, tableId) {
	assertNotEmpty(tableId, "TableId for chart is undefined, null or empty: " + tableId);
	
	this.htmlCode = container;
	this.tableId = tableId;
	this.loadingImages = new Array();
	this.udpateCounter = new Array();
			
	// listen to events
	this.eventListener = new Listener(this);
	this.eventListener.addEvent(WINDOW_HIDDEN_EVENT, processWindowHiddenChart);
	this.eventListener.addEvent(WINDOW_SHOWN_EVENT, processWindowShownChart);
	this.eventListener.addEvent("tabDeactivated", processTabDeactivatedEventChart);
	this.eventListener.addEvent("tabActivated", processTabActivatedEventChart);
	this.eventListener.addEvent("chartShown", processChartShownEvent);
	this.eventListener.addEvent("chartHidden", processChartHiddenEvent);
	this.eventListener.addEvent("chartClosed", processChartClosedEvent);
	this.eventListener.addEvent("probeNameChanged", processProbeNameChangedEvent);
	this.eventListener.addEvent("chartDetached", processChartDetachedEvent);
	this.eventListener.addEvent("chartAttached", processChartAttachedEvent);

	// create the repeater object used for repeatedly calling the updateChart method
	this.repeater = new Repeater(updateChart, this, 0);
	
	// if this is true the chart will be updated
	this.updating = false;
	
	// if we are currently loading gauge chart data this is set to true
	this.loadingGaugeData = false;
	
	// the sample values shown in the chart
	this.sampleValues = "";
}	


// udpating


/**
 * Returns true if the chart is beeing updated periodically, false 
 * if the chart is not being updated. 
 */
ChartContainer.prototype.isUpdating = function() {
	return this.updating;
}


/**
 * Starts the updating of the chart. This is overridden to not start the 
 * updating sequence. A chart container is updated when a new interval is ready at the server. 
 */
ChartContainer.prototype.startUpdate = function() {
	if  (!this.isUpdating() && ChartManager.instance().getLoadChartStatus(this.window.getId())) {
		this.updating = true;	
		this.updateChart();
	}
}


/**
 * Stops the updating of the chart.
 */
ChartContainer.prototype.cancelUpdate = function() {
	this.repeater.stop();
	this.updating = false;	
}


/**
 * Updates this chart.
 */
ChartContainer.prototype.updateChart = function() {
	if (!ChartManager.instance().getLoadChartStatus(this.window.getId()) || !this.updating) {
		return;
	}
	var imageTags = this.getHtmlImages();
	for (var i = 0; i < imageTags.length; i++) {
		if (this.isGaugeChart(imageTags[i]) && ((isFirefox() || isSafari()) && this.isChartCreatorAppletLoaded())) {
			this.requestGaugeChartData();
		}
		else if (this.isGaugeChart(imageTags[i])) {
			this.setUrl(imageTags[i], this.createGaugeChartUrl());
		}
		else if (this.isLineChart(imageTags[i]) && (isFirefox() || isSafari()) && this.isChartCreatorAppletLoaded()) {
			this.requestLineChartData();
		}
		else if (this.isLineChart(imageTags[i])) {
			this.setUrl(imageTags[i], this.createLineChartUrl());
		}
	}
	
	if ((!isFirefox() && !isSafari()) || !this.isChartCreatorAppletLoaded()) {
		this.requestTimeToNextInterval();
	}
}


/**
 * Requests the data needed to create a gauge chart. 
 */
ChartContainer.prototype.requestGaugeChartData = function() {
	if (this.updating == true && !this.loadingGaugeData) {
		this.loadingGaugeData = true
		getResponse(this.createGaugeDataUrl(), "gaugeChartDataCallBack", this);
	}
}


/**
 * The callback method for the loading of the gauge chart data. Creates the new url and sets it.
 * @param chartData The chart data loaded.
 */
ChartContainer.prototype.gaugeChartDataCallBack = function(chartData) {
	if (isEmpty(chartData) || this.isAccessDeniedResponse(chartData) || !this.updating) {
		this.loadingGaugeData = false;
		return;
	}
	this.setUrl(this.getGaugeChartImage(), this.createGaugeChartUrlData(chartData));
	this.timeToNextIntervalData(chartData);
	this.loadingGaugeData = false;
}


/**
 * The callback method for the loading of the chart data. Creates the new url and sets it.
 * @param chartData The chart data loaded.
 */
ChartContainer.prototype.lineChartDataCallBack = function(chartData) {
	if (isEmpty(chartData) || this.isAccessDeniedResponse(chartData) || !this.updating) {
		this.loadingLineChartData = false;
		return;
	}
	this.parseLineChartData(chartData);
	this.setUrl(this.getLineChartImage(), this.createLineChartUrlData(chartData));
	this.timeToNextIntervalData(chartData);
	this.loadingLineChartData = false;
}


/**
 * Creates a line chart url based on the loaded chart data xml.
 * @param chartData The block of xml data containing the chart data. 
 */ 
ChartContainer.prototype.createLineChartUrlData = function(chartData) {
	var url = this.createLineChartUrl() + "&";
	url = this.addParameterToUrl("locale", chartData, url);
	url = this.addParameterToUrl("sampleAxisLabel", chartData, url);
	url += "rangeLabelPostfix= " + this.sampleValuesUnit + "&";
	url = this.addParameterToUrl("range", chartData, url);
	url = this.addParameterToUrl("rangeDecimalCount", chartData, url);
	url = this.addParameterToUrl("error", chartData, url);
	url += "sampleValues=" + this.sampleValues;
	return url;
}


/**
 * Get the loading image. The loading image is used for silent image loading in Internet Explorer. 
 * If the chart is not currently loading this will return null.
 * @param id The id of the image to get the loading image for. 
 * @return The loading image, null if the chart is not loading.
 * @throws Error if id is undefined, null or empty. 
 */
ChartContainer.prototype.getLoadingImage = function(id) {
	assertNotEmpty(id, "The id of the image to get the loading image for is undefined, null or empty");
	return this.loadingImages[id];
}


/**
 * Sets a new url for a image.
 * @param image the image to set the url for.
 * @param newUrl The url to set for the image.
 */
ChartContainer.prototype.setUrl = function(image, newUrl) {
	if (!ChartManager.instance().getLoadChartStatus(this.window.getId())) {
		return;
	}
	if (Chart.MAX_UPDATE_COUNTER == this.udpateCounter[image.id]) {
		// no more images are loaded until the last finishes
		return;
	}
	
	this.loading = true;
	this.increaseUpdateCounter(image);
	this.initLoadingImage(image);
	if ((isFirefox() || isSafari()) && this.isChartCreatorAppletLoaded()) {
		this.loadingImages[image.id].src = this.chartCreatorApplet().createChart(newUrl)
	}
	else {
		this.loadingImages[image.id].src = this.addForwardIpParameter(newUrl);
	}

	if (!this.firstLoad && !this.loadingShown) {
		this.showLoading();
	}
}


/**
 * Called when a chart container has done loading.
 * @param image The image that has been udpated. 
 */
ChartContainer.prototype.chartUpdated = function(image) {
	if (!isDefined(this.loadingImages[image.id]) || isNull(this.loadingImages[image.id])) {
		// can happen if the image is closed while updating
		return;
	}
	var imageUpdated = this.loadingImages[image.id];
	this.loadingImages[image.id] = null;
	
	imageUpdated.onload = "";
	this.setUpdatedImageSize(imageUpdated);
	this.replaceImage(imageUpdated);
	this.decreaseUpdateCounter(imageUpdated);
	deallocateImage(this.loadingImages[image.id]);
	
	if (this.loadingShown && this.allImagesLoaded()) {
		this.hideLoading();
	}
	this.firstLoad = true;
	this.loading = !this.allImagesLoaded();
	
	if (this.timeToNextInterval >= 0 && !this.repeater.isDelaying()) {
		this.repeater.stop();
		this.repeater.setTime(this.timeToNextInterval + 1);
		this.repeater.delay();
	}
}


/**
 * Inits the loading image for a given chart. 
 * @param image The image to create the loading image for. 
 */
ChartContainer.prototype.initLoadingImage = function(image) {
	deallocateImage(this.loadingImages[image.id]);
	this.loadingImages[image.id] = null;
	this.loadingImages[image.id] = new Image();
	this.loadingImages[image.id].id = image.id;
	this.loadingImages[image.id].onload = chartUpdated;
}


/**
 * Sets the size of a updated image. 
 * @param updatedImage The updated image.
 */
ChartContainer.prototype.setUpdatedImageSize = function(updatedImage) {
	// images in a image container do not have a absolute size
	updatedImage.style.width = "100%";
	updatedImage.style.height = "100%";
}


/**
 * Replaces the image in the container with id like the given image.
 * @param image The image to insert.
 */
ChartContainer.prototype.replaceImage = function(image) {
	if (!isDefined(this.htmlCode.parentNode) || isNull(this.htmlCode.parentNode)) {
		// the image has no parent, can happen if the image is closed
		// while updating
		return;	
	}
	var images = this.getHtmlImages();
	for (var i = 0; i < images.length; i++) {
		if (image.getAttribute("id") == images[i].getAttribute("id")) {
			images[i].parentNode.replaceChild(image, images[i]);
			return;
		}
	}
}


// url


/**
 * Creates a url for the gauge chart. 
 * returns The newly created gauge chart url.
 */
ChartContainer.prototype.createGaugeChartUrl = function() {
	var url = ChartContainer.GAUGE_CHART_BASE_URL + this.createEntryUrl() + "sid=" + getSid() + "&chartId=" + this.getId() + "&";
	url = replaceParameterValue(url, "width",  Math.round(this.getWidth() * 0.3), "&"); 
	url = replaceParameterValue(url, "height", this.getHeight(), "&");
	if (needTimestamp(url) || this.markedForTimestamp) {
		url = this.updateChartTimestamp(url);
		this.markedForTimestamp = false;
	}
	return url;
}


/**
 * Creates a gauge chart url based on some xml chart data. 
 * @param chartData The block of xml containing the chart data. 
 * @return The gauge chart url based on the data in chartData.
 */
ChartContainer.prototype.createGaugeChartUrlData = function(chartData) {
	var url = this.createGaugeChartUrl() + "&";
	url = this.addParameterToUrl("range", chartData, url);
	url = this.addParameterToUrl("rangeDecimalCount", chartData, url);
	url = this.addParameterToUrl("pointer", chartData, url);
	url = this.addParameterToUrl("pointer_2", chartData, url);
	url = this.addParameterToUrl("rangeLabelPostfix", chartData, url);
	url = this.addParameterToUrl("sectors", chartData, url);
	url = this.addParameterToUrl("error", chartData, url);
	return url;
}


/**
 * Creates a url for the line chart.
 * @returns The newly created line url.
 */
ChartContainer.prototype.createLineChartUrl = function() {
	var url = this.getBaseUrl();
	url += Chart.LINECHART_PARAMETERS + this.createEntryUrl();
	url = replaceParameterValue(url, "width",  Math.round(this.getWidth() * 0.7), "&"); 
	url = replaceParameterValue(url, "height", this.getHeight(), "&");
	if (needTimestamp(url) || this.markedForTimestamp) {
		url = this.updateChartTimestamp(url);
		this.markedForTimestamp = false;
	}
	return url;
}


/**
 * Fits the chart into a container.
 * @param container The container to fit the chart into.
 * @param reload If this is true the chart is reloaded if it does not fin into the container.
 * @throws Error if contianer is undefined or null.
 */
ChartContainer.prototype.fitIntoContainer = function(container, reload) {
	assertDefined(container, "The container to fit the chart into is undefined or null");
	if (this.fitsIntoContainer(container)) {
		return;
	}
	
	this.adjustSizeToContainer(container);

	if (!reload) {
		return;
	}
	this.updateChart();	
}


/**
 * Gets the line chart container. 
 * @return The html tag containing the line chart. 
 */
ChartContainer.prototype.getLineChartContainer = function() {
	return getElementWithAttribute(this.getHtmlCode(), "lineChartContainer", "name", "div");
}


/**
 * Gets the image tag for the line chart.
 * @return The image tag for the line chart.  
 */
ChartContainer.prototype.getLineChartImage = function() {
	var images = this.getHtmlImages();
	for (var i = 0; i < images.length; i++) {
		if (!this.isGaugeChart(images[i])) {
			return images[i];
		}
	}
}


/**
 * Gets the gauge chart container. 
 * @return The html tag container the gauge chart.
 */
ChartContainer.prototype.getGaugeChartContainer = function() {
	return getElementWithAttribute(this.getHtmlCode(), "gaugeChartContainer", "name", "div");
}


/**
 * Gets the image tag for the gauge chart. 
 * @return The image tag for the gauge chart. 
 */
ChartContainer.prototype.getGaugeChartImage = function() {
	var images = this.getHtmlImages();
	for (var i = 0; i < images.length; i++) {
		if (this.isGaugeChart(images[i])) {
			return images[i];
		}
	}
}


/**
 * Checks if a given image tag is a gauge chart. 
 * @param image The image to check.
 * @param True if the image to check is a gauge chart. 
 */
ChartContainer.prototype.isGaugeChart = function(image) {
	if (!isEmpty(image.id) && image.id.indexOf("gauge") > -1) {
		return true;
	}
	return false;
}


/**
 * Checks if a given image tag is line chart. 
 * @param image The image to check.
 * @return True if the image to check is a line chart. 
 */
ChartContainer.prototype.isLineChart = function(image) {
	if (!isEmpty(image.id) && image.id.indexOf("line") > -1) {
		return true;
	}
	return false;
}


// loading animation


/**
 * Returns true if all of the images in the container have loaded.
 */
ChartContainer.prototype.allImagesLoaded = function() {
	var images = this.getHtmlImages();
	for (var i = 0; i < images.length; i++) {
		if (exists(this.loadingImages[images[i].id])) {
			return false;
		}
	}
	return true;
}



/**
 * Increased a images updateCounter by one.
 * @param image The html code of the image to increase the counter for.
 */
ChartContainer.prototype.increaseUpdateCounter = function(image) {
	if (!isDefined(this.udpateCounter[image.id])) {
		this.udpateCounter[image.id] = 0;
	}
	this.udpateCounter[image.id] = this.udpateCounter[image.id] + 1;
}


/**
 * Decrease a images updateCounter by one.
 * @param image The html code of the image to decrease the counter for.
 */ 
ChartContainer.prototype.decreaseUpdateCounter = function(image) {
	if (Chart.MAX_UPDATE_COUNTER == this.udpateCounter[image.id]) {
		this.udpateCounter[image.id] = 0;
	}
	else {
		this.udpateCounter[image.id] = this.udpateCounter[image.id] - 1;
	}
}


/**
 * Creates the url to request the gauge chart data. 
 * @return The url to request the guage chart data. 
 */
ChartContainer.prototype.createGaugeDataUrl = function() {
	var resource = getParameterValue(this.getBaseUrl(), "resource", "&");
	var url = "/?resource=" + resource + "&" + ChartContainer.GAUGE_CHART_DATA_BASE_URL + "sid=" + getSid(); 
	url += "&field=" + this.getField();
	url += "&src=" + this.getSrcAddress(); 
	url += "&dst=" + this.getDstAddress();
	url += "&protocol=" + this.getProtocol()
	url += "&sectorColors=" + ChartContainer.GAUGE_SECTOR_COLORS; 
	url += "&currentColor=" + ChartContainer.GAUGE_CURRENT_COLOR;
	url += "&averageColor=" + ChartContainer.GAUGE_AVERAGE_POINTER;
	return url + "&chartId=" + this.getId() + "&";
}



/**
 * Hides the loading animation and shows the chart.
 */
ChartContainer.prototype.hideLoading = function() {
	this.htmlCode.removeChild(this.loadingAnimation.getHtmlCode());
	if (this.window.activeChart.title == this.title) {
		setClass(this.getLineChartContainer(), "lineChartContainer");
		var gaugeChartClass = isIE() ? "ieGaugeChartContainer" : "gaugeChartContainer";
		setClass(this.getGaugeChartContainer(), gaugeChartClass);
	}
	this.loadingShown = false;
}


/**
 * Shows a loading animation instead of the chart.
 */
ChartContainer.prototype.showLoading = function() {
	if (this.loadingShown) {
		return;
	}	
	
	this.loadingShown = true;
	setClass(this.getLineChartContainer(), "inactive");
	setClass(this.getGaugeChartContainer(), "inactive");
	this.createLoadingScreen();
	this.loadingAnimation.show();
}