/**
 * This class manages and updates a line chart. 
 */

/**
 * Create a instance of this class. A linechart is a subclass
 * of ProbeImage.
 * @param chartCode The html code of the chart.
 * @param title The title of this chart.
 * @param window The window the chart is in.
 * @param tableId The id of the table this chart is for.
 * @return A instance of this class.
 * @throws Error if chart is undefined or null.
 * @throws Error if title is undefined, null or empty.
 * @throws Error if window is undefined or null.
 * @throws Error if tableId is undefined, null or empty.
 */
function createLineChart(chartCode, title, window, tableId) {
	assertExists(chartCode, "The charts html code is undefined or null");
	assertNotEmpty(title, "Title for chart is undefined, null or empty: " + title);
	assertExists(window, "The window the chart container is in is undefined or null");
	assertNotEmpty(tableId, "TableId for chart is undefined, null or empty: " + tableId);

	var superInstance = new Chart(chartCode, Chart.INTERVAL_CHART, title, window);
	var subInstance = new LineChart(tableId);
	subInstance = createSubClass(superInstance, subInstance);
	
	ChartManager.instance().removeChart(superInstance);
	ChartManager.instance().registerNewChart(subInstance);
	
	return subInstance;
}


// construction


/**
 * Constructs a image container. A image container contains multiple images.
 * @param tableId The id of the table this image is for.
 * @throws Error if tableId is undefined, null or empty.
 */
function LineChart(tableId) {
	assertNotEmpty(tableId, "TableId for chart is undefined, null or empty: " + tableId);
	
	this.tableId = tableId;
	
	// listen to events
	this.eventListener = new Listener(this);
	this.eventListener.addEvent(WINDOW_HIDDEN_EVENT, processWindowHiddenChart);
	this.eventListener.addEvent(WINDOW_SHOWN_EVENT, processWindowShownChart);
	this.eventListener.addEvent("tabDeactivated", processTabDeactivatedEventChart);
	this.eventListener.addEvent("tabActivated", processTabActivatedEventChart);
	this.eventListener.addEvent("chartShown", processChartShownEvent);
	this.eventListener.addEvent("chartHidden", processChartHiddenEvent);
	this.eventListener.addEvent("chartClosed", processChartClosedEvent);
	this.eventListener.addEvent("probeNameChanged", processProbeNameChangedEvent);
	this.eventListener.addEvent("chartDetached", processChartDetachedEvent);
	this.eventListener.addEvent("chartAttached", processChartAttachedEvent);
	
	// create the repeater object used for repeatedly calling the updateChart method
	this.repeater = new Repeater(updateChart, this, 0);
	
	// if this is true the chart will be updated
	this.updating = false;
	
	// the sample values shown in the line chart
	this.sampleValues = "";
}	


// updating


/**
 * Returns true if the chart is beeing updated periodically, false 
 * if the chart is not being updated. 
 */
LineChart.prototype.isUpdating = function() {
	return this.updating;
}


/**
 * Starts the updating of the chart. This is overridden to not start the 
 * updating sequence. A line chart is updated when a new interval is ready at the server. 
 */
LineChart.prototype.startUpdate = function() {
	if (!this.isUpdating() && ChartManager.instance().getLoadChartStatus(this.window.getId())) {
		this.updating = true;	
		this.updateChart();
	}
}


/**
 * Stops the updating of the chart.
 */
LineChart.prototype.cancelUpdate = function() {
	this.repeater.stop();
	this.updating = false;	
}



/**
 * Updates this chart.
 */
LineChart.prototype.updateChart = function() {
	if ((isFirefox() || isSafari()) && this.isChartCreatorAppletLoaded()) {
		this.requestLineChartData();	
	}
	else {
		this.setUrl(this.htmlCode, this.createUrl());
	}
}


/**
 * The callback method for the loading of the chart data. Creates the new url and sets it.
 * @param chartData The chart data loaded.
 */
LineChart.prototype.lineChartDataCallBack = function(chartData) {
	if (isEmpty(chartData) || this.isAccessDeniedResponse(chartData) || !this.updating) {
		this.loadingLineChartData = false;
		return;
	}
	this.parseLineChartData(chartData);
	this.setUrl(this.htmlCode, this.createLineChartUrlData(chartData));
	this.timeToNextIntervalData(chartData);
	this.loadingLineChartData = false;
}


/**
 * Creates a line chart url based on the loaded chart data.
 * @param chartData The chart data represented as a string.  
 */ 
LineChart.prototype.createLineChartUrlData = function(chartData) {
	var url = this.createUrl() + "&";
	url = this.addParameterToUrl("locale", chartData, url);
	url = this.addParameterToUrl("sampleAxisLabel", chartData, url);
	url += "rangeLabelPostfix= " + this.sampleValuesUnit + "&";
	url = this.addParameterToUrl("range", chartData, url);
	url = this.addParameterToUrl("rangeDecimalCount", chartData, url);
	url = this.addParameterToUrl("error", chartData, url);
	url += "sampleValues=" + this.sampleValues;
	return url;
}


/**
 * Called when a chart has done loading.
 */
LineChart.prototype.chartUpdated = function() {
	if (!this.updating) {
		return;
	}
	
	this.callSuper("chartUpdated");
	if ((!isFirefox() && !isSafari()) || !this.isChartCreatorAppletLoaded()) {
		this.requestTimeToNextInterval();
	}
	
	if (this.timeToNextInterval >= 0 && !this.repeater.isDelaying()) {
		this.repeater.stop();
		this.repeater.setTime(this.timeToNextInterval + 1);
		this.repeater.delay();
	}
}


/**
 * Creates a url for a line chart.
 * @param image The image to create the url for. 
 * @param timestamp If this is set to true a timestamp is added to the request if needed.
 * @returns The newly created url.
 */
LineChart.prototype.createUrl = function() {
	var url = this.getBaseUrl();
	url += Chart.LINECHART_PARAMETERS + this.createEntryUrl();
	url = replaceParameterValue(url, "width", this.getWidth(), "&"); 
	url = replaceParameterValue(url, "height", this.getHeight(), "&");
	if (needTimestamp(url) || this.markedForTimestamp) {
		url = this.updateChartTimestamp(url);
		this.markedForTimestamp = false;
	}
	return url;
}