/**
 * This class updates and manages a line chart container. A line chart container container can contain 
 * multiple line charts. 
 */
 
 
/**
 * The default height to set for each of the line charts in pixels. 
 */
LineChartContainer.LINE_CHART_DFEAULT_HEIGHT = 65;
 

/**
 * The min height of a chart image tag in pixels.  
 */
LineChartContainer.LINE_CHART_MIN_HEIGHT = 65;


/**
 * The width of a scrollbar in the browser.
 */
LineChartContainer.BROWSER_SCROLLBAR_WIDTH = 16;


/**
 * The parameters used in a line chart with a singel background color. 
 */
LineChartContainer.LINECHART_PARAMETERS_SINGLE_BACKGROUND_COLOR = "chart=line&width=792&height=0&background=%23cccccc&3dmodeon=true&antialias=true&3ddepth=5&valueLinesOn=true&stackedOn=true&sampleColors=%231e7096f0|%232492c7f0&alternateChartBackground=%23e7e7e7&servletMode=true&";


/**
 * The parameters used in the axis label chart. 
 */
LineChartContainer.AXIS_LABEL_CHART_PARAMETERS = "chart=line&width=844&height=25&background=%23cccccc&3dmodeon=true&antialias=true&3ddepth=5&valueLinesOn=true&stackedOn=true&alternateChartBackground=%23e7e7e7&protocol=0&src=0.0.0.0&dst=0&field=1&locale=no,NO&range=0&servletMode=true&graphInsets=0,0,30,0";


/**
 * To prevent the chart and the table to send scrolled events to each other indefinetily the charts waits
 * this amount of time in milliseconds before it fires a scrolled event. 
 */
LineChartContainer.FIRE_SCROLL_EVENT_THRESHOLD = 1000;


/**
 * The amount of time to wait before the chart is reloaded after it is scrolled in milliseconds. 
 */
LineChartContainer.SCROLL_RELOAD_TIMEOUT = 1000;


/**
 * Factory method to create a line chart container.
 * @param chartCode The html code of the chart container. 
 * @param title The title to set for this chart. 
 * @param window The window the chart is in.
 * @param tableId The id of the table this chart is for.
 * @return A instance of this class.
 * @throws Error if chartCode is undefined or null.
 * @throws Error if title is undefined, null or empty.
 * @throws Error if window is undefined or null.
 * @throws Error if tableId is undefined, null or empty. 
 */
function createLineChartContainer(chartCode, title, window, tableId){
	assertExists(chartCode, "The charts html code is undefined or null");
	assertNotEmpty(title, "Title for chart is undefined, null or empty: " + title);
	assertExists(window, "The window the chart container is in is undefined");
	assertNotEmpty(tableId, "TableId for chart is undefined, null or empty: " + tableId);
	
	var superInstance = new Chart(chartCode, Chart.LINE_CHART_CONTAINER, title, window);
	var subInstance = new LineChartContainer(tableId);
	subInstance = createSubClass(superInstance, subInstance);
	
	ChartManager.instance().removeChart(superInstance);
	ChartManager.instance().registerNewChart(subInstance);
	
	return subInstance;
}


// construction


/**
 * Constructs a line chart containe code. A line chart container contains multiple line charts 
 * and one chart showing only the range axis label. 
 * @param tableId The id of the table this chart is for.
 * @throws Error if tableId is undefined, null or empty.
 */
function LineChartContainer(tableId) {
	assertNotEmpty(tableId, "TableId for chart is undefined, null or empty: " + tableId);
	
	this.tableId = tableId;
	this.table = TableManager.instance().getTable(tableId);
	this.loadingImages = new Array();
	this.udpateCounter = new Array();
	
	// listen to events
	this.eventListener = new Listener(this);
	this.eventListener.addEvent("tableUpdated", processUpdatedEventContainer);
	this.eventListener.addEvent("tableSorted", processSortedEventContainer);
	this.eventListener.addEvent("tableIndexChanged", processIndexChangedEventContainer);
	this.eventListener.addEvent(WINDOW_HIDDEN_EVENT, processWindowHiddenChart);
	this.eventListener.addEvent(WINDOW_SHOWN_EVENT, processWindowShownChart);
	this.eventListener.addEvent("tabDeactivated", processTabDeactivatedEventChart);
	this.eventListener.addEvent("tabActivated", processTabActivatedEventChart);
	this.eventListener.addEvent("chartShown", processChartShownEvent);
	this.eventListener.addEvent("chartHidden", processChartHiddenEvent);
	this.eventListener.addEvent("chartClosed", processChartClosedEvent);
	this.eventListener.addEvent("chartDetached", processChartDetachedEvent);
	this.eventListener.addEvent("chartAttached", processChartAttachedEvent);
	this.eventListener.addEvent("resized", processWindowResizedContainer);
	this.eventListener.addEvent("scrolled", processTableScrolledEventContainer);
	
	// create the repeater object used for repeatedly calling the updateChart method
	this.repeater = new Repeater(updateChart, this, 0);
	
	// if this is true the chart will be updated
	this.updating = false;
	
	// indicates whether the vertical scrollbar is visible
	this.verticalScrollbarVisible = false;
	
	// the charts currently loading data
	this.loadingData = new Array(); 
	
	// the sample values for each chart image tag
	this.sampleValues = new Array();
	
	// the last time the chart data was requested
	this.dataTime =  new Array();
	
	// the unit used in the sample values
	this.sampleValuesUnit = new Array();
	
	// the sample axis label to use
	this.sampleAxisLabel = "";
	
	// a comma separated list of the src address of all the entries in the table. The chart is updated when 
	// new interval entries are available or when a new entry is loaded into the table 
	this.entries = "";
	
	// the time when table was last scrolled, the chart does not fire any scroll events for a given 
	// threshold of time after the table has been scrolled, to prevent the chart and the table to send scrolled
	// events to each other indefinetily. 
	this.scrollTimer = 0;
	
	// the scroll timeout id used, after the chart is scrolled we wait for some time to reload the visible charts, 
	// this is the timeout id used
	this.scrollTimeOutId;
	
	// indicating whether a chart image tag has been loaded, used to only update charts in the current 
	// scrolling, but the first time they are all updated even the ones not visible due to the current scrolling
	this.loaded = new Array();
}


// event handling


/**
 * Process a table scrolled event. When the table is scrolled the chart is scrolled also
 * @param event The event to process.  
 */
function processTableScrolledEventContainer(event) {
	if (event.getData()["id"] != this.table.getId()) {
		// the chart only cares about events from its table, none of the others
		return;
	}
	
	this.scrollTimer = (new Date()).getTime();	
	var factor = this.getMaxScrollHeight() / event.getData()["maxScrollHeight"];
	this.imageContainer().scrollTop = Math.round(event.getData()["scrollTop"] * factor);
	this.scrolledChartUpdate();
}


/**
 * Processes a sorted event from the table.
 * @param event The sorted event to process.
 */
function processSortedEventContainer(event) {
	if (event.getData() != this.table.getId()) {
		// the chart only cares about events from its table, none of the others
		return;
	}
	this.sortImages();
	if (this.updating || this.window.getActiveChart() == this) {
		this.restartUpdate();
	}
}


/**
 * Processes a updated event from the table.
 * @param event The updated event to process.
 */
function processUpdatedEventContainer(event) {
	if (event.getData()["id"] != this.table.getId()) {
		// the chart only cares about events from its table, none of the others
		return;
	}
	this.removeNotExistingEntries();
	if (!this.chartUpdateNeeded() && !this.chartSortNeeded()) {
		return;
	}
	
	this.loaded = new Array();
	this.entries = this.table.getFieldValues("src", -1);
	this.updateChartTags();
	this.sortImages();
	this.checkHeight();
	this.adjustChartsWidth();
	if (this.updating || this.window.getActiveChart() == this) {
		this.restartUpdate();
	}
}


/**
 * Processes a window resized event. 
 * @param event The event to process.
 */
function processWindowResizedContainer(event) {
	if (event.getData()["id"] != this.window.getId()) {
		// the chart only cares about events from its table, none of the others
		return;
	}
	
	this.imageContainer().style.height = (this.getHeight() - this.axisLabelChartHeight()) + "px";
	this.imageContainer().style.width = this.getWidth() + "px";	
	var width = this.verticalScrollbarVisible ? this.getWidth() - LineChartContainer.BROWSER_SCROLLBAR_WIDTH : this.getWidth();
	this.setChartsWidth(width);	
	
	var deltaHeight = event.getData()["height"];
	if (deltaHeight < 0) {
		var height = this.getHeight() - this.axisLabelChartHeight();
		this.setChartsHeight(height < this.minHeight() ? this.minHeight() : height);
	}
	else if (this.imagesHeight() < this.getHeight()) {
		this.setChartsHeight(this.getHeight() - this.axisLabelChartHeight()); 
	}
	this.adjustChartsWidth();
}


/**
 * Processes a index changed event.
 * @param event The event to process. 
 */
function processIndexChangedEventContainer(event) {
	if (event.getData()["id"] != this.tableId) {
		// the chart only cares about events from its table, none of the others
		return;
	}
	
	this.loaded = new Array();
	this.sampleValues = new Array();
	this.dataTime =  new Array();
	this.sampleValuesUnit = new Array();
	this.sampleAxisLabel = "";	
	this.setField(parseInt(appletColumnMap[this.tableId][event.getData()["newIndex"]]));
	this.restartUpdate();
}


/**
 * Process a scroll event for this chart. 
 */
LineChartContainer.prototype.handleScrollEvent = function() {
	if (this.scrollTimer != 0 && (new Date()).getTime() < this.scrollTimer + LineChartContainer.FIRE_SCROLL_EVENT_THRESHOLD) {
		this.scrollTimer = 0;
		return;
	}
	
	this.fireScrolledEvent();
	this.scrolledChartUpdate();
}


// implementation


/**
 * Gets the maximum scroll height for the chart. When the scrolltop is at this value
 * the chart is scrolled all the way to the bottom. 0 if no scrollbar visible.  
 */
LineChartContainer.prototype.getMaxScrollHeight = function() {
	return this.imagesHeight() - parseInt(this.imageContainer().style.height) - this.axisLabelChartHeight();
}


/**
 * Depending on the size of the chart container, the size of each chart tag and how many charts are 
 * present there might be scrollbars visible for the chart. When the vertical scrollbar is shown 
 * the width of the charts need to be adjusted. This checks if scrollbar is visible and adjusts the 
 * size of the charts according.  
 */
LineChartContainer.prototype.adjustChartsWidth = function() {
	var verticalScrollbarVisible = this.isVerticalScrollbarVisible();
	if (verticalScrollbarVisible && !this.verticalScrollbarVisible) {
		this.setChartsWidth(this.getWidth() - LineChartContainer.BROWSER_SCROLLBAR_WIDTH);
		this.verticalScrollbarVisible = true;
	}
	else if (!verticalScrollbarVisible && this.verticalScrollbarVisible) {
		this.setChartsWidth(this.getWidth());
		this.verticalScrollbarVisible = false;
	}
}


/**
 * Checks whether the vertical scrollbar is visible. 
 * @return True if the vertical scrollbar is visible, false otherwise. 
 */
LineChartContainer.prototype.isVerticalScrollbarVisible = function() {
	if (this.getHeight() < this.imagesHeight()) {
		return true;
	}
	return false;
}


/**
 * Returns true if the chart is beeing updated periodically, false 
 * if the chart is not being updated. 
 */
LineChartContainer.prototype.isUpdating = function() {
	return this.updating;
}


/**
 * Starts the updating of the chart. This is overridden to not start the 
 * updating sequence. A line chart container chart is updated when its table is udpated. 
 */
LineChartContainer.prototype.startUpdate = function() {
	if (!this.isUpdating() && ChartManager.instance().getLoadChartStatus(this.window.getId())) {
		this.updating = true;
		this.updateChart();
	}
}



/**
 * Stops the updating of the chart.
 */
LineChartContainer.prototype.cancelUpdate = function() {
	this.updating = false;	
	this.udpateCounter = new Array();
	this.loadingData = new Array();
	this.loadingLineChartData = false;
}


/**
 * Updates this chart.
 */
LineChartContainer.prototype.updateChart = function() {
	if (!ChartManager.instance().getLoadChartStatus(this.window.getId()) || !this.updating) {
		return;
	}
	
	this.loading = true;
	if ((isFirefox() || isSafari()) && this.isChartCreatorAppletLoaded()) {
		this.requestLineChartData();
		return;
	}
	
	this.requestTimeToNextInterval();	
	var imageTags = this.lineChartImageTagsToUpdate();
	for (var i = 0; i < imageTags.length; i++) {
		this.setUrl(imageTags[i], this.createUrl(imageTags[i]));
	}
	this.updateAxisLabelChart();
}


/**
 * Updates the axis label chart. 
 */
LineChartContainer.prototype.updateAxisLabelChart = function() {
	this.setUrl(this.axisLabelImageTag(), this.createAxisLabelUrl());
}


/**
 * Sets a new url for a image.
 * @param image the image to set the url for.
 * @param newUrl The url to set for the image.
 */
LineChartContainer.prototype.setUrl = function(image, newUrl) {
	if (!ChartManager.instance().getLoadChartStatus(this.window.getId())) {
		return;
	}
	if (Chart.MAX_UPDATE_COUNTER == this.udpateCounter[image.id]) {
		// no more images are loaded until the last finishes
		return;
	}
	
	this.loading = true;
	this.increaseUpdateCounter(image);
	this.initLoadingImage(image);
	if ((isFirefox() || isSafari()) && this.isChartCreatorAppletLoaded()) {
		this.loadingImages[image.id].src = this.chartCreatorApplet().createChart(newUrl)
	}
	else {
		this.loadingImages[image.id].src = this.addForwardIpParameter(newUrl);
	}

	if (!this.firstLoad && !this.loadingShown) {
		this.showLoading();
	}
}



/**
 * Called when a chart container has done loading.
 * @param image The image that has been udpated. 
 */
LineChartContainer.prototype.chartUpdated = function(image) {
	if (!exists(this.loadingImages[image.id])) {
		// can happen if the image is closed while updating
		return;
	}
	var imageUpdated = this.loadingImages[image.id];
	this.loadingImages[image.id] = null;
	
	imageUpdated.onload = "";
	this.replaceImage(imageUpdated);
	this.decreaseUpdateCounter(imageUpdated);
	deallocateImage(this.loadingImages[image.id]);
	
	if (this.loadingShown && this.allImagesLoaded()) {
		this.hideLoading();
	}
	this.firstLoad = true;
	this.loading = !this.allImagesLoaded();
	this.loaded[imageUpdated.id] = true;
	
	if (this.timeToNextInterval >= 0 && !this.repeater.isDelaying()) {
		this.repeater.stop();
		this.repeater.setTime(this.timeToNextInterval + 1);
		this.repeater.delay();
	}
}


/**
 * Requests the data needed to create a line chart url.
 */
LineChartContainer.prototype.requestLineChartData = function() {
	if (!this.updating || this.loadingLineChartData) {
		return;
	}
	var images = this.lineChartImageTagsToUpdate();
	for (var i = 0; i < images.length; i++) {
		this.loadingLineChartData = true;
		this.loadingData[images[i].id] = true;
		getResponse(this.createLineChartDataUrl(images[i]), "lineChartDataCallBack", this);
	}
}



/**
 * The callback method for the loading of the chart data. Creates the new url and sets it.
 * @param chartData The chart data loaded.
 */
LineChartContainer.prototype.lineChartDataCallBack = function(chartData) {
	if (isEmpty(chartData) || this.isAccessDeniedResponse(chartData) || !this.updating) {
		this.loadingLineChartData = false;
		this.loadingData = new Array();
		return;
	}
	var image = this.getImage(getParameterValue(chartData, "chartId", "&"), this.lineChartImageTags());
	if (!exists(image) || !this.loadingData[image.id]) {
		return;
	}

	this.parseLineChartData(chartData, image);
	this.setUrl(image, this.createLineChartUrlData(chartData, image));
	this.sampleAxisLabel = URLDecode(getParameterValue(chartData, "sampleAxisLabel", "&"));
	this.loadingData[image.id] = false;
	if (this.allChartDataLoaded()) {
		this.timeToNextIntervalData(chartData);
		this.updateAxisLabelChart();
	}
	this.loadingLineChartData = !this.allChartDataLoaded();	
}


/**
 * Parses the line chart data.
 * @param chartData The loaded chart data.
 * @param image The image the chart data is for.  
 */
LineChartContainer.prototype.parseLineChartData = function(chartData, image) {
	if (this.intervalTime != this.intervalTimeChartData(chartData) || this.isIntervalDataDisabled(chartData)) {
		// if the interval time is changed or the interval data has been
		// disabled, clear the interval data
		this.sampleValues = new Array();
	}
	
	this.intervalTime = this.intervalTimeChartData(chartData, image);
	var dataTimeParameter = getParameterValue(chartData, "lastIntervalTime", "&");
	this.dataTime[image.id] = isNumberString(dataTimeParameter) ? parseInt(dataTimeParameter) : 0;
	this.setSampleValueChartData(chartData, image);
}


/**
 * Sets the line chart sample values from a line chart data response.
 * @param chartDataResponse The response to the get the sample values from. 
 * @param image The image to set the sample values for. 
 */
LineChartContainer.prototype.setSampleValueChartData = function(chartDataResponse, image) {
	var currentUnit = this.sampleValuesUnit[image.id];
	this.sampleValuesUnit[image.id] = exists(currentUnit) ? currentUnit : "bit/s";
	this.recalculateSampleValues(this.sampleValuesUnit[image.id], trim(getParameterValue(chartDataResponse, "rangeLabelPostfix", "&")), image);
	this.sampleValuesUnit[image.id] = trim(getParameterValue(chartDataResponse, "rangeLabelPostfix", "&"));
	this.sampleValues[image.id] = this.addSampleValues(chartDataResponse, this.sampleValues[image.id]);	
}


/**
 * Recalculates the sample values for a new unit if needed. 
 * @param unit The unit the sample values data currently is in.
 * @param newUnit The new unit for the sample values data. 
 * @param image The chart image tag to recalcuate the sample values for. 
 */
LineChartContainer.prototype.recalculateSampleValues = function(unit, newUnit, image) {
	if (!this.sampleValuesUnit[image.id].equalsIgnoreCase(newUnit) && !isEmpty(this.sampleValues[image.id])) {
		this.sampleValues[image.id] = this.changeThroughputUnit(unit, newUnit, this.sampleValues[image.id]);
	}
}


/**
 * Checks whether all chart images have done loading their data. 
 * @return True if all line chart images have loaded data, false otherwise.  
 */
LineChartContainer.prototype.allChartDataLoaded = function() {
	for (imageId in this.loadingData) {
		if (this.loadingData[imageId] == true) {
			return false;
		}
	}
	return true;
}


/**
 * Creates the url to request the line chart data. 
 * @param image The line chart image tag to create the data url for. 
 * @return The url to request the chart data. 
 */
LineChartContainer.prototype.createLineChartDataUrl = function(image) {
	var resource = getParameterValue(this.getBaseUrl(), "resource", "&");
	var url = "/?resource=" + resource + "&" + Chart.INTERVAL_CHART_DATA_BASE_URL + "sid=" + getSid(); 
	url += "&field=" + this.getField();
	url += "&src=" + image.id; 
	url += "&dst=" + this.getDstAddress();
	url += "&protocol=" + this.getProtocol();
	this.dataTime[image.id] = !exists(this.dataTime[image.id]) ? 0 : this.dataTime[image.id]; 
	url += "&time=" + this.dataTime[image.id] + "&";
	return url + "chartId=" + image.id;
}


/**
 * Creates the url for the axis label chart. 
 * @return The created url for the axis label chart. 
 */
LineChartContainer.prototype.createAxisLabelUrl = function() {
	var url = this.getBaseUrl() + LineChartContainer.AXIS_LABEL_CHART_PARAMETERS;
	url = replaceParameterValue(url, "width",  this.getWidth() + "", "&"); 
	url = replaceParameterValue(url, "height", "25", "&");
	url += "&sampleValuesOn=false";
	if ((isFirefox() || isSafari()) && this.isChartCreatorAppletLoaded()) {
		url += "&sampleAxisLabel=" + this.sampleAxisLabel + "&"; 
	}	
	
	if (needTimestamp(url) || this.markedForTimestamp) {
		url = this.updateChartTimestamp(url);
		this.markedForTimestamp = false;
	}
	return url;
}


/**
 * Creates a line chart url based on the loaded chart data.
 * @param chartData The chart data represented as a string.  
 * @param image The chart image tag to create the url for. 
 */ 
LineChartContainer.prototype.createLineChartUrlData = function(chartData, image) {
	var url = this.createUrl(image) + "&";
	url = this.addParameterToUrl("locale", chartData, url);
	url += "rangeLabelPostfix= " + this.sampleValuesUnit[image.id] + "&";
	url = this.addParameterToUrl("range", chartData, url);
	url = this.addParameterToUrl("rangeDecimalCount", chartData, url);
	url += "servletMode=true&";
	url = this.addParameterToUrl("error", chartData, url);
	url += "sampleValues=" + this.sampleValues[image.id];
	return url;
}


/**
 * Creates a url for a line chart.
 * @param image The image to create the url for. 
 * @returns The newly created url.
 */
LineChartContainer.prototype.createUrl = function(image) {
	var url = this.getBaseUrl();
	url += this.isFirstImage(image) ? Chart.LINECHART_PARAMETERS : LineChartContainer.LINECHART_PARAMETERS_SINGLE_BACKGROUND_COLOR;
	url += this.createEntryUrl(image);
	url = replaceParameterValue(url, "width",  parseInt(image.style.width), "&"); 
	url = replaceParameterValue(url, "height", parseInt(image.style.height), "&");
	url += "sampleAxisLabeOn=false";
	url += "&legendOn=true&seriesLabels=" + this.legendLabel(image);
	url += "&graphInsets=10,70,10,125";
	if (needTimestamp(url) || this.markedForTimestamp) {
		url = this.updateChartTimestamp(url);
		this.markedForTimestamp = false;
	}
	return url;
}


/**
 * Creates the entry part of a chart url for a given chart image tag. 
 * @param image The chart image tag to create the entry part of the url for. 
 * @return The entry part of the url.
 */
LineChartContainer.prototype.createEntryUrl = function(image) {
	var entryUrl = "protocol=" + this.getProtocol() + "&";
	entryUrl += "src=" + image.id + "&";
	entryUrl += "dst=" + this.getDstAddress() + "&";
	return entryUrl + "field=" + this.getField() + "&"
}


/**
 * Get the legend label for a given chart image tag. 
 * @param image The chart image tag to get the legend label for. 
 * @return The legend label to use for the given chart image tag. 
 */
LineChartContainer.prototype.legendLabel = function(image) {
	var row = this.table.getRowWithFieldValue("src", image.id);
	var field = exists(row) ? getFieldFromRow("probeName", row) : null;
	if (exists(field) && !isEmpty(field.getValue())) {
		return field.getValue();
	}
	else if (exists(row) && exists(getFieldFromRow("src", row))) {
		return getFieldFromRow("src", row).getValue();
	}
	return "";
}


/**
 * Replaces the image in the container with id like the given image.
 * @param image The image to insert.
 */
LineChartContainer.prototype.replaceImage = function(image) {
	if (!isDefined(this.htmlCode.parentNode) || isNull(this.htmlCode.parentNode)) {
		// the image has no parent, can happen if the image is closed
		// while updating
		return;	
	}
	if ("axisLabel" == image.getAttribute("id")) {
		image.style.padding = "0px";
		image.style.margin = "0px";
		this.getHtmlCode().replaceChild(image, this.axisLabelImageTag());
		return;
	}
	var images = this.lineChartImageTags();
	for (var i = 0; i < images.length; i++) {
		if (image.getAttribute("id") == images[i].getAttribute("id")) {
			images[i].parentNode.replaceChild(image, images[i]);
			return;
		}
	}
}


/**
 * Returns true if all of the images in the container have loaded.
 */
LineChartContainer.prototype.allImagesLoaded = function() {
	var images = this.lineChartImageTags();
	for (var i = 0; i < images.length; i++) {
		if (exists(this.loadingImages[images[i].id])) {
			return false;
		}
	}
	if (exists(this.loadingImages[this.axisLabelImageTag().id])) {
		return false;
	}	
	return true;
}


/**
 * Inits the loading image for a given chart. 
 * @param image The image to create the loading image for. 
 */
LineChartContainer.prototype.initLoadingImage = function(image) {
	deallocateImage(this.loadingImages[image.id]);
	this.loadingImages[image.id] = null;
	this.loadingImages[image.id] = new Image();
	this.loadingImages[image.id].id = image.id;
	this.loadingImages[image.id].onload = chartUpdated;
	this.loadingImages[image.id].style.height = image.style.height;
	this.loadingImages[image.id].style.width = image.style.width;
}


/**
 * Get the loading image. The loading image is used for silent image loading in Internet Explorer. 
 * If the chart is not currently loading this will return null.
 * @param id The id of the image to get the loading image for. 
 * @return The loading image, null if the chart is not loading.
 * @throws Error if id is undefined, null or empty. 
 */
LineChartContainer.prototype.getLoadingImage = function(id) {
	assertNotEmpty(id, "The id of the image to get the loading image for is undefined, null or empty");
	return this.loadingImages[id];
}


/**
 * Increased a images updateCounter by one.
 * @param image The html code of the image to increase the counter for.
 */
LineChartContainer.prototype.increaseUpdateCounter = function(image) {
	if (!isDefined(this.udpateCounter[image.id])) {
		this.udpateCounter[image.id] = 0;
	}
	this.udpateCounter[image.id] = this.udpateCounter[image.id] + 1;
}


/**
 * Decrease a images updateCounter by one.
 * @param image The html code of the image to decrease the counter for.
 */ 
LineChartContainer.prototype.decreaseUpdateCounter = function(image) {
	if (Chart.MAX_UPDATE_COUNTER == this.udpateCounter[image.id]) {
		this.udpateCounter[image.id] = 0;
	}
	else {
		this.udpateCounter[image.id] = this.udpateCounter[image.id] - 1;
	}
}


/**
 * Updates the chart image tags with data from the table. New image tags are created, 
 * old ones are removed, the images get the correct size set and they are sorted, but
 * they are not updated. 
 */
LineChartContainer.prototype.updateChartTags = function() {
	this.removeCharts();	
	var data = this.table.getData();
	var images = this.lineChartImageTags();
	for (var i = 0; i < data.length; i++) {
		var src = getFieldFromRow("src", data[i]).getValue();
		var image = this.getImage(src, images);
		if (!exists(image)){
			this.addImage(src);
		}
	}
}


/**
 * Checks the height of the charts, and change it if necessary.  
 */
LineChartContainer.prototype.checkHeight = function() {
	if (this.imagesHeight() < this.getHeight()) {
		var diff = this.imagesHeight() - this.getHeight();
		this.setHeight(this.imagesHeight());
		this.imageContainer().style.height = (this.imagesHeight() - this.axisLabelChartHeight()) + "px";
		this.fireChartResizedEvent(diff, 0);
	}	
}


/**
 * Sorts the images according to the data in the table. 
 */
LineChartContainer.prototype.sortImages = function() {
	var data = this.table.getData();
	var images = this.lineChartImageTags();
	var container = this.imageContainer();
	for (var i = 0; i < data.length; i++) {
		var src = getFieldFromRow("src", data[i]).getValue();
		var index = this.getImageIndex(src, images);
		if (index == i) {
			// the images has the right position
			continue;
		}
		container.insertBefore(images[index], images[i]);
		images = this.lineChartImageTags();
	}
}


/**
 * Removes image tags for the line chart tags not in the table any more. 
 */
LineChartContainer.prototype.removeCharts = function() { 
	var images = this.lineChartImageTags();
	if (!exists(images) || images.length == 0) {
		return;
	}
	for (var i = images.length - 1; i >= 0; i--) {
		if (!exists(this.table.getRowWithFieldValue("src", images[i].id))) {
			this.imageContainer().removeChild(images[i]);
		}
	}
}


/**
 * Adds line chart image tag. 
 * @param id The id to set for the image. 
 */
LineChartContainer.prototype.addImage = function(id) {
	var image = document.createElement("img");
	image.setAttribute("id", id);
	image.style.width = (this.isVerticalScrollbarVisible() ? this.getWidth() - LineChartContainer.BROWSER_SCROLLBAR_WIDTH : this.getWidth()) + "px";
	image.style.height = this.chartHeight() + "px";
	this.imageContainer().appendChild(image);
}


/**
 * Sets the height of all the be chart image tags to be equal to a given height. The height of the axis
 * label tag is not set. 
 * @param height The height to set. 
 */
LineChartContainer.prototype.setChartsHeight = function(height) {
	var images = this.lineChartImageTags();
	if (images.length == 0) {
		return;
	}
	
	var chartHeight = Math.round(height / this.getLineChartCount());
	for (var i = 0; i < images.length; i++) {
		images[i].style.height = parseInt(chartHeight) + "px";
	}
	
	// check for rounding errors
	if ((this.imagesHeight() - this.axisLabelChartHeight()) == height) {
		return;
	}
	var diff = height - (this.imagesHeight() - this.axisLabelChartHeight());
	var images = this.lineChartImageTags();
	images[images.length - 1].style.height = (parseInt(images[images.length - 1].style.height) + diff) + "px";
}


/**
 * Calculates the min height for the charts. 
 * @return The minimumm height for the current amount of chart image tags. 
 */
LineChartContainer.prototype.minHeight = function() {
	return this.getLineChartCount() * LineChartContainer.LINE_CHART_MIN_HEIGHT;
}


/**
 * Sets the width of all chart image tags. The width of the axis label tag is not set. 
 * @param width The width to set. 
 */
LineChartContainer.prototype.setChartsWidth = function(width) {
	var images = this.lineChartImageTags();
	if (images.length == 0) {
		return;
	}
	for (var i = 0; i < images.length; i++) {
		images[i].style.width = parseInt(width) + "px";
	}
}


/**
 * Gets the height of the axis label chart. 
 * @return The height of the axis label chart. 
 */
LineChartContainer.prototype.axisLabelChartHeight = function() {
	return parseInt(this.axisLabelImageTag().style.height);
}


/**
 * Gets the height of all the images in this chart including the axis label chart. 
 * @return height The height of all the images, including the axis label chart. 
 */
LineChartContainer.prototype.imagesHeight = function() {
	var height = 0;
	var images = this.lineChartImageTags();
	for (var i = 0; i < images.length; i++) {
		height += parseInt(images[i].style.height);
	}
	return height + this.axisLabelChartHeight();
}


/**
 * Gets the height to use for the line charts. All line charts should be of the same height. 
 */
LineChartContainer.prototype.chartHeight = function() {
	if (this.lineChartImageTags().length == 0) {
		return LineChartContainer.LINE_CHART_DFEAULT_HEIGHT;
	}
	return parseInt(this.lineChartImageTags()[0].style.height);
}


/**
 * Checks whether a given image is the first in the image container. 
 * @param image The image to check.
 * @return True if image is the first image in the container, false otherwise. 
 */
LineChartContainer.prototype.isFirstImage = function(image) {
	var images = this.lineChartImageTags();
	return images.length == 0 ? false : (images[0] == image);
}

 
/**
 * Gets a image with a given id from a given array of images. 
 * @param id The idof the image to search for. 
 * @param images Search in this array of image tags. 
 * @return The first image with id in images, null if no image with id in images. 
 */
LineChartContainer.prototype.getImage = function(id, images) {
	var index = this.getImageIndex(id, images);
	if (index == -1) {
		return null;
	} 
	return images[index];
}
 

/**
 * Gets the index of a image with a given id for a array of images. 
 * @param id The id to search for. 
 * @param images The array of images to search in. 
 * @return The index of the image with id in images, -1 if no image with id in images. 
 */
LineChartContainer.prototype.getImageIndex = function(id, images) {
	for (var i = 0; exists(images) && i < images.length; i++) {
		if (images[i].id == id) {
			return i;
		}			
	}
	return -1;
}


/**
 * Gets the count of line chart images in this line chart container. 
 */
LineChartContainer.prototype.getLineChartCount = function() {
	if (exists(this.imageContainer().getElementsByTagName("img"))) {
		return this.imageContainer().getElementsByTagName("img").length;
	}
	return 0;
}


/**
 * Gets the axis label image tag. 
 * @return The axis label image tag. 
 */
LineChartContainer.prototype.axisLabelImageTag = function() {
	return getElementWithAttribute(this.getHtmlCode(), "axisLabel", "id", "img");
}


/**
 * Gets the chart image tags that need to be reloaded. Chart image tags not visible in the 
 * current scrolling do not need to be updated. 
 * @return Chart image tags in the current scroll position. 
 */
LineChartContainer.prototype.lineChartImageTagsToUpdate = function() {
	var images = this.lineChartImageTags(); 
	if (!this.isVerticalScrollbarVisible()) {
		return images;
	}
	
	var start = Math.floor(this.imageContainer().scrollTop / this.chartHeight());
	var max = start + Math.ceil(parseInt(this.imageContainer().style.height) / this.chartHeight());
	var imagesToUpdate = new Array();
	for (var i = 0; i < images.length; i++) {
		if ((i >= start && i <= max) || !exists(this.loaded[images[i].id])) { 
			imagesToUpdate[imagesToUpdate.length] = images[i];
		}
	} 
	return imagesToUpdate;
}

 
/**
 * Gets the chart image tags as a array.
 * @return All the chart image tags as a array, null if no line chart image tags. 
 */
LineChartContainer.prototype.lineChartImageTags = function() {
	return this.imageContainer().getElementsByTagName("img");
}


/**
 * Gets the container for the image tags. 
 */
LineChartContainer.prototype.imageContainer = function() {
	return getElementWithAttribute(this.getHtmlCode(), "imageContainer", "name", "div");
}



/**
 * Hides the loading animation and shows the chart.
 */
LineChartContainer.prototype.hideLoading = function() {
	this.htmlCode.removeChild(this.loadingAnimation.getHtmlCode());
	if (this.window.activeChart.title == this.title) {
		setClass(this.imageContainer(), "active");
		setClass(this.axisLabelImageTag(), "active");
	}
	this.loadingShown = false;
}



/**
 * Shows a loading animation instead of the chart.
 */
LineChartContainer.prototype.showLoading = function() {
	if (this.loadingShown) {
		return;
	}	
	
	this.loadingShown = true;
	setClass(this.imageContainer(), "inactive");
	setClass(this.axisLabelImageTag(), "inactive");
	this.createLoadingScreen();
	this.loadingAnimation.show();
}


/**
 * Removes data for any non existing entries. 
 */
LineChartContainer.prototype.removeNotExistingEntries = function() {
	if (isEmpty(this.entries)) {
		return true;
	}
	var tokenizer = new StringTokenizer(this.entries, ",");
	while (true) {
		if (!tokenizer.hasMoreTokens()) {
			break;
		}
		var src = tokenizer.nextToken();
		var row = this.table.getRowWithFieldValue("src", src);
		if (!exists(row)) {
			// the entry does not exist in the table any more
			this.sampleValues[src] = undefined;
			this.dataTime[src] = undefined;
			this.sampleValuesUnit[src] = undefined;
			this.loadingData[src] = undefined;
			this.loaded[src] = undefined;
		}
	}
}


/**
 * Returns true if the charts need to be changed in order as a response to a table update. 
 * This will also return true if the charts need to be updated. 
 */
LineChartContainer.prototype.chartSortNeeded = function() {
	if (isEmpty(this.entries)) {
		this.entries = this.table.getFieldValues("src", -1);
		return true;
	}
	var newEntries = this.table.getFieldValues("src", -1);
	return isEmpty(newEntries) ? false : newEntries.indexOf(this.entries) == -1;
}


/**
 * Returns true if a the charts need to be udpated to response as a table update.  
 */
LineChartContainer.prototype.chartUpdateNeeded = function() {
	if (isEmpty(this.entries)) {
		this.entries = this.table.getFieldValues("src", -1);
		return true;
	}
	
	var newEntries = this.table.getFieldValues("src", -1);
	var newEntriesTokenizer = isEmpty(newEntries) ? null : new StringTokenizer(newEntries, ",");
	var currentEntriesTokenizer = new StringTokenizer(this.entries, ",");
	if  (exists(newEntriesTokenizer) && newEntriesTokenizer.countTokens() != currentEntriesTokenizer.countTokens()) {
		return true;
	}
	while (true) {
		if (!exists(newEntriesTokenizer) || !newEntriesTokenizer.hasMoreTokens()) {
			break;
		}
		if (this.entries.indexOf(newEntriesTokenizer.nextToken()) == -1) {
			return true;
		}
	}
	return false;
}


/**
 * Updates the charts visible in the current scrolling as a response to a scroll event. 
 */
LineChartContainer.prototype.scrolledChartUpdate = function() {
	if (exists(this.scrollTimeOutId)) {
		window.clearTimeout(this.scrollTimeOutId);
	}
	var thisObj = this;
	this.scrollTimeOutId = window.setTimeout(function(){ thisObj.restartUpdate() }, LineChartContainer.SCROLL_RELOAD_TIMEOUT);
}


/**
 * Fires a chart scrolled event. 
 */
LineChartContainer.prototype.fireScrolledEvent = function() {
	var eventData = new Array();
	eventData["id"] = this.getId();
	eventData["scrollTop"] = this.imageContainer().scrollTop; 
	eventData["maxScrollHeight"] = this.getMaxScrollHeight(); 
	getEventServer().notifyListeners(new Event("scrolled", eventData));
}