/**
 * This file contains functions to load and set layouts.
 */


/**
 * The request used to save the layout. The sid and the actual
 * layout to save need to be added to this request. 
 */
var SAVE_LAYOUT_REQUEST = "/?resource=layoutmanager&mode=set&command=layout&action=set&";


/**
 * The request to load the layout. The sid needs to be added to this request. 
 */
var LOAD_LAYOUT_REQUEST = "/?resource=layoutmanager&mode=get&command=layout&";


/**
 * The name of the layout set event. 
 */
var LAYOUT_SET_EVENT = "layoutSet";


/**
 * The name of the event signalising that the layout manager has started setting the layout. 
 */
var START_SET_LAYOUT_EVENT = "startSettingLayout";


/**
 * The name of the event fired when we start setting a windows layout.
 */
var START_SET_WINDOW_LAYOUT = "startSettingWindowLayout";


/**
 * The name of the event fired setting a windows layout has finished.
 */
var WINDOW_LAYOUT_SET = "windowLayoutSet"; 
 
 
/**
 * Windows that are not be included in the layout. 
 */
var noLayoutWindows = ["triggervalueswindow", "alarmactionswindow", "alarmDetailWindow", 
	"alarmactionslibrarywindow", "licenseStatusWindow", "slaveDetails", "slaveSetup", "messageWindow"];


/**
 * The current layout used. 
 */
var currentLayout = new Array();


// services


/**
 * Load the layout data.
 */
function loadLayout() {
	loadXML(createLayoutUrl(), setLayout);
}


/**
 * Creates the url to load a layout. 
 * @return The url to use to load the layout. 
 */
function createLayoutUrl() {
	return LOAD_LAYOUT_REQUEST + "&sid=" + getSid() + "&tabName=" + currentTabName;
}


/**
 * Saves the current layout to the server.
 */
function saveLayout() {
	var xmlLayout = createLayout();
	var request = SAVE_LAYOUT_REQUEST + "sid=" + getSid() + "&tabName=" + currentTabName + "&xml=" + encodeURIComponent(xmlLayout);
	postRequest("/", request);
}


/**
 * Callback function for loading the layout. Sets the specified layout. 
 * @param layout The layout to set as xml data. 
 * @throws Error if layout is undefined. 
 */
function setLayout(layout) {
	assertDefined(layout, "The layout to set is undefined");
	fireStartSettingLayoutEvent();
	if (!isLayout(layout)) {
		setDefaultLayout();
		return;
	}
	
	resetLayoutSet();
	setWindowsLayout(layout);
	resetWindowsNotInLayout();
	fireLayoutSetEvent();
}


/**
 * Resets windows not in the current layout.
 */
function resetWindowsNotInLayout() {
	var windows = WindowManager.instance().getWindows();
	for (var i = 0; i < windows.length; i++) {
		if (exists(windows[i]) && (isProfileWindow(windows[i]) || isSetupWindow(windows[i]))) {
			continue;
		}
		if (exists(windows[i]) && !windows[i].isCurrentLayoutSet() && !exists(getWindowLayout(windows[i].getId())) && (exists(windows[i].isDetachedWindow) && !windows[i].isDetachedWindow())) {
			WindowManager.instance().hideWindow(windows[i].getId());
			windows[i].resetCharts();
		}
	}
}


/**
 * Sets the current layout for all the windows.
 * @param layout The layout to set. 
 */
function setWindowsLayout(layout) {
	currentLayout = parseLayout(layout);
	for (var i = 0; i < currentLayout.length; i++) {
		if (needToSetWindowLayout(currentLayout[i])) {
			setWindowLayout(currentLayout[i]);
		}
		else if (!needToSetWindowLayout(currentLayout[i]) && minimized(currentLayout[i])) {
			WindowManager.instance().createStatusbarButton(getWindow(currentLayout[i]));
			WindowManager.instance().minimizeWindow(getWindow(currentLayout[i]).getId());
		}
		else {
			hideWindow(currentLayout[i])
		}
	}
}


/**
 * Hides a window.
 * @param windowLayout The windows xml layuot.
 */ 
function hideWindow(windowLayout) {
	var windowId = getTagNodeValue("windowId", windowLayout);
	if (!WindowManager.instance().existsWindow(windowId)) {
		// the window does not exist, and therefore does not need to be hidden
		return;
	}
	var window = getWindow(windowLayout);
	if (exists(window) && !window.isHidden()) {
		WindowManager.instance().hideWindow(window.getId());
		hideDetachedWindows(window);
	}
	if (exists(window)) {
		window.setCurrentLayoutSet(false);
	}
}


/**
 * Hides all the detached windows of a window. 
 */
function hideDetachedWindows(window) {
	var windows = window.getDetachedWindows();
	for (var i = 0; i < windows.length; i++) {
		WindowManager.instance().hideWindow(windows[i].getId());
	}
}


/**
 * Returns true if a xml window layout needs to be set, false otherwise.
 * @param windowLayout The window layout to check.
 */
function needToSetWindowLayout(windowLayout) {
	var status = getTagNodeValue("status", windowLayout);
	var windowId = getTagNodeValue("windowId", windowLayout);
	var window = !isEmpty(windowId) && WindowManager.instance().existsWindow(windowId) ? getWindow(windowLayout) : null;
	if (exists(window) && getDetachedCharts(windowLayout).length != window.getDetachedCharts().length) {
		// if the window contains some detached charts that are not in the layout or vice versa
		// the layout needs to be set to configure the window and its charts correctly
		return true;
	}
	if ("hidden" == status && containsDetachedVisibleChart(windowLayout)) {
		// the window contains a visible detached chart, the windows layout
		// need to set even if itself is hidden because of the detached chart
		return true;
	}
	else if ("hidden" == status) {
		return false;
	}
	else if (minimized(windowLayout) && containsDetachedVisibleChart(windowLayout)) {
		return true;
	}
	else if (minimized(windowLayout)) {
		return false;
	}
	
	return true;
}


/**
 * Checks if a window status is for the window to be minimzed. 
 * @param windowLayout The window layout to check.
 * @return True if the window is to be minimized according to this wlayout, false otherwise. 
 */ 
function minimized(windowLayout) {
	var status = getTagNodeValue("status", windowLayout);
	if ("minimized" == status) {
		return true;
	}
	return false;
}


/**
 * Returns true if a windwos xml layout contains a detached chart which is not hidden, false otherwise.
 * @param windowLayout The window layout to check.
 */
function containsDetachedVisibleChart(windowLayout) {
	var charts = windowLayout.getElementsByTagName("chart");
	for (var i = 0; i < charts.length; i++) {
		if (!isDetachedChart(charts[i])) {
			continue;
		}
		var status = getTagNodeValue("status", charts[i]);
		if ("hidden" != status) {
			// the chart is detached and visible
			return true;
		}
	}
	return false;
}

 
/**
 * Sets the default layout.
 */
function setDefaultLayout() {
	initDefaultWindows();
	initDefaultTables();
	saveLayout();
	fireLayoutSetEvent();
}


/**
 * Get the layout for a given window.
 * @param id The of the window to get the layout for. 
 * @return The layout for the window, null if not defined, null if not found.
 * @throws Error if id is undefined, null or empty.
 */
function getWindowLayout(id) {
	assertNotEmpty(id, "Id is undefined, null or empty");
	for (var i = 0; i < currentLayout.length;  i++) {
		var windowId = getTagNodeValue("windowId", currentLayout[i]);
		if (id == windowId) {
			return currentLayout[i];
		}
	}
	return null;
}	


/**
 * Gets the layout for a given chart in a given window.
 * @param chartId The id of the chart to get the layout for.
 * @param windowId The id of the window the chart is in.
 * @return The xml layout of the chart, null if not defined in window or if no layout has been laoded.
 * @throws Error if chartId is undefined, null or empty.
 * @throws Error if windowId is undefined, null or empty.
 */
function getChartLayout(chartId, windowId) {
	assertNotEmpty(chartId, "ChartId is undefined, null or empty");
	assertNotEmpty(windowId, "Window id is undefined, null or empty");
	var windowLayout = getWindowLayout(windowId);
	if (isNull(windowLayout)) {
		return null;
	}
	
	var charts = getCharts(windowLayout);
	for (var i = 0; i < charts.length; i++) {
		if (chartId = getTagNodeValue("chartId", charts[i])) {
			return charts[i];		
		}
	}
	return null;
}


/**
 * Creates the layout for all windows, tables and charts.
 */
function createLayout() {
	var xmlString = "<layout>";
	
	if (currentLayout.length == 0) {
		// no layout has been loaded yet, save all the existing windows layout
		return xmlString + createExistingWindowLayout() + "</layout>";
	}
	
	// loop through the currentlayout and update the layout of the existing windows,
	// this way layout of uninitialized windows are not lost
	xmlString += createCurrentWindowLayout();
	
	// check for any existing windows that have not previously saved a layout
	// this can happen for windows that have not been initilized after the last
	// layout was saved
	xmlString += createExistingWindowLayout(xmlString);

	return xmlString + "</layout>";
}


/**
 * Creates the layout for all the existing windows.
 * @param if This is given only windows that are not in this layout are added.  
 * @return The layout created. 
 */
function createExistingWindowLayout(layout) {
	var xmlString = "";
	var windows = WindowManager.instance().getWindows();
	for (var i = 0; i < windows.length; i++) {
		if (exists(layout) && layout.indexOf(windows[i].getId()) > -1)  {
			continue;
		}
		xmlString += createWindowLayout(windows[i]);
	}
	return xmlString;
}


/**
 * Creates the layout for all windows in the current layout. This way the layout 
 * of windows not initialized yet are not lost.
 * @return The created layout. 
 */
function createCurrentWindowLayout() {
	var xmlString = "";
	for (var i = 0; i < currentLayout.length; i++) {
		var windowId = getTagNodeValue("windowId", currentLayout[i]);	
		if (WindowManager.instance().existsWindow(windowId)) {
			xmlString += createWindowLayout(getWindow(currentLayout[i]));
		}
		else {
			xmlString += xmlToString(currentLayout[i]) + "\n";
		}	
	}
	return xmlString;
}


/**
 * Creates the layout for a window.
 * @param window The window to create the layout for.
 * @return The layout for the window.
 */
function createWindowLayout(window) {
	if (isProfileWindow(window) || isSetupWindow(window) || !isLayoutWindow(window)) {
		// setup windows and the profile window are not to be saved as a part of a layout
		return "";
	}
	else if (!window.isDetachedWindow || !window.isDetachedWindow()) {
		return window.toXml() + "\n";
	}
	return "";
}


/**
 * Returns true if the window is to be included in a layout, false otherwise. 
 * @param window The window to check. 
 */
function isLayoutWindow(window) {
	for (var i = 0 ; i < noLayoutWindows.length; i++) {
		if (noLayoutWindows[i] == window.id) {
			return false;
		}
	}
	return true;
}


/**
 * Sets the layout for one windows based on some given xml data. 
 * @param windowLayout The xml data containing the window layout to set. 
 */
function setWindowLayout(windowLayout) {
	if (!exists(windowLayout)) {
		return;
	}
	this.fireSetWindowLayoutEvent(windowLayout);
	var window = getWindow(windowLayout);
	if (isNull(window)) {
		return;
	}
	prepareToSetLayout(windowLayout);
	setTableLayout(windowLayout);
	window.setXmlLayout(windowLayout);
	setChartsLayout(windowLayout);
	finishSetLayout(windowLayout);
	fireWindowLayoutSetEvent(window);
}


/**
 * Finishes setting the layout. 
 * @param layout The xml data containing the window layout to set. 
 */
function finishSetLayout(layout) {
	var tableXml = layout.getElementsByTagName("table");
	for (var i = 0; i < tableXml.length; i++) {
		var table = tableXml.length == - 1 ? null : getTable(tableXml[i]);
		if (exists(table)) {
			table.settingLayout(false);
		}
	}
}


/**
 * Prepares the window and the table for getting their layout set. 
 * @param layout The xml data containing the window layout to set. 
 */
function prepareToSetLayout(layout) {
	var window = getWindow(layout);
	if (!isNull(window)) {
		window.setLayout = true;
		window.setCurrentLayoutSet(true);	
		window.chartLayoutSet = new Array();
	}
	
	var tableXml = layout.getElementsByTagName("table");
	for (var i = 0; i < tableXml.length; i++) {
		var table = tableXml.length == - 1 ? null : getTable(tableXml[i]);
		if (exists(table)) {
			table.settingLayout(true);
		}
	}
	
	// need to set the window status before we can set the layout of the table and
	// charts, because the window needs to be visible before we can set the layout
	window.setXmlStatus(layout);
	if (!window.isShown()) {
		WindowManager.instance().showWindow(window.getId());
	}
	
	window.resetCharts();
}


/**
 * Resets the layout set status of all the windows.
 */
function resetLayoutSet() {
	var windows = WindowManager.instance().getWindows();
	for (var i = 0; i < windows.length; i++) {
		windows[i].setCurrentLayoutSet(false);	
	}
}


/**
 * Sets the layout for a table. 
 * @param layout The layout for the window the table is in.
 */
function setTableLayout(layout) {
	var tableXml = layout.getElementsByTagName("table");
	if (tableXml.length == -1) {
		return;
	}
	for (var i = 0; i < tableXml.length; i++) {
		var table = getTable(tableXml[i]);
		if (isNull(table)) {
			return;
		}
		table.setXmlLayout(tableXml[i]);
	}	
}


/**
 * Sets the layout for the charts. 
 * @param layout The layout for the window the charts are in. 
 */
function setChartsLayout(layout) {
	var window = getWindow(layout);
	if (!exists(window.charts)) {
		return;
	}
	
	
	// in IE the select tag for the charts is hidden when a window is inactive
	// therefore we need to activate the window before we can add any charts
	// because of the size of the replaceField needs to be calculated using
	// offsetHeight and offsetWidth
	var activeWindow = WindowManager.instance().getActiveWindow();
	WindowManager.instance().activateWindow(window.getId());
	var charts = getCharts(layout);
	for (var i = 0; i < charts.length; i++) {
		var chartXml = charts[i];
		if (!isActiveChart(chartXml, layout) && !isDetachedChart(chartXml) && !isTopNChart(chartXml)) {
			// the chart is not the active chart and not visible, insert it into the
			// chart selector, do not set its layout.
			addToSelector(window.chartSelector, getTagNodeValue("chartId", chartXml), URLDecode(getTagNodeValue("title", chartXml)), false);
			continue;	
		}
		setChartLayout(chartXml, window);
	}
	
	// reactivate the window active before this one
	if (exists(activeWindow)) {
		WindowManager.instance().activateWindow(activeWindow.getId());
	}
	// set back this windows original status
	window.setXmlStatus(layout);
}


/**
 * Sets the layout of one chart. 
 * @param chartLayout The xml chart layout to set. 
 * @param window The window the chart is in. 
 */
function setChartLayout(chartLayout, window) {
	var chart = createChartFromXml(chartLayout, window);
	if (exists(chart) && isDetachedChart(chartLayout)) {
		setDetachedChartLayout(chart, chartLayout, window);
	}
	else if (exists(chart)) {
		attachChartToWindow(chart, window);
	}
}


/**
 * Sets the layout for a detached chart to set. 
 * @param chart The chart to detach.
 * @param layout The layout of the detached chart.
 * @param window The window the chart to detach is in. 
 */
function setDetachedChartLayout(chart, layout, window) {
	window.chartLayoutSet[window.chartLayoutSet.length] = chart.getId();
	setClass(chart.htmlCode, "active");
	chart.htmlCode.style.width = window.container.style.width;
	chart.htmlCode.style.height = Chart.DEFAULT_HEIGHT + "px";

	var newWindow = createChartWindow(chart, uniqueInteger() + "", chart.getTitle(), window.id);
	newWindow.detachedWindow = true;
	
	// remove the chart from the window
	removeFromSelector(window.chartSelector, chart.getId());
	window.removeResizableElement(chart.getHtmlCode());
	window.removeChart(chart);	
	window.chartDetached(chart, newWindow);
	newWindow.setXmlLayout(layout);
}


/**
 * Returns true if a chart is the active chart of a window, false otherwise.
 * @param chart The xml layout of the chart to check.
 * @param window The xml layout of the window the chart is in. 
 */
function isActiveChart(chart, window) {
	var activeChartId = getTagNodeValue("activeChartId", window);
	var chartId = getTagNodeValue("chartId", chart);
	if (activeChartId == chartId) {
		return true;
	}
	return false;
}


/**
 * Returns true if a chart represented as a block of xml is a topn chart, false otherwise.
 * @param chartLaytou The block of xml code representing the chart to check.
 */
function isTopNChart(chartLayout) {
	var chartType = getTagNodeValue("type", chartLayout);
	if (Chart.TOPN_CHART == chartType) {
		return true;
	}
	return false;
}


/**
 * Returns true if a chart represented as a block of xml is detached, false otherwise. 
 * @param chartLayout The block of xml code representing the chart. 
 */
function isDetachedChart(chartLayout) {
	var status = getTagNodeValue("chartStatus", chartLayout);
	if ("detached" == status) {
		return true;
	}
	return false;
}


/**
 * Get the table a xml layout is for. 
 * @param tableLayout the xml data containing the layout for the table. 
 * @reutn The table object, null if no window with the given id exists, or no id is in tableLayout. 
 */
function getTable(tableLayout) {
	var tableId = getTagNodeValue("tableId", tableLayout);
	if (!isDefined(tableId) || isNull(tableId) || isEmpty(tableId)) {
		return null;
	}
	return TableManager.instance().getTable(tableId);
}


/**
 * Get the window a xml layout is for. 
 * @param windowLayout The xml data containing the layout for the window. 
 * @return The window object, null if no window with the given id exists, or no id is in windowLayout.
 */
function getWindow(windowLayout) {
	var windowId = getTagNodeValue("windowId", windowLayout);
	if (!isDefined(windowId) || isNull(windowId) || isEmpty(windowId)) {
		return null;
	}
	return WindowManager.instance().getWindow(windowId);
}


/**
 * Get the chart a xml layout is for. 
 * @param chartLayout The xml data containing the layout for the chart. 
 * @param window The window the chart is in. 
 * @return The chart object, null if chart is not defined or noe chart id is in chartLayout.
 */
function getChart(chartLayout, window) {
	var chartId = getTagNodeValue("chartId", chartLayout);
	if (!isDefined(chartId) || isNull(chartId) || isEmpty(chartId)) {
		return null;
	}
	return window.getChart(chartId);
}


/**
 * Returns true if a block of xml is a layout.
 * @param xml The xml to check.
 */
function isLayout(xml) {
	var windows = xml.getElementsByTagName("window");	
	if (windows.length == 0) {
		return false;
	}
	return true;
}


/**
 * Returns true if a window is the layout window.
 * @param window The window to check.
 */
function isProfileWindow(window) {
	if (window.getId().indexOf("profile") > -1) {
		return true;
	}
	return false;
}


/**
 * Returns true if a window is a setup window, false otherwise.
 */
function isSetupWindow(window) {
	if (!exists(top.setupWindowLoadingFunction)) {
		return false;
	}
	for (var setupWindowId in top.setupWindowLoadingFunction) {
		if (setupWindowId == window.id) {
			return true;
		}
	}
	return false;
}


/**
 * Fires a window layou tset event. 
 * @param window The window the layout has been set for. 
 */
function fireWindowLayoutSetEvent(window) {
	getEventServer().notifyListeners(new Event(WINDOW_LAYOUT_SET, window.getId()));
}


/**
 * Fires a starting to set window layout event. 
 * @param windowLayout The window layout to be set for the window.
 */
function fireSetWindowLayoutEvent(windowLayout) {
	var windowId = getTagNodeValue("windowId", windowLayout);
	if (!isDefined(windowId) || isNull(windowId) || isEmpty(windowId)) {
		return null;
	}
	getEventServer().notifyListeners(new Event(START_SET_WINDOW_LAYOUT, windowId));
}


/**
 * Fires a start setting layout event. 
 */
function fireStartSettingLayoutEvent() {
	getEventServer().notifyListeners(new Event(START_SET_LAYOUT_EVENT, null));
}


/**
 * Fires a layout set event.
 */
function fireLayoutSetEvent() {
	getEventServer().notifyListeners(new Event(LAYOUT_SET_EVENT, null));
}


/**
 * Get the detached charts from a window xml layout.
 * @param windowLayout The window layout to get the detached charts from.
 * @return A array containing the detached charts layouts, a empty array if 
 * the window has no detached charts.
 */
function getDetachedCharts(windowLayout) {
	var charts = getCharts(windowLayout);
	var detachedCharts = new Array();
	for (var i = 0; i < charts.length; i++) {
		if (isDetachedChart(charts[i])) {
			detachedCharts[detachedCharts.length] = charts[i];
		}	
	}
	return detachedCharts;
}


/**
 * Get the charts from a window xml layout.
 * @param windowLayout The window layout to get the charts from.
 * @return A array containing the charts layouts, a empty array if the
 * window has no charts. 
 */
function getCharts(windowLayout) {
	return exists(windowLayout) ? windowLayout.getElementsByTagName("chart") : new Array();
}


/**
 * Parses a given layout in a array of windows.
 * @param layout The layout to parse.
 * @return a array of windows tags.
 */
function parseLayout(layout) {
	return layout.getElementsByTagName("window");	
}

