/**
 * The url used to delete protocol entries.
 */
MyHostsTable.DELETE_HOST_ENTRY_URL = MyTable.BASE_DELETE_ENTRY_URL + "command=myhosts&";


/**
 * The url used to add protocol entries.
 */
MyHostsTable.ADD_HOST_ENTRY_URL = MyTable.BASE_ADD_ENTRY_URL + "command=myhosts&";


/**
 * Factory method to create new MyHostsTable objects.
 * @param id The id of the table to update.
 * @param url The url to load the data from.
 * @param parentElement The element the table is to be inserted in. 
 * @param windowId The id of the window the table is in. 
 * @param Optional time parameter. The time between each update.
 * @returns A new probeTable object.
 * If this is not specified the default value is used.
 * @throws Error if id is undefined, null or empty.
 * @throws Error if url is undefined, null or empty.
 * @throws Error if parentElement is undefined, or null.
 * @throws Error if windowId is undefined, null or empty.
 */
function createMyHostsTable(id, url, parentElement, windowId, time) {
	assertNotEmpty(id, "Id for the table is undefined, empty or null: " + id);
	assertNotEmpty(url, "Url to set for the table is undefined empty or null: " + url);
	assertDefined(parentElement, "The element to insert the table into is undefined or null");
	assertNotEmpty(windowId, "windowId to set for the table is undefined empty or null: " + windowId);

	var subInstance = new MyHostsTable();
	var superInstance = createMyTable(id, url, parentElement, windowId, time);
	var instance = createSubClass(superInstance, subInstance);
	
	TableManager.instance().removeTable(TableManager.instance().getTable(id));
	TableManager.instance().registerNewTable(instance);

	// let the event processor call event handler functions as a method
	// of the mytable object instance
	superInstance.getEventProcessor().changeObject(instance);
	
	// overwrite the repeater to call the load function on the MyTable object	
	instance.repeater = new Repeater(load, instance, time);

	return instance;
}


/**
 * Subclasses table.js to add probe specifc behaviour.
 */
function MyHostsTable() {
}


// services


/**
 * Adds a host to the myhosts table. This is called when the right click choice
 * Add to my hosts is clicked. 
 */ 
function addToMyHosts() {
	if (lastEntryFieldName != null && lastTableObject != null && lastTdNode != null) {
		var row = lastTableObject.getRowFromTd(lastTdNode);
		var address = getFieldFromRow("address", row).getValue();
		WindowManager.instance().activateWindow("myhostswindow");
		var myHostsTable = TableManager.instance().getTable("myHostsTable");
		if (myHostsTable.isPainted() == false) {
			myHostsTable.paint();
		}
		
		var addressField = getInput(myHostsTable.getAddEntryContainerCode(), "address");
		addressField.value = address;
		myHostsTable.addEntry();
	}
}


/**
 * Adds a given host to the myhosts table.
 * @param host The ip address of the host to add (10.0.2.3).
 * @throws Error if hostName is undefined, null or empty.  
 */
MyHostsTable.prototype.addHost = function(host) {
	assertNotEmpty(host, "The host to add is undefined, null or empty");
	var addressField = getInput(this.getAddEntryContainerCode(), "address");
	addressField.value = host;
	this.addEntry();
}


/**
 * Checks for valid input before a entry is added. 
 * @returns True if the input is valid, false otherwise. 
 */
MyHostsTable.prototype.validInput = function() {
	var address = getElementWithAttribute(this.getAddEntryTableCode(), "address", "name", "input").value;
	if (address == "[Enter IP]") {
		this.setMessage(undefined, "Please enter an IP address");
		return false;
	}
	if (isEmpty(address)) {
		this.setMessage(undefined, "Invalid ip address: " + address);
		return false;
	}
	if (isValidIpAddress(address)) {
		return true;
	}
	
	this.setMessage(undefined, "Invalid ip address: " + address);
	return false;
}


/**
 * Checks if a entry to add already exists. This is to be overridden by subclases.
 * @returns True if the entry already exists false otherwise. 
 */
MyHostsTable.prototype.exists = function() {
	var address = getElementWithAttribute(this.getAddEntryTableCode(), "address", "name", "input").value;
	if (this.getRowIndexFieldValue(this.getIdFields()[0],address) > -1) {
		this.setMessage(undefined, "The host already exists");
		return true;
	}
	return false;
}



/**
 * A entry has been added. Reset the add entry fields.
 */ 
MyHostsTable.prototype.entryAdded = function() {
	this.clearCommentField();
}


/**
 * Returns the url to add a protocol entry.
 */
MyHostsTable.prototype.createAddEntryUrl = function() {
	var hostField = getElementWithAttribute(this.getAddEntryTableCode(), "address", "name", "input");
	var host = hostField.value;
	var commentField = getElementWithAttribute(this.getAddEntryTableCode(), "comment", "name", "input");
	var comment = ""
	if (isDefined(commentField) && !isNull(commentField) && commentField.value != "[Enter a comment]") {
		comment = commentField.value;
	}
	
	return MyHostsTable.ADD_HOST_ENTRY_URL + "address=" + host + "&comment=" + encodeURIComponent(comment) + "&";
}


/**
 * Creates the url needed to delete a protocol entry.
 * @param value The value for the id field of the row to delete. 
 * @return The url to delete the entry.
 */
MyHostsTable.prototype.createDeleteEntryUrl = function(value) {
	var address = getParameterValue(value, this.getIdFields()[0], "&");
	return MyHostsTable.DELETE_HOST_ENTRY_URL + "&address=" + address + "&";
}


/**
 * Creates the url to change the comment.
 * @param value The value for the id field of the entry to change the comment for.
 */ 
MyHostsTable.prototype.createChangeCommentUrl = function(value) {
	var row = this.getTableCode().rows[this.getIdRowNumber(value)];
	var addressField = getHtmlFieldFromRow(this.getIdFields()[0], row);
	var commentCell = getHtmlFieldFromRow("comment", row);
	var address = getTekstValue(addressField);
	var comment = getElementWithAttribute(commentCell, "comment", "name", "input").value;
	
	return MyHostsTable.ADD_HOST_ENTRY_URL + "address=" + address + "&comment=" + comment + "&";
}


/**
 * Paint the add entry row.
 * @param row The row to paint into.
 */
MyHostsTable.prototype.paintAddEntryRow = function(row) {
	var cellCode;
	var thisObj = this;
	for (var i = (this.columnOrder.length -1); i >= 0; i--) {
		fieldName = this.columnOrder[i].getName();
		if (!this.isVisibleColumn(fieldName)) {
			continue;
		}

		if ("address".equalsIgnoreCase(fieldName)) {
			cellCode = row.insertCell(0);
			cellCode.setAttribute("name", fieldName);
			var addressField = this.createAddEntryInputfield("address", MyTable.INPUT_FIELD_SIZE, "[Enter IP]", MyTable.IP_INPUT_FIELDS_MAX_LENGTH);
			addressField.onfocus = function() {
				addressField.value = "";
				tt_Hide();
			}
			addressField.onmouseover = function() {
				if (thisObj.rightClickMenuShown == false) {
					Tip(helptext('myprotocolsperhostaddress'), BGCOLOR, '#ffffe1', ABOVE, false, DELAY, 500);
				}
			}
			cellCode.appendChild(addressField);
		}
		else if ("comment".equalsIgnoreCase(fieldName)) {
			cellCode = row.insertCell(0);
			cellCode.setAttribute("name", fieldName);
			var commentField = this.createAddEntryInputfield("comment", MyTable.INPUT_FIELD_SIZE, "[Enter a comment]");
			commentField.onfocus = function() {
				commentField.value = "";
				tt_Hide();
			}
			commentField.onmouseover = function() {
				if (thisObj.rightClickMenuShown == false) {
					Tip(helptext('myprotocolscomment'), BGCOLOR, '#ffffe1', ABOVE, false, DELAY, 500);
				}
			}
			cellCode.appendChild(commentField);
		}
		else {
			cellCode = row.insertCell(0);
			cellCode.setAttribute("name", fieldName);
		}
		this.setCellWidth(cellCode, fieldName);
		cellCode = null;	
	}
	
	var addButtonCell = row.insertCell(row.cells.length);
	addButtonCell.appendChild(this.createAddEntryLink());
}