/**
 * The url used to delete protocol entries.
 */
MyProtocolsPerHostTable.DELETE_PROTOCOL_PER_HOST_ENTRY_URL = MyTable.BASE_DELETE_ENTRY_URL + "command=myprotocolsperhost&";


/**
 * The url used to add protocol entries.
 */
MyProtocolsPerHostTable.ADD_PROTOCOL_PER_HOST_ENTRY_URL = MyTable.BASE_ADD_ENTRY_URL + "command=myprotocolsperhost&";


/**
 * Factory method to create new MyProtocolsTable objects.
 * @param id The id of the table to update.
 * @param url The url to load the data from.
 * @param parentElement The element the table is to be inserted in. 
 * @param windowId The id of the window the table is in. 
 * @param Optional time parameter. The time between each update.
 * @returns A new probeTable object.
 * If this is not specified the default value is used.
 * @throws Error if id is undefined, null or empty.
 * @throws Error if url is undefined, null or empty.
 * @throws Error if parentElement is undefined, or null.
 * @throws Error if windowId is undefined, null or empty.
 */
function createMyProtocolsPerHostTable(id, url, parentElement, windowId, time) {
	assertNotEmpty(id, "Id for the table is undefined, empty or null: " + id);
	assertNotEmpty(url, "Url to set for the table is undefined empty or null: " + url);
	assertDefined(parentElement, "The element to insert the table into is undefined or null");
	assertNotEmpty(windowId, "windowId to set for the table is undefined empty or null: " + windowId);

	var subInstance = new MyProtocolsPerHostTable();
	var superInstance = createMyTable(id, url, parentElement, windowId, time);
	var instance = createSubClass(superInstance, subInstance);
	
	TableManager.instance().removeTable(TableManager.instance().getTable(id));
	TableManager.instance().registerNewTable(instance);

	// let the event processor call event handler functions as a method
	// of the mytable object instance
	superInstance.getEventProcessor().changeObject(instance);
	
	// overwrite the repeater to call the load function on the MyTable object	
	instance.repeater = new Repeater(load, instance, time);

	return instance;
}


/**
 * Subclasses table.js to add probe specifc behaviour.
 */
function MyProtocolsPerHostTable() {
}


/**
 * Checks for valid input before a entry is added. 
 * @returns True if the input is valid, false otherwise. 
 */
MyProtocolsPerHostTable.prototype.validInput = function() {
	var port = getElementWithAttribute(this.getAddEntryTableCode(), "port", "name", "input").value;
	var address = getElementWithAttribute(this.getAddEntryTableCode(), "address", "name", "input").value;
	if (address == "[Enter IP]") {
		this.setMessage(undefined, "Please enter an IP address");
		return false;
	}
	if (isEmpty(address) || !isValidIpAddress(address)) {
		this.setMessage(undefined, "Invalid ip address: " + address);
		return false;
	}
	if (!isValidPortIdString(port)) {
		this.setMessage(undefined, "Invalid port: " + port);
		return false;
	}
	
	return true;
}


/**
 * Checks if a entry to add already exists. This is to be overridden by subclases.
 * @returns True if the entry already exists false otherwise. 
 */
MyProtocolsPerHostTable.prototype.exists = function() {
	var values = new Array();
	values[0] = getElementWithAttribute(this.getAddEntryTableCode(), "address", "name", "input").value;
	values[1] = getElementWithAttribute(this.getAddEntryTableCode(), "port", "name", "input").value;
	
	if (this.getRowIndexFieldValues(this.getIdFields(), values) > -1) {
		this.setMessage(undefined, "The entry already exists");
		return true;
	}
	return false;
}



/**
 * A entry has been added. Reset the add entry fields.
 */ 
MyProtocolsPerHostTable.prototype.entryAdded = function() {
	var protocolSelect = getElementWithAttribute(this.getAddEntryTableCode(), "protocolSelect", "name", "select");
	this.setProtocolPrefix(protocolSelect.value);
	this.clearCommentField();
}


/**
 * Returns the url to add a protocol entry.
 */
MyProtocolsPerHostTable.prototype.createAddEntryUrl = function() {
	var protocolSelect = getElementWithAttribute(this.getAddEntryTableCode(), "protocolSelect", "name", "select");
	var protocol = protocolSelect.value;
	var portField = getElementWithAttribute(this.getAddEntryTableCode(), "port", "name", "input");
	var port = portField.value;
	var commentField = getElementWithAttribute(this.getAddEntryTableCode(), "comment", "name", "input");
	var comment = "";
	if (isDefined(commentField) && !isNull(commentField) && commentField.value != "[Enter a comment]") {
		comment = commentField.value;
	}
	var hostField = getElementWithAttribute(this.getAddEntryTableCode(), "address", "name", "input");
	var host = hostField.value;

	// check if the protocol to add contains the prefix for the choosen protocol
	// add the prefix if not	
	if (port.indexOf(PROTOCOL_PREFIXES[protocol]) == -1) {
		port = PROTOCOL_PREFIXES[protocol] + port;
	}

	return MyProtocolsPerHostTable.ADD_PROTOCOL_PER_HOST_ENTRY_URL + "port=" + port + "&address=" + host + "&comment=" + encodeURIComponent(comment) + "&";
}


/**
 * Creates the url needed to delete a protocol entry.
 * @param value The value for the id field of the row to delete. 
 * @return The url to delete the entry.
 */
MyProtocolsPerHostTable.prototype.createDeleteEntryUrl = function(value) {
	// get the port from the request
	var port = getParameterValue(value, "protocolId", "&");
	var address = getParameterValue(value, "address", "&");
	return MyProtocolsPerHostTable.DELETE_PROTOCOL_PER_HOST_ENTRY_URL + "&port=" + port + "&address=" + address + "&";
}


/**
 * Creates the url to change the comment.
 * @param value The value for the id field of the entry to change the comment for.
 */ 
MyProtocolsPerHostTable.prototype.createChangeCommentUrl = function(value) {
	var row = this.getTableCode().rows[this.getIdRowNumber(value)];
	var portField = getHtmlFieldFromRow("protocolId", row);
	var commentCell = getHtmlFieldFromRow("comment", row);
	var port = getTekstValue(portField);
	var comment = getElementWithAttribute(commentCell, "comment", "name", "input").value;
	var hostField = getHtmlFieldFromRow("address", row);
	var host = getTekstValue(hostField);

	return MyProtocolsPerHostTable.ADD_PROTOCOL_PER_HOST_ENTRY_URL + "port=" + port + "&address=" + host + "&comment=" + comment + "&";
}


/**
 * Paint the add entry row.
 * @param row The row to paint into.
 */
MyProtocolsPerHostTable.prototype.paintAddEntryRow = function(row) {
	var cellCode;
	var thisObj = this;
	for (var i = (this.columnOrder.length -1); i >= 0; i--) {
		fieldName = this.columnOrder[i].getName();
		if (!this.isVisibleColumn(fieldName)) {
			continue;
		}
		
		if ("address".equalsIgnoreCase(fieldName)) {
			cellCode = row.insertCell(0);
			cellCode.setAttribute("name", fieldName);
			var addressField = this.createAddEntryInputfield("address", MyTable.INPUT_FIELD_SIZE, "[Enter IP]", MyTable.IP_INPUT_FIELDS_MAX_LENGTH);
			addressField.onfocus = function() {
				addressField.value = "";
				tt_Hide();
			}
			addressField.onmouseover = function() {
				if (thisObj.rightClickMenuShown == false) {
					Tip(helptext('myprotocolsperhostaddress'), BGCOLOR, '#ffffe1', ABOVE, false, DELAY, 500);
				}
			}
			cellCode.appendChild(addressField);
		}
		else if ("protocolName".equalsIgnoreCase(fieldName)) {
			cellCode = row.insertCell(0);
			cellCode.setAttribute("name", fieldName);
			
			var protocolSelect = document.createElement("select");
			protocolSelect.style.width = "98%";
			protocolSelect.setAttribute("name", "protocolSelect");
			protocolSelect.onmouseover = function() {
				if (thisObj.rightClickMenuShown == false) {
					Tip(helptext('myprotocolselect'), BGCOLOR, '#ffffe1', ABOVE, false, DELAY, 500);
				}
			}
			var thisObj = this;
			protocolSelect.onchange = function() {
				thisObj.setProtocolPrefix(this.options[this.selectedIndex].value);
			}
			cellCode.appendChild(protocolSelect);
			if (isIE()) {
				// the select is somehow added by itself the windows select tags
				// but in the wrong order, so it would use the wrong replaceField
				// reset the replace fields so new replace fields for all selects are created
				this.window.replaceFields = new Array();
			}
			
			var ethernetOption = document.createElement("option");
			ethernetOption.setAttribute("value", "ethernet");
			protocolSelect.appendChild(ethernetOption);
			ethernetOption.appendChild(document.createTextNode("Ethernet"));
			
			var ipOption = document.createElement("option");
			ipOption.setAttribute("value", "ip");
			protocolSelect.appendChild(ipOption);
			ipOption.appendChild(document.createTextNode("IP"));
			
			var tcpOption = document.createElement("option");
			tcpOption.setAttribute("value", "tcp");
			protocolSelect.appendChild(tcpOption);
			tcpOption.appendChild(document.createTextNode("TCP"));
			tcpOption.selected = true;
			
			var udpOption = document.createElement("option");
			udpOption.setAttribute("value", "udp");
			protocolSelect.appendChild(udpOption);
			udpOption.appendChild(document.createTextNode("UDP"));
		}
		else if ("protocolId".equalsIgnoreCase(fieldName)) {
			cellCode = row.insertCell(0);
			cellCode.setAttribute("name", fieldName);
			var portInputField = this.createAddEntryInputfield("port", MyTable.INPUT_FIELD_SIZE, PROTOCOL_PREFIXES["tcp"], MyTable.PORT_INPUT_FIELDS_MAX_LENGTH);
			portInputField.onmouseover = function() {
				if (thisObj.rightClickMenuShown == false) {
					Tip(helptext('myprotocolsport'), BGCOLOR, '#ffffe1', ABOVE, false, DELAY, 500);
				}
			}
			portInputField.onfocus = tt_Hide;
			cellCode.appendChild(portInputField);
		}	
		else if ("comment".equalsIgnoreCase(fieldName)) {
			cellCode = row.insertCell(0);
			cellCode.setAttribute("name", fieldName);
			var commentField = this.createAddEntryInputfield("comment", MyTable.INPUT_FIELD_SIZE, "[Enter a comment]");
			commentField.onfocus = function() {
				commentField.value = "";
				tt_Hide();
			}
			commentField.onmouseover = function() {
				if (thisObj.rightClickMenuShown == false) {
					Tip(helptext('myprotocolscomment'), BGCOLOR, '#ffffe1', ABOVE, false, DELAY, 500);
				}
			}
			cellCode.appendChild(commentField);
		}
		else {
			cellCode = row.insertCell(0);
			cellCode.setAttribute("name", fieldName);
		}
		this.setCellWidth(cellCode, fieldName);
		cellCode = null;	
	}
	
	var addButtonCell = row.insertCell(row.cells.length);
	addButtonCell.appendChild(this.createAddEntryLink());
}


/**
 * Set the prefix of a protocol in the protocol input field. 
 * @param protocol The protocol to set the prefix for.  
 */
MyProtocolsPerHostTable.prototype.setProtocolPrefix = function(protocol) {
	var portField = getElementWithAttribute(this.getAddEntryTableCode(), "port", "name", "input");
	if (!isEmpty(PROTOCOL_PREFIXES[protocol])){
		portField.value = PROTOCOL_PREFIXES[protocol];
	}
}