/**
 * The url used to delete protocol entries.
 */
MyProtocolsTable.DELETE_PROTOCOL_ENTRY_URL = MyTable.BASE_DELETE_ENTRY_URL + "command=myprotocols&";


/**
 * The url used to add protocol entries.
 */
MyProtocolsTable.ADD_PROTOCOL_ENTRY_URL = MyTable.BASE_ADD_ENTRY_URL + "command=myprotocols&";


/**
 * Factory method to create new MyProtocolsTable objects.
 * @param id The id of the table to update.
 * @param url The url to load the data from.
 * @param parentElement The element the table is to be inserted in. 
 * @param windowId The id of the window the table is in. 
 * @param Optional time parameter. The time between each update.
 * If this is not specified the default value is used.
 * @returns A new probeTable object.
 * @throws Error if id is undefined, null or empty.
 * @throws Error if url is undefined, null or empty.
 * @throws Error if parentElement is undefined, or null.
 * @throws Error if windowId is undefined, null or empty.
 */
function createMyProtocolsTable(id, url, parentElement, windowId, time) {
	assertNotEmpty(id, "Id for the table is undefined, empty or null: " + id);
	assertNotEmpty(url, "Url to set for the table is undefined empty or null: " + url);
	assertDefined(parentElement, "The element to insert the table into is undefined or null");
	assertNotEmpty(windowId, "windowId to set for the table is undefined empty or null: " + windowId);

	var subInstance = new MyProtocolsTable();
	var superInstance = createMyTable(id, url, parentElement, windowId, time);
	var instance = createSubClass(superInstance, subInstance);
	
	TableManager.instance().removeTable(TableManager.instance().getTable(id));
	TableManager.instance().registerNewTable(instance);

	// let the event processor call event handler functions as a method
	// of the mytable object instance
	superInstance.getEventProcessor().changeObject(instance);
	
	// overwrite the repeater to call the load function on the MyTable object	
	instance.repeater = new Repeater(load, instance, time);

	return instance;
}


/**
 * Subclasses table.js to add probe specifc behaviour.
 */
function MyProtocolsTable() {
}


// services


/**
 * Adds a protocol to the myprotocol table. This is called when the right click choice
 * Add to my protocol is clicked. 
 */ 
function addToMyProtocols() {
	if (lastEntryFieldName != null && lastTableObject != null && lastTdNode != null) {
		var row = lastTableObject.getRowFromTd(lastTdNode);
		var protocolId = getFieldFromRow("protocolId", row).getValue();
		WindowManager.instance().activateWindow("myprotocolswindow");
		var myProtocolTable = TableManager.instance().getTable("myProtocolsTable");
		if (myProtocolTable.isPainted() == false) {
			myProtocolTable.paint();
		}
		var protocolIdField = getInput(myProtocolTable.getAddEntryContainerCode(), "port");
		protocolIdField.value = protocolId;
		myProtocolTable.setProtocolSelect(protocolId);
		myProtocolTable.addEntry();
	}
}


/**
 * Adds a given protocol to the myprotocols table. 
 * @param protocolId The id (1.2048.6.80) of the protocol to add. 
 * @throws Error if protocolId is undefined, null or empty.
 */
MyProtocolsTable.prototype.addProtocol = function(protocolId) {
	assertNotEmpty(protocolId, "Protocol id to add is undefined, null or empty");
	getInput(this.getAddEntryContainerCode(), "port").value = protocolId;
	this.setProtocolSelect(protocolId);
	this.addEntry();
}


/**
 * Sets the correct value for the protocol select based on a protocol id string.
 * For protocol id 1.2048.6.80 for example TCP is selected in the protocol select.
 * @param protocolId The protocol id to select the protocol for. 
 * @throws Error if protocol id is undefined, null or empty.
 */
MyProtocolsTable.prototype.setProtocolSelect = function(protocolId) {
	assertNotEmpty(protocolId, "Protocol id is undefined, null or empty");
	
	var protocolSelect = getElementWithAttribute(this.getAddEntryContainerCode(), "protocolSelect", "name", "select");
	if (protocolId.indexOf(PROTOCOL_PREFIXES["udp"]) > -1) {
		protocolSelect.selectedIndex = 3;
	}
	else if (protocolId.indexOf(PROTOCOL_PREFIXES["tcp"]) > -1) {
		protocolSelect.selectedIndex = 2;
	}
	else if (protocolId.indexOf(PROTOCOL_PREFIXES["ip"]) > -1) {
		protocolSelect.selectedIndex = 1;
	}
	else if (protocolId.indexOf(PROTOCOL_PREFIXES["ethernet"]) > -1) {
		protocolSelect.selectedIndex = 0;
	}
}


/**
 * Checks for valid input before a entry is added. 
 * @returns True if the input is valid, false otherwise. 
 */
MyProtocolsTable.prototype.validInput = function() {
	var port = getElementWithAttribute(this.getAddEntryTableCode(), "port", "name", "input").value;
	if (isValidPortIdString(port)) {
		return true;
	}
	
	this.setMessage(undefined, "Invalid port: " + port);
	return false;
}


/**
 * Checks if a entry to add already exists.
 * @returns True if the entry already exists false otherwise. 
 */
MyProtocolsTable.prototype.exists = function() {
	var port = getElementWithAttribute(this.getAddEntryTableCode(), "port", "name", "input").value;
	if (this.getRowIndexFieldValue(this.getIdFields()[0], port) > -1) {
		this.setMessage(undefined, "The port already exists");
		return true;
	}
	return false;
}



/**
 * A entry has been added. Reset the add entry fields.
 */ 
MyProtocolsTable.prototype.entryAdded = function() {
	var protocolSelect = getElementWithAttribute(this.getAddEntryTableCode(), "protocolSelect", "name", "select");
	this.setProtocolPrefix(protocolSelect.value);
	this.clearCommentField();
}


/**
 * Returns the url to add a protocol entry.
 */
MyProtocolsTable.prototype.createAddEntryUrl = function() {
	var protocolSelect = getElementWithAttribute(this.getAddEntryTableCode(), "protocolSelect", "name", "select");
	var protocol = protocolSelect.value;
	var portField = getElementWithAttribute(this.getAddEntryTableCode(), "port", "name", "input");
	var port = portField.value;
	var commentField = getElementWithAttribute(this.getAddEntryTableCode(), "comment", "name", "input");
	var comment = "";
	if (isDefined(commentField) && !isNull(commentField) && commentField.value != "[Enter a comment]") {
		comment = commentField.value;
	}

	// check if the protocol to add contains the prefix for the choosen protocol
	// add the prefix if not	
	if (port.indexOf(PROTOCOL_PREFIXES[protocol]) == -1) {
		port = PROTOCOL_PREFIXES[protocol] + port;
	}
	
	return MyProtocolsTable.ADD_PROTOCOL_ENTRY_URL + "port=" + port + "&comment=" + encodeURIComponent(comment) + "&";
}


/**
 * Creates the url needed to delete a protocol entry.
 * @param value The value for the id field of the row to delete. 
 * @return The url to delete the entry.
 */
MyProtocolsTable.prototype.createDeleteEntryUrl = function(value) {
	// get the port from the request
	var port = getParameterValue(value, this.getIdFields()[0], "&");
	return MyProtocolsTable.DELETE_PROTOCOL_ENTRY_URL + "&port=" + port + "&";
}


/**
 * Creates the url to change the comment.
 * @param value The value for the id field of the entry to change the comment for.
 */ 
MyProtocolsTable.prototype.createChangeCommentUrl = function(value) {
	var row = this.getTableCode().rows[this.getIdRowNumber(value)];
	var portField = getHtmlFieldFromRow(this.getIdFields()[0], row);
	var commentCell = getHtmlFieldFromRow("comment", row);
	var port = getTekstValue(portField);
	var comment = getElementWithAttribute(commentCell, "comment", "name", "input").value;
	
	return MyProtocolsTable.ADD_PROTOCOL_ENTRY_URL + "port=" + port + "&comment=" + comment + "&";
}


/**
 * Paint the add entry row.
 * @param row The row to paint into.
 */
MyProtocolsTable.prototype.paintAddEntryRow = function(row) {
	var cellCode;
	var thisObj = this;
	for (var i = (this.columnOrder.length -1); i >= 0; i--) {
		fieldName = this.columnOrder[i].getName();
		if (!this.isVisibleColumn(fieldName)) {
			continue;
		}

		if ("protocolName".equalsIgnoreCase(fieldName)) {
			cellCode = row.insertCell(0);
			cellCode.setAttribute("name", fieldName);
			
			var protocolSelect = document.createElement("select");
			protocolSelect.style.width = "98%";
			protocolSelect.setAttribute("name", "protocolSelect");
			protocolSelect.onmouseover = function() {
				if (thisObj.rightClickMenuShown == false) {
					Tip(helptext('myprotocolselect'), BGCOLOR, '#ffffe1', ABOVE, false, DELAY, 500);
				 }
			}
			var thisObj = this;
			protocolSelect.onchange = function() {
				thisObj.setProtocolPrefix(this.options[this.selectedIndex].value);
			}
			cellCode.appendChild(protocolSelect);
			if (isIE() && this.window.selectsHidden()) {
				// the select is somehow added by itself the windows select tags
				// but in the wrong order, so it would use the wrong replaceField
				// reset the replace fields so new replace fields for all selects are created
				// when the selects already are hidden though we can not reset the replacefields
				// because else both the selects and the replacefields would be visible when 
				// the window is activated, this should only happen in the tests though
				this.window.replaceFields = new Array();
			}
			
			var ethernetOption = document.createElement("option");
			ethernetOption.setAttribute("value", "ethernet");
			protocolSelect.appendChild(ethernetOption);
			ethernetOption.appendChild(document.createTextNode("Ethernet"));
			
			var ipOption = document.createElement("option");
			ipOption.setAttribute("value", "ip");
			protocolSelect.appendChild(ipOption);
			ipOption.appendChild(document.createTextNode("IP"));
			
			var tcpOption = document.createElement("option");
			tcpOption.setAttribute("value", "tcp");
			protocolSelect.appendChild(tcpOption);
			tcpOption.appendChild(document.createTextNode("TCP"));
			tcpOption.selected = true;
			
			var udpOption = document.createElement("option");
			udpOption.setAttribute("value", "udp");
			protocolSelect.appendChild(udpOption);
			udpOption.appendChild(document.createTextNode("UDP"));
		}
		else if ("protocolId".equalsIgnoreCase(fieldName)) {
			cellCode = row.insertCell(0);
			cellCode.setAttribute("name", fieldName);
			var protocolIdField = this.createAddEntryInputfield("port", MyTable.INPUT_FIELD_SIZE, PROTOCOL_PREFIXES["tcp"], MyTable.PORT_INPUT_FIELDS_MAX_LENGTH); 
			protocolIdField.onmouseover = function() {
				if (thisObj.rightClickMenuShown == false) {
					Tip(helptext('myprotocolsport'), BGCOLOR, '#ffffe1', ABOVE, false, DELAY, 500);
				}
			}
			cellCode.appendChild(protocolIdField);
			protocolIdField.onfocus = tt_Hide;
		}
		else if ("comment".equalsIgnoreCase(fieldName)) {
			cellCode = row.insertCell(0);
			cellCode.setAttribute("name", fieldName);
			var commentField = this.createAddEntryInputfield("comment", MyTable.INPUT_FIELD_SIZE, "[Enter a comment]");
			commentField.onfocus = function() {
				commentField.value = "";
				tt_Hide();
			}
			commentField.onmouseover = function() {
				if (thisObj.rightClickMenuShown == false) {
					Tip(helptext('myprotocolscomment'), BGCOLOR, '#ffffe1', ABOVE, false, DELAY, 500);
				}
			}
			cellCode.appendChild(commentField);
		}
		else {
			cellCode = row.insertCell(0);
			cellCode.setAttribute("name", fieldName);
		}
		this.setCellWidth(cellCode, fieldName);
		cellCode = null;	
	}
	
	var addButtonCell = row.insertCell(row.cells.length);
	addButtonCell.appendChild(this.createAddEntryLink());
}


/**
 * Set the prefix of a protocol in the protocol input field. 
 * @param protocol The protocol to set the prefix for.  
 */
MyProtocolsTable.prototype.setProtocolPrefix = function(protocol) {
	var portField = getElementWithAttribute(this.getAddEntryTableCode(), "port", "name", "input");
	if (!isEmpty(PROTOCOL_PREFIXES[protocol])){
		portField.value = PROTOCOL_PREFIXES[protocol];
	}
}