	
/**
 * The number of failed updates before the updating is stopped. 
 */
ProbeTable.MAX_FAILED_UPDATE = 5;


/**
 * Factory method to create new ProbeTable objects.
 * @param id The id of the table to update.
 * @param url The url to load the data from.
 * @param parentElement The element the table is to be inserted in. 
 * @param windowId The id of the window the table is in. 
 * @param Optional time parameter. The time between each update.
 * @returns A new probeTable object.
 * If this is not specified the default value is used.
 * @throws Error if id is undefined, null or empty.
 * @throws Error if url is undefined or null.
 * @throws Error if parentElement is undefined, or null.
 * @throws Error if windowId is undefined, null or empty.
 */
function createProbeTable(id, url, parentElement, windowId, time) {
	assertNotEmpty(id, "Id for the table is undefined, empty or null: " + id);
	assertExists(url, "Url to set for the table is undefined or empty: " + url);
	assertDefined(parentElement, "The element to insert the table into is undefined or null");
	assertNotEmpty(windowId, "windowId to set for the table is undefined empty or null: " + windowId);
	
	var subInstance = new ProbeTable(windowId, time);
	var superInstance = new Table(id, url, parentElement, time);
	var instance = createSubClass(superInstance, subInstance);
	
	TableManager.instance().removeTable(TableManager.instance().getTable(id));
	TableManager.instance().registerNewTable(instance);
	
	return instance;
}


/**
 * Subclasses table.js to add probe specifc behaviour.
 * @param id The id of the table to update.
 * @param url The url to load the data from.
 * @param windowName The name of the window the table is in.
 * @param Optional time parameter. The time between each update.
 * If this is not specified the default value is used.
 */
function ProbeTable(windowName, time) {
	// set the properties of this class
	this.windowName = windowName;
	this.window = WindowManager.instance().getWindow(windowName);
	if (isDefined(this.initEventProcessor)) {
		this.initEventProcessor();	
	}

	// check if time is specified
	if (isDefined(time) && !isNull(time)) {
		this.setTime(time);
	}
	// use default time value
	else {
		this.time = Table.DEFAULT_TIME;
	}
	
	// if this is set to true the table is currently loading data
	this.isLoading = false;
		
	// this counter is used to count the failed updates, when this counter
	// has reached MAX_FAILED_UPDATE the updating of the table is stopped
	this.failedUpdateCounter = 0;
	
	// when the summary tab is deactivated the updating of tables is stopped until
	// the summary tab is activated again. If this is true the table was active
	// when the tab was deactivated and is to be activated again when the summary tab
	// is activated. 
	this.activeTabDeactivated = false;
	
	// When the topn chart for this window is detached the table is top updated even if hte window 
	// the table is in is closed
	this.topnChartDetached = false;
	
	// create the repeater object used for repeatedly calling the load method
	this.repeater = new Repeater(load, this, this.time);

	// entries shown in the table have this tagname in the xml documents loaded and parsed
	this.tagName = "entry";
	
	// Set to true if the right click menu for this table is shown, false if not. If the right click 
	// menu is shown the tooltip is not to be shown
	this.rightClickMenuShown = false;
	
	// set to true if the no access html code is shown
	this.noAccessShown = false;
}


// services


/**
 * Parses some xml data into a array of fields.
 * For each element in the xml data there is created a field in the table
 * The name of the field is the name of the xml data, and the fields value
 * is set based on the elements text value.
 * @param data The xml data to create the row from.
 * @param Optional The unique id for each row.
 * @return The array containing the fields.
 * @throws Error if data is null or undefined.
 */
ProbeTable.prototype.parseXmlRow = function (data) {
	assertDefined(data, "Data is not defined or null");
	if (!data.hasChildNodes()) {
		// the xml data has no entries
		return;
	}

	var row = new Array();
	var counter = 0;
	var index = -1;
	var len = data.childNodes.length;
	for (var i = 0; i < len; i++) {
		// get the index (id) if it exists
		if (data.childNodes[i].nodeName == "index") {
			index = data.childNodes[i].firstChild.nodeValue;
		}
		if (data.childNodes[i].nodeType != 1) {
			// the node is not a element node
			continue;
		}
		if (!isNull(data.childNodes[i].firstChild) && data.childNodes[i].firstChild.nodeType == 1) {
			// the child of the node is not a element node, also parse this elements
			// fields, this is for example used in triggered alarms xml, where the defined alarm
			// is a inner xml tag of the triggered alarm xml tag
			var newRow = this.parseXmlRow(data.childNodes[i]);
			arrayCopy(newRow, row);
			counter += newRow.length;
			continue;
		}
		
		var fieldValue = "";
		if (!isNull(data.childNodes[i].firstChild)) {
			fieldValue = data.childNodes[i].firstChild.nodeValue;
		}
		var nodeName = data.childNodes[i].nodeName;
		var field = this.createField(nodeName, fieldValue, index);
		
		// add the field
		row[counter] = field;
		counter++;
	}
	return row;
}


/**
 * Creates a field with given values.
 * @param name The name to set for the created field.
 * @param value The value to set for the field.
 * @param index The index to set for the field.
 * @return The created field.
 */
ProbeTable.prototype.createField = function(name, value, index) {
	if (index > -2) {
		return new Field(name, value, index);
	}
	else {
		return new Field(name, value);
	}
}


/**
 * Set the time between the updates.
 * @param time The time to set.
 */
ProbeTable.prototype.setTime = function(time) {
	this.time = parseInt(time);
	if (isNaN(this.time)) {
		this.time = Table.DEFAULT_TIME;
	}
	if (isDefined(this.repeater)) {
		this.repeater.setTime(this.time);
	}
}


/**
 * Return the window the table is in. 
 * @return The window set. 
 */
ProbeTable.prototype.getWindow = function() {
	return this.window;
}


// data formatting


/**
 * Set data into the table based on a xmlDocument. To 
 * show the data on screen call paint(). 
 * @aram xmlDoc The data in xml format.
 * @param tagName The name of the tag the data for each row is in.
 * @param throws Error if xmlDoc is undefined or null.
 * @param throws Error if tagName is undefined, null or empty.
 */
ProbeTable.prototype.setData = function (xmlDoc, tagName) {
	assertDefined(xmlDoc, "XmlDoc is undefined or null");
	assertNotEmpty(tagName, "TagName is undefined, null or empty");
	var rows = xmlDoc.getElementsByTagName(tagName);
	
	var len = rows.length;
	for (var i = 0; i < len; i++) {
		var row = this.parseXmlRow(rows[i]);
		this.rows[this.rowCounter] = row;
		this.checkProbeNameField(row);
		this.rowCounter++;
	}
	
	if (!this.sortable) {
		return;
	}
	
	// sort the table
	if (exists(this.columnToSort)) {
		this.sort(this.columnToSort);
	}
	if (this.reversed) {
		this.reverseColumn(this.columnToSort);
	}
}


/**
 * If the probe name field is empty use the src address insted. 
 */
ProbeTable.prototype.checkProbeNameField = function(row) {
	var probeNameField = getFieldFromRow("probeName", row);
	if (exists(probeNameField) && isEmpty(probeNameField.getValue())) {
		probeNameField.setValue(getFieldFromRow("src", row).getValue());
	}
}


/**
 * Inserts a field value into a given html node. 
 * This method is overridden to add KB, MB, and GB to byte type columns. 
 * @param field The field to insert the value for.
 * @param node The node to insert the value into.
 */
ProbeTable.prototype.insertFieldValue = function(field, node, cell) {
	if (field.getName() == "hostName" || field.getName() == "srcHostName" || field.getName() == "dstHostName") {
		node.appendChild(document.createTextNode(URLDecode(field.getValue())));
	}
	else if (field.getName().toLowerCase().indexOf("octet") > -1 && !this.isHeader(field)) {
		var formattedValue = formatOctetValue(parseInt(field.getValue()));
		node.appendChild(document.createTextNode(formattedValue));
	}
	else if ("alarmCount".equalsIgnoreCase(field.getName()) && !this.isHeader(field)) {
		cell.onmouseover = showTableTooltip;
		this.insertAlarmCount(field, node, cell);
	}
	else if ("alarmCount".equalsIgnoreCase(field.getName()) && this.isHeader(field)) {
		var image = document.createElement("img");
		image.setAttribute("name", "alarmCountImage");
		image.style.height = "12px"; 
		image.style.width = "12px";
		image.src = "/themes/black/images/alarm.jpg";
		image.onmouseover = showTableTooltip;
		node.appendChild(image);
	}
	else if ("status".equalsIgnoreCase(field.getName()) && !this.isHeader(field)) {
		this.insertSlaveStatusText(field, node, cell);
	}
	else {
		node.appendChild(document.createTextNode(field.getValue()));
	}
}


/**
 * Inserts the slave status text for a given field. 
 * @param field The field to insert the status text for.
 * @param node The node to insert the value into.
 * @param cell The table cell node is in.
 */
ProbeTable.prototype.insertSlaveStatusText = function(field, node, cell) {
	node.appendChild(document.createTextNode(slaveStatusText(field.getValue())));
	setClass(cell, slaveStatusTextFieldClass(field.getValue()));
}


/**
 * Shows the tooltip for a table field.
 */
function showTableTooltip(event) {
	event = exists(event) ? event : window.event;
	var target = getTargetElement(event);
	var name = target.getAttribute("name");
	if (!exists(name)) {
		name = findTdNode(target).getAttribute("name");
	}
	
	var srcElement = isIE() || isOpera() ? event.srcElement : event.target;
	var table = tableObject(srcElement);
	if (exists(table) && table.rightClickMenuShown == true) {
		// tables right click menu is shown, do not show tooltip also
		return;	
	}
	
	Tip(helptext(name), BGCOLOR, '#ffffe1', ABOVE, false, DELAY, 500);
}


/**
 * Inserts a given fields alarmcount into a html node.
 * @param field The field to insert the value for.
 * @param node The node to insert the value into.
 * @param cell The table cell node is in. 
 */
ProbeTable.prototype.insertAlarmCount = function(field, node, cell) {
	if (isEmpty(field.getValue())) {
		// no alarm is defined for this entry
		return;
	}
	// the upper eight bits are the severity
	var severity = (parseInt(field.getValue()) & 0xff000000) >> 24;
	var alarmCount = (parseInt(field.getValue()) & 0x00ffffff); 
	setClass(cell, getClassName(cell) + " " + severities[severity].toLowerCase());
	
	
	var row = this.getRowFromField(field);
	var nameField = getFieldFromRow("protocolName", row);

	node.appendChild(document.createTextNode(alarmCount));
}


// setup


/**
 * Shows the setup for this window.
 * @param columns The number of colums the fields to choose should be displayed in.
 * @param event The onclick event that triggered the calling of this function.
 */
ProbeTable.prototype.showSetup = function(columns, event) {
	if (!event) {
			// IE
			event = window.event;
	}
	
	if (isDefined(this.tooltip) && ("tooltip" == this.tooltip.getAttribute("class") || "tooltip" == this.tooltip.getAttribute("className"))) {
		// the tooltip is already shown
		return;
	}
	
	var setup = this.createSetup(columns);
	// set a very large z-index so the tooltop is on top of the table
	this.tooltip = showTooltip(setup, event.clientX, event.clientY, 100000000, false, Desktop.instance().getDesktopCode());
	
	this.imageResized = false;
	if (exists(this.window)) {
		// might not exist in tests
		this.window.addMoveElement(this.tooltip);
		this.window.setup = this.tooltip;
	}
	this.checkChooserFields();
}


/**
 * Hides the setup of this table. This is overidden to remove the setup
 * as one of the elements to move along with the window.
 */
ProbeTable.prototype.closeSetup = function() {
	closeTooltip(this.tooltip, Desktop.instance().getDesktopCode());
	if (exists(this.window)) {
		this.window.removeMoveElement(this.tooltip);
	}
	this.tooltip = undefined;
	
	if (this.imageResized) {
		// the size of the chart has been changed reload it
		var width = parseInt(this.window.activeChart.htmlCode.style.width);
		var height = parseInt(this.window.activeChart.htmlCode.style.height);
		if (exists(this.window)) {
			this.window.activeChart.height = height;
			this.window.activeChart.width = width;
			this.window.activeChart.updateChart(); 
		}
	}
}


/**
 * Toggles the visibility of a column. If the column is visible it is made invisible
 * and if it is unvisible it is made visible.
 * @param name The name of the column. 
 */
ProbeTable.prototype.toggleColumnVisibility = function (name) {
	var madeVisible = false;
	if (this.isVisibleColumn(name)) {
		this.setInvisibleColumn(name);
	}
	else {
		madeVisible = true;
		this.setVisibleColumn(name);
	}

	if (isOpera()) {
		// this is needed in Opera to avoid problems with the 
		// cell width when the scrollbar is shown and hidden
		var tableContainer = this.getTableContainerCode();
		tableContainer.style.display = "none";
		tableContainer.style.display = "";	
	}
	
	if (!isSafari()) {
		// in safari we can not paint the table until we have made room 
		// for the new column, or else we get scrollbars. While in other browsers
		// espically in Opera resizing first can lead to problems
		this.paint();
	}
	
	if (madeVisible) {
		var column = this.getColumn(name);
		var diff = column.getWidth() + this.calculateExtraWidthSize();
		this.resizeContainers(0, diff);
		
		var eventData = new Array();
		eventData["id"] = this.id;
		eventData["height"] = 0;
		eventData["width"] = diff;
		getEventServer().notifyListeners(new Event("tableResized", eventData));
	}
	else {
		var column = this.getColumn(name);
		var diff = column.getWidth() + this.calculateExtraWidthSize();
		this.resizeAllColumns(diff);
	}
	
	if (isSafari()) {
		// in safari we can not paint the table until we have made room 
		// for the new column, or else we get scrollbars. While in other browsers
		// espically in Opera resizing first can lead to problems
		this.paint();
	}
}


/**
 * Calculates the extra width of each table cell, extra width can be any padding and borders. 
 * This assumes the padding and borders are equal for all cells. 
 */
ProbeTable.prototype.calculateExtraWidthSize = function() {
	// get the width of the first column
	var row = this.rows[0];
	if (!isDefined(row) || isNull(row)) {
		return 0;
	}
	var field;
	// get a field which html code still exsits, the html code of a field might have 
	// been deleted
	for (var i = 1; i < row.length; i++) {
		field = row[i];
		if (!isNull(getHtmlFieldFromRow(field.getName(), this.getTableCode().rows[0]))) {
			break;
		}
		if (i == row.length -1) {
			// all html columns have been deleted
			return 0;
		}
	}
	
	var column = this.getColumn(field.getName());
	var columnWidth = column.getWidth();
	
	// get the actual width
	var htmlCell = getHtmlFieldFromRow(field.getName(), this.getTableCode().rows[0]);
	var htmlCellWidth = htmlCell.offsetWidth;
	
	return htmlCellWidth - columnWidth;
}


/**
 * Adjusts the image size on response to a field chooser click.
 */
ProbeTable.prototype.changeImageSize = function() {
	if (!isDefined(this.window.activeChart) || isNull(this.window.activeChart)) {
		return;
	}
	var windowWidth = parseInt(this.window.htmlCode.style.width);
	var newImageWidth = windowWidth - Window.RESIZE_AREA_SIZE;
	this.imageWidthDiff += newImageWidth - parseInt(this.window.activeChart.htmlCode.style.height);
	this.imageResized = true;
	this.window.activeChart.htmlCode.style.width = newImageWidth + "px";
	if (!this.window.isResizaAreaVisible()) {
		this.window.makeResizeAreaVisible();
	}
}


// sorting


/**
 * Compares to rows. Each row is a array.
 * @param a The first row to sort.
 * @param b The second row to sort.
 * @return A negative value if the sort field of row a is less than b the sort field of row b, a positive
 * value if a's sort field is larger than b's.
 */
function sortFunction(a, b) {
	var aValue = getSortValue(a);
	var bValue = getSortValue(b);
	if (isAlarmCountFieldSorted()) {
		return sortAlarmCountValues(aValue, bValue);
	}
	else if (isDottedNumber(aValue) || isDottedNumber(bValue)) {
		return sortDottedNumbers(aValue, bValue);
	}
	else if (isFloat(aValue) || isFloat(bValue)) {
		return sortFloats(aValue, bValue);
	}
	else if(isNumberString(aValue) || isNumberString(bValue)) {
		return sortNumbers(aValue, bValue);
	}
	else {
		aValue = aValue.toUpperCase();
		bValue = bValue.toUpperCase();
		return sortStrings(aValue, bValue);
	}
}


/**
 * Compares two alarmCount values.
 * @param a The first alarmCount value.
 * @param b The second alarmCount value.
 * @return -1 for less, 0 for equal, 1 for larger.
 */
function sortAlarmCountValues(a, b) {
	if (isEmpty(a) && !isEmpty(b)) {
		return 1;
	}
	else if (!isEmpty(a) && isEmpty(b)) {
		return -1;
	}
	else if (isEmpty(a) && isEmpty(b)) {
		return 0;
	}
	var aValue = (parseInt(a) & 0x00ffffff);
	var bValue = (parseInt(b) & 0x00ffffff);
	return sortNumbers(aValue, bValue);
}


/**
 * Returns true if the current sorted field is the alarmCount field, false otherwise.
 */
function isAlarmCountFieldSorted() {
	if ("alarmCount".equalsIgnoreCase(columnToSort)) {
		return true;
	}
	return false;
}


/**
 * Sorts two octet fields.
 * @param a The first row to sort.
 * @param b The second row to sort.
 * @return A negative value if the sort field of row a is less than b the sort field of row b, a positive
 */
function sortOctets(a, b) {
	aValue = getFieldFromRow("octets", a).getValue();
	bValue = getFieldFromRow("octets", b).getValue();
	return sortNumbers(aValue, bValue);
}
		
		
/**
 * Returns true if a string contains a dotted number. 
 * A dotted number might be a IP address (10.0.0.0) or a 
 * port id number (1.2048.6.80)
 * @param string The string to check.
 */
function isDottedNumber(string) {
	var index = string.indexOf(".");
	if (index < 0) {
		// the string contains no punctuations
		return false;
	}
	
	var number = parseInt(string);
	if (isNaN(number)) {
		// the string contains no numbers
		return false;
	}
	
	return true;
}


/**
 * Compares two dotted numbers.
 * A dotted number might be a IP address (10.0.0.0) or a 
 * @param a The first port number.
 * @param b The second port number.
 * @return -1 for less, 0 for equal, 1 for larger.
 */
function sortDottedNumbers(a, b) {
	if(isDottedNumber(a) && (!isDottedNumber(b))) {
		// dotted numbers like ip addresses should come first
		return -1;	
	}
	else if(isDottedNumber(b) && (!isDottedNumber(a))) {
		// dotted numbers like ip addresses should come first
		return 1;	
	}
	
	var aStartIndex = 0;
	var bStartIndex = 0;

	while(true) {
		var aIndex = a.indexOf(".", aStartIndex);
		var bIndex = b.indexOf(".", bStartIndex);
	
		if (aIndex < 0 || bIndex < 0) {
			var aSlice = parseInt(a.substring(aStartIndex));
			var bSlice = parseInt(b.substring(bStartIndex));
			return bSlice - aSlice;
		}

		var aSlice = parseInt(a.substring(aStartIndex, aIndex));
		var bSlice = parseInt(b.substring(bStartIndex, bIndex));
		
		if (aSlice != bSlice) {
			return bSlice - aSlice;	
		}

		aStartIndex = aIndex + 1;
		bStartIndex = bIndex + 1;
	}
}


// updating


/**
 * Updates a html table cell of the table according to the data in the specified field.
 * @param dataField The field containing the data to update the cell with.
 * @param htmlCell The html cell that is to be updated with the data in dataField.
 * @param htmlRow The row the cell to update is in.
 */
ProbeTable.prototype.updateTableCell = function(dataField, htmlCell, htmlRow) {
	if (dataField == null) {
		var dataValue = "";
	}
	else {
		var dataValue = dataField.getValue();
	}
	var htmlValue = getTekstValue(htmlCell, "linkRequest");
	// the alarm count field needs to be updated anyway, because when the html field value is 0
	// and the dataField.value also is 0 the severity still could be different.
	if (dataValue == htmlValue && dataField.getName() != "alarmCount") {
		return;
	}
	
	var newTableCell = document.createElement("td");
	this.setAlignment(newTableCell, dataField);
	htmlRow.replaceChild(newTableCell, htmlCell);
	if (isDefined(dataField) && !isNull(dataField)) {
		this.createCellCode(dataField, newTableCell, dataField.getName());
	}
	
	// explicit deallocation to prevent memory leak in IE
	newTableCell = null;
	htmlCell = null;
}


/**
 * Set the tag name of the entries in the xml documents loaded.
 * @param tagName The name of the entry tags to set. 
 * @throws Error if tagName is undefined, null or empty.
 */
ProbeTable.prototype.setEntryTagName = function(tagName) {
	assertNotEmpty(tagName, "TagName to set is undefined, null or empty");
	
	this.tagName = tagName;
}


/**
 * Cancels the updating of this table.
 */
ProbeTable.prototype.cancelUpdate = function() {
	this.isLoading = false;
	this.repeater.stop();
}


/**
 * Restarts the updating of this table. 
 */
ProbeTable.prototype.restartUpdate = function() {
	this.isLoading = false;
	this.repeater.stop();
	this.repeater.start();
}


/**
 * Loads data into the table. The data is loaded from the url set. 
 * The loaded xml data is stored in a property called xmlData.
 * This is overidden because we may need to change the
 * window size on refreshing the data.
 * @param functionToCall Optional argument, this function is called if specified when the data
 * has been loaded. The function is called as a method of this table object.
 */
function load(functionToCall) {
	if (this.isLoading == true || this.layoutSetInProgress == true || isEmpty(this.url)) {
		// the table is already loading data, wait for the loading to complete
		// or the tables layout is currently set, do not update the table while its layout is set
		// because it can cause multiple updates
		return;
	}
	
	this.isLoading = true;
	var thisObj = this;
	loadXML(this.url, function(xmlData) {
		thisObj.isLoading = false;
		if (thisObj.isNoAccess(xmlData) && !thisObj.noAccessShown) {
			thisObj.showNoAccess();
			thisObj.fireNoAccessShownEvent();
			// used to make the loading screen dissappear the first time data is loaded
			thisObj.fireUpdatedEvent();
			return;
		}
		else if (!thisObj.isNoAccess(xmlData) && thisObj.noAccessShown) {
			thisObj.hideNoAccess();
			thisObj.fireNoAccessHiddenEvent();
		}
		else if (thisObj.noAccessShown) {
			return;
		}
		thisObj.updateTable(xmlData);
		if (isDefined(functionToCall)) {
			functionToCall.call(thisObj);
		}
		// reset the failed update counter
		this.failedUpdateCounter = 0;
	}, undefined, function() {
		// error processing
		thisObj.isLoading = false;
		thisObj.failedUpdateCounter++;
		if (thisObj.failedUpdateCounter == ProbeTable.MAX_FAILED_UPDATE) {
			thisObj.cancelUpdate();
		}
	});
}


/**
 * Updates the table with loaded data.
 * @param xmlData The xml data loaded.
 */
ProbeTable.prototype.updateTable = function(xmlData) {
	this.clear();
	this.setData(xmlData, this.tagName);
	this.updateHtmlTable();
	this.fireUpdatedEvent();
}


// events


/**
 * Inits the event processor.
 */
ProbeTable.prototype.initEventProcessor = function() {
	this.eventListener = new Listener(this);
	
	this.eventListener.addEvent("resized", processResizeEvent);
	this.eventListener.addEvent(WINDOW_HIDDEN_EVENT, processWindowHiddenEvent);
	this.eventListener.addEvent(WINDOW_SHOWN_EVENT, processWindowShownEvent);
	this.eventListener.addEvent("tabActivated", processTabActivatedEventTable);
	this.eventListener.addEvent("tabDeactivated", processTabDeactivatedEventTable);
	this.eventListener.addEvent("chartDetached", processChartDetachedEventTable);
	this.eventListener.addEvent("chartAttached", processChartAttachedEventTable);
	this.eventListener.addEvent("topNChanged", processTopNChangedTable);
	this.eventListener.addEvent("rightClickMenuShown", processRightClickMenuShownEvent);
	this.eventListener.addEvent("rightClickMenuHidden", processRightClickMenuHiddenEvent);
	this.eventListener.addEvent(SLAVE_SET_EVENT, processSlaveSetProbeTable);
	this.eventListener.addEvent(SLAVE_DELETED_EVENT, processSlaveDeletedProbeTable);
}


/**
 * Get the event processor of this table.
 * @return This tables event processor, undefined if the event processor has not ben inited for this table.
 */
ProbeTable.prototype.getEventProcessor = function() {
	return this.eventListener;	
}


/**
 * Processes a right click menu shown event. 
 * @param event The event to process.
 */
function processRightClickMenuShownEvent(event) {
	if (this.id != event.getData()) {
		return;
	}
	this.rightClickMenuShown = true;
	tt_Hide();
}


/**
 * Processes a right click menu hidden event. 
 * @param event The event to process. 
 */
function processRightClickMenuHiddenEvent(event) {
	if (this.id != event.getData()) {
		return;
	}
	this.rightClickMenuShown = false;
}


/**
 * Processes a topn changed event. 
 * @param event The event to process. 
 */
function processTopNChangedTable(event) {
	if (this.id != event.getData()["id"]) {
		return;
	}
	
	this.setN(parseInt(event.getData()["n"]));
	this.restartUpdate();
}


/**
 * Processes a chart attached event. 
 * @param event The event to process. 
 */
function processChartAttachedEventTable(event) {
	if (!this.isTopNChart(event.getData()["chart"])) {
		return;
	}

	this.topnChartDetached = false;
}


/**
 * Processes a chart detached event. 
 * @param event The event to process. 
 */
function processChartDetachedEventTable(event) {
	if (!this.isTopNChart(event.getData()["chart"])) {
		return;
	}
	
	this.topnChartDetached = true;
}


/**
 * Processes a tab activated event. 
 * @param event The event to process. 
 */
function processTabActivatedEventTable(event) {
	if (!currentTabName.equalsIgnoreCase(event.getData())) {
		return;
	}
	if (!this.activeTabDeactivated) {
		return;
	}
	this.update();
	this.activeTabDeactivated = false;
}


/**
 * Processes a tab deactivated event. 
 * @param event The event to process. 
 */
function processTabDeactivatedEventTable(event) {
	if (!currentTabName.equalsIgnoreCase(event.getData())) {
		return;
	}
	
	if (!this.isUpdating()) {
		return;
	}
	this.cancelUpdate();
	this.activeTabDeactivated = true;
}


/**
 * Processes a window shown event. 
 * @param event The event to process. 
 */
function processWindowShownEvent(event) {
	if (!this.windowName.equalsIgnoreCase(event.getData())) {
		// the event is not from the window this table is in
		return;
	}
	this.update();
}


/**
 * Processes a window hidden event. 
 * @param event The event to process. 
 */
function processWindowHiddenEvent(event) {
	if (!this.windowName.equalsIgnoreCase(event.getData())) {
		// the event is not from the window this table is in
		return;
	}
	if (this.topnChartDetached) {
		// the topn chart for this table has been detached, keep updating the table
		return;
	}
	this.cancelUpdate();
}


/**
 * Process a resize event.
 * @param event The resize event to process.
 */
function processResizeEvent(event) {
	if (event.getData()["id"] != this.windowName) {
		// the table only cares about events from its window, none of the others
		return;
	}
	if (this.noAccessShown) {
		return;
	}
	
	var resizedWidth = event.getData()["width"];
	this.resizeTable(0, resizedWidth);
}


/**
 * Processes a slave set event.
 * @param event The event to process. 
 */
function processSlaveSetProbeTable(event) {
	if ("topNNetworksTable" != this.id || !this.isUpdating()) {
		return;
	}
	this.restartUpdate();
}


/**
 * Process a slave deleted event.
 * @param event The event to process. 
 */
function processSlaveDeletedProbeTable(event) {
	if ("topNNetworksTable" != this.id || !this.isUpdating()) {
		return;
	}
	this.restartUpdate();
}


/**
 * Fires a no access hidden event. 
 */
ProbeTable.prototype.fireNoAccessHiddenEvent = function() {
	getEventServer().notifyListeners(new Event("noAccessHidden", this.id));
}


/**
 * Fires a no access shown event.
 */
ProbeTable.prototype.fireNoAccessShownEvent = function() {
	getEventServer().notifyListeners(new Event("noAccessShown", this.id));
}


/**
 * Fires a updated event.
 */
ProbeTable.prototype.fireUpdatedEvent = function() {
	var eventData = new Array();
	eventData["id"] = this.id;
	// the table code might not be created yet if the user does
	// not have access to the table
	if (isDefined(this.getTableCode()) && !isNull(this.getTableCode())) {
		eventData["newRowCount"] = this.getTableCode().rows.length
	}
	getEventServer().notifyListeners(new Event("tableUpdated", eventData));
}


// no access services


/**
 * Returns true if a xmlDocument is noaccess.html.
 * @param xmlDocument The xml document to check.
 */
ProbeTable.prototype.isNoAccess = function (xmlDocument) {
	if (xmlDocument.getElementsByTagName("noaccess").length != 0) {
		// a html error block is received
		return true;
	}

	if(!isNull(getElementWithAttribute(xmlDocument, "images/noaccess.jpg", "src", "img"))) {
		// noaccess html is received
		return true;
	}
	return false;
}


/**
 * Hides the no access for this window.
 */
ProbeTable.prototype.hideNoAccess = function() {
	if (!isDefined(this.noAccess)) {
		return;
	}
	
	var noAccesHeight = this.noAccess.offsetHeight;
	setClass(this.noAccess, "inactive");
	this.noAccessShown = false;
	
	if (isDefined(this.originalClass) && !isNull(this.originalClass) && !isEmpty(this.originalClass)) {
		setClass(this.getTableContainerCode(), this.originalClass);
	}
	else {
		setClass(this.getTableContainerCode(), "active");
	}
	
	// if we not add a new timestamp to the active image IE will not update the image
	// we update the active image here because else it could take upto one update interval
	// until a image is shown
	this.window.activeChart.markForTimestamp();
	this.window.activeChart.updateChart();
	this.window.removeResizableElement(this.noAccess);
}


/**
 * Shows no acces over this windows content. 
 */
ProbeTable.prototype.showNoAccess = function() {
	if (!isDefined(this.noAccess)) {
 		var noAccessTable = this.createNoAccessCode();
 		setClass(noAccessTable, "inactive");
	 	this.parentElement.appendChild(noAccessTable);
		this.noAccess = noAccessTable;
	}
	
	this.activateNoAccess();
 	this.noAccessShown = true;
}


/**
 * Activates the no access div of this window.
 */
ProbeTable.prototype.activateNoAccess = function() {
	// the table code is deactivated, and the original class is saved in this.originalClass
	this.originalClass = this.getTableContainerCode().getAttribute("class");
	if (!isDefined(this.originalClass) || isNull(this.originalClass)) {
		// IE
		this.originalClass = this.getTableContainerCode().getAttribute("className");
	}
	
	this.originalTableWidth = this.window.htmlCode.offsetWidth;
	this.originalTableHeight = this.getTableContainerCode().offsetHeight;
 	setClass(this.getTableContainerCode(), "inactive");
 	setClass(this.noAccess, "active");
 	
	this.window.setResizableElements(this.noAccess);
}


/**
 * Creates the html code for the no access screen to show.
 */
ProbeTable.prototype.createNoAccessCode = function() {
	var outerTable = document.createElement("table");
	outerTable.setAttribute("width", "100%");
	outerTable.setAttribute("height", "100%");
	outerTable.setAttribute("name", "noAccess");
	outerTable.setAttribute("bgColor", "white");
	var outerTableBody = document.createElement("TBODY");
	outerTable.appendChild(outerTableBody);
	
	var row = document.createElement("tr");
	var cell = document.createElement("td");
	var center = document.createElement("div");
	center.setAttribute("align", "center");
	
	var noAccessTable = document.createElement("table");
	noAccessTable.setAttribute("bgColor", "#EBEBEB");
	noAccessTable.width = "250px";
	setClass(noAccessTable, "noAccessTable");
	var noAccessTableBody = document.createElement("TBODY");
	noAccessTable.appendChild(noAccessTableBody);
	
	var noAccessRow = document.createElement("tr");
	var noAccessCell = document.createElement("td");
	
	var noAccessDiv = document.createElement("div");
	noAccessDiv.setAttribute("align", "center");
	setClass(noAccessDiv, "noAccessDiv");

	// the no access image
	var noAccessImage = document.createElement("img");
	noAccessImage.src = "images/noaccess.jpg";
	noAccessImage.setAttribute("width", "48");
	noAccessImage.setAttribute("height", "48");
	noAccessImage.setAttribute("name", "noAccessImage");
	
	// the no acces text
	var noAccessBr = document.createElement("br");
	var noAccessText = document.createTextNode("No Access");
	
	noAccessDiv.appendChild(noAccessImage);
	noAccessDiv.appendChild(noAccessBr);
	noAccessDiv.appendChild(noAccessText);
	noAccessCell.appendChild(noAccessDiv);
	noAccessRow.appendChild(noAccessCell);
	noAccessTableBody.appendChild(noAccessRow);
	center.appendChild(noAccessTable);
	cell.appendChild(center);
	row.appendChild(cell);
	outerTableBody.appendChild(row);
	
	return outerTable;
}


/**
 * Returns true if a given chart is the topn chart for this table. 
 * @param chart The chart to check.
 */
ProbeTable.prototype.isTopNChart = function(chart) {
	var tableId = this.id.substring(0, this.id.length -5);
	tableId = tableId.toLowerCase();
	var chartId = chart.getId().toLowerCase();
	if (chartId.indexOf(tableId) == -1) {
		return false;
	}
	return true;
}
