/** 
 * This class is responsible for managing tables.
 */
 

/**
 * The max number of data rows visible in the table without a scrollbar. If more rows are
 * added to the table a scrollbar is used.
 */
ProbeTableManager.MAX_ROW_NO_SCROLLBAR = 5;


/**
 * The update time for the tables.
 */
ProbeTableManager.TABLE_UPDATE_TIME = 5;

 
/**
 * The functions used to init the tables, indexed by the table name. 
 */ 
var initTableFunction = {summaryTable: initSummaryTable, topNProtocolsTable: initTopNProtocolsTable, 
	myProtocolsTable: initMyProtocolsTable, topNHostsTable: initTopNHostsTable, myHostsTable: initMyHostsTable,
	topNHostListenersTable: initTopNHostListnersTable, topNHostTalkersTable: initTopNHostTalkersTable, 
	topNConversationsTable: initTopNConversationsTable, myConversationsTable: initMyConversationsTable,
	myProtocolsPerHostTable: initMyProtocolsPerHostTable, profilesTable: initProfilesTable, 
	allDefinedAlarmsTable: initAllDefinedAlarmsTable, summaryDefinedAlarmsTable: initSummaryDefinedAlarmsTable,
	protocolDefinedAlarmsTable: initProtocolDefinedAlarmsTable, hostsDefinedAlarmsTable: initHostsDefinedAlarmTable,
	conversationsDefinedAlarmsTable: initConversationsDefinedAlarmTable, allTriggeredAlarmsTable: initAllTriggeredAlarmTable,
	summaryTriggeredAlarmsTable: initSummaryTriggeredAlarmsTable, protocolTriggeredAlarmsTable: initProtocolTriggeredAlarmsTable,
	hostsTriggeredAlarmsTable: initHostTriggeredAlarmTable, conversationsTriggeredAlarmsTable: initConversationsTriggeredAlarmTable,
	triggervaluestable: initTriggerValuesTable, emailAlarmActionsTable: initEmailAlarmActionsTable, 
	emailAlarmActionLibraryTable: initEmailAlarmActionsLibraryTable, programAlarmActionLibraryTable: initProgramAlarmActionsLibraryTable,
	programAlarmActionsTable: initProgramAlarmActionsTable, protocolDetailTable : initProtocolDetailTable,
	protocolTopHostsTable: initProtocolTopHostsTable, protocolTopConversationsTable: initProtocolTopConversationsTable,
	nlhostDetailTable: initNlHostDetailTable, nlhostTopProtocolsTable: initNlHostTopProtocolsTable,
	nlhostTopConversationsTable: initNlHostTopConversations, topNNetworksTable: initTopNNetworksTable,
	allTriggeredMasterAlarmsTable: initAllTriggeredMasterAlarmTable};
	 
 
/**
 * Array containing the tables inited indexed by their id.
 */
var initedTables = new Array();


// construction


/**
 * Creates the probeTableManager.
 */
function createProbeTableManager() {
	var instance = new ProbeTableManager();
	TableManager.instanceObject = createSubClass(TableManager.instance(), instance);
}


/**
 * Get the instance of the probeTableManager.
 */
ProbeTableManager.prototype.instance = function() {
	return TableManager.instanceObject;
}


/**
 * Constructs the probeTableManager.
 */
function ProbeTableManager() {


	/**
	 * The event listener used. 
	 */
	this.eventListener = new Listener(this);
	
	 
	// set the events to listen to
	this.eventListener.addEvent(START_SWITCH_PROBE_EVENT, ProbeTableManager.processStartSwitchProbeEvent);
	this.eventListener.addEvent(DONE_SWITCH_PROBE_EVENT, ProbeTableManager.processDoneSwitchProbeEvent);
	
	// the tables stopped from updating while the probe is switched are stored here
	// they are set to restart updating when the probe swtiching is done
	this.stoppedTables = new Array();
}


// services


/**
 * Get a table. This is overridden to init the table if it is not inited yet.
 * @param id The id of the table to get.
 * @return The table, null if no table with this id exists.
 * @throws Error if id is undefined, empty or null.
 */
ProbeTableManager.prototype.getTable = function(id) {
	assertNotEmpty(id, "Id is undefined, empty, or null");

	var len = TableManager.tables.length;
	if (len == 0) {
		initFieldProperties();
	}
	for (var i = 0; i < len; i++) {
		if (id == TableManager.tables[i].id) {
			return TableManager.tables[i];
		}
	}
	
	if (exists(initTableFunction[id]) && !initedTables[id]) {
		return initTableFunction[id]();
	}
	return null;
}


/**
 * Initializes the default tables.
 */
function initDefaultTables() {
	if (currentTabName == MASTER_MAIN_TAB_NAME) {
		TableManager.instance().getTable("topNNetworksTable").update();
		TableManager.instance().getTable("allTriggeredMasterAlarmsTable").update();
	}
	else {	
		TableManager.instance().getTable("summaryTable").update();	
		TableManager.instance().getTable("topNProtocolsTable").update();
	}
}


// event processing


/**
 * Processes a start switch probe event. Stop all tables from updating while 
 * the current probe is switched. 
 * @param event The event to process. 
 */
ProbeTableManager.processStartSwitchProbeEvent = function() {
	for (var i = 0; i < TableManager.tables.length; i++) {
		if (TableManager.tables[i].isUpdating()) {
			this.stoppedTables[TableManager.tables[i].getId()] = TableManager.tables[i];
			TableManager.tables[i].cancelUpdate();
		}
	}
}


/**
 * Process a probe switch done event. When the probe switcing is done the 
 * tables can load data again. 
 * @param event The event to process. 
 */
ProbeTableManager.processDoneSwitchProbeEvent = function() {
	for (tableId in this.stoppedTables) {
		this.stoppedTables[tableId].restartUpdate();
	}
}


// implementation


/**
 * Inits the field properties.
 */
function initFieldProperties() {
	Table.addProperty("bold", makeBold);
}


/**
 * Init the summary table.
 */
function initSummaryTable() {
	initedTables["summaryTable"] = true;
	var summaryCell = getElementWithAttribute(WindowManager.instance().getWindow("summarywindow").getHtmlCode(), "windowContentCell", "id", "td");
	var summaryTable;
	summaryTable = createProbeTable('summaryTable', '/?resource=Statistics.summary&command=summary&mode=getdata&fields=index,probeName,probeDescription,protocolCount,hostCount,conversationCount,cardCount,packetCount,octetCount,src,alarmCount&format=xml&sid=' + getSid(), summaryCell, 'summarywindow', ProbeTableManager.TABLE_UPDATE_TIME);
	summaryTable.setSize(827, 35);
	summaryTable.setClass("defaultTable summaryTable");
	summaryTable.setHeaderRowClass("summaryTableHeaderRow");
	summaryTable.setHeaderTableClass("defaultTable summaryTableHeader");
	summaryTable.toggleAlternateRowClass(true, "even");
	summaryTable.setRowClass("odd");
	summaryTable.setMouseOverClass("mouseover");
	// the sum of the column widths is smaller than the width set for the table because the browser adds some 
	// additional pixels to each column in the form of padding and borders
	summaryTable.addColumns("alarmCount", "", "Triggered alarm count", 30, "probeName", "Probe name", "", 162, "probeDescription", "Probe description", "", 163, "octetCount", "Bytes", "", 61, "packetCount", "Packets", "", 68, "hostCount", "Hosts", "", 53, "conversationCount", "Conversations", "", 110, "protocolCount", "Protocols", "", 78, "cardCount", "Cards", "", 57);
	summaryTable.setVisibleColumns("alarmCount", "probeName", "probeDescription", "protocolCount", "hostCount", "conversationCount", "cardCount", "packetCount", "octetCount");
	summaryTable.setHeaderProperty("bold");
	summaryTable.setTextAlignments("alarmCount", "right", "protocolCount", "right", "hostCount", "right", "conversationCount", "right", "cardCount", "right", "packetCount", "right", "octetCount", "right");
	summaryTable.setLinkFields("packetCount", "octetCount", "protocolCount", "hostCount", "conversationCount", "cardCount");
	summaryTable.setLinkUrlCreator(createUrl);
	summaryTable.setRightClickMenuColumns("probeName", "probeDescription", "protocolCount", "hostCount", "conversationCount", "cardCount", "packetCount", "octetCount");	
	summaryTable.setRightClickMenuFunction(showDefaultMenu);
	
	return summaryTable;
}


/**
 * Inits the topn networks table.
 */
function initTopNNetworksTable() {
	initedTables["topNNetworksTable"] = true;
	var topNNetworksCell = getElementWithAttribute(WindowManager.instance().getWindow("topnNetworksWindow").getHtmlCode(), "windowContentCell", "id", "td");
	var topNNetworksTable = createTopNTable('topNNetworksTable', '/?resource=Statistics.summary&command=topn&mode=getdata&top=5&sortBy=octetCount&fields=index,probeName,probeDescription,protocolCount,hostCount,conversationCount,cardCount,packetCount,octetCount,src,alarmCount,status,src&format=xml&sid=' + getSid(), topNNetworksCell, 'topnNetworksWindow', ProbeTableManager.TABLE_UPDATE_TIME);
	topNNetworksTable.setSize(932, 103);
	topNNetworksTable.setClass("defaultTable summaryTable");
	topNNetworksTable.setHeaderRowClass("summaryTableHeaderRow");
	topNNetworksTable.setHeaderTableClass("defaultTable summaryTableHeader");
	topNNetworksTable.setRowClass("odd");
	topNNetworksTable.toggleAlternateRowClass(true, "even");
	topNNetworksTable.setMouseOverClass("mouseover");
	// the sum of the column widths is smaller than the width set for the table because the browser adds some 
	// additional pixels to each column in the form of padding and borders
	topNNetworksTable.addColumns("src", "", "", 0, "alarmCount", "", "Triggered alarm count", 30, "probeName", "Probe name", "", 162, "probeDescription", "Probe description", "", 162, "status", "Probe status", "", 100, "octetCount", "Bytes", "", 61, "packetCount", "Packets", "", 68, "hostCount", "Hosts", "", 53, "conversationCount", "Conversations", "", 110, "protocolCount", "Protocols", "", 78, "cardCount", "Cards", "", 57);
	topNNetworksTable.setVisibleColumns("alarmCount", "probeName", "probeDescription", "protocolCount", "hostCount", "conversationCount", "cardCount", "packetCount", "octetCount", "status");
	topNNetworksTable.setHeaderProperty("bold");
	topNNetworksTable.setTextAlignments("alarmCount", "right", "protocolCount", "right", "hostCount", "right", "conversationCount", "right", "cardCount", "right", "packetCount", "right", "octetCount", "right");
	topNNetworksTable.setLinkFields("packetCount", "octetCount", "protocolCount", "hostCount", "conversationCount", "cardCount");
	topNNetworksTable.setLinkUrlCreator(createUrl);
	topNNetworksTable.setSortable(true);
	topNNetworksTable.setSortFunction(sortFunction);
	topNNetworksTable.setColumnToSort("octetCount");
	topNNetworksTable.setRightClickMenuColumns("probeName", "probeDescription", "protocolCount", "hostCount", "conversationCount", "cardCount", "packetCount", "octetCount");
	topNNetworksTable.setRightClickMenuFunction(showTopNetworksTableMenu);
	topNNetworksTable.setScrollObject("topnnetworkschartimage");
	return topNNetworksTable;
}


/**
 * Inits the protocol detail table. 
 */
function initProtocolDetailTable() {
	initedTables["protocolDetailTable"] = true;
	var parent = getElementWithAttribute(WindowManager.instance().getWindow("protocolDetailWindow").getHtmlCode(), "windowContentCell", "id", "td");
	var protocolDetailTable = createProbeTable('protocolDetailTable', '/?resource=Statistics.protocol&command=topnFilter&mode=getdata&top=1&srcFilterValue=-1&dstFilterValue=-1&protocolFilterValue=-1&sortBy=octets&fields=index,protocolName,protocolId,protocolDescription,octets,packets,hosts,conversations,alarmCount&format=xml&sid=' + getSid(), parent, 'protocolDetailWindow', ProbeTableManager.TABLE_UPDATE_TIME);
	protocolDetailTable.setSize(827, 35);
	protocolDetailTable.setClass("defaultTable summaryTable");
	protocolDetailTable.setHeaderRowClass("summaryTableHeaderRow");
	protocolDetailTable.setHeaderTableClass("defaultTable summaryTableHeader");
	protocolDetailTable.setRowClass("odd");
	protocolDetailTable.setMouseOverClass("mouseover");
	// the sum of the column widths is smaller than the width set for the table because the browser adds some 
	// additional pixels to each column in the form of padding and borders
	protocolDetailTable.addColumns("alarmCount", "", "Triggered alarm count", 30, "protocolName", "Name", "", 186, "protocolId", "Port", "", 89, "protocolDescription", "Description", "", 186, "octets", "Bytes", "", 60, "packets", "Packets", "", 68, "hosts", "Hosts", "", 56, "conversations", "Conversations", "", 111);
	protocolDetailTable.setVisibleColumns("alarmCount", "protocolName", "protocolId", "protocolDescription", "octets", "hosts", "conversations", "packets");
	protocolDetailTable.setHeaderProperty("bold");
	protocolDetailTable.setTextAlignments("alarmCount", "right", "octets", "right", "hosts", "right", "conversations", "right", "packets", "right");
	protocolDetailTable.setLinkUrlCreator(createUrl);
	protocolDetailTable.setLinkFields("octets", "hosts", "conversations", "packets");
	protocolDetailTable.setSortable(true);
	protocolDetailTable.setSortFunction(sortFunction);
	protocolDetailTable.setRightClickMenuColumns("protocolName", "protocolId", "protocolDescription", "octets", "packets", "hosts", "conversations");
	protocolDetailTable.setRightClickMenuFunction(showProtocolTableMenu);
	return protocolDetailTable;
}


/**
 * Inits the table showing top hosts for a protocol.
 */
function initProtocolTopHostsTable() {
	initedTables["protocolTopHostsTable"] = true;
	var parent = getElementWithAttribute(WindowManager.instance().getWindow("protocolDetailWindow").getHtmlCode(), "middleContentCell", "id", "td");
	var protocolTopHostsTable = createTopNTable('protocolTopHostsTable', '/?resource=Statistics.alhost&command=topnFilter&mode=getdata&top=4&srcFilterValue=-1&dstFilterValue=-1&protocolFilterValue=-1&sortBy=inOctets&fields=index,hostName,address,protocolName,protocolId,inPackets,inOctets,outPackets,outOctets,alarmCount&format=xml&sid=' + getSid(), parent, 'protocolDetailWindow', ProbeTableManager.TABLE_UPDATE_TIME);
	protocolTopHostsTable.setSize(827, 86);
	protocolTopHostsTable.setClass("defaultTable summaryTable");
	protocolTopHostsTable.setHeaderRowClass("summaryTableHeaderRow");
	protocolTopHostsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	protocolTopHostsTable.setRowClass("odd");
	protocolTopHostsTable.toggleAlternateRowClass(true, "even");
	protocolTopHostsTable.setMouseOverClass("mouseover");
	// the sum of the column widths is smaller than the width set for the table because the browser adds some 
	// additional pixels to each column in the form of padding and borders
	protocolTopHostsTable.addColumns("alarmCount", "", "Triggered alarm count", 30, "hostName", "Host name", "", 186, "address", "Address", "", 110, "inPackets", "Packets in", "", 118, "inOctets", "Bytes in", "", 117, "outPackets", "Packets out", "", 115, "outOctets", "Bytes out", "", 115);
	protocolTopHostsTable.setVisibleColumns("alarmCount", "hostName", "address", "inPackets", "inOctets", "outPackets", "outOctets");
	protocolTopHostsTable.setHeaderProperty("bold");
	protocolTopHostsTable.setTextAlignments("alarmCount", "right", "inOctets", "right", "inPackets", "right", "outPackets", "right", "outOctets", "right");
	protocolTopHostsTable.setLinkFields("octets", "inOctets", "inPackets", "outPackets", "outOctets");
	protocolTopHostsTable.setColumnToSort("inOctets");
	protocolTopHostsTable.setSortable(true);
	protocolTopHostsTable.setSortFunction(sortFunction);
	protocolTopHostsTable.setLinkUrlCreator(createUrl);
	protocolTopHostsTable.setRightClickMenuColumns("hostName", "address", "inPackets", "inOctets", "outPackets", "outOctets");
	protocolTopHostsTable.setRightClickMenuFunction(showDetailTableMenu);
	return protocolTopHostsTable;
}


/**
 * Inits the table showing top conversations for a protocol.
 */
function initProtocolTopConversationsTable() {
	initedTables["protocolTopConversationsTable"] = true;
	var parent = getElementWithAttribute(WindowManager.instance().getWindow("protocolDetailWindow").getHtmlCode(), "bottomContentCell", "id", "td");
	var protocolTopHostsTable = createTopNTable('protocolTopConversationsTable', '/?resource=Statistics.almatrix&command=topnFilter&mode=getdata&top=4&srcFilterValue=-1&dstFilterValue=-1&protocolFilterValue=-1&sortBy=octets&fields=index,srcHostName,srcAddress,dstHostName,dstAddress,protocolName,protocolId,packets,octets,alarmCount&format=xml&sid=' + getSid(), parent, 'protocolDetailWindow', ProbeTableManager.TABLE_UPDATE_TIME);
	protocolTopHostsTable.setSize(827, 86);
	protocolTopHostsTable.setClass("defaultTable summaryTable");
	protocolTopHostsTable.setHeaderRowClass("summaryTableHeaderRow");
	protocolTopHostsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	protocolTopHostsTable.setRowClass("odd");
	protocolTopHostsTable.toggleAlternateRowClass(true, "even");
	protocolTopHostsTable.setMouseOverClass("mouseover");
	// the sum of the column widths is smaller than the width set for the table because the browser adds some 
	// additional pixels to each column in the form of padding and borders
	protocolTopHostsTable.addColumns("alarmCount", "", "Triggered alarm count", 30, "srcHostName", "Src host name", "", 186, "srcAddress", "Src address", "", 113, "dstHostName", "Dst host name", "", 186, "dstAddress", "Dst address", "", 114, "packets", "Packets", "", 81, "octets", "Bytes", "", 81);
	protocolTopHostsTable.setVisibleColumns("alarmCount", "srcHostName", "srcAddress", "dstHostName", "dstAddress", "packets", "octets");
	protocolTopHostsTable.setHeaderProperty("bold");
	protocolTopHostsTable.setTextAlignments("alarmCount", "right", "packets", "right", "octets", "right");
	protocolTopHostsTable.setLinkFields("octets", "packets");
	protocolTopHostsTable.setColumnToSort("octets");
	protocolTopHostsTable.setSortable(true);
	protocolTopHostsTable.setSortFunction(sortFunction);
	protocolTopHostsTable.setLinkUrlCreator(createUrl);
	protocolTopHostsTable.setRightClickMenuColumns("srcHostName", "srcAddress", "dstHostName", "dstAddress", "packets", "octets");
	protocolTopHostsTable.setRightClickMenuFunction(showDetailTableMenu);
	return protocolTopHostsTable;
}


/** 
 * Inits the nlhost detail table. 
 */
function initNlHostDetailTable() {
	initedTables["nlhostDetailTable"] = true;
	var parent = getElementWithAttribute(WindowManager.instance().getWindow("nlhostDetailWindow").getHtmlCode(), "windowContentCell", "id", "td");
	var nlhostDetailTable = createProbeTable('nlhostDetailTable', '/?resource=Statistics.nlhost&command=topnFilter&mode=getdata&top=1&srcFilterValue=-1&dstFilterValue=-1&protocolFilterValue=-1&sortBy=octets&fields=index,hostName,address,octets,protocols,conversations,packets,alarmCount&format=xml&sid=' + getSid(), parent, 'nlhostDetailWindow', ProbeTableManager.TABLE_UPDATE_TIME);
	nlhostDetailTable.setSize(867, 35);
	nlhostDetailTable.setClass("defaultTable summaryTable");
	nlhostDetailTable.setHeaderRowClass("summaryTableHeaderRow");
	nlhostDetailTable.setHeaderTableClass("defaultTable summaryTableHeader");
	nlhostDetailTable.setRowClass("odd");
	nlhostDetailTable.setMouseOverClass("mouseover");
	// the sum of the column widths is smaller than the width set for the table because the browser adds some 
	// additional pixels to each column in the form of padding and borders
	nlhostDetailTable.addColumns("alarmCount", "", "Triggered alarm count", 30, "hostName", "Host name", "", 206, "address", "Address", "", 130, "octets", "Bytes", "", 118, "packets", "Packets", "", 117, "conversations", "Conversations", "", 115, "protocols", "Protocols", "", 115);
	nlhostDetailTable.setVisibleColumns("alarmCount", "hostName", "address", "octets", "packets", "protocols", "conversations");
	nlhostDetailTable.setHeaderProperty("bold");
	nlhostDetailTable.setTextAlignments("alarmCount", "right", "octets", "right", "octets", "right", "packets", "right", "protocols", "right", "conversations", "right");
	nlhostDetailTable.setLinkFields("octets", "octets", "packets", "hosts", "conversations", "protocols");
	nlhostDetailTable.setColumnToSort("octets");
	nlhostDetailTable.setSortable(true);
	nlhostDetailTable.setSortFunction(sortFunction);
	nlhostDetailTable.setLinkUrlCreator(createUrl);
	nlhostDetailTable.setRightClickMenuColumns("hostName", "address", "octets", "packets", "conversations", "protocols");
	nlhostDetailTable.setRightClickMenuFunction(showHostTableMenu);
	return nlhostDetailTable;
}


/**
 * Inits the table showing topn protocols for a nlhost entry. 
 */
function initNlHostTopProtocolsTable() {
	initedTables["nlhostTopProtocolsTable"] = true;
	var parent = getElementWithAttribute(WindowManager.instance().getWindow("nlhostDetailWindow").getHtmlCode(), "middleContentCell", "id", "td");
	var nlhostTopProtocolsTable = createTopNTable('nlhostTopProtocolsTable', '/?resource=Statistics.alhost&command=topnFilter&mode=getdata&top=4&srcFilterValue=-1&dstFilterValue=-1&protocolFilterValue=-1&sortBy=inOctets&fields=index,hostName,address,protocolName,protocolId,inPackets,inOctets,outPackets,outOctets,alarmCount&format=xml&sid=' + getSid(), parent, 'nlhostDetailWindow', ProbeTableManager.TABLE_UPDATE_TIME);
	nlhostTopProtocolsTable.setSize(867, 86);
	nlhostTopProtocolsTable.setClass("defaultTable summaryTable");
	nlhostTopProtocolsTable.setHeaderRowClass("summaryTableHeaderRow");
	nlhostTopProtocolsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	nlhostTopProtocolsTable.setRowClass("odd");
	nlhostTopProtocolsTable.toggleAlternateRowClass(true, "even");
	nlhostTopProtocolsTable.setMouseOverClass("mouseover");
	// the sum of the column widths is smaller than the width set for the table because the browser adds some 
	// additional pixels to each column in the form of padding and borders
	nlhostTopProtocolsTable.addColumns("alarmCount", "", "Triggered alarm count", 30, "protocolName", "Protocol name", "", 206, "protocolId", "Port", "", 130, "inPackets", "Packets in", "", 118, "inOctets", "Bytes in", "", 117, "outPackets", "Packets out", "", 115, "outOctets", "Bytes out", "", 115);
	nlhostTopProtocolsTable.setVisibleColumns("alarmCount", "protocolName", "protocolId", "inPackets", "inOctets", "outPackets", "outOctets");
	nlhostTopProtocolsTable.setHeaderProperty("bold");
	nlhostTopProtocolsTable.setTextAlignments("alarmCount", "right", "inOctets", "right", "inPackets", "right", "outPackets", "right", "outOctets", "right");
	nlhostTopProtocolsTable.setLinkFields("octets", "inOctets", "inPackets", "outPackets", "outOctets");
	nlhostTopProtocolsTable.setColumnToSort("inOctets");
	nlhostTopProtocolsTable.setSortable(true);
	nlhostTopProtocolsTable.setSortFunction(sortFunction);
	nlhostTopProtocolsTable.setLinkUrlCreator(createUrl);
	nlhostTopProtocolsTable.setRightClickMenuColumns("protocolName", "protocolId", "inPackets", "inOctets", "outPackets", "outOctets");
	nlhostTopProtocolsTable.setRightClickMenuFunction(showDetailTableMenu);
	return nlhostTopProtocolsTable;
}


/**
 * Inits the table showing topn conversations for a nlhost entry. 
 */
function initNlHostTopConversations() {
	initedTables["nlhostTopConversationsTable"] = true;
	var parent = getElementWithAttribute(WindowManager.instance().getWindow("nlhostDetailWindow").getHtmlCode(), "bottomContentCell", "id", "td");
	var nlhostTopConversationsTable = createTopNTable('nlhostTopConversationsTable', '/?resource=Statistics.nlMatrix&command=topnFilter&mode=getdata&top=4&matchConversations=true&srcFilterValue=-1&dstFilterValue=-1&protocolFilterValue=-1&sortBy=octets&fields=index,srcHostName,srcAddress,dstHostName,dstAddress,packets,octets,protocols,alarmCount&format=xml&sid=' + getSid(), parent, 'nlhostDetailWindow', ProbeTableManager.TABLE_UPDATE_TIME);
	nlhostTopConversationsTable.setSize(867, 86);
	nlhostTopConversationsTable.setClass("defaultTable summaryTable");
	nlhostTopConversationsTable.setHeaderRowClass("summaryTableHeaderRow");
	nlhostTopConversationsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	nlhostTopConversationsTable.setRowClass("odd");
	nlhostTopConversationsTable.toggleAlternateRowClass(true, "even");
	nlhostTopConversationsTable.setMouseOverClass("mouseover");
	// the sum of the column widths is smaller than the width set for the table because the browser adds some 
	// additional pixels to each column in the form of padding and borders
	nlhostTopConversationsTable.addColumns("alarmCount", "", "Triggered alarm count", 30, "srcHostName", "Source host", "", 156, "srcAddress", "Source Address", "", 120, "dstHostName", "Destination host", "", 156, "dstAddress", "Destination Address", "", 150, "octets", "Bytes", "", 62, "packets", "Packets", "", 71, "protocols", "Protocols", "", 81);
	nlhostTopConversationsTable.setVisibleColumns("alarmCount", "srcHostName", "srcAddress", "dstHostName", "dstAddress", "octets", "packets", "protocols");
	nlhostTopConversationsTable.setHeaderProperty("bold");
	nlhostTopConversationsTable.setTextAlignments("alarmCount", "right", "octets", "right", "packets", "right", "protocols", "right");
	nlhostTopConversationsTable.setLinkFields("octets", "packets", "protocols");
	nlhostTopConversationsTable.setColumnToSort("octets");
	nlhostTopConversationsTable.setSortable(true);
	nlhostTopConversationsTable.setSortFunction(sortFunction);
	nlhostTopConversationsTable.setLinkUrlCreator(createUrl);
	nlhostTopConversationsTable.setRightClickMenuColumns("srcHostName", "srcAddress", "dstHostName", "dstAddress", "octets", "packets", "protocols");
	nlhostTopConversationsTable.setRightClickMenuFunction(showDetailTableMenu);
	return nlhostTopConversationsTable;
}


/**
 * Inits the topnProtocols table.
 */
function initTopNProtocolsTable() {
	initedTables["topNProtocolsTable"] = true;
	var topNProtocolsCell = getElementWithAttribute(WindowManager.instance().getWindow("topnprotocolswindow").getHtmlCode(), "windowContentCell", "id", "td");
	var topNProtocolsTable = createTopNTable('topNProtocolsTable', '/?resource=Statistics.protocol&command=topn&mode=getdata&top=5&sortBy=octets&fields=index,protocolName,protocolId,protocolDescription,octets,packets,hosts,conversations,alarmCount&format=xml&sid=' + getSid(), topNProtocolsCell, 'topnprotocolswindow', ProbeTableManager.TABLE_UPDATE_TIME);
	topNProtocolsTable.setSize(827, 103);
	var topNWindow = WindowManager.instance().getWindow("topnprotocolswindow");
	topNProtocolsTable.setClass("defaultTable summaryTable");
	topNProtocolsTable.setColumnToSort("octets");
	topNProtocolsTable.setHeaderRowClass("summaryTableHeaderRow");
	topNProtocolsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	topNProtocolsTable.setRowClass("odd");
	topNProtocolsTable.toggleAlternateRowClass(true, "even");
	topNProtocolsTable.setMouseOverClass("mouseover");
	// the sum of the column widths is smaller than the width set for the table because the browser adds some 
	// additional pixels to each column in the form of padding and borders
	topNProtocolsTable.addColumns("alarmCount", "", "Triggered alarm count", 30, "protocolName", "Name", "", 186, "protocolId", "Port", "", 89, "protocolDescription", "Description", "", 186, "octets", "Bytes", "", 60, "packets", "Packets", "", 68, "hosts", "Hosts", "", 56, "conversations", "Conversations", "", 111);
	topNProtocolsTable.setVisibleColumns("alarmCount", "protocolName", "protocolId", "protocolDescription", "octets", "hosts", "conversations", "packets");
	topNProtocolsTable.setHeaderProperty("bold");
	topNProtocolsTable.setTextAlignments("alarmCount", "right", "octets", "right", "hosts", "right", "conversations", "right", "packets", "right");
	topNProtocolsTable.setLinkUrlCreator(createUrl);
	topNProtocolsTable.setLinkFields("octets", "hosts", "conversations", "packets");
	topNProtocolsTable.setSortable(true);
	topNProtocolsTable.setSortFunction(sortFunction);
	topNProtocolsTable.setRightClickMenuColumns("protocolName", "protocolId", "protocolDescription", "octets", "packets", "hosts", "conversations");
	topNProtocolsTable.setRightClickMenuFunction(showProtocolTableMenu);
	return topNProtocolsTable;
}	


/**
 * Inits the myprotocol table
 */
function initMyProtocolsTable() {
	initedTables["myProtocolsTable"] = true;
	var myProtocolsCell = getElementWithAttribute(WindowManager.instance().getWindow("myprotocolswindow").getHtmlCode(), "windowContentCell", "id", "td");
	var myProtocolsTable = createMyProtocolsTable('myProtocolsTable', '/?resource=Statistics&command=myprotocols&mode=get&fields=index,protocolName,protocolId,protocolDescription,comment,octets,packets,hosts,conversations,alarmCount&format=xml&sid=' + getSid(), myProtocolsCell, 'myprotocolswindow', ProbeTableManager.TABLE_UPDATE_TIME);
	myProtocolsTable.setSize(847, 103);
	myProtocolsTable.setClass("defaultTable summaryTable");
	myProtocolsTable.setHeaderRowClass("summaryTableHeaderRow");
	myProtocolsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	myProtocolsTable.setRowClass("odd");
	myProtocolsTable.toggleAlternateRowClass(true, "even");
	myProtocolsTable.setMouseOverClass("mouseover");
	// the sum of the column widths is smaller than the width set for the table because the browser adds some 
	// additional pixels to each column in the form of padding and borders
	myProtocolsTable.addColumns("alarmCount", "", "Triggered alarm count", 30, "protocolName", "Name", "", 186, "protocolId", "Port", "", 99, "protocolDescription", "Description", "", 186, "comment", "Comment", "", 186, "octets", "Bytes", "", 60, "packets", "Packets", "", 78, "hosts", "Hosts", "", 56, "conversations", "Conversations", "", 111);
	myProtocolsTable.setVisibleColumns("alarmCount", "protocolName", "protocolId", "comment", "octets", "hosts", "conversations", "packets");
	myProtocolsTable.setHeaderProperty("bold");
	myProtocolsTable.setTextAlignments("alarmCount", "right", "octets", "right", "hosts", "right", "conversations", "right", "packets", "right");
	myProtocolsTable.setIdField("protocolId");
	myProtocolsTable.setLinkUrlCreator(createUrl);
	myProtocolsTable.setLinkFields("octets", "hosts", "conversations", "packets");
	myProtocolsTable.setColumnToSort("octets");
	myProtocolsTable.setSortable(true);
	myProtocolsTable.setSortFunction(sortFunction);
	myProtocolsTable.setRightClickMenuColumns("protocolName", "protocolId", "protocolDescription", "comment", "octets", "packets", "hosts", "conversations");
	myProtocolsTable.setRightClickMenuFunction(showDefaultMenu);
	
	// setting the height of a input field in safari does not work. In safari
	// the height of a input field is determined by the font height used. Other
	// browsers allow the height of the input fields to be set. Therefore we can not 
	// set the size of the input fields to fit into the table cells used, so we need to make
	// the table larger for the input fields in safari.
	var deltaHeight = 0;
	if (isSafari()) {
		// each input field is 3 pixels larger than in FF
		deltaHeight = ProbeTableManager.MAX_ROW_NO_SCROLLBAR * 3;
	}
	
	// IE adds 2 pixels off padding to the input fields, one above and one under each input field.
	// to make enough room for the input fields we need to extend the windows height with one px for each row.
	if (isIE()) {
		deltaHeight = ProbeTableManager.MAX_ROW_NO_SCROLLBAR;
	}
	
	if (deltaHeight > 0) {
		myProtocolsTable.resizeTable(deltaHeight, 0);
		var myProtocolsWindow = WindowManager.instance().getWindow("myprotocolswindow");
		myProtocolsWindow.getHtmlCode().style.height = (parseInt(myProtocolsWindow.getHtmlCode().style.height) + deltaHeight) + "px";
	}
	
	return myProtocolsTable;
}	


/**
 * Inits the topnHosts table.
 */
function initTopNHostsTable() {
	initedTables["topNHostsTable"] = true;
	var topNHostsCell = getElementWithAttribute(WindowManager.instance().getWindow("topnhostswindow").getHtmlCode(), "windowContentCell", "id", "td");
	var topNHostsTable = createTopNTable('topNHostsTable', '/?resource=Statistics.nlhost&command=topn&mode=getdata&top=5&sortBy=octets&fields=index,hostName,address,octets,protocols,conversations,packets,alarmCount&format=xml&sid=' + getSid(), topNHostsCell, 'topnhostswindow', ProbeTableManager.TABLE_UPDATE_TIME);
	topNHostsTable.setSize(827, 103);
	topNHostsTable.setClass("defaultTable summaryTable");
	topNHostsTable.setHeaderRowClass("summaryTableHeaderRow");
	topNHostsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	topNHostsTable.setRowClass("odd");
	topNHostsTable.toggleAlternateRowClass(true, "even");
	topNHostsTable.setMouseOverClass("mouseover");
	// the sum of the column widths is smaller than the width set for the table because the browser adds some 
	// additional pixels to each column in the form of padding and borders
	topNHostsTable.addColumns("alarmCount", "", "Triggered alarm count", 30, "hostName", "Host name", "", 186, "address", "Address", "", 110, "octets", "Bytes", "", 118, "packets", "Packets", "", 117, "conversations", "Conversations", "", 115, "protocols", "Protocols", "", 115);
	topNHostsTable.setVisibleColumns("alarmCount", "hostName", "address", "octets", "packets", "protocols", "conversations");
	topNHostsTable.setHeaderProperty("bold");
	topNHostsTable.setTextAlignments("alarmCount", "right", "octets", "right", "octets", "right", "packets", "right", "protocols", "right", "conversations", "right");
	topNHostsTable.setLinkFields("octets", "octets", "packets", "hosts", "conversations", "protocols");
	topNHostsTable.setColumnToSort("octets");
	topNHostsTable.setSortable(true);
	topNHostsTable.setSortFunction(sortFunction);
	topNHostsTable.setLinkUrlCreator(createUrl);
	topNHostsTable.setRightClickMenuColumns("hostName", "address", "octets", "packets", "conversations", "protocols");
	topNHostsTable.setRightClickMenuFunction(showHostTableMenu);
	return topNHostsTable;
}


/**
 * Inits the myHosts table.
 */
function initMyHostsTable() {
	initedTables["myHostsTable"] = true;
	var myHostsCell = getElementWithAttribute(WindowManager.instance().getWindow("myhostswindow").getHtmlCode(), "windowContentCell", "id", "td");
	var myHostsTable = createMyHostsTable('myHostsTable', '/?resource=Statistics&command=myhosts&mode=get&fields=index,hostName,address,comment,inPackets,inOctets,outPackets,outOctets,packets,octets,protocols,conversations,alarmCount&format=xml&sid=' + getSid(), myHostsCell, 'myhostswindow', ProbeTableManager.TABLE_UPDATE_TIME);
	myHostsTable.setSize(847, 103);
	myHostsTable.setClass("defaultTable summaryTable");
	myHostsTable.setHeaderRowClass("summaryTableHeaderRow");
	myHostsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	myHostsTable.setRowClass("odd");
	myHostsTable.toggleAlternateRowClass(true, "even");
	myHostsTable.setMouseOverClass("mouseover");
	// the sum of the column widths is smaller than the width set for the table because the browser adds some 
	// additional pixels to each column in the form of padding and borders
	myHostsTable.addColumns("alarmCount", "", "Triggered alarm count", 30, "hostName", "Host name", "", 159, "address", "IP Address", "", 110, "comment", "Comment", "", 165, "inOctets", "Bytes in", "", 70, "inPackets", "Packets in", "", 80, "outOctets", "Bytes out", "", 80, "outPackets", "Packets out", "", 85, "octets", "Bytes", "", 75, "packets", "Packets", "", 75, "protocols", "Protocols", "", 80, "conversations", "Conversations", "", 112);
	myHostsTable.setVisibleColumns("alarmCount", "hostName", "address", "comment", "inOctets", "outOctets", "protocols", "conversations");
	myHostsTable.setHeaderProperty("bold");
	myHostsTable.setTextAlignments("alarmCount", "right", "inPackets", "right", "inOctets", "right", "outPackets", "right", "outOctets", "right", "octets", "right", "packets", "right", "protocols", "right", "conversations", "right");
	myHostsTable.setIdField("address");
	myHostsTable.setLinkFields("inOctets", "outOctets", "inPackets", "outPackets", "octets", "octets", "packets", "hosts", "conversations", "protocols");
	myHostsTable.setLinkUrlCreator(createUrl);
	myHostsTable.setColumnToSort("inOctets");
	myHostsTable.setSortable(true);
	myHostsTable.setSortFunction(sortFunction);
	myHostsTable.setRightClickMenuColumns("hostName", "address", "comment", "inOctets", "inPackets", "outOctets", "outPackets", "octets", "packets", "protocols", "conversations");
	myHostsTable.setRightClickMenuFunction(showDefaultMenu);
	
	// setting the height of a input field in safari does not work. In safari
	// the height of a input field is determined by the font height used. Other
	// browsers allow the height of the input fields to be set. Therefore we can not 
	// set the size of the input fields to fit into the table cells used, so we need to make
	// the table larger for the input fields in safari.
	var deltaHeight = 0;
	if (isSafari()) {
		// each input field is 3 pixels larger than in FF
		deltaHeight = ProbeTableManager.MAX_ROW_NO_SCROLLBAR * 3;
	}
	
	// IE adds 2 pixels off padding to the input fields, one above and one under each input field.
	// to make enough room for the input fields we need to extend the windows height with one px for each row.
	if (isIE()) {
		deltaHeight = ProbeTableManager.MAX_ROW_NO_SCROLLBAR;
	}
	
	if (deltaHeight > 0) {
		myHostsTable.resizeTable(deltaHeight, 0);
		var myHostsWindow = WindowManager.instance().getWindow("myhostswindow");
		myHostsWindow.getHtmlCode().style.height = (parseInt(myHostsWindow.getHtmlCode().style.height) + deltaHeight) + "px";
	}
	return myHostsTable;
}


/**
 * Inits the topnHostListeners table.
 */
function initTopNHostListnersTable() {
	initedTables["topNHostListenersTable"] = true;
	var topNHostsListenersCell = getElementWithAttribute(WindowManager.instance().getWindow("topnhostlistenerswindow").getHtmlCode(), "windowContentCell", "id", "td");
	var topNHostListenersTable = createTopNTable('topNHostListenersTable', '/?resource=Statistics.nlhost&command=topn&mode=getdata&top=5&sortBy=inOctets&fields=index,hostName,address,inOctets,protocols,conversations,inPackets,alarmCount&format=xml&sid=' + getSid(), topNHostsListenersCell, 'topnhostlistenerswindow', ProbeTableManager.TABLE_UPDATE_TIME);
	topNHostListenersTable.setSize(827, 103);
	topNHostListenersTable.setClass("defaultTable summaryTable");
	topNHostListenersTable.setHeaderRowClass("summaryTableHeaderRow");
	topNHostListenersTable.setHeaderTableClass("defaultTable summaryTableHeader");
	topNHostListenersTable.setRowClass("odd");
	topNHostListenersTable.toggleAlternateRowClass(true, "even");
	topNHostListenersTable.setMouseOverClass("mouseover");
	// the sum of the column widths is smaller than the width set for the table because the browser adds some 
	// additional pixels to each column in the form of padding and borders
	topNHostListenersTable.addColumns("alarmCount", "", "Triggerd alarm count", 30, "hostName", "Host name", "", 186, "address", "Address", "", 110, "inOctets", "Bytes", "", 118, "inPackets", "Packets", "", 117, "conversations", "Conversations", "", 115, "protocols", "Protocols", "", 115);
	topNHostListenersTable.setVisibleColumns("alarmCount", "hostName", "address", "inOctets", "inPackets", "protocols", "conversations");
	topNHostListenersTable.setHeaderProperty("bold");
	topNHostListenersTable.setTextAlignments("alarmCount", "right", "inOctets", "right", "inPackets", "right", "protocols", "right", "conversations", "right");
	topNHostListenersTable.setLinkFields("inOctets", "protocols", "conversations", "inPackets");
	topNHostListenersTable.setColumnToSort("inOctets");
	topNHostListenersTable.setSortable(true);
	topNHostListenersTable.setSortFunction(sortFunction);
	topNHostListenersTable.setLinkUrlCreator(createUrl);
	topNHostListenersTable.setRightClickMenuColumns("hostName", "address", "inOctets", "inPackets", "conversations", "protocols");
	topNHostListenersTable.setRightClickMenuFunction(showHostTableMenu);
	return topNHostListenersTable;
}


/**
 * Inits the topnHostTalkers table.
 */
function initTopNHostTalkersTable() {
	initedTables["topNHostTalkersTable"] = true;
	var topNHostsTalkersCell = getElementWithAttribute(WindowManager.instance().getWindow("topnhosttalkerswindow").getHtmlCode(), "windowContentCell", "id", "td");	
	var topNHostTalkersTable = createTopNTable('topNHostTalkersTable', '/?resource=Statistics.nlhost&command=topn&mode=getdata&top=5&sortBy=outOctets&fields=index,hostName,address,outOctets,protocols,conversations,outPackets,alarmCount&format=xml&sid=' + getSid(), topNHostsTalkersCell, 'topnhosttalkerswindow', ProbeTableManager.TABLE_UPDATE_TIME);
	topNHostTalkersTable.setSize(827, 103);
	topNHostTalkersTable.setClass("defaultTable summaryTable");
	topNHostTalkersTable.setHeaderRowClass("summaryTableHeaderRow");
	topNHostTalkersTable.setHeaderTableClass("defaultTable summaryTableHeader");
	topNHostTalkersTable.setRowClass("odd");
	topNHostTalkersTable.toggleAlternateRowClass(true, "even");
	topNHostTalkersTable.setMouseOverClass("mouseover");
	// the sum of the column widths is smaller than the width set for the table because the browser adds some 
	// additional pixels to each column in the form of padding and borders
	topNHostTalkersTable.addColumns("alarmCount", "", "Triggered alarm count", 30, "hostName", "Host name", "", 186, "address", "Address", "", 110, "outOctets", "Bytes", "", 118, "outPackets", "Packets", "", 117, "conversations", "Conversations", "", 115, "protocols", "Protocols", "", 115);
	topNHostTalkersTable.setVisibleColumns("alarmCount", "hostName", "address", "outOctets", "outPackets", "protocols", "conversations");
	topNHostTalkersTable.setHeaderProperty("bold");
	topNHostTalkersTable.setTextAlignments("alarmCount", "right", "outOctets", "right", "outPackets", "right", "protocols", "right", "conversations", "right");
	topNHostTalkersTable.setLinkFields("octets", "outOctets", "protocols", "conversations", "outPackets");
	topNHostTalkersTable.setColumnToSort("outOctets");
	topNHostTalkersTable.setSortable(true);
	topNHostTalkersTable.setSortFunction(sortFunction);
	topNHostTalkersTable.setLinkUrlCreator(createUrl);
	topNHostTalkersTable.setRightClickMenuColumns("hostName", "address", "outOctets", "outPackets", "conversations", "protocols");
	topNHostTalkersTable.setRightClickMenuFunction(showHostTableMenu);
	return topNHostTalkersTable;
}


/**
 * Inits the topnConversations table.
 */
function initTopNConversationsTable() {
	initedTables["topNConversationsTable"] = true;
	var topNConversationCell = getElementWithAttribute(WindowManager.instance().getWindow("topnconversationwindow").getHtmlCode(), "windowContentCell", "id", "td");	
	var topNConversationsTable = createTopNTable('topNConversationsTable', '/?resource=Statistics.nlmatrix&command=topn&mode=getdata&top=5&sortBy=octets&fields=index,srcHostName,srcAddress,dstHostName,dstAddress,octets,protocols,packets,alarmCount&format=xml&sid=' + getSid(), topNConversationCell, 'topnconversationwindow', ProbeTableManager.TABLE_UPDATE_TIME);
	topNConversationsTable.setSize(867, 103);
	topNConversationsTable.setClass("defaultTable summaryTable");
	topNConversationsTable.setHeaderRowClass("summaryTableHeaderRow");
	topNConversationsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	topNConversationsTable.setRowClass("odd");
	topNConversationsTable.toggleAlternateRowClass(true, "even");
	topNConversationsTable.setMouseOverClass("mouseover");
	// the sum of the column widths is smaller than the width set for the table because the browser adds some 
	// additional pixels to each column in the form of padding and borders
	topNConversationsTable.addColumns("alarmCount", "", "Triggered alarm count", 30, "srcHostName", "Source host", "", 156, "srcAddress", "Source Address", "", 120, "dstHostName", "Destination host", "", 156, "dstAddress", "Destination Address", "", 150, "octets", "Bytes", "", 62, "packets", "Packets", "", 71, "protocols", "Protocols", "", 81);
	topNConversationsTable.setVisibleColumns("alarmCount", "srcHostName", "srcAddress", "dstHostName", "dstAddress", "octets", "packets", "protocols");
	topNConversationsTable.setHeaderProperty("bold");
	topNConversationsTable.setTextAlignments("alarmCount", "right", "octets", "right", "packets", "right", "protocols", "right");
	topNConversationsTable.setLinkFields("octets", "packets", "protocols");
	topNConversationsTable.setColumnToSort("octets");
	topNConversationsTable.setSortable(true);
	topNConversationsTable.setSortFunction(sortFunction);
	topNConversationsTable.setLinkUrlCreator(createUrl);
	topNConversationsTable.setRightClickMenuColumns("srcHostName", "srcAddress", "dstHostName", "dstAddress", "octets", "packets", "protocols");
	topNConversationsTable.setRightClickMenuFunction(showConversationTableMenu);
	return topNConversationsTable;
}


/**
 * Inits the myConversations table.
 */
function initMyConversationsTable() {
	initedTables["myConversationsTable"] = true;
	var myConversationCell = getElementWithAttribute(WindowManager.instance().getWindow("myconversationwindow").getHtmlCode(), "windowContentCell", "id", "td");	
	var myConversationsTable = createMyConversationsTable('myConversationsTable', '/?resource=Statistics&command=myconversations&mode=get&fields=index,srcHostName,srcAddress,dstHostName,dstAddress,octets,packets,protocols,comment,alarmCount&format=xml&sid=' + getSid(), myConversationCell, 'myconversationwindow', ProbeTableManager.TABLE_UPDATE_TIME);
	myConversationsTable.setSize(991, 103);
	myConversationsTable.setClass("defaultTable summaryTable");
	myConversationsTable.setHeaderRowClass("summaryTableHeaderRow");
	myConversationsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	myConversationsTable.setRowClass("odd");
	myConversationsTable.toggleAlternateRowClass(true, "even");
	myConversationsTable.setMouseOverClass("mouseover");
	// the sum of the column widths is smaller than the width set for the table because the browser adds some 
	// additional pixels to each column in the form of padding and borders
	myConversationsTable.addColumns("alarmCount", "", "Triggered alarm count", 30, "srcHostName", "Source host", "", 166, "srcAddress", "Source IP", "", 107, "dstHostName", "Destination host", "", 165, "dstAddress", "Dest IP", "", 107, "comment", "Comment", "", 151, "octets", "Bytes", "", 70, "packets", "Packets", "", 70, "protocols", "Protocols", "", 80);
	myConversationsTable.setVisibleColumns("alarmCount", "srcHostName", "srcAddress", "dstHostName", "dstAddress", "octets", "comment", "packets", "protocols");
	myConversationsTable.setHeaderProperty("bold");
	myConversationsTable.setTextAlignments("alarmCount", "right", "octets", "right", "packets", "right", "protocols", "right");
	myConversationsTable.setIdFields("srcAddress", "dstAddress");
	myConversationsTable.setColumnToSort("octets");
	myConversationsTable.setSortable(true);
	myConversationsTable.setSortFunction(sortFunction);
	myConversationsTable.setLinkFields("octets", "packets", "protocols");
	myConversationsTable.setLinkUrlCreator(createUrl);
	myConversationsTable.setRightClickMenuColumns("srcHostName", "srcAddress", "dstHostName", "dstAddress", "comment", "octets", "packets", "protocols");
	myConversationsTable.setRightClickMenuFunction(showDefaultMenu);
	
	
	// setting the height of a input field in safari does not work. In safari
	// the height of a input field is determined by the font height used. Other
	// browsers allow the height of the input fields to be set. Therefore we can not 
	// set the size of the input fields to fit into the table cells used, so we need to make
	// the table larger for the input fields in safari.
	var deltaHeight = 0;
	if (isSafari()) {
		// each input field is 3 pixels larger than in FF
		deltaHeight = ProbeTableManager.MAX_ROW_NO_SCROLLBAR * 3;
	}
	
	// IE adds 2 pixels off padding to the input fields, one above and one under each input field.
	// to make enough room for the input fields we need to extend the windows height with one px for each row.
	if (isIE()) {
		deltaHeight = ProbeTableManager.MAX_ROW_NO_SCROLLBAR;
	}
	
	if (deltaHeight > 0) {
		myConversationsTable.resizeTable(deltaHeight, 0);
		var myConversationWindow = WindowManager.instance().getWindow("myconversationwindow");
		myConversationWindow.getHtmlCode().style.height = (parseInt(myConversationWindow.getHtmlCode().style.height) + deltaHeight) + "px";
	}
	return myConversationsTable;
}	


/**
 * Inits the myProtocolsPerHost table.
 */
function initMyProtocolsPerHostTable() {
	initedTables["myProtocolsPerHostTable"] = true;
	var myProtocolsPerHostCell = getElementWithAttribute(WindowManager.instance().getWindow("myprotocolsperhostwindow").getHtmlCode(), "windowContentCell", "id", "td");
	var myProtocolsPerHostTable = createMyProtocolsPerHostTable('myProtocolsPerHostTable', '/?resource=Statistics&command=myprotocolsperhost&mode=get&fields=index,hostName,address,protocolName,protocolId,comment,inOctets,inPackets,outOctets,outPackets,alarmCount&format=xml&sid=' + getSid(), myProtocolsPerHostCell, 'myprotocolsperhostwindow', ProbeTableManager.TABLE_UPDATE_TIME);
	myProtocolsPerHostTable.setSize(888, 103);
	myProtocolsPerHostTable.setClass("defaultTable summaryTable");
	myProtocolsPerHostTable.setHeaderRowClass("summaryTableHeaderRow");
	myProtocolsPerHostTable.setHeaderTableClass("defaultTable summaryTableHeader");
	myProtocolsPerHostTable.setRowClass("odd");
	myProtocolsPerHostTable.toggleAlternateRowClass(true, "even");
	myProtocolsPerHostTable.setMouseOverClass("mouseover");
	myProtocolsPerHostTable.addColumns("alarmCount", "", "Triggered alarm count", 30, "hostName", "Host", "", 140, "address", "IP address", "", 107, "protocolName", "Protocol", "", 140, "protocolId", "Port", "", 99, "comment", "Comment", "", 182, "inOctets", "Bytes in", "", 70, "inPackets", "Packets in", "", 68, "outOctets", "Bytes out", "", 80, "outPackets", "Packets out", "", 60);
	myProtocolsPerHostTable.setVisibleColumns("alarmCount", "hostName", "address", "protocolName", "protocolId", "comment", "inOctets", "outOctets");
	myProtocolsPerHostTable.setHeaderProperty("bold");
	myProtocolsPerHostTable.setTextAlignments("alarmCount", "right", "inPackets", "right", "inOctets", "right", "outOctets", "right", "outPackets", "right");
	myProtocolsPerHostTable.setColumnToSort("inOctets");
	myProtocolsPerHostTable.setSortable(true);
	myProtocolsPerHostTable.setSortFunction(sortFunction);
	myProtocolsPerHostTable.setIdFields("address", "protocolId");
	myProtocolsPerHostTable.setLinkFields("inPackets", "inOctets", "outOctets", "outPackets");
	myProtocolsPerHostTable.setLinkUrlCreator(createUrl);
	myProtocolsPerHostTable.setRightClickMenuColumns("hostName", "address", "protocolName", "protocolId", "comment", "inOctets", "inPackets", "outOctets", "outPackets");
	myProtocolsPerHostTable.setRightClickMenuFunction(showDefaultMenu);
	
	// setting the height of a input field in safari does not work. In safari
	// the height of a input field is determined by the font height used. Other
	// browsers allow the height of the input fields to be set. Therefore we can not 
	// set the size of the input fields to fit into the table cells used, so we need to make
	// the table larger for the input fields in safari.
	var deltaHeight = 0;
	if (isSafari()) {
		// each input field is 3 pixels larger than in FF
		deltaHeight = ProbeTableManager.MAX_ROW_NO_SCROLLBAR * 3;
	}
	
	// IE adds 2 pixels off padding to the input fields, one above and one under each input field.
	// to make enough room for the input fields we need to extend the windows height with one px for each row.
	if (isIE()) {
		deltaHeight = ProbeTableManager.MAX_ROW_NO_SCROLLBAR;
	}
	
	if (deltaHeight > 0) {
		myProtocolsPerHostTable.resizeTable(deltaHeight, 0);
		var myProtocolsPerHostWindow = WindowManager.instance().getWindow("myprotocolsperhostwindow", "myProtocolsPerHostTable");
		myProtocolsPerHostWindow.getHtmlCode().style.height = (parseInt(myProtocolsPerHostWindow.getHtmlCode().style.height) + deltaHeight) + "px";
	}
	
	return myProtocolsPerHostTable;
}


/**
 * Inits the profiles table.
 */
function initProfilesTable() {
	initedTables["profilesTable"] = true;
	var myProfileCell = getElementWithAttribute(WindowManager.instance().getWindow("profilewindow").getHtmlCode(), "windowContentCell", "id", "td");
	var profileTable = createProfileTable("profilesTable", "/?resource=profilemanager&mode=get&command=allprofiles&sid=" + getSid(), myProfileCell, "profilewindow");
	profileTable.setSize(592, 18);
	profileTable.setClass("defaultTable summaryTable");
	profileTable.setHeaderRowClass("summaryTableHeaderRow");
	profileTable.setHeaderTableClass("defaultTable summaryTableHeader");
	profileTable.setRowClass("odd");
	profileTable.toggleAlternateRowClass(true, "even");
	profileTable.addColumns("name", "Name", "", 291, "description", "Description", "", 291);
	profileTable.setVisibleColumns("name", "description");
	profileTable.setHeaderProperty("bold");
	profileTable.setEntryTagName("profile");
	profileTable.setIdField("name");
	return profileTable;
}


/**
 * Inits the trigger values table.
 */
function initTriggerValuesTable() {
	initedTables["triggervaluestable"] = true;
	var triggerValuesCell = getElementWithAttribute(WindowManager.instance().getWindow("triggervalueswindow").getHtmlCode(), "windowContentCell", "id", "td");
	var triggerValuesTable = createTriggerValuesTable('triggervaluestable', '', triggerValuesCell, 'triggervalueswindow', 0);
	triggerValuesTable.setSize(411, 103);
	triggerValuesTable.setClass("defaultTable summaryTable");
	triggerValuesTable.setHeaderRowClass("summaryTableHeaderRow");
	triggerValuesTable.setHeaderTableClass("defaultTable summaryTableHeader");
	triggerValuesTable.setRowClass("odd");
	triggerValuesTable.setMouseOverClass("mouseover");
	triggerValuesTable.toggleAlternateRowClass(true, "even");
	triggerValuesTable.addColumns("triggerTime", "Trigger time", "", 200, "triggerValue", "Trigger Value", "", 200, "maxEmails", "", "", 0, "maxEmailTime", "", "", 0);
	triggerValuesTable.setVisibleColumns("triggerTime", "triggerValue");
	triggerValuesTable.setHeaderProperty("bold");
	triggerValuesTable.setColumnToSort("triggerTime");
	triggerValuesTable.setSortable(true);
	triggerValuesTable.setSortFunction(sortFunction);
	return triggerValuesTable;
}


/**
 * Inits the email alarm actions table.
 */
function initEmailAlarmActionsTable() {
	initedTables["emailAlarmActionsTable"] = true;
	var alarmActionsCell = getElementWithAttribute(WindowManager.instance().getWindow("alarmactionswindow").getHtmlCode(), "emailAlarmTable", "name", "div");
	var alarmActionsTable = createEmailActionTable('emailAlarmActionsTable', LOAD_EMAIL_ACTION_BASE_URL + "sid=" + getSid() + "&alarmId=&", alarmActionsCell, 'alarmactionswindow', ProbeTableManager.TABLE_UPDATE_TIME);
	alarmActionsTable.setSize(711, 103);
	alarmActionsTable.setClass("defaultTable summaryTable");
	alarmActionsTable.setHeaderRowClass("summaryTableHeaderRow");
	alarmActionsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	alarmActionsTable.setRowClass("odd");
	alarmActionsTable.setMouseOverClass("mouseover");
	alarmActionsTable.setMarkClass("mouseover");
	alarmActionsTable.toggleAlternateRowClass(true, "even");
	alarmActionsTable.addColumns("to", "Email", "", 350, "subject", "Subject", "", 245, "count", "Sent count", "", 100, "id", "", "", 0, "maxEmails", "", "", 0, "maxEmailTime", "", "", 0);
	alarmActionsTable.setVisibleColumns("to", "Subject", "count");
	alarmActionsTable.setIdField("id");
	alarmActionsTable.setHeaderProperty("bold");
	alarmActionsTable.setEntryTagName("action");
	alarmActionsTable.setColumnToSort("to");
	alarmActionsTable.setSortable(true);
	alarmActionsTable.setSortFunction(sortFunction);
	alarmActionsTable.setRightClickMenuColumns("to", "subject", "count");
	alarmActionsTable.setRightClickMenuFunction(showAlarmActionMenu);
	return alarmActionsTable;
}


/**
 * Inits the program alarm actions table. 
 */
function initProgramAlarmActionsTable() {
	initedTables["programAlarmActionsTable"] = true;
	var alarmActionsCell = getElementWithAttribute(WindowManager.instance().getWindow("alarmactionswindow").getHtmlCode(), "programAlarmTable", "name", "div");
	var alarmActionsTable = createProgramActionTable('programAlarmActionsTable', LOAD_PROGRAM_ACTION_BASE_URL + "sid=" + getSid() + "&alarmId=&", alarmActionsCell, 'alarmactionswindow', ProbeTableManager.TABLE_UPDATE_TIME);
	alarmActionsTable.setSize(711, 103);
	alarmActionsTable.setClass("defaultTable summaryTable");
	alarmActionsTable.setHeaderRowClass("summaryTableHeaderRow");
	alarmActionsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	alarmActionsTable.setRowClass("odd");
	alarmActionsTable.setMouseOverClass("mouseover");
	alarmActionsTable.setMarkClass("mouseover");
	alarmActionsTable.toggleAlternateRowClass(true, "even");
	alarmActionsTable.addColumns("programName", "Name", "Name", 150, "parameters", "Parameters", "Parameters", 300, "description", "Description", "Description", 245, "id", "", "", 0);
	alarmActionsTable.setVisibleColumns("programName", "parameters", "description");
	alarmActionsTable.setIdField("id");
	alarmActionsTable.setHeaderProperty("bold");
	alarmActionsTable.setEntryTagName("action");
	alarmActionsTable.setColumnToSort("programName");
	alarmActionsTable.setSortable(true);
	alarmActionsTable.setSortFunction(sortFunction);
	alarmActionsTable.setRightClickMenuColumns("programName", "parameters", "description");
	alarmActionsTable.setRightClickMenuFunction(showAlarmActionMenu);
	return alarmActionsTable;
}



/**
 * Inits the email alarm actions library table.
 */
function initEmailAlarmActionsLibraryTable() {
	initedTables["emailAlarmActionLibraryTable"] = true;
	var alarmActionsLibraryCell = getElementWithAttribute(WindowManager.instance().getWindow("alarmactionslibrarywindow").getHtmlCode(), "emailAlarmTable", "name", "div");
	var alarmActionsLibraryTable = createEmailActionTable('emailAlarmActionLibraryTable', LOAD_LIBRARY_EMAIL_ACTION_BASE_URL + "sid=" + getSid(), alarmActionsLibraryCell, 'alarmactionslibrarywindow', ProbeTableManager.TABLE_UPDATE_TIME);
	alarmActionsLibraryTable.setSize(711, 103);
	alarmActionsLibraryTable.setClass("defaultTable summaryTable");
	alarmActionsLibraryTable.setHeaderRowClass("summaryTableHeaderRow");
	alarmActionsLibraryTable.setHeaderTableClass("defaultTable summaryTableHeader");
	alarmActionsLibraryTable.setRowClass("odd");
	alarmActionsLibraryTable.setMouseOverClass("mouseover");
	alarmActionsLibraryTable.setMarkClass("mouseover");
	alarmActionsLibraryTable.toggleAlternateRowClass(true, "even");
	alarmActionsLibraryTable.addColumns("to", "Email", "", 400, "subject", "Subject", "", 300, "id", "", "", 0, "maxEmails", "", "", 0, "maxEmailTime", "", "", 0);
	alarmActionsLibraryTable.setVisibleColumns("to", "Subject", "count");
	alarmActionsLibraryTable.setIdField("id");
	alarmActionsLibraryTable.setHeaderProperty("bold");
	alarmActionsLibraryTable.setEntryTagName("action");
	alarmActionsLibraryTable.setColumnToSort("to");
	alarmActionsLibraryTable.setSortable(true);
	alarmActionsLibraryTable.setSortFunction(sortFunction);
	alarmActionsLibraryTable.setRightClickMenuColumns("to", "subject");
	alarmActionsLibraryTable.setRightClickMenuFunction(showAlarmActionLibraryMenu);
	return alarmActionsLibraryTable;
}


/**
 * Inits the prorgram actions library table.
 */
function initProgramAlarmActionsLibraryTable() {
	initedTables["programAlarmActionLibraryTable"] = true;
	var alarmActionsLibraryCell = getElementWithAttribute(WindowManager.instance().getWindow("alarmactionslibrarywindow").getHtmlCode(), "programAlarmTable", "name", "div");
	var alarmActionsLibraryTable = createProgramActionTable('programAlarmActionLibraryTable', LOAD_LIBRARY_PROGRAM_ACTION_BASE_URL + "sid=" + getSid(), alarmActionsLibraryCell, 'alarmactionslibrarywindow', ProbeTableManager.TABLE_UPDATE_TIME);
	alarmActionsLibraryTable.setSize(711, 103);
	alarmActionsLibraryTable.setClass("defaultTable summaryTable");
	alarmActionsLibraryTable.setHeaderRowClass("summaryTableHeaderRow");
	alarmActionsLibraryTable.setHeaderTableClass("defaultTable summaryTableHeader");
	alarmActionsLibraryTable.setRowClass("odd");
	alarmActionsLibraryTable.setMouseOverClass("mouseover");
	alarmActionsLibraryTable.setMarkClass("mouseover");
	alarmActionsLibraryTable.toggleAlternateRowClass(true, "even");
	alarmActionsLibraryTable.addColumns("programName", "Name", "Name", 150, "parameters", "Parameters", "Parameters", 300, "description", "Description", "Description", 245, "id", "", "", 0);
	alarmActionsLibraryTable.setVisibleColumns("programName", "parameters", "description");
	alarmActionsLibraryTable.setIdField("id");
	alarmActionsLibraryTable.setHeaderProperty("bold");
	alarmActionsLibraryTable.setEntryTagName("action");
	alarmActionsLibraryTable.setColumnToSort("programName");
	alarmActionsLibraryTable.setSortable(true);
	alarmActionsLibraryTable.setSortFunction(sortFunction);
	alarmActionsLibraryTable.setRightClickMenuColumns("programName", "parameters", "description");
	alarmActionsLibraryTable.setRightClickMenuFunction(showAlarmActionLibraryMenu);
	return alarmActionsLibraryTable;
}


/**
 * Inits the all defined alarms table.
 */
function initAllDefinedAlarmsTable() {
	initedTables["allDefinedAlarmsTable"] = true;
	var allDefinedAlarmsCell = getElementWithAttribute(WindowManager.instance().getWindow("alldefinedalarmswindow").getHtmlCode(), "windowContentCell", "id", "td");
	var allDefinedAlarmsTable = createAlarmTable("allDefinedAlarmsTable", "/?resource=AlarmManager&mode=get&command=defined&format=xml&sid=" + getSid(), allDefinedAlarmsCell, "alldefinedalarmswindow", ProbeTableManager.TABLE_UPDATE_TIME);
	allDefinedAlarmsTable.setSize(1046, 103);
	allDefinedAlarmsTable.setClass("defaultTable summaryTable");
	allDefinedAlarmsTable.setHeaderRowClass("summaryTableHeaderRow");
	allDefinedAlarmsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	allDefinedAlarmsTable.setRowClass("odd");
	allDefinedAlarmsTable.setMouseOverClass("mouseover");
	allDefinedAlarmsTable.toggleAlternateRowClass(true, "even");
	allDefinedAlarmsTable.addColumns("table", "Entry type", "", 130,  "entry", "Entry", "", 317,"counterId", "Field", "", 94, "description", "Description", "", 188, "severity", "Severity", "", 94, "absoluteAlarm", "Type", "", 94, "triggerType", "Trigger type", "", 94, "upperTriggerValue", "Upper trigger", "", 94, "upperUnit", "upper unit", "", 94, "lowerTriggerValue", "Lower Trigger", "", 94, "lowerUnit", "lower unit", "", 94, "samplingTime", "Sampling time", "", 94, "visibility", "Visibility", "", 94);
	allDefinedAlarmsTable.setVisibleColumns("table", "entry", "counterId", "description", "severity", "triggerType", "absoluteAlarm");
	allDefinedAlarmsTable.setHeaderProperty("bold");
	allDefinedAlarmsTable.setEntryTagName("defined");
	allDefinedAlarmsTable.setIdField("id");
	allDefinedAlarmsTable.setSortable(true);
	allDefinedAlarmsTable.setSortFunction(sortFunction);
	allDefinedAlarmsTable.setRightClickMenuColumns("table", "entry", "counterId", "description", "severity", "absoluteAlarm", "triggerType", "upperTriggerValue", "upperUnit", "lowerTriggerValue", "lowerUnit", "samplingTime", "visibility");
	allDefinedAlarmsTable.setRightClickMenuFunction(showDefinedAlarmsMenu);
	return allDefinedAlarmsTable;
}


/**
 * Inits the summary defined alarms table.
 */
function initSummaryDefinedAlarmsTable() {
	initedTables["summaryDefinedAlarmsTable"] = true;
	var summaryDefinedAlarmsCell = getElementWithAttribute(WindowManager.instance().getWindow("summarydefinedalarmswindow").getHtmlCode(), "windowContentCell", "id", "td");
	var summaryDefinedAlarmsTable = createAlarmTable("summaryDefinedAlarmsTable", "/?resource=AlarmManager&mode=get&table=summary&command=defined&format=xml&sid=" + getSid(), summaryDefinedAlarmsCell, "summarydefinedalarmswindow", ProbeTableManager.TABLE_UPDATE_TIME);
	summaryDefinedAlarmsTable.setSize(589, 103);
	summaryDefinedAlarmsTable.setClass("defaultTable summaryTable");
	summaryDefinedAlarmsTable.setHeaderRowClass("summaryTableHeaderRow");
	summaryDefinedAlarmsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	summaryDefinedAlarmsTable.setRowClass("odd");
	summaryDefinedAlarmsTable.setMouseOverClass("mouseover");
	summaryDefinedAlarmsTable.toggleAlternateRowClass(true, "even");
	summaryDefinedAlarmsTable.addColumns("counterId", "Field", "", 94, "description", "Description", "", 188, "severity", "Severity", "", 94, "absoluteAlarm", "Type", "", 94, "triggerType", "Trigger type", "", 94, "upperTriggerValue", "Upper trigger", "", 94, "upperUnit", "upper unit", "", 94, "lowerTriggerValue", "Lower Trigger", "", 94, "lowerUnit", "lower unit", "", 94, "samplingTime", "Sampling time", "", 94, "visibility", "Visibility", "", 94);
	summaryDefinedAlarmsTable.setVisibleColumns("counterId", "description", "severity", "triggerType", "absoluteAlarm");
	summaryDefinedAlarmsTable.setHeaderProperty("bold");
	summaryDefinedAlarmsTable.setEntryTagName("defined");
	summaryDefinedAlarmsTable.setIdField("id");
	summaryDefinedAlarmsTable.setSortable(true);
	summaryDefinedAlarmsTable.setSortFunction(sortFunction);
	summaryDefinedAlarmsTable.setRightClickMenuColumns("counterId", "description", "severity", "absoluteAlarm", "triggerType", "upperTriggerValue", "upperUnit", "lowerTriggerValue", "lowerUnit", "samplingTime", "visibility");
	summaryDefinedAlarmsTable.setRightClickMenuFunction(showDefinedAlarmsMenu);
	return summaryDefinedAlarmsTable;
}


/**
 * Inits the protocol defined alarms table.
 */
function initProtocolDefinedAlarmsTable() {
	initedTables["protocolDefinedAlarmsTable"] = true;
	var protocolDefinedAlarmsCell = getElementWithAttribute(WindowManager.instance().getWindow("protocoldefinedalarmswindow").getHtmlCode(), "windowContentCell", "id", "td");
	var protocolDefinedAlarmsTable = createAlarmTable("protocolDefinedAlarmsTable", "/?resource=AlarmManager&mode=get&table=protocol&command=defined&format=xml&sid=" + getSid(), protocolDefinedAlarmsCell, "protocoldefinedalarmswindow", ProbeTableManager.TABLE_UPDATE_TIME);
	protocolDefinedAlarmsTable.setSize(693, 103);
	protocolDefinedAlarmsTable.setClass("defaultTable summaryTable");
	protocolDefinedAlarmsTable.setHeaderRowClass("summaryTableHeaderRow");
	protocolDefinedAlarmsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	protocolDefinedAlarmsTable.setRowClass("odd");
	protocolDefinedAlarmsTable.setMouseOverClass("mouseover");
	protocolDefinedAlarmsTable.toggleAlternateRowClass(true, "even");
	protocolDefinedAlarmsTable.addColumns("pid", "Entry", "", 99, "counterId", "Field", "", 94, "description", "Description", "", 188, "severity", "Severity", "", 94, "absoluteAlarm", "Type", "", 94, "triggerType", "Trigger type", "", 94, "upperTriggerValue", "Upper trigger", "", 94, "upperUnit", "upper unit", "", 94, "lowerTriggerValue", "Lower Trigger", "", 94, "lowerUnit", "lower unit", "", 94, "samplingTime", "Sampling time", "", 94, "visibility", "Visibility", "", 94);
	protocolDefinedAlarmsTable.setVisibleColumns("src", "pid", "counterId", "description", "severity", "absoluteAlarm", "triggerType");
	protocolDefinedAlarmsTable.setHeaderProperty("bold");
	protocolDefinedAlarmsTable.setEntryTagName("defined");
	protocolDefinedAlarmsTable.setIdField("id");
	protocolDefinedAlarmsTable.setSortable(true);
	protocolDefinedAlarmsTable.setSortFunction(sortFunction);
	protocolDefinedAlarmsTable.setRightClickMenuColumns("pid", "counterId", "description", "severity", "absoluteAlarm", "triggerType", "upperTriggerValue", "upperUnit", "lowerTriggerValue", "lowerUnit", "samplingTime", "visibility");
	protocolDefinedAlarmsTable.setRightClickMenuFunction(showDefinedAlarmsMenu);
	return protocolDefinedAlarmsTable;
}


/**
 * Inits the host defind alarms table.
 */
function initHostsDefinedAlarmTable() {
	initedTables["hostsDefinedAlarmsTable"] = true;
	var hostsDefinedAlarmsCell = getElementWithAttribute(WindowManager.instance().getWindow("hostsdefinedalarmswindow").getHtmlCode(), "windowContentCell", "id", "td");
	var hostsDefinedAlarmsTable = createAlarmTable("hostsDefinedAlarmsTable", "/?resource=AlarmManager&mode=get&table=nlhost&command=defined&format=xml&sid=" + getSid(), hostsDefinedAlarmsCell, "hostsdefinedalarmswindow", ProbeTableManager.TABLE_UPDATE_TIME);
	hostsDefinedAlarmsTable.setSize(688, 103);
	hostsDefinedAlarmsTable.setClass("defaultTable summaryTable");
	hostsDefinedAlarmsTable.setHeaderRowClass("summaryTableHeaderRow");
	hostsDefinedAlarmsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	hostsDefinedAlarmsTable.setRowClass("odd");
	hostsDefinedAlarmsTable.setMouseOverClass("mouseover");
	hostsDefinedAlarmsTable.toggleAlternateRowClass(true, "even");
	hostsDefinedAlarmsTable.addColumns("src", "Entry", "", 94, "counterId", "Field", "", 94, "description", "Description", "", 188, "severity", "Severity", "", 94, "absoluteAlarm", "Type", "", 94, "triggerType", "Trigger type", "", 94, "upperTriggerValue", "Upper trigger", "", 94, "upperUnit", "upper unit", "", 94, "lowerTriggerValue", "Lower Trigger", "", 94, "lowerUnit", "lower unit", "", 94, "samplingTime", "Sampling time", "", 94, "visibility", "Visibility", "", 94);
	hostsDefinedAlarmsTable.setVisibleColumns("src", "dst", "pid", "counterId", "description", "severity", "absoluteAlarm", "triggerType");
	hostsDefinedAlarmsTable.setHeaderProperty("bold");
	hostsDefinedAlarmsTable.setEntryTagName("defined");
	hostsDefinedAlarmsTable.setIdField("id");
	hostsDefinedAlarmsTable.setSortable(true);
	hostsDefinedAlarmsTable.setSortFunction(sortFunction);
	hostsDefinedAlarmsTable.setRightClickMenuColumns("src", "counterId", "description", "severity", "absoluteAlarm", "triggerType", "upperTriggerValue", "upperUnit", "lowerTriggerValue", "lowerUnit", "samplingTime", "visibility");
	hostsDefinedAlarmsTable.setRightClickMenuFunction(showDefinedAlarmsMenu);
	return hostsDefinedAlarmsTable;
}


/**
 * Inits the conversationstDefinedAlarms table.
 */
function initConversationsDefinedAlarmTable() {
	initedTables["conversationsDefinedAlarmsTable"] = true;
	var conversationsDefinedAlarmsCell = getElementWithAttribute(WindowManager.instance().getWindow("conversationsdefinedalarmswindow").getHtmlCode(), "windowContentCell", "id", "td");
	var conversationsDefinedAlarmsTable = createAlarmTable("conversationsDefinedAlarmsTable", "/?resource=AlarmManager&mode=get&table=nlmatrix&command=defined&format=xml&sid=" + getSid(), conversationsDefinedAlarmsCell, "conversationsdefinedalarmswindow", ProbeTableManager.TABLE_UPDATE_TIME);
	conversationsDefinedAlarmsTable.setSize(787, 103);
	conversationsDefinedAlarmsTable.setClass("defaultTable summaryTable");
	conversationsDefinedAlarmsTable.setHeaderRowClass("summaryTableHeaderRow");
	conversationsDefinedAlarmsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	conversationsDefinedAlarmsTable.setRowClass("odd");
	conversationsDefinedAlarmsTable.setMouseOverClass("mouseover");
	conversationsDefinedAlarmsTable.toggleAlternateRowClass(true, "even");
	conversationsDefinedAlarmsTable.addColumns("entry", "Entry", "", 193, "counterId", "Field", "", 94, "description", "Description", "", 188, "severity", "Severity", "", 94, "absoluteAlarm", "Type", "", 94, "triggerType", "Trigger type", "", 94, "upperTriggerValue", "Upper trigger", "", 94, "upperUnit", "upper unit", "", 94, "lowerTriggerValue", "Lower Trigger", "", 94, "lowerUnit", "lower unit", "", 94, "samplingTime", "Sampling time", "", 94, "visibility", "Visibility", "", 94);
	conversationsDefinedAlarmsTable.setVisibleColumns("entry", "counterId", "description", "severity", "absoluteAlarm", "triggerType");
	conversationsDefinedAlarmsTable.setHeaderProperty("bold");
	conversationsDefinedAlarmsTable.setEntryTagName("defined");
	conversationsDefinedAlarmsTable.setIdField("id");
	conversationsDefinedAlarmsTable.setSortable(true);
	conversationsDefinedAlarmsTable.setSortFunction(sortFunction);
	conversationsDefinedAlarmsTable.setRightClickMenuColumns("entry", "counterId", "description", "severity", "absoluteAlarm", "triggerType", "upperTriggerValue", "upperUnit", "lowerTriggerValue", "lowerUnit", "samplingTime", "visibility");
	conversationsDefinedAlarmsTable.setRightClickMenuFunction(showDefinedAlarmsMenu);
	return conversationsDefinedAlarmsTable;
}


/**
 * Inits the allTriggeredAlarms table.
 */
function initAllTriggeredAlarmTable() {
	initedTables["allTriggeredAlarmsTable"] = true;
	var allTriggeredAlarmsCell = getElementWithAttribute(WindowManager.instance().getWindow("alltriggeredalarmswindow").getHtmlCode(), "windowContentCell", "id", "td");
	var allTriggeredAlarmsTable = createTriggeredAlarmTable("allTriggeredAlarmsTable", "/?resource=AlarmManager&mode=get&command=triggered&count=1&format=xml&sid=" + getSid(), allTriggeredAlarmsCell, "alltriggeredalarmswindow", ProbeTableManager.TABLE_UPDATE_TIME);
	allTriggeredAlarmsTable.setSize(1070, 103);
	allTriggeredAlarmsTable.setClass("defaultTable summaryTable");
	allTriggeredAlarmsTable.setHeaderRowClass("summaryTableHeaderRow");
	allTriggeredAlarmsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	allTriggeredAlarmsTable.setRowClass("odd");
	allTriggeredAlarmsTable.setMouseOverClass("mouseover");
	allTriggeredAlarmsTable.toggleAlternateRowClass(true, "even");
	allTriggeredAlarmsTable.addColumns("triggerTime", "Time", "", 109, "triggerCount", "Count", "", 59, "table", "Entry type", "", 130, "entry", "Entry", "", 302, "counterId", "Field", "", 94, "description", "Description", "", 188, "triggerValue", "Value", "", 74, "severity", "Severity", "", 74, "upperTriggerValue", "Upper trigger", "", 94, "upperUnit", "upper unit", "", 94, "lowerTriggerValue", "Lower Trigger", "", 94, "lowerUnit", "lower unit", "", 94, "absoluteAlarm", "Absolute alarm", "", 94, "samplingTime", "Sampling time", "", 94, "triggerType", "Trigger type", "", 94, "visibility", "Visibility", "", 94);
	allTriggeredAlarmsTable.setVisibleColumns("table", "counterId", "description", "severity", "triggerValue", "triggerTime", "triggerCount", "entry");
	allTriggeredAlarmsTable.setHeaderProperty("bold");
	allTriggeredAlarmsTable.setEntryTagName("alarm");
	allTriggeredAlarmsTable.setIdField("id");
	allTriggeredAlarmsTable.setColumnToSort("triggerTime");
	allTriggeredAlarmsTable.setSortable(true);
	allTriggeredAlarmsTable.setSortFunction(sortFunction);
	allTriggeredAlarmsTable.setRightClickMenuColumns("triggerTime", "triggerCount", "table", "entry", "counterId", "description", "triggerValue", "severity", "upperTriggerValue", "upperUnit", "lowerTriggerValue", "lowerUnit", "absoluteAlarm", "samplingTime", "triggerType", "visibility");
	allTriggeredAlarmsTable.setRightClickMenuFunction(showTriggeredAlarmsMenu);
	return allTriggeredAlarmsTable;
}


/**
 * Inits the allTriggeredAlarms table.
 */
function initAllTriggeredMasterAlarmTable() {
	initedTables["allTriggeredMasterAlarmsTable"] = true;
	var allTriggeredAlarmsCell = getElementWithAttribute(WindowManager.instance().getWindow("alltriggeredmasteralarmswindow").getHtmlCode(), "windowContentCell", "id", "td");
	var allTriggeredMasterAlarmsTable = createTriggeredAlarmTable("allTriggeredMasterAlarmsTable", "/?resource=AlarmManager&mode=get&command=masterTriggered&count=1&format=xml&sid=" + getSid(), allTriggeredAlarmsCell, "alltriggeredmasteralarmswindow", ProbeTableManager.TABLE_UPDATE_TIME);
	allTriggeredMasterAlarmsTable.setSize(1175, 103);
	allTriggeredMasterAlarmsTable.setClass("defaultTable summaryTable");
	allTriggeredMasterAlarmsTable.setHeaderRowClass("summaryTableHeaderRow");
	allTriggeredMasterAlarmsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	allTriggeredMasterAlarmsTable.setRowClass("odd");
	allTriggeredMasterAlarmsTable.setMouseOverClass("mouseover");
	allTriggeredMasterAlarmsTable.toggleAlternateRowClass(true, "even");
	allTriggeredMasterAlarmsTable.addColumns("triggerTime", "Time", "", 109, "triggerCount", "Count", "", 59, "srcIp", "Source Probe", "", 110, "table", "Entry type", "", 130, "entry", "Entry", "", 292, "counterId", "Field", "", 94, "description", "Description", "", 188, "triggerValue", "Value", "", 74, "severity", "Severity", "", 74, "upperTriggerValue", "Upper trigger", "", 94, "upperUnit", "upper unit", "", 94, "lowerTriggerValue", "Lower Trigger", "", 94, "lowerUnit", "lower unit", "", 94, "absoluteAlarm", "Absolute alarm", "", 94, "samplingTime", "Sampling time", "", 94, "triggerType", "Trigger type", "", 94, "visibility", "Visibility", "", 94);
	allTriggeredMasterAlarmsTable.setVisibleColumns("table", "srcIp","counterId", "description", "severity", "triggerValue", "triggerTime", "triggerCount", "entry");
	allTriggeredMasterAlarmsTable.setHeaderProperty("bold");
	allTriggeredMasterAlarmsTable.setEntryTagName("alarm");
	allTriggeredMasterAlarmsTable.setIdField("id");
	allTriggeredMasterAlarmsTable.setColumnToSort("triggerTime");
	allTriggeredMasterAlarmsTable.setSortable(true);
	allTriggeredMasterAlarmsTable.setSortFunction(sortFunction);
	allTriggeredMasterAlarmsTable.setRightClickMenuColumns("triggerTime", "srcIp", "triggerCount", "table", "entry", "counterId", "description", "triggerValue", "severity", "upperTriggerValue", "upperUnit", "lowerTriggerValue", "lowerUnit", "absoluteAlarm", "samplingTime", "triggerType", "visibility");
	allTriggeredMasterAlarmsTable.setRightClickMenuFunction(showTriggeredMasterAlarmsMenu);
	allTriggeredMasterAlarmsTable.toggleEditLink(false);
	return allTriggeredMasterAlarmsTable;
}



/**
 * Inits the summaryTriggeredAlarms table.
 */
function initSummaryTriggeredAlarmsTable() {
	initedTables["summaryTriggeredAlarmsTable"] = true;
	var summaryTriggeredAlarmsCell = getElementWithAttribute(WindowManager.instance().getWindow("summarytriggeredalarmswindow").getHtmlCode(), "windowContentCell", "id", "td");
	var summaryTriggeredAlarmsTable = createTriggeredAlarmTable("summaryTriggeredAlarmsTable", "/?resource=AlarmManager&mode=get&command=triggered&table=summary&count=1&format=xml&sid=" + getSid(), summaryTriggeredAlarmsCell, "summarytriggeredalarmswindow", ProbeTableManager.TABLE_UPDATE_TIME);
	summaryTriggeredAlarmsTable.setSize(634, 103);
	summaryTriggeredAlarmsTable.setClass("defaultTable summaryTable");
	summaryTriggeredAlarmsTable.setHeaderRowClass("summaryTableHeaderRow");
	summaryTriggeredAlarmsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	summaryTriggeredAlarmsTable.setRowClass("odd");
	summaryTriggeredAlarmsTable.setMouseOverClass("mouseover");
	summaryTriggeredAlarmsTable.toggleAlternateRowClass(true, "even");
	summaryTriggeredAlarmsTable.addColumns("triggerTime", "Time", "", 115, "triggerCount", "Count", "", 59, "counterId", "Field", "", 94, "description", "Description", "", 188, "triggerValue", "Value", "", 74, "severity", "Severity", "", 74, "upperTriggerValue", "Upper trigger", "", 94, "upperUnit", "upper unit", "", 94, "lowerTriggerValue", "Lower Trigger", "", 94, "lowerUnit", "lower unit", "", 94, "absoluteAlarm", "Absolute alarm", "", 94, "samplingTime", "Sampling time", "", 94, "triggerType", "Trigger type", "", 94, "visibility", "Visibility", "", 94);
	summaryTriggeredAlarmsTable.setVisibleColumns("counterId", "description", "severity", "triggerValue", "triggerTime", "triggerCount");
	summaryTriggeredAlarmsTable.setHeaderProperty("bold");
	summaryTriggeredAlarmsTable.setEntryTagName("alarm");
	summaryTriggeredAlarmsTable.setIdField("id");
	summaryTriggeredAlarmsTable.setColumnToSort("triggerTime");
	summaryTriggeredAlarmsTable.setSortable(true);
	summaryTriggeredAlarmsTable.setSortFunction(sortFunction);
	summaryTriggeredAlarmsTable.setRightClickMenuColumns("triggerTime", "triggerCount", "counterId", "description", "triggerValue", "severity", "upperTriggerValue", "upperUnit", "lowerTriggerValue", "lowerUnit", "absoluteAlarm", "samplingTime", "triggerType", "visibility");
	summaryTriggeredAlarmsTable.setRightClickMenuFunction(showTriggeredAlarmsMenu);
	return summaryTriggeredAlarmsTable;
}


/**
 * Inits the protocolTriggeredAlarms table.
 */
function initProtocolTriggeredAlarmsTable() {
	initedTables["protocolTriggeredAlarmsTable"] = true;
	var protocolTriggeredAlarmsCell = getElementWithAttribute(WindowManager.instance().getWindow("protocoltriggeredalarmswindow").getHtmlCode(), "windowContentCell", "id", "td");
	var protocolTriggeredAlarmsTable = createTriggeredAlarmTable("protocolTriggeredAlarmsTable", "/?resource=AlarmManager&mode=get&command=triggered&table=protocol&count=1&format=xml&sid=" + getSid(), protocolTriggeredAlarmsCell, "protocoltriggeredalarmswindow", ProbeTableManager.TABLE_UPDATE_TIME);
	protocolTriggeredAlarmsTable.setSize(738, 103);
	protocolTriggeredAlarmsTable.setClass("defaultTable summaryTable");
	protocolTriggeredAlarmsTable.setHeaderRowClass("summaryTableHeaderRow");
	protocolTriggeredAlarmsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	protocolTriggeredAlarmsTable.setRowClass("odd");
	protocolTriggeredAlarmsTable.setMouseOverClass("mouseover");
	protocolTriggeredAlarmsTable.toggleAlternateRowClass(true, "even");
	protocolTriggeredAlarmsTable.addColumns("triggerTime", "Time", "", 115, "triggerCount", "Count", "", 59, "pid", "Entry", "", 99, "counterId", "Field", "", 94, "description", "Description", "", 188, "triggerValue", "Value", "", 74, "severity", "Severity", "", 74, "upperTriggerValue", "Upper trigger", "", 94, "upperUnit", "upper unit", "", 94, "lowerTriggerValue", "Lower Trigger", "", 94, "lowerUnit", "lower unit", "", 94, "absoluteAlarm", "Absolute alarm", "", 94, "samplingTime", "Sampling time", "", 94, "triggerType", "Trigger type", "", 94, "visibility", "Visibility", "", 94);
	protocolTriggeredAlarmsTable.setVisibleColumns("table", "pid", "counterId", "description", "severity", "triggerValue", "triggerTime", "triggerCount");
	protocolTriggeredAlarmsTable.setHeaderProperty("bold");
	protocolTriggeredAlarmsTable.setEntryTagName("alarm");
	protocolTriggeredAlarmsTable.setIdField("id");
	protocolTriggeredAlarmsTable.setColumnToSort("triggerTime");
	protocolTriggeredAlarmsTable.setSortable(true);
	protocolTriggeredAlarmsTable.setSortFunction(sortFunction);
	protocolTriggeredAlarmsTable.setRightClickMenuColumns("triggerTime", "triggerCount", "pid", "counterId", "description", "triggerValue", "severity", "upperTriggerValue", "upperUnit", "lowerTriggerValue", "lowerUnit", "absoluteAlarm", "samplingTime", "triggerType", "visibility");
	protocolTriggeredAlarmsTable.setRightClickMenuFunction(showTriggeredAlarmsMenu);
	return protocolTriggeredAlarmsTable;
}


/**
 * Inits the host triggered alarms table.
 */
function initHostTriggeredAlarmTable() {
	initedTables["hostsTriggeredAlarmsTable"] = true;
	var hostsTriggeredAlarmsCell = getElementWithAttribute(WindowManager.instance().getWindow("hoststriggeredalarmswindow").getHtmlCode(), "windowContentCell", "id", "td");
	var hostsTriggeredAlarmsTable = createTriggeredAlarmTable("hostsTriggeredAlarmsTable", "/?resource=AlarmManager&mode=get&command=triggered&table=nlhost&count=1&format=xml&sid=" + getSid(), hostsTriggeredAlarmsCell, "hoststriggeredalarmswindow", ProbeTableManager.TABLE_UPDATE_TIME);
	hostsTriggeredAlarmsTable.setSize(733, 103);
	hostsTriggeredAlarmsTable.setClass("defaultTable summaryTable");
	hostsTriggeredAlarmsTable.setHeaderRowClass("summaryTableHeaderRow");
	hostsTriggeredAlarmsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	hostsTriggeredAlarmsTable.setRowClass("odd");
	hostsTriggeredAlarmsTable.setMouseOverClass("mouseover");
	hostsTriggeredAlarmsTable.toggleAlternateRowClass(true, "even");
	hostsTriggeredAlarmsTable.addColumns("triggerTime", "Time", "", 115, "triggerCount", "Count", "", 59, "src", "Entry", "", 94, "counterId", "Field", "", 94, "description", "Description", "", 188, "triggerValue", "Value", "", 74, "severity", "Severity", "", 74, "upperTriggerValue", "Upper trigger", "", 94, "upperUnit", "upper unit", "", 94, "lowerTriggerValue", "Lower Trigger", "", 94, "lowerUnit", "lower unit", "", 94, "absoluteAlarm", "Absolute alarm", "", 94, "samplingTime", "Sampling time", "", 94, "triggerType", "Trigger type", "", 94, "visibility", "Visibility", "", 94);
	hostsTriggeredAlarmsTable.setVisibleColumns("src", "counterId", "description", "severity", "triggerValue", "triggerTime", "triggerCount");
	hostsTriggeredAlarmsTable.setHeaderProperty("bold");
	hostsTriggeredAlarmsTable.setEntryTagName("alarm");
	hostsTriggeredAlarmsTable.setIdField("id");
	hostsTriggeredAlarmsTable.setColumnToSort("triggerTime");
	hostsTriggeredAlarmsTable.setSortable(true);
	hostsTriggeredAlarmsTable.setSortFunction(sortFunction);
	hostsTriggeredAlarmsTable.setRightClickMenuColumns("triggerTime", "triggerCount", "src", "counterId", "description", "triggerValue", "severity", "upperTriggerValue", "upperUnit", "lowerTriggerValue", "lowerUnit", "absoluteAlarm", "samplingTime", "triggerType", "visibility");
	hostsTriggeredAlarmsTable.setRightClickMenuFunction(showTriggeredAlarmsMenu);
	return hostsTriggeredAlarmsTable;
}


/**
 * Init the conversationTriggeredAlarm table.
 */
function initConversationsTriggeredAlarmTable() {
	initedTables["conversationsTriggeredAlarmsTable"] = true;
	var conversationsTriggeredAlarmsCell = getElementWithAttribute(WindowManager.instance().getWindow("conversationstriggeredalarmswindow").getHtmlCode(), "windowContentCell", "id", "td");
	var conversationsTriggeredAlarmsTable = createTriggeredAlarmTable("conversationsTriggeredAlarmsTable", "/?resource=AlarmManager&mode=get&command=triggered&table=nlmatrix&count=1&format=xml&sid=" + getSid(), conversationsTriggeredAlarmsCell, "conversationstriggeredalarmswindow", ProbeTableManager.TABLE_UPDATE_TIME);
	conversationsTriggeredAlarmsTable.setSize(832, 103);
	conversationsTriggeredAlarmsTable.setClass("defaultTable summaryTable");
	conversationsTriggeredAlarmsTable.setHeaderRowClass("summaryTableHeaderRow");
	conversationsTriggeredAlarmsTable.setHeaderTableClass("defaultTable summaryTableHeader");
	conversationsTriggeredAlarmsTable.setRowClass("odd");
	conversationsTriggeredAlarmsTable.setMouseOverClass("mouseover");
	conversationsTriggeredAlarmsTable.toggleAlternateRowClass(true, "even");
	conversationsTriggeredAlarmsTable.addColumns("triggerTime", "Time", "", 115, "triggerCount", "Count", "", 59, "entry", "Entry", "", 193, "counterId", "Field", "", 94, "description", "Description", "", 188, "triggerValue", "Value", "", 74, "severity", "Severity", "", 74, "upperTriggerValue", "Upper trigger", "", 94, "upperUnit", "upper unit", "", 94, "lowerTriggerValue", "Lower Trigger", "", 94, "lowerUnit", "lower unit", "", 94, "absoluteAlarm", "Absolute alarm", "", 94, "samplingTime", "Sampling time", "", 94, "triggerType", "Trigger type", "", 94, "visibility", "Visibility", "", 94);
	conversationsTriggeredAlarmsTable.setVisibleColumns("entry", "counterId", "description", "severity", "triggerValue", "triggerTime", "triggerCount");
	conversationsTriggeredAlarmsTable.setHeaderProperty("bold");
	conversationsTriggeredAlarmsTable.setEntryTagName("alarm");
	conversationsTriggeredAlarmsTable.setIdField("id");
	conversationsTriggeredAlarmsTable.setColumnToSort("triggerTime");
	conversationsTriggeredAlarmsTable.setSortable(true);
	conversationsTriggeredAlarmsTable.setSortFunction(sortFunction);
	conversationsTriggeredAlarmsTable.setRightClickMenuColumns("triggerTime", "triggerCount", "entry", "counterId", "description", "triggerValue", "severity", "upperTriggerValue", "upperUnit", "lowerTriggerValue", "lowerUnit", "absoluteAlarm", "samplingTime", "triggerType", "visibility");
	conversationsTriggeredAlarmsTable.setRightClickMenuFunction(showTriggeredAlarmsMenu);
	return conversationsTriggeredAlarmsTable;
}