/**
 * This class overrides probetable to implement topn table behaviour.
 */


// constants


/**
 * The default n value used in the topn tables. 
 */
TopNTable.DEFAULT_N = 5;


// construction


/**
 * Factory method to create new TopNTable objects.
 * @param id The id of the table to update.
 * @param url The url to load the data from.
 * @param parentElement The element the table is to be inserted in. 
 * @param windowId The id of the window the table is in. 
 * @param Optional time parameter. The time between each update.
 * @returns A new probeTable object.
 * If this is not specified the default value is used.
 * @throws Error if id is undefined, null or empty.
 * @throws Error if url is undefined, null or empty.
 * @throws Error if parentElement is undefined, or null.
 * @throws Error if windowId is undefined, null or empty.
 */
function createTopNTable(id, url, parentElement, windowId, time) {
	assertNotEmpty(id, "Id for the table is undefined, empty or null: " + id);
	assertNotEmpty(url, "Url to set for the table is undefined empty or null: " + url);
	assertDefined(parentElement, "The element to insert the table into is undefined or null");
	assertNotEmpty(windowId, "windowId to set for the table is undefined empty or null: " + windowId);

	var subInstance = new TopNTable();
	var superInstance = createProbeTable(id, url, parentElement, windowId, time);
	var instance = createSubClass(superInstance, subInstance);
	
	TableManager.instance().removeTable(TableManager.instance().getTable(id));
	TableManager.instance().registerNewTable(instance);

	// let the event processor call event handler functions as a method
	// of the mytable object instance
	superInstance.getEventProcessor().changeObject(instance);

	// overwrite the repeater to call the load function on the TopNTable object	
	instance.repeater = new Repeater(load, instance, time);

	return instance;
}



/**
 * Subclasses table.js to add probe specifc behaviour.
 */
function TopNTable() {
	// the n value for this topn table
	this.n = TopNTable.DEFAULT_N;
}


// services


/**
 * Sets the n value for this table. The new n value is inserted into the tables url but 
 * the table is not reloaded. 
 * @param n The n value to set. 
 * @throws Error if n is undefined, not a number negative. 
 */
TopNTable.prototype.setN = function(n) {
	assertDefined(n, "N is undefined");
	assertNotNegative(n, "N is not a number or negative: " + n);
	this.n = n;
	
	// insert the new n into the request
	var oldUrl = this.url;
	var index = oldUrl.indexOf("top=") + "top=".length;
	var endIndex = oldUrl.indexOf("&", index);
	var newRequest = oldUrl.substring(0, index);
	newRequest = newRequest + n;
	newRequest = newRequest + oldUrl.substring(endIndex);
	this.url = newRequest;
}


/**
 * Returns the n value for the this table.
 */
TopNTable.prototype.getN = function() {
	return this.n;
}	


/**
 * Changes the index of this table to the selected value.
 */
TopNTable.prototype.changeIndex = function() {
	var newTableIndex = this.getCurrentIndex();
	if (exists(newTableIndex) && !isEmpty(newTableIndex)) {
		this.url = replaceParameterValue(this.url, "sortBy", newTableIndex, "&");
		this.isLoading = false;
		load.call(this, tableIndexChanged);
	}
}


/**
 * Get the current table index. 
 * @return The current index of the table, null if no option is selected.
 */
TopNTable.prototype.getCurrentIndex = function() {
	var indexSelector = document.getElementById("indexSelect" + this.id);
	var index = indexSelector.selectedIndex;
	if (index == -1) {
		// no option is selected
		return null;
	}
	return indexSelector.options[index].getAttribute("value");
}


/**
 * Called when a table index has changed and new data has been loaded.
 */
function tableIndexChanged()  {
	var tableIndex = getParameterValue(this.url, "sortBy", "&");
	this.fireIndexChangedEvent(tableIndex);	
	if(this.sortedColumn != tableIndex && this.isPainted()) {
		this.sortColumn(tableIndex);
	}
}

 
/**
 * Fires a index changed event.
 * @param newIndex The new table index.
 */
TopNTable.prototype.fireIndexChangedEvent = function(newIndex) {
	if (!eventsAvailable()) {
		return;
	}
	
	var data = new Array();
	data["id"] = this.id;
	data["newIndex"] = newIndex;
	getEventServer().notifyListeners(new Event("tableIndexChanged", data));
}



/**
 * Sets a layout for the table based on a block of xml.  
 * This is overridden to set the n value.
 * @param xmlLayout The block of xml containing the layout to set. 
 */
TopNTable.prototype.setXmlLayout = function(xmlLayout) {
	var n = getTagNodeValue("n", xmlLayout);
	if (!isEmpty(n) && isNumberString(n) && !isNegative(n)) {
		var eventData = new Array();
		eventData["id"] = this.id;
		eventData["n"] = parseInt(n);
		eventData["oldN"] = this.n;
		getEventServer().notifyListeners(new Event("topNChanged", eventData));
	}
	
	var argumentsToSuper = new Array();
	argumentsToSuper[0] = xmlLayout;
	this.callSuper("setXmlLayout", argumentsToSuper);
}