/**
 * This class can show details for a protocol or nlhost entry. For a protocol this window contains the 
 * three tables: the detailed infor for this protocol, the top hosts for this protocol and the top conversations.
 * For nlhosts this window also contains three tables: the detailed info table, the top protocols and the top 
 * conversations. 
 */

 
// construction


/**
 * Factory method to create new entry DetailWindow objects.
 * @returns A new detailWindow object.
 * @param detailTableId The id of the detail table (protocol detail or nlhost detail).
 * @param middleTableId The id of the middle table (top hosts for a protocol for example).
 * @param bottomTableId The id of the bottom table (top conversations for a protocol for example).
 * @throws Error if id is undefined, null or empty.
 * @throws Error if detailTableId is undefined, null or empty.
 * @throws Error if middleTableId is undefined, null or empty.
 * @throws Error if bottomTableId is undefined, null or empty.
 */
function createEntryDetailWindow(id, detailTableId, middleTableId, bottomTableId) {
	assertNotEmpty(id, "Id to set for the window is undefined, null or empty: " + id);
	assertNotEmpty(detailTableId, "Detail table id is undefined, null or empty: " + detailTableId);
	assertNotEmpty(middleTableId, "Middle table id is undefined, null or empty: " + middleTableId);
	assertNotEmpty(bottomTableId, "Bottom table id is undefined, null or empty: " + bottomTableId);
	
	var superInstance = createProbeWindow(id, detailTableId);
	superInstance.removeResizableElement(superInstance.container);
	var subInstance = new EntryDetailWindow(detailTableId, middleTableId, bottomTableId);
	subInstance = createSubClass(superInstance, subInstance);
	
	subInstance["superactivateWindow"] = superInstance["superactivateWindow"];
	subInstance["superdeactivateWindow"] = superInstance["superdeactivateWindow"];
	
	subInstance.init();
	WindowManager.instance().removeWindow(superInstance);
	WindowManager.instance().registerNewWindow(subInstance);
	
	// let the event processor call event handler functions as a method
	// of the object instance
	subInstance.getEventProcessor().changeObject(subInstance);
	subInstance.getEventProcessor().addEvent("tableResized", processTableResizedEventDetailWindow);
	subInstance.getEventProcessor().addEvent("topNChanged", processTopNChangedEventEntryWindow);
	subInstance.getEventProcessor().addEvent("noAccessShown", processNoAccessShownEntryWindow);
	subInstance.getEventProcessor().addEvent("noAccessHidden", processNoAccessHiddenEntryWindow);
	
	return subInstance;
}


/**
 * Constructs a entry detail window. 
 * @param detailTableId The id of the detail table (protocol detail or nlhost detail).
 * @param middleTableId The id of the middle table (top hosts for a protocol for example).
 * @param bottomTableId The id of the bottom table (top conversations for a protocol for example).
 * @throws Error if detailTableId is undefined, null or empty.
 * @throws Error if middleTableId is undefined, null or empty.
 * @throws Error if bottomTableId is undefined, null or empty.
 */
function EntryDetailWindow(detailTableId, middleTableId, bottomTableId) {
	assertNotEmpty(detailTableId, "Detail table id is undefined, null or empty: " + detailTableId);
	assertNotEmpty(middleTableId, "Middle table id is undefined, null or empty: " + middleTableId);
	assertNotEmpty(bottomTableId, "Bottom table id is undefined, null or empty: " + bottomTableId);
	
	this.middleTableId = middleTableId;
	this.bottomTableId = bottomTableId;
	
	// set to true if the middle table section is collapsed
	this.middleSectionColapsed = false;
	
	// set to true if the bottom table section is collapsed
	this.bottomSectionColapsed = false;
	
	// true if no access is shown in the top table
	this.topNoAccesShown = false;
	
	// true if no access is shown in the middle table	
	this.middleNoAccessShown = false;
	
	// true if no access is shown in the bottom table
	this.bottomNoAccessShown = false;
}



/**
 * Initiallises this class.
 */
EntryDetailWindow.prototype.init = function() {
	this.callSuper("init");
	
	this.topTableSection = getElementWithAttribute(this.htmlCode, "windowContentCell", "id", "td");
	this.middleTableHeader = getElementWithAttribute(this.htmlCode, "middleTableHeaderRow", "name", "tr");
	this.middleTableSection = getElementWithAttribute(this.htmlCode, "middleContent", "id", "tr");
	this.bottomTableHeader = getElementWithAttribute(this.htmlCode, "bottomTableHeaderRow", "name", "tr");
	this.bottomTableSection = getElementWithAttribute(this.htmlCode, "bottomContent", "id", "tr");
	
	// the windows drag and resize helper are initialized in the window constructor, we need to change the helpers
	// window reference to be this window not the super instance of Window
	this.dragHelper.window = this;
	this.resizeHelper.window = this;
	
	// the source address for the tables and charts in this window
	this.src = "";
	
	// the destination address for the tables and charts in this window
	this.dst = "";
	
	// the protocol for the tables and charts in this window
	this.protocol = "";
}



// services


/**
 * Expands the middle table section.
 */
EntryDetailWindow.prototype.expandMiddleSection = function() {
	if ("active" == getClassName(this.middleTableSection)) {
		return;
	}
	
	setClass(this.middleTableSection, "active");
	
	// udpate the height of the window
	var windowHeight = parseInt(this.htmlCode.style.height);
	var height = this.middleTableSection.offsetHeight;
	this.htmlCode.style.height = (windowHeight + height) + "px";	
	this.height = parseInt(this.htmlCode.style.height);
	
	var expandMiddleButton = getElementWithAttribute(this.htmlCode, "expandMiddleButton", "id", "div");
	var colapseMiddleButton = getElementWithAttribute(this.htmlCode, "colapseMiddleButton", "id", "div");
	setClass(expandMiddleButton, "inactive");
	setClass(colapseMiddleButton, "colapseButton WindowButton");
	this.middleSectionColapsed = false;
}


/**
 * Colapses the middle table section.
 */
EntryDetailWindow.prototype.colapseMiddleSection = function() {
	if ("inactive" == getClassName(this.middleTableSection)) {
		return;
	}
	
	var windowHeight = parseInt(this.htmlCode.style.height);
	var height = this.middleTableSection.offsetHeight;
	setClass(this.middleTableSection, "inactive");
	this.htmlCode.style.height = (windowHeight - height) + "px";	
	this.height = parseInt(this.htmlCode.style.height);
	
	var expandMiddleButton = getElementWithAttribute(this.htmlCode, "expandMiddleButton", "id", "div");
	var colapseMiddleButton = getElementWithAttribute(this.htmlCode, "colapseMiddleButton", "id", "div");
	setClass(expandMiddleButton, "expandButton WindowButton");
	setClass(colapseMiddleButton, "inactive");
	this.middleSectionColapsed = true;
}


/**
 * Expands the bottom table section.
 */
EntryDetailWindow.prototype.expandBottomSection = function() {
	if ("active" == getClassName(this.bottomTableSection)) {
		return;
	}
	
	setClass(this.bottomTableSection, "active");
	
	// udpate the height of the window
	var windowHeight = parseInt(this.htmlCode.style.height);
	var height = this.bottomTableSection.offsetHeight;
	this.htmlCode.style.height = (windowHeight + height) + "px";	
	this.height = parseInt(this.htmlCode.style.height);
	
	var expandMiddleButton = getElementWithAttribute(this.htmlCode, "expandBottomButton", "id", "div");
	var colapseMiddleButton = getElementWithAttribute(this.htmlCode, "colapseBottomButton", "id", "div");
	setClass(expandMiddleButton, "inactive");
	setClass(colapseMiddleButton, "colapseButton WindowButton");
	this.bottomSectionColapsed = false;
}


/**
 * Colapses the bottom table section.
 */
EntryDetailWindow.prototype.colapseBottomSection = function() {
	if ("inactive" == getClassName(this.bottomTableSection)) {
		return;
	}
	
	var windowHeight = parseInt(this.htmlCode.style.height);
	var height = this.bottomTableSection.offsetHeight;
	setClass(this.bottomTableSection, "inactive");
	this.htmlCode.style.height = (windowHeight - height) + "px";	
	this.height = parseInt(this.htmlCode.style.height);
	
	var expandMiddleButton = getElementWithAttribute(this.htmlCode, "expandBottomButton", "id", "div");
	var colapseMiddleButton = getElementWithAttribute(this.htmlCode, "colapseBottomButton", "id", "div");
	setClass(expandMiddleButton, "expandButton WindowButton");
	setClass(colapseMiddleButton, "inactive");
	this.bottomSectionColapsed = true;
}



/**
 * Hides the collapse middle table section button for a mouse out event, if collapsed.
 */
function onMouseOutCollapseMiddle(element, windowName, className) {
	setClass(element, className);
	var currentWindow = WindowManager.instance().getWindow(windowName);
	if (currentWindow.middleSectionColapsed == true) {
		setClass(element, "colapseButton WindowButton inactive");
	}
	else {
		setClass(element, className);
	}
}



/**
 * Hides the expand middle table section button for a mouse out event, if expanded.
 */
function onMouseOutExpandMiddle(element, windowName, className) {
	setClass(element, className);
	var currentWindow = WindowManager.instance().getWindow(windowName);
	if (currentWindow.middleSectionColapsed == false) {
		setClass(element, "expandButton WindowButton inactive");
	}
	else {
		setClass(element, className);
	}
}


/**
 * Hides the collapse bottom table section button for a mouse out event, if collapsed.
 */
function onMouseOutCollapseBottom(element, windowName, className) {
	setClass(element, className);
	var currentWindow = WindowManager.instance().getWindow(windowName);
	if (currentWindow.bottomSectionColapsed == true) {
		setClass(element, "colapseButton WindowButton inactive");
	}
	else {
		setClass(element, className);
	}
}



/**
 * Hides the expand bottom table section button for a mouse out event, if expanded.
 */
function onMouseOutExpandBottom(element, windowName, className) {
	setClass(element, className);
	var currentWindow = WindowManager.instance().getWindow(windowName);
	if (currentWindow.bottomSectionColapsed == false) {
		setClass(element, "expandButton WindowButton inactive");
	}
	else {
		setClass(element, className);
	}
}


/**
 * Activates a window. This is overridden to activate the headers of the two extra tables. 
 */
EntryDetailWindow.prototype.activateWindow = function() {
	if(this.isActive()) {
		// the window is already active
		return;
	}
	this.callSuper("activateWindow");
	
	if (isDefined(this.chartSelectorArea) && (!isNull(this.chartSelectorArea)) && !this.isChartSelectorHidden()) {
		setClass(this.chartSelectorArea, "chartSelector");
	}
	setClass(this.middleTableHeader, "activeWindowHeader");
	setClass(this.bottomTableHeader, "activeWindowHeader");
}


/**
 * Deactivates a window. This is overridden to deactivate the headers of the two extra tables. 
 */
EntryDetailWindow.prototype.deactivateWindow = function() {
	this.callSuper("deactivateWindow");
	if (isDefined(this.setup)) {
		setClass(this.setup, "inactive");
	}	

	setClass(this.htmlCode, "inactiveBorderColor");
	if (isDefined(this.chartSelectorArea) && (!isNull(this.chartSelectorArea)) && !this.isChartSelectorHidden()) {
		setClass(this.chartSelectorArea, "inactiveChartSelector"); 
	}
	setClass(this.middleTableHeader, "inactiveWindowHeader");
	setClass(this.bottomTableHeader, "inactiveWindowHeader");
}



/**
 * Set the src ip address for the tables and charts in this window.
 * @param address The address to set as a string(10.0.2.33).
 * @throws Error if address is undefined or null.
 */
EntryDetailWindow.prototype.setSrcAddress = function(address) {
	assertNotEmpty(address, "Source ip address to set for the chart is undefined, null or empty");
	
	this.src = address;
}


/**
 * Returns the src ip address for the tables and charts in this window, empty string if not set.
 */
EntryDetailWindow.prototype.getSrcAddress = function() {
	return this.src;
}


/**
 * Set the dst ip address for the tables and charts in this window.
 * @param address The address to set as a string(10.0.2.33).
 * @throws Error if address is undefined or null.
 */
EntryDetailWindow.prototype.setDstAddress = function(address) {
	assertNotEmpty(address, "Destination ip address to set for the chart is undefined, null or empty");
	
	this.dst = address;
}


/**
 * Returns the dst ip address for the tables and charts in this window, empty string if not set.
 */
EntryDetailWindow.prototype.getDstAddress = function() {
	return this.dst;
}


/**
 * Set the protocol for the tables and charts in this window.
 * @param protocol The protocol to set as a string(1.2048.6.80).
 * @throws Error if protocol is undefined or null.
 */
EntryDetailWindow.prototype.setProtocol = function(protocol) {
	assertNotEmpty(protocol, "Protocol to set for the chart is undefined, null or empty");
	
	this.protocol = protocol;
}



/**
 * Returns the protocol for the tables and charts in this window, empty string if not set.
 */
EntryDetailWindow.prototype.getProtocol = function() {
	return this.protocol;
}


// event handling



/**
 * Processes a no access hidden event from a table. 
 * @param event The event to process. 
 */
function processNoAccessHiddenEntryWindow(event) {
	if (!this.containsTable(event.getData())) {
		return;
	}
	
	// the no access height might be different than the table size resize the window according
	var table = TableManager.instance().getTable(event.getData());
	var noAccessHeight;
	if (this.tableId.equalsIgnoreCase(event.getData())) {
		noAccessHeight = this.topNoAccessHeight
		this.topNoAccesShown = false;
	}
	else if (this.middleTableId.equalsIgnoreCase(event.getData())) {
		noAccessHeight = this.middleNoAccessHeight
		this.middleNoAccessShown = false;
	}
	else if (this.bottomTableId.equalsIgnoreCase(event.getData())) {
		noAccessHeight = this.bottomNoAccessHeight
		this.bottomNoAccessShown = false;
	}
	var heightDiff = noAccessHeight - parseInt(table.getTableContainerCode().style.height);
	this.htmlCode.style.height = (parseInt(this.htmlCode.style.height) - heightDiff) + "px";
	
	// show the charts if one of the tables has access
	if (!this.topNoAccesShown || !this.middleNoAccessShown || !this.bottomNoAccessShown) {
		this.showChartSelector();
		this.showChartSection();
		this.makeResizable();
	}
	
	// the window might have been resize while no acces was shown for a table, resize the table id needed
	if (this.calculateContentCellWidth() != parseInt(table.getTableContainerCode().style.width)) {
		var diff = this.calculateContentCellWidth() - parseInt(table.getTableContainerCode().style.width);
		table.resizeTable(0, diff);
	}
}


/**
 * Processes a no acces shown event from a table. 
 * @param event The event to process.
 */
function processNoAccessShownEntryWindow(event) {
	if (!this.containsTable(event.getData())) {
		return;
	}
	
	// the no access height might be different than the table size resize the window according
	var table = TableManager.instance().getTable(event.getData());
	var noAccessHeight = table.noAccess.offsetHeight;
	this.removeResizableElement(table.noAccess);
	if (this.tableId.equalsIgnoreCase(event.getData())) {
		this.topNoAccessHeight = noAccessHeight; 
		this.topNoAccesShown = true;
	}
	else if (this.middleTableId.equalsIgnoreCase(event.getData())) {
		this.middleNoAccessHeight = noAccessHeight;
		this.middleNoAccessShown = true;
	}
	else if (this.bottomTableId.equalsIgnoreCase(event.getData())) {
		this.bottomNoAccessHeight = noAccessHeight;
		this.bottomNoAccessShown = true;
	}
	var heightDiff = noAccessHeight - parseInt(table.getTableContainerCode().style.height);
	this.htmlCode.style.height = (parseInt(this.htmlCode.style.height) + heightDiff) + "px";
	
	// hide the charts if none of the table have access
	if (this.topNoAccesShown && this.middleNoAccessShown && this.bottomNoAccessShown) {
		this.hideChartSection();
 		this.hideChartSelector();
 		this.makeNotResizable();
	}
}


/**
 * Processing a table resized event. 
 * @param event The event to process. 
 */
function processTableResizedEventDetailWindow(event) {
	if (!this.containsTable(event.getData()["id"])) {
		return;
	}
	// resize the other two tables
	if (this.tableId == event.getData()["id"]) {
		TableManager.instance().getTable(this.middleTableId).resizeTable(0, event.getData()["width"]);
		TableManager.instance().getTable(this.bottomTableId).resizeTable(0, event.getData()["width"]);
	}
	else if (this.middleTableId == event.getData()["id"]) {
		TableManager.instance().getTable(this.tableId).resizeTable(0, event.getData()["width"]);
		TableManager.instance().getTable(this.bottomTableId).resizeTable(0, event.getData()["width"]);
	}
	else {
		TableManager.instance().getTable(this.tableId).resizeTable(0, event.getData()["width"]);
		TableManager.instance().getTable(this.middleTableId).resizeTable(0, event.getData()["width"]);
	}
	
	// resize the window itself
	var width = event.getData()["width"];
	var height = event.getData()["height"];
	var resizedHeight = this.resizeWindowHeight(height);
	var resizedWidth = this.resizeWindowWidth(width);
	this.resizeElements(height, width, false);
	this.resized();
}


/**
 * Processes a top n change event. This is overridden to set the correct html input fields. 
 * @param event The event to process.
 */
function processTopNChangedEventEntryWindow(event) {
	if (!this.containsTable(event.getData()["id"])) {
		return;
	}
	
	var middleTableTopnInput = getElementWithAttribute(this.getHtmlCode(), "topN" + this.middleTableId, "name", "input");
	var bottomTopnInput = getElementWithAttribute(this.getHtmlCode(), "topN" + this.bottomTableId, "name", "input");
	var n = event.getData()["n"];	
	if (this.middleTableId == event.getData()["id"] && !isNull(middleTableTopnInput) && n != middleTableTopnInput.value) {
		middleTableTopnInput.value = n;
	}
	else if (this.bottomTableId == event.getData()["id"] && !isNull(bottomTopnInput) && n != bottomTopnInput.value) {
		bottomTopnInput.value = n;
	}
}


// layout


/**
 * Sets the layout for the window based on a block of xml.
 * @param xmlLayout The block of xml containing the layout to set. 
 */
EntryDetailWindow.prototype.setXmlLayout = function(xmlLayout) {
	var chartSectionHidden = (this.chartCount == 0);
	if (chartSectionHidden) {
		// the chart section needs to be visible when the layout is set, or else the height
		// of the window will not be set correct
		this.showCharts();
	}
	this.setXmlContentColapsedStatus(xmlLayout);
	this.setXmlChartColapsedStatus(xmlLayout);
	this.setXmlMiddleTableColapsedStatus(xmlLayout);
	this.setXmlBottomTableColapsedStatus(xmlLayout);
	this.setXmlPosition(xmlLayout);
	this.setXmlActiveChart(xmlLayout);
	this.setXmlSize(xmlLayout);
	this.setXmlStatus(xmlLayout);
	this.setXmlProtocol(xmlLayout);
	this.setXmlSrcAddress(xmlLayout);
	this.setXmlDstAddress(xmlLayout);
	if (chartSectionHidden) {
		this.hideCharts();
	}
}


/**
 * Returns this window as xml. This is overridden to also include the colapsed status 
 * of the middle and bottom section, and the multiple tables.
 */
EntryDetailWindow.prototype.toXml = function() {
	if (!this.isShown()) {
		// when the detail window is shown no layout for the window needs to be set for the window
		// because next time the user searches and shows the window the is reset anyway
		return ""; 
	}
	var xmlString = 
		"<window>" +
			"<windowId>" + this.id + "</windowId>" + 
			"<xPosition>" + this.getXPosition() + "</xPosition>" +
			"<yPosition>" + this.getYPosition() + "</yPosition>" +
			"<width>" + this.getWidth() + "</width>" +
			"<height>" + this.getHeight() + "</height>";
	xmlString += this.activeChartNameToXml();
	xmlString += this.statusToXml();
	xmlString += this.colapsedContentToXml();
	xmlString += this.colapsedChartToXml();
	xmlString += "<middleColapsed>" + this.middleSectionColapsed + "</middleColapsed>";
	xmlString += "<bottomColapsed>" + this.bottomSectionColapsed + "</bottomColapsed>";
	xmlString += "<src>" + this.src + "</src>";
	xmlString += "<dst>" + this.dst + "</dst>";
	xmlString += "<protocol>" + this.protocol + "</protocol>";
	
	xmlString += TableManager.instance().getTable(this.tableId).toXml();
	xmlString += TableManager.instance().getTable(this.middleTableId).toXml();
	xmlString += TableManager.instance().getTable(this.bottomTableId).toXml();
	
	xmlString += this.chartsToXml() + "</window>";
	
	return xmlString;
}


// implementation


/**
 * Sets the colapsed status of the middle table from a xml layout.
 * @param layout The layout to set the status from.
 */
EntryDetailWindow.prototype.setXmlMiddleTableColapsedStatus = function(layout) {
	var middleColapsed = getTagNodeValue("middleColapsed", layout);
	if ("true" == middleColapsed && !this.middleSectionColapsed) {
		this.colapseMiddleSection();
	}	
	else if ("false" == middleColapsed && this.middleSectionColapsed) {
		this.expandMiddleSection();
	}
}


/**
 * Sets the colapsed status of the middle table from a xml layout.
 * @param layout The layout to set the status from.
 */
EntryDetailWindow.prototype.setXmlBottomTableColapsedStatus = function(layout) {
	var bottomColapsed = getTagNodeValue("bottomColapsed", layout);
	if ("true" == bottomColapsed && !this.bottomSectionColapsed) {
		this.colapseBottomSection();
	}
	else if ("false" == bottomColapsed && this.bottomSectionColapsed) {
		this.expandBottomSection();
	}
}

 
/**
 * Sets the protocol from a xml layout into the tables in this window.
 * @param layout The xml layout to get the protocol from. 
 */
EntryDetailWindow.prototype.setXmlProtocol = function(layout) {
	var protocol = getTagNodeValue("protocol", layout);
	if (isEmpty(protocol)) {
		return;
	}
	var detailTable = TableManager.instance().getTable(this.tableId);
	detailTable.setUrl(replaceParameterValue(detailTable.getUrl(), "protocolFilterValue", protocol, "&"));
	var middleTable = TableManager.instance().getTable(this.middleTableId);
	middleTable.setUrl(replaceParameterValue(middleTable.getUrl(), "protocolFilterValue", protocol, "&"));
	var bottomTable = TableManager.instance().getTable(this.bottomTableId);
	bottomTable.setUrl(replaceParameterValue(bottomTable.getUrl(), "protocolFilterValue", protocol, "&"));
}


/**
 * Sets the source address from a xml layout into the tables in this window.
 * @param layout The xml layout to get the address from.  
 */
EntryDetailWindow.prototype.setXmlSrcAddress = function(layout) {
	var src = getTagNodeValue("src", layout);
	if (isEmpty(src)) {
		return;
	}
	var detailTable = TableManager.instance().getTable(this.tableId);
	detailTable.setUrl(replaceParameterValue(detailTable.getUrl(), "srcFilterValue", src, "&"));
	var middleTable = TableManager.instance().getTable(this.middleTableId);
	middleTable.setUrl(replaceParameterValue(middleTable.getUrl(), "srcFilterValue", src, "&"));
	var bottomTable = TableManager.instance().getTable(this.bottomTableId);
	bottomTable.setUrl(replaceParameterValue(bottomTable.getUrl(), "srcFilterValue", src, "&"));
}


/**
 * Sets the destination address from a xml layout into the tables in this window. 
 * @param layout The xml layout to get the address from.
 */
EntryDetailWindow.prototype.setXmlDstAddress = function(layout) {
	var dst = getTagNodeValue("dst", layout);
	if (isEmpty(dst)) {
		return;
	}
	var detailTable = TableManager.instance().getTable(this.tableId);
	detailTable.setUrl(replaceParameterValue(detailTable.getUrl(), "dstFilterValue", dst, "&"));
	var middleTable = TableManager.instance().getTable(this.middleTableId);
	middleTable.setUrl(replaceParameterValue(middleTable.getUrl(), "dstFilterValue", dst, "&"));
	var bottomTable = TableManager.instance().getTable(this.bottomTableId);
	bottomTable.setUrl(replaceParameterValue(bottomTable.getUrl(), "dstFilterValue", dst, "&"));
}


/**
 * Calculates the width of the content cell.
 * @return the width of the content cell. 
 */
EntryDetailWindow.prototype.calculateContentCellWidth = function() {
	return parseInt(this.htmlCode.style.width) - (Window.RESIZE_AREA_SIZE * 2) - (Window.BORDER_WIDTH * 2);	
}


/**
 * Returns true if a given table name matches one of the tables in this window. 
 * @param tableName The name of the table to check.
 */
EntryDetailWindow.prototype.containsTable = function(tableName) {
	if (this.tableId == tableName || this.middleTableId == tableName || this.bottomTableId == tableName) {
		return true;
	}
	return false;
}