/**
 * The height of the gauge legend in pixels. We need to set this as a fixed size since the
 * legend can be hidden or shown when the windows is hidden and than offsetHeight of the 
 * legend is null.
 */
ProbeWindow.GAUGE_LEGEND_HEIGHT = 14;


/**
 * Factory method to create new ProbeWindow objects.
 * @returns A new probeWindow object.
 * @param id The id of the table in the window. This is needed for catching events from the table.
 * @throws Error if id is undefined, null or empty.
 */
function createProbeWindow(id, tableId) {
	assertNotEmpty(id, "Id to set for the window is undefined, null or empty: " + id);
	
	var superInstance = new Window(id);
	var subInstance = new ProbeWindow(id, tableId);
	subInstance = createSubClass(superInstance, subInstance);
	subInstance.init();
	WindowManager.instance().removeWindow(superInstance);
	return subInstance;
}


/**
 * Constructs a probeWindow.
 * @param id The id of the window.
 * @param id The id of the table in the window. This is needed for catching events from the table.
 * @throws Error if id is undefined, null or empty.
 */
function ProbeWindow(id, tableId) {
	assertNotEmpty(id, "Id to set for the window is undefined, null or empty: " + id);
	
	this.id = id;
	this.tableId = tableId;
	
	// init the event processor
	if (isDefined(this.initEventProcessor)) {
		this.initEventProcessor();
	}
	
	// true if no access is shown in the window
	this.noAccessShown = false;
	
	// contains all the charts detached from this window
	this.detachedCharts = new Array();
	// contains the windows created for the detached charts
	this.detachedChartsWindows = new Array();	
	// true if this is a detached window
	this.detachedWindow = false;
	
	WindowManager.instance().registerNewWindow(this);
}



/**
 * Initiallises this class.
 */
ProbeWindow.prototype.init = function() {
	this.container = this.getContainer();
	if (!isNull(this.container)) {
		this.setResizableElements(this.container);
	}

	this.charts = new Array();
	// this array contains all the chart ids set for the window
	this.chartLayoutSet = new Array();
	this.chartCount = 0;
	this.activeChart = null;
	this.chartSelector = getElementWithAttribute(this.htmlCode, "chartselector", "name", "select");
	this.selectorTextCell = getElementWithAttribute(this.htmlCode, "selectorText", "name", "td");
	this.selectorCell = getElementWithAttribute(this.htmlCode, "chartSelectorCell", "name", "td");
	this.chartSelectorArea = getElementWithAttribute(this.htmlCode, "chartSelectorContainer", "name", "tr");
	// true if the chart section has been colapsed, false otherwise
	this.colapsedChart = false;
	
	this.setDragExceptions();
	
	// the windows drag and resize helper are initialized in the window constructor, we need to change the helpers
	// window reference to be this window not the super instance of Window
	this.dragHelper.window = this;
	this.resizeHelper.window = this;
}


// layout


/**
 * Returns this window as xml.
 */
ProbeWindow.prototype.toXml = function() {
	var xmlString = 
		"<window>" +
			"<windowId>" + this.id + "</windowId>" + 
			"<xPosition>" + this.getXPosition() + "</xPosition>" +
			"<yPosition>" + this.getYPosition() + "</yPosition>" +
			"<width>" + this.getWidth() + "</width>" +
			"<height>" + this.getHeight() + "</height>";
	xmlString += this.activeChartNameToXml();
	xmlString += this.statusToXml();
	xmlString += this.colapsedContentToXml();
	xmlString += this.colapsedChartToXml();
	
	if (isDefined(this.tableId) && !isNull(this.tableId)) {
		xmlString += TableManager.instance().getTable(this.tableId).toXml();
	}
	
	xmlString += this.chartsToXml() + "</window>";
	
	return xmlString;
}


/**
 * Returns the windows chart colapsed status as a xml string.
 */
ProbeWindow.prototype.colapsedChartToXml = function() {
	return "<chartColapsed>" + this.isChartColapsed() + "</chartColapsed>";
}


/**
 * Returns this windows charts as a xml string. This includes detached windows.
 */
ProbeWindow.prototype.chartsToXml = function() {
	var chartsXml = "<charts>" + this.existingChartsToXml() + this.detachedChartsToXml();
	
	// only the active and detached charts get their layout set when it is loaded,
	// other charts are created when they are activated, charts in the layout that are 
	// not created yet need also to be included
	var charts = getCharts(getWindowLayout(this.id));
	for (var i = 0;  i < charts.length; i++) {
		var chartId = getTagNodeValue("chartId", charts[i]);
		if (this.isChartLayoutSet(chartId) && !exists(this.getChart(chartId))) {
			// the has been closed 
			continue;
		}
		if (chartsXml.indexOf(chartId + "</chartId>") == -1) {
			chartsXml += xmlToString(charts[i]);
		}
	}
	
	return chartsXml + "</charts>";
}


/**
 * Returns the xml layout of the existing charts, a empty string if no charts exist.
 */
ProbeWindow.prototype.existingChartsToXml = function() {
	var charts = "";
	var len = this.charts.length;
	for (var i = 0; i < len; i++) {
		charts += this.charts[i].toXml();
	}
	
	return charts;
}


/**
 * Returns the xml layout of the windows detached charts as a string, 
 * empty string if the window has no detached charts.
 */
ProbeWindow.prototype.detachedChartsToXml = function() {
	var detachedChartsXml = "";
	var detachedCharts = this.getDetachedCharts();
	for (var i = 0; i < this.detachedCharts.length; i++) {
		detachedChartsXml += detachedCharts[i].toXml();
	}	
	return detachedChartsXml;
}


/**
 * Returns the name of the active chart as xml string.
 */
ProbeWindow.prototype.activeChartNameToXml = function() {
	if (exists(this.getActiveChart())) {
		return "<activeChartId>" + this.getActiveChart().getId() + "</activeChartId>";
	}
	return "<activeChartId></activeChartId>";
}


/**
 * Sets the layout for the window based on a block of xml.
 * @param xmlLayout The block of xml containing the layout to set. 
 */
ProbeWindow.prototype.setXmlLayout = function(xmlLayout) {
	var chartSectionHidden = (this.chartCount == 0);
	if (chartSectionHidden) {
		// the chart section needs to be visible when the layout is set, or else the height
		// of the window will not be set correct
		this.showCharts();
	}
	this.setXmlContentColapsedStatus(xmlLayout);
	this.setXmlChartColapsedStatus(xmlLayout);
	this.setXmlPosition(xmlLayout);
	this.setXmlActiveChart(xmlLayout);
	this.setXmlSize(xmlLayout);
	this.setXmlStatus(xmlLayout);
	if (chartSectionHidden) {
		this.hideCharts();
	}
}


/**
 * Set the chart colapsed status according to a given xml layout.
 * @param layout The layout containing the given chart colapsed status to set. 
 */
ProbeWindow.prototype.setXmlChartColapsedStatus = function(layout) {
	var status = getTagNodeValue("chartColapsed", layout);
	if  (status == "true" && !this.isChartColapsed()) {
		this.colapseChartSection();
	}
	else if (status == "false" && this.isChartColapsed()) {
		this.expandChartSection();
	}
}


/**
 * Sets the active chart according to a xml layout layout. 
 * @param layout The layout to set for the window. 
 */
ProbeWindow.prototype.setXmlActiveChart = function(layout) {
	var activeChartId = getTagNodeValue("activeChartId", layout);
	if (isEmpty(activeChartId)) {
		// invalid chart id
		return;
	}
	if (isNull(this.getChart(activeChartId))) {
		// the window has no chart with this id
		return;
	}
	selectInSelector(this.chartSelector, activeChartId);	
	this.showSelectedChart();
}


/**
 * Returns true if a chart is contained in the windows
 * current layout, false otherwise. 
 * @param chartId The id of the chart to check.
 * @return True if the chart with the given id is in this windows current layout, 
 * false otherwise and if the layout not has been laoded.
 */
ProbeWindow.prototype.chartExistsInLayout = function(chartId) {
	if (exists(this.getChartLayout(chartId))) {
		return true;
	}
	return false;
}


/**
 * Sets the current layout for a given chart.
 * @param chartId The id of the chart to set the layout for. 
 */
ProbeWindow.prototype.setChartLayout = function(chartId) {
	var chartLayout = this.getChartLayout(chartId);
	if (exists(chartLayout)) {
		setChartLayout(chartLayout, this);
	}
}


/**
 * Returns the current layout for a given chart, null if the chart is not
 * in this windows current layout, or if no layout has been laoded yet.
 * @param chartId The id of the chart to get the layout for. 
 */
ProbeWindow.prototype.getChartLayout = function(chartId) {
	var layout = getWindowLayout(this.id);
	if (!exists(layout)) {
		// layout has not been loaded yet
		return null;
	}
	
	var charts = getCharts(layout);
	var length = charts.length;
	for (var i = 0; i < length; i++) {
		var id = getTagNodeValue("chartId", charts[i]);
		if (chartId == id) {
			return charts[i];
		}
	}
	return null;
}


/**
 * Reset the window. Removes all the interval charts also detached ones.
 * Attaches the topn window if it is detached. 
 */
ProbeWindow.prototype.resetCharts = function() {
	this.clearChartSelector();
	this.clearIntervalCharts();
	this.clearChartContainers();
	this.clearDetachedCharts();
}


/**
 * Clears the chart selector.
 */
ProbeWindow.prototype.clearChartSelector = function() {
	if (!exists(this.chartSelector)) {
		return;
	}
	var length = this.chartSelector.options.length;
	for (var i = length -1; i >= 0; i--) {
		var chartId = this.chartSelector.options[i].value;
		if (exists(this.getChart(chartId)) && (this.getChart(chartId).getType() == Chart.TOPN_CHART || this.getChart(chartId).getType() == Chart.LINE_CHART_CONTAINER)) {
			// do not remove the topn chart from the selector
			continue;	
		}
		removeFromSelector(this.chartSelector, this.chartSelector.options[i].value);
	}
}


/**
 * Clears all the interval charts. 
 */
ProbeWindow.prototype.clearIntervalCharts = function() {
	var intervalCharts = this.getChartsWithType(Chart.INTERVAL_CHART);
	for (var i = 0; i < intervalCharts.length; i++) {
		this.closeChart(intervalCharts[i]);
	}
}


/**
 * Clears all the chart container charts. 
 */
ProbeWindow.prototype.clearChartContainers = function() {
	var chartContainers = this.getChartsWithType(Chart.CHART_CONTAINER);
	for (var i = 0; i < chartContainers.length; i++) {
		this.closeChart(chartContainers[i]);
	}
	this.hideLegend();
}


/**
 * Clears the detached charts. 
 */
ProbeWindow.prototype.clearDetachedCharts = function() {
	// attach the topn window if it is detached
	if (isDefined(this.topNWindow)) {
		this.topNWindow.attach(this.id);
	}
	
	// remove all the detached charts
	for (var i = 0; i < this.detachedCharts.length; i++) {
		this.detachedChartsWindows[i].closeChart(this.detachedCharts[i]);
		this.detachedChartsWindows[i].closeWindow();
	}
	this.detachedCharts = new Array();
	this.detachedChartsWindows = new Array();
}



// mouseover of buttons


/**
 * Hides the collapse button on a mouse out event, if collapsed.
 */
function onMouseOutCollapse(element, windowName, className) {
	setClass(element, className);
	var currentWindow = WindowManager.instance().getWindow(windowName);
	if (currentWindow.colapsedContent == true) {
		setClass(element, "colapseButton WindowButton inactive");
	}
	else {
		setClass(element, className);
	}
}


/**
 * Hides the expand button on a mouse out event, if expanded.
 */
function onMouseOutExpand(element, windowName, className) {
	setClass(element, className);
	var currentWindow = WindowManager.instance().getWindow(windowName);

	if (currentWindow.colapsedContent == false) {
		setClass(element, "expandButton WindowButton inactive");
	}
	else {
		setClass(element, className);
	}
}


/**
 * Hides the collapse button for a chart on a mouse out event, if collapsed.
 */
function onMouseOutCollapseChart(element, windowName, className) {
	setClass(element, className);
	var currentWindow = WindowManager.instance().getWindow(windowName);
	if (currentWindow.colapsedChart == true) {
		setClass(element, "colapseButton WindowButton inactive");
	}
	else {
		setClass(element, className);
	}
}


/**
 * Hides the expand button for a chart on a mouse out event, if expanded.
 */
function onMouseOutExpandChart(element, windowName, className) {
	setClass(element, className);
	var currentWindow = WindowManager.instance().getWindow(windowName);

	if (currentWindow.colapsedChart == false) {
		setClass(element, "expandButton WindowButton inactive");
	}
	else {
		setClass(element, className);
	}
}


/**
 * Hides the chart close button on mouse out when the current chart is not closable.
 * @param button The close button the onmouesout event occured on.
 * @param windowId windowId The id of the window the close chart button is in.
 * @param className The className to set if the chart is closable, if the chart
 * is not closable the class inactive will be set. 
 */
function onMouseOutCloseChart(button, windowId, className) {
	var window = WindowManager.instance().getWindow(windowId);
	if (!isDefined(window) || isNull(window)) {
		return;
	}	
	
	if (isNull(window.getActiveChart())) {
		// the last attached chart was closed deactivate the close button
		setClass(button, "inactive");
		return;
	}
	
	if (window.getActiveChart().isClosable()) {
		setClass(button, className);
	}
	else {
		setClass(button, "inactive");
	}
}


// activate/deactivate 


/**
 * Activates a window. this is overridden to activate the
 * selector bar and change the color when a window is actived.
 */
ProbeWindow.prototype.activateWindow = function() {
	if(this.isActive()) {
		// the window is already active
		return;
	}
	this.callSuper("activateWindow");
	
	if (isDefined(this.chartSelectorArea) && (!isNull(this.chartSelectorArea)) && !this.isChartSelectorHidden()) {
		setClass(this.chartSelectorArea, "chartSelector");
	}
}


/**
 * Deactivates a window. This method is overridden to deactivate
 * the chart selector when the window is deactivated and hide the
 * setup if setup is shown.
 */
ProbeWindow.prototype.deactivateWindow = function() {
	this.callSuper("deactivateWindow");
	if (isDefined(this.setup)) {
		setClass(this.setup, "inactive");
	}	

	setClass(this.htmlCode, "inactiveBorderColor");
	if (isDefined(this.chartSelectorArea) && (!isNull(this.chartSelectorArea)) && !this.isChartSelectorHidden()) {
		setClass(this.chartSelectorArea, "inactiveChartSelector"); 
	}
}


/**
 * Deactivates the header of a window. This method is overriden from Window
 * to deactivate the chart selector when the window is deactivated
 */
ProbeWindow.prototype.deactivateWindowHeader = function() {
	var header = getElementWithAttribute(this.htmlCode, "windowHeader", "id", "tr");
	setClass(header, "inactiveWindowHeader");
}


// visibility


/**
 * Hides a window. This is overridden to deactivate the link
 * in the menu for this window.
 */
ProbeWindow.prototype.hide = function() {
	this.callSuper("hide");
	
	// when a window is hidden safari does not fire the mouseout event on the close button
	// so the mouseover effect of the button is still visible the next time the window is made visible
	// set the correct class for the closebutton manually
	var closeButton = getElementWithAttribute(this.getHtmlCode(), "closeButton", "id", "div");
	if (exists(closeButton)) {
		setClass(closeButton, "closeButton WindowButton");
	}
}


/**
 * Colapse the content of the window. This is overridden to make the window
 * unresizalbe when the chart section is also colpsed. This does not 
 * colapse the chart section only the "windowContentCell" is colapsed.
 */
ProbeWindow.prototype.colapseContent = function() {
	this.callSuper("colapseContent");
	
	if (isDefined(this.container) && !isNull(this.container) && !this.colapsedChart) {
		// the chart section is not colapsed the window is to be resizable
		this.makeResizable();
	}
	this.colapsedContent = true;
}


/**
 * Expands the content of the window.
 */
ProbeWindow.prototype.expandContent = function() {
	this.callSuper("expandContent");
	
	this.colapsedContent = false;
}


/**
 * Returns true if the chart section has been colapsed false otherwise. 
 */
ProbeWindow.prototype.isChartColapsed = function() {
	return this.colapsedChart;
}


/**
 * Colapses the chart area of a window. 
 */
ProbeWindow.prototype.colapseChartSection = function() {
	this.hideChartSection();
	if (this.chartCount == 0) {
		this.hideChartSelector();
	}
	this.showExpandButton();
	this.colapsedChart = true;
	
	if (this.colapsedContent) {
		// the content is also colapsed the window is to be unresizable
		this.makeNotResizable();
	}
	
	this.hideLegend();
	this.fireChartColapsedEvent(this);
}


/**
 * Shows the expand chart area button of this window, the collapse button is hidden.
 */
ProbeWindow.prototype.showExpandButton = function() {
	this.setDivClass("expandChartButton", "expandButton WindowButton");
	this.setDivClass("colapseChartButton", "inactive");
}


/**
 * Expand the chart area.
 */ 
ProbeWindow.prototype.expandChartSection = function() {
	this.showChartSection();
	if (this.chartCount == 0) {
		this.showChartSelector();
	}	
	
	this.showColapseButton();
	this.colapsedChart = false;
	this.makeResizable();
	this.showLegend();
	this.fireChartExpandedEvent(this);
}


/**
 * Shows the collapse chart area button of this window, the expand button is hidden.
 */
ProbeWindow.prototype.showColapseButton = function() {
	this.setDivClass("expandChartButton", "inactive");
	this.setDivClass("colapseChartButton", "colapseButton WindowButton");
}



/**
 * Hides everything regarding to the charts (chart section, legend, chart selector).
 */
ProbeWindow.prototype.hideCharts = function() {
	this.hideLegend();
	this.hideChartSection();
	this.hideChartSelector();
}


/**
 * Hides this windows chart section. The chart section
 * is the chart selector and the space for the charts.
 */
ProbeWindow.prototype.hideChartSection = function() {
	// because of problems in IE we need to set the whole row invisible, not 
	// only the cell representing the container, or else 1 pixel of the row will still be visible,
	// while in Safari setting the row invisible has no effect
	var chartRow = getElementWithAttribute(this.htmlCode, "chartRow", "name", "tr");
	if (!exists(chartRow)) {
		return;
	}
	var chartCell = getElementWithAttribute(this.htmlCode, "chartarea", "id", "td");
	if ("inactive" == chartRow.getAttribute("class") || "inactive" == chartRow.getAttribute("className")) {
		// the chart section is already hidden
		return;
	} 
	
	var container = this.container;
	if (exists(this.activeChart)) {
		this.expandedContainerHeight = parseInt(this.activeChart.getHeight());	
	}
	else {
		this.expandedContainerHeight = parseInt(this.container.style.height);	
	}
	// this can happen if the layout is set and the windows only chart is detached
	// because of a problem in IE the container will than have a height of 1 not 0
	this.expandedContainerHeight = this.expandedContainerHeight == 0 && this.container.offsetHeight == 1 ? 1 : this.expandedContainerHeight;
		
	setClass(chartRow, "inactive");
	setClass(chartCell, "inactive");
	
	// update the size of the window
	var windowHeight = parseInt(this.htmlCode.style.height);
	this.htmlCode.style.height = windowHeight - this.expandedContainerHeight + "px";
	this.height = parseInt(this.htmlCode.style.height);
	
	// the container and the active chart are not to be resized 
	// when they are not shown
	if (!isNull(this.container)) {
		this.removeResizableElement(this.container);
	}
	if (!isNull(this.activeChart)) {
		this.removeResizableElement(this.activeChart.htmlCode);
	}
	
	// set the expanded container height back to 0 when it was set to 1 because of the IE bug above
	this.expandedContainerHeight = parseInt(this.container.style.height) == 0 ? 0 : this.expandedContainerHeight;
}	


/**
 * Hides the chart selector. 
 */
ProbeWindow.prototype.hideChartSelector = function() {
	if (!exists(this.chartSelectorArea)) {
		// the window has no chart selector
		return;
	}
	this.expandedChartSelectorHeight = this.calculateChartSelectorHeight();
	var windowHeight = parseInt(this.htmlCode.style.height);
	this.htmlCode.style.height = windowHeight - this.expandedChartSelectorHeight + "px";
	this.height = parseInt(this.htmlCode.style.height);
		
	// In IE we need to set the the table row invisible, or else one pixel
	// of the row remains visible, while in safari setting a table row
	// invisible has no effect
	setClass(this.chartSelectorArea, "inactive");
	var chartSelectorCell = getElementWithAttribute(this.getHtmlCode(), "chartSelectorCell", "name", "td");
	var chartButtons = getElementWithAttribute(this.getHtmlCode(), "chartButtons", "name", "td");
	setClass(chartSelectorCell, "inactive");
	setClass(chartButtons, "inactive");
}


/**
 * Calculates the height of the chart selector table row.
 * @return The height of the chart selector. 
 */
ProbeWindow.prototype.calculateChartSelectorHeight = function() {
	var height = 0;
	// In safari table rows are reported to have height 0, so we have to measure the
	// on one of the table cells.
	var chartSelectorCell = getElementWithAttribute(this.getHtmlCode(), "chartSelectorCell", "name", "td");
	if (exists(chartSelectorCell)) {
		var chartButtons = getElementWithAttribute(this.getHtmlCode(), "chartButtons", "name", "td");
		height = chartSelectorCell.offsetHeight;
		height = chartButtons.offsetHeight > height ? chartButtons.offsetHeight : height;
		height = this.chartSelectorArea.offsetHeight > height ? this.chartSelectorArea.offsetHeight : height;
	}
	return height;
}



/**
 * Shows everything regarding to the charts (chart section, legend, chart selector).
 */
ProbeWindow.prototype.showCharts = function() {
	this.showChartSection();
	this.showChartSelector();
	this.showLegend();
}


/**
 * Shows this window chart section. The chart section
 * is the chart selector and the space for the charts.
 */
ProbeWindow.prototype.showChartSection = function() {
	if (!exists(this.container)) {
		return;
	}
	// In IE we need to set the row invisible or else one pixel of the row will still be
	// visible, while in Safari setting the row invisible has no effect
	var chartRow = getElementWithAttribute(this.htmlCode, "chartRow", "name", "tr");
	var chartCell = getElementWithAttribute(this.htmlCode, "chartarea", "id", "td");
	var container = this.container;
	var containerHeight = parseInt(container.getAttribute("height"));	

	if ("inactive" == chartRow.getAttribute("class") || "inactive" == chartRow.getAttribute("className")) {
		// because of problems in IE we need to set the whole row invisible, not 
		// only the cell representing the container, or else 1 pixel of the row will still be visible
		setClass(chartRow, "active");
		setClass(chartCell, "active");

		// update the height of the window
		var windowHeight = parseInt(this.htmlCode.style.height);
		this.htmlCode.style.height = windowHeight + this.expandedContainerHeight + "px";
		this.height = parseInt(this.htmlCode.style.height);

		var windowWidth = parseInt(this.htmlCode.style.width);
		windowWidth = windowWidth - (Window.RESIZE_AREA_SIZE * 2) - (Window.BORDER_WIDTH * 2);
		this.container.style.width = windowWidth + "px";

		// add the container and the active chart to the elements
		// that should be resized by the window
		if (!isNull(this.container)) {
			this.setResizableElements(this.container);
		}
		if (!isNull(this.activeChart)) {
			this.activeChart.fitIntoContainer(this.container, true);
			this.setResizableElements(this.activeChart.htmlCode);
		}
	}
}


/**
 * Returns true if the chart selector is hidden.
 */
ProbeWindow.prototype.isChartSelectorHidden = function() {
	var chartRow = getElementWithAttribute(this.htmlCode, "chartSelectorContainer", "name", "tr");
	if ("inactive" == chartRow.getAttribute("class") || "inactive" == chartRow.getAttribute("className")) {
		return true;
	}
	return false;
}


/**
 * Shows the chart selector.
 */
ProbeWindow.prototype.showChartSelector = function() {
	if (!exists(this.chartSelectorArea)) {
		return
	}
	if ("inactive" != this.chartSelectorArea.getAttribute("class") || "inactive" != this.chartSelectorArea.getAttribute("className")) {
		return;
	}
	
	var classToSet = "chartSelector";
	if ("inactiveBorderColor" == this.htmlCode.getAttribute("class") || "inactiveBorderColor" == this.htmlCode.getAttribute("className")) {
		classToSet = "inactiveChartSelector";
	}
	
	// In IE we need to set the the table row invisible, or else one pixel
	// of the row remains visible, while in safari setting a table row
	// invisible has no effect
	setClass(this.chartSelectorArea, classToSet);
	var chartSelectorCell = getElementWithAttribute(this.getHtmlCode(), "chartSelectorCell", "name", "td");
	var chartButtons = getElementWithAttribute(this.getHtmlCode(), "chartButtons", "name", "td");
	setClass(chartSelectorCell, "chartSelector");
	setClass(chartButtons, "chartButtons");
	
	// increase the height of the window
	var chartSelectorHeight = this.expandedChartSelectorHeight;
	var windowHeight = parseInt(this.htmlCode.style.height);
	this.htmlCode.style.height = windowHeight + chartSelectorHeight + "px";
	this.height = parseInt(this.htmlCode.style.height);
}


// attaching/deattaching 


/**
 * Returns true if this window is a detached chart window, false otherwise.
 */
ProbeWindow.prototype.isDetachedWindow = function() {
	return this.detachedWindow;
}


/**
 * Detaches the active chart into a window.
 * @param event The event that triggered the deattach.
 */
ProbeWindow.prototype.detach = function() {
	var chartWindow = this.detachChart(this.activeChart);
	if (this.chartCount > 0) {
		this.showSelectedChart();
	}
	else {
		this.setResizable(false);
	}
	
	// deactive the parent window and activate the new window
	WindowManager.instance().deactivateWindow(this.id)
	WindowManager.instance().activateWindow(chartWindow.getId());
}


/**
 * Detaches a chart. 
 * @param chart The chart to detach. 
 * @return The new window the chart is detached into.
 * @throws Error if chart is undefined or null.
 */
ProbeWindow.prototype.detachChart = function(chart) {
	assertExists(chart, "The chart to detach is undefined or null");
	if (isNull(this.getChart(chart.getId()))) {
		// the chart is not in this window
		return;
	}
	if (this.chartSelector.selectedIndex < 0) {
		return;
	}
	
	var newWindow = createChartWindow(chart, uniqueInteger() + "", chart.getTitle(), this.id);
	newWindow.detachedWindow = true;

	// remove the chart from the window
	removeFromSelector(this.chartSelector, chart.getId());
	this.removeResizableElement(chart.getHtmlCode());
	this.removeChart(chart);	
	
	this.chartDetached(chart, newWindow);
	return newWindow;
}


/**
 * A chart has been detached. 
 * @param chart The chart that has been detached. 
 * @param window The window created for the chart. 
 */
ProbeWindow.prototype.chartDetached = function(chart, window) {
	if (chart == this.activeChart) {
		this.activeChart = null;
	}
	// hide the container if no other charts
	if (this.chartCount == 0) {
		this.hideCharts();
	}	
	
	// if the detached chart is the topn chart save a reference to the window
	if (chart.getType() == Chart.TOPN_CHART) {
		this.topNWindow = window;
	}
	
	// when a window is detached safari does not fire the mouseout event on the detach button
	// so the mouseover effect of the button is still visible after the detach
	// set the correct class for the closebutton manually
	var detachButton = getElementWithAttribute(this.getHtmlCode(), "detachButton", "id", "div");
	setClass(detachButton, "detachButton WindowButton");
	
	this.fireDetachedEvent(chart, window);
	
	this.detachedCharts[this.detachedCharts.length] = chart;
	this.detachedChartsWindows[this.detachedChartsWindows.length] = window;
}


/**
 * Gets the detached chart windows for this window. 
 * @return Array containing window instances for charts that have been detached from this window. 
 */
ProbeWindow.prototype.getDetachedWindows = function() {
	return this.detachedChartsWindows;
}


/**
 * Gets the window a detached chart is in.
 * @param chartId The id of the detached chart. 
 * @return The window a chart is detached into, null if not found. 
 */
ProbeWindow.prototype.getDetachedWindow = function(chartId) {
	var index = this.getDetachedChartIndex(chartId);
	if (index > -1) {
		return this.detachedChartsWindows[index];
	}	
	return null;
}


/**
 * Removes a detached chart. 
 * @param chart The detached chart to remove. 
 */
ProbeWindow.prototype.removeDetachedChart = function(chart) {
	var index = this.getDetachedChartIndex(chart.getId());
	if (index == -1) {
		// chart was not found among the detached charts
		return;
	}
	this.detachedCharts[index] = null;
	this.detachedCharts = removeNullEntries(this.detachedCharts);
	this.detachedChartsWindows[index] = null;
	this.detachedChartsWindows = removeNullEntries(this.detachedChartsWindows);
}


/**
 * Returns a array containing the detached charts.
 */
ProbeWindow.prototype.getDetachedCharts = function() {
	return this.detachedCharts;
}


/**
 * Gets the index of a detached chart into the detachedCharts array.
 * @param chartId The id of the chart to get the index for.
 * @return The index of chart into the detachedCharts array, -1 if not found. 
 */
ProbeWindow.prototype.getDetachedChartIndex = function(chartId) {
	for (var i = 0; i < this.detachedCharts.length; i++) {
		if (chartId == this.detachedCharts[i].getId()) {
			return i;
		}
	}
	return -1;
}


/**
 * Attaches a window to its parent.
 * @param parentWindowId The window to attach to.
 * @throws Error if parentWindow is not defined, null or empty.
 */
ProbeWindow.prototype.attach = function(parentWindowId) {
	assertNotEmpty(parentWindowId, "Windwow to attach to is not defined");
	
	var chartToAttach = this.activeChart;
	var parentWindow = WindowManager.instance().getWindow(parentWindowId);
	chartToAttach.window = parentWindow;
	
	// close window and activate parent window
	this.closeWindow();
	if (parentWindow.isAttachedChartTitle(chartToAttach.getTitle())) {
		// the chart is already attached, close the window, and show the 
		// chart in the parent window
		parentWindow.activateChartWithTitle(chartToAttach.getTitle());
		return;
	}
	
	// add the chart to the parent window
	setClass(this.activeChart.getHtmlCode(), "inactive");
	parentWindow.appendChart(this.activeChart);
	parentWindow.chartAttached(chartToAttach, this);
	
	if (this.chartCount == 1) {
		this.setResizable(true);
	}
}


/**
 * A chart has been attached. 
 * @param chart The chart that has been attached. 
 * @param window The window the chart was in before it was attached. 
 */
ProbeWindow.prototype.chartAttached = function(chart, window) {	
	if (isDefined(this.colapsedChart) && this.colapsedChart) {
		// parent windows chart section is colapsed
		this.expandChartSection();
	}
	else if (this.chartCount == 0) {
		this.showCharts();
	}

	// if the atached chart is the topn chart remove the reference to the topn window
	if (chart.getType() == Chart.TOPN_CHART) {
		this.topNWindow = undefined;
	}
	
	this.fireChartAttachedEvent(chart, this);
	this.removeDetachedChart(chart);
}


// charts


/**
 * Get the current active chart. 
 * @return The currently active chart, null if no chart is active for this window.
 */
ProbeWindow.prototype.getActiveChart = function () {
	return this.activeChart;
}


/**
 * Activates a chart with a given title. 
 * @param title The title of the chart to activate. 
 * @throws Error if title is undefined, null or empty. 
 */
ProbeWindow.prototype.activateChartWithTitle = function(title) {
	assertNotEmpty(title, "Title of the chart to activate is undefined, null or empty");
	if (!this.isAttachedChartTitle(title)) {
		// the chart does noe exits
		return;
	}
	var chart = this.getChartWithTitle(title);
	selectInSelector(this.chartSelector, chart.getId());
	this.showSelectedChart();
}


/**
 * Activates a chart with a given id.
 * @param id The id of the chart to activate.
 * @throws Error if id is undefined, null or empty. 
 */
ProbeWindow.prototype.activateChart = function(id) {
	assertNotEmpty(id, "The id of the chart to activate is undefined, null or empty");
	if (!this.isAttachedChartId(id)) {
		// the chart does not exist
		return;
	}
	
	selectInSelector(this.chartSelector, id);
	this.showSelectedChart();
}


/**
 * Remove a chart.
 * @param chart The chart to remove.
 */
ProbeWindow.prototype.removeChart = function(chart) {
	assertDefined(chart, "The chart is not defined");
	var len = this.charts.length;
	for (var i = 0; i < len; i++) {
		if (isDefined(this.charts[i]) && !isNull(this.charts[i]) && chart.getId() == this.charts[i].getId()) {
			this.charts[i] = null;
			this.chartCount--;
			this.charts = removeNullEntries(this.charts);
		}
	}
}	
 

/**
 * Get a chart.
 * @param the id of the chart to get.
 * @return The chart object, null if not found.
 * @throws Error if id is undefined or empty.
 */
ProbeWindow.prototype.getChart = function(id) {
	assertNotEmpty(id, "Invalid id: " + id);
	
	if (this.charts.length == 0) {
		return null;
	}

	var len = this.charts.length;
	for (var i = 0; i < len; i++) {
		if (id == this.charts[i].getId()) {
			return this.charts[i];
		}
	}
	return null;
}


/**
 * Get a chart with a given title. 
 * @param title The title of the chart to get. 
 * @return The first chart with the given title, null if not found.
 * @throws Error if title is undefined, null or empty. 
 */
ProbeWindow.prototype.getChartWithTitle = function(title) {
	assertNotEmpty(title, "Title to get chart for is undefined, null or empty");
	
	if (this.charts.length == 0) {
		return null;
	}
	
	var len = this.charts.length;
	for (var i = 0; i < len; i++) {
		if (title == this.charts[i].getTitle()) {
			return this.charts[i];
		}
	}
	return null;
}


/**
 * Gets this windows topn chart, null if this window has no topn chart.
 */
ProbeWindow.prototype.getTopNChart = function() {
	if (isDefined(this.topNWindow)) {
		return this.topNWindow.getActiveChart();
	}
	var chart = this.getChartWithType(Chart.TOPN_CHART);
	if (chart == undefined) {
		return null;
	}
	return chart;
}


/**
 * Get the charts.
 * @return A array containing all the chart objects in this window. 
 */
ProbeWindow.prototype.getCharts = function() {
	return this.charts;
}


/**
 * Returns the first chart of a given type, undefined if no chart of this type
 * is attached to the window.
 * @param type The type to get.
 */
ProbeWindow.prototype.getChartWithType = function(type) {
	for (var i = 0; i < this.charts.length; i++) {
		if (type == this.charts[i].getType()) {
			return this.charts[i];
		}
	}	
	return undefined;
}


/**
 * Gets all the charts with a given chart type. 
 * @param type The type to get all the charts for. 
 * @return A array containing all the charts for that type. 
 * @param throws Error if type is undefined or null.
 */
ProbeWindow.prototype.getChartsWithType = function(type) {
	assertExists(type, "Type to get all charts for is undefined or null");
	var chartsWithType = new Array();
	for (var i = 0; i < this.charts.length; i++) {
		if (type == this.charts[i].getType()) {
			chartsWithType[chartsWithType.length] = this.charts[i];
		}
	}
	return chartsWithType;
}


/**
 *
 * Shows the chart window selected in this windows chart drop down box.
 */
ProbeWindow.prototype.showSelectedChart = function() {
	var chartSelector = getElementWithAttribute(this.htmlCode, "chartselector", "name", "select");
	var chartToShow = chartSelector.value;
	if (!isNull(this.activeChart) && chartToShow == this.activeChart.title) {
		// the chart to be shown is already the active chart
		return;
	}

	if (isEmpty(chartToShow)) {
		// no chart to show
		return;
	}
	else if (!exists(this.getChart(chartToShow)) && this.chartExistsInLayout(chartToShow)) {
		// the chart has not been inited yet
		this.setChartLayout(chartToShow);
		return;
	}

	if (this.activeChart == this.getChart(chartToShow)) {
		return;
	}
	this.hideProbeChart(this.activeChart);
	var chart = this.getChart(chartToShow);
	if (isNull(chart) || !isDefined(chart)) {
		return;
	}
	this.showProbeChart(chart);
	this.showSelectedLegend(chart);
}

 
/**
 * Hides a chart.
 * @param chart The probechart to hide.
 */
ProbeWindow.prototype.hideProbeChart = function(chart) {
	if (!isNull(chart)) {
		setClass(chart.htmlCode, "inactive");
		this.removeResizableElement(chart.htmlCode);
		this.fireChartHiddenEvent(chart);
	}
}


/**
 * Shows a chart. 
 * @param chart The probechart to show.
 */
ProbeWindow.prototype.showProbeChart = function(chart) {
	setClass(chart.htmlCode, "chartContainer");
	this.activeChart = chart;
	this.setResizableElements(chart.getHtmlCode());
	this.fireChartShownEvent(chart);
	
	var closeChartButton = getElementWithAttribute(this.htmlCode, "closeChartButton", "id", "div");	
	if (!chart.isClosable()) {
		// hide the close button
		setClass(closeChartButton, "inactive");
	}
	else if (getClassName(closeChartButton) == "inactive") {
		setClass(closeChartButton, "closeButton WindowButton");
	}
}


/**
 * Changes the size of the container by a given amount of pixels. 
 * @param deltaHeight The amount of pixels to change the height of the container with. 
 */
ProbeWindow.prototype.changeContainerSize = function(deltaHeight) {
	if (isDefined(this.container) && !isNull(this.container)) {
		var containerHeight = parseInt(this.container.style.height);
		var newContainerHeight = containerHeight + deltaHeight;
		this.setContainerSize(newContainerHeight);
	}
}


/**
 * Sets the container height according to the height
 * of a given chart.
 * @param height The height to set the container height to. 
 */
ProbeWindow.prototype.setContainerSize = function(height) {
	var containerHeight = parseInt(this.container.style.height);
	var windowHeight = parseInt(this.htmlCode.style.height);
	var containerHeightDiff = containerHeight - height
	this.container.style.height = height + "px";
	this.htmlCode.style.height = (windowHeight - containerHeightDiff) + "px"; 
	this.height = this.height - containerHeightDiff;
}


/**
 * Appends a new probe chart to the window.
 * @param chartToAdd The probechart object to append.
 * @throws Error if chartToAappend is not defined.
 */
ProbeWindow.prototype.appendChart = function(chartToAppend) {
	assertDefined(chartToAppend, "chart to append not defined");
	if (this.chartCount == 0) {
		this.showCharts();
	}
	if (this.chartCount == 0) {
		this.setResizable(true);
	}
	this.container.appendChild(chartToAppend.htmlCode);
	this.setContainerSize(chartToAppend.getHeight());
	this.addChart(chartToAppend);
}


/**
 * Adds a probe chart to the window. The html code of the window
 * must be already appended to the window.
 * @throws Error if chartToAdd is not defined.
 */
ProbeWindow.prototype.addChart = function(chartToAdd) {
	assertDefined(chartToAdd, "chart to add not defined");
	if (this.chartCount == 0) {
		this.showCharts();
	}
	if (this.chartCount == 0) {
		this.setResizable(true);
	}
	
	this.charts[this.chartCount] = chartToAdd;
	this.chartLayoutSet[this.chartLayoutSet.length] = chartToAdd.getId();
	this.chartCount++;
	if (getIndexInSelector(this.chartSelector, chartToAdd.getId()) == -1) {
		// the chart is not in the selector yet, charts from the layout that are not shown
		// may already be in the selector
		addToSelector(this.chartSelector, chartToAdd.getId(), chartToAdd.getTitle(), true);
	}
	if(!this.isActive() && isIE()) {
		// the select tag is hidden also add the chart to the select replace div
		this.addToSelectReplaceDiv(chartToAdd.getTitle(), this.chartSelector);
	}
	this.showSelectedChart();
}


/**
 * Adds some text to the replace div for a select.
 * @param text The text set in the replace div.
 * @param select The select tag to insert some text in the replace tag for.
 * @param className The class to set for the select replace div tag. 
 */
ProbeWindow.prototype.addToSelectReplaceDiv = function(text, select, className) {
	var replaceField = this.getReplaceField(select);
	if(!isDefined(replaceField)) {
		replaceField = this.createReplaceField(select);
		this.replaceFields[this.getSelectIndex(select)] = replaceField;
		this.selectReplaceClasses[this.getSelectIndex(select)] = className;
		this.replaceSelect(select, replaceField);
		return;
	}
	var text = select.options[select.selectedIndex].firstChild.nodeValue;
	var nobr = document.createElement("nobr");
	nobr.appendChild(document.createTextNode(text));
	if (replaceField.hasChildNodes()) {
		replaceField.replaceChild(nobr, replaceField.firstChild);
	}
	else {
		replaceField.appendChild(nobr);
	}
}


/**
 * Close the active chart. The html code of the chart
 * is removed from the window.
 */
ProbeWindow.prototype.closeActiveChart = function() {
	this.closeChart(this.activeChart);
}


/**
 * Closes a chart. 
 * @param chart The chart to remove. 
 * @throws Error if the chart to remove is undefined or null.
 */
ProbeWindow.prototype.closeChart = function(chart) {
	assertDefined(chart, "The chart to remove is undefined or null");
	
	chart.htmlCode.onload = "";
	ChartManager.instance().removeChart(chart);
	this.fireChartClosedEvent(chart);

	// remove the html code of the chart
	if (!isNull(this.container)) {
		this.container.removeChild(chart.htmlCode);
	}

	// remove the chart from the chart selector
	if (!isNull(this.chartSelector)) {
		removeFromSelector(this.chartSelector, chart.getId());
		this.showSelectedChart();
	}

	this.removeChart(chart);
	if (this.chartCount == 0 && !isNull(this.chartSelector)) {
		this.activeChart = null;
		this.hideCharts();
		this.hideLegend();
	}
	this.removeResizableElement(chart.getHtmlCode());
}


/**
 * Returns true if a chart has set its layout, false otherwise.
 * @param id The id to check.
 * @thrwos Error if id is undefined, null or empty.
 */
ProbeWindow.prototype.isChartLayoutSet = function(id) {
	assertNotEmpty(id, "The id to check is undefined, null or empty");
	var len = this.chartLayoutSet.length;
	for (var i = 0; i < len; i++) {
		if (this.chartLayoutSet[i] == id) {
			return true;
		}
	}
	return false;
}


/**
 * Returns true if a chart with a given id is attached in this window
 * @param id The id to check.
 */
ProbeWindow.prototype.isAttachedChartId = function(id) {
	var len = this.charts.length;
	for (var i = 0; i < len; i++) {
		if (this.charts[i].getId() == id) {
			return true;
		}
	}
	
	return false;
}


/**
 * Returns true if a chart with a given title is attached in this window. 
 * @param title The tilte to check.
 */
ProbeWindow.prototype.isAttachedChartTitle = function(title) {
	var len = this.charts.length;
	for (var i = 0; i < len; i++) {
		if (this.charts[i].getTitle() == title) {
			return true;
		}
	}
	
	return false;
}


/**
 * Returns true if a given id is the id of the avtive chart, false otherwise.
 * @param id The id to check.
 * @throws Error if the id is undefined, empty or null.
 */
ProbeWindow.prototype.isActiveChartId = function(id) {
	assertNotEmpty(id, "The id to check is undefined, empty or null");
	
	if (!isDefined(this.activeChart) || isNull(this.activeChart)) {
		return;
	}
	
	if (id == this.activeChart.getId()) {
		return true;
	}
	return false;
}


// legend


/**
 * Shows a legend.
 */
ProbeWindow.prototype.showLegend = function() {
	// show the gauge chart legend
	// because of problems in IE we need to set the whole row invisible, not 
	// only the cell representing the legend, or else 1 pixel of the row will still be visible
	var gaugeChartLegendRow = getElementWithAttribute(this.htmlCode, "gaugeLegendRow", "id", "tr");
	var gaugeChartLegend = getElementWithAttribute(this.htmlCode, "gaugeLegend", "name", "td");
	if (!exists(gaugeChartLegendRow) || !exists(gaugeChartLegend) || !this.hasLegend(this.activeChart)) {
		return;
	}
	
	if ("inactive" == gaugeChartLegend.getAttribute("class") || "inactive" == gaugeChartLegend.getAttribute("className")) {
		setClass(gaugeChartLegendRow, "active");
		setClass(gaugeChartLegend, "gaugeLegend");
	
		// update the size of the window
		var windowHeight = parseInt(this.htmlCode.style.height);
		this.htmlCode.style.height = windowHeight + ProbeWindow.GAUGE_LEGEND_HEIGHT + "px";
		this.height = parseInt(this.htmlCode.style.height);
	}
}


/**
 * Hides the gauge legend.
 */
ProbeWindow.prototype.hideLegend = function() {
	// update the size of the window
	var gaugeChartLegendRow = getElementWithAttribute(this.htmlCode, "gaugeLegendRow", "id", "tr");
	var gaugeChartLegend = getElementWithAttribute(this.htmlCode, "gaugeLegend", "name", "td");
	if (!exists(gaugeChartLegendRow) || !exists(gaugeChartLegend)) {
		return;
	}
	
	if ("inactive" != gaugeChartLegend.getAttribute("class") || "inactive" != gaugeChartLegend.getAttribute("className")) {
		var windowHeight = parseInt(this.htmlCode.style.height);
		this.htmlCode.style.height = (windowHeight - ProbeWindow.GAUGE_LEGEND_HEIGHT) + "px";
		this.height = parseInt(this.htmlCode.style.height);
		
		// because of problems in IE we need to set the whole row invisible, not 
		// only the cell representing the legend, or else 1 pixel of the row will still be visible,
		// while in safari setting the class on the row has no effect
		setClass(gaugeChartLegend, "inactive");
		setClass(gaugeChartLegendRow, "inactive");
	}
}


/**
 * Returns true if a chart has a legend, false otherwise.
 * @param chart The chart to check if it has a legend.
 */
ProbeWindow.prototype.hasLegend = function(chart) {
	if (!exists(chart)) {
		return false;
	}
	var gaugeChartLegend = getElementWithAttribute(this.htmlCode, "gaugechartlegend", "id", "img");
	if (this.id == "summarywindow" && chart.getId().indexOf("Packets") > -1) {
		return true;
	}
	else if (this.id == "summarywindow" && chart.getId().indexOf("Bytes") > -1) {
		return true;
	}
	else if (exists(gaugeChartLegend) && this.id != "summarywindow") {
		return true;
	}
	return false;
}


/**
 * Shows the selected charts legend if one is present.
 * @param chart The chart to show the legend for.
 */
ProbeWindow.prototype.showSelectedLegend = function(chart) {
	if (!this.hasLegend(chart)) {
		this.hideLegend();
		return;
	}
	
	this.showLegend();
}


// dragging


/**
 * The window is not to be moved on any of its buttons, add them to the 
 * drag exceptions. This method is overridden to add the topN input field
 * to the drag exceptions.
 */
ProbeWindow.prototype.setDragExceptions = function() {
	this.callSuper("setDragExceptions");
	this.addDragException("topN", "name", "input");
	this.addDragException("indexSelecttopNProtocolsTable", "name", "select");
	var indexSelect = getElementWithAttribute(this.getHtmlCode(), "indexSelect", "name", "select");
	if (!exists(indexSelect)) {
		return;		
	}
	
	this.dragHelper.addDragException(indexSelect);
	// also add all the selects options to the exception, if not added firefox will drag the window
	// on click on the options instead of changing the selected index in the select 
	for (var i = 0; i < indexSelect.options.length; i++) {
		this.dragHelper.addDragException(indexSelect.options[i]);
	}
	
}


// resizing


/**
 * Resizes the elements that are to be resized along with the window.
 * @param deltaHeight The amount of pixels to increase the height with.
 * @param deltaWidth The amount of pixels to increase the width with.
 * @param fireResizeEvent, optional argument, if this is set to false no resize event will be fired
 */
ProbeWindow.prototype.resizeElements = function(deltaHeight, deltaWidth, fireResizeEvent) {
	if (this.resizableElements.length > 0) {
		resizeHtmlElements(this.resizableElements, deltaHeight, deltaWidth);
	}
	
	if (!isDefined(fireResizeEvent) || (isDefined(fireResizeEvent) && fireResizeEvent)) {
		this.fireResizeEvent(deltaHeight, deltaWidth);	
	}
}


/**
 * Called when a window has been resized.
 * This is overridden to reload charts on resizes.
 */
ProbeWindow.prototype.resized = function() {
	// during the resize the resize area can have been made invisible, expand
	// the size of the window to show it if neccessary
	this.makeResizeAreaVisible();
	// make sure the size of the window is correct 
	this.checkWindowSize();
	if (this.activeChart) {
		this.checkActiveChartSize();	
	}
	if (this.container) {
		this.checkContainerSize();
	}
	this.relaodChart();
}


/**
 * Relaod the chart if it exists. 
 */
ProbeWindow.prototype.relaodChart = function() {
	if (!isDefined(this.activeChart) || isNull(this.activeChart)) {
		// no chart to resize
		return;
	} 
	if (this.noAccessShown) {
		// no access is shown, do not resize the chart
		return;
	}
	this.activeChart.restartUpdate();
}


/**
 * If the size of the active chart is not correct according to the
 * size of the window it is adjusted. After a resize the size of the 
 * active chart can be different from the windows size. This can happen if 
 * the window was made smaller than it can be.
 */
ProbeWindow.prototype.checkActiveChartSize = function() {
	var windowContentWidth = parseInt(this.htmlCode.style.width) - this.getBorderWidth();
	if (this.activeChart) {
		this.activeChart.setWidth(windowContentWidth);
	}
}


/**
 * If the container size is not correct according to the
 * size of the window it is adjusted. After a resize the size of the 
 * container can be different from the windows size. This can happen if 
 * the window was made smaller than it can be.
 */
ProbeWindow.prototype.checkContainerSize = function() {
	var containerWidth = parseInt(this.container.style.width);
	var windowContentWidth = parseInt(this.htmlCode.style.width) - this.getBorderWidth();
	if (containerWidth != windowContentWidth && windowContentWidth > 0) {
		this.container.style.width = windowContentWidth + "px";
	}
}


/**
 * Returns the border width in pixels. This includes the resize area and any border around the window.
 */
ProbeWindow.prototype.getBorderWidth = function() {
	var borderWidth = (Window.RESIZE_AREA_SIZE * 2);
	if (!isOpera()) {
		borderWidth = borderWidth + (Window.BORDER_WIDTH * 2);
	}
	return borderWidth;
}


/**
 * Increase the size of the window to make the resizeArea visible if neccessary.
 * This method is overriden to reload the chart if any.
 */
ProbeWindow.prototype.makeResizeAreaVisible = function() {
	returnValue = this.callSuper("makeResizeAreaVisible");
	
	// reload the chart if any
	if (returnValue && !isNull(this.activeChart)) {
		this.activeChart.updateChart();
	}
	return returnValue;
}


// util functions


/**
 * Gets the windows title. This is overridden to get the correct title for topn windows.
 * A topn window title might look like this: "Top  hosts indexed by Bytes Packets Conversations Protocols",
 * format this to like: "Top hosts". 
 * @return The title of the window.
 */
ProbeWindow.prototype.getWindowTitle = function () {
	var titleNode = getElementWithAttribute(this.getHtmlCode(), "windowTitle", "name", "td");
	var title = trimNewLines(trimTabs(getTekstValue(titleNode)));
	
	if (title.indexOf("Top") == -1) {
		return title;	
	}
	var start = title.indexOf("indexed");
	if (start > -1) {
		title = title.substring(0, start);
	}
	return trim(title);
}


/**
 * Set the class of a div tag. 
 * @param id The id of the div.
 * @param classToSet The class to set for the div tag.
 */
ProbeWindow.prototype.setDivClass = function(id, classToSet) {
	var divTag = getElementWithAttribute(this.htmlCode, id, "id", "div");
	if (exists(divTag)) {
		setClass(divTag, classToSet);
	}
}


/**
 * Removes a given option from a selector.
 * @param selector The selector to remove the option from.
 * @param option The value of the option to remove.
 */
function removeFromSelector(selector, option) {
	assertDefined(selector, "Selector is not defined");
	assertDefined(option, "The option is not defined");

	var removeOption = getElementWithAttribute(selector, option, "value", "option");
	if (getIndexInSelector(selector, option) > -1) {
		selector.removeChild(removeOption);
	}
}


/**
 * Adds a given option to a selector.
 * @param selector The selector to add the option to.
 * @param value The value to add for the option.
 * @param text The text to be shown in the drop down box for this option.
 * @param select Set this to true if the new option should be selected, false to not select.
 * @throws Error if the selector is not defined.
 * @throws Error if the value is not defined.
 * @throws Error if the text is not defined.
 */
function addToSelector(selector, value, text, select) {
	assertDefined(selector, "Selector is not defined");
	assertDefined(value, "Value is not defined");
	assertDefined(text, "Text is not defined");

	var option = document.createElement("option");
	option.setAttribute("value", value);
	var textNode = document.createTextNode(text);
	option.appendChild(textNode);

	// insert the new option in the first position
	var firstOption = selector.options[0];
	selector.insertBefore(option, firstOption);
	if (select) {
		selector.selectedIndex = 0;
	}
}


// events


/**
 * Inits the event processor.
 */
ProbeWindow.prototype.initEventProcessor = function() {
	this.eventListener = new Listener(this);
	
	this.eventListener.addEvent("probeNameChanged", probeNameChanged);
	this.eventListener.addEvent("noAccessShown", processNoAccessShown);
	this.eventListener.addEvent("noAccessHidden", processNoAccessHidden);
	this.eventListener.addEvent("chartResized", processChartResizedEvent);
	this.eventListener.addEvent("tableResized", processTableResizedEvent);
	this.eventListener.addEvent("topNChanged", processTopNChangedEvent);
}


/**
 * Get the event processor of this window.
 * @return This windows event processor, undefined if the event processor has not ben inited for this table.
 */
ProbeWindow.prototype.getEventProcessor = function() {
	return this.eventListener;	
}


/**
 * Processes a topn changed event. 
 * @param event The event to process. 
 */
function processTopNChangedEvent(event) {
	if (this.tableId != event.getData()["id"]) {
		return;
	} 
	
	var topNInput = getElementWithAttribute(this.getHtmlCode(), "topN", "name", "input");
	if (!isNull(topNInput) && event.getData()["n"] != topNInput.value) {
		topNInput.value = event.getData()["n"];
	}
}


/**
 * Processes a table resize event.
 * @param event The event to process.
 */
function processTableResizedEvent(event) {
	if (this.tableId != event.getData()["id"]) {
		return;
	} 
	
	var width = event.getData()["width"];
	var height = event.getData()["height"];
	var resizedHeight = this.resizeWindowHeight(height);
	var resizedWidth = this.resizeWindowWidth(width);
	this.resizeElements(height, width, false);
	this.resized();
}


/**
 * Processes a chart resized event.
 * @param event The event to process. 
 */
function processChartResizedEvent(event) {
	if (!this.isActiveChartId(event.getData()["id"])) {
		return;
	}
	
	this.changeContainerSize(event.getData()["deltaHeight"]);
}


/**
 * Processes a no access hidden event from a table. 
 * @param event The event to process. 
 */
function processNoAccessHidden(event) {
	if (!isDefined(this.tableId) || !this.tableId.equalsIgnoreCase(event.getData())) {
		return;
	}

	// restore the original width before the chart section is shown, because else the chart width
	// is going to be incorrect
	this.htmlCode.style.width = this.originalWidth + "px";
	
	if (this.charts.length > 0) {
		this.showChartSelector();
		this.showChartSection();
	}
	if (isDefined(this.activeChart) && !isNull(this.activeChart) && this.hasLegend(this.activeChart)) {
		this.showLegend();
	}
	this.noAccessShown = false;
	
	// restore the originl window height
	this.htmlCode.style.height = this.originalHeight + "px";
	this.resized();
}


/**
 * Processes a no acces shown event from a table. 
 * @param event The event to process.
 */
function processNoAccessShown(event) {
	if (!isDefined(this.tableId) || !this.tableId.equalsIgnoreCase(event.getData())) {
		return;
	}

	// save the original height before the no access is shown
	this.originalHeight = parseInt(this.htmlCode.style.height);
	this.originalWidth = parseInt(this.htmlCode.style.width);
	
	// set the height of the no access table to 103 and the width to match the width of the table
	var table = TableManager.instance().getTable(this.tableId);
	var heightDiff = 103 - parseInt(table.getTableContainerCode().style.height);
	table.noAccess.style.height = "103px";
	this.htmlCode.style.height = (parseInt(this.htmlCode.style.height) + heightDiff) + "px";
	table.noAccess.style.width = table.getTableContainerCode().style.width;

	this.hideChartSection();
 	this.hideChartSelector();
 	if (isDefined(this.activeChart) && !isNull(this.activeChart) && this.hasLegend(this.activeChart)) {
		this.hideLegend();
	}
	
	this.noAccessShown = true;
}


/**
 * Processes a probe name changed event.
 * @param event The event to process.
 */
function probeNameChanged(event) {
	if ("summarywindow" != this.id) {
		return;
	}
	// replace the old probename in the title with the new one
	var oldName = event.getData()["oldProbeName"];
	var newName = event.getData()["newProbeName"];
	
	if (oldName == newName || isEmpty(oldName) || isEmpty(newName)) {
		return;
	}
	
	var options = this.chartSelector.options;
	var optionLength = options.length;
	for (var i = 0; i < optionLength; i++) {
		var optionText = options[i].firstChild.nodeValue;
		if(optionText.indexOf(oldName) > -1) {
			var newText = replace(optionText, oldName, newName);
			this.chartSelector.options[i].firstChild.nodeValue = newText;
			var newOptionValue = replace(options[i].getAttribute("value"), oldName, newName);
			this.chartSelector.options[i].setAttribute("value", newOptionValue);
		}
	}
}


/**
 * Fires a resize event.
 * @param deltaHeight The amount of pixels to increase the height with.
 * @param deltaWidth The amount of pixels to increase the width with.
 */
ProbeWindow.prototype.fireResizeEvent = function(deltaHeight, deltaWidth) {
	var eventData = new Array();
	eventData["id"] = this.id;
	eventData["height"] = deltaHeight;
	eventData["width"] = deltaWidth;

	getEventServer().notifyListeners(new Event("resized", eventData));
}


/**
 * Fires a chart closed event. 
 * @param chart The chart closed. 
 */
ProbeWindow.prototype.fireChartClosedEvent = function(chart) {
	getEventServer().notifyListeners(new Event("chartClosed", chart));
}


/**
 * Fires a chart shown event. 
 * @param chart The chart that has been shown. 
 */
ProbeWindow.prototype.fireChartShownEvent = function(chart) {
	getEventServer().notifyListeners(new Event("chartShown", chart));
}


/**
 * Fire a chart hidden event. 
 * @param chart The chart that has been hidden.
 */
ProbeWindow.prototype.fireChartHiddenEvent = function(chart) {
	getEventServer().notifyListeners(new Event("chartHidden", chart));
}


/**
 * Fires a chart attached event. 
 * @param chart The chart attached. 
 * @param window The window the chart has been attached to.
 */
ProbeWindow.prototype.fireChartAttachedEvent = function(chart, window) {
	var arguments = new Array();
	arguments["chart"] = chart;
	arguments["window"] = window;
	getEventServer().notifyListeners(new Event("chartAttached", arguments));
}


/**
 * Fire a chart detached event. 
 * @param chart The chart that has been detached. 
 * @param window The window the chart has been detached into. 
 */
ProbeWindow.prototype.fireDetachedEvent = function(chart, window) {
	var arguments = new Array();
	arguments["chart"] = chart;
	arguments["window"] = window;
	getEventServer().notifyListeners(new Event("chartDetached", arguments));
}


/**
 * Fire a chart expanded event. 
 * @param window The window whose chart section has been expanded. 
 */
ProbeWindow.prototype.fireChartExpandedEvent = function(window) {
	getEventServer().notifyListeners(new Event("chartExpanded", window));
}


/**
 * Fire a chart colapsed event. 
 * @param window The window whose chart section has been colapsed. 
 */
ProbeWindow.prototype.fireChartColapsedEvent = function(window) {
	getEventServer().notifyListeners(new Event("chartColapsed", window));
}


// html


/**
 * Return the html code of ths windows container. 
 */
ProbeWindow.prototype.getContainer = function() {
	var tds = this.htmlCode.getElementsByTagName("td");
	var len = tds.length;
	for (var i = 0; i < len; i++) {
		if ("chartarea" == tds[i].getAttribute("id")) {
			return tds[i];
		}	
	}
	return null;
}