

// construction


/**
 * The single instance of this class.
 */
var ProfileManager = new Object();


// constants


/**
 * The valid characters/digits used for the profile name.
 */
ProfileManager.LEGAL_PROFILE_NAME_CHARACTERS = "abcdefghijklmnopqrstuvwxyz1234567890_- ";
	

/**
 * The base url used to add profiles.
 */
ProfileManager.BASE_ADD_ENTRY_URL = "/?resource=ProfileManager&mode=set&command=profile&action=set&sid=" + getSid() + "&";


/**
 * The url used to load profiles from the server.
 */
ProfileManager.LOAD_PROFILES_URL = "/?resource=ProfileManager&mode=get&command=allprofiles&sid=" + getSid() + "&";

/**
 * The base url used to delete a profile. 
 */
ProfileManager.BASE_DELETE_PROFILES_URL = "/?resource=ProfileManager&mode=set&command=profile&action=delete&sid=" + getSid() + "&";


/**
 * The url to load the active profile.
 */
ProfileManager.ACTIVE_PROFILE_URL = "/?resource=ProfileManager&mode=get&command=activeProfile&sid=" + getSid() + "&";


/**
 * The url to load the default profile.
 */
ProfileManager.DEFAULT_PROFILE_URL = "/?resource=ProfileManager&mode=get&command=defaultprofile&sid=" + getSid() + "&";


/**
 * The url to activate a profile.
 */
ProfileManager.ACTIVATE_PROFILE_URL = "/?resource=ProfileManager&mode=set&command=activateProfile&sid=" + getSid() + "&";


/**
 * The url to set a default profile.
 */
ProfileManager.SET_DEFAULT_PROFILE_URL = "/?resource=ProfileManager&mode=set&command=defaultprofile&action=set&sid=" + getSid() + "&";


/**
 * The url used to change a profile.
 */
ProfileManager.CHANGE_PROFILE_URL = "/?resource=ProfileManager&mode=set&command=changeProfileName&sid=" + getSid() + "&";;


/**
 * The max width set for the profile menu. 
 */
ProfileManager.PROFILE_MENU_MAX_WIDTH = "400";


/**
 * The error message shown if there are invalid characters in the profile name. 
 */
ProfileManager.INVALID_CHAR_IN_PROFLIE_NAME_MESSAGE = "Invalid characters in profile name. Valid characters are a-z, space, digits and - _";


/**
 * The message returned from the server if the active profile was deleted.
 */
ProfileManager.ACTIVE_PROFILE_DELETED_MESSAGE = "Can not delete a active profile";


/**
 * The message returned from the server if the default profile was removed.
 */
ProfileManager.DEFAULT_PROFILE_DELETED_MESSAGE = "Can not delete the default profile";


/**
 * The message returned from the server if a active profile was edited.
 */
ProfileManager.ACTIVE_PROFILE_EDITED_MESSAGE = "Can not change the name of a active profile";


/**
 * The message returned from the server if a default profile was edited. 
 */
ProfileManager.DEFAULT_PROFILE_EDITED_MESSAGE = "Can not change the name of the default profile";


// object connections


/**
 * The container where messages are set.
 */
ProfileManager.messageContainer;


/**
 * The profile managers event processor.
 */
ProfileManager.eventListener = new Listener(ProfileManager);


/**
 * Add the windowShown event to the events processed by the profileManager.
 */
ProfileManager.eventListener.addEvent(WINDOW_SHOWN_EVENT, processWindowShownEventProfileManager);


/**
 * Add the layout set event to the events processed by the profileManager. 
 */
ProfileManager.eventListener.addEvent(LAYOUT_SET_EVENT, processLayoutSetEventProfileManager);


/**
 * The array where all the profiles are stored.
 */
ProfileManager.profiles = new Array();


/**
 * The name of the active profile.
 */
ProfileManager.activeProfile = "";


/**
 * The default profile.
 */ 
ProfileManager.defaultProfile = null;


/**
 * The loading animation used when profiles are loaded.  
 */
ProfileManager.loadingAnimation;


// services


/**
 * Adds a profile.
 */
ProfileManager.addProfile = function() {
	var profileTable = TableManager.instance().getTable("profilesTable");
	var nameField = getElementWithAttribute(profileTable.getTableContainerCode(), "addNameField", "name", "input");
	var descriptionField = getElementWithAttribute(profileTable.getTableContainerCode(), "addDescriptionField", "name", "input");
	var name = nameField.value;
	if (isEmpty(name)) {
		this.setMessage(undefined, "Name is empty");
		return;
	}
	if (this.existsProfile(name)) {
		this.setMessage(undefined, "A profile with that name already exists");
		return;
	}
	if (!this.isValidProfileName(name)) {
		this.setMessage(undefined, ProfileManager.INVALID_CHAR_IN_PROFLIE_NAME_MESSAGE);
		return;
	}
	
	this.loading = false;
	var description = descriptionField.value;
	var url = this.setProfileUrl(name, description);
	loadXML(url, addProfileCallBack);
	
	nameField.value = "";
	descriptionField.value = "";
}


/**
 * Change the name and or description of a profile. 
 * Names and descriptions of profiles can be changed in the profiles table.
 * @param oldName The old name of the profile to change.
 * @param newName The new name to set for the profile.
 * @param input The input field where the profile name has been changed.
 * @throws Error if oldName is undefined, null or empty.
 * @throws Error if newName is undefined, null or empty.
 * @throws Error if input is undefined or null.
 */
ProfileManager.changeProfileName = function(oldName, newName, input) {
	assertNotEmpty(oldName, "Old name of profile to change is undefined null or empty");
	assertExists(newName, "New name to set for the profile is undefined null or empty");
	assertExists(input, "The input field for the profile is undefined or null");
	
	try {
		if (isEmpty(newName)) {
			throw new Error("Name is empty");
		}
		if (this.existsProfile(newName)) {
			throw new Error("A profile with that name already exists");
		}
		if (!this.isValidProfileName(newName)) {
			throw new Error(ProfileManager.INVALID_CHAR_IN_PROFLIE_NAME_MESSAGE);
		}
		if (oldName == this.getActiveProfileName()) {
			throw new Error("Can not change the name of the active profile");
		}
		if (exists(this.getDefaultProfile()) && oldName == this.getDefaultProfile().getName()) {
			throw new Error("Can not change the name of the default profile");
		}
		loadXML(ProfileManager.changeProfileUrl(oldName, newName), modifyProfileCallBack);
	}
	catch (e) {
		this.setMessage(undefined, e.message);
		input.value = oldName;
	}	
}


/**
 * Change the description of a profile.
 * @param name The name of the profile to change the profile for.
 * @param description The description to set. 
 * @throws Error if name is undefined, null or empty.
 * @throws Error if description is undefined or null.
 */
ProfileManager.changeProfileDescription = function(name, description) {
	assertNotEmpty(name, "The name of the profile is undefined null or empty");
	assertExists(description, "The new description to set is undefined null or empty");
	
	loadXML(this.setProfileUrl(name, description), descriptionChangedCallBack);
}


/**
 * Loads the profiles from the server.
 */
ProfileManager.loadProfiles = function () {
	if (this.loading) {
		// the profiles are already being loaded
		return;
	}
	this.loading = true;
	loadXML(this.loadProfilesUrl(), profilesLoaded);
}


/**
 * Sets a given profile as the default profile.
 * @param profile The profile to set as default.
 * @throws Error if profiles is undefined or null. 
 */
ProfileManager.setDefaultProfile = function(profile) {
	assertExists(profile, "The profile to set as defualt is undefined or null");
	if (exists(this.defaultProfile) && this.defaultProfile.getName() != this.getActiveProfileName()) {
		this.addNameInputField(this.defaultProfile.getName());
		this.addDeleteProfileLink(this.defaultProfile.getName());
	}
	this.defaultProfile = profile;
	this.removeNameInputField(this.defaultProfile.getName());
	this.removeDeleteProfileLink(this.defaultProfile.getName());
}


/**
 * Gets the default profile set. 
 * @return The default profile, null if none set. 
 */
ProfileManager.getDefaultProfile = function() {
	return this.defaultProfile;
}


/**
 * Sets the selected profile in the defaultProfileSelector as the default profile.
 */
ProfileManager.setSelectedDefaultProfile = function() {
	var selectedOption = this.defaultProfileSelect().options[this.defaultProfileSelect().selectedIndex];
	var profile = isEmpty(selectedOption.value) ? null : this.getProfile(selectedOption.value);
	if (exists(profile)) {
		loadXML(ProfileManager.setDefaultUrl(selectedOption.value), setDefaultCallBack);
	}
}
 

/**
 * Callback method for the setting of the default profile.
 * @param xml The response from the server.
 * @param url The url used to set the default parameter.
 */
function setDefaultCallBack(xml, url) {
	ProfileManager.setProfileMessage(xml);
	if (containsErrorMessage(xml)) {
		loadDefaultProfile();
	}
	else {
		var name = URLDecode(getParameterValue(url, "name", "&"));
		var profile = ProfileManager.getProfile(name);
		ProfileManager.setDefaultProfile(profile);
	}
}


/**
 * Loads the default profile.
 */
function loadDefaultProfile() {
	loadXML(ProfileManager.loadDefaultProfileUrl(), defaultProfilesLoaded);
}


/**
 * Callback function for the loading of the default profile. 
 * @param xml The response from the server.
 */
function defaultProfilesLoaded(xml) {
	var name = URLDecode(getTagNodeValue("name", xml));
	var profile = isEmpty(name) ? null : ProfileManager.getProfile(name);
	if (profile != null) {
		ProfileManager.setDefaultProfile(profile);
		selectInSelector(ProfileManager.defaultProfileSelect(), name);
	}
}


// active profile


/**
 * Activates the profile selected in the active profile selector.
 */
ProfileManager.activateSelectedProfile = function() {
	var selectedOption = this.activeProfileSelect().options[this.activeProfileSelect().selectedIndex];
	if (!isEmpty(selectedOption.value)) {
		this.activateProfile(selectedOption.value);
	}
}


/**
 * Activates a profile.
 * @param name The name of the profile to activate.
 * @throws Error if name is undefined, null or empty.
 */
ProfileManager.activateProfile = function(name) {
	assertNotEmpty(name, "Name of the protocol to activate is undefined, null or empty");
	// if the profiles window is shown make it appear ontop of the loading screen
	if (WindowManager.instance().getWindow("profilewindow").isShown()) {
		WindowManager.instance().getWindow("profilewindow").setZIndex(LoadingAnimation.ZINDEX + 1);
	}
	
	loadXML(this.activateProfileUrl(name), profileActivatedCallBack);
}


/**
 * Gets the active profile.
 * @return The active profile.
 */
ProfileManager.getActiveProfileName = function() {
	return this.activeProfile;
}


/**
 * Callback function for the activation of a profile.
 * @param xml The response from the server.
 * @param url The url used to set the active profile.
 */
function profileActivatedCallBack(xml, url) {
	ProfileManager.setProfileMessage(xml);
	if (containsErrorMessage(xml)) {
		loadActiveProfile();
		ProfileManager.hideLoadingProfile();
		return;
	}
	
	var name = URLDecode(getParameterValue(url, "name", "&"));
	ProfileManager.showLoadingProfile();	
	loadLayout();
	ProfileManager.markAsInactive(ProfileManager.activeProfile);
	ProfileManager.activeProfile = name;
	ProfileManager.markAsActive(ProfileManager.activeProfile);
	selectInSelector(ProfileManager.activeProfileSelect(), name);
}


/**
 * Selects a active profile given as xml in the active profile selector.
 * @param profile The profile to select as xml. 
 */
function selectActiveProfile(profile) {
	var name = URLDecode(getTagNodeValue("name", profile));
	if (!isEmpty(name)) {
		selectInSelector(ProfileManager.activeProfileSelect(), name);
		ProfileManager.markAsInactive(ProfileManager.activeProfile);
		ProfileManager.activeProfile = name;
		ProfileManager.markAsActive(name);
	}
}


/**
 * Sets a profile.
 * @param profile The profile to add. 
 * @throws Error if profile is undefined or null.
 */
ProfileManager.setProfile = function(profile) {
	assertExists(profile, "Profile to add does not exist");
	this.profiles[profile.getName()] = profile;
}


/**
 * Clears all the profiles, also the default and active profile are cleared.
 */
ProfileManager.clearProfiles = function() {
	this.profiles = new Array();
	this.activeProfile = "";
	this.defaultProfile = null;
}


/**
 * Get a profile with a given name.
 * @param name The name of the profile to get.
 * @return The profile with name, null if undefined.
 * @throws Error if name is undefined, null or empty.
 */
ProfileManager.getProfile = function(name) {
	assertNotEmpty(name, "Name of profile to get is undefined, null or empty");
	var profile = this.profiles[name];
	if (exists(profile)) {
		return profile;
	}
	return null;
}


/**
 * Gets all the profiles as a array. 
 * @return The profiles as a array, null if no profiles. 
 */
ProfileManager.getProfiles = function() {
	var array = new Array();
	for (var profileName in this.profiles) {
		array[array.length] = this.profiles[profileName];
	}
	return array.length == 0 ? null : array;
}


/**
 * Removes a profile with a given name. 
 * @param name The name of the profile to remove.
 * @throws Error if name is undefined, null or empty.
 */
ProfileManager.removeProfile = function(name) {
	assertNotEmpty(name, "The name of the profile to remove is undefined, null or empty");
	
	try {
		if (name == this.getActiveProfileName()) {
			throw new Error("Can not delete a active profile");
		}
		if (exists(this.getDefaultProfile()) && name == this.getDefaultProfile().getName()) {
			throw new Error("Can not delete the default profile");
		}
		loadXML(this.removeUrl(name), removeProfileCallback);
	}
	catch (e) {
		this.setMessage(undefined, e.message);
	}
}


/**
 * Callback method for removing a profile.
 * @param response The response from the server. 
 * @param url The url used to remove the profile. 
 */
function removeProfileCallback(response, url) {
	var window = WindowManager.instance().getWindow("profilewindow")
	var name = URLDecode(getParameterValue(url, "name", "&"));
	var errorMessage = getErrorMessage(response);
	var defaultProfileName = exists(ProfileManager.getDefaultProfile()) ? ProfileManager.getDefaultProfile().getName() : null;
	if (containsErrorMessage(response)) {
		ProfileManager.setProfileMessage(response);	
	}
	if (ProfileManager.ACTIVE_PROFILE_DELETED_MESSAGE.equalsIgnoreCase(errorMessage) && ProfileManager.getActiveProfileName() != name) {
		ProfileManager.setMessage(undefined, ProfileManager.ACTIVE_PROFILE_DELETED_MESSAGE + ". The profile might be active in another session");
	}
	else if (ProfileManager.DEFAULT_PROFILE_DELETED_MESSAGE.equalsIgnoreCase(errorMessage) && exists(defaultProfileName) && defaultProfileName != name) {
		// the default profile has been changed reload it
		ProfileManager.setMessage(undefined, ProfileManager.DEFAULT_PROFILE_DELETED_MESSAGE + ". The default profile might have been changed in another session");
		ProfileManager.loadProfiles();
	} 
	
	if (!containsErrorMessage(response) && !isEmpty(name)) {
		// remove the profile from the table
		TableManager.instance().getTable("profilesTable").deleteEntry("name=" + name + "&")
		// remove the entry from the default and active profile selector tags
		removeOption(name, ProfileManager.defaultProfileSelect());
		removeOption(name, ProfileManager.activeProfileSelect());
		
		ProfileManager.removeProfileFromMenu(name);
		ProfileManager.removeFromProfiles(name);
	}
}


// events


/**
 * Processes a layout set event.
 * @param event The event to process.
 */
function processLayoutSetEventProfileManager() {
	ProfileManager.hideLoadingProfile();
}


/**
 * Processes a window shown event.
 * @param event The event to process.
 */
function processWindowShownEventProfileManager(event) {
	if (event.getData() != "profilewindow") {
		return;
	}
	ProfileManager.loadProfiles();
}


// implementation


/**
 * Creates a change profile url.
 * @param oldName The old name of the profile to change.
 * @param newName The name to set for the profile.
 */
ProfileManager.changeProfileUrl = function(oldName, newName) {
	var url = ProfileManager.CHANGE_PROFILE_URL + "name=" + encodeURIComponent(oldName);
	return url + "&newName=" + encodeURIComponent(newName) + "&tabName=" + currentTabName;
}


/**
 * Creates the set defualt profile url.
 * @param name The name of the profile to set as default.
 */
ProfileManager.setDefaultUrl = function(name) {
	return ProfileManager.SET_DEFAULT_PROFILE_URL + "name=" + encodeURIComponent(name) + "&tabName=" + currentTabName;
}


/**
 * Creates the url to activate a profile.
 * @param name The name of the profile to activate.
 */
ProfileManager.activateProfileUrl = function(name) {
	return ProfileManager.ACTIVATE_PROFILE_URL + "name=" + encodeURIComponent(name) + "&tabName=" + currentTabName;
}


/**
 * Loads the active profile.
 */
function loadActiveProfile() {
	loadXML(ProfileManager.loadActiveProfileUrl(), selectActiveProfile);
}


/**
 * Creates the url to load the active profile. 
 * @return The url to load the active profile. 
 */
ProfileManager.loadActiveProfileUrl = function() {
	return ProfileManager.ACTIVE_PROFILE_URL + "tabName=" + currentTabName;
}


/**
 * Creates a remove profile url.
 * @param name The name of the profile that has been deleted.
 */
ProfileManager.removeUrl = function(name) {
	return ProfileManager.BASE_DELETE_PROFILES_URL + "name=" + encodeURIComponent(name) + "&tabName=" + currentTabName;
}



/**
 * Create a add profile url.
 * @param name The name of the profile.
 * @param description The description of the profile.
 * @throws Error if the name is undefined, null or empty.
 * @throws Error if the description is undefined or empty. 
 */
ProfileManager.setProfileUrl = function(name, description) {
	assertNotEmpty(name, "Name is undefined null or empty");
	assertExists(description, "Description is undefined or null");
	
	var url = ProfileManager.BASE_ADD_ENTRY_URL + "name=" + encodeURIComponent(name); 
	return url + "&description=" + encodeURIComponent(description) + "&tabName=" + currentTabName;
}


/**
 * Creates the load profile url.
 * @return The url to use to load profiles.
 */
ProfileManager.loadProfilesUrl = function() {
	return ProfileManager.LOAD_PROFILES_URL + "tabName=" + currentTabName;
}


/**
 * Creates the load default profile url.
 * @return The url to use to load the default profile.  
 */
ProfileManager.loadDefaultProfileUrl = function() {
	return ProfileManager.DEFAULT_PROFILE_URL + "tabName=" + currentTabName;
}


/**
 * Callback method for the adding of profiles.
 * @param profiles The xml profiles to parse. 
 */
function addProfileCallBack(xmlData) {
	ProfileManager.setProfileMessage(xmlData);
	if (containsSuccessMessage(xmlData)) {
		ProfileManager.loadProfiles();
		saveLayout();
	}
}


/**
 * Callback method for changing the description.
 * @param xmlData The response from the server.
 * @param url The url used to change the description
 */
function descriptionChangedCallBack(xmlData, url) {
	if (!containsSuccessMessage(xmlData)) {
		return;
	}
	
	ProfileManager.setMessage("Successfully set the profile description");
	var name = URLDecode(getParameterValue(url, "name", "&"));
	var description = URLDecode(getParameterValue(url, "description", "&"));
	var profile = ProfileManager.getProfile(name);
	if (!isNull(profile)) {
		profile.setDescription(description);
		ProfileManager.updateTableWithNewProfile(profile, profile.getName());
	}
}


/**
 * Callback method for modifying profiles.
 * @param xmlData The response from the server.
 * @param url The url used to modify the profile.
 */
function modifyProfileCallBack(xmlData, url) {
	ProfileManager.setProfileMessage(xmlData);
	var oldName = URLDecode(getParameterValue(url, "name", "&"));
	var newName = URLDecode(getParameterValue(url, "newName", "&"));
	var errorMessage = getErrorMessage(xmlData);
	var defaultProfileName = exists(ProfileManager.getDefaultProfile()) ? ProfileManager.getDefaultProfile().getName() : null;
	if (ProfileManager.ACTIVE_PROFILE_EDITED_MESSAGE.equalsIgnoreCase(errorMessage) && ProfileManager.getActiveProfileName() != oldName ) {
		ProfileManager.setMessage(undefined, ProfileManager.ACTIVE_PROFILE_EDITED_MESSAGE + ". The profile might be active in another session");
	}
	else if (ProfileManager.DEFAULT_PROFILE_EDITED_MESSAGE.equalsIgnoreCase(errorMessage) && exists(defaultProfileName) &&  defaultProfileName != oldName) {
		// the default profile has changed reload it
		ProfileManager.setMessage(undefined, ProfileManager.DEFAULT_PROFILE_EDITED_MESSAGE + ". The default profile might have been changed in another session");
		ProfileManager.loadProfiles();
	}
	if (containsErrorMessage(xmlData)) {
		ProfileManager.setProfileNameField(oldName, oldName);
		return;
	}
	
	ProfileManager.updateMenuWithNewName(oldName, newName);
	var profile = ProfileManager.getProfile(oldName);
	ProfileManager.removeFromProfiles(oldName);
	if (exists(profile)) {
		profile.setName(newName);
		ProfileManager.setProfile(profile);
		ProfileManager.updateTableWithNewProfile(profile, oldName);
		ProfileManager.updateDefaultProfileSelectWithNewProfile(profile, oldName);
		ProfileManager.updateActiveProfileSelectWithNewProfile(profile, oldName);
	}
}


/**
 * Callback function for the loading of the profiles.
 * @param profiles The profiles loaded.
 */
function profilesLoaded(xmlData) {
	if (!exists(xmlData)) {
		return;
	}
	parseProfiles(xmlData);
	TableManager.instance().getTable("profilesTable").updateTable(xmlData);
	addProfilesAsOptions(ProfileManager.activeProfileSelect());
	loadActiveProfile();
	addProfilesAsOptions(ProfileManager.defaultProfileSelect());
	loadDefaultProfile();
	setProfilesMenu();
	ProfileManager.loading = false;
}


/**
 * Parses the xml profiles into Profile objects.
 * @param xmlProfiles The profiles to parse. 
 */
function parseProfiles(xmlProfiles) {
	ProfileManager.profiles = new Array();
	var profiles = xmlProfiles.getElementsByTagName("profile");
	for (var i = 0; i < profiles.length; i++) {
		var profile = createProfile(profiles[i]);
		ProfileManager.setProfile(createProfile(profiles[i]));
	}
}


/**
 * Sets the profiles in the top menu.
 */
function setProfilesMenu() {
	var profilesMenu = getElementWithAttribute(document.body, "profiles_menu", "id", "img");
	var index = 0;
	clearProfileMenu();
	for (profileName in ProfileManager.profiles) {
		var name = profileName;
		// the first two choices in the profiles menu are manage and save
		profilesMenu.menucontent[index + 2] = ProfileManager.createTopMenuLink(name, "");
		index++;
	}
	profilesMenu.menuwidth = calculateMenuWidth();

	// add menu divider if necessary
	if (profilesMenu.menucontent.length >= 3 && profilesMenu.menucontent[1].indexOf("<hr") < 0) {
		var menu_divider = '<div style="width: 100%; height: 100%;"><hr size="1" color="626262" width="98%"></div>';
		profilesMenu.menucontent[1] += menu_divider;
	}
	
	// remove menu divider if there are no profiles available
	if (profilesMenu.menucontent.length == 2) {
		var saveMenuChoice = profilesMenu.menucontent[1];
		var hrIndex = saveMenuChoice.indexOf("<hr");
		if (hrIndex > -1) {
			profilesMenu.menucontent[1] = saveMenuChoice.substring(0, hrIndex);;
		}
	}
}


/**
 * Clears the profile top menu for profiles. The two topmost options
 * (Manage and save) are not removed. 
 */
function clearProfileMenu() {
	var menuContainer = document.getElementById("menucontainer");
	var profilesMenu = getElementWithAttribute(document.body, "profiles_menu", "id", "img");
	var index = 0;
	// the first two options are manage and save, do not remove them
	for (var i = menuContainer.childNodes.length - 1; i > 1; i--) {
		menuContainer.removeChild(menuContainer.childNodes[i]);
		profilesMenu.menucontent[i] = null;
		profilesMenu.menucontent = removeNullEntries(profilesMenu.menucontent);
	}
}


/**
 * Calculates and returns the width of the profiles menu, based on the length of the largest profile name.
 */
function calculateMenuWidth() {
	// the minimum width of the menu
	var minimumWidth = 120;
	// the length of the profile link, excluding the name/value of the link
	var defaultProfileLinkLength = 78;
	// the value to multiply the length of the profile name with, in order to get the correct width
	var menuWidthFactor = 5.7;
	
	var menuWidth = Math.max(minimumWidth, ((getLargestProfileLink().length - defaultProfileLinkLength)/2) * menuWidthFactor);
	menuWidth = menuWidth > ProfileManager.PROFILE_MENU_MAX_WIDTH ? ProfileManager.PROFILE_MENU_MAX_WIDTH : menuWidth;
	return menuWidth;	
}


/**
 * Gets the largest profile link. The length of the value can be used to calculate the menu width. 
 */
function getLargestProfileLink() {
	var profilesMenu = getElementWithAttribute(document.body, "profiles_menu", "id", "img");
	var largestProfileLink = "";
	for (var i = 0; i < profilesMenu.menucontent.length; i++) {
		if (profilesMenu.menucontent[i].length > largestProfileLink.length) {
			largestProfileLink = profilesMenu.menucontent[i];
		}
	}
	return largestProfileLink;
}	


// messages


/**
 * Set a message from a xml block.
 * @param xmlData The xml data containing the mesasge.
 * @throws Error if xmlData is not defined or null.
 */
ProfileManager.setProfileMessage = function(xmlData) {
	assertExists(xmlData, "Xmldata containing the message is not defined");
	
	if (containsErrorMessage(xmlData)) {
		this.setMessage(undefined, getErrorMessage(xmlData));
	}
	else if (containsSuccessMessage(xmlData)) {
		this.setMessage(getSuccessMessage(xmlData),undefined);
	}
}


/**
 * Set the tables mesage field.
 * @param successMessage The success message to set. 
 * @param errorMessage The error message to set. 
 */
ProfileManager.setMessage = function(sucessMessage, errorMessage) {
	// increase the height to make room for the message container, must 
	// increase the height of the window before the message is shown
	// to avoid flickering of the table in firefox
	if (this.messageHeightIncreased == false) {
		this.messageHeightIncreased = true;
		this.increaseHeight(15);
	}
	if (isDefined(sucessMessage) && !isEmpty(sucessMessage)) {
		setClass(this.getMessageContainer(), "messageContainer ok");
		this.getMessageContainer().innerHTML = "<nobr>" + sucessMessage + "</nobr>";
	}
	else if (isDefined(errorMessage) && !isEmpty(errorMessage)) {
		setClass(this.getMessageContainer(), "messageContainer error");
		this.getMessageContainer().innerHTML = "<nobr>" + errorMessage + "</nobr>";
	}
}


/**
 * Gets the message container, null if not created yet.
 */
ProfileManager.getMessageContainer = function() {
	var window = WindowManager.instance().getWindow("profilewindow")
	if (!isDefined(this.messageContainer) || isNull(this.messageContainer)) {
		this.messageContainer = getElementWithAttribute(window.getHtmlCode(), "profileMessageContainer", "name", "div");
	}
	return this.messageContainer;
}


/**
 * Clears the profile message field.
 */
ProfileManager.clearProfileMessage = function() {
	this.getMessageContainer().innerHTML = "";
	this.messageHeightIncreased = false;
	var height = this.getMessageContainer().offsetHeight;
	setClass(this.getMessageContainer(), "inactive");
	if (exists(height) && isNumber(height)) {
		this.increaseHeight(height * -1); 
	}
}


/**
 * Returns true if a name is a valid profile name.
 * @param name The name to check.
 */
ProfileManager.isValidProfileName = function(name) {
	assertNotEmpty(name, "Profile name to verify is undefined, null or empty");
	for (var i = 0; i < name.length; i++)  {
		var c = name.charAt(i);
		if (ProfileManager.LEGAL_PROFILE_NAME_CHARACTERS.indexOf(c.toLowerCase()) < 0) {
			return false;
		}
	}
	return true;
}


/**
 * Returns true if a profile with a given name exists.
 * @param name The name to check.
 * @throws Error if name is undefined, null or empty.
 */
ProfileManager.existsProfile = function(name) {
	assertNotEmpty(name, "Name is undefined, null or empty");
	for (profileName in this.profiles) {
		if (profileName == name) {
			return true;
		}
	}
	return false;
}


/**
 * Increase the heigh of the profile window.
 * @param height The height to increase the window size with.
 */
ProfileManager.increaseHeight = function(height) {
	var window = WindowManager.instance().getWindow("profilewindow");
	var windowHeight = parseInt(window.getHtmlCode().style.height);
	var newWindowHeight = windowHeight + height;
	window.getHtmlCode().style.height = newWindowHeight + "px";
}


/**
 * Hides the loading screen.
 */
ProfileManager.hideLoadingProfile = function() {
	if (exists(this.loadingAnimation)) {
		this.loadingAnimation.hide();
	}
}


/**
 * Shows a loading frame while a profile is loaded.
 */
ProfileManager.showLoadingProfile = function() {
	if (exists(this.loadingAnimation)) {
		this.loadingAnimation.show();
		return;
	}
	
	var iframeCode = getElementWithAttribute(top.document.body, currentTabName, "name", "iframe");	
	if (!exists(iframeCode)) {
		// happens in tests
		return;
	}	
	var height = iframeCode.offsetHeight + "px";
	var width = iframeCode.offsetWidth + "px";
	this.loadingAnimation = new LoadingAnimation(document.body, height, width, "Loading profile...");
	this.loadingAnimation.show();
}


/**
 * Updates the active profile selector with a new profile. 
 * @param profile The profile to update the active profile selector with. 
 * @param oldName The old name of the profile to update. 
 */ 
ProfileManager.updateActiveProfileSelectWithNewProfile = function(profile, oldName) {
	var profileSelect = this.activeProfileSelect();
	var index = getOptionIndex(oldName, profileSelect);
	if (index == -1) {
		return;
	}
	
	var option = document.createElement("option");
	option.setAttribute("value", profile.getName());
	option.appendChild(document.createTextNode(profile.getName()));
	replaceOption(option, index, profileSelect);
}


/**
 * Updates the default profile selector with a new profile.
 * @param profile The profile to update the default profile selector with. 
 * @param oldName The old name of the profile to update. 
 */
ProfileManager.updateDefaultProfileSelectWithNewProfile = function(profile, oldName) {
	var defaultProfileSelect = this.defaultProfileSelect();
	var index = getOptionIndex(oldName, defaultProfileSelect);
	if (index == -1) {
		return;
	}
	
	var option = document.createElement("option");
	option.setAttribute("value", profile.getName());
	option.appendChild(document.createTextNode(profile.getName()));
	replaceOption(option, index, defaultProfileSelect);
}


/**
 * A profile name has been changed, update the table with the new profile. 
 * @param profile The new profile. 
 * @param oldName The old profile name. 
 */
ProfileManager.updateTableWithNewProfile = function(profile, oldName) {
	var table = TableManager.instance().getTable("profilesTable"); 
	var index = table.getRowIndexFieldValue("name", oldName);
	var row = table.getRow(index);
	var htmlRow = table.getHtmlRow(index);
	if (!exists(row) || !exists(htmlRow)) {
		return;
	}
	// update the table data
	var field = getFieldFromRow("name", row);
	field.setValue(profile.getName());
	field = getFieldFromRow("description", row);
	field.setValue(profile.getDescription());
	// update the change name input field
	var input = getElementWithAttribute(getHtmlFieldFromRow("name", htmlRow), CHANGE_NAME_INPUT_FIELD, "name", "input");
	if (exists(input)) {
		table.setChangeNameActions(profile.getName(), input);
		input.value = profile.getName();
	} 
	// update the change description input field
	input = getElementWithAttribute(getHtmlFieldFromRow("description", htmlRow), CHANGE_DESCRIPTION_INPUT_FIELD, "name", "input");
	table.setChangeDescriptionActions(profile.getDescription(), input);
	input.value = profile.getDescription();
	// update the delete link
	var deleteLink = getElementWithAttribute(htmlRow, MyTable.DELETE_LINK_NAME, "name", "a");
	if (exists(deleteLink)) {
		table.setDeleteLinkOnClick(deleteLink, row);
	}
}
 
 
/**
 * Creates a new name input field for a given profile in the profile table.
 * @param name The name of the profile 
 */
ProfileManager.addNameInputField = function(name) {
	// replace the cell with a new that contains a input field for the name
	var table = TableManager.instance().getTable("profilesTable");
	var htmlRow = table.getHtmlRowWithFieldText(name, "name");
	var cell = htmlRow == null || htmlRow.cells == null || htmlRow.cells.length == 0 ? null : htmlRow.cells[0];
	var row = table.getRowWithFieldValue("name", name);
	var field = row == null ? null : getFieldFromRow("name", row);
	if (cell == null || field == null) {
		return;
	}
	var newCell = document.createElement("td");
	table.setAlignment(newCell, field);
	htmlRow.replaceChild(newCell, cell);
	table.createCellCode(field, newCell, field.getName());
}


/**
 * Adds a delete link to a profile in the profile table. 
 * @param name The name of the profile.  
 */
ProfileManager.addDeleteProfileLink = function(name) {
	var table = TableManager.instance().getTable("profilesTable");
	var index = table.getRowIndexFieldValue("name", name);
	var htmlRows = table.getTableCode().rows;
	var htmlRow = index > -1 && htmlRows != null && htmlRows.length > 0 ? htmlRows[index] : null;
	var row = table.getRowWithFieldValue("name", name);
	if (exists(htmlRow) && exists(row)) {
		htmlRow.deleteCell(2);
		table.addButtonColumn(row, htmlRow);
	}
}


/**
 * Removes the name input field for a given profile in the profile table. 
 * @param name The name of the profile. 
 */
ProfileManager.removeNameInputField = function(name) {
	var input = this.getNameInputField(name);
	if (input == null) {
		return;
	}
	var parent = input.parentNode;
	parent.removeChild(input);
	parent.appendChild(document.createTextNode(name));
}


/**
 * Removes a delete link for a given profile in the profile table. 
 * @param name The name of the profile. 
 */
ProfileManager.removeDeleteProfileLink = function(name) {
	var table = TableManager.instance().getTable("profilesTable");
	var index = table.getRowIndexFieldValue("name", name);
	var htmlRows = table.getTableCode().rows;
	var htmlRow = index > -1 && htmlRows != null && htmlRows.length > 0 ? htmlRows[index] : null;
	var cell = htmlRow == null || htmlRow.cells == null || htmlRow.cells.length == 0 ? null : htmlRow.cells[2];
	var deleteLink = cell == null ? null : getElementWithAttribute(cell, "deleteLink", "name", "a");
	if (deleteLink == null) {
		return;
	}
	deleteLink.parentNode.removeChild(deleteLink);
}	


/**
 * Removes a profile from the drop down menu.
 * @param name The old name to remove. 
 */
ProfileManager.removeProfileFromMenu = function(name) {
	var profilesMenu = getElementWithAttribute(document.body, "profiles_menu", "id", "img");
	for (var i = 0; i < profilesMenu.menucontent.length; i++) {
		if (profilesMenu.menucontent[i].indexOf(name) > -1) {
			profilesMenu.menucontent[i] = null;
			profilesMenu.menucontent = removeNullEntries(profilesMenu.menucontent);
		}
	}
}


/**
 * Updates the drop down menu with a new name.
 * @param oldName The old name to replace.
 * @param newName The name to set. 
 */
ProfileManager.updateMenuWithNewName = function(oldName, newName) {
	var profilesMenu = getElementWithAttribute(document.body, "profiles_menu", "id", "img");
	for (var i = 0; i < profilesMenu.menucontent.length; i++) {
		if (profilesMenu.menucontent[i].indexOf(oldName) > -1) {
			profilesMenu.menucontent[i] = this.createTopMenuLink(newName, "");
		}
	}
	profilesMenu.menuwidth = calculateMenuWidth();
}



/**
 * Marks a profile as inactive in the menu and the profile table.
 * @param name The name of the profile to deactivate.
 */
ProfileManager.markAsInactive = function(name) {
	if (isEmpty(name)) {
		return;
	}
	var defaultProfile = ProfileManager.getDefaultProfile();
	if (!exists(defaultProfile) || name != defaultProfile.getName()) {
		// when the profile still is the default profile, one should not be
		// able to change the profiles name or delete it
		this.markAsInactiveTable(name);
	}
	this.markAsInactiveMenu(name);
}


/**
 * Marks a profile as active in the menu and in the profile table.
 * @param name The name of the profile to activate.
 */
ProfileManager.markAsActive = function(name) {
	if (isEmpty(name)) {
		return;
	}
	this.markAsActiveMenu(name);
	this.markAsActiveTable(name);
}


/**
 * Maks a profile as inactive in the profile table.
 * @param profile The profile to deactivate. 
 */
ProfileManager.markAsInactiveTable = function(name) {
	if (isEmpty(name)) {
		return;
	}
	
	this.addNameInputField(name);
	this.addDeleteProfileLink(name);
}


/**
 * Marks a profile as inactive in the menu.
 * @param name The name of the profile to deactivate.  
 */
ProfileManager.markAsInactiveMenu = function(name) {
	if (isEmpty(name)) {
		return;
	}
	var profilesMenu = getElementWithAttribute(document.body, "profiles_menu", "id", "img");
	for (var i = 0; i < profilesMenu.menucontent.length; i++) {
		if (profilesMenu.menucontent[i].indexOf("name=\"" + name + "\"") > -1) {
			profilesMenu.menucontent[i] = this.createTopMenuLink(name, "");
		}
	}
	profilesMenu.menuwidth = calculateMenuWidth();
}


/**
 * Marks a profile as active in the profile table. 
 * @param name The name of the profile to marks as active.
 */
ProfileManager.markAsActiveTable = function(name) {
	if (isEmpty(name)) {
		return;
	}
	
	this.removeNameInputField(name);
	this.removeDeleteProfileLink(name);
}


/**
 * Marks a profile as active in the menu. 
 * @param name The name of the profile to marks as active.
 */
ProfileManager.markAsActiveMenu = function(name) {
	if (isEmpty(name)) {
		return;
	}
	
	var profilesMenu = getElementWithAttribute(document.body, "profiles_menu", "id", "img");
	for (var i = 0; i < profilesMenu.menucontent.length; i++) {
		if (profilesMenu.menucontent[i].indexOf("name=\"" + name + "\"") > -1) {
			profilesMenu.menucontent[i] = this.createTopMenuLink(name, "activeProfile");
		}
	}
	profilesMenu.menuwidth = calculateMenuWidth();
}


/**
 * Creates the html code for a profiles menu link in the top menu.
 * @param name The name of the profile to create the link for. 
 * @param className The class to set for the link.
 * @return The link created for the profile. 
 */
ProfileManager.createTopMenuLink = function(name, className) {
	return '<nobr><a href="#" style="overflow: hidden" name="' + name + '" class="' + className + '" onClick="menuhasfocus=\'no\';hide();ProfileManager.activateProfile(\'' + name + '\');">' + name + '</a></nobr>';	
}


/**
 * Removes a profile from the profiles array.
 * @param name The name of the profile to remove. 
 */
ProfileManager.removeFromProfiles = function(name) {
	this.profiles[name] = null;
	var newArray = new Array();
	for (var profileName in this.profiles) {
		if (exists(this.profiles[profileName])) {
			newArray[profileName] = this.profiles[profileName];
		}
	}
	this.profiles = newArray;
}	


/**
 * Sets the value of a profile name input field in the profiles table to a given value. 
 * @param value The value to set. 
 * @param name The name of the profile to set the value for. 
 */
ProfileManager.setProfileNameField = function(value, name) {
	var input = this.getNameInputField(name);
	if (input != null) {
		input.value = value;
	}		
}


/**
 * Gets a name input field for a given profile in the profiles table. 
 * @param name The name of the proflie to get the input field for.
 * @return The input field, null if no input field. 
 */
ProfileManager.getNameInputField = function(name) {
	var table = TableManager.instance().getTable("profilesTable");
	var index = table.getRowIndexFieldValue("name", name);
	var htmlRows = table.getTableCode().rows;
	var htmlRow = index > -1 && htmlRows != null && htmlRows.length > 0 ? htmlRows[index] : null;
	var cell = htmlRow == null || htmlRow.cells == null || htmlRow.cells.length == 0 ? null : htmlRow.cells[0];
	return cell == null ? null : getInput(cell, "changeNameField"); 
}


/**
 * Gets the default profile selector tag. 
 * @return The default profile selector. 
 */
ProfileManager.defaultProfileSelect = function() {
	return getElementWithAttribute(WindowManager.instance().getWindow("profilewindow").getHtmlCode(), "defaultProfileSelector", "name", "select");
}


/**
 * Gets the active profile selector tag. 
 * @return The active profile selector. 
 */
ProfileManager.activeProfileSelect = function() {
	return getElementWithAttribute(WindowManager.instance().getWindow("profilewindow").getHtmlCode(), "activeProfileSelector", "name", "select");
}	


/**
 * Adds profiles given as xml to a selector tag as options.
 * @param selector The selector to add the options to.
 */
function addProfilesAsOptions(select) {
	// go through all the options and update them
	var profiles = ProfileManager.getProfiles();
	// the first option is the select a profile option
	var counter = updateOptions(select);
	if (counter < select.options.length) {
		removeOptions(counter + 1, select);
	}
		
	// there are more profiles in the profile manager than there are options
	// in the select, add them
	for (var i = counter; profiles != null && i < profiles.length; i++) {
		var option = document.createElement("option");
		option.setAttribute("value", profiles[i].getName());
		option.innerHTML = profiles[i].getName();
		select.appendChild(option);
	}	
}


/**
 * Updates the options in a given select tag according to the profile in the 
 * profile manager. 
 * @param select The select tag to update the options in. 
 * @return The number of options updated. 
 */
function updateOptions(select) {
	// go through all the options and update them
	var profiles = ProfileManager.getProfiles();
	// the first option is the select a profile option
	var counter = 0;
	for (var i = 1; profiles != null && i < profiles.length; i++) {
		if (i >= (select.options.length - 1)) {
			break;
		}		
		var option = select.options[i];
		option.setAttribute("value", profiles[i - 1].getName());
		option.innerHTML = profiles[i - 1].getName();
		counter++;
	} 
	return counter;
}


/**
 * Removes options from a select tag. 
 * @param index The index where to start removing the options. 
 * @param select The select where to remove the options from.
 */
function removeOptions(index, select) {
	for (var i = select.options.length -1; i >= index; i--) {
		select.remove(i);
	}
}