/**
 * This class represents the profile table shown in the profile window.
 */

 
/**
 * The name of the change name input fields in the table.
 */
var CHANGE_NAME_INPUT_FIELD = "changeNameField";


/**
 * The name of the change description input fields in the table.
 */
var CHANGE_DESCRIPTION_INPUT_FIELD = "changeDescriptionField";


/**
 * 
 */
var PROFILE_BUTTONS_CELL_WIDTH = 15;


/**
 * Construct the profiletable.
 * @param id The id of the profile table.
 * @param url The url to load the data from.
 * @param parentElement The element the table is to be inserted in. 
 * @param windowId The id of the window the table is in. 
 * @throws Error if id is undefined, null or empty.
 * @throws Error if url is undefined, null or empty.
 * @throws Error if parentElement is undefined, or null.
 * @throws Error if windowId is undefined, null or empty. 
 */ 
function createProfileTable(id, url, parentElement, windowId) {
	assertNotEmpty(id, "Id for the profile table is undefined, empty or null: " + id);
	assertNotEmpty(url, "Url to set for the profile table is undefined empty or null: " + url);
	assertDefined(parentElement, "The element to insert the profile table into is undefined or null");
	assertNotEmpty(windowId, "windowId to set for the profile table is undefined empty or null: " + windowId);
	
	var subInstance = new ProfileTable();
	var superInstance = createMyTable(id, url, parentElement, windowId, undefined);
	var instance = createSubClass(superInstance, subInstance);
	instance["superupdateHtmlTable"] = superInstance["superupdateHtmlTable"];
	instance["supercreateRowCode"] = superInstance["supercreateRowCode"];
	TableManager.instance().removeTable(TableManager.instance().getTable(id));
	TableManager.instance().registerNewTable(instance);

	// let the event processor call event handler functions as a method
	// of the mytable object instance
	superInstance.getEventProcessor().changeObject(instance);
	superInstance.getEventProcessor().removeEvent(WINDOW_SHOWN_EVENT);
	return instance;
}


/**
 * Subclasses myTable to add behaviour specific for the profile table.
 */
function ProfileTable() {
}


// services


/**
 * Creates the url needed to delete a entry.
 * @param value The value for the id field of the row to delete. 
 * @return The url to delete the entry.
 */
ProfileTable.prototype.createDeleteEntryUrl = function() {
	return "";
}


/**
 * A entry has been deleted remove the chart for that entry if any. 
 * @param deletedRow The row that has been deleted. 
 */
ProfileTable.prototype.entryDeleted = function(deletedRow) {
	this.adjustProfileSize();
}


/**
 * Returns the url to add a entry.
 */
ProfileTable.prototype.createAddEntryUrl = function() {
	return "";
}


/**
 * Updates the html table according to the data in memory. This is overridden
 * to set the size of the profile window and table.
 * This is overridden to make room for the delete column.
 */
ProfileTable.prototype.updateHtmlTable = function() {
	if (!isNull(this.getTableCode()) && this.getTableCode().rows.length > 0) {
		this.adjustProfileSize();
	}
	this.callSuper("updateHtmlTable");
	
	if (!this.cellWidthDecreased && this.calculateDeleteColumnWidth() > 0) {
		// make room for the delete column
		this.resizeAllColumns(this.calculateDeleteColumnWidth() * (-1));
		this.cellWidthDecreased = true;
	}
	
	// the add entry table should always be right under the data table, to do this
	// we need to set the height of the data table container for each row that is added
	if (isDefined(this.getDataTableContainer()) && !isNull(this.getDataTableContainer())) {
		this.setTableContainerHeight(this.calculateTableContainerHeight());
	}
	this.adjustProfileSize();
}


/**
 * Adjusts the size of the profile table and window according to how many profiles are shown.
 */
ProfileTable.prototype.adjustProfileSize = function() {
	// to adjust the size the window must be visible, because we use offsetHeight
	var window = WindowManager.instance().getWindow("profilewindow");
	var shown = false;
	var originalX = window.getXPosition();
	var originalY = window.getYPosition();
	if (!window.isShown()) {	
		window.setPosition(0 - window.getHeight(), 0 - window.getWidth());
		shown = true;
		WindowManager.instance().showWindow(window.getId());
	}

	// calculate the height diff
	var height = this.calculateTableContainerHeight() + this.getAddEntryContainerCode().offsetHeight + this.getHeaderTableCode().offsetHeight;
	var heightDiff = height - parseInt(this.getTableContainerCode().style.height);
	// adjust the size of the profile window and table
	window.getHtmlCode().style.height = (parseInt(window.getHtmlCode().style.height) + heightDiff) + "px";
	var container = this.getTableContainerCode();
	resizeHtmlElement(container, heightDiff, 0);
	this.adjustSize();
	
	if (shown) {
		window.setPosition(originalY, originalX);
		WindowManager.instance().hideWindow(window.getId());
	}
}


/**
 * Adds the button cell to a given table row. 
 * @param data The array to create data from.
 * @param row The row to add the delete cell to.
 */
ProfileTable.prototype.addButtonColumn = function(data, row) {
	var cell = row.insertCell(row.cells.length);
	cell.setAttribute("name", "buttonCell");
	cell.style.padding = "2px";
	cell.setAttribute("valign", "middle");
	setClass(cell, "deleteCell");
	
	cell.style.width = PROFILE_BUTTONS_CELL_WIDTH + "px";
	var nobr = document.createElement("nobr");
	cell.appendChild(nobr);
	this.addDeleteLink(nobr, data);
}


/**
 * Set the onclick value for a delete link. This is overridden to set
 * the correct onclick actions.  
 * @param deleteLink The link to set the onclick value for. 
 * @param data The array to create data from.
 */
ProfileTable.prototype.setDeleteLinkOnClick = function(deleteLink , data) {
	// the id of the delete link is used to identify the row that is to be deleted
	// onclick of the link, update the id with the correct value
	deleteLink.setAttribute("id", this.getRowId(data));
	var thisObj = this;
	
	deleteLink.onclick = function() {
		ProfileManager.removeProfile(getParameterValue(this.id, "name", "&"));
		// hide tooltip when clicked
		tt_Hide();
	}
}


/**
 * Calculate the height to set for the data table container based on the amount
 * of rows in the table.
 */
ProfileTable.prototype.calculateTableContainerHeight = function() {
	var rowCount = this.rows.length;
	var rowHeight = this.getRowHeight();
	var height = 0;
	if (rowCount > ProbeTableManager.MAX_ROW_NO_SCROLLBAR) {
		height = ProbeTableManager.MAX_ROW_NO_SCROLLBAR * rowHeight;
	}
	else if (rowCount >= 0) {
		height = rowHeight * rowCount;
	}
	return height;
}



/**
 * Gets the row height. 
 * @return The height of the rows.
 */
ProfileTable.prototype.getRowHeight = function() {
	// because of trouble in safari with cell borders and offsetHeight the
	// rowHeight can not be calculated in safari
	if (isSafari()) {
		return 24;
	}
	return this.callSuper("getRowHeight");
}


/**
 * Inserts a field value into a given html node. 
 * This method is overridden to add KB, MB, and GB to byte type columns, and add 
 * input fields to the comment fields. 
 * @param field The field to insert the value for.
 * @param node The node to insert the value into.
 */
ProfileTable.prototype.insertFieldValue = function(field, node, cell) {
	if (field != null && !this.isHeader(field) && "name".equalsIgnoreCase(field.getName())) {
		var nameInputField = this.changeNameInputField(field.getValue());
		node.appendChild(nameInputField);
		cell.onmouseover = function() {
			Tip(helptext('profilename'), BGCOLOR, '#ffffe1', ABOVE, false, DELAY, 500);
		}
	}
	else if (field != null && !this.isHeader(field) && "description".equalsIgnoreCase(field.getName())) {
		var descriptionInputField = this.changeDescriptionInputField(field);
		node.appendChild(descriptionInputField);
		cell.onmouseover = function() {
			Tip(helptext('profiledescription'), BGCOLOR, '#ffffe1', ABOVE, false, DELAY, 500);
		}
	}
	else {
		// this is the default profile name
		node.appendChild(document.createTextNode(field.getValue()));
	}
}


/**
 * Paint the add entry row.
 * @param row The row to paint into.
 */
ProfileTable.prototype.paintAddEntryRow = function(row) {
	var cellCode;
	for (var i = (this.columnOrder.length -1); i >= 0; i--) {
		fieldName = this.columnOrder[i].getName();
		if (!this.isVisibleColumn(fieldName)) {
			continue;
		}

		if ("name".equalsIgnoreCase(fieldName)) {
			cellCode = row.insertCell(0);
			cellCode.setAttribute("name", fieldName);
			var nameInput = this.createAddNameInputField();
			cellCode.appendChild(nameInput);
			nameInput.onmouseover = function() {
				Tip(helptext('addProfileName'), BGCOLOR, '#ffffe1', ABOVE, false, DELAY, 500);
			}
			nameInput.onfocus = tt_Hide;
		}
		else if ("description".equalsIgnoreCase(fieldName)) {
			cellCode = row.insertCell(0);
			cellCode.setAttribute("name", fieldName);
			var descriptionInput = this.createAddDescriptionInputField();
			cellCode.appendChild(descriptionInput);
			descriptionInput.onmouseover = function() {
				Tip(helptext('addProfileDescription'), BGCOLOR, '#ffffe1', ABOVE, false, DELAY, 500);
			}
			descriptionInput.onfocus = tt_Hide;
		}
		this.setCellWidth(cellCode, fieldName);
		cellCode = null;	
	}
	var addButtonCell = row.insertCell(row.cells.length);
	addButtonCell.onmouseover = function() {
		Tip(helptext('addmytable'), BGCOLOR, '#ffffe1', ABOVE, false, DELAY, 500);
	}
	addButtonCell.appendChild(this.createAddProfileLink());
}


/**
 * Creates the add profile button.
 */
ProfileTable.prototype.createAddProfileLink = function() {
	var addLink = document.createElement("a");
	var addImage = document.createElement("img");
	addImage.src = MyTable.ADD_ENTRY_IMAGE_URL;	
	addImage.style.cursor = "pointer";
	addLink.appendChild(addImage);

	var thisObj = this;
	addLink.onclick = function() {
		ProfileManager.addProfile();
	}
	return addLink;
}


/**
 * Creates the change description name input field.
 * @param field The description field.
 * @return The input field where users can change the comment.
 */
ProfileTable.prototype.changeDescriptionInputField = function(field) {
	var inputField = this.createInputField(CHANGE_DESCRIPTION_INPUT_FIELD, MyTable.INPUT_FIELD_SIZE);
	setClass(inputField, "profile");
	
	inputField.value = field.getValue();
	this.setChangeDescriptionActions(field.getValue(), inputField);
	return inputField;
}


/**
 * Set the actions for the change description input field.
 * @param description The description of the profile this field is for. 
 * @param inputField The input field to set the actions for. 
 */
ProfileTable.prototype.setChangeDescriptionActions = function(description, inputField) {
	inputField.onkeyup = function(event) {
			event = exists(event) ? event : window.event;
			if (event.keyCode != 13) { 
				ProfileManager.clearProfileMessage();
				return;
			}
			if (description != this.value) {
				var row = TableManager.instance().getTable("profilesTable").getRowFromTd(findTdNode(this));
				var nameField = getFieldFromRow("name", row);
				ProfileManager.changeProfileDescription(nameField.getValue(), this.value);
			}
		}
		
	inputField.onblur = function() {
		if (description != this.value) {
			var row = TableManager.instance().getTable("profilesTable").getRowFromTd(findTdNode(this));
			var nameField = getFieldFromRow("name", row);
			ProfileManager.changeProfileDescription(nameField.getValue(), this.value);
		}
	}
	
	inputField.onfocus = function() {
		ProfileManager.clearProfileMessage();
	}
}


/**
 * Creates the change name input field, here users can change the name of existing profiles.
 * @param name The name of the profile.
 * @return The input field created. 
 */
ProfileTable.prototype.changeNameInputField = function(name) {
	var inputField = this.createInputField(CHANGE_NAME_INPUT_FIELD, MyTable.INPUT_FIELD_SIZE);
	setClass(inputField, "profile");
	this.setChangeNameActions(name, inputField);
	inputField.value = name;
	return inputField;
}


/**
 * Set the actions for the change name input field.
 * @param name The name of the profile this input field is for.
 * @param inputField The input field to set the actions for.
 */
ProfileTable.prototype.setChangeNameActions = function(name, inputField) {
	inputField.onkeyup = function(event) {
			event = exists(event) ? event : window.event;
			if (!isEmpty(this.value) && !ProfileManager.isValidProfileName(this.value)) {
				ProfileManager.setMessage(undefined, ProfileManager.INVALID_CHAR_IN_PROFLIE_NAME_MESSAGE);
			}
			else if (isEmpty(this.value) || ProfileManager.isValidProfileName(this.value)) {
				ProfileManager.clearProfileMessage();
			}
			if (event.keyCode == 13 && name != this.value) {
				ProfileManager.changeProfileName(name, this.value, this);
			}
		}
		
	inputField.onblur = function() {
		if (name != this.value) {
			ProfileManager.changeProfileName(name, this.value, this);
		}
	}
	
	inputField.onfocus = function() {
		ProfileManager.clearProfileMessage();
	}
}


/**
 * Create the name input field, where users can enter the name of new protocols.
 */
ProfileTable.prototype.createAddNameInputField = function() {
	var inputField = this.createInputField("addNameField", MyTable.INPUT_FIELD_SIZE);
	setClass(inputField, "addTableEntry");
		
	inputField.onkeyup = function(event) {
		event = exists(event) ? event : window.event;
		if (!isEmpty(this.value) && !ProfileManager.isValidProfileName(this.value)) {
			ProfileManager.setMessage(undefined, ProfileManager.INVALID_CHAR_IN_PROFLIE_NAME_MESSAGE);
		}
		else if (isEmpty(this.value) || ProfileManager.isValidProfileName(this.value)) {
			ProfileManager.clearProfileMessage();
		}
		
		if (event.keyCode == 13) { 
			ProfileManager.addProfile();
		}
	}	
	
	return inputField;
}


/**
 * Creates the description input field, where users can set a description of new profile to add. 
 */
ProfileTable.prototype.createAddDescriptionInputField = function () {
	var inputField = this.createInputField("addDescriptionField", MyTable.INPUT_FIELD_SIZE);
	setClass(inputField, "addTableEntry");
		
	inputField.onkeyup = function(event) {
		event = exists(event) ? event : window.event;
		if (event.keyCode == 13) { 
			ProfileManager.addProfile();
		}
	}	
	
	return inputField;
}


/**
 * Creates the container for the table. The container contains both the header table
 * and the data table. This is overriden to append the table as the first child in the window.
 * @return The created container.
 */
ProfileTable.prototype.createContainer = function() {
	var container = document.createElement("div");
	container.setAttribute("id", "tableContainer" + this.id);
	this.parentElement.insertBefore(container, this.parentElement.firstChild);
	return container;
}


/**
 * Creates a field with given values.
 * @param name The name to set for the created field.
 * @param value The value to set for the field.
 * @param index The index to set for the field.
 * @return The created field.
 */
ProfileTable.prototype.createField = function(name, value, index) {
	var argumentsToSuper = new Array();
	argumentsToSuper[0] = name;
	// url decode all the values once they are created in the profiles table
	argumentsToSuper[1] = URLDecode(value);
	argumentsToSuper[2] = index;
	return this.callSuper("createField", argumentsToSuper);
}