/**
 * Any previous error messages are stored here.
 */
var previousMessage = "";


/**
 * The id of the window the previous message is in. 
 */
var previousWindowId = "";


/**
 * Set to true if the setup windows have been inited.
 */
var initedSetupWindows = false;


/**
 * The load functions for each setup window.
 */
top.setupWindowLoadingFunction = new Array();
top.setupWindowLoadingFunction["statisticsSetup"] = loadStatisticsSetup;
top.setupWindowLoadingFunction["resetSettings"] = loadResetSettings;
top.setupWindowLoadingFunction["userSetup"] = loadUsers;
top.setupWindowLoadingFunction["filterSetup"] = loadFilters;
top.setupWindowLoadingFunction["probeConfig"] = loadProbeConfig;
top.setupWindowLoadingFunction["emailConfiguration"] = loadEmailConfig;
top.setupWindowLoadingFunction["slaveSetup"] = loadSlaveProbes;


/**
 * Initializes the setup windows.
 */
function initSetupWindows() {
	createSetupWindows("div", "window");
	showSetupWindow("statisticsSetup", true);
	initedSetupWindows = true;
}


/**
 * Parses the html code and each element with class set to a given value is transformed into a setup window.
 * The created windows are all deactivated, hidden and have no position set.
 * @param elementType The type of html elements with class windowClass to transform into windows.
 * @param windowClass The class of the elements to transform into windows.
 * @throws Error if elementType is undefined, null or empty.
 * @throws Error if className is undefined, null or empty.
 */
function createSetupWindows(elementType, windowClass) {
	assertNotNull(elementType, "The element type to get is undefined, null or empty: " + elementType);
	assertNotNull(windowClass, "The windowClass is unefined, null or empty: " + windowClass);
	
	var elements = getElementsWithClass(elementType, windowClass);
	var len = elements.length;
	for (var i = 0; i < len; i++) {
		var newWindow = createWindow(elements[i], elements[i].getAttribute("id"), elements[i].getAttribute("name"));
		WindowManager.instance().removeWindow(newWindow);
		newWindow = createProbeWindow(elements[i].getAttribute("id"));
		setupWindowUpdated(newWindow.id);
		WindowManager.instance().hideWindow(newWindow.getId());
		newWindow.setResizable(false);
		newWindow.toggleAutomaticPositioning(true);
	}
}


/**
 * Shows a setup window.
 * @param id The id of the window to activate.
 * @param on Set this to true and the window is shown, false for hiding a window
 */
function showSetupWindow(id, on) {
	if (!on) {
		WindowManager.instance().hideWindow(id);
		return;
	}
	
	var windowToShow = WindowManager.instance().getWindow(id);
	if (windowToShow.isHidden()) {
		// the window was hidden, clear the previous message
		clearWindowMessageField(windowToShow);
		loadWindowSetup(windowToShow);
	}
	
	WindowManager.instance().activateWindow(windowToShow.getId());
	windowToShow.resized();
}


/**
 * Starts the loading of the setup information in a window.
 * @param windowToLoad The window to load the setup information for.
 */
function loadWindowSetup(windowToLoad) {
	if (isDefined(top.setupWindowLoadingFunction[windowToLoad.getId()])) {
		top.setupWindowLoadingFunction[windowToLoad.getId()]();
	}
}


/**
 * Reloads the currently shown setup windows.
 */
function reloadShownWindows() {
	if (!initedSetupWindows) {
		return;
	}
	for (windowId in top.setupWindowLoadingFunction) {
		if (exists(WindowManager.instance().getWindow(windowId)) && WindowManager.instance().getWindow(windowId).isShown()) {
			loadWindowSetup(WindowManager.instance().getWindow(windowId));
		}
	}
}


/**
 * Clears all windows message fields. 
 */
function clearAllMessageFields() {
	var windows = WindowManager.instance().getWindows();
	for (var i = 0 ; i < windows.length; i++) {
		clearWindowMessageField(windows[i]);
	}
}


/**
 * Clear the message field of a window.
 * @param windowToClear The window to clear the message field in. 
 */
function clearWindowMessageField(windowToClear) {
	var messageFields = getElementsWithAttribute(windowToClear.getHtmlCode(), "messageField", "name", "td");
	for (var i = 0; i < messageFields.length; i++) {
		messageFields[i].innerHTML = "";
	}	
	setupWindowUpdated(windowToClear.id);
}


/**
 * A setup window has been updated, check the size is correct.
 * @param id The id of the window that has been updated.
 */
function setupWindowUpdated(id) {
	if (!isDefined(id)) {
		return;
	}
	
	var windowToAdjust = WindowManager.instance().getWindow(id);
	if (!isDefined(windowToAdjust) || isNull(windowToAdjust)) {
		return;
	}
	
	if (windowToAdjust.isHidden()) {
		// do not resize the window when it is hidden, because than the offsetWidth and height is 0
		return;
	}
	
	// this is a hack to resize the windows in Opera
	windowToAdjust.getHtmlCode().style.height = "0px";
	windowToAdjust.resized();
}


// message fields


/**
 * Sets the value of the message field. The message field must have the name or id "messageField". 
 * @param htmlCode The htmlCode for where to search for the "messageField" attribute.
 * @param attribute The attribute containing the "messageField" value.
 * @param elementType The type of the element to search for, i.e. td, table, img...
 * @param errorMessage The error message.
 * Optional parameter specifying the success message, if no error message was found.
 */
function setWindowMessageField(htmlCode, attribute, elementType, errorMessage, successMessage, windowId) {
	// clear the previous message field first, if found
	clearPreviousWindowMessage();

	var messageField = getElementWithAttribute(htmlCode, "messageField", attribute, elementType);				
	if (errorMessage != null && errorMessage.length > 0 && exists(messageField)) {
		setClass(messageField, "error");
		messageField.innerHTML = errorMessage;
	}
	else if (successMessage != null && successMessage.length > 0 && exists(messageField)) {
		setClass(messageField, "ok");
		messageField.innerHTML = successMessage;
	}
	setupWindowUpdated(windowId);

	previousMessage = messageField;
	previousWindowId = windowId;
}


/**
 * Clears the previous message field, if it exists.
 */
function clearPreviousWindowMessage() {
	if (isDefined(previousMessage) && !isEmpty(previousWindowId)) {
		previousMessage.innerHTML = "";
		setupWindowUpdated(previousWindowId);
	}
}