/**
 * This file contains methods used to add/edit/delete/show slaves in the setup screen.
 */
 

// constants

 
/**
 * The title of the add slave window. 
 */
var ADD_SLAVE_WINDOW_TITLE = "Add probe";


/**
 * The base url used to add a slave. 
 */
var ADD_SLAVE_BASE_URL = "?resource=SlaveProbeManager&mode=set&command=addSlave&sid=";


/**
 * The base url used to delete a slave. 
 */
var DELETE_SLAVE_BASE_URL = "?resource=SlaveProbeManager&mode=set&command=deleteSlave&sid=";


/**
 * The base url to edit a slave. 
 */
var EDIT_SLAVE_BASE_URL = "?resource=SlaveProbeManager&mode=set&command=editSlave&sid=";


/**
 * The message shown when the slave ip address entered is invalid.
 */
var INVALID_SLAVE_IP_ADDRESS_MESSAGE = "Invalid ip address. Valid format is xxx.xxx.xxx.xxx"; 


/**
 * The message shown when the slave ip address entered contains invalid numbers or characters.
 */
var INVALID_SLAVE_IP_NUMBERS_MESSAGE = "Invalid number in ip address. Valid is 0-255";


/**
 * The message shown when the slave port entered is invalid. 
 */
var INVALID_SLAVE_PORT_MESSAGE = "Invalid port. Valid is 0-65535";
 

/**
 * The error message shown when the slave password entered is incorrect.
 */
var INVALID_SLAVE_PASSWORD_MESSAGE = "Invalid characters in the password. Valid characters are a-z, space and digits";


/**
 * The characters allowed as slave passwords. 
 */
var VALID_SLAVE_PASSWORD_CHARACTERS = "abcdefghijklmnopqrstuvwxyz1234567890 ";

 
/**
 * The error message shown when no slave password was entered. 
 */
var NO_SLAVE_PASSWORD_MESSAGE = "No password specified"; 


/**
 * The add slave status image. 
 */
var ADD_SLAVE_STATUS_IMAGE = "themes/black/images/server.jpg";


/**
 * The status image when the master probe has no contact with the slave. 
 */
var CONTACT_SLAVE_STATUS_IMAGE = "themes/black/images/connected.jpg";


/**
 * The status image shown when the master probe has no contact to the slave. 
 */
var NO_CONTACT_SLAVE_STATUS_IMAGE = "themes/black/images/disconnected.jpg";


/**
 * The name of the event fired when a slave has been set. 
 */
var SLAVE_SET_EVENT = "SlaveSet";


/**
 * The name of the event fired when a slave has been deleted. 
 */
var SLAVE_DELETED_EVENT = "SlaveDeleted";

 
/**
 * The name of the event fired before we start to switch the current probe. 
 */
var START_SWITCH_PROBE_EVENT = "startProbeSwitch";


/**
 * The name of the event fired after the probe switch is done. 
 */
var DONE_SWITCH_PROBE_EVENT = "doneProbeSwitch";


/**
 * The event listener used to listen to event. 
 */
var slaveEventListener = new Listener();


/**
 * The drop down used to let the user select a slave. 
 */
var slaveDropDown;


/**
 * The ip address of the currently selected slave. 
 */
var selectedIp; 


/**
 * The previously selected ip address. Used when a error occured during loading 
 * of some slave data. 
 */
var previousSlaveIp;


/**
 * The window to show on slave after a slave layout has been loaded. This is used in the drilldown from the 
 * MasterMain page to the MyNetworks page. 
 */
var slaveWindowToShow;


/**
 * The loading animation used while switching slaves. 
 */
var slaveLoadingAnimation;


// services


/**
 * Inits the slaves setup. 
 */
function initSlaves() {
	slaveEventListener.addEvent(INPUT_ERROR_MESSAGE_SET_EVENT, processIpErrorMessageSetup);	
	slaveEventListener.addEvent(INPUT_ERROR_MESSAGE_CLEARED_EVENT, processIpErrorMessageSetup);
}


/**
 * Initializes the add/edit slave window.
 */
function initSlaveWindow() {
	var slaveIp = getInput(slaveDetailWindow().getHtmlCode(), "slaveIp");
	var messageField = getElementWithAttribute(slaveDetailWindow().getHtmlCode(), "slaveDetailMessageField", "name", "div");
	var ipFieldValidator = createIpFieldValidator(slaveIp, messageField);
	ipFieldValidator.setEnterFunction(setSlave);
	
	var port = getInput(slaveDetailWindow().getHtmlCode(), "slavePort");
	var portFieldValidator = createPortFieldValidator(port, messageField);
	portFieldValidator.setEnterFunction(setSlave);
	
	var password = getInput(slaveDetailWindow().getHtmlCode(), "slavePassword");
	var passwordFieldValidator = createPasswordFieldValidator(password, messageField, VALID_SLAVE_PASSWORD_CHARACTERS, INVALID_SLAVE_PASSWORD_MESSAGE);
	passwordFieldValidator.setEnterFunction(setSlave);
}


/**
 * Loads the slaves. 
 */
function loadSlaveProbes() {
	getSlaveProbeManager().loadSlaves(slavesLoaded);
}


/**
 * Callback function for the loading of the slaves.
 * @param slaves The loaded slaves. 
 */
function slavesLoaded(slaves) {
	var slavesTag = getElementWithAttribute(document.body, "slaves", "id", "div");
	slavesTag.innerHTML = "";
	var table = createSlavesTable();
	
	for (var i = 0; i < slaves.length; i++) {
		createSlaveRowCode(slaves[i], table, i + 1);
	}

	slavesTag.appendChild(table);
	updateSlaveSetupWindowSize();
}


/**
 * Shows the add slave window.
 */
function showAddSlaveWindow() {
	clearSlaveDetailFields();
	setSlaveStatusImage();
	showSlaveDetailWindow();
	
	slaveDetailWindow().setWindowTitle("Add probe");
	var slaveDetailButton = getElementWithAttribute(slaveDetailWindow().getHtmlCode(), "slaveDetails_button", "id", "input");
	slaveDetailButton.value = "Add probe";
	clearSlaveDetailMessage();
	
	getInput(slaveDetailWindow().getHtmlCode(), "slaveIp").focus();
}


/**
 * Shows the edit slave window. 
 */
function showEditSlaveWindow() {
	clearSlaveDetailFields();
	showSlaveDetailWindow();
	
	slaveDetailWindow().setWindowTitle("Edit probe");
	var slaveDetailButton = getElementWithAttribute(slaveDetailWindow().getHtmlCode(), "slaveDetails_button", "id", "input");
	slaveDetailButton.value = "Edit probe";
	clearSlaveDetailMessage();
	
	getInput(slaveDetailWindow().getHtmlCode(), "slavePort").focus();
}


/**
 * Callback function for the loading of a slave. 
 * @param slave The slave loaded.
 */
function slaveLoaded(slave) {
	showEditSlaveWindow();
	setSlaveDetailFields(slave);
}


/**
 * Adds or edits a slave based on the values set in the slave detail window.  
 */
function setSlave() {
	var window = WindowManager.instance().getWindow("slaveDetails");
	if (ADD_SLAVE_WINDOW_TITLE.equalsIgnoreCase(window.getWindowTitle())) {
		addSlave();	
	}
	else {
		editSlave();
	}
}


/**
 * Edits a slave based on the values set in the add slave window. 
 */
function editSlave() {
	clearSlaveDetailMessage();
	var url = this.createEditSlaveUrl();
	if (!isEmpty(url)) {
		loadXML(url, slaveEditedCallBack);
	}
}

 
/**
 * Adds a slave based on the values set in the add slave window. 
 */
function addSlave() {
	clearSlaveDetailMessage();
	var url = this.createAddSlaveUrl();
	if (!isEmpty(url)) {
		loadXML(url, slaveSetCallBack);
	}
}


/**
 * Deletes a slave. 
 * @param slave The slave to delete.
 */
function deleteSlave(slave) {
	if (!exists(slave)) {
		return;
	}
	var url = this.createDeleteSlaveUrl(slave);
	if (!isEmpty(url)) {
		loadXML(url);
	}
	getSlaveProbeManager().deleteSlave(slave);
	deleteSlaveRow(slave);
	var messageField = getElementWithAttribute(slaveSetupWindow().getHtmlCode(), "messageField", "name", "td")
	setSlaveMessage("Successfully deleted slave", undefined, messageField);
	updateSlaveSetupWindowSize();
	fireSlaveDeletedEvent(slave);
}


/**
 * Callback function for adding of slaves. 
 * @param response The response from the server.
 */
function slaveSetCallBack(response) {
	var messageField = getElementWithAttribute(slaveDetailWindow().getHtmlCode(), "slaveDetailMessageField", "name", "div");
	if (containsSuccessMessage(response)) {
		setSlaveMessage(getSuccessMessage(response), undefined, messageField);
		loadSlaveProbes();
		fireSlaveSetEvent();
	}
	else {
		setSlaveMessage(undefined, getErrorMessage(response), messageField);
	}
	updateSlaveDetailWindowSize();
}


/**
 * Callback function for editing slaves. 
 * @param response The response from the server.
 */
function slaveEditedCallBack(response) {
	messageField = getElementWithAttribute(slaveSetupWindow().getHtmlCode(), "messageField", "name", "td");
	if (containsSuccessMessage(response)) {
		setSlaveMessage(getSuccessMessage(response), undefined, messageField);
		loadSlaveProbes();
		fireSlaveSetEvent();
		WindowManager.instance().hideWindow("slaveDetails");
	}
	else {
		setSlaveMessage(undefined, getErrorMessage(response), messageField);
		updateSlaveDetailWindowSize();
	}
}


/**
 * Creates the edit slave url based on the value set in the slave detail window. 
 * @return The url created based on the values set in the slave detail window. 
 */
function createEditSlaveUrl() {
	try {
		return EDIT_SLAVE_BASE_URL + getSid() + "&" + createSlaveParameterUrl();
	}
	catch (e) {
		var messageField = getElementWithAttribute(slaveDetailWindow().getHtmlCode(), "slaveDetailMessageField", "name", "div");
		setSlaveMessage(undefined, e.message, messageField);
		updateSlaveDetailWindowSize();
	}
}


/**
 * Creates the add slave url based on the values set in the add slaves window. 
 * @return The url based on the values set in the add slave window. 
 */
function createAddSlaveUrl() {
	try {
		return ADD_SLAVE_BASE_URL + getSid() + "&" + createSlaveParameterUrl();
	}
	catch (e) {
		var messageField = getElementWithAttribute(slaveDetailWindow().getHtmlCode(), "slaveDetailMessageField", "name", "div");
		setSlaveMessage(undefined, e.message, messageField);
		updateSlaveDetailWindowSize();
	}
}


/**
 * Creates the part of a slave url containing the parameters, based on the values set in the slave
 * detail window. 
 * @return The parameters url created. 
 * @throws Error if one the parameters set is invalid. 
 */
function createSlaveParameterUrl() {
	var url = "";
	var window = WindowManager.instance().getWindow("slaveDetails");
	var ip = getInput(window.getHtmlCode(), "slaveIp");
	if (isEmpty(ip.value) || !isValidIpAddress(ip.value)) {
		ip.focus();
		throw new Error(INVALID_SLAVE_IP_ADDRESS_MESSAGE);
	}
	url += "ip=" + ip.value + "&";
	
	var port = getInput(window.getHtmlCode(), "slavePort");
	if (isEmpty(port.value) || !isNumberString(port.value) || !isValidPort(port.value)) {
		port.focus();
		throw new Error(INVALID_SLAVE_PORT_MESSAGE);
	}
	url += "port=" + port.value + "&";
	
	var login = encodeURIComponent(getInput(window.getHtmlCode(), "slaveLogin").value);
	url += "login=" + login + "&";
	
	var password = getInput(window.getHtmlCode(), "slavePassword");
	if (isEmpty(password.value)) {
		password.focus();
		throw new Error(NO_SLAVE_PASSWORD_MESSAGE);
	}	
	if (!isValidPassword(password.value)) {
		password.focus();
		throw new Error(INVALID_SLAVE_PASSWORD_MESSAGE);
	}	
	url += "password=" + password.value + "&";
	return url;
}


/**
 * Creates the delete slave url.
 * @param slave The slave to delete. 
 * @return The url to delete the slave. 
 */
function createDeleteSlaveUrl(slave) {
	return DELETE_SLAVE_BASE_URL + getSid() + "&ip=" + slave.getIpAddress() + "&";
}


/**
 * Sets the slave message.
 * @param successMessage The success message to set.
 * @param errorMessage The error message to set. 
 * @param messageField The field to set the message in.
 */
function setSlaveMessage(successMessage, errorMessage, messageField) {
	if (isDefined(successMessage) && !isEmpty(successMessage)) {
		setClass(messageField, "messageContainer ok");
		messageField.innerHTML = successMessage;
	}
	else if (isDefined(errorMessage) && !isEmpty(errorMessage)) {
		setClass(messageField, "messageContainer error");
		messageField.innerHTML = errorMessage;
	}
}


/**
 * Clears the message field in the slave detail window.
 */
function clearSlaveDetailMessage() {
	clearSlaveMessageField(getElementWithAttribute(slaveDetailWindow().getHtmlCode(), "slaveDetailMessageField", "name", "div"));	
	updateSlaveDetailWindowSize();
}


/**
 * Clears the message field in the slave setup window. 
 */
function clearSlaveSetupMessage() {
	clearSlaveMessageField(getElementWithAttribute(slaveSetupWindow().getHtmlCode(), "messageField", "name", "td"));
	updateSlaveSetupWindowSize();
}


/**
 * Clears a given message field.
 * @param messageField The message field to clear. 
 */
function clearSlaveMessageField(messageField) {
	messageField.innerHTML = "";
}


/**
 * Creates the html code for a slave row in the slaves table. 
 * @param slave The slave to create the row code for. 
 * @param table The table to add the row to.
 * @param index The index of this slave into the table.
 */
function createSlaveRowCode(slave, table, index) {
	var row = table.insertRow(index);
	row.setAttribute("name", slave.getIpAddress());
	
	var cell = row.insertCell(0);
	var editLink = document.createElement("a");
	editLink.appendChild(document.createTextNode(slave.getIpAddress()));
	editLink.href = "#";
	editLink.onclick = function() {
		// update the slave data
		getSlaveProbeManager().updateSlave(slave, slaveLoaded);
		return false;
	}
	cell.appendChild(editLink);
	
	cell = row.insertCell(1);
	cell.appendChild(document.createTextNode(URLDecode(slave.getName())));
	
	cell = row.insertCell(2);
	cell.appendChild(document.createTextNode(URLDecode(slave.getDescription())));
	
	cell = row.insertCell(3);
	cell.appendChild(document.createTextNode(slaveStatusText(slave.getStatus())));
	setClass(cell, slaveStatusTextFieldClass(slave.getStatus()));
	
	cell = row.insertCell(4);
	var deleteLink = document.createElement("a");
	deleteLink.setAttribute("title", "Delete slave");
	deleteLink.href ="#";
	deleteLink.onclick = function() {
		deleteSlave(slave);
		return false;
	}
	var deleteImage = document.createElement("img");
	deleteImage.src = "images/icon_delete.gif";
	deleteImage.setAttribute("border", "0");
	deleteLink.appendChild(deleteImage);
	cell.appendChild(deleteLink);
}

 
/**
 * Creates the html code for table containing the slaves.
 * @return The created table.  
 */
function createSlavesTable() {
	var table = document.createElement("table");
	table.width = "600px";
	table.cellpadding = "0px";
	table.cellspacing = "0px";
	setClass(table, "defaultTable setupBox");
	table.setAttribute("name", "slavesSetupTable");
	var tbody = document.createElement("tbody");
	table.appendChild(tbody);
	
	var headerRow = table.insertRow(0);
	setClass(headerRow, "header");
	
	var cell = headerRow.insertCell(0);
	cell.style.verticalAlign = "top";
	cell.appendChild(document.createTextNode("Ip address"));
	
	cell = headerRow.insertCell(1);
	cell.style.verticalAlign = "top";
	cell.appendChild(document.createTextNode("Name"));
	
	cell = headerRow.insertCell(2);
	cell.style.verticalAlign = "top";
	cell.appendChild(document.createTextNode("Description"));
	
	cell = headerRow.insertCell(3);
	cell.style.verticalAlign = "top";
	cell.appendChild(document.createTextNode("Status"));
	
	cell = headerRow.insertCell(4);
	cell.style.verticalAlign = "top";
	var reloadImage = document.createElement("img");
	reloadImage.src = "themes/black/images/reload.jpg";
	reloadImage.setAttribute("border", "0");
	reloadImage.onclick = loadSlaveProbes;
	reloadImage.style.padding = "0px";
	reloadImage.style.margin = "0px";
	reloadImage.style.cursor = "pointer";
	cell.appendChild(reloadImage);
	cell.style.padding = "0px";
	cell.style.paddingTop = "2px";
	
	return table;
}
 

/**
 * Deletes the html table row for a given slave. 
 * @param slave The slave to remove from the html table. 
 */
function deleteSlaveRow(slave) {
	var slavesSetupTable = getElementWithAttribute(document.body, "slavesSetupTable", "name", "table");
	var tbody = slavesSetupTable.getElementsByTagName("tbody")[0];
	var row = getElementWithAttribute(slavesSetupTable, slave.getIpAddress(), "name", "tr");
	tbody.removeChild(row);
}


/**
 * Returns the slave setup window.
 */
function slaveSetupWindow() {
	return WindowManager.instance().getWindow("slaveSetup");
}


/**
 * Returns the slave detail window.
 */
function slaveDetailWindow() {
	return WindowManager.instance().getWindow("slaveDetails");
}


/**
 * Updates the size of the slave setup window to its actual size. When the slave setup
 * is changed and slaves are added or removed the height of the slave setup window
 * can change.   
 */
function updateSlaveSetupWindowSize() {
	// this hack is needed to set the window size correct if the window 
	// content is made smaller
	slaveSetupWindow().getHtmlCode().style.height = "0px";
	slaveSetupWindow().resized();
}


/**
 * Updates the size of the slave detail window to its actual size. When a message is shown
 * or hidden the size of the window changes. 
 */
function updateSlaveDetailWindowSize() {
	// this hack is needed to set the window size correct if the window 
	// content is made smaller
	slaveDetailWindow().getHtmlCode().style.height = "0px";
	slaveDetailWindow().resized();
}


/**
 * Shows the slave detail window. 
 */
function showSlaveDetailWindow() {
	var hidden = slaveDetailWindow().isHidden(); 
	WindowManager.instance().activateWindow("slaveDetails");
	if (hidden) {
		// the window was hidden, clear the previous message
		clearSlaveDetailMessage();
	}
	slaveDetailWindow().resized();
}


/**
 * Shows the slave setup window. 
 */
function showSlaveSetupWindow() {
	if (slaveSetupWindow().isHidden()) {
		// the window was hidden, clear the previous message
		clearSlaveSetupMessage();
	}
	WindowManager.instance().activateWindow("slaveSetup");
	slaveSetupWindow().resized();	
}

 
// events


/**
 * Processes a ip error message set event.
 * @param event The event to process. 
 */
function processIpErrorMessageSetup() {
	updateSlaveDetailWindowSize();
}


/**
 * Fires a slave set event.
 */
function fireSlaveSetEvent() {
	var ip = getInput(slaveDetailWindow().getHtmlCode(), "slaveIp");
	getEventServer().notifyListeners(new Event(SLAVE_SET_EVENT, ip.value));
}


/**
 * Fires a slave deleted event.
 * @param slave The slave deleted. 
 */
function fireSlaveDeletedEvent(slave) {
	getEventServer().notifyListeners(new Event(SLAVE_DELETED_EVENT, slave.getIpAddress()));
}


// implementation


/**
 * Sets the fields in the slave detail window according to one slave. 
 * @param slave The slave to ge the values from.
 */
function setSlaveDetailFields(slave) {
	var window = WindowManager.instance().getWindow("slaveDetails");
	getInput(window.getHtmlCode(), "slaveIp").value = slave.getIpAddress();
	getInput(window.getHtmlCode(), "slaveIp").readOnly = true;
	getInput(window.getHtmlCode(), "slavePort").value = slave.getPort();
	getInput(window.getHtmlCode(), "slavePassword").value = slave.getPassword();
	getInput(window.getHtmlCode(), "slaveLogin").value = slave.getLogin();
	setSlaveStatusImage(slave);	
}


/**
 * Clears the fields in the slave detail window.
 */
function clearSlaveDetailFields() {
	var window = WindowManager.instance().getWindow("slaveDetails");
	getInput(window.getHtmlCode(), "slaveIp").value = "";
	getInput(window.getHtmlCode(), "slaveIp").readOnly = false;
	getInput(window.getHtmlCode(), "slavePort").value = "7030";
	getInput(window.getHtmlCode(), "slavePassword").value = "";
	getInput(window.getHtmlCode(), "slaveLogin").value = "admin";
}


/**
 * Sets the slave status image in the slave detail window according to a given slaves status,
 * if no slave is given the add slave probe image is shown.  
 * @param slave The slave to set the status image for. 
 */
function setSlaveStatusImage(slave) {
	var slaveImage = getElementWithAttribute(document.body, "slaveStatusImage", "name", "img");
	if (!exists(slave)) {
		slaveImage.src = ADD_SLAVE_STATUS_IMAGE;
		slaveImage.setAttribute("title", "Add a probe");
	}
	else if (SLAVE_PROBE_RUNNING_STATUS == slave.getStatus()) {
		slaveImage.src = CONTACT_SLAVE_STATUS_IMAGE;
		slaveImage.setAttribute("title", "Probe is running");
	}
	else if (SLAVE_PROBE_STOPPED_STATUS == slave.getStatus()) {
		slaveImage.src = CONTACT_SLAVE_STATUS_IMAGE;
		slaveImage.setAttribute("title", "Probe stopped");
	}
	else if (SLAVE_SIMULATION_DONE_STATUS == slave.getStatus()) {
		slaveImage.src = CONTACT_SLAVE_STATUS_IMAGE;
		slaveImage.setAttribute("title", "Simulation done");
	}
	else if (SLAVE_NO_CONTACT_STATUS == slave.getStatus()) {
		slaveImage.src = NO_CONTACT_SLAVE_STATUS_IMAGE;
		slaveImage.setAttribute("title", "No contact");
	}
	else if (INVALID_SLAVE_PASSWORD_LOGIN == slave.getStatus()) {
		slaveImage.src = NO_CONTACT_SLAVE_STATUS_IMAGE;
		slaveImage.setAttribute("title", "Invalid password and/or login");
	}
}


// MyNetworks slave selection


/**
 * Inits the slave selector. 
 */
function initSlaveSelector() {
	var currentProbeContainer = getElementWithAttribute(document.body, "currentProbeContainer", "id", "div");
	var slaveDropDownContainer = getElementWithAttribute(document.body, "slaveDropDownContainer", "id", "div");
	slaveDropDown = new DropDown(currentProbeContainer);
	slaveDropDown.setParentNode(slaveDropDownContainer);
	slaveDropDown.setSelectedListener(slaveSelected);
	slaveEventListener = new Listener();
	slaveEventListener.addEvent("tabActivated", tabActivatedSlave);
	slaveEventListener.addEvent(DropDown.SHOWN_EVENT, slaveDropDownShown);
	slaveEventListener.addEvent(DropDown.HIDDEN_EVENT, slaveDropDownHidden);
}


/**
 * Loads the slaves, once the slaves have been loaded the populateSlaveDropDown
 * function is called and the slaves are added to the dropdown menu.  
 */
function loadSlavesDropDown() {
	getSlaveProbeManager().loadSlaves(populateSlaveDropDown);	
}


/** 
 * Populates the slave dropdown menu with the slaves currently in the slave probe manager.  
 */
function populateSlaveDropDown() {
	var slaves = getSlaveProbeManager().getSlaves();
	slaveDropDown.clear();
	var dropDownElements = new Array();
	dropDownElements.push(createLocalDropDownCode());
	for (var i = 0; exists(slaves) && i < slaves.length; i++) {
		dropDownElements.push(createProbeDropDownElementCode(slaves[i].getIpAddress(), getSlaveName(slaves[i])));
	}
	var section = new DropDownSection(dropDownElements, slaveDropDown);
	section.setHeader("Probes");
	slaveDropDown.addSection(section);
	highlightCurrentSlave();
}


/**
 * Creates the html code for the local probe for the dropdown menu. 
 * @return The html code to use in the dropdown menu for the local probe.  
 */
function createLocalDropDownCode() {
	var text = isEmpty(getProbeName()) ? getServerAddress() : getProbeName();
	return createProbeDropDownElementCode(getServerAddress(), text);
}

 
/**
 * Creates the html code for a probe to be used in the dropdown menu. 
 * @param id The id to set for the html code.
 * @param text The text to set in the dropdown element. 
 * @return The html code created. 
 */
function createProbeDropDownElementCode(id, text) {
	var probeContainer = document.createElement("div");
	probeContainer.setAttribute("id", id);
	probeContainer.appendChild(document.createTextNode(text));
	return probeContainer;
}


/**
 * A slave has been selected in the dropdown menu (clicking enter or mouse click). 
 * @param element The dropdown menu element for the slave that has been selected.  
 */
function slaveSelected(element) {
	if (selectedIp == element.getAttribute("id")) {
		// the same slave has been selected again
		return;
	}
	selectSlave(element.getAttribute("id"));
}


/**
 * Selects a given slave and shows a given window for the slave. 
 * @param slaveIp The dotted ip address as a string of the slave to set (10.0.2.33).
 * @param windowId The id of the window to set. 
 * @throws Error if slaveIp is undefined, null, empty or not a valid ip address.
 * @throws Error if windowId is undefined, null or empty. 
 */
function showSlaveWindow(slaveIp, windowId) {
	assertExists(slaveIp, "The ip address of the slave to set dow not exist");
	assertExists(windowId, "The id of the window to show is undefined, null or empty");
	assertTrue(isValidIpAddress(slaveIp), "The ip of the slave to show is not a valid ip address");
	
	showSlaveLoadingAnimation();
	// the local probe has the ip 0.0.0.0 set
	slaveIp = "0.0.0.0" == slaveIp ? getServerAddress() : slaveIp;	
	getSlaveProbeManager().loadSlaves(function() {
		populateSlaveDropDown();
		if (slaveIp == selectedIp && exists(WindowManager.instance().getWindow(windowId))) {
			WindowManager.instance().activateWindow(windowId);
			hideSlaveLoadingAnimation();
		}
		else {
			selectSlave(slaveIp);
		}
	});
	slaveWindowToShow = windowId;
}

 
/**
 * Select a slave with a given ip address in the dropdown menu. 
 * @param slaveIp The ip address as a dotted string (10.0.2.33) of the slave to select. 
 */
function selectSlave(slaveIp) {
	if (!exists(slaveIp) || isEmpty(slaveIp)) {
		hideSlaveLoadingAnimation();
		return;
	}
	var probeExists = exists(getSlaveProbeManager().getSlave(slaveIp)) || slaveIp == getServerAddress();
	if (slaveIp == selectedIp) {
		hideSlaveLoadingAnimation();	
		return;
	}
	if (!probeExists && isEmpty(selectedIp)) {
		//  no slave has been selected yet, select the local one
		MessageManager.instance().showErrorMessage("Could not contact probe: " + (exists(slave) ? getSlaveName(slave) : selectedIp));
		selectSlave(getServerAddress());
	}
	else if (!probeExists) {
		// probe does not exist, do not change the selection
		MessageManager.instance().showErrorMessage("Could not contact probe: " + (exists(slave) ? getSlaveName(slave) : selectedIp));
		hideSlaveLoadingAnimation();
		return;
	}
	// switch the current probe to the given probe
	getEventServer().notifyListeners(new Event(START_SWITCH_PROBE_EVENT, slaveIp));
	previousSlaveIp = selectedIp;
	selectedIp = slaveIp;
	highlightCurrentSlave();
	udpateCurrentProbeField();
	loadSlaveData();
}


/**
 * Gets the ip address of the selected slave. 
 */
function getSelectedSlave() {
	return selectedIp;
}


/**
 * Highlights the current selected slave in the slave selection drop down menu. 
 */
function highlightCurrentSlave() {
	if (exists(getSlaveDropDownElement(selectedIp))) {
		slaveDropDown.highlightElement(getSlaveDropDownElement(selectedIp));
	}
}


/**
 * Gets the dropdown element used for a given slave. 
 * @param ip The ip address of the slave to get the element for. 
 * @return The element used for the slave with ip in the dropdown menu, null if no slave with ip. 
 */
function getSlaveDropDownElement(ip) {
	var elements = slaveDropDown.getElements();
	for (var i = 0; exists(elements) && i < elements.length; i++) {
		if (elements[i].getAttribute("id") == ip) {
			return elements[i];			
		}
	}
	return null;
}


/**
 * Hides the current probe field. 
 */
function hideCurrentProbeField() {
	var currentProbeContainer = getElementWithAttribute(document.body, "currentProbeContainer", "id", "div");
	setClass(currentProbeContainer, "inactive");
	slaveDropDown.hide();
}


/**
 * Shows the current probe field. 
 */
function showCurrentProbeField() {
	var currentProbeContainer = getElementWithAttribute(document.body, "currentProbeContainer", "id", "div");
	setClass(currentProbeContainer, "currentProbeContainer");
}


/**
 * A tab has been activated, show the probe selection field, and reload the slaves 
 * if the tab shown is the Main tab.  
 * @param event The event to process. 
 */
function tabActivatedSlave(event) {
	if (SUMMARY_TAB_NAME != event.getData() || !getMasterMode()) {
		return;
	}
	showCurrentProbeField();
	loadSlavesDropDown();
}


/**
 * Updates the current probe field with current selected slaves name. 
 */
function udpateCurrentProbeField() {
	var probeName = "";
	slave = isEmpty(selectedIp) ? null : getSlaveProbeManager().getSlave(selectedIp);
	if (selectedIp == getServerAddress()) {
		probeName = isEmpty(getProbeName()) ? getServerAddress() : getProbeName();
	}
	else if (exists(slave)) {
		probeName = !isEmpty(slave.getName()) ? slave.getName() : slave.getIpAddress();
	}
	
	if (!isEmpty(probeName)) {
		var currentProbeName = getElementWithAttribute(document.body, "currentProbeName", "id", "div");
		currentProbeName.innerHTML = "<nobr>Current probe: " + probeName + "</nobr>";
	}
}


/**
 * Gets the name of a slave. If the slaves name is empty the slaves ip address is returned. 
 * @param slave The slave to get the name for. 
 * @return The slaves name or its ip address if no name set. 
 */
function getSlaveName(slave) {
	return isEmpty(slave.getName()) ? slave.getIpAddress() : slave.getName();
}


/**
 * Loads data for the current selected slave.
 */
function loadSlaveData() {
	showSlaveLoadingAnimation();
	setForwardIp(getSelectedSlave() == getServerAddress() ? null : getSelectedSlave());	
	ProfileManager.loadProfiles();
	loadSlaveLayout();
}


/**
 * Loads the layout for a slave. We use this instead of the layout managers load method to 
 * check for errors for example if the slave could not be connected to or a connection
 * timeout appeared. 
 */
function loadSlaveLayout() {
	loadXML(createLayoutUrl(), slaveLayoutLoaded, null, slaveLayoutError);
}


/**
 * Callback method for the loading of slave layouts.  
 * @param layout The layout to set as xml data.
 * @param 
 */
function slaveLayoutLoaded(layout) {
	if (!exists(layout) || getErrorMessage(layout) == "No layout set") {	
		// this might be the first time the tab is loaded for the current probe
		// set the default layout 
		setDefaultLayout();
		getEventServer().notifyListeners(new Event(DONE_SWITCH_PROBE_EVENT, null));
		hideSlaveLoadingAnimation();
		return;
	}
	else if (containsErrorMessage(layout)) {
		slaveLayoutError(getErrorMessage(layout));
		getEventServer().notifyListeners(new Event(DONE_SWITCH_PROBE_EVENT, null));
		hideSlaveLoadingAnimation();
		return;
	}
	setLayout(layout);
	if (!isEmpty(slaveWindowToShow) && exists(WindowManager.instance().getWindow(slaveWindowToShow))) {
		WindowManager.instance().activateWindow(slaveWindowToShow);
		slaveWindowToShow = null;
	}
	getEventServer().notifyListeners(new Event(DONE_SWITCH_PROBE_EVENT, null));
	hideSlaveLoadingAnimation();
}


/**
 * A error has occured during loading of the slave layout. Most likely the slave
 * can not be contacted any more. Load the previsous slaves data. 
 * @param errorMessage The error message to set. 
 */
function slaveLayoutError(message) {
	if (isEmpty(message)) {
		var slave = getSlaveProbeManager().getSlave(selectedIp);
		message	= "Could not contact probe: " + (exists(slave) ? getSlaveName(slave) : selectedIp);
	}
	MessageManager.instance().showErrorMessage(message);
	// select the previous probe, the local probe is no probe has been selected before 
	selectSlave(isEmpty(previousSlaveIp) ? getServerAddress() : previousSlaveIp);
}


/**
 * Hides the slave loading animation. 
 */
function hideSlaveLoadingAnimation() {
	if (exists(slaveLoadingAnimation)) {
		slaveLoadingAnimation.hide();
	}
}


/**
 * Shows the slave loading animation. 
 */
function showSlaveLoadingAnimation() {
	if (!isDoneLoading()) {
		// when the page has not loaded the page loading animation is still shown,
		// no need to show the show the slave loading too
		return;
	}
	if (!exists(slaveLoadingAnimation)) {
		initSlaveLoadingAnimation();
	}
	slaveLoadingAnimation.show();
}


/**
 * Inits the slave loading animation. 
 */
function initSlaveLoadingAnimation() {
	var iframeCode = getElementWithAttribute(top.document.body, currentTabName, "name", "iframe");	
	if (!exists(iframeCode)) {
		// happens in tests
		return;
	}	
	var height = iframeCode.offsetHeight + "px";
	var width = iframeCode.offsetWidth + "px";
	slaveLoadingAnimation = new LoadingAnimation(document.body, height, width, "Swtiching probe...");
}


/**
 * The slave dropdown has been shown, also show its parent element. 
 */
function slaveDropDownShown() {
	var slaveDropDownContainer = getElementWithAttribute(document.body, "slaveDropDownContainer", "id", "div");
	setClass(slaveDropDownContainer, "slaveDropDownContainer");
}


function slaveDropDownHidden() {
	var slaveDropDownContainer = getElementWithAttribute(document.body, "slaveDropDownContainer", "id", "div");
	setClass(slaveDropDownContainer, "inactive");
}