/**
 * This class stores the attributes of a slave probe. 
 */
 

// constants


/**
 * The running status for a slave probe. 
 */
var SLAVE_PROBE_RUNNING_STATUS = "1";


/**
 * The stopped status for a slave probe.
 */
var SLAVE_PROBE_STOPPED_STATUS = "2";


/**
 * A slaves status when the reading of packets from a file is done. 
 */
var SLAVE_SIMULATION_DONE_STATUS = "3";


/**
 * A slaves status when it can not be contacted. 
 */
var SLAVE_NO_CONTACT_STATUS = "0";


/**
 * A slaves status when the wrong password or login is sett.
 */
var INVALID_SLAVE_PASSWORD_LOGIN = "4";


// construction


/**
 * Factory method to create a slaveProbe from a given block of xml.
 * @param slaveXml The slave xml data to create the slave from.
 * @throws Error if slaveXml is undefined or null.
 */
function createSlaveProbeXml(slaveXml) {
	assertExists(slaveXml, "The xml data to create the slave from is undefined or null");
	var ip = getTagNodeValue("ip", slaveXml);
	var slave = new SlaveProbe(ip);
	
	slave.setAttribute("name", getTagNodeValue("name", slaveXml)); 
	slave.setAttribute("description", getTagNodeValue("description", slaveXml));
	slave.setAttribute("port", getTagNodeValue("port", slaveXml));
	slave.setAttribute("login", getTagNodeValue("login", slaveXml));
	slave.setAttribute("password", getTagNodeValue("password", slaveXml));
	slave.setAttribute("status", getTagNodeValue("status", slaveXml));
	
	return slave;
}


/**
 * Constructs a slave probe. 
 * @param address The ip address of this slave probe entry as a dotted string.
 * @throws Error if address is undefined, null, empty or not a valid ip address (10.0.2.3). 
 */
function SlaveProbe(address) {
	assertNotEmpty(address, "Invalid ip address for the slave: " + address);
	if (!isValidIpAddress(address)) {
		throw new Error("Invalid ip address for the slave: " + address);
	}
	
	this.address = address;
	
	
	/**
	 * The port used for this slave. 
	 */
	this.port;
	
	
	/**
	 * The name of this slave. 
	 */
	this.name;
	
	
	/**
	 * The description for this slave. 
	 */
	this.description;
	
	
	/**
	 * The login for this slave. 
	 */
	this.login;
	
	
	/**
	 * The status to for the slave. 
	 */
	this.status = null;
	
	
	/**
	 * The password to use for the slave. 
	 */
	this.password;
}


// services


/**
 * Get the ip address.
 * @return The ip address set for this slave as a dotted string (10.0.2.3).
 */
SlaveProbe.prototype.getIpAddress = function() {
	return this.address;
}


/**
 * Set the port for this slave. 
 * @param port The port to set. 
 * @throws Error if port is not valid. 
 */
SlaveProbe.prototype.setPort = function(port) {
	assertIsNumber(port, "The port to set for the slave is not a number: " + port);
	if (!isValidPort(port)) {
		throw new Error("The port to set for the slave is not valid: " + port);
	}
	this.port = port;	
}


/**
 * Get the port set for this slave. 
 * @return The port set, undefined if no port set yet.
 */
SlaveProbe.prototype.getPort = function(port) {
	return this.port;
}


/**
 * Set the name for this probe. 
 * @param name The name to set. 
 * @throws Erorr if name is undefined or null.
 */
SlaveProbe.prototype.setName = function(name) {
	assertExists(name, "The name to set for the slave is undefined or null");
	this.name = name;
}


/**
 * Gets the name set for this slave. 
 * @return The name set, undefined if no name set yet.
 */
SlaveProbe.prototype.getName = function() {
	return this.name;
}


/**
 * Sets the description for this slave. 
 * @param description The description to set. 
 * @throws Error if description is undefined or null.
 */
SlaveProbe.prototype.setDescription = function (description) {
	assertExists(description, "The description to set for the slave is undefined or null");
	this.description = description;
}


/**
 * Get the description set for this slave.  
 * @return The description set, undefined if no description set yest. 
 */
SlaveProbe.prototype.getDescription = function() {
	return this.description;
}


/**
 * Set the login to use for this slave. 
 * @param login The login to set. 
 * @throws Error if login is undefined, null or empty.
 */
SlaveProbe.prototype.setLogin = function(login) {
	assertNotEmpty(login, "The login to set is undefined, null or empty");
	this.login = login;	
}


/**
 * Gets the login to use for this slave. 
 * @return The login, undefined if no login set yet.
 */
SlaveProbe.prototype.getLogin = function() {
	return this.login;
}


/**
 * Set the status to use for the slave. 
 * @param status The status to set for this slave.
 * @throws Error if the slave status is not one of the valid statuses. 
 */
SlaveProbe.prototype.setStatus = function(status) {
	assertValidSlaveStatus(status, "Invalid slave status"); 
	this.status = status;
}
  
  
/**
 * Gets the status set for this slave. 
 * @return The status set, undefined if no status set yet. 
 */
SlaveProbe.prototype.getStatus = function() {
	return this.status;
}


/**
 * Sets the password to use for this slave. 
 * @param password The password to set. 
 * @throws Error if the password is undefined, null or empty.
 */
SlaveProbe.prototype.setPassword = function(password) {
	assertNotEmpty(password, "The password to set for the slave is undefined null or empty");
	this.password = password;
}
 
 
/**
 * Gets the password set for this slave. 
 * @return The password set, undefined if no password set yet. 
 */
SlaveProbe.prototype.getPassword = function() {
	return this.password;
}
 
 
/**
 * Sets a slave attribute.
 * @param name The name of the attribute to set. 
 * @param value The value to set for the attribute.
 */
SlaveProbe.prototype.setAttribute = function(name, value) {
	if ("name".equalsIgnoreCase(name) && exists(value)) {
		this.setName(value);
	}
	else if ("description".equalsIgnoreCase(name) && exists(value)) {
		this.setDescription(URLDecode(value));
	}
	else if ("port".equalsIgnoreCase(name) && !isEmpty(value) && isNumberString(value)) {
		this.setPort(parseInt(value));
	}
	else if ("login".equalsIgnoreCase(name) && !isEmpty(value)) {
		this.setLogin(URLDecode(value));
	}
	else if ("password".equalsIgnoreCase(name) && !isEmpty(value)) {
		this.setPassword(value);
	}
	else if ("status".equalsIgnoreCase(name) && !isEmpty(value) && isNumberString(value)) {
		this.setStatus(value);
	}
}
 
 
// implementation


/**
 * Throws a error if a status is not one of the valid slave statueses.
 * @param status The status to check.
 * @param message The message to throw when status is not valid. 
 * @throws Error if status is not valid. 
 */
function assertValidSlaveStatus(status, message) {
	if (SLAVE_PROBE_RUNNING_STATUS.equalsIgnoreCase(status)) {
		return;
	}
	if (SLAVE_PROBE_STOPPED_STATUS.equalsIgnoreCase(status)) {
		return;
	}
	if (SLAVE_SIMULATION_DONE_STATUS.equalsIgnoreCase(status)) {
		return;
	}
	if (INVALID_SLAVE_PASSWORD_LOGIN.equalsIgnoreCase(status)) {
		return;
	}
	if (SLAVE_NO_CONTACT_STATUS.equalsIgnoreCase(status)) {
		return;
	}
	throw new Error(message);
}