/**
 * This class shows text suggestions for a text input field. 
 */
 
 
// construction


/**
 * Constructs a instance of this class.
 * @param field The text field to suggest values for. 
 * @param baseUrl The base url used to request values from the server.
 * @param options The option to use for this instance.
 * @throws Error field is undefined or null.
 * @throws Error if baseUrl is undefined, null or empty.
 */
function TextSuggest(field, baseUrl, options) {
	assertExists(field, "The field to suggest value for is undefined or null");
	assertNotEmpty(baseUrl, "The base url to use to request suggestions is undefined, null or empty");
	this.id = field.getAttribute("id");
	this.textInput = field;
	this.textInput.suggest = this;
	
	this.url = baseUrl;
	this.setOptions(options);
	this.injectSuggestedBehaviour();
	
	// the loaded protocol and nlhost options
	this.protocolSuggestions = [];
	this.nlHostSuggestions = []
		
	// the nlhost and protocol suggestions matching the current input field value
	this.matchingProtocols = [];
	this.matchingNlHosts = [];
	
	// the last request string sendt to the server
	this.lastRequestString = "";
	
	// the protocol header shown in the suggestions
	this.protocolHeader;
	
	// the nlhost header shown in the suggestions
	this.nlHostHeader;
	
	// the currently selected suggestion, null if none selected
	// if the user presses enter and selects one suggestion no more suggestions are shown until 
	// the user types something more in the input field, the user has chosen one suggestion
	this.selectedSuggestion = null;
	
	// the dropdown menu used to show results
	this.dropDown = new DropDown();
	this.dropDown.setParentNode(this.htmlCode);
	this.dropDown.setSelectedListener(this.getDropDownSelectListener());
}


// services


/**
 * Gets the dropdown menu used in this suggest component. 
 * @return The instance ofthe dropdown menu in use. 
 */
TextSuggest.prototype.getDropDown = function() {
	return this.dropDown;
}


/**
 * Hides the suggestions.
 */
TextSuggest.prototype.hideSuggestions = function() {
	setClass(this.htmlCode, "inactive");
}


/**
 * Checks if the suggestions are shown.
 * @return True if the suggestions are shown, false otherwise. 
 */
TextSuggest.prototype.suggestionsShown = function() {
	return this.options.suggestDivClassName == getClassName(this.htmlCode);
}


/**
 * Shows the suggestions.
 */
TextSuggest.prototype.showSuggestions = function() {
	if (exists(this.options.showSuggestionsFunction) && typeof this.options.showSuggestionsFunction == "function") {
		this.options.showSuggestionsFunction();
	}
	setClass(this.htmlCode, this.options.suggestDivClassName);
	// show the dropdown
	this.dropDown.repaint();
	this.dropDown.show();
}


/**
 * Clears all the input field and hides any suggestions. 
 */
TextSuggest.prototype.clear = function() {
	this.lastRequestString = "";
	this.hideSuggestions();
	this.textInput.value = "";
}


// implementation


/**
 * Sets the options to use.
 * @param options The options to set. 
 */
TextSuggest.prototype.setOptions = function(options) {
	// set the default options
	this.options = {
		slectionColor : "black",
		// the x position to set for the suggestions div
		xPos : 0,
		// the y position to set for the suggestions div
		yPos : 0,
		ignoreCase : true,
		matchAnyWhere : true,
		// the class set for the div tag containing all the suggestions
		suggestDivClassName : "suggestDiv",
		// the class name set for the span tag containing the match text
		matchClassName : "match",
		// the max count of choices to show for hosts and protocols
		maxCount : 10,
		// set this to true to show the protocol header in the suggestions shown
		showProtocolHeader : true,
		// set this to true to show the nlhost header in the suggestions shown
		showNlHostHeader : true,
		// function to call before the suggestions are shown
		showSuggestionsFunction : null,
		// set this to true to show nlhost and protocol images
		showSuggestionImages : true,
		// if this field is set the selecteds value is inserted into this input field if valueIndex is set
		valueField : null,
		// function to call when a suggestion has been selected
		selectedFunction : null
	}
	// subclass the object with the default values. The values set in options are overridden in the resulting instance
	this.options = createSubClass(this.options, options);
}


/**
 * A suggestion has been selected. Sets the value in the input and value 
 * field according to the given selection.
 */
TextSuggest.prototype.suggestionSelected = function() {
	var suggestion = this.getHighlightedSuggestion();
	if (!this.suggestionsShown() || !exists(suggestion)) {
		return;
	}
	
	this.selectedSuggestion = suggestion;
	this.textInput.value = URLDecode(suggestion[0]);
	this.setValueField(suggestion);
	this.callSelectedFunction();
}


/**
 * Calls the selected function. 
 */
TextSuggest.prototype.callSelectedFunction = function() {
	if (exists(this.options.selectedFunction) && typeof this.options.selectedFunction == "function") {
		this.options.selectedFunction();
	}
}


/**
 * Sets a given suggestion into the value field.  
 * @param suggestion The suggestion to insert into the value field.  
 */
TextSuggest.prototype.setValueField = function(suggestion) {
	if (exists(this.options.valueField) && suggestion.length >= 3) {
		this.options.valueField.value = suggestion[2];
	}	
}


/**
 * Gets the highlighted suggestion. 
 * @return The current highlighted suggestion, null if none is highlighted or there are no suggestions. 
 */
TextSuggest.prototype.getHighlightedSuggestion = function() {
	var suggestion = null;
	var highlightedIndex = this.dropDown.getHighlightedIndex();
	if (highlightedIndex != -1 && highlightedIndex <= this.matchingProtocols.length -1) {
		// the highlighted suggestion is a protocol suggestion
		suggestion = this.matchingProtocols[highlightedIndex];
	}
	else if (highlightedIndex != -1 && highlightedIndex <= ((this.matchingProtocols.length + this.matchingNlHosts.length) - 1)) {
		// the highlighted suggestion is a nlhost selection
		suggestion = this.matchingNlHosts[highlightedIndex - this.matchingProtocols.length];
	}
	return suggestion;	
}


/**
 * Gets the listener function listening to when a element has been selected in the dropdown menu. 
 * @return The function to set as a listener to the dropdown menu. 
 */
TextSuggest.prototype.getDropDownSelectListener = function() {
	var textSuggest = this;
	return function() {
		textSuggest.suggestionSelected();
		textSuggest.hideSuggestions();
	};
}


// suggestions loading 


/**
 * Sends a request for suggestions. 
 */
TextSuggest.prototype.sendRequestForSuggestions = function() {
	this.lastRequestString = this.textInput.value;
	if (this.handlingRequest) {
		this.pendingRequest = true;
		return;
	}
	
	this.handlingRequest = true;
	getResponse(this.createSuggestUrl(), "suggestionsLoaded", this);
}



/**
 * Creates the request for suggestions.
 * @return The created request.  
 */
TextSuggest.prototype.createSuggestUrl = function() {
	return this.url + "ignoreCase=" + this.options.ignoreCase + "&matchAnyWhere=" + this.options.matchAnyWhere + "&entry=" + this.lastRequestString + "&";
}


/**
 * Creates the suggestions from a response from the server. 
 * @param response The response from the server. 
 */
TextSuggest.prototype.createSuggestions = function(response) {
	// this creates the two array containing the protocol and nlhost responses
	eval(response);
	this.protocolSuggestions = protocolOptions;
	this.nlHostSuggestions = nlhostOptions;
}


/**
 * Suggestions have been loaded, show them.
 * @param response The response from the server.
 */
TextSuggest.prototype.suggestionsLoaded = function(response) {
	this.createSuggestions(response);
	if (this.protocolSuggestions.length == 0 && this.nlHostSuggestions.length == 0) {
		this.hideSuggestions();
		this.handlingRequest = false;
		return;
	}
	
	this.updateDropDown();
	this.handlingRequest = false;
	
	if (this.pendingRequest) {
		this.pendingRequest = false;
		this.sendRequestForSuggestions();
	}
}


// html code handling


/**
 * Adds the behaviour needed for the suggest component to work.
 */
TextSuggest.prototype.injectSuggestedBehaviour = function() {
	this.createSuggestionsDiv();
	new TextSuggestEventHandler(this);
}


/**
 * Updates the dropdown menu with the suggestions in memory. 
 */
TextSuggest.prototype.updateDropDown = function() {
	this.dropDown.clear();
	this.addDropDownSections();
	if (this.matchingProtocols.length != 0 || this.matchingNlHosts.length != 0) {
		this.showSuggestions();
		this.dropDown.highlightElementAt(0);
	}
}


/**
 * Adds sections according to the matching protocols and hosts to the dropdown. 
 */
TextSuggest.prototype.addDropDownSections = function() {
	this.matchingProtocols = this.matchEntries(this.protocolSuggestions, this.options.maxCount);
	this.matchingNlHosts = this.matchEntries(this.nlHostSuggestions, this.options.maxCount);
	
	var protocolSection = this.createProtocolSection(this.matchingProtocols);
	var nlHostSection = this.createNlHostSection(this.matchingNlHosts);
	if (protocolSection != null) {
		this.dropDown.addSection(protocolSection);
	}
	if (nlHostSection != null) {
		this.dropDown.addSection(nlHostSection);
	}		
}


/**
 * Creates a dropdown section for the given nlhost options. 
 * @param nlHosts Array containing the nlhosts options to create the section for. 
 * @return The created nlhost section, null if nlhosts is null or empty.  
 */
TextSuggest.prototype.createNlHostSection = function(nlHosts) {
	var elements = new Array();
	var regExp = this.createRegExp(this.textInput.value);
	for (var i = 0; exists(nlHosts) && i < nlHosts.length; i++) {
		elements.push(this.createSuggestionHtmlCode(nlHosts[i], i + (this.matchingProtocols.length), regExp, false));
	}
	
	if (elements.length == 0) {
		return;
	}
	
	var section = new DropDownSection(elements, this.dropDown);
	if (this.options.showNlHostHeader) {
		section.setHeader(this.createNlHostHeader());
	}
	return section;
}


/**
 * Creates a dropdown section for the given protocol options. 
 * @param protocols Array containing the protocol options to create the section for.
 * @return The created protocol section, null if protocols is null or empty. 
 */
TextSuggest.prototype.createProtocolSection = function(protocols) {
	var elements = new Array();
	var regExp = this.createRegExp(this.textInput.value);
	for (var i = 0; exists(protocols) && i < protocols.length; i++) {
		elements.push(this.createSuggestionHtmlCode(protocols[i], i, regExp, true));
	}
	
	if (elements.length == 0) {
		return;
	}
	
	var section = new DropDownSection(elements, this.dropDown);
	if (this.options.showProtocolHeader) {
		section.setHeader(this.createProtocolHeader());		
	}
	return section;
}


/**
 * Creates the html code for a suggestion.
 * @param suggestion The suggestion to create the span for. 
 * @param n The index into the visible suggestions for the suggestion to create the span for. 
 * @param regExp The regular expression to use to match the suggestions text with the text in the input field. 
 * @param protocol True if this is a protocol suggestion, false for nlhost.
 * @return The created span tag. 
 */
TextSuggest.prototype.createSuggestionHtmlCode = function(suggestion, n, regExp, protocol) {
	var suggestionSpan = this.createSuggestionContainerCode(n);
	var textTag = this.createSuggestionTextCode(suggestion, regExp);
	
	var suggestTable = document.createElement("table");
	suggestTable.style.width = "100%";
	var row = suggestTable.insertRow(0);
	var cell = row.insertCell(0);
	var image = document.createElement("div");
	if (this.options.showSuggestionImages && protocol) {
		setClass(image, "protocolSuggestImage");
	}
	else if (this.options.showSuggestionImages) {
		setClass(image, "nlHostSuggestImage");
	}
	cell.appendChild(image);
	
	cell = row.insertCell(1);
	cell.style.width = "80%";
	cell.style.overflow = "hidden";
	cell.setAttribute("name", "textCell");
	cell.appendChild(textTag);
	
	cell = row.insertCell(2);
	cell.style.width = "20%";
	cell.align = "right";
	cell.setAttribute("name", "valueCell");
	cell.appendChild(document.createTextNode(formatOctetValue(suggestion[1])));
	
	suggestionSpan.appendChild(suggestTable);
	return suggestionSpan;
}


/**
 * Each suggestion contains of a text and a value. This creates the html code for 
 * the text part of a suggestions. 
 * @param suggestion The suggestion to create the span for. 
 * @param regExp The regular expression to use to match the suggestions text with the text in the input field. 
 * @return The created html code.
 */
TextSuggest.prototype.createSuggestionTextCode = function(suggestion, regExp) {
	var textTag = document.createElement("div");
	textTag.style.width = "80%";
	textTag.style.overflow = "hidden";
	// create and append the text
	var textValues = this.splitTextValues(URLDecode(suggestion[0]), this.textInput.value.length, regExp)
	var textMatchSpan = document.createElement("span");
	setClass(textMatchSpan, this.options.matchClassName);
	textMatchSpan.appendChild(document.createTextNode(textValues.mid));
	textTag.appendChild(document.createTextNode(textValues.start));
	textTag.appendChild(textMatchSpan);
	textTag.appendChild(document.createTextNode(textValues.end));
	return textTag;
}


/**
 * Creates the html code for the nlhost header.
 * @return The create html code. 
 */
TextSuggest.prototype.createNlHostHeader = function() {
	var nlHostHeader = document.createElement("span");
	nlHostHeader.style.width = "100%";
	nlHostHeader.style.display = "block";
	nlHostHeader.id = "nlHostHeader";
	nlHostHeader.appendChild(document.createTextNode("Hosts"));
	return nlHostHeader;
}


/**
 * Creates the html code for the protocol header.
 * @return The created html code.  
 */
TextSuggest.prototype.createProtocolHeader = function() {
	var protocolHeader = document.createElement("span");
	protocolHeader.style.width = "100%";
	protocolHeader.style.display = "block";
	protocolHeader.id = "protocolHeader";
	protocolHeader.appendChild(document.createTextNode("Protocols"));
	return protocolHeader;
}


/**
 * Creates the html container for a suggestion.
 * @param n The index of this suggestions.
 * @return The created html code. 
 */
TextSuggest.prototype.createSuggestionContainerCode = function(n) {
	var suggestionSpan = document.createElement("span");
	suggestionSpan.style.width = "100%";
	suggestionSpan.style.display = "block";
	suggestionSpan.id = this.id + "_" + n;
	var thisObj = this;
	return suggestionSpan;
}


/**
 * Adds the div tag where the suggestions are shown to the document.
 */
TextSuggest.prototype.createSuggestionsDiv = function() {
	this.htmlCode = document.createElement("div");
	setClass(this.htmlCode, "inactive");
	this.htmlCode.style.position = "absolute";
	// set a high zIndex to make sure the suggestions are shown above any content in the page
	this.htmlCode.style.zIndex = 999999;
	document.body.appendChild(this.htmlCode);
	this.htmlCode.style.left = parseInt(this.options.xPos) + "px";
	this.htmlCode.style.top = parseInt(this.options.yPos) + "px";
}


/**
 * Splits given text values.
 * @param text The text to split.
 * @param length The length of the value to match. 
 * @param regExp The regular expression to use.
 * @return The splitted text with start, mid and end properties set.  
 */
TextSuggest.prototype.splitTextValues = function(text, length, regExp) {
	var startPos = text.search(regExp);
	var matchText = text.substring(startPos, startPos + length);
	var startText = startPos == 0 ? "" : text.substring(0, startPos);
	var endText = text.substring(startPos + length);
	return {start: startText, mid: matchText, end: endText};
}


/**
 * Matches suggested entries according to the value in the input field. 
 * @param entries The entries to match. 
 * @param maxCount If this is parameter is provided this stops matching entries after max count
 * entries have been matched. 
 * @return The entries matching the current value in the input field.  
 */
TextSuggest.prototype.matchEntries = function(entries, maxCount) {
	var matchArray = new Array();
	var regExp = this.createRegExp(this.textInput.value);
	for (var i = 0; i < entries.length; i++) {
		var matches = URLDecode(entries[i][0]).match(regExp);
		if (matches) {
			matchArray.push(entries[i]);
		}
		if (exists(maxCount) && isNumber(maxCount) && matchArray.length == maxCount) {
			return matchArray;
		}				
	}
	return matchArray;
}


/**
 * Creates the regular expression used to match values. 
 * @param value The value to match by this expression.
 * @return The expression to match entries. 
 */
TextSuggest.prototype.createRegExp = function(value) {
	// to determine whether to ignore case while matching using regular expressions
	var regExpFlags = this.options.ignoreCase ? "i" : "";
	// to determine whether to match at the beginning of a string or anywhere 
	var startRegExp = this.options.matchAnyWhere ? "" : "^";
	// a . is  special character in regular expressions that stands for all characters
	// escape them with a \ (since the value is a string and string literals
	// and regular expressions use the \ character for escape sequences we have
	// to use \ characters in the final string for the Regular expression constructor,
	value = value.replace("\.", "\\.");
	return new RegExp(startRegExp  + value, regExpFlags);
}


/**
 * Checks whether we need to load more data. 
 * @return True if we need to load more data, false otherwise. 
 */
TextSuggest.prototype.needToLoadData = function() {
	if (this.matchingProtocols.length == 0 && this.protocolSuggestions.length != 0) {
		// no more matching protocols and last time data was loaded there was still some protocols returned
		// no need to load more data if last time did not return any protocols either
		return true;
	}
	else if (this.matchingNlHosts.length == 0 && this.nlHostSuggestions.length != 0) {
		// no more matching hosts and last time data was loaded there were still some hosts returned
		// no need to load more data if last time did not return any hosts either
		return true;
	}
	else if (this.matchingProtocols.length == 0 || this.matchingNlHosts.length == 0) {
		// send a request for more data if we have neither protocols nor hosts
		return true;
	}
	return false;
}


// event handling


/**
 * Handles and processes text input
 */
TextSuggest.prototype.handleTextInput = function() {
	if (exists(this.selectedSuggestion)) {
		// data is loaded asynchronously so the server can return data after the user already
		// has selected a suggestion, if so do not shown the loaded suggestions
		return;
	}
	if (this.textInput.value == "") {
		this.clear();
		return;
	}
	else if (this.textInput.value.indexOf(this.lastRequestString) == -1 || isEmpty(this.lastRequestString)) {
		// search text has been made smaller than the string used for the current data (from ip.http
		// ip.ht for example) need to load new data since there now might be more matching suggestions
		this.sendRequestForSuggestions();
		return;
	}
	
	this.updateDropDown();
	if (this.needToLoadData()) {
		this.sendRequestForSuggestions();
	}
}


/**
 * The controller. Handles key events and onblur events for the suggest component. 
 * @param textSuggest A reference to the text suggest component to handle events for.
 * @param throws Error if textSuggest is undefined or null. 
 */
function TextSuggestEventHandler(textSuggest) {
	assertExists(textSuggest, "The text suggestion to handle key events for is undefined or null");
	
	this.textSuggest = textSuggest;
	this.input = this.textSuggest.textInput;
	this.addKeyHandling();
}


/**
 * Adds the key handling event handlers.  
 */
TextSuggestEventHandler.prototype.addKeyHandling = function() {
	registerEventHandler("keyup", this.keyUpListenerFunction(), this.input);
	registerEventHandler("blur", this.onblurListenerFunction(), this.input);
}


/**
 * Returns the function to set as a listener for onblur events. 
 */
TextSuggestEventHandler.prototype.onblurListenerFunction = function() {
	var eventHandler = this;
	return function (event) {
		eventHandler.onblurHandler(event);
	};
}


/**
 * Returns the function to set as a listener for keyup events. 
 */
TextSuggestEventHandler.prototype.keyUpListenerFunction = function() {
	var eventHandler = this;
	return function (event) {
		eventHandler.keyUpHandler(event);
	};
}

 
/**
 * Handles key up event. 
 * @param event The event to process. 
 */
TextSuggestEventHandler.prototype.keyUpHandler = function(event) {
	event = exists(event) ? event : window.event;
	this.textSuggest.selectedSuggestion = null;
	if (!this.isFucntionKeyPressed(event)) {
		this.textSuggest.handleTextInput();
	}
}


/**
 * Returns tur if a key event is for one of the function keys, false otherwise. 
 * Function keys like enter, escape, up and down arrows are handled by the dropdown menu. 
 * @param event The event to test. 
 */
TextSuggestEventHandler.prototype.isFucntionKeyPressed = function(event) {
	var enterKey = 13;
	var upArrow = 38;
	var downArrow = 40;
	var escapeKey = 27;
	if (event.keyCode == upArrow || event.keyCode == downArrow) {
		return true;
	}
	if (event.keyCode == enterKey || event.keyCode == escapeKey) {
		return true;
	}
	return false;
}


/**
 * Handles the onblur event. 
 * @param event The event to process.
 */
TextSuggestEventHandler.prototype.onblurHandler = function(event) {
	if ("inactive" != getClassName(this.textSuggest.htmlCode)) {
		this.textSuggest.suggestionSelected();
	}
	this.textSuggest.hideSuggestions();
}
