/**
 * Makes sure some text in a input field is valid. If not a message is shown.
 */


/**
 * The event fired when a error message has been set. 
 */
var INPUT_ERROR_MESSAGE_SET_EVENT = "ipErrorMessageSet";


/**
 * The event fired when the error message has been cleared. 
 */
var INPUT_ERROR_MESSAGE_CLEARED_EVENT = "ipErrorMessageCleared";

 
// construction


/**
 * Constructs the instance of this class.
 * @param inputField The input field where the user types the port.
 * @param messageField The field where to show messages in.
 * @throws Error if inputField is null or empty.
 * @throws Erorr if message is null or empty.
 */
function InputFieldValidator(inputField, messageField) {
	assertExists(inputField, "The input field to validate does not exist");
	assertExists(messageField, "The message field doees not exist");
	
	this.inputField = inputField;
	this.messageField = messageField;
	this.addListeners();
	
	// the function to call when enter has been pressed in the input field
	this.enterFunction;
	
	// the message to throw when the input is invalid
	this.errorMessage;
	
	// the function to call to validate the input
	this.validationFunction;
	
	// the function to call on the onkeyup event
	this.onKeyUpFunction;
	
	// the function to call on the onblur event
	this.onblurFunction;
	
	// the function to call on the onfocus event 
	this.onFocusFunction;
	
	// the original class of the input field
	this.originalClass;
	
	// set to true when the input field has been marked as having a error
	this.marked = false;
} 


// services


/**
 * Set the function to call when enter is pressed in the input field.
 * @param enterFunction The function to call when enter has been pressed. 
 */
InputFieldValidator.prototype.setEnterFunction = function(enterFunction) {
	this.enterFunction = enterFunction;
}


/**
 * Set the function to use to validate the input. 
 * @param validationFunction The function to use.
 * @throws Error if validationFunction is undefined or null.
 */
InputFieldValidator.prototype.setValidationFunction = function(validationFunction) {
	assertExists(validationFunction, "The validation function does not exist");
	this.validationFunction = validationFunction;
}


/**
 * Set the function to call on the onkeyup event. The function is called before any validation
 * of the input is done. 
 * @param keyUpFunction The on key up function to set. 
 */
InputFieldValidator.prototype.setOnKeyUpFunction = function(keyUpFunction) {
	this.onKeyUpFunction = keyUpFunction;
}


/**
 * Set the onblur function to call on the onblur event. The function is
 * called before any validation of the input is done.  
 * @param onBlurFunction The onblur function to set. 
 */
InputFieldValidator.prototype.setOnBlurFunction = function(onBlurFunction) {
	this.onBlurFunction = onBlurFunction;
}


/**
 * Set the onfocus function to call on the on focus event. The function is called
 * before any validation of the input is done. 
 * @param onFocusFunction The on focus function to set.
 */
InputFieldValidator.prototype.setOnFocusFunction = function(onFocusFunction) {
	this.onFocusFunction = onFocusFunction;
}


/**
 * Sets the error message to set on invalid input. 
 * @param message The message to set.
 * @throws Error if message is undefined, null or empty.
 */
InputFieldValidator.prototype.setMessage = function(message) {
	assertNotEmpty(message, "The error message is undefined, null or empty");
	this.errorMessage = message;
}


// event handling


/**
 * Handles the keyup events generated when the user types in something in the input field.
 * @param event The onkeyup event to process. 
 */
InputFieldValidator.prototype.keyUpHandler = function(event) {
	event = exists(event) ? event : window.event;
	if (!exists(this.validationFunction) || !exists(this.errorMessage)) {
		return;
	} 	
	if (!isEmpty(this.inputField.value) && !this.validationFunction(this.inputField.value)) {
		this.setErrorMessage(this.errorMessage);
		this.markError();
	}
	else {
		this.clearMessage();
		this.markValid();
	}
	if (exists(event) && event.keyCode == 13) { 
		this.enterPressed();
	}
}


/**
 * The input field has lost focus, validate the input.
 */
InputFieldValidator.prototype.onBlurHandler = function() {
	if (!exists(this.validationFunction) || !exists(this.errorMessage)) {
		return;
	} 	
	if (isEmpty(this.inputField.value) || !this.validationFunction(this.inputField.value)) {
		this.setErrorMessage(this.errorMessage);
		this.markError();
	}
}


/**
 * The input fiels has been focused, validate the input.
 */
InputFieldValidator.prototype.onFocusHandler = function() {
	if (!exists(this.validationFunction) || !exists(this.errorMessage)) {
		return;
	} 	
	if (!isEmpty(this.inputField.value) && !this.validationFunction(this.inputField.value)) {
		this.setErrorMessage(this.errorMessage);
		this.markError();
	}
	else {
		this.clearMessage();
		this.markValid();
	}
}

 
// implementation


/**
 * Marks the input field as having invalid input. 
 */
InputFieldValidator.prototype.markError = function() {
	if (this.marked) {
		// the input field is already marked
		return;
	}
	
	this.originalClass = getClassName(this.inputField);
	if (!exists(this.originalClass) || isEmpty(this.originalClass)) {
		setClass(this.inputField, "error");
	}
	else {
		setClass(this.inputField, this.originalClass + " error");
	}
	this.marked = true;
}


/**
 * Marks the input field as having valid input. 
 */
InputFieldValidator.prototype.markValid = function() {
	if (!this.marked) {
		return;
	}
	setClass(this.inputField, exists(this.originalClass) ? this.originalClass : "");
	this.marked = false;
}


/**
 * Enter has been pressed in the input field, call the enter function if any is set. 
 */
InputFieldValidator.prototype.enterPressed = function() {
	if (exists(this.enterFunction)) {
		this.enterFunction();
	}
}


/**
 * Sets a error message.
 * @param message The message to set. 
 */
InputFieldValidator.prototype.setErrorMessage = function(message) {
	setClass(this.messageField, "messageContainer error");
	this.messageField.innerHTML = message;
	getEventServer().notifyListeners(new Event(INPUT_ERROR_MESSAGE_SET_EVENT, this));
}


/**
 * Clears the message field.
 */
InputFieldValidator.prototype.clearMessage = function() {
	this.messageField.innerHTML = "";
	getEventServer().notifyListeners(new Event(INPUT_ERROR_MESSAGE_CLEARED_EVENT, this));
}


/**
 * Adds the listeners to the ip field. 
 */
InputFieldValidator.prototype.addListeners = function() {
	var thisObj = this;
	this.inputField.onkeyup = function(event) {
		if (exists(thisObj.onKeyUpFunction)) {
			thisObj.onKeyUpfunction();
		}
		thisObj.keyUpHandler(event);
	}
	
	this.inputField.onblur = function(event) {
		if (exists(thisObj.onBlurFunction)) {
			thisObj.onBlurFunction();
		}
		thisObj.onBlurHandler(event);
	}		
	
	this.inputField.onfocus = function(event) {
		if (exists(thisObj.onFocusFunction)) {
			thisObj.onFocusFunction();
		}
		thisObj.onFocusHandler(event);
	}
}