/**
 * This class makes sure input the user types in a given input field is a
 * valid mac address. If the user types in a invalid mac address a error message
 * is shown in a given field.
 */
 

/**
 * The message set when a mac address with invalid format has been entered. 
 */
var INVALID_MAC_ADDRESS_MESSAGE = "Invalid mac address set. Valid format is xx:xx:xx:xx:xx:xx";


/**
 * The message set when the mac address contains invalid numbers.
 */
var INVALID_MAC_ADDRESS_NUMBER_MESSAGE = "Invalid number in mac address. Valid is 0-99 and a-f";

 
// construction


/**
 * Factory method to create a mac address input field validator. 
 * @param macField The mac address fields whose value to validate. 
 * @param messageField The message field where error messages are to be shown.
 * @throws Error if macField is null or undefined. 
 * @throws Error if messageField is null or undefined. 
 */
function createMacFieldValidator(macField, messageField) {
	assertExists(macField, "The mac address field to validate is undefined or null");
	assertExists(messageField, "The message field is null or empty");
	
	var subInstance = new MacFieldValidator();
	var superInstance = new InputFieldValidator(macField, messageField);
	subInstance = createSubClass(superInstance, subInstance);
	subInstance.addListeners();
	return subInstance; 
}


/**
 * Constructs a instance of this class. 
 */
function MacFieldValidator() {
	
}


// event handling


/**
 * Handles the keyup events generated when the user types in the mac address. 
 * @param event The onkeyup event to process. 
 */
MacFieldValidator.prototype.keyUpHandler = function(event) {
	event = exists(event) ? event : window.event;	
	if (!isEmpty(this.inputField.value) && !validMacNumbers(this.inputField.value)) {
		this.setErrorMessage(INVALID_MAC_ADDRESS_NUMBER_MESSAGE);
		this.markError();
		return;
	}
	if (this.countMacNumbers() > 6) {
		this.setErrorMessage(INVALID_MAC_ADDRESS_MESSAGE);
		this.markError();
		return;
	}
	
	this.clearMessage();
	this.markValid();
	if (exists(event) && event.keyCode == 13) { 
		this.enterPressed();
	}
}


/**
 * The slave mac field has lost focus, check the mac address is set correct.
 */
MacFieldValidator.prototype.onBlurHandler = function() {
	if (!isEmpty(this.inputField.value) && !validMacNumbers(this.inputField.value)) {
		this.setErrorMessage(INVALID_MAC_ADDRESS_NUMBER_MESSAGE);
		this.markError();
	}
	else if (isEmpty(this.inputField.value) || !isValidMac(this.inputField.value)) {
		this.setErrorMessage(INVALID_MAC_ADDRESS_MESSAGE);
		this.markError();
	}
}


/**
 * The mac field has been focused, validate the input and check and set any error messages.
 */
MacFieldValidator.prototype.onFocusHandler = function() {
	if (!isEmpty(this.inputField.value) && !validMacNumbers(this.inputField.value)) {
		this.setErrorMessage(INVALID_MAC_ADDRESS_NUMBER_MESSAGE);
		this.markError();
	}
	else if (!isEmpty(this.inputField.value) && !isValidMac(this.inputField.value)) {
		this.setErrorMessage(INVALID_MAC_ADDRESS_MESSAGE);
		this.markError();
	}
	else {
		this.clearMessage();
		this.markValid();
	}
}	
 
 
// implementation


/**
 * Counts the number of numbers in the mac field (00:01 will return 2).
 * @return The count of numbers in the ip field (00:01:02 returns 3).
 */
MacFieldValidator.prototype.countMacNumbers = function() {
	if (isEmpty(this.inputField.value)) {
		return 0;
	}
	var tokenizer = new StringTokenizer(this.inputField.value, ":");
	return tokenizer.countTokens();
}