/**
 * This class makes sure the input in a password field is valid. If not 
 * a error message is shown. 
 */


// construction


/**
 * Factory method to create a password field validator. 
 * @param passwordField The field to validate. 
 * @param messageField The field to show any messages in. 
 * @param legalCharacters A string containing the characters that should be considered
 * as legal in this password field. 
 * @param errorMessage The error message to throw when the password entered is invalid. 
 * @return The password field validator created. 
 * @throws Error if passwordField is null or undefined.
 * @throws Error if messageField is null or undefined. 
 * @throws Error if legalCharacters is null, undefined or empty.
 * @throws Error if errorMessage is null, undefined or empty.  
 */
function createPasswordFieldValidator(passwordField, messageField, legalCharacters, errorMessage) {
	assertExists(passwordField, "The password field to validate is undefined or null");
	assertExists(messageField, "The message field is undefined or null");
	assertNotEmpty(legalCharacters, "The valid characters to use for this password validator are undefined, null or empty");
	assertNotEmpty(errorMessage, "The error message is undefined, null or empty");
	
	var subInstance = new PasswordFieldValidator(legalCharacters, errorMessage);
	var superInstance = new InputFieldValidator(passwordField, messageField);
	subInstance = createSubClass(superInstance, subInstance);
	subInstance.addListeners();
	return subInstance;
}


/**
 * Constructs a password field validator. 
 * @param legalCharacters The valid characters to use for this password field.
 * @param errorMessage The error message to throw when the password entered is invalid.
 * @throws Error if legalCharacters are undefined, null or empty.
 * @throws Error if errorMessage is null, undefined or empty.  
 */
function PasswordFieldValidator(legalCharacters, errorMessage) {
	assertNotEmpty(legalCharacters, "The valid characters to use for this password validator are undefined, null or empty");
	assertNotEmpty(errorMessage, "The error message is undefined, null or empty");
	
	this.legalCharacters = legalCharacters;
	this.errorMessage = errorMessage; 
}
 
 
// event handling


/**
 * Handles the keyup events generated when the user types in the ip address. 
 * @param event The onkeyup event to process. 
 */
PasswordFieldValidator.prototype.keyUpHandler = function(event) {
	event = exists(event) ? event : window.event;
	if (!isValidPassword(this.inputField.value, this.legalCharacters)) {
		this.setErrorMessage(this.errorMessage);
		this.markError();
		return;
	}
	else {
		this.clearMessage();
		this.markValid();
	}
	if (exists(event) && event.keyCode == 13) { 
		this.enterPressed();
	}
}



/**
 * The password field has lost focus, check the password field is not empty.
 */
PasswordFieldValidator.prototype.onBlurHandler = function() {
	if (isEmpty(this.inputField.value) || !isValidPassword(this.inputField.value, this.legalCharacters)) {
		this.setErrorMessage(this.errorMessage);	
		this.markError();
	}
}


/**
 * The password field has been focused, validate the input and set any error messages. 
 */
PasswordFieldValidator.prototype.onFocusHandler = function() {
	if (!isEmpty(this.inputField.value) && !isValidPassword(this.inputField.value, this.legalCharacters)) {
		this.setErrorMessage(this.errorMessage);	
		this.markError();
	}
	else {
		this.clearMessage();
		this.markValid();
	}
}