/**
 * Provides utility functions
 */


// constants


/**
 * One Kilo Byte. Used when returning KB/MB/GB labels.
 */
var KILO_BYTE = 1024;
	
	
/**
 * One Mega Byte. Used when returning KB/MB/GB labels.
 */
var MEGA_BYTE = 1024 * KILO_BYTE;

	
/**
 * One Giga Byte. Used when returning KB/MB/GB labels.
 */
var GIGA_BYTE = 1024 * MEGA_BYTE;


/**
 * The number of decimals used for octet counts in the table.
 */
var OCTET_DECIMAL_COUNT = 1;


/**
 * This label is used for bits per second.
 */
var BITS = "bit/s";


/**
 * This label is used for kilobits per second.
 */
var KBITS = "Kbit/s";


/**
 * This label is used for megabits per second.
 */
var MBITS = "Mbit/s";


/**
 * This label is used for gigabits per second.
 */
var GBITS = "Gbit/s";


/**
 * The class to set to hide the log selector frame.
 */
var LOG_SELECTOR_FRAME_INACTIVE_CLASS = "inactiveLogSelector";


// services


/**
 * Returns the session id.
 */
function getSid() {
	var sid = getElementWithAttribute(top.document.body, "sid", "name", "input");
	if (isDefined(sid) && !isNull(sid)) {
		return sid.value;
	}
	return "";
}
 

/**
 * Returns the ip address of the server. This is used in the summary line and gauge chart. 
 */
function getServerAddress() {
	var addressField = getElementWithAttribute(document.body, "localAddress", "name", "input");
	if (isDefined(addressField) && !isNull(addressField)) {
		return addressField.value;
	}
	return "";
}


/**
 * Checks if the probe is run in master mode. 
 * @return True if the probe is run in master mode, false otherwise.
 */
function getMasterMode() {
	var masterMode = getElementWithAttribute(top.document.body, "masterMode", "name", "input");
	if (isDefined(masterMode) && !isNull(masterMode) && "true" == masterMode.value) {
		return true;
	}
	return false;
}

/**
 * Returns the probe name set, a empty string if no probe name set.  
 */
function getProbeName() {
	var probeNameField = getElementWithAttribute(document.body, "probeName", "name", "input");
	if (exists(probeNameField)) {
		return probeNameField.value; 
	}
	return "";
}


/**
 * Adds KB, MB, and GB to a octet value.
 * @param value The octet value to format.
 * @return The formatted octet value.
 */
function formatOctetValue(value) {
	if (value < KILO_BYTE) {
		return value;
	}
	else if (value < MEGA_BYTE) {
		value = roundNumber((value / KILO_BYTE), OCTET_DECIMAL_COUNT) + " KB";
	}
	else if (value < GIGA_BYTE) {
		value = roundNumber((value / MEGA_BYTE), OCTET_DECIMAL_COUNT) + " MB";
	}
	else {
		value = roundNumber((value / GIGA_BYTE), OCTET_DECIMAL_COUNT) + " GB";
	}

	return value;
}



/**
 * Changes a byte value from one unit into another. 
 * @param value The value to change.
 * @param originalUnit The unit value is in (BITS, KBITS, MBITS, GBITS).
 * @param newUnit The unit to change the value into (BITS, KBITS, MBITS, GBITS). 
 * @throws Erorr if value is not a number or negative.
 * @throws Error if originalUnit is undefined, null or empty.
 * @throws Error if newUnit is undefined, null or empty.
 */
function changeThroughputValueUnit(value, originalUnit, newUnit) {
	assertIsNumber(value, "The value to change the unit for is not a number");
	assertNotEmpty(originalUnit, "The original unit is undefined null or empty");
	assertNotEmpty(newUnit, "The new unit is undefined null or empty");
	var newValue;
	if (BITS == originalUnit && KBITS == newUnit) {
		newValue = value / 1000;
	}
	else if (BITS == originalUnit && MBITS == newUnit) {
		newValue = value / (1000 * 1000);
	}
	else if (BITS == originalUnit && GBITS == newUnit) {
		newValue = value / (1000 * 1000 * 1000);
	}
	else if (KBITS == originalUnit && MBITS == newUnit) {
		newValue = value / 1000;
	}
	else if (KBITS == originalUnit && GBITS == newUnit) {
		newValue = value / (1000 * 1000);
	}
	else if (MBITS == originalUnit && GBITS == newUnit) {
		newValue = value / 1000;
	}
	else if (GBITS == originalUnit && MBITS == newUnit) {
		newValue = value * 1000;
	}
	else if (GBITS == originalUnit && KBITS == newUnit) {
		newValue = value * (1000 * 1000);
	}
	else if (GBITS == originalUnit && BITS == newUnit) {
		newValue = value * (1000 * 1000 * 1000);
	}
	else if (MBITS == originalUnit && KBITS == newUnit) {
		newValue = value * 1000;
	}
	else if (MBITS == originalUnit && BITS == newUnit) {
		newValue = value * (1000 * 1000);
	}
	else if (KBITS == originalUnit && BITS == newUnit) {
		newValue = value * 1000;
	}
	else {
		return value;
	}
	
	if (newValue.toString().indexOf("e") > -1) {
		// number is of format 1e+7 or 1e-7, get the number as 10000000 or -1000000
		// this only works if the number is not larger than 1e+21 or 
		// smaller than 1e-21
		var stringValue = newValue.toString();
		var startIndex = stringValue.indexOf("e");
		var exponent = stringValue.substring(startIndex, stringValue.length);
		// skip the e+ or e-
		exponent = exponent.substring(2);
		newValue = newValue.toFixed(parseInt(exponent));	
	}
	return newValue;
}


/**
 * Returns the version number set in the index.html file, a empty string if not set. 
 */
function getVersion() {
	var version = getElementWithAttribute(top.document.body, "probeVersion", "name", "input");
	if (exists(version)) {
		return version.value;	
	}
	return "";
}


/**
 * Gets the formatted version number. A version number 2.5.1 is formatted to be 251, a empty
 * string if no version set. 
 */
function getFormattedVersion() {
	var version = getVersion();
	if (isEmpty(version)) {
		return "";
	}
	
	var tokenizer = new StringTokenizer(version, ".");
	var formatted = "";
	while(true) {
		if (!tokenizer.hasMoreTokens()) {
			break;
		}
		formatted += tokenizer.nextToken();	
	}
	return formatted;
}


/**
 * Gets the host id.
 */
function getHostId() {
	var hostid = getElementWithAttribute(top.document.body, "hostid", "name", "input");
	if (exists(hostid)) {
		return hostid.value;	
	}
	return "";
}


/**
 * Checks if a given yPosition is in the desktop regarding the window height and the current scrollheight.
 * @param yPos The y position to check.
 * @return True if yPos is in the screen, false otherwise.
 */
function isYpositionInDesktop(yPos) {
	var screenHeight = Desktop.instance().getDesktopCode().offsetHeight;
	var scrollHeight = Desktop.instance().getDesktopCode().scrollHeight;
	if (scrollHeight > screenHeight) {
		return scrollHeight > yPos;
	}
	else {
		return screenHeight > yPos;
	}
}


/**
 * Checks if a given xPosition is in the desktop regarding the window width and the current scrollwidth.
 * @param xPos The x position to check.
 * @return True if xPos is in the screen, false otherwise. 
 */
function isXPositionInDesktop(xPos) {
	var screenWidth = Desktop.instance().getDesktopCode().offsetWidth;
	var scrollWidth = Desktop.instance().getDesktopCode().scrollWidth;
	if (scrollWidth > screenWidth) {
		return scrollWidth > xPos;
	}
	else {
		return screenWidth > xPos;
	}
}



/**
 * Calculates the max y position that can be set for a element so that the element is not placed outside
 * the current scrolling of the desktop. 
 * @return The max y value that can be set. 
 */
function calculateMaxYPos() {
	var screenHeight = Desktop.instance().getDesktopCode().offsetHeight;
	var scrollHeight = Desktop.instance().getDesktopCode().scrollHeight;
	if (scrollHeight > screenHeight) {
		return scrollHeight; 
	}
	return screenHeight;
}


/**
 * Calculates the max x position that can be set for a element so that the element is not placed outside
 * the current scrolling of the desktop.
 * @return The max x value that can be set. 
 */
function calculateMaxXPos() {
	var screenWidth = Desktop.instance().getDesktopCode().offsetWidth;
	var scrollWidth = Desktop.instance().getDesktopCode().scrollWidth;
	if (scrollWidth > screenWidth) {
		return scrollWidth;
	}
	return screenWidth;
}


/**
 * Checks if a password is valid.
 * @param password The password to check.
 * @param legal The characters that are legal in the given password. 
 * @return True if the password is valid, false otherwise.  
 */
function isValidPassword(password, legal) {
	if (isEmpty(legal)) {
		legal = "abcdefghijklmnopqrstuvwxyz1234567890 ";
	}
	password = password.toLowerCase();
	for (i = 0; i < password.length; i++) {
		if (legal.indexOf(password.charAt(i)) == -1) {
			return false;
		}
	}
	return true;
}


/**
 * Hides the log selector iframe. When the previous activated tab was the log tab 
 * the log selector frame needs to be hidden before the size for the summary frame
 * can be calculated.
 */
function hideLogSelector() {
	var logSelectorFrame = getElementWithAttribute(top.document.body, "LogSelector", "name", "iframe");
	setClass(logSelectorFrame, LOG_SELECTOR_FRAME_INACTIVE_CLASS);
}



/**
 * Set the frame size to fill the whole window.
 * @param name The name of the frame. 
 */
function setFrameSize(name) {
	var parentWindow = window.parent;
	// this is a reference to the iframes html code in index.html
	var iframeCode = getElementWithAttribute(parentWindow.document.body, name, "name", "iframe");
	if (isNull(iframeCode)) {
		return;
	}
	var frameTopOffset = calculateTopOffset(iframeCode);
	
	var parentWindowHeight = getWindowHeight(top);
	var frameHeight = parentWindowHeight - frameTopOffset;
	// if main is not the active tab any more do not change the size of the tab
	if (top.tabSection.isActive(name) && loadingScreenRemoved && frameHeight != parseInt(iframeCode.style.height)) {
		iframeCode.style.height = frameHeight + "px";
	}
}



/**
 * Gets the status text to show for a slave in a html table. 
 * @param status The status value to get the text for. 
 * @return The status text for the given status values.  
 */
function slaveStatusText(status) {
	if (isEmpty(status)) {
		return "";
	}
	if (status.equalsIgnoreCase(SLAVE_NO_CONTACT_STATUS)) {
		return "No contact";
	}
	else if (status.equalsIgnoreCase(SLAVE_PROBE_RUNNING_STATUS)) {
		return "Running";
	}
	else if (status.equalsIgnoreCase(SLAVE_PROBE_STOPPED_STATUS)) {
		return "Stopped";
	}
	else if (status.equalsIgnoreCase(SLAVE_SIMULATION_DONE_STATUS)) {
		return "Simulation done";
	}
	else if (status.equalsIgnoreCase(INVALID_SLAVE_PASSWORD_LOGIN)) {
		return "Invalid password or login set";
	}
	return "";
}


/**
 * Gets the css class to set for a slaves status text field in the html code given a status. 
 * @param status The status value to get the class for. 
 * @return The class to set for the slaves status text field. 
 */
function slaveStatusTextFieldClass(status) {
	if (isEmpty(status)) {
		return "statusNoContact";
	}
	if (status.equalsIgnoreCase(SLAVE_NO_CONTACT_STATUS)) {
		return "statusNoContact";
	}
	else if (status.equalsIgnoreCase(SLAVE_PROBE_RUNNING_STATUS)) {
		return "statusRunning";
	}
	else if (status.equalsIgnoreCase(SLAVE_PROBE_STOPPED_STATUS)) {
		return "statusStopped";
	}
	else if (status.equalsIgnoreCase(SLAVE_SIMULATION_DONE_STATUS)) {
		return "statusSimulationDone";
	}
	else if (status.equalsIgnoreCase(INVALID_SLAVE_PASSWORD_LOGIN)) {
		return "statusInvalidPasswordLogin";
	}
	return "";
}


/**
 * Change the number of top entries to show.
 * @param e The key event, used for checking if the enter key is pressed.
 * @param table The id of the table to update.
 * @param windowId The id of the window the table is in.
 * @param topNInputName Optional argument, the name of the topn input field, if not provided a default value is used. 
 */
function changeTopN(e, tableId, windowId, topNInputName) {
	var tableWindow = WindowManager.instance().getWindow(windowId);
	var table = TableManager.instance().getTable(tableId);
	
	// get the new n
	var topNInput = getElementWithAttribute(tableWindow.getHtmlCode(), exists(topNInputName) ? topNInputName : "topN", "name", "input");
	var n = topNInput.value;
	if (n <= 0) {
		// no data to load, insert the current topn 
		topNInput.value = table.getN();
		return;
	}
	if (isNaN(parseInt(n))) {
		// n is not a number, insert the current topn 
		topNInput.value = table.getN();
		return;
	}
	
	// fire a event that the topn has changed
	var eventData = new Array();
	eventData["id"] = tableId;
	eventData["n"] = n;
	eventData["oldN"] = table.getN();
	getEventServer().notifyListeners(new Event("topNChanged", eventData));
}