/**
 * This class represents a slider. The slider lets the user slide a button from 0 to a given 
 * max size. 
 */
 

// constants


/**
 * The default class of the slider container.
 */
var DEFAULT_SLIDER_CONTAINER_CLASS = "sliderContainer";


// construction


/**
 * Constructs a slider.
 * @param maxValue The max value of the slider.
 * @throws Error if maxValue is negative or not a number.  
 */
function Slider(maxValue) {
	assertIsNumber(maxValue, "The max value to set is not a number");
	if (maxValue < 0) {
		throw new Error("The max value to set is negative");
	}
	this.maxValue = maxValue;
	
	
	/**
	 * The html code of the slider. 
	 */
	this.htmlCode;
	
	/**
	 * The parent element to add the sliders html code to. If this is set before the html code is set the 
	 * html code for the slider is added to this element, if it is not set the html code is added to the document.
	 */
	this.parentElement;
	
	
	/**
	 * This input field contains the value for this slider as a number. 
	 */
	this.valueInput;
	
	
	/**
	 * The slider element.
	 */
	this.slider;
	
	
	/**
	 * The drag helper used to drag the slider button.
	 */
	this.dragHelper;
	
	
	/**
	 * This is the slider course. 
	 */
	this.sliderCourse;
	
	
	/**
	 * The class name to set for the container containing the slider, if not set the default class is set. 
	 */
	this.className;
	
	
	/**
	 * If this is true the slider can not be moved.
	 */
	this.disabled = false;
	
	
	/**
	 * The sliders current value.
	 */
	this.value = 0;
}


// services


/**
 * Sets up everything needed for the slider, adds and creates the html code of the slider.
 * @param className The class name to set for the container containing the slider, if not set the default class is set.   
 */
Slider.prototype.init = function(className) {
	this.className = className;
	if (!exists(this.htmlCode)) {
		this.createHtmlCode();
		this.initDragHelper();
	}
}	


/**
 * Set the parent element of the slider. This needs to be set before the slider is inited and the html code is created. 
 * If no parent element is set the slider is added to the document. 
 * @parentElement The parent element to set. If this i set before the html code is created the slider is added as 
 * a child to the parent element.
 * @throws Error if parentElement is undefined or null. 
 */ 
Slider.prototype.setParent = function(parentElement) {
	assertExists(parentElement, "The parent element is undefined or null");
	this.parentElement = parentElement;
}


/**
 * Set the disabled status of the slider. When disabled the slider can not be moved 
 * and the input field is disabled.
 * @param disabled True to disable the slider, false to enable.
 * @param className If provided this class is set for the slider.
 * @throws Error if disabled is undefined or null.  
 */
Slider.prototype.toggleDisabled = function(disabled, className) {
	assertExists(disabled, "The disabled status to for the slider set is undefined or null");
	if (disabled == true) {
		this.valueInput.disabled = true;
		this.disabled = true;
		setClass(this.htmlCode, className);
	}
	else {
		this.valueInput.disabled = false;
		this.disabled = false;
		var classToSet = exists(this.className) ? this.className : DEFAULT_SLIDER_CONTAINER_CLASS;
		setClass(this.htmlCode, classToSet);
	}
}


/**
 * Gets the value of the slider.
 * @return The sliders current value. 
 */
Slider.prototype.getValue = function() {
	return this.value;
}


/**
 * Sets a value for the slider. The slider button is moved to the correct button according to 
 * the given value.
 * @param value The value to set. 
 * @throws Error if value is undefined, null, not a number, negative or larget than the max value for the slider. 
 */
Slider.prototype.setValue = function(value) {
	assertExists(value, "The value to set for the slider is undefined or null");
	assertIsNumber(value, "The value to set for the slider is not a number");
	if (value < 0 || value > this.maxValue) {
		throw new Error("The value to set is negative or larger than the max value: " + value);
	}
	if (this.value == value) {
		this.valueInput.value = value;
		return;
	}
	var length = this.sliderCourse.offsetWidth;
	var pixelRatio = this.maxValue / (length - this.slider.offsetWidth);
	this.slider.style.left = (Math.ceil(value / pixelRatio)) + "px";
	this.value = value;
	this.valueInput.value = value;	
}


// implementation


/**
 * The mouse has been pressed on the slider button, begin to slide the button.
 */
Slider.prototype.beginDrag = function(event) {
	if (!this.disabled) {
		this.dragHelper.beginDrag(event);
	}	
}


/**
 * Initialized the drag helper used to drag the slider button.
 */
Slider.prototype.initDragHelper = function() {
	this.dragHelper = new DragHelper(this.slider);
	this.dragHelper.setDragMode(DRAG_MODE_X);
	this.dragHelper.setXMin(0);
	this.dragHelper.setXMax(this.sliderCourse.offsetWidth - (this.slider.offsetWidth / 2));
	this.dragHelper.setMoveFunction("sliderMoved", this);
}


/**
 * The slider has been moved. Show the sliders value in the input field.
 */
Slider.prototype.sliderMoved = function() {
	var newValue = this.getSliderValue(parseInt(this.slider.style.left));
	this.valueInput.value = newValue;
	this.value = newValue;
}


/**
 * Get the slider value for a give position.
 * @param pos The position to get a slider value for. 
 * @return The value for the position (between 0 and the max value set);
 */
Slider.prototype.getSliderValue = function(pos) {
	var length = this.sliderCourse.offsetWidth;
	var pixelRatio = this.maxValue / (length - this.slider.offsetWidth);
	var value = Math.ceil(pos * pixelRatio); 
	return value > this.maxValue ? this.maxValue : value;
}


/**
 * Enter has been pressed in the input field, set the value.
 * @param event The key event that triggered this action.
 */
Slider.prototype.enterPressed = function(event) {
	if (!isNumber(parseInt(this.valueInput.value))) {
		this.setValue(this.value);
		return;
	}
	var value = parseInt(this.valueInput.value);
	value = value < 0 ? 0 : value;
	value = value > this.maxValue ? this.maxValue : value; 
	this.setValue(value);
}


/**
 * Creates the html code of the slider. 
 */
Slider.prototype.createHtmlCode = function() {
	this.htmlCode = document.createElement("div");
	var classToSet = exists(this.className) ? this.className : DEFAULT_SLIDER_CONTAINER_CLASS;
	setClass(this.htmlCode, classToSet);
	var parentElement = exists(this.parentElement) ? this.parentElement : document.body;
	parentElement.appendChild(this.htmlCode);
	
	var slider = document.createElement("div");
	setClass(slider, "slider");
	this.htmlCode.appendChild(slider);
	
	this.sliderCourse = document.createElement("div");
	setClass(this.sliderCourse, "sliderCourse");
	slider.appendChild(this.sliderCourse);
	var sliderCourseLeft = document.createElement("div");
	setClass(sliderCourseLeft, "sliderCourseLeft");
	this.sliderCourse.appendChild(sliderCourseLeft);
	var sliderCourseMiddle = document.createElement("div");
	setClass(sliderCourseMiddle, "sliderCourseMiddle");
	this.sliderCourse.appendChild(sliderCourseMiddle);
	var sliderCourseRight = document.createElement("div");
	setClass(sliderCourseRight, "sliderCourseRight");
	this.sliderCourse.appendChild(sliderCourseRight);
	
	this.slider = document.createElement("div");
	setClass(this.slider, "sliderButton");
	this.slider.style.left = "0px";
	var thisObj = this;
	this.slider.onmousedown = function(event) {
		thisObj.beginDrag(event);
	}
	slider.appendChild(this.slider);
	
	this.valueInput = document.createElement("input");
	this.valueInput.setAttribute("type", "text");
	this.valueInput.setAttribute("maxLength", "4");
	this.valueInput.setAttribute("size", "4")
	setClass(this.valueInput, "valueInput");
	var thisObj = this;
	this.valueInput.onkeyup = function(event) {
		event = exists(event) ? event : window.event;
		if (event.keyCode == 13) { 
			thisObj.enterPressed(event);
		}
	}
	this.valueInput.onblur = function(event) {
		event = exists(event) ? event : window.event;
		thisObj.enterPressed(event);
	}
	
	this.htmlCode.appendChild(this.valueInput);
	this.valueInput.value = 0;
}