/**
 * This class manages the statusbar.
 */
 

// constants


/**
 * The name set for the statusbar html tag.
 */
var STATUSBAR_NAME = "statusbar";
 

/**
 * The name of the container for the statusbar buttons. 
 */
var STATUSBAR_BUTTONS_CONTAINER_NAME = "statusbarButtons";


/**
 * The name of the statusbar tray container. 
 */
var STATUSBAR_TRAY_CONTAINER_NAME = "tray";


/**
 * The event fired when a button is added to the status bar. 
 */
var STATUSBAR_BUTTON_ADDED_EVENT = "statusbarButtonAdded";


/**
 * The event fired when a button is removed from the status bar.
 */
var STATUSBAR_BUTTON_REMOVED_EVENT = "statusbarButtonRemoved";


/**
 * The event fired when a button is minimized.
 */
var STATUSBAR_BUTTON_MINIMIZED_EVENT = "statusbarButtonMinimized";


/**
 * The event fired when a button has been activated. 
 */
var STATUSBAR_BUTTON_ACTIVATED_EVENT = "statusbarButtonActivated";
 
 
/**
 * The event fired when a button has been deactivated. 
 */
var STATUSBAR_BUTTON_DEACTIVATED_EVENT = "statusbarButtonDeactivated";


/**
 * The name of the right click menu used for the status bar buttons.
 */ 
var STATUSBAR_RIGHT_CLICK_MENU_NAME = "statusBarRightClickMenu";


/**
 * The id of the minimize item in the right click menu. 
 */
var MINIMIZE_RIGHT_CLICK_MENU_ITEM_ID = "minimizeItem";


/**
 * The id of the maximize item in the right click menu.
 */
var MAXIMIZE_RIGHT_CLICK_MENU_ITEM_ID = "maximizeItem";


/**
 * The id of the close item in the right click menu.
 */
var CLOSE_RIGHT_CLICK_MENU_ITEM_ID = "closeItem";


/**
 * The id of the help item in the right click menu.
 */
var HELP_RIGHT_CLICK_MENU_ITEM_ID = "helpItem";


// construction


/**
 * The instance of this class used. 
 */
Statusbar.instanceObject = new Statusbar();


/**
 * Constructs a instance of this class. 
 */
function Statusbar() {

	
	/**
	 * The statusbars html code, undefined if the statusbar has not been inited yet. 
	 */	 
	this.htmlCode;	
	
	
	/**
	 * The container containing the statusbar buttons. 
	 */
	this.statusbarButtons;
	
	
	/**
	 *	 The statusbar tray.
	 */
	this.tray;
	
	
	/**
	 * The even processor used to process events.  
	 */
	this.eventProcessor;
	
	
	/**
	 * Containing the buttons in the statusbar. 
	 */
	this.buttons = new Array();
	
	
	/**
	 * The largest button id used. 
	 */
	this.largestButtonId = -1;
	
	
	/**
	 * A reference to the currently activated button. 
	 */
	this.activeButton;
	
	
	/**
	 * This is the button the right click menu is shown over. 
	 */
	this.rightClickButton;
	
	
	/**
	 * The right click menu.
	 */
	this.rightClickMenu;
}


/**
 * Gets the instance of this class used. 
 */
Statusbar.instance = function() {
	return this.instanceObject;
}


// services


/**
 * Inits the statusbar.
 */
Statusbar.prototype.init = function() {
	this.createStatusbar();
	this.createRightClickMenu();
}


// services


/**
 * Gets the current height of the statusbar.
 * @return The height of the statusbar, 0 if the statusbar has not been created yet.  
 */
Statusbar.prototype.getHeight = function() {
	if (exists(this.htmlCode)) {
		return this.htmlCode.offsetHeight;
	}
	return 0;
}


/**
 * Adds a statusbar button.
 * @param button The button to add. 
 * @throws Error if button is undefined or null.
 */
Statusbar.prototype.addButton = function(button) {
	button.setId(this.largestId() + 1);
	this.buttons[this.buttons.length] = button;
	this.statusbarButtons.appendChild(button.getHtmlCode());
	this.fireButtonAddedEvent();
}


/**
 * Gets a button with a given id. 
 * @param id The id to get the button for. 
 * @return The button found, null if no button found. 
 * @throws Error if id is undefined or null.
 */
Statusbar.prototype.getButton = function(id) {
	assertExists(id, "The id of the button is undefined or null");
	for (var i = 0; i < this.buttons.length; i++) {
		if (this.buttons[i].getId() == id) {
			return this.buttons[i];
		}
	}
	return null;
}


/**
 * Gets the first button containing some given text. 
 * @param text The text for the button to get.
 * @return The first button containing the given text, null if no button with text exists.
 * @throws Error if text is undefined, null or empty.
 */
Statusbar.prototype.getButtonWithText = function(text) {
	assertNotEmpty(text, "The text to get the button with is undefined, null or empty");
	for (var i = 0; i < this.buttons.length; i++) {
		if (this.buttons[i].getText() == text) {
			return this.buttons[i];
		}
	}
	return null;
}


/**
 * Gets the buttons in the status bar. 
 * @return A array containing the buttons in the status bar. 
 */
Statusbar.prototype.getButtons = function() {
	return this.buttons;
}


/**
 * Gets the current active button.
 * @return The current active button, null if no button is currently active.  
 */
Statusbar.prototype.getActiveButton = function() {
	return this.activeButton;
}
 

/**
 * Removes all the buttons from the status bar. 
 */
Statusbar.prototype.removeButtons = function() {
	for (var i = this.buttons.length - 1; i >= 0; i--) {
		this.removeButton(this.buttons[i]);
	}
}


/**
 * Removes a button.
 * @param button The button to remove. 
 * @throws Error if button is undefined or null.
 */
Statusbar.prototype.removeButton = function(button) {
	if (this.activeButton == button) {
		this.activeButton = null;
	}
	for (var i = 0; i < this.buttons.length; i++) {
		if (this.buttons[i] == button) {
			this.buttons[i] = null;
			this.buttons = removeNullEntries(this.buttons);
		}
	}	
	this.removeButtonCode(button);
	this.fireButtonRemovedEvent(button);
}


/**
 * Activates a button. 
 * @param button The button to activate. 
 * @throws Error if button is undefined or null.
 */
Statusbar.prototype.activateButton = function(button) {
	assertExists(button, "The button to activate is null or undefined");
	if (button.isActive()) {
		return;
	}
	if (exists(this.activeButton)) {
		this.activeButton.deactivate();
	}
	this.activeButton = button;
	button.activate();
	this.fireButtonActivatedEvent(button);
}


/**
 * Deactivates a button.
 * @param button The button to deactivate. 
 * @throws Error if button is undefined or null.
 */
Statusbar.prototype.deactivateButton = function(button) {
	assertExists(button, "The button to activate is null or undefined");
	if (button.isInactive()) {
		return;
	}	
	if (exists(this.activeButton) && this.activeButton == button) {
		this.activeButton = null;
	}
	button.deactivate();
	this.fireButtonDeactivatedEvent(button);
}


/**
 * Minimizes a button. 
 * @param button The button to deactivate. 
 * @throws Error if button is undefined or null.
 */
Statusbar.prototype.minimizeButton = function(button) {
	assertExists(button, "The button to activate is null or undefined");
	if (exists(this.activeButton) && this.activeButton == button) {
		this.activeButton = null;
	}
	if (!button.isMinimized()) {
		button.minimize();
		this.fireButtonMinimizedEvent(button);
	}
}


/**
 * One of the buttons has been clicked. This function is called in the scope of 
 * that buttons html code. 
 */
function buttonClicked() {
	var button = Statusbar.instance().getButton(this.id);
	if (!exists(button)) {
		return;
	}
	if (button.isActive()) {
		Statusbar.instance().minimizeButton(button);
	}
	else {
		Statusbar.instance().activateButton(button);
		button.executeFunction();
	}
}


/**
 * The mouse has been moved over a button set the mouseover class.
 */
function mouseOverButton() {
	var button = Statusbar.instance().getButton(this.id);
	if (!exists(button)) {
		return;
	}
	button.mouseOver();
}


/**
 * The mouse has been moved out of a buttons html code, set back the original class.
 */
function mouseOutOfButton() {
	var button = Statusbar.instance().getButton(this.id);
	if (!exists(button)) {
		return;
	}
	button.mouseOut();
}
	
	
/**
* One of the buttons has been rightclicked. Shows the right click menu.
* @param event The event that triggered this.
*/
function showStatusBarRightClickMenu(event) {
	event = exists(event) ? event : window.event;
	var target = event.target;
	target = exists(target) ? target : event.srcElement; 
	// the element clicked is the buttons left, middle or right part, the button itself is the parent node
	var button = Statusbar.instance().getButton(target.parentNode.getAttribute("id"));
	// set the right click items for this button
	Statusbar.instance().clearRightClickMenu();
	Statusbar.instance().setRightClickItems(button);
	Statusbar.instance().rightClickButton = button;
	// show/hide the minimize/maximize menu items 
	if (button.isMinimized()) {
		showRightClickItem(STATUSBAR_RIGHT_CLICK_MENU_NAME, MAXIMIZE_RIGHT_CLICK_MENU_ITEM_ID);
		hideRightClickItem(STATUSBAR_RIGHT_CLICK_MENU_NAME, MINIMIZE_RIGHT_CLICK_MENU_ITEM_ID);
	}
	else {
		showRightClickItem(STATUSBAR_RIGHT_CLICK_MENU_NAME, MINIMIZE_RIGHT_CLICK_MENU_ITEM_ID);
		hideRightClickItem(STATUSBAR_RIGHT_CLICK_MENU_NAME, MAXIMIZE_RIGHT_CLICK_MENU_ITEM_ID);
	}
	showRightClickMenu(STATUSBAR_RIGHT_CLICK_MENU_NAME, event);
}


/** 
 * The minimize right click menu item has been clicked. Minimize the according button.
 */
function minimizeClicked() {
	Statusbar.instance().minimizeButton(Statusbar.instance().rightClickButton);
}


/**
 * The maximize right click menu item has been clicked. Maximize the according button. 
 */
function maximizeClicked() {
	Statusbar.instance().activateButton(Statusbar.instance().rightClickButton);
}


/**
 * The close right click menu item has been clicked. Close the according button.
 */
function closeClicked() {
	Statusbar.instance().removeButton(Statusbar.instance().rightClickButton);
}


/**
 * THe help right click item has been clicked. 
 */
function helpClicked() {
	if (exists(showStatusbarHelp())) {
		showStatusbarHelp();
	}
}


// implementation


/**
 * Gets the largest id used for a button. 
 * @return The largest id used.
 */
Statusbar.prototype.largestId = function() {
	return this.largestButtonId++;
}


/**
 * Creates the right click menu for the status bar buttons.
 */
Statusbar.prototype.createRightClickMenu = function() {
	if (!this.isRightClickMenuAvailable()) {
		return;
	}
	this.rightClickMenu = createRightClickContainer(STATUSBAR_RIGHT_CLICK_MENU_NAME, document.body);
}


/**
 * Sets the reight click items for a given button.
 * @param button The button to set the items for. 
 */
Statusbar.prototype.setRightClickItems = function(button) {
	var rightClickItems = button.getRightClickItems();
	for (var i = 0; i < rightClickItems.length; i++) {
		var item = rightClickItems[i];
		addRightClickMenuItem(STATUSBAR_RIGHT_CLICK_MENU_NAME, item.getId(), item.getText(), item.getFunctionToCall()); 
	}
}


/**
 * Clears the right click menu for all items.
 */
Statusbar.prototype.clearRightClickMenu = function() {
	for (var i = this.rightClickMenu.childNodes.length -1; i >= 0; i--) {
		this.rightClickMenu.removeChild(this.rightClickMenu.childNodes[i]);
	}
}


/**
 * Checks if the right click menu is available.
 * @return True if the right click menu is available, false otherwise. 
 */
Statusbar.prototype.isRightClickMenuAvailable = function() {
	try {
		if (isDefined(createRightClickContainer)) {
			return true;
		}
	}
	catch(Error) {
		return false;
	}
}


/**
 * Creates the html code of the statusbar.
 */
Statusbar.prototype.createStatusbar = function() {
	this.htmlCode = document.createElement("div");
	this.htmlCode.setAttribute("name", STATUSBAR_NAME);
	setClass(this.htmlCode, "statusbar");
	document.body.appendChild(this.htmlCode);
	
	// create the container where statusbar buttons are shown
	this.statusbarButtons = document.createElement("div");
	this.statusbarButtons.setAttribute("name", STATUSBAR_BUTTONS_CONTAINER_NAME);
	setClass(this.statusbarButtons, "statusbarButtons");
	this.htmlCode.appendChild(this.statusbarButtons);
	
	this.tray = document.createElement("div");
	this.tray.setAttribute("name", STATUSBAR_TRAY_CONTAINER_NAME);
	setClass(this.tray, "statusbartray");
	this.htmlCode.appendChild(this.tray);
	
	var clear = document.createElement("div");
	clear.style.clear = "both";
	this.htmlCode.appendChild(clear);
}


/**
 * Removes the html code of a button.
 * @param button This buttons code is removed. 
 */
Statusbar.prototype.removeButtonCode = function(button) {
	for (var i = 0; i < this.statusbarButtons.childNodes.length; i++) {
		if (button.getId() == this.statusbarButtons.childNodes[i].getAttribute("id")) {
			this.statusbarButtons.removeChild(this.statusbarButtons.childNodes[i]);
		}
	}	
}


/**
 * Fires a button added event.
 * @param button the button added. 
 */
Statusbar.prototype.fireButtonAddedEvent = function(button) {
	if (!eventsAvailable()) {
		return;
	}
	getEventServer().notifyListeners(new Event(STATUSBAR_BUTTON_ADDED_EVENT, button));
}


/**
 * Fires a button removed event. 
 * @param button The button removed. 
 */
Statusbar.prototype.fireButtonRemovedEvent = function(button) {
	if (!eventsAvailable()) {
		return;
	}
	getEventServer().notifyListeners(new Event(STATUSBAR_BUTTON_REMOVED_EVENT, button));
}
 

/**
 * Fires a button activated event. 
 * @param button The button activated. 
 */
Statusbar.prototype.fireButtonActivatedEvent = function(button) {
	if (!eventsAvailable()) {
		return;
	}
	getEventServer().notifyListeners(new Event(STATUSBAR_BUTTON_ACTIVATED_EVENT, button));
}


/**
 * Fires a button deactivated event. 
 * @param button The button deactivated. 
 */
Statusbar.prototype.fireButtonDeactivatedEvent = function(button) {
	if (!eventsAvailable()) {
		return;
	}
	getEventServer().notifyListeners(new Event(STATUSBAR_BUTTON_DEACTIVATED_EVENT, button));
}


/**
 * Fires a button minimized event. 
 * @param button The button minimized.
 */ 
Statusbar.prototype.fireButtonMinimizedEvent = function(button) {
	if (!eventsAvailable()) {
		return;
	}
	getEventServer().notifyListeners(new Event(STATUSBAR_BUTTON_MINIMIZED_EVENT, button));
}


/**
 * Returns true if a EventServer is available.
 */
function eventsAvailable() {
	try {
		if (isDefined(getEventServer)) {
			return true;
		}
	}
	catch(Error) {
		return false;
	}
	try {
		if (isDefined(Event)) {
			return true;
		}
	}
	catch(Error) {
		return false;
	}
	return true;
}

