/**
 * This class is a button that can be shown in the statusbar. 
 */


// constants


/**
 * The class set for a inactive button. 
 */
var INACTIVE_BUTTON_CLASS = "inactiveStatusbarButton";


/**
 * The class set for a active button.
 */
var ACTIVE_BUTTON_CLASS = "activeStatusbarButton";


/**
 * The class set when the mouse is over the button.
 */
var MOUSE_OVER_BUTTON_CLASS = "mouseoverStatusbarButton";


/**
 * The minimized button class.
 */
var MINIMIZED_BUTTON_CLASS = "inactiveStatusbarButton";


/**
 * The class name of the left part of the status bar button.
 */
var LEFT_BUTTON_CLASS = "leftStatusbarButton";

 
/**
 * The class name for the middle part of the status bar buttons html code.
 */
var MIDDLE_BUTTON_CLASS = "middleStatusbarButton";


/**
 * The class name for the right part of the status bar buttons html code. 
 */
var RIGHT_BUTTON_CLASS = "rightStatusbarButton";


// construction


/**
 * Constructs a statusbar button.
 * @param text The text to show. 
 * @param functionToCall The function to call when the buttons i clicked. 
 * @param object The object to call the function on when a button is clicked. 
 * @throws Error if text is undefined, null or empty. 
 */
function StatusbarButton(text, functionToCall, object) {
	assertNotEmpty(text, "The text to show is undefined, null or empty: " + text);
	
	this.text = text;
	this.functionToCall = functionToCall;
	this.object = object;
	
	/**
	 * The id of the button. This is set by the statusbar. 
	 */
	this.id;
	
	
	/**
	 * The buttons html code.
	 */
	this.htmlCode;
	
	
	/**
	 * Set to true if this button is active.
	 */
	this.active = false;
	
	
	/**
	 * Set to true if this buttons is inactive.
	 */
	this.inactive = false;
	
	
	/**
	 * Set to true if this button is minimized.
	 */
	this.minimized = false;
	
	
	/**
	 * The array containing the right click items to be shown for this button.
	 */
	this.rightClickItems = new Array();
	
	
	// add the default right click items
	this.rightClickItems[0] = new StatusButtonRightClickItem(MINIMIZE_RIGHT_CLICK_MENU_ITEM_ID, "Minimize", minimizeClicked);
	this.rightClickItems[1] = new StatusButtonRightClickItem(MAXIMIZE_RIGHT_CLICK_MENU_ITEM_ID, "Maximize", maximizeClicked);
	this.rightClickItems[2] = new StatusButtonRightClickItem(CLOSE_RIGHT_CLICK_MENU_ITEM_ID, "Close", closeClicked);
	this.rightClickItems[3] = new StatusButtonRightClickItem(HELP_RIGHT_CLICK_MENU_ITEM_ID, "Help", helpClicked);
}


// services


/**
 * Get the text shown in the button. 
 * @return The text shown in the button. 
 */
StatusbarButton.prototype.getText = function() {
	return this.text;
}


/**
 * Set the text to be shown in the button. 
 * @param text The text to show. 
 * @throws Error if text is undefined, null or empty.
 */
StatusbarButton.prototype.setText = function(text) {
	assertNotEmpty(text, "The text to set is undefined, null or empty: " + text);
	this.text = text;
	
	if (!exists(this.htmlCode)) {
		return;
	}
	// remove the existing text
	for (var i = this.getHtmlCode().childNodes.length - 1; i >= 0 ; i--) {
		this.getHtmlCode().removeChild(this.getHtmlCode().childNodes[i]);
	}
	// add the new text
	this.htmlCode.appendChild(this.createLeftButtonCode());
	this.htmlCode.appendChild(this.createMiddleButtonCode(text));
	this.htmlCode.appendChild(this.createRightButtonCode());
}


/**
 * Get the function to call when the button is clicked. 
 * @return The function to call.
 */
StatusbarButton.prototype.getFunctionToCall = function() {
	return this.functionToCall;
}


/**
 * Set the function to call when a buttons has been clicked.
 * @param functionToCall The function to set.
 * @throws Error if functionToCall is null or undefined.  
 */
StatusbarButton.prototype.setFunctionToCall = function(functionToCall) {
	assertExists(functionToCall, "The function to set is undefined or null.");
	this.functionToCall = functionToCall;
}


/**
 * Gets the object to call the function on when the button is clicked. 
 * @return The object to call the function on, undefined if no object is sett.
 */
StatusbarButton.prototype.getObject = function() {
	return this.object;
}


/**
 * Set the object to call the function on when the button is clicked.
 * @param object The object to set.  
 */
StatusbarButton.prototype.setObject = function(object) {
	this.object = object;
}



/**
 * Get the id of this button. 
 * @param id The id set, undefined if no id set yet. 
 */
StatusbarButton.prototype.getId = function() {
	return this.id;
}	
 
 
/**
 * Set the unique id for this button. This is set by the statusbar. 
 * @param id The id to set. 
 * @throws Error if id is undefined or null.
 */
StatusbarButton.prototype.setId = function(id) {
	assertExists(id, "The id to set for the statusbar button is undefined or null");
	this.id = id;
}


/**
 * Gets the html code of this button. 
 * @return The html code of this button. 
 */
StatusbarButton.prototype.getHtmlCode = function() {
	if (!exists(this.htmlCode)) {
		this.createHtmlCode();
	}
	return this.htmlCode;
}


/**
 * Activates the button. 
 */
StatusbarButton.prototype.activate = function() {
	setClass(this.htmlCode, ACTIVE_BUTTON_CLASS);
	this.active = true;
	this.minimized = false;
	this.inactive = false;
}


/**
 * Executes the onclick function.
 */
StatusbarButton.prototype.executeFunction = function() {
	if (exists(this.object)) {
		this.functionToCall.call(this.object, this);
	}
	else if (exists(this.functionToCall)) {
		this.functionToCall(this);
	}
}


/**
 * Deactivates the button.
 */
StatusbarButton.prototype.deactivate = function() {
	setClass(this.htmlCode, INACTIVE_BUTTON_CLASS);
	this.inactive = true;
	this.minimized = false;
	this.active = false;
}	


/**
 * Minimizes the button.
 */
StatusbarButton.prototype.minimize = function() {
	setClass(this.htmlCode, MINIMIZED_BUTTON_CLASS);
	this.minimized = true;
	this.active = false;
	this.inactive = false;
}


/**
 * Checks if the button is activated.
 * @return True if the button is activated, false otherwise. 
 */
StatusbarButton.prototype.isActive = function() {
	return this.active;
}


/**
 * Checks if the button is inactive.
 * @return True if the button is inactive, false otherwise. 
 */
StatusbarButton.prototype.isInactive = function() {
	return this.inactive;
}


/**
 * Checks if the buttons is minimized.
 * @return True if the button is minimized, false otherwise. 
 */
StatusbarButton.prototype.isMinimized = function() {
	return this.minimized;
}


/**
 * Gets the right click menu items for this button.
 * @return A array containing the right click menu items for this button.
 */
StatusbarButton.prototype.getRightClickItems = function() {
	return this.rightClickItems;
}


/**
 * Add a right click menu item. 
 * @param item The right click menu item to add. 
 * @throws Error if item is undefined, null or empty.
 */
StatusbarButton.prototype.addRightClickItem = function(item) {
	assertExists(item, "The item to add is undefined or null");
	this.rightClickItems[this.rightClickItems.length] = item;
}


/**
 * Removes a right click items.
 * @param itemId The id of the right click item to remove.
 * @throws Error if itemId is undefined, null or empty. 
 */
StatusbarButton.prototype.removeRightClickItem = function(itemId) {
	assertNotEmpty(itemId, "The item id to remove is undefined, null or empty");
	for (var i = 0; i < this.rightClickItems.length; i++) {
		if (this.rightClickItems[i].getId() == itemId) {
			this.rightClickItems[i] = null;
			this.rightClickItems = removeNullEntries(this.rightClickItems);
		}
	}
}


/**
 * The mouse has been moved over this button set the mouseover class.
 */
StatusbarButton.prototype.mouseOver = function() {
	if (this.isActive()) {
		return;
	}
	setClass(this.htmlCode, MOUSE_OVER_BUTTON_CLASS);
}


/**
 * The mouse has been moved out of the buttons html code set back the original class.
 */
StatusbarButton.prototype.mouseOut = function() {
	if (this.isActive()) {
		return;
	}
	else if (this.isMinimized()) {
		setClass(this.htmlCode, MINIMIZED_BUTTON_CLASS);
	}
	else {
		setClass(this.htmlCode, INACTIVE_BUTTON_CLASS);
	}
}


// implementation


/**
 * Creates the html code of this button.
 */
StatusbarButton.prototype.createHtmlCode = function() {
	this.htmlCode = document.createElement("div");
	this.htmlCode.setAttribute("id", this.id);
	setClass(this.htmlCode, INACTIVE_BUTTON_CLASS);
	this.htmlCode.onclick = buttonClicked;
	this.htmlCode.onmouseover = mouseOverButton;
	this.htmlCode.onmouseout = mouseOutOfButton;
	this.htmlCode.oncontextmenu = showStatusBarRightClickMenu;
	
	this.htmlCode.appendChild(this.createLeftButtonCode());
	this.htmlCode.appendChild(this.createMiddleButtonCode(this.text));
	this.htmlCode.appendChild(this.createRightButtonCode());
	this.inactive = true;
} 
	
	
/**
 * Creates the left part of the html code for the button.
 * @return The html code created for the left part. 
 */
StatusbarButton.prototype.createLeftButtonCode = function() {
	var left = document.createElement("div");
	setClass(left, LEFT_BUTTON_CLASS);
	return left;
}


/**
 * Creates the middle part of the html code for the button.
 * The middle part is the one containing the text.
 * @param text The text to set. 
 * @return The html code created for the middle part. 
 */
StatusbarButton.prototype.createMiddleButtonCode = function(text) {
	var middle = document.createElement("div");
	setClass(middle, MIDDLE_BUTTON_CLASS);
	middle.appendChild(document.createTextNode(text));
	return middle;
}


/**
 * Creates the right part of the html code for the button.
 * @return The html code created for the right part. 
 */
StatusbarButton.prototype.createRightButtonCode = function() {
	var right = document.createElement("div");
	setClass(right, RIGHT_BUTTON_CLASS);
	return right;
}
 