/**
 * Constructs a column in the table.
 * @param name The name of the column. This needs to be unique for this column.
 * @param width The width to set for this column.
 * @param headerText The header text of this column.
 * @param fieldChooserText The text to show in the field chooser for this column.
 * @throws Error if name is undefined, null or empty.
 * @throws Error if width is negative.
 * @throws Error if headerText is undefined or null.
 */
function Column(name, width, headerText, fieldChooserText) {
	assertNotEmpty(name, "Name is undefined, null or empty: " + name);
	assertNotNegative(width, "Width to set for the table is negative: " + width);
	assertExists(headerText, "Header text is undefined or null: " + headerText);
	
	this.name = name;
	this.width = width;
	this.headerText = headerText;
	this.fieldChooserText = fieldChooserText;
	this.alignment;
	this.visible = false;
	this.properties = new Array();
}


// services


/**
 * Get the name of this column.
 */
Column.prototype.getName = function() {
	return this.name;
}


/**
 * Returns the width of this column.
 */
Column.prototype.getWidth = function() {
	return this.width;
}


/**
 * Get the alignment value of this column.
 * @return The alignment value set, undefined if no value has been set.
 */
Column.prototype.getAlignment = function() {
	return this.alignment;
}


/**
 * Get the header text for this column.
 * @return The header for this column.
 */
Column.prototype.getHeaderText = function() {
	return this.headerText;
}


/**
 * Get the field chooser text for this column.
 * @return The field chooser text for this column.
 */
Column.prototype.getFieldChooserText = function() {
	return this.fieldChooserText;
}


/**
 * Set a alignment for this column.
 * @param value The alignment value to set for this column.
 */
Column.prototype.setAlignment = function(value) {
	assertNotEmpty(value, "Alignment value to set for column is undefined, null or empty: " + value);
	
	this.alignment = value;
}


/**
 * Set the width of this column.
 * @param width The width to set.
 */
Column.prototype.setWidth = function(width) {
	if (width < 0) {	
		width = 0;
	}	
	this.width = width;
}


/**
 * Set visibility.
 * @param visibility True if this column is visible, false for not visible.
 * @throws Error if visibility is not undefined or null.
 */
Column.prototype.setVisibility = function(visibility) {
	assertExists(visibility, "Visibility to set is undefined or null");
	if (visibility == true) {
		this.visible = true;
	}
	else if (visibility == false) {
		this.visible = false;
	}
}



/**
 * Returns this column as xml string.
 */
Column.prototype.toXml = function() {
	var xmlString =
		"<column>" + 
			"<name>" + this.getName() + "</name>" +
			"<visible>" + this.visible + "</visible>" +
		"</column>";
	return xmlString;			
}			


/**
 * Sets a properties for the column.
 * @param property The property to set.
 * @throws Error if property is undefined, null or empty.
 */
Column.prototype.setProperty = function(property) {
	assertNotEmpty(property, "Property is undefined, null or empty");
	this.properties[this.properties.length] = property;
}
