/**
 * The column to sort on.
 */
var columnToSort = 0;


/**
 * The name of the image indicating ascending sort order.
 */
Table.SORT_DOWN_IMAGE = "/images/sort_down.gif";


/**
 * The name of the image indicating descending sort order.
 */
Table.SORT_UP_IMAGE = "/images/sort_up.gif";


/**
 * The width of a scrollbar in the browser.
 */
Table.BROWSER_SCROLLBAR_WIDTH = 16;


/** 
 * The default laoding time of the tables.
 */
Table.DEFAULT_TIME = 30;


/**
 * To prevent the table and its scrolling objects to send scrolled events to each other indefinetily 
 * the table waits this amount of time in milliseconds before it fires a scrolled event. 
 */
Table.FIRE_SCROLL_EVENT_THRESHOLD = 1000;


/**
 * Constructs an updatable table. The table will update itself.
 * @param id The id of the table to update.
 * @param url The url to load the data from.
 * @param parentElement The element the table is to be inserted in. 
 * @param Optional time parameter. The time between each update.
 * If this is not specified the default value is used.
 * @throws Error if id is undefined, null or empty.
 * @throws Error if url is undefined or null.
 */
function Table(id, url, parentElement, time) {
	assertNotEmpty(id, "Id for the table is undefined, empty or null: " + id);
	assertExists(url, "Url to set for the table is undefined or empty: " + url);
	assertDefined(parentElement, "The element to insert the table into is undefined or null");

	// set the properties of this class
	this.parentElement = parentElement;
	this.url = url;
	this.id = id;
	this.rows = new Array();
	this.rowCounter = 0;
	this.header = new Array();
	this.visibleColumns = new Array();
	this.rightClickMenuColumns = new Array();
	this.linkFields = new Array();
	this.columnOrder = new Array();
	this.sortDataColumns = new Array();
	this.reversed = false;
	this.sortable = false;
	this.width = 0;
	this.height = 0;
	this.className;
	this.headerRowClass;
	this.headerTableClass;
	this.alternateBackgroundColor = false;
	// unique identifier for each row
	this.rowFieldId = "";
	// indicating whether a vertival scrollbar is shown
	this.verticalScrollBar = false;
	// set this to true when a xml layout for this table is being set, when the xml layout is being set
	// the table is not updated
	this.layoutSetInProgress = false;
	
	// check if time is specified
	if (isDefined(time) && !isNull(time)) {
		this.setTime(time);
	}
	// use default time value
	else {
		this.time = Table.DEFAULT_TIME;
	}

	this.timeOutId = 0;
	
	// create the repeater object used for repeatedly calling the load method
	this.repeater = new Repeater(load, this, this.time);
	
	// if this is true the table has been painted
	this.painted = false;
	TableManager.instance().registerNewTable(this);
	
	// used to listen to events 	
	this.eventListener = new Listener(this);
	this.eventListener.addEvent("scrolled", processScrollEventTable);
	
	// the table listens to scroll events and scroll according to these events
	this.scrollObjects = new Array();
	
	// the time when one of the tables scroll objects was last scrolled, the table does not fire any scroll 
	// events for a given threshold of time after the one if its scroll objects has been scrolled, to prevent
	// the table and its scrollobject to send scrolled events to each other indefinetily. 
	this.scrollTimer = 0;
	
	// stores the max scrollheight of the table, if the table is scrolled to this value it is 
	// scrolled all the way to the bottom
	this.maxScrollHeight = 0;
}


// services


/**
 * Gets the id of this table. 
 * @return The id of this table.
 */
Table.prototype.getId = function() {
	return this.id;
}


/**
 * Returns the a array with one entry for each row in the table.
 * Each entry is also a array containing one entry for each cell in the table.
 */
Table.prototype.getData = function() {
	return this.rows;
}


/**
 * Creates a comma separated list of the given field values in all the rows. 
 * @param dataFieldName The name of the data column, if this is a array field values for all the fields
 * in the array are included in the list.
 * @param count The number of entries to include, -1 for all. 
 * @return A comma separated list of the count first values for each rows field value with dataFieldName. 
 */
Table.prototype.getFieldValues = function(dataFieldName, count) {
	var rows = this.getData();	
	count = count < 0 ? rows.length : count;
	var valueList = "";
	for (var i = 0; i < rows.length && i < count; i++) {
		var row = rows[i];
		valueList += this.getFieldValue(row, dataFieldName);
		if (i < rows.length - 1 && i < count - 1) {
	 		valueList += ",";
	 	}
	}
	return valueList;
}


/**
 * Creates a comma separated list of some given fields values in a row.
 * @param row The data row the create the list from.
 * @param dataFieldName The name of the column to include in the list, if this is a array all 
 * the field names are included in the list. 
 */
Table.prototype.getFieldValue = function(row, dataFieldName) {
	var list = "";
	if (dataFieldName instanceof Array) {
		for (var i = 0; i < dataFieldName.length; i++) {
			var field = getFieldFromRow(dataFieldName[i], row);
			list += this.getFieldValue(row, field.getName());
			if (i < dataFieldName.length - 1) {
				list += " - ";
			}
		}
		return list;
	}

	var field = getFieldFromRow(dataFieldName, row);
	if (isDefined(field) && !isNull(field)) {
		list = field.getValue();
	}
	return list;
}


/**
 * Returns the max value in a column. 
 * @param columnName The name of the column to get the max value from.
 */
Table.prototype.maxFieldValue = function(columnName) {
	var max = 0;
	var rows = this.getData();
	for (var i = 0; i < rows.length; i++) {
		var row = rows[i];
		var field = getFieldFromRow(dataFieldName, row);
		if (parseInt(field.getValue()) > max) {
			max = parseInt(field.getValue());
		}
	}
	return max;
}



/**
 * Returns this table as xml, the data is not included in the xml. 
 */
Table.prototype.toXml = function() {
	var xmlString = 
		"<table>" +
			"<tableId>" + this.id + "</tableId>" +
			"<n>" + getParameterValue(this.url, "top", "&") + "</n>" +
			"<updateInterval>" + this.getUpdateInterval() + "</updateInterval>";
			
	xmlString += exists(this.columnToSort) ? "<sortColumn>" + this.columnToSort + "</sortColumn>" : "";
	xmlString += "<sortColumnReversed>" + this.reversed + "</sortColumnReversed>";
	xmlString += exists(this.getCurrentIndex) ? "<index>" + this.getCurrentIndex() + "</index>" : "";
				
	xmlString += "<columns>";
	for (var i = 0; i < this.columnOrder.length; i++) {
		xmlString += this.columnOrder[i].toXml();
	}
	xmlString +="</columns></table>";
	return xmlString;
}


/**
 * Set this to true if the layout is being set, set to false if setting 
 * the layout is done. While the layout is being set the table is not updated 
 * to prevent multiple calls to load caused by for example sorting or changing
 * the index. 
 * @param settingLayout Set to true if the layout is being set, false if layout sett is done.
 * @throws Error if settingLayout is undefined or null.
 */
Table.prototype.settingLayout = function(settingLayout) {
	assertExists(settingLayout, "Settinglayout is null or undefined");
	if (true == settingLayout) {
		this.layoutSetInProgress = true;
		this.cancelUpdate();
	}
	else if (false == settingLayout) {
		this.layoutSetInProgress = false;
		this.restartUpdate();
	}
}

 
/**
 * Sets a layout for the table based on a block of xml.  
 * @param xmlLayout The block of xml containing the layout to set. 
 */
Table.prototype.setXmlLayout = function(xmlLayout) {
	var columns = xmlLayout.getElementsByTagName("columns");
	var needRepaint = false;
	if (isDefined(columns) && isDefined(columns[0]) && !isNull(columns[0])) {
		needRepaint = this.setXmlColumnsLayout(columns[0]);
	}
	
	var updateInterval = getTagNodeValue("updateInterval", xmlLayout);
	if (!isEmpty(updateInterval)) {
		this.setTime(updateInterval);
	}
	
	needRepaint = this.setXmlSortColumn(xmlLayout) == true && this.isPainted() ? true : needRepaint;
	needRepaint = this.setXmlSortOrder(xmlLayout) == true && this.isPainted() ? true : needRepaint;
	this.setXmlIndex(xmlLayout);
	if (this.isPainted() && needRepaint) {
		this.paint();
	}
}


/**
 * Sets the table index from a xml layout.
 * @param layout The layout containing the index. 
 * @return True if the index has been change, false otherwise.
 */
Table.prototype.setXmlIndex = function(layout) {
	if (!exists(this.getCurrentIndex)) {
		return false;
	}
	var index = getTagNodeValue("index", layout);
	if (!isEmpty(index) && !this.getCurrentIndex().equalsIgnoreCase(index)) {
		var indexSelector = document.getElementById("indexSelect" + this.id);
		selectInSelector(indexSelector, index);
		this.changeIndex();
		tableIndexChanged.call(this);
		return true;
	}
	return false;
}


/**
 * Set the xml column sort order. 
 * @param layout The layout contianing the sort order to set. 
 * @param True if the sort order was changed, false otherwise.
 */
Table.prototype.setXmlSortOrder = function(layout) {
	var sortOrder = getTagNodeValue("sortColumnReversed", layout);
	if (exists(sortOrder) && !sortOrder.equalsIgnoreCase(this.reversed + "")) {
		this.reversed = "true".equalsIgnoreCase(sortOrder) ? true : false;
		return true;
	}
	return false;
}


/**
 * Set xml sort column.
 * @param layout The layout contianing the sort order to set. 
 * @return True if the column sorted has been changed. 
 */
Table.prototype.setXmlSortColumn = function(layout) {
	var sortColumn = getTagNodeValue("sortColumn", layout);
	var validColumn = !isEmpty(sortColumn) && exists(this.getColumn(sortColumn));  
	if (validColumn && (isEmpty(this.columnToSort) || !this.columnToSort.equalsIgnoreCase(sortColumn))) {
		this.setColumnToSort(sortColumn);
		return true;
	}
	return false;
}


/**
 * Sets a xml based layout for the columns.
 * @param layout The layout to set for the columns. 
 * @return True if one of the columns visibility status has been changed, false otherwise.
 */
Table.prototype.setXmlColumnsLayout = function(layout) {
	var columns = layout.getElementsByTagName("column");
	if (!isDefined(columns)) {
		// no columns in the layout
		return;
	}	
	
	var columnVisibilityChanged = false;
	for (var i = 0; i < columns.length; i++) {
		var visibilityChanged = this.setXmlColumnLayout(columns[i]);
		columnVisibilityChanged = columnVisibilityChanged == true ? true : visibilityChanged;
	}
	return columnVisibilityChanged;
}


/**
 * Sets a xml based layout for a column.
 * @param layout The layout to set for a column.
 * @return True if the columns visibility status has been changed, false otherwise.
 */
Table.prototype.setXmlColumnLayout = function(layout) {
	var name = getTagNodeValue("name", layout);
	if (!isEmpty(column)) {
		// no column name specified
		return;
	}
	
	var column = this.getColumn(name);
	if (!isDefined(column) || isNull(column)) {
		// column with given name does not exist
		return;
	}
	
	var visibility = getTagNodeValue("visible", layout);
	if (visibility != "true" && visibility != "false") {
		// invalid value for visibility
		return;
	}
	var isVisible = this.isVisibleColumn(name);
	if (!visibility.equalsIgnoreCase(isVisible + "")) {
		this.toggleColumnVisibility(name);
	}
	return !visibility.equalsIgnoreCase(isVisible + "");
}


/**
 * Sets a scroll object. The table will listen to any scroll events by all its scroll objects. 
 * @param id The id of the object to listen to. 
 * @throws Error if the id is undefined, null or empty. 
 */
Table.prototype.setScrollObject = function(id) {
	assertNotEmpty(id, "The id of the scroll object to listen to is undefined, null or empty");
	this.scrollObjects[this.scrollObjects.length] = id;
}


/**
 * Returns true if a element with a given id is one of the scroll elements. 
 * @param id The id to check.
 * @return True if id is one of the scroll elements, false otherwise. 
 * @throws Error if id is undefined null or empty.
 */
Table.prototype.isScrollObject = function(id) {
	assertNotEmpty(id, "The id to check is undefined, null or empty");
	for (var i = 0; i < this.scrollObjects.length; i++) {
		if (this.scrollObjects[i] == id){
			return true;
		}
	}
	return false;
}


// row classes


/**
 * Set row class
 * @param classToSet The class to set.
 * @param throws Error if class is undefined, null or empty
 */
Table.prototype.setRowClass = function(classToSet) {
	assertNotEmpty(classToSet, "The rowClass to set is empty");
	this.rowClass = classToSet;
}


/**
 * Toggle the alternation of the row classes. The class to alternate with
 * will be set on all even rows. 
 * @param alternateClassOn True to alternate the row class.
 * @param alternateClass The class to alternate the background with.
 * @throws Error if color is undefined, null or empty.
 */
Table.prototype.toggleAlternateRowClass = function(alternateClassOn, alternateClass) {
	assertDefined(alternateClass, "Class is null or undefined");
	this.alternateRowClassOn = alternateClassOn;
	this.alternateClass = alternateClass;
}


/**
 * Set the mouse over class for a row.
 * @param mouseOverClass The mouse over class to set.
 * @throws Error if class is undefined, null or empty.
 */
Table.prototype.setMouseOverClass = function(mouseOverClass) {
	assertDefined(mouseOverClass, "The mouseover class to set is null or undefined");
	this.mouseOverClass = mouseOverClass;
}


/**
 * Sets the class for a row.
 * @param row The html row to set the class for.
 * @param index The index of the row in the table.
 */
Table.prototype.addRowClasses = function(row, index) {
	this.addRowClass(row, index);
	this.addRowMouseOverClass(row);
}


/**
 * Adds the row class to a row.
 * @param row The html row to set the class for.
 * @param index The index of the row in the table.
 */
Table.prototype.addRowClass = function(row, index) {
	if (exists(this.getRowClass(index))) {
 		setClass(row, this.getRowClass(index));
 	}
}


/**
 * Gets the class to set for a row with a given index. 
 * @param index The index of the row. 
 * @return The class to set, null if no class to set. 
 */
Table.prototype.getRowClass = function(index) {
	if (!isDefined(this.rowClass) || this.getTableCode().rows[index].counter > 0) {
		return null;
	}
	if (isDefined(this.rowClass) && !this.alternateRowClassOn) {
		return this.rowClass;
	}
	if (index % 2 == 0) {
		return this.rowClass;
 	}
 	else {
	 	return this.alternateClass;
 	}
}


/**
 * Adds the mouseover class for a row.
 * @param row The html row to set the color for.
 */
Table.prototype.addRowMouseOverClass = function(row) {
	if (!isDefined(this.mouseOverClass)) {
		return;
	}

	var mouseOverClass = this.mouseOverClass;
	var originalClass = row.getAttribute("class");
	row.counter = 0;
	row.onmouseover = function () {
		this.counter = this.counter + 1;
		setClass(this, mouseOverClass);
	}
	row.onmouseout = function () {
		this.counter = 0;
		setClass(this, originalClass);
	}
}


// header classes


/**
 * Set the class to use for the header table.
 * @param className The name of the class to set. 
 * @throws Error if class is undefined, null or empty.
 */
Table.prototype.setHeaderTableClass = function(className) {
	assertDefined(className, "The header class to set is null or undefined");
	this.headerTableClass = className;
	var tableCode = this.getHeaderTableCode();
	if (isDefined(tableCode) && !isNull(tableCode)) {
		setClass(tableCode, className);
	}
}


/**
 * Set the class to use for the header row.
 * @param headerClass The class to set for the header.
 * @throws Error if class is undefined, null or empty. 
 */
Table.prototype.setHeaderRowClass = function(headerClass) {
	assertDefined(headerClass, "The header class to set is null or undefined");
	this.headerRowClass = headerClass;
}


/**
 * Sets a mouseover class for the header row.
 * @param headerMouseOverClass The class to set.
 * @throws Error if class is undefined, null or empty.
 */
Table.prototype.setHeaderMouseOverClass = function(headerMouseOverClass) {
	assertDefined(headerMouseOverClass, "The header mouseover class to set is null or undefined");
	this.headerMouseOverClass = headerMouseOverClass; 
}


/**
 * Sets the header class for a row.
 * @param headerRow The header row to set the class for.
 */
Table.prototype.addHeaderClass = function(headerRow) {
	if (isDefined(this.headerRowClass)) {
		setClass(headerRow, this.headerRowClass);
	}
	this.addHeaderMouseOverClass(headerRow);
}


/**
 * Adds the mouseover class to the header.
 * @param headerRow The header row to set the mouseover for.
 */
Table.prototype.addHeaderMouseOverClass = function(headerRow) {
	if (!isDefined(this.headerMouseOverClass)) {
		return;
	}

	var mouseOverClass = this.headerMouseOverClass;
	var originalClass = headerRow.getAttribute("class");
	headerRow.onmouseover = function () {
		setClass(this, mouseOverClass);
	}
	
	if (isDefined(originalClass) && !isNull(originalClass)) {
		headerRow.onmouseout = function () {
			setClass(this, originalClass);
		}
	}
}


// links


/**
 * Sets the colum names that should have a link on the cell.
 */
Table.prototype.setLinkFields = function () {
	var len = arguments.length;
	for (var i = 0; i < len; i++) {
		if (isEmpty(arguments[i])) {
			continue;
		}
		this.linkFields[i] = arguments[i];
	}
}


/**
 * Set link url creator. linkUrlCreator must be a function. The function is called 
 * each time a url for one of the link fields needs to be generated.
 * The function will be called as a method of the table object. The link created 
 * will be given as the first argument and the field the link is for as the second.
 * @param linkUrlCreator The function called when a link needs to be generated.
 * @throws Error if linkUrlCreator is undefined or null.
 */
Table.prototype.setLinkUrlCreator = function(linkUrlCreator) {
	assertDefined(linkUrlCreator, "LinkUrlCreator is undefined or null");
	
	this.linkUrlCreator = linkUrlCreator;
}


/**
 * Returns true if a given field is one of the link fields.
 * @param field The field to check.
 * @throws Error if the field is undefined or null.
 */
Table.prototype.isLinkField = function (field) {
	assertDefined(field, "Field is undefined or null");
	var len = this.linkFields.length;
	for (var i = 0; i < len; i++) {
		if (isDefined(field.getName()) && this.linkFields[i] == field.getName()) {
			return true;
		}
	}
	return false;
}


// right-click event


/**
 * Sets the columns to have a right-click menu enabled.
 */
Table.prototype.setRightClickMenuColumns = function () {
	var len = arguments.length;
	for (var i = 0; i < len; i++) {
		if (isEmpty(arguments[i])) {
			continue;
		}
		this.setRightClickMenuColumn(arguments[i]);
	}
}


/**
 * Set a column to have a right-click menu attached to it.
 * @param name The name of the column.
 * @throws Error if name is undefined, null or empty.
 */
Table.prototype.setRightClickMenuColumn = function(name) {
	assertNotEmpty(name, "Name is undefiend, empty or null");
	var column = this.getColumn(name);
	if (this.isRightClickMenuColumn(name)) {
		// the column is already defined with a right-click menu 
		return;
	}
	if (isDefined(column) && !isNull(column)) {
		this.rightClickMenuColumns[this.rightClickMenuColumns.length] = column;
	}
}


/**
 * Returns true if a given field is already defined with a right-click menu.
 * @param columnName The name of the column to check.
 * @throws Error if the field is undefined or null.
 */
Table.prototype.isRightClickMenuColumn = function (columnName) {
	assertNotEmpty(columnName, "ColumnName is undefined, null or empty: " + columnName);
	var len = this.rightClickMenuColumns.length;
	for (var i = 0; i < len; i++) {
		if (this.rightClickMenuColumns[i].getName() == columnName) {
			return true;
		}
	}
	return false;
}


/**
 * Sets the function to be called when a right-click menu event is triggered on this table.
 */
Table.prototype.setRightClickMenuFunction = function (functionName) {
	assertExists(functionName, "functionName is undefined or null");
	this.rightClickMenuFunction = functionName;
}


// field properties


/**
 * Add a property to the table. Properties can be set for fields or columns.
 * For each property there must be specified a handler function. 
 * the handler function takes the html code of the table cell as argument
 * if a property bold was set the handler would get td node, the field object and the table object
 * as argument. The value of the field is set as chield of the td node. The makeBold property
 * hanlder would enclose the childs of the td node inside a <b> node.
 * @param propertyName The name of the property to make available for setting in the fields.
 * Any previous property with the same name is overridden.
 * @param handler The handler function that is to handle the property.
 * @throws Error if property name is undefined, null or empty.
 * @throws Error if the handler is undefined or null.
 */
Table.addProperty = function (propertyName, handler) {
	assertNotEmpty(propertyName, "PropertyName is undefined, null or empty");
	assertDefined(handler, "Handler is undefined or null");
	
	if (!isDefined(Table.properties)) {
		Table.properties = new Array();
	}
	Table.properties[propertyName] = handler;
}


/**
 * Makes the content of a table cell bold.
 * @param cell The html code of the table cell to make bold.
 */
function makeBold(cell) {
	var boldNode = document.createElement("b");
	
	// append all the cells child nodes to the boldNode
	for (var j = cell.childNodes.length -1; j >= 0; j--) {
		var child = cell.childNodes[j];				
		cell.removeChild(child);
		boldNode.appendChild(child);
	}	
	
	cell.appendChild(boldNode);
}


/**
 * Set properties in the header fields. If no handler for the 
 * property is specified the property is ignored.
 * @param property The property to set.
 * @throws Error if the property is undefined, null or empty.
 */
Table.prototype.setHeaderProperty = function(property) {
	assertNotEmpty(property, "Property is undefined, empty or null");
	
	var len = this.header.length;
	for (var i = 0; i < len; i++) {
		var field = this.header[i];
		field.setProperty(property);
	}
}


/**
 * Set properties for all fields in a colum. If no handler for 
 * the property is specified the property is ignored.
 * @param property The property to set.
 * @param columnName The name of the column to set the property for.
 * @throws Error if property is undefined, null or empty.
 * @throws Error if columnName is undefined, null, empty or if no column with columnName exists in the table.
 */
Table.prototype.setColumnProperty = function(property, columnName) {
	assertNotEmpty(property, "Property is undefined, null or empty");
	var column = this.getColumn(columnName);
	assertExists(column, "No column with the specified name exists");
	column.setProperty(property);	
}


/**
 * Returns the field at a given position, null if there is no 
 * field at that position.
 * @param row The row to get the field from. 
 * @param column The column to get hte field from.
 * @throws Error if row is negative.
 * @throws Error if column is negative.
 */
Table.prototype.getField = function (row, column)  {
	assertNotNegative(row, "Row is negative");
	assertNotNegative(column, "Column is negative");
	
	if (row > this.rows.length-1) {
		return null;
	}
	var row = this.rows[row];
	if (column > row.length) {
		return null;
	}
	return row[column];
	
}


/**
 * Gets a row with a given value for a given field.
 * @param fieldName The name of the field whose value to search for. 
 * @param value The value of the field fieldName in the row to get. 
 * @return The first row with value for fieldName, null if no row found. 
 * @throws Error if fieldName is undefined, null or empty.
 * @throws Error if value is undefined, null or empty. 
 */
Table.prototype.getRowWithFieldValue = function(fieldName, value) {
	assertDefined(fieldName, "FieldName is undefined, null or empty: " + fieldName);
	assertDefined(value, "Value is undefined, null or empty: " + value);
	
	for (var i = 0; i < this.rows.length; i++) {
		var field = getFieldFromRow(fieldName, this.rows[i]);
		if (value == field.getValue()) {
			return this.rows[i];
		}
	}
	return null;
}


/**
 * Get the first row index for the row that has a given value for a given field, 
 * -1 if the now row with the the given field value exists. 
 * @param fieldName The name of the field.
 * @param fieldValue The value to search for. 
 * @return The first row whos value for the field with fieldName equals fieldValue.
 * @throws Error if fieldName is undefined, null or empty.
 * @throws Error if value is undefined, null or empty.
 */
Table.prototype.getRowIndexFieldValue = function(fieldName, value) {
	assertDefined(fieldName, "FieldName is undefined, null or empty: " + fieldName);
	assertDefined(value, "Value is undefined, null or empty: " + value);
	
	for (var i = 0; i < this.rows.length; i++) {
		var field = getFieldFromRow(fieldName, this.rows[i]);
		if (value == field.getValue()) {
			return i;
		}
	}	
	return -1;
}


/**
 * Get a field with a given name from a row.
 * @param name The name of the field.
 * @param row The row to get the field from.
 * @return The field, null if no field with name found in the row.
 * @throws Error if the name is undefined, null or empty.
 * @throws Error if row is undefined or null.
 */
function getFieldFromRow(name, row) {
	assertNotEmpty(name, "Name is undefined, null or empty");
	assertDefined(row, "Row is undefined or null");

	var len = row.length;
	for (var i = 0; i < len; i++) {
		if (row[i].getName && name.equalsIgnoreCase(row[i].getName())) {
			return row[i];
		}
	}	
	return null;
}


/**
 * Get a html table cell with a given name from a html table row.
 * @param name The name of the field.
 * @param row The html table row to get the field from.
 * @return The field, null if no field with name found in the row.
 * @throws Error if the name is undefined, null or empty.
 * @throws Error if row is undefined or null.
 */
function getHtmlFieldFromRow(name, row) {
	assertNotEmpty(name, "Name is undefined, null or empty");
	assertDefined(row, "Row is undefined or null");

	var len = row.cells.length;
	for (var i = 0; i < len; i++) {
		if (name.equalsIgnoreCase(row.cells[i].getAttribute("name"))) {
			return row.cells[i];
		}
	}	
	return null;
}


/**
 * Gets the row containing a field.
 * @param field The field to get the row for.
 * @return The row, null if no row contains the field.
 * @throws Error if the field is undefined or null.
 */
Table.prototype.getRowFromField = function(field) {
	assertDefined(field, "Field is undefined or null");
	
	var fieldName = field.getName();	
	var len = this.rows.length;
	for (var i = 0; i < len; i++) {
		var tempField = getFieldFromRow(fieldName, this.rows[i]);
		if (field == tempField) {
			return this.rows[i];
		}
	}
	return null;
}


/**
 * Gets the row from the specified html td tag.
 * @param td The td tag from which to get the row.
 */
Table.prototype.getRowFromTd = function(td) {
	assertExists(td, "Td element is undefined or null");
	
	for (var i = 0; i < this.getTableCode().rows.length; i++) {
		if (td.getAttribute("name") != null) {
			var cell = getHtmlFieldFromRow(td.getAttribute("name"), this.getTableCode().rows[i]);
			if (cell == td) {
				return this.rows[i];
			}
		}
	}
	
	return null;
}


/**
 * Gets the html row which contains a specified td tag. 
 * @param td The td tag fro which to get the html row. 
 * @return The html row found, null if the td is not in any rows of the table. 
 * @throws Error if td is undefined or null.
 */
Table.prototype.getHtmlRowFromTd = function(td) {
	assertExists(td, "Td element is undefined or null: " + td);
	for (var i = 0; i < this.getTableCode().rows.length; i++) {
		if (td.getAttribute("name") != null) {
			var cell = getHtmlFieldFromRow(td.getAttribute("name"), this.getTableCode().rows[i]);
			if (cell == td) {
				return this.getTableCode().rows[i];
			}
		}
	}
	return null;
}


/**
 * Gets the row containing a given text for a field.
 * @param text The text for the field.
 * @param fieldName The name of the field the text is for.
 * @return The row with text in the field with name fieldName, null if not found.
 * @throws Error if text if undefined, null or empty.
 * @throws Error if fieldName if undefined, null or empty.
 */
Table.prototype.getHtmlRowWithFieldText = function(text, fieldName) {
	assertNotEmpty(text, "Text to search for is undefined, null or empty: " + text);
	assertNotEmpty(fieldName, "FieldName is undefined, null or empty: " + fieldName);
	
	var rows = this.getTableCode().rows;
	var length = rows.length;
	for (var i = 0; i < length; i++) {
		var cell = getHtmlFieldFromRow(fieldName, rows[i]);
		if (exists(cell) && getTekstValue(cell) == text) {
			return rows[i];
		}
	}
	return null;
}


/**
 * Gets the index of a row a field is in.
 * @param field The field to get the row index for.
 * @return The row index, -1 if no row containing this field is found.
 * @throws Error if field is undefined or null.
 */
Table.prototype.getRowIndex = function(field) {
	assertDefined(field, "Field to get row index for is undefined or null: " + field);

	var fieldName = field.getName();	
	var len = this.rows.length;
	for (var i = 0; i < len; i++) {
		var tempField = getFieldFromRow(fieldName, this.rows[i]);
		if (field == tempField) {
			return i;
		}
	}
	return -1;
}


/**
 * Gets the index of a html row. 
 * @param row The html row to get the index for. 
 * @return The index of the html row, -1 if not found. 
 * @throws Error if row is undefined or null. 
 */
Table.prototype.getHtmlRowIndex = function(row) {
	assertExists(row, "Row is undefined null or empty");
	var rows = this.getTableCode().rows;
	for (var i = 0; i < rows.length; i++) {
		if (rows[i] == row) {
			return i;
		}
	}
	return -1;
}


/**
 * Get the first row index for the row that has some given values for some given fields, 
 * -1 if the now row with the the given field values exists. 
 * @param fieldNames The name of the fields.
 * @param fieldValues The values to search for. 
 * @return The first row whos value for the field with fieldName equals fieldValue.
 */
Table.prototype.getRowIndexFieldValues = function(fieldNames, fieldValues) {
	var length = this.rows.length;
	for (var i = 0; i < length; i++) {
		// for each row go check all the specified fields
		for (var j = 0; j < fieldNames.length; j++) {
			var field = getFieldFromRow(fieldNames[j], this.rows[i]);
			if (!isDefined(field) || isNull(field)) {
				break;
			}
			if (field.getValue() != fieldValues[j]) {
				break;
			}
			if (j == fieldNames.length -1) {
				return i;
			}
		}
	}	
	return -1;
}


/**
 * Gets the row count. This counts the rows in memory not the html rows. 
 * @return The rows in the table.
 */
Table.prototype.getRowCount = function() {
	return this.rows.length;
}


/**
 * Gets a specified data row.
 * @param index The index of the row to get. 
 * @return The row for the given index, null if there is no row for the given index. A row 
 * is array containing the fields in the row.
 * @throws Error if index is negative.  
 */
Table.prototype.getRow = function(index) {
	assertNotNegative(index, "The index of the row to get is negative");
	if (index < this.rows.length) {
		return this.rows[index];
	}	
	return null;
}


/**
 * Gets a specified html row.
 * @param index the index of the row to get. 
 * @return The row for the given index, null if there is no row the given index. 
 * @throws Erorr if index is negative.
 */
Table.prototype.getHtmlRow = function(index) {
	assertNotNegative(index, "The index of the row to get is negative");
	var rows = exists(this.getTableCode()) ? this.getTableCode().rows : null;
	if (exists(rows) && index < rows.length) {
		return rows[index];
	}
	return null;	
}


/**
 * Gets the header row. 
 * @return The header row, not the html code of the header. 
 */
Table.prototype.getHeaderRow = function() {
	return this.header;
}


// columns


/**
 * Returns the column with a given name.
 * @param name The name of the column to get.
 * @return The column, undefined if no column with this name exists.
 */
Table.prototype.getColumn = function(name) {
	var len = this.columnOrder.length;
	for (var i = 0; i < len; i++) {
		if (name.equalsIgnoreCase(this.columnOrder[i].getName())) {
			return this.columnOrder[i];
		}
	}
	return undefined;
}


/**
 * Returns all visible columns.
 */
Table.prototype.getVisibleColumns = function() {
	return this.visibleColumns;
}


/**
 * Returns all columns having a right-click menu.
 */
Table.prototype.getRightClickMenuColumns = function() {
	return this.rightClickMenuColumns;
}


/**
 * Add columns to the table. For each column there are four arguments needed, the
 * name of the column, the text to be shown as the header for the column, the width of the column
 * and the text to be shown for the column in the field chooser. If the field chooser text argument
 * is undefined, null or empty the header text of the column is shown in the field chooser. 
 * The column name must be unique for this column. Data added to this table through setData
 * must have the same element name as the column to be shown.
 * @throws Error if the number of arguments is not even. For each column to add four 
 * arguments are required.
 */
Table.prototype.addColumns = function() {
	if (arguments.length % 4 != 0) {
		alert("wrong number of arguments");
	}
	
	var len = arguments.length;
	for (var i = 0; i < len; i++) {
		var name = arguments[i];
		i++;
		var headerText = arguments[i];
		i++;
		var fieldChooserText = arguments[i];
		i++;
		var width = arguments[i];
		this.addColumn(name, headerText, width, fieldChooserText);
	}
}


/**
 * Add a column to the table. The columns are painted in the order they are added.
 * @param name The name of the column.
 * @param headerText The text to be shown as header for this column.
 * @param width The width of this column.
 * @param fieldChooserText The text to show for this column in the field chooser. If this is 
 * undefined, null or empty the header text will be shown in the field chooser. 
 * @throws Error if id is undefined, null or empty.
 * @throws Error if headerText is undefined, null or empty.
 * @throws Error if width is negative.
 */
Table.prototype.addColumn = function (name, headerText, width, fieldChooserText) {
	assertNotEmpty(name, "Name is undefined, null or empty");
	assertExists(headerText, "HeaderText is undefined or null");
	assertNotNegative(width, "Width to set for the column is negative");
	
	var column = new Column(name, width, headerText, fieldChooserText);
	this.columnOrder[this.columnOrder.length] = column;
		
	var headerField = new Field(name, column.getHeaderText());
	this.header[this.header.length] = headerField;
}


/**
 * Set the names of the columns that should be visible.
 */
Table.prototype.setVisibleColumns = function () {
	var len = arguments.length;
	for (var i = 0; i < len; i++) {
		if (isEmpty(arguments[i])) {
			continue;
		}
		this.setVisibleColumn(arguments[i]);
	}
}


/**
 * Set a column to be visible.
 * @param name The name of the column.
 * @throws Error if name is undefined, null or empty.
 */
Table.prototype.setVisibleColumn = function(name) {
	assertNotEmpty(name, "Name is undefiend, empty or null");
	var column = this.getColumn(name);
	if (this.isVisibleColumn(name)) {
		// the column is already visible
		return;
	}
	if (isDefined(column) && !isNull(column)) {
		this.visibleColumns[this.visibleColumns.length] = column;
		column.setVisibility(true);
	}
}


/**
 * Sets columns as invisible.
 * @param Optional arguments, the columns to set invisible.
 */
Table.prototype.setInvisibleColumns = function() {
	var len = arguments.length;
	for (var i = 0; i < len; i++) {
		if (isDefined(arguments[i]) && !isNull(arguments[i])) {
			this.setInvisibleColumn(arguments[i]);
		}	
	}
}


/**
 * Set a column invisible column.
 * @param name The name of the column to set invisible.
 * @throws Error if name is undefined, empty or null.
 */
Table.prototype.setInvisibleColumn = function(name) {
	assertNotEmpty(name, "Name is undefined, empty or null");

	var len = this.visibleColumns.length;
	for (var i = 0; i < len; i++) {
		if (this.visibleColumns[i].getName() == name) {
			this.visibleColumns[i] = null;
			this.getColumn(name).setVisibility(false);
		}
	}
	this.visibleColumns = removeNullEntries(this.visibleColumns);
}


/**
 * Returns true if a given field is one of the visible fields.
 * @param columnName The name of the column to check.
 * @throws Error if the field is undefined or null.
 */
Table.prototype.isVisibleColumn = function (columnName) {
	assertNotEmpty(columnName, "ColumnName is undefined, null or empty: " + columnName);
	var len = this.visibleColumns.length;
	for (var i = 0; i < len; i++) {
		if (this.visibleColumns[i].getName() == columnName) {
			return true;
		}
	}
	return false;
}


/**
 * Get the html code of all the data cells in one column.
 * @param name The name of the column to get the html fields for.
 * @return A array containing all the table cells in the column with name, not including the header cell. 
 * @throws Error if name is undefined null or empty.
 */
Table.prototype.getColumnDataHtmlCells = function(name) {
	assertDefined(name, "Name is undefined, null or empty: " + name);
	
	// go through all rows and get the cell with the correct name
	// starting at row 1 the first data row after the header
	var tableCode = this.getTableCode();
	var len = tableCode.rows.length;
	var dataCells = new Array;
	for (var i = 1; i < len; i++) {
		var row = tableCode.rows[i];
		var cell = getElementWithAttribute(row, name, "name", "td");
		if (isDefined(cell) && !isNull(cell)) {
			dataCells[dataCells.length] = cell;
		}
	}
	
	return dataCells;
}


/**
 * Get the html code of all the cells in one column.
 * @param name The name of the column to get the html fields for.
 * @return A array containing all the table cells in the column with name, including the header cell. 
 * @throws Error if name is undefined null or empty.
 */
Table.prototype.getAllColumnHtmlCells = function(name) {
	assertDefined(name, "Name is undefined, null or empty: " + name);
	var dataCells = new Array;
	// add the header cell for this column
	var headerTable = this.getHeaderTableCode();

	if (isDefined(headerTable) && !isNull(headerTable)) {
		dataCells[0] = getElementWithAttribute(headerTable, name, "name", "td");
	}

 	// go through the data table and all cells in the column	
 	var tableCode = this.getTableCode();
 	if (isNull(tableCode) || !isDefined(tableCode)) {
 		return dataCells;
 	}

	var len = tableCode.rows.length;
	for (var i = 0; i < len; i++) {
		var row = tableCode.rows[i];
		var cell = getElementWithAttribute(row, name, "name", "td");
		if (isDefined(cell) && !isNull(cell)) {
			dataCells[dataCells.length] = cell;
		}
	}
	return dataCells;
}


/**
 * Sets the text alignment on the columns. 
 * Valid parameters are "left", "middle" and "right".
 */
Table.prototype.setTextAlignments = function() {
	if (arguments.length % 2 != 0) {
		alert("wrong number of arguments");
	}

	var len = arguments.length;
	for (var i = 0; i < len; i++) {
		var name = arguments[i];
		i++;
		var value = arguments[i];
		this.setTextAlignment(name, value);
	}
}


/**
 * Sets the text aligmnent on the given column.
 */
Table.prototype.setTextAlignment = function(name, value) {
	assertNotEmpty(name, "Name is undefined, null or empty");
	assertNotEmpty(value, "Value is undefined, null or empty");
	
	var column = this.getColumn(name);
	if (isDefined(column) && !isNull(column)) {
		column.setAlignment(value);
	}
}


// inserting data into table


/**
 * Set data into the table based on a xmlDocument. To 
 * show the data on screen call paint().
 * @param xmlDoc The data in xml format.
 * @param tagName The name of the tag the data for each row is in.
 * @param throws Error if xmlDoc is undefined or null.
 * @param throws Error if tagName is undefined, null or empty.
 */
Table.prototype.setData = function (xmlDoc, tagName) {
	assertDefined(xmlDoc, "XmlDoc is undefined or null");
	assertNotEmpty(tagName, "TagName is undefined, null or empty");
	var rows = xmlDoc.getElementsByTagName(tagName);
	
	var len = rows.length;
	for (var i = 0; i < len; i++) {
		var row = this.parseXmlRow(rows[i]);
		this.rows[this.rowCounter] = row;
		this.rowCounter++;
	}
	
	if (!this.sortable) {
		return;
	}
	
	// sort the table
	if (exists(this.columnToSort)) {
		this.sort(this.columnToSort);
	}
	if (this.reversed) {
		this.reverseColumn(this.columnToSort);
	}
}


/**
 * Parses some xml data into a array of fields.
 * For each element in the xml data there is created a field in the table
 * The name of the field is the name of the xml data, and the fields value
 * is set based on the elements text value.
 * @param data The xml data to create the row from.
 * @param Optional The unique id for each row.
 * @return The array containing the fields.
 * @throws Error if data is null or undefined.
 */
Table.prototype.parseXmlRow = function (data) {
	assertDefined(data, "Data is not defined or null");
	if (!data.hasChildNodes()) {
		// the xml data has no entries
		return;
	}
	
	var row = new Array();
	var counter = 0;
	var len = data.childNodes.length;
	for (var i = 0; i < len; i++) {
		if (data.childNodes[i].nodeType != 1) {
			// the node is not a text node
			continue;
		}
		if (!isNull(data.childNodes[i].firstChild) && data.childNodes[i].firstChild.nodeType == 1) {
			// the child of the node is not a element node, also parse this elements
			// fields
			var newRow = this.parseXmlRow(data.childNodes[i]);
			arrayCopy(newRow, row);
			counter += newRow.length;
			continue;
		}
		
		var fieldValue = "";
		if (!isNull(data.childNodes[i].firstChild)) {
			fieldValue = data.childNodes[i].firstChild.nodeValue;
		}
		var field = new Field(data.childNodes[i].nodeName, fieldValue);
		// add the field
		row[counter] = field;
		counter++;
	}
	return row;
}


/**
 * Clears all the data in the table.
 */
Table.prototype.clear = function() {
	// clear all the rows
	this.rows = new Array();
	this.rowCounter = 0;
}


// resizing


/**
 * Set the size of the table.
 * @param width The width to set.
 * @param height The height to set.
 * @throws Error if width is negative.
 * @throws Error if height is negative.
 */
Table.prototype.setSize = function(width, height) {
	assertNotNegative(width, "The width to set is negative: " + width);
	assertNotNegative(height, "The height to set is negative: " + height);
	
	this.height = height;
	this.width = width;
	
	var tableContainer = this.getTableContainerCode();
	if(!isDefined(tableContainer) || isNull(tableContainer)) {
		tableContainer = this.createContainer();
	}
	
	tableContainer.style.width = width + "px";
	tableContainer.style.height = height + "px";
}


/**
 * Returns the width set for the table.
 */
Table.prototype.getWidth = function() {
	return this.width;
}


/**
 * Returns the height of the table. 
 */
Table.prototype.getHeight = function() {
	return this.height;
}


/**
 * Adjusts the size of the table and table cells if scrollbars are shown or hidden.
 */
Table.prototype.adjustSize = function() {
	var isVerticalScrollBarVisible = this.isVerticalScrollBarVisible();
	// if table is updated when it is not visible the scrollbar might not be detected correct
	if (isVerticalScrollBarVisible && !this.verticalScrollBar && this.isVisible()) {
		// the vertical scroll bar has been shown
		this.verticalScrollBarShown();
		this.verticalScrollBar = true;	
		return;
	}
	else if (!isVerticalScrollBarVisible && this.verticalScrollBar && this.isVisible()) {
		this.verticalScrollBarHidden();
		this.verticalScrollBar = false;	
		return;
	}
}


/**
 * A vertical scrollbar has been shown, adjust the size of the table.
 */
Table.prototype.verticalScrollBarShown = function() {
	this.resizeAllColumns(Table.BROWSER_SCROLLBAR_WIDTH * -1);
	// this is needed in Opera to avoid problems with the 
	// cell width when the scrollbar is shown and hidden
	var tableContainer = this.getTableContainerCode();
	tableContainer.style.display = "none";
	tableContainer.style.display = "";
}



/**
 * A vertical scrollbar has been hidden, adjust the size of the table.
 */
Table.prototype.verticalScrollBarHidden = function() {
	this.resizeAllColumns(Table.BROWSER_SCROLLBAR_WIDTH);
	// this is needed in Opera to avoid problems with the 
	// cell width when the scrollbar is shown and hidden
	var tableContainer = this.getTableContainerCode();
	tableContainer.style.display = "none";
	tableContainer.style.display = "";
}
 

/**
 * Handles a scroll event.
 * @param event The event to handle.
 */
Table.prototype.handleScrollEvent = function(event) {
	if (!eventsAvailable()) {
		return;
	}	
	if (this.scrollTimer != 0 && (new Date()).getTime() < this.scrollTimer + Table.FIRE_SCROLL_EVENT_THRESHOLD) {
		this.scrollTimer = 0;
		return;
	}
	
	var eventData = new Array();
	eventData["id"] = this.id;
	eventData["scrollTop"] = this.getDataTableContainer().scrollTop; 
	eventData["maxScrollHeight"] = this.getMaxScrollHeight(); 
	getEventServer().notifyListeners(new Event("scrolled", eventData));
}


/**
 * Processes scroll events. 
 * @param event The event to process. 
 */
function processScrollEventTable(event) {
	if (!this.isScrollObject(event.getData()["id"])) {
		// the chart only cares about events from its table, none of the others
		return;
	}
	
	this.scrollTimer = (new Date()).getTime();
	var factor = this.getMaxScrollHeight() / event.getData()["maxScrollHeight"];
	this.getDataTableContainer().scrollTop = Math.round(event.getData()["scrollTop"] * factor);
}


/**
 * Gets the maximum scroll height for the table. When the scrolltop is at this value
 * the table is scrolled all the way to the bottom. 0 if no scrollbar is visible.   
 */
Table.prototype.getMaxScrollHeight = function() {
	// the height might be 0 if the table is not visible
	var tableHeight = this.getTableCode().offsetHeight;
	if (tableHeight <= 0 && this.maxScrollHeight == 0) {
		// the table is not visible and no previous max height has been calculated
		this.maxScrollHeight = 0;
	}
	else if(tableHeight > 0) {
		this.maxScrollHeight = this.getTableCode().offsetHeight - parseInt(this.getDataTableContainer().style.height);
	}
	return this.maxScrollHeight;
}


/**
 * Returns true if the scrollbars are visible, false otherwise.
 */
Table.prototype.isVerticalScrollBarVisible = function() {
	if (!exists(this.getTableCode())) {
		return false;
	}
	var tableHeight = this.getTableCode().offsetHeight;
	var tableContainerHeight = this.getDataTableContainer().offsetHeight;
	if (tableHeight > tableContainerHeight) {
		return true;
	}
	
	var headerHeight = this.getHeaderTableCode().offsetHeight;
	var tableCode = this.getTableCode();
	if (tableCode.rows.length == 0) {
		// no data rows are present
		return false;
	}
	
	// in safari we can not measure the height of a row with offsetHeight on the row itself
	// we have to do it on one of the rows cells
	var dataRowHeight = 0;
	if (isDefined(tableCode.rows[0].cells[0])) {
		dataRowHeight = tableCode.rows[0].cells[0].offsetHeight
	}
	var totalHeight = (dataRowHeight * tableCode.rows.length) + headerHeight;
	if (totalHeight > this.getTableContainerCode().offsetHeight) {
		return true;
	}
	return false;
}


/**
 * Returns true if the table is visible, false otherwise. 
 */
Table.prototype.isVisible = function() {
	if (this.painted && this.getTableCode().offsetHeight == 0) {
		return false;
	}
	return true;
}


/**
 * Resizes the table.
 * @param deltaHeight The height to change the size of the table with.
 * @param deltaWidth The width to change the size of the table with.
 * @throws Error if deltaHeight is undefined.
 * @throws Error if deltaWidth is undefined.
 */
Table.prototype.resizeTable = function(deltaHeight, deltaWidth) {
	assertDefined(deltaHeight, "DeltaHeight is undefined");
	assertDefined(deltaWidth, "DeltaWidth is undefined");

	// resize containers
	this.resizeContainers(deltaHeight, deltaWidth);
	
	// resize columns in the table
	if (deltaWidth != 0) {
		this.resizeAllColumns(deltaWidth);
	}

	this.adjustSize();
}


/**
 * Resizes the table containers.
 * @param deltaHeight The height to change the size of the table with.
 * @param deltaWidth The width to change the size of the table with.
 * @throws Error if deltaHeight is undefined.
 * @throws Error if deltaWidth is undefined.
 */
Table.prototype.resizeContainers = function(deltaHeight, deltaWidth) {
	assertDefined(deltaHeight, "DeltaHeight is undefined");
	assertDefined(deltaWidth, "DeltaWidth is undefined");
	
	var container = this.getTableContainerCode();
	var dataTableContainer = this.getDataTableContainer();
	var headerTableContainer = this.getHeaderContainerCode();
	if (isDefined(container) && !isNull(container)) {
		resizeHtmlElement(container, deltaHeight, deltaWidth);
	}
	if (isDefined(dataTableContainer) && !isNull(dataTableContainer)) {
		resizeHtmlElement(dataTableContainer, deltaHeight, deltaWidth);
	}
	if (isDefined(headerTableContainer) && !isNull(headerTableContainer)) {
		resizeHtmlElement(headerTableContainer, 0, deltaWidth);
	}
	this.width += deltaWidth;
	this.height += deltaHeight;
}


/**
 * Resizes the width of all columns by deltaWdith. Deltawidth is evenly distributed among the table columns. 
 * @param deltaWidth The width to add to the columns width. 
 */
Table.prototype.resizeAllColumns = function(deltaWidth) {
	if (this.getColumnCountWidth() == 0) {
		// all the cells have size 0 no cells are to be resized
		return;
	}
	
	// all the cells need to be resized with a total of deltawidth, deltawidth is distributed equally 
	// on all the cells. No cell can be made smaller than 0. The amount of pixels of the new cell width
	// below 0 must be distributed to the other cells.
	var extra = 0;
	var resizeColumns = this.getResizeColumns(deltaWidth);
	
	// calculate the delta width for each cell
	var columnDeltaWidth = roundDown(deltaWidth / resizeColumns.length);
	// because of the rounding the last column might need to be resized some extra pixels
	var extraRoundingWidth = deltaWidth - (columnDeltaWidth * resizeColumns.length);
	var resizeColumns = this.getResizeColumns(deltaWidth);
	for (var i = (resizeColumns.length -1); i >= 0; i--) {
		var column = resizeColumns[i];
		extra += columnDeltaWidth - this.resizeColumn(column, columnDeltaWidth); 
	}

	// because of rounding there might be som extra width, we resize the largest column 
	// with the extra width
	extra += extraRoundingWidth - this.resizeColumn(this.getLargestWidthColumn(), extraRoundingWidth);

	// because columns can not be made smaller than zero there can still be some extra width
	// the extra width is distributed on each cell with a size larger than 0, if the extra width
	// not 0 per column
	resizeColumns = this.getResizeColumns(deltaWidth);
	if (resizeColumns.length > 0) {
		var extraColumnWidth = roundDown(extra / resizeColumns.length);
		if (extraColumnWidth > 0 || extraColumnWidth < 0) {
			this.resizeAllColumns(extra);
			return;
		}
	}

	// keep on going resizing the largest columns until there are no more extra pixels
	while(true) {
		if (extra == 0) {
			break;
		}
		if (this.getColumnCountWidth() == 0) {
			// all columns have size 0
			break;
		}
		
		extra -= this.resizeColumn(this.getLargestWidthColumn(), extra);
	}
}


/**
 * Get all the columns that are to be resized given a deltaWidth.
 * @param deltaWidth The delta width to resize the columns with.
 */
Table.prototype.getResizeColumns = function(deltaWidth) {
	if (deltaWidth > 0) {
		// all columns are to resized
		return this.getVisibleColumns();
	}
	
	// if deltaWidth is negative only columns with a size larger than 0 are to be resized
	var resizeColumns = new Array();
	for (var i = 0; i < this.columnOrder.length; i++) {
		if (this.columnOrder[i].getWidth() > 0 && this.isVisibleColumn(this.columnOrder[i].getName())) {
			resizeColumns[resizeColumns.length] = this.columnOrder[i];
		}
	}
	return resizeColumns;
}


/**
 * Rounds a number down. When the number is negative it is rounded up (closer to zero).
 * @param number The number to round.
 * @return The rounded numver.
 */
function roundDown(number) {
	if (number < 0) {
		return Math.ceil(number);
	}
	else {
		return Math.floor(number);
	}
}


/**
 * Resizes a given column.
 * @param column The column to resize.
 * @param deltaWidth The width to add to the column width. 
 * @return The amount of pixels the table was actually resized with.
 */
Table.prototype.resizeColumn = function(column, deltaWidth) {
	var extra = 0;
	var resized = deltaWidth;
	if ((column.getWidth() + deltaWidth) < 0) {
		extra += (column.getWidth() + deltaWidth) * -1;
		resized = column.getWidth() * -1;
	}
	var columnCells = this.getAllColumnHtmlCells(column.getName());
	column.setWidth(column.getWidth() + deltaWidth);
	this.resizeCells(columnCells, 0, deltaWidth);
	return resized;
}


/**
 * Get the column with the largest width.
 * @return The column with the largest width. 
 */
Table.prototype.getLargestWidthColumn = function() {
	var largest;
	var length = this.columnOrder.length;
	for (var i = 0; i < length; i++) {
		if(!this.isVisibleColumn(this.columnOrder[i].getName())) {
			continue;
		}
		if (!isDefined(largest)) {
			largest = this.columnOrder[i]
		}
		if (this.columnOrder[i].getWidth() > largest.getWidth()) {
			largest = this.columnOrder[i];
		}
	}
	return largest;
}


/**
 * Get the count of columns in the table that have a width larger than 0.
 */
Table.prototype.getColumnCountWidth = function() {
	var counter = 0;
	
	var length = this.columnOrder.length;
	for (var i = 0; i < length; i++) {
		if (this.columnOrder[i].getWidth() > 0) {
			counter ++;
		}
	}
	return counter;
}


/**
 * Resizes some given table cells.
 * @param cells The table cells to resize.
 * @param deltaHeight The amount of pixels to change the height.
 * @param deltaWidth The amount of pixels to change the width.
 */
Table.prototype.resizeCells = function(cells, deltaHeight, deltaWidth) {
	var len = cells.length;
	for (var i = 0; i < len; i++) {
		this.resizeCell(cells[i], deltaHeight, deltaWidth);
	}
}



/**
 * Resizes a given table cell.
 * @param cell The table cell to resize.
 * @param deltaHeight The amount of pixels to change the height.
 * @param deltaWidth The amount of pixels to change the width.
 */
Table.prototype.resizeCell = function(cell, deltaHeight, deltaWidth) {
	// resize the div inside the cell and the cell itself
	resizeHtmlElement(cell.firstChild, deltaHeight, deltaWidth);
	resizeHtmlElement(cell, deltaHeight, deltaWidth);
}


// creating html code for the table


/**
 * Delete a row with a given index. The row is deleted both from memory and the html code. 
 * @param index The index of the row. 
 * @throws Error is row is negative or larger than the row count. 
 */
Table.prototype.deleteRow = function(index) {
	assertNotNegative(index, "The index is negative or undefined: " + index);
	if (index > this.rows.length - 1) {
		throw new Error("Index larger than row count: " + index);
	}
	
	// delete the html row
	this.getTableCode().deleteRow(index);
	// delete the row from memory
	this.rows[index] = null;
	this.rows = removeNullEntries(this.rows);
}


/**
 * Set the class for the table.
 * @paran className The name of the class to set.
 * @throws Error if className is undefined, null or empty.
 */
Table.prototype.setClass = function(className) {
	assertNotEmpty(className, "The class name to set is undefined, null or empty: " + className);
	
	this.className = className;
	var tableCode = this.getTableCode();
	if (isDefined(tableCode) && !isNull(tableCode)) {
		setClass(tableCode, className);
	}
}


/**
 * Creates the container for the table. The container contains both the header table
 * and the data table.
 * @return The created container.
 */
Table.prototype.createContainer = function() {
	var container = document.createElement("div");
	container.setAttribute("id", "tableContainer" + this.id);
	this.parentElement.appendChild(container);
	return container;
}


/**
 * Creates the header table.
 */
Table.prototype.createHeaderTable = function() {
	var container = this.getTableContainerCode();
	if (!isDefined(container) || isNull(container)) {
		container = this.createContainer();
	}
	
	// create the header table
	var divContainer = document.createElement("div");
	divContainer.style.overflow = "auto";
	divContainer.style.width = this.width + "px";
	divContainer.setAttribute("id", "headerTableContainer" + this.id);
	var headerTable = document.createElement("table");
	headerTable.setAttribute("id", "headerTable" + this.id);
	if (isDefined(this.className) && !isNull(this.className)) {
		setClass(headerTable, this.headerTableClass);
	}
	divContainer.appendChild(headerTable);
	container.appendChild(divContainer);
}


/**
 * Creates the data table.
 */
Table.prototype.createDataTable = function() {
	var container = this.getTableContainerCode();
	if (!isDefined(container) || isNull(container)) {
		container = this.createContainer();
	}
	
	// create the data table
	var dataTableContainer = document.createElement("div");
	dataTableContainer.setAttribute("id", "dataTableContainer" + this.id);
	dataTableContainer.style.overflow = "auto";
	var headerHeight = this.getHeaderTableCode().offsetHeight;
	dataTableContainer.style.height = (this.height - headerHeight) + "px";
	dataTableContainer.style.width = this.width + "px";
	container.appendChild(dataTableContainer);
	var thisObj = this;
	dataTableContainer.onscroll = function(event) {
		thisObj.handleScrollEvent(event);
	};
	
	var dataTable = document.createElement("table");
	if (isDefined(this.className) && !isNull(this.className)) {
		setClass(dataTable, this.className);	
	}
	dataTable.setAttribute("id", this.id);
	dataTableContainer.appendChild(dataTable);
}


/**
 * Paints this table based on the rows in memory. All
 * the old html code of the table is deleted.
 */
Table.prototype.paint = function() {
	this.clearHtmlTable();
	this.painted = true;
	this.paintHeader();
	this.paintDataRows();
	
	// adjust the size of the table cells if scrollbars are visible
	this.adjustSize();	
	
}


/**
 * Returns true if the table has been painted, false otherwise.
 */
Table.prototype.isPainted = function() {
	return this.painted;
}


/**
 * Updates the html table according to the data in memory.
 */
Table.prototype.updateHtmlTable = function() {
	if (!this.painted) {
		// the table has not been painted yet. 
		this.paint();
		return;
	}
	
	var counter = this.updateTableRows();
	if (counter < (this.getTableCode().rows.length)) {
		// some of the rows in the html table are not in memory
		// any more, delete them
		this.deleteRows(counter);
	}
	for (var i = counter; i < this.rows.length; i++) {
		// we have more rows in memory than in the html table, add them to the html table
		var tableCode = this.getTableCode();
		var row = tableCode.insertRow(i);
		this.createRowCode(this.rows[i], row);
		this.addRowClasses(row, i);		
	}
	// adjust the size of the table cells if scrollbars are visible
	this.adjustSize();
}


/**
 * Updates the rows in the html table according to the data rows in memory.
 * @return The number of rows updated.
 */
Table.prototype.updateTableRows = function() {
	var counter = 0;
	var length = this.rows.length;
	var table = this.getTableCode();
	var dataRow, htmlRow;

	for (var i = 0; i < length && table.rows.length >= 1 && i < table.rows.length; i++) {
		dataRow = this.rows[i];
		htmlRow = table.rows[i];
		this.updateTableRow(dataRow, htmlRow);
		counter++;
		// explicit deallocation to prevent memory leak in IE
		dataRow = null;
		htmlRow = null;
	}
	
	// explicit deallocation to prevent memory leak in IE
	dataRow = null;
	htmlRow = null;
	table = null;
	return counter;
}


/**
 * Deletes all the rows in the html table starting from a index.
 * @param index All rows including the row at this index will be deleted.
 */
Table.prototype.deleteRows = function(index) {
	var table = this.getTableCode();
	var tableLength = table.rows.length;
	for (var i = tableLength -1; i >= index; i--) {
		table.deleteRow(i);
	}
}


/**
 * Updates a html row of the table according to the data in the specified data row.
 * @param dataRow The array containing the data to insert into the html row.
 * @param htmlRow The html row that is to be update with the data in dataRow.
 */
Table.prototype.updateTableRow = function(dataRow, htmlRow) {
	var htmlCellCounter = 0;
	var dataField, htmlCell, fieldName;
	
	for (var i = 0; i < this.columnOrder.length; i++) {
		fieldName = this.columnOrder[i].getName();
		if (this.isVisibleColumn(fieldName)) {
			dataField = getFieldFromRow(fieldName, dataRow);
			htmlCell = htmlRow.cells[htmlCellCounter]
			htmlCellCounter++;
			this.updateTableCell(dataField, htmlCell, htmlRow);
			// explicit deallocation to prevent memory leak in IE
			dataField = null;
			htmlCell = null;
		}
	}
	
	// explicit deallocation to prevent memory leak in IE
	dataField = null;
	htmlCell = null;
}


/**
 * Updates a html table cell of the table according to the data in the specified field.
 * @param dataField The field containing the data to update the cell with.
 * @param htmlCell The html cell that is to be updated with the data in dataField.
 * @param htmlRow The row the cell to update is in.
 */
Table.prototype.updateTableCell = function(dataField, htmlCell, htmlRow) {
	if (dataField == null) {
		var dataValue = "";
	}
	else {
		var dataValue = dataField.getValue();
	}
	var htmlValue = getTekstValue(htmlCell, "linkRequest");
	if (dataValue == htmlValue) {
		return;
	}
	
	var newTableCell = document.createElement("td");
	this.setAlignment(newTableCell, dataField);
	htmlRow.replaceChild(newTableCell, htmlCell);
	if (isDefined(dataField) && !isNull(dataField)) {
		this.createCellCode(dataField, newTableCell, dataField.getName());
	}
	
	// explicit deallocation to prevent memory leak in IE
	newTableCell = null;
	htmlCell = null;
}


/**
 * Paints the data rows.
 */
Table.prototype.paintDataRows = function() {
	if (!isDefined(this.getTableCode()) || isNull(this.getTableCode())) {
		this.createDataTable();
	}

	var rowsLength = this.rows.length;
	var tableCode = this.getTableCode();
	for (var i = 0; i < rowsLength; i++) {
		var row = tableCode.insertRow(i);
		this.createRowCode(this.rows[i], row);		
		this.addRowClasses(row, i);
	 }
}


/**
 * Paints the header of the table.
 */
Table.prototype.paintHeader = function() {
	if (!isDefined(this.getHeaderTableCode()) || isNull(this.getHeaderTableCode())) {
		this.createHeaderTable();
	}

	var headerTable = this.getHeaderTableCode();
	var row = headerTable.insertRow(headerTable.rows.length - 1);
	
	row.setAttribute("name", "header");
	this.createRowCode(this.header, row);
	this.addHeaderClass(row);
}


/**
 * Create a table cell for each entry in a array.
 * @param data The array to create data from.
 * @param row The row to insert the table cells created.
 * @throws Error if data is null or not defined.
 * @throws Error if row is null or not defined.
 */
Table.prototype.createRowCode = function(data, row) {
	assertDefined(data, "Data is null or undefined");
	assertDefined(row, "Row is null or undefined");
	var cellCode;
	var fieldName;
	
	for (var i = (this.columnOrder.length -1); i >= 0; i--) {
		fieldName = this.columnOrder[i].getName();
		if (this.isVisibleColumn(fieldName)) {
			cellCode = row.insertCell(0);
			var field = getFieldFromRow(fieldName, data);
			this.createCellCode(field, cellCode, fieldName);
			cellCode = null;	
		}
		fieldName = null;
	}
}


/**
 * Create a table cell for a field.
 * @param field The field to create a cell for.
 * @param cellCode The code for the cell.
 * @param columnName The name of the colum the cell is in.
 */
Table.prototype.createCellCode = function (field, cellCode, columnName) {
	cellCode.setAttribute("name", columnName);
	if (field == null) {
		// no data for this field exists
		cellCode.appendChild(document.createTextNode(""));
	}
	else {
		this.setAlignment(cellCode, field);
		var nobr = document.createElement("nobr");
		this.insertFieldValue(field, nobr, cellCode);
		cellCode.appendChild(nobr);
		this.applyProperties(cellCode, field);
	}

	// check if the column should have a right-click menu connected to it
	if (exists(field) && this.isRightClickMenuColumn(field.getName()) && !this.isHeader(field)) {
		cellCode.oncontextmenu = this.rightClickMenuFunction;
		
		// handle events if left-click and a key is press, to get around 
		// a problem with Opera not supporting right-click
		cellCode.onmousedown = this.rightClickMenuFunction;
	}
	this.setCellWidth(cellCode, columnName);
}


/**
 * Inserts a field value into a given html node.
 * @param field The field to insert the value for.
 * @param node The node to insert the value into.
 */
Table.prototype.insertFieldValue = function(field, node) {
	node.appendChild(document.createTextNode(field.getValue()));
}


/**
 * Sets the width of a cell.
 * @param cellCode the html code of the cell.
 * @param columnName The name of the column the cell is in.
 */
Table.prototype.setCellWidth = function(cellCode, columnName) {
	// the content of the cell is placed inside a div tag to manage possible overflow
	var div = document.createElement("div");
	div.style.overflow = "hidden";
	if (isIE()) {
		// adds a ... when text overflows, this is a Microsoft extension
		// to the CSS standard and therefore only works in Internett Explorer
		div.style.textOverflow = "ellipsis";
	}
	appendAll(cellCode, div);
	cellCode.appendChild(div);
	
	// set the widht of the cell
	var column = this.getColumn(columnName);
	div.style.width = column.getWidth() + "px";
	cellCode.style.width = column.getWidth() + "px";
}


/**
 * Sets the alignemt in a html table cell.
 * @param cell The table cell to set the alignment in.
 * @param field The field the cell is for.
 */
Table.prototype.setAlignment = function(cell, field) {
	if (!isDefined(field) || isNull(field) || !isDefined(field.getName())) {
		return;
	}
	
	var column = this.getColumn(field.getName());
	if (!isDefined(column) || isNull(column)) {
		// no column for this field found
		return;
	}
	
	if (isDefined(column.getAlignment())) {
		cell = this.alignText(cell, column.getAlignment());
	}
}


/**
 * Applies the properties set for the field.
 * @param code The code create for the field.
 * @param field The field object to apply properties for.
 */
Table.prototype.applyProperties = function (code, field) {
	// check if the cells in this column should have a link
	if (this.isLinkField(field) && !this.isHeader(field)) {
		code = this.link(code, field);
	}
	code = this.applyFieldProperties(code, field);
	return this.applyColumnProperties(code, field);
}


/**
 * Applies the properties set for a field.
 * @param code The code create for the field.
 * @param field The field object to apply properties for.
 */
Table.prototype.applyFieldProperties = function (code, field) {
	var len = field.properties.length;
	for (var i = 0; i < len; i++) {
		if (!isDefined(Table.properties)) {
			// no property handler for the fields has been specified.
			break;
		}
		if (isDefined(Table.properties[field.properties[i]]) && !isNull(Table.properties[field.properties[i]])) {
			Table.properties[field.properties[i]](code, field, this);		
		}
	}
	return code;
}


/**
 * Applies the properties set for the column a field is in.
 * @param code The code create for the field.
 * @param field The field object to apply properties for.
 */
Table.prototype.applyColumnProperties = function (code, field) {
	var column = this.getColumn(field.getName());
	var len = column.properties.length;
	for (var i = 0; i < len; i++) {
		if (!isDefined(Table.properties)) {
			// no property handler for the fields has been specified.
			break;
		}
		if (isDefined(Table.properties[column.properties[i]]) && !isNull(Table.properties[column.properties[i]])) {
			Table.properties[column.properties[i]](code, field, this);		
		}
	}
	return code;

}


/**
 * Sets the text alignment for a field.
 * @param cellCode The cell to set alignment for.
 * @param alignment The alignment value (left, middle, right).
 */
Table.prototype.alignText = function (cellCode, alignment) {
	if (alignment == "right") {
		setClass(cellCode, "alignRight");
	}
	else if (alignment == "middle") {
		setClass(cellCode, "alignMiddle");
	}
	else {
		// default alignment is left
	}
	return cellCode;
}


/**
 * Deletes all the existing data rows from the html code.
 * The data in memory is not deleted.
 */
Table.prototype.clearHtmlTable = function() {
	this.clearHeaderTable();
	this.clearDataTable();
	this.painted = false;
}


/**
 * Clears the header table.
 */
Table.prototype.clearHeaderTable = function() {
	var headerTable = this.getHeaderTableCode();
	if (!isDefined(headerTable) || isNull(headerTable)) {
		return;
	}
	
	var len = headerTable.rows.length;
	for (var i = len-1; i >= 0; i--) {
		if (i < 0) {
			break;
		}
		headerTable.deleteRow(i);
	}
}


/**
 * Deletes all the rows in the data table.
 */
Table.prototype.clearDataTable = function() {
	var tableCode = this.getTableCode();
	if (!isDefined(tableCode) || isNull(tableCode)) {
		return;
	}
	
	var len = tableCode.rows.length;
	for (var i = len-1; i >= 0; i--) {
		if (i < 0) {
			break;
		}
		tableCode.deleteRow(i);
	}
}


/**
 * Checks if the specified field is a header field.
 */
Table.prototype.isHeader = function(field) {
	var len = this.header.length;
	for (var i = 0; i < len; i++) {
		if (this.header[i] == field) {
			return true;
		}
	}
	return false;
}


/**
 * Checks if the specifield row is the header row.
 * @param row The row to check.
 * @return True if the provided row is the header row.
 */
Table.prototype.isHeaderRow = function(row) {
	if (this.header == row) {
		return true;
	}
	return false;
}


/**
 * Creates a link around the specified cell code.
 * @param cellCode The cell code which to make a link around.
 * @param field the field the link is to be created for.
 * @return The cellCode with the created link inside.
 */
Table.prototype.link = function (cellCode, field) {
	var linkNode = document.createElement("a");
	linkNode.setAttribute("href", "#");
	// append all the cells child nodes to the link
	appendAll(cellCode, linkNode);	
	// append the link to the cell
	cellCode.appendChild(linkNode);

	// call the url creator
	if(isDefined(this.linkUrlCreator)) {
		this.linkUrlCreator.call(this, cellCode, field);
	}
	// explicit deallocation to avoid memory leak on internet explorer
	linkNode = null;
	return cellCode;
}


/**
 * Set sortable to true. Note the table is not sortable until it is paintet for the next time.
 * @param sortable If this is true the table is made sortable on the next paint, false 
 * and the table is made unsortable.
 * @throws Error if sortable is undefined.
 */
Table.prototype.setSortable = function(sortable) {
	assertDefined(sortable, "Sortable is undefined");
	
	if (sortable == true) {
		this.sortable = true;
		Table.addProperty("sortable", addSortLink);
		this.setHeaderProperty("sortable");
	}
	else {
		this.sortable = false;
	}
}


/**
 * Makes this table sortable.
 */
Table.prototype.makeSortable = function(sortFunc) {
	this.sortable = true;
	Table.addProperty("sortable", addSortLink);
	this.setHeaderProperty("sortable");
	this.paint();
}


/**
 * Adds a sort link to a html node.
 * @param code The code of the cell to make sortable.
 * @param field The field for this code.
 * @param table The table the field is in.
 */
function addSortLink(code, field, table) {
	var sortLink;
	if (isEmpty(field.getValue())) {
		code.onclick = function() {
			TableManager.instance().getTable(table.id).sortColumn(field.getName());
		}
	}
	else {
		sortLink = createSortLink(table, field.getName());
		appendAll(code, sortLink);
	}
	appendSortLink(table, field, sortLink, code);
	
	// explicit deallocation to avoid memory leak on internet explorer	
	if (exists(sortLink)) {
		sortLink.onClick = null;
		sortLink = null;
		delete sortLink;
	}
}


/**
 * Append a sort link to a given html node. 
 * @param table The table the field is in.
 * @param field The field for this code.
 * @param sortLink The sort link to append.
 * @param code The html code to append the sort link to.
 */
function appendSortLink(table, field, sortLink, code) {
	var sortNobr = document.createElement("nobr");
	// set the sort image to the left of right aligned columns
	var column = table.getColumn(field.getName());
	if (isDefined(column.getAlignment()) && column.getAlignment() == "right") {
		sortNobr.appendChild(createSortImage(code, field, table));
	}
	
	// append all the code child nodes to the link
	if (exists(sortLink)) {
		sortNobr.appendChild(sortLink);
	}
	else {
		appendAll(code, sortNobr);
	}
	
	// set image to the right of left/middle aligned columns	
	if (column.getAlignment() != "right") {
		sortNobr.appendChild(createSortImage(code, field, table));
	}
	code.appendChild(sortNobr);
}


/**
 * Creates the sort link. 
 * @param table The table the sort link is for.
 * @param columnName The name of the column this sort field is to sort. 
 * @return The sort link created.
 */
function createSortLink(table, columnName) {
	var sortLink = document.createElement("a");
	sortLink.setAttribute("href", "#");
	sortLink.setAttribute("name", "sortLink");
	sortLink.onclick = function() {
		TableManager.instance().getTable(table.id).sortColumn(columnName);
	}
	return sortLink;
}


/**
 * Creates the sort image.
 * @param code The code of the cell to make sortable.
 * @param field The field for this code.
 * @param table The table the field is in.
 * @return The sort image created. 
 */
function createSortImage(code, field, table) {
	var sortImage = document.createElement("img");
	// set the size of the image to 0 to avoid problems in firefox with the image being
	// displayed to large, the correct size is set when the image has loaded
	sortImage.style.width = "0px";
	sortImage.style.height = "0px";
	sortImage.setAttribute("border", "0");
	sortImage.setAttribute("name", "sortImage");
	setClass(sortImage, "invisible");
	sortImage.onload = function() {
		this.style.height = "8px";
		this.style.width = "16px";
	}
	sortImage.src = Table.SORT_DOWN_IMAGE;

	
	// set current column visible
	var dataColumnSorted = exists(table.sortDataColumns[field.getName()]) && table.sortDataColumns[field.getName()] == table.columnToSort;
	if ((field.getName() == table.columnToSort || dataColumnSorted) && !table.reversed) {
		sortImage.src = Table.SORT_DOWN_IMAGE;
		setClass(sortImage, "visible");		
	}
	else if (field.getName() == table.columnToSort || dataColumnSorted) {
		sortImage.src = Table.SORT_UP_IMAGE;
		setClass(sortImage, "visible");		
	}
	return sortImage;
}


/**
 * Swaps the sort indicator image.
 * @param The table which is being sorted.
 * @param columnName The name of the column to sort.
 */
function swapSortImage(table) {
	var header = table.getHeaderTableCode();
	var cell = getElementWithAttribute(header, table.columnToSort, "name", "td");
	var currentSortImage = getElementWithAttribute(cell, "sortImage", "name", "img");
	try {
		if (currentSortImage.src.indexOf(Table.SORT_DOWN_IMAGE) > -1) {
			currentSortImage.setAttribute("src", Table.SORT_UP_IMAGE);;
		}
		else if (currentSortImage.src.indexOf(Table.SORT_UP_IMAGE) > -1) {
			currentSortImage.src = Table.SORT_DOWN_IMAGE;
		}
	}
	// can happen if the sorting is not finished yet
	catch (e) {}
}


// sorting


/**
 * Set the sort function to use. If no custom sort function
 * is given all entries are assumed to be strings and 
 * are sorted thereafter.
 * @param sort The sort function.
 * @throws Error if sort is undefined or null.
 */
Table.prototype.setSortFunction = function(sort) {
	assertDefined(sort, "Sort function is undefined or null");
	this.sortFunction = sort;
}


/**
 * Set the column to sort for this table.
 * @param column The column to sort on.
 * @throws Error if column is undefined null, negative or exceeds the number of columns in this table.
 */
Table.prototype.setColumnToSort = function(column) {
	assertDefined(column, "Column to sort is undefined or null");
	columnToSort = column;
	var sortDataColumn = this.sortDataColumns[column];
	if (isDefined(sortDataColumn)) {
		// the column is sorted using sortDataColumn' data 
		columnToSort = sortDataColumn;
	}
	this.columnToSort = column;
}


/**
 * Return the name of the column the table is currently sorted on.
 */
Table.prototype.getSortColumn = function() {
	if (isDefined(this.sortDataColumns[this.sortedColumn]) && !isNull(this.sortDataColumns[this.sortedColumn])) {
		return this.sortDataColumns[this.sortedColumn];
	}
	return this.sortedColumn;
}


/**
 * Set a sort column for a column. With a sort column set
 * fields in the column are sorted according to the values of the
 * fields in the sort column instead their own values.
 * @param columnName The name of the fields to set the sort field for.
 * @param sortFieldName The name of the sort field to set.
 * @threws Error if fieldName is undefined, null or empty.
 * @throws Error if sortFieldName if undefined, null or empty.
 */
Table.prototype.setSortDataColumn = function(columnName, sortFieldName) {
	assertNotEmpty(columnName, "ColumnName is undefined or empty");
	assertNotEmpty(sortFieldName, "SortFieldName is undefined or empty");
	this.sortDataColumns[columnName] = sortFieldName;
}


/**
 * Sorts a column. After sorting the table is painted. If the 
 * column to sort is the column that has been sorted on last time
 * the column is reversed.
 * If no custom sort function
 * is given all entries are assumed to be strings and 
 * are sorted thereafter.
 * @param column The name of the column to sort.
 * @throws Error if column is undefined null or negative.
 */
Table.prototype.sortColumn = function(column) {
	assertDefined(column, "Column to sort is undefined or null");
	if (column < 0) {
		throw new Error("Column to sort is negative");
	}

	// swap the sort indicator image
	if (column == this.columnToSort) {
		swapSortImage(this);
	}

	if (isDefined(this.sortedColumn) && this.sortedColumn == column && (!this.reversed)) {
		this.sortedColumn = column;
		this.reverseColumn(column);
		this.reversed = true;
	}
	else {
		this.reversed = false;
		this.sort(column);
	}

	this.paint();
	this.fireSortedEvent();
}


/**
 * Fires a sorted event if the eventserver is available.
 */
Table.prototype.fireSortedEvent = function() {
	if (!eventsAvailable()) {
		return;
	}
	
	getEventServer().notifyListeners(new Event("tableSorted", this.id));
}


/**
 * Returns true if a EventServer is available.
 */
function eventsAvailable() {
	try {
		if (isDefined(getEventServer)) {
			return true;
		}
	}
	catch(Error) {
		return false;
	}
	try {
		if (isDefined(Event)) {
			return true;
		}
	}
	catch(Error) {
		return false;
	}
	return true;
}


/**
 * Sorts a column. If no custom sort function
 * is given all entries are assumed to be strings and 
 * are sorted thereafter.
 * @param column The column to sort.
 * @throws Error if column is undefined null or negative.
 */
Table.prototype.sort = function(column) {
	assertDefined(column, "Column to sort is undefined or null");
	if (column < 0) {
		throw new Error("Column to sort is negative");
	}
	
	this.sortedColumn = column;
	this.setColumnToSort(column);
	if (isDefined(this.sortFunction)) {
		// the parameters passed to the sort function are
		// two arrays. Each array contains a row in the table. To 
		// get the field in the row to sort on call getSortField
		// with the row as parameter
		this.rows.sort(this.sortFunction);
	}
	else {
		this.rows.sort(defaultSortFunc);
	}
}


/**
 * Default sorting function. This function takes two arguments, each of them is 
 * a array holding one row, to get the field to sort on use getSrotField().
 * @param a The first row to sort.
 * @param b The second row to sort.
 * @return A negative value if the sort field of row a is less than b the sort field of row b, a positive
 * value if a's sort field is larger than b's.
 */
function defaultSortFunc(a, b) {
	var aValue = getSortValue(a);
	var bValue = getSortValue(b);
		
	if (isFloat(aValue) || isFloat(bValue)) {
		return sortFloats(aValue, bValue);
	}
	else if(isNumberString(aValue) || isNumberString(bValue)) {
		return sortNumbers(aValue, bValue);
	}
	else {
		// sort as strings
		aValue = aValue.toUpperCase();
		bValue = bValue.toUpperCase();
		return sortStrings(aValue, bValue);
	}
}


/**
 * Get the value to sort for a row.
 * @param row The row to get the value to sort from.
 * @return The value that is to be sorted for the given row.
 */
function getSortValue(row) {
	var field = getSortField(row);
	return field.getValue();
}


/**
 * Sorts two numbers.
 * @param a The unparsed string representation of the first number.
 * @param b The unparsed string representation of the secound number.
 * @returns A negative value if a is less than b, a positive value if a is largen than b, 
 * 0 if the numbers are equal.
 */
function sortNumbers(a, b) {
	if(isNumberString(a) && (!isNumberString(b))) {
		// numbers should come before text
		return -1;
	}
	else if(isNumberString(b) && (!isNumberString(a))) {
		// numbers should come before text
		return 1;
	}

	aValue = parseInt(a);
	bValue = parseInt(b);
	return bValue - aValue;
}


/**
 * Sort two float values. The floats can be of type
 * 10,5 or 10.5.
 * @param a The unparsed string representation of the first float.
 * @param b The unparsed string representation of the second float.
 * @returns A negative value if a is less than b, a positive value if a is larger than b, 
 * 0 if the numbers are equal.
 */
function sortFloats(a, b) {
	if(isFloat(a) && (!isFloat(b))) {
		// numbers should come before text
		return -1;
	}
	else if(isFloat(b) && (!isFloat(a))) {
		// numbers should come before text
		return 1;
	}
	
	var index = a.indexOf(",");
	if (index >= 0) {
		// replace the , with a .
		var a = a.substring(0, index) + "." + a.substring(index + 1);
	}
	
	var index = b.indexOf(",");
	if (index >= 0) {
		// replace the , with a .
		var b = b.substring(0, index) + "." + b.substring(index + 1);
	}
	
	var aValue = parseFloat(a);
	var bValue = parseFloat(b);
	return bValue - aValue;
}



/**
 * Compares two strings.
 * @param a String a.
 * @param b String b.
 * @return -1 if string a is less than string b, 1 ohterwise.
 */
function sortStrings(a, b) {
	var sortArray = new Array();
	sortArray[0] = a;
	sortArray[1] = b;
	sortArray.sort();
	if (sortArray[0] == a) {
		return -1;
	}
	else {
		return 1;
	}
}


/**
 * Reverses the data in a column.
 * @param column The column to reverse.
 * @throws Error if column is undefined null or negative. 
 */
Table.prototype.reverseColumn = function(column) {
	assertDefined(column, "Column to sort is undefined or null");
	if (column < 0) {
		throw new Error("Column to sort is negative");
	}
	
	this.rows.reverse();
}


/**
 * Get the field to sort on for a row. 
 * @param row The row to get the sort field for.
 * @throws Error if row is undefined or null.
 */
getSortField = function (row) {
	assertDefined(row, "Row is undefined or null");
	return getFieldFromRow(columnToSort, row);
}


/**
 * Set the time between the updates.
 * @param time The time to set.
 */
Table.prototype.setTime = function(time) {
	this.time = parseInt(time);
	if (isNaN(this.time)) {
		this.time = Table.DEFAULT_TIME;
	}
	if (isDefined(this.repeater)) {
		this.repeater.setTime(this.time);
	}
}


// setup


/**
 * Create the setup for this table. To make the setup visible it needs to be appended
 * to the html document. In IE checkboxes in the field chooser can not 
 * be checked when the form is not appendet to the document so you need to call
 * this table objects checkChooserFields() method after you appendet the chooser
 * to the document.
 * @param The number of columns the fields should displayed in.
 */
Table.prototype.createSetup = function(columns) {
	if (!isDefined(this.setupTable)) {
		this.setupTable = document.createElement("table");
	}
	else {
		setClass(this.setupTable, "active");
		// clear the setup table
		var len = this.setupTable.rows.length -1;
		for (var i = len; i >= 0; i--) {
			if (i < 0) {
				break;
			}
			this.setupTable.deleteRow(i);
		} 
	}
	
	this.setupTable.setAttribute("name", "setupTable");
	this.setupTable.border = 0;
	this.setupTable.cellPadding = 2;
	this.setupTable.cellSpacing = 2;
	var tableBody = document.createElement("TBODY");
	this.setupTable.appendChild(tableBody);
	
	// the close link
	var header = this.setupTable.insertRow(0);
	var cell = document.createElement("TD");
	cell.setAttribute("align", "right");
	var closeLink = document.createElement("a");
	var id = this.id;
	closeLink.onclick = function() {
		TableManager.instance().getTable(id).closeSetup();
	}
	var closeImage = document.createElement("img");
	closeImage.src = "images/setup_close.gif";
	closeImage.setAttribute("border", "0");
	closeImage.setAttribute("alt", "Close");
	closeLink.appendChild(closeImage);
	cell.appendChild(closeLink);
	header.appendChild(cell);
	
	// add the row containing the interval input field
	var intervalRow = this.setupTable.insertRow(1);
	var intervalCell = document.createElement("TD");
	var intervalText = document.createTextNode("Update interval: ");
	intervalCell.appendChild(intervalText);
	intervalRow.appendChild(intervalCell);
	var interValInput = document.createElement("input");
	interValInput.setAttribute("type", "text");
	interValInput.setAttribute("name", "intervalInput");
	interValInput.setAttribute("maxLength", "2");
	interValInput.setAttribute("size", "1");
	interValInput.setAttribute("value", this.time);
	interValInput.onchange = function() {
		TableManager.instance().getTable(id).changeUpdateInterval(this.value);
	}
	intervalCell.appendChild(interValInput);
	intervalCell.appendChild(document.createTextNode(" seconds"));
	
	// add the error mesage if the update interval has beeen set incorrect
	var intervalErrorMessage = this.setupTable.insertRow(2);
	var intervalErrorMessageCell = document.createElement("TD");
	intervalErrorMessageCell.setAttribute("name", "intervalErrorMessage");
	setClass(intervalErrorMessageCell, "error");
	intervalErrorMessage.appendChild(intervalErrorMessageCell);
	
	// add the field chooser
	var fieldChooserRow = this.setupTable.insertRow(3);
	var fieldChooserCell = document.createElement("TD");
	var fieldChooser = this.createFieldChooser(columns);
	document.body.appendChild(fieldChooser);
	fieldChooserCell.appendChild(fieldChooser);
	fieldChooserRow.appendChild(fieldChooserCell);
	
	// explicit deallocation to avoid memory leak on internet explorer	
	closeLink = null;
	interValInput = null;	
	
	return this.setupTable;
}


/**
 * Changes the update interval for this table on response 
 * to a change in setup for this table.
 * @param time The new time to set.
 */
Table.prototype.changeUpdateInterval = function(time) {
	time = parseInt(time);
	
	var intervalErrorMessage = getElementWithAttribute(this.setupTable, "intervalErrorMessage", "name", "td");
	
	// clear the error message if any
	if (!isNull(intervalErrorMessage.firstChild)) {
		intervalErrorMessage.removeChild(intervalErrorMessage.firstChild);
	}
	
	// check for invalid values
	if (isNaN(time)) {
		var errorText = document.createTextNode("Update interval must be a number");
		intervalErrorMessage.appendChild(errorText);
		return;
	}
	else if (time < 0) {
		var errorText = document.createTextNode("Update interval is must be positive");
		intervalErrorMessage.appendChild(errorText);
		return;
	}
	
	// set the interval
	this.setTime(time);
	this.restartUpdate();
}


/**
 * Hides the setup of this table.
 */
Table.prototype.closeSetup = function() {
	document.body.removeChild(this.setupTable);		
}


/**
 * Create the field chooser. To make the field Chooser visible it needs to be appended
 * to the html document. In IE checkboxed can not 
 * be checked when the form is not appendet to the document so you need to call
 * this table objects checkChooserFields() method after you appendet the chooser
 * to the document.
 * @param The number of columns the fields should displayed in.
 * @return A reference to a form containing the fields to choose from. To make
 * the form visible it has to be appended to the html code.
 * This is the response of the caller.
 */
Table.prototype.createFieldChooser = function(columns) {
	var tableCode = this.getTableCode();
	if (!isDefined(this.fieldChooser)) {
		this.fieldChooser = document.createElement("form");
	}
	else {
		setClass(this.fieldChooser, "active");
	}
	
	var table = this.populateFieldChooser(columns);
	this.fieldChooser.appendChild(table);
	this.checkChooserFields();
	
	return this.fieldChooser;
}


/**
 * Hides the field chooser form if one is created.
 */
Table.prototype.hideFieldChooser = function() {
	setClass(this.fieldChooser, "inactive");
}


/**
 * Populates the field chooser with the fields. If the table containing
 * the fields already exists 
 * @param The number of columns the fields should displayed in.
 */
Table.prototype.populateFieldChooser = function(columns) {
	var buttonTable = getElementWithAttribute(this.fieldChooser, "fieldChooserTable", "name", "table");
	if (isDefined(buttonTable) && !isNull(buttonTable)) {
		// the table containing the fields already exists, delete it and
		// populate it with the current fields
		this.fieldChooser.removeChild(buttonTable);
	}

	buttonTable = document.createElement("table");
	buttonTable.setAttribute("name", "fieldChooserTable");
	buttonTable.border = 0;
	buttonTable.cellPadding = 2;
	buttonTable.cellSpacing = 2;
	var tableBody = document.createElement("TBODY");
	buttonTable.appendChild(tableBody);
	
	// create the checkBoxes
	this.createCheckBoxes(columns, tableBody);	
	return buttonTable;
}


/**
 * Appends the column checkboxes to a table.
 * @param The number of columns the fields should displayed in.
 * @param tableBody The body of the table the checkboxes should be inserted in.
 */
Table.prototype.createCheckBoxes = function(columns, tableBody) {
	// loop through all the fields
	var fieldChooserColumns = this.fieldChooserColumns();
	var len = fieldChooserColumns.length;
	for (var i = 0; i < len; i++) {
		var row = document.createElement("TR");
		
		for (var j = 0; j < columns; j++) {
			// create the checkbox and text for each field 
			var column = fieldChooserColumns[i];
			if (!exists(column)) {
				break;
			}
			var cell = document.createElement("TD");
			var checkBox = document.createElement("input");
			checkBox.setAttribute("type", "checkbox");
			checkBox.setAttribute("name", column.getName());
			var fieldName = column.getName();
			var id = this.id
			checkBox.onclick = function() {
				TableManager.instance().getTable(id).toggleColumnVisibility(this.getAttribute("name"));
			}
			var text = "";
			if (!isEmpty(column.getFieldChooserText())) {
				text = column.getFieldChooserText();
			}	
			else {
				text = column.getHeaderText();
			}
			var checkBoxText = document.createTextNode(text);
			cell.appendChild(checkBox);
			cell.appendChild(checkBoxText);		
			row.appendChild(cell);
						
			if (j < columns -1) {
				i++;
			}
			
			// explicit deallocation to avoid memory leak on internet explorer	
			checkBox = null;
		}
		tableBody.appendChild(row);
	}
}


/**
 * Gets the columns to be shown in the field chooser, columns
 * with no fieldChooser text or header text set are not shown in the field chooser. 
 * @return The columns to show in the field chooser.
 */
Table.prototype.fieldChooserColumns = function() {
	var columns = new Array;
	for (var i = 0; i < this.columnOrder.length; i++) {
		var column = this.columnOrder[i];
		if (isEmpty(column.getFieldChooserText()) && isEmpty(column.getHeaderText())) {
			continue;
		}
		columns[columns.length] = column;
	}
	return columns;
}


/**
 * Toggles the visibility of a column. If the column is visible it is made invisible
 * and if it is unvisible it is made visible.
 * @param name The name of the column. 
 */
Table.prototype.toggleColumnVisibility = function(field) {
	var madeVisible = false;
	if (this.isVisibleColumn(name)) {
		this.setInvisibleColumn(name);
	}
	else {
		madeVisible = true;
		this.setVisibleColumn(name);
	}

	if (isOpera()) {
		// this is needed in Opera to avoid problems with the 
		// cell width when the scrollbar is shown and hidden
		var tableContainer = this.getTableContainerCode();
		tableContainer.style.display = "none";
		tableContainer.style.display = "";	
	}
	
	if (!isSafari()) {
		// in safari we can not paint the table until we have made room 
		// for the new column, or else we get scrollbars. While in other browsers
		// espically in Opera resizing first can lead to problems
		this.paint();
	}
	
	if (madeVisible) {
		var column = this.getColumn(name);
		var diff = column.getWidth() + this.calculateExtraWidthSize();
		this.resizeContainers(0, diff);
		
		var eventData = new Array();
		eventData["id"] = this.id;
		eventData["height"] = 0;
		eventData["width"] = diff;
		getEventServer().notifyListeners(new Event("tableResized", eventData));
	}
	else {
		var column = this.getColumn(name);
		var diff = column.getWidth() + this.calculateExtraWidthSize();
		this.resizeAllColumns(diff);
	}
	
	if (isSafari()) {
		// in safari we can not paint the table until we have made room 
		// for the new column, or else we get scrollbars. While in other browsers
		// espically in Opera resizing first can lead to problems
		this.paint();
	}
}


/**
 * Check the visible fields and uncheck the invisible fields in the 
 * field chooser.
 */
Table.prototype.checkChooserFields = function () {
	var inputs = this.fieldChooser.getElementsByTagName("input");
	var inputsLength = inputs.length;
	for (var i = 0; i < inputsLength; i++) {
		if ("checkbox" == inputs[i].getAttribute("type") && this.isVisibleColumn(inputs[i].getAttribute("name"))) {
			inputs[i].checked = true;
		}
		else if ("checkbox" == inputs[i].getAttribute("type")) {
			inputs[i].checked = false;
		}
	}
}


/**
 * Returns the update interval set. 
 */
Table.prototype.getUpdateInterval = function() {
	return this.repeater.getTime();
}


/**
 * Returns true if the table is beeing updated periodically, false 
 * if the table is not being updated. 
 */
Table.prototype.isUpdating = function() {
	return this.repeater.isRepeating();
}


/**
 * Updates the data, in intervals specified by the object's time property.
 */
Table.prototype.update = function() {
	this.repeater.start();
}


/**
 * Loads data into the table. The data is loaded from the url set. 
 * The loaded xml data is stored in a property called xmlData.
 */
function load(){
	if (this.layoutSetInProgress == true || isEmpty(this.url)) {
		// the layout is currently being set, do not update the table
		return;
	}
	var thisObj = this;
	loadXML(this.url, function(xmlData)  {
		thisObj.clear();
		thisObj.setData(xmlData, "entry");
		thisObj.xmlData = xmlData;
		thisObj.paint();
	});
}


/**
 * Restarts the updating of this table. 
 */
Table.prototype.restartUpdate = function() {
	this.repeater.stop();
	this.repeater.start();
}


/**
 * Cancels the updating of this table.
 */
Table.prototype.cancelUpdate = function() {
	this.repeater.stop();
}


/**
 * Gets the url the table uses to load data.
 * @return The url used. 
 */ 
Table.prototype.getUrl = function() {
	return this.url;
}


/**
 * Set the url used to load data by the table.
 * @param url The url to set. 
 * @throws Error if url is undefined, null or empty.
 */
Table.prototype.setUrl = function(url) {
	assertNotEmpty("Url to set is undefined null or empty", url);
	this.url = url;
}


/*************** Start, utility functions for chart images ****************/

/**
 * Creates a link with the specified url and value.
 * @param textNode The text node to show in the link.
 */
function createLink(textNode) {
	var link = document.createElement('a');
	link.setAttribute("href", "#");
	link.appendChild(textNode);
	return link;
}


/**
 * Return the html code of a table, null if the table has not been created yet.
 */
Table.prototype.getTableCode = function() {
	if (isNull(this.getTableContainerCode())) {
		return undefined;
	}
	if (!isDefined(this.htmlCode) || isNull(this.htmlCode)) {
		this.htmlCode =  document.getElementById(this.id);
	}
	return this.htmlCode;
}


/**
 * Returns the html code of the header container, null if the header container does not exist. 
 */
Table.prototype.getHeaderContainerCode = function() {
	if (!isDefined(this.headerContainer) || isNull(this.headerContainer)) {
		this.headerContainer = getElementWithAttribute(document.body, "headerTableContainer" + this.id, "id", "div");
	}
	return this.headerContainer;
}


/**
 * Returns the html code of the header table, null if not create yet.
 */
Table.prototype.getHeaderTableCode = function() {
	if (isNull(this.getTableContainerCode())) {
		return undefined;
	}
	
	if(!isDefined(this.headerTable) || isNull(this.headerTable)) {
		this.headerTable = getElementWithAttribute(this.getTableContainerCode(), "headerTable" + this.id, "id", "table");
	}
	return this.headerTable;
}

 
/**
 * Return the html code of the table container, null if the table container does not exist.
 * This container contains all the html code for the table.
 */
Table.prototype.getTableContainerCode = function() {
	if (!isDefined(this.tableContainer) || isNull(this.tableContainer)) {
		this.tableContainer = getElementWithAttribute(document.body, "tableContainer" + this.id, "id", "div");
	}
	return this.tableContainer;
}


/**
 * Return the container containing the data table, null if the container does not exit.
 */
Table.prototype.getDataTableContainer = function() {
	if (!isDefined(this.dataTableContainer) || isNull(this.dataTableContainer)) {
		this.dataTableContainer = getElementWithAttribute(document.body, "dataTableContainer" + this.id, "id", "div");
	}
	return this.dataTableContainer;
}