/**
 * Shows a tooltip.
 * @param content A reference to a html element to display in the tooltip.
 * @param xPos The offset from the left of the screen.
 * @param yPos The offset from the top of the screen.
 * @param zIndex The z index to set for the tooltip.
 * @param optional parameter, if this is set to true the slides down
 * from the yPos to its full size.
 * @param optional parameter, if this is provided the tooltip is appended to this element, not the document.
 * @returns The tooltip created and shown.
 * @thrwos Error if element is null or undefined.
 * @throws Error if xPos is negative.
 * @throws Error if yPos is negative.
 * @throws Error is zIndex is negative.
 */
function showTooltip(content, xPos, yPos, zIndex, slide, parentElement) {
	assertDefined(content, "The content is null or undefined");
	assertNotNegative(xPos, "Xpos is negative");
	assertNotNegative(yPos, "Ypos is negative");
	
	var tooltip = createTooltip(content, xPos, yPos, zIndex);
	tooltip.style.visibility = "hidden";
	if (exists(parentElement)) {
		parentElement.appendChild(tooltip);
	}
	else {
		document.body.appendChild(tooltip);
	}
	
	if (arguments.length == 5 && arguments[4] && !isOpera()) {
		tooltip.style.visibility = "hidden";
	}
	
	// check the position of the tooltip
	if (yPos > (window.innerHeight - tooltip.offsetHeight)) {
		yPos = window.innerHeight - tooltip.offsetHeight;
	}
	else if (yPos < 0) {
		yPos = 0; 
	}
	
	xPos = xPos - tooltip.offsetWidth;
	if (xPos > (window.innerWidth - tooltip.offsetWidth)) {
		xPos = window.innerWidth - tooltip.offsetWidth;
	}
	else if (xPos < 0) {
		xPos = 0; 
	}
	
	// set the position of the tooltip
	tooltip.style.top = yPos + "px";
	tooltip.style.left = xPos + "px";
	tooltip.style.zIndex = zIndex;
	
	if (arguments.length == 5 && arguments[4] && !isOpera()) {
		slideTooltip(tooltip, 10, 1);
	}
	tooltip.style.visibility = "visible";
	return tooltip;
}


/**
 * Slides inn a tooltip. The size of the tooltip is increased to make it 
 * it look like it slides in.
 * @param tooltip The tooltip to slide.
 * @param inc The number of pixels to increase the size of the tooltip each time interval.
 * @param time The time between each time the size of the tooltip is increased.
 */
function slideTooltip(tooltip, inc, time) {
	if (!isDefined(slideTooltip.originalHeight) || isNull(slideTooltip.originalHeight)) {
		slideTooltip.originalHeight = tooltip.offsetHeight;
		slideTooltip.originalWidth = tooltip.offsetWidth;
		slideTooltip.originalLeftOffset = parseInt(tooltip.style.left);
		slideTooltip.currentHeight = 0;
		slideTooltip.currentWidth = 0;
		tooltip.style.overflow = "hidden";
		tooltip.style.height = "0px";
		tooltip.style.width = "0px";
		slideTooltip.currentLeftOffset = slideTooltip.originalLeftOffset + slideTooltip.originalWidth;
		tooltip.style.left = slideTooltip.currentLeftOffset + "px";
		
		tooltip.style.visibility = "visible";
	}
	
	slideTooltip.currentHeight += inc;
	slideTooltip.currentWidth += inc;
	slideTooltip.currentLeftOffset -= inc;
	
	// increase the height of the tooltip if the tooltip not already has its max height
	if (slideTooltip.currentHeight > slideTooltip.originalHeight) {
		slideTooltip.currentHeight = slideTooltip.originalHeight;
	}
	tooltip.style.width = slideTooltip.currentWidth + "px";
	
	// increase the width of the tooltip if the tooltip not already has its max width
	if (slideTooltip.currentWidth > slideTooltip.originalWidth) {
		slideTooltip.currentWidth = slideTooltip.originalWidth
	}
	tooltip.style.height = slideTooltip.currentHeight + "px";
	
	// move the tooltip to the left
	if (slideTooltip.currentLeftOffset < slideTooltip.originalLeftOffset) {
		slideTooltip.currentLeftOffset = slideTooltip.originalLeftOffset;
	}
	tooltip.style.left = slideTooltip.currentLeftOffset + "px";
		
	if (slideTooltip.currentSize < slideTooltip.originalSize) {
		window.setTimeout(function(){ slideTooltip(tooltip, inc, time) }, time);
	}
	else if (slideTooltip.currentWidth < slideTooltip.originalWidth) {
		window.setTimeout(function(){ slideTooltip(tooltip, inc, time) }, time);
	}
	else {
		slideTooltip.originalHeight = null;
	}
}


/**
 * Closes a tooltip.
 * @param tooltip The tooltip to close.
 * @param optional parameter, the parent element to remove the tooltip from, if not provided the document is used.
 * @throws Error if tooltip is undefined or null.
 */
function closeTooltip(tooltip, parentElement) {
	assertDefined(tooltip, "Tooltip is undefined or null");
	if (exists(parentElement)) {
		parentElement.removeChild(tooltip);
	}
	else {
		document.body.removeChild(tooltip);
	}	
}


/**
 * Creates the html code for a tooltip.
 * @param content A reference to a html element to display in the tooltip.
 * @returns The tooltip created and shown.
 */
function createTooltip(content, xPos, yPos, zIndex) {
	var div = document.createElement("div");
	var tooltip = document.createElement("table");
	div.appendChild(tooltip);
	tooltip.setAttribute("name", "toolTipTable");
	var tableBody = document.createElement("TBODY");
	tooltip.appendChild(tableBody);
	
	var row = tooltip.insertRow(0);
	var cell = document.createElement("TD");
	cell.appendChild(content);
	row.appendChild(cell);
	
	setClass(div, "tooltip");
	
	return div;
}


/**
 * Returns the help text for a given name (alias).
 * This is just to avoid a lot of text in the html code.
 */
function helptext(name) {
	// alarms
	if (name == "protocolnumber") {
		return "Port number. For example, TCP port 80 is:<br>1.2048.6.80 (where 6 means TCP)";
	}
	if (name == "deltaalarm") {
		return "A delta alarm is triggered when the<br>monitored delta value for the<br>given field is larger/lower than a specified value";
	}
	if (name == "samplingtime") {
		return "An alarm is triggered if the difference of the values between two samples<br>are larger/less than the specified upper/lower trigger value";
	}
	if (name == "absolutealarm") {
		return "An absolute alarm is triggered if the value of specified entry<br>is higher than the specified upper trigger value";
	}
	if (name == "triggeronhigh") {
		return "The alarm will be triggered when the value of<br>the entry is higher than the specified trigger value";
	}
	if (name == "triggeronlow") {
		return "The alarm will be triggered when the value of<br>the entry is lower than the specified trigger value";
	}
	if (name == "editAlarmImage") {
		return "Edit entry";
	}
	
	// tables
	if (name == "alarmCount") {
		return "The number of times an alarm<br>has been triggered for the entry";
	}
	if (name == "alarmCountImage") {
		return "The number of times an alarm<br>has been triggered for the entry";
	}
	
	// my tables
	if (name == "myprotocolsport") {
		return "Port number. For example, TCP port 80 is:<br>1.2048.6.80 (where 6 means TCP)";
	}
	if (name == "addmytable") {
		return "Add entry";
	}
	if (name == "deleteEntryImage") {
		return "Delete entry";
	}
	if (name == "myprotocolselect") {
		return "The protocol type";
	}
	if (name == "myprotocolscomment") {
		return "A comment for this entry";
	}
	if (name == "myprotocolsperhostaddress") {
		return "The IP address for this entry";
	}
	if (name == "myconversationssrcaddress") {
		return "The source IP address for this entry";
	}
	if (name == "myconversationsdstaddress") {
		return "The destination IP address for this entry";
	}
	if (name == "myconversationscomment") {
		return "A comment for this entry";
	}
	
	// profiles
	if (name == "profilename") {
		return "Profile name. This text is also<br>shown in the \"Profiles\" menu";
	}
	if (name == "profiledescription") {
		return "Profile description";
	}
	if (name == "activeprofileselector") {
		return "Select a profile to activate now";
	}
	if (name == "defaultprofileselector") {
		return "Select the default profile to<br>activate when the client is started";
	}
	if (name == "addProfileName") {
		return "Enter a name for the profile to add. Press Enter to add the profile";
	}
	if (name == "addProfileDescription") {
		return "Enter a description for the profile to add. Press Enter to add the profile";
	}
	
	// setup
	// SMTP settings
	if (name == "smtpserver") {
		return "The server used for sending e-mails (outgoing mail server)";
	}
	if (name == "smtpport") {
		return "The port used for sending e-mails. The default SMTP port is 25.";
	}
	if (name == "smtpauthentication") {
		return "Some SMTP servers require login and password to send e-mails";
	}
	if (name == "smtpfromaddress") {
		return "This address is shown as the sender of e-mails";
	}
	if (name == "smtpfromname") {
		return "Optionally the name of the sender can be specified as well";
	}
	
	if (name == "setChartMode") {
		return "In normal mode the highest peak of <br>all the interval values is used as <br>max value for the " +
		"gauge.<br>Set a max value and the max value <br>of the gauge will not be set larger <br>than the set max value" +
		"<br>Click to read more";
	}
	
	if (name == "programActions") {
		return "Put programs to be run into the <br>programActions folder";
	}
}