

/**
 * Throws a error if a boolean value is not true.
 * @param value The value to check.
 * @param message The error message to throw.  
 * @throws Error if value is undefined or null. 
 * @throws Error if value is not true.
 */
function assertTrue(value, message) {
	assertExists(value, "The value to check does not exist");
	if (value != true) {
		throw new Error(message != null ? message : "");
	}
}


/**
 * Throws an error if a number is negative.
 * @param number The number.
 * @param message The error message to throw.
 * @throws Error if number is undefined, null or not a number.
 * @throws Error if number is not a number.
 */
function assertNotNegative(number, message) {
	assertExists(number, "The value does  not exist");
	assertIsNumber(number, message);
	if (isNegative(number)) {
		throw new Error(message != null ? message : "");
	}
}


/**
 * Throws an error if a value is not a number. 
 * @param value The value to check. 
 * @param message The error message to throw.
 * @param throws Error if value is undefined or null.
 */
function assertIsNumber(value, message) {
	assertExists(value, "The value does  not exist");
	if (!isNumber(value)) {
		throw new Error(message != null ? message : "");
	}	
}


/**
 * Throws an error if a string is null, undefined or empty.
 * @param string The string.
 * @param message The error message to throw.
 */
function assertNotEmpty(string, message) {
	assertDefined(string, message);
	if (isEmpty(string)) {
		throw new Error(message != null ? message : "");
	}
}
 

/**
 * Asserts a variable exists.
 * @param value The value to check.
 * @param message The error message to throw.
 */
function assertExists(value, message) {
	if (!exists(value)) {
		throw new Error(message != null ? message : "");
	}
}


/**
 * Throws an error if a value is undefined or null.
 * This works with both objects and variables.
 * @param value The value to check.
 * @param message The error message to throw.
 */
function assertDefined(value, message) {
	if (value == undefined || value == null) {
		throw new Error(message != null ? message : "");
	}
}


/**
 * Throws an error if a value is null.
 * @param value The value to check.
 * @param message The error message to throws.
 */
function assertNotNull(value, message) {
	assertDefined(value, "Value is undefined");
	if (isNull(value)) {
		throw new Error(message != null ? message : "");
	 }
}


/**
 * Returns true if a value exits, a value exists if it is not 
 * undefined or null, returns false otherwise. 
 * @param value The value to check.
 */
function exists(value) {
	if (value == undefined || value == null) {
		return false;
	}
	return true;
}

 
/**
 * Returns true if a number is less than 0.
 * @param number The number to check.
 */
function isNegative(number) {
	if (isString(number) && isNumberString(number)) {
		number = parseInt(number);
	}
	if (number < 0) {
		return true;
	}
	return false;
}


 
/**
 * Returns true if a value is a number.
 * @param number The value to chek.
 * @throws Error if number is undefined or null.
 */ 
function isNumber(number) {
	assertExists(number, "Number is undefined or null");
	if (isNaN(number)) {
		return false;
	}	
	return typeof(number) == "number";
}


/**
 * Returns true if a value is a integer. 
 * @param int The value to check.
 * @throws Error if int is undefined or null.
 */
function isInteger(int) {
	assertExists(int, "The integer to check is undefined or null");
	if (!isNumber(int)) {
		return false;
	}
	
	var intString = int.toString();
	for (var i = 0; i < intString.length; i++) {
		var c = intString.charAt(i);
		if (!isDigit(c) && c != "-") {
			return false;
		}	
	} 
	return true;
}


/**
 * Returns true if a given characters is a digit. 
 * @param c The character to check. 
 */
function isDigit (c) {
	return ((c >= "0") && (c <= "9"))
}

/**
 * Returns true if a variable is a string. 
 * @param variable The variable to check.
 * @param Error if variable is undefined or null. 
 */
function isString(variable) {
	assertExists(variable, "The variable does not exist");
	return "string" == typeof(variable);
}


/**
 * Returns true if a string is a number. 
 * @param string The string to check.
 * @throws Error if the string is undefined or null. 
 */ 
function isNumberString(string) {
	assertDefined(string, "String to check is undefined or null: " + string);
	// check there are only numbers in the string
	var numberValue = string * 1;
	if (isNaN(numberValue) || isEmpty(string)) {
		return false;
	}	
	return true;
}


/**
 * Returns true if a string is a integer.
 * @param string The string to check.
 * @throws Error if the string is undefined or null.
 */
function isIntString(string) {
	assertDefined(string, "String to check is undefined or null: " + string);
	if (string.length == 0) {
		return false;
	}
	// check there are only numbers in the string
	for (var i = 0; i < string.length; i++) {
		var c = string.charAt(i);
		if (!isDigit(c) && c != "-") {
			return false;
		}	
	} 
	return true;
}


/**
 * Returns true if a string is not null or empty.
 * @param string The string.
 * @return True if the string is not empty or null, false otherwise.
 */
function isEmpty(string) {
	if (isNull(string)) {
		return true;
	}
	if (!isDefined(string)) {
		return true;
	}
	
	if (string.length == 0 || trim(string).length == 0) {
		return true;
	}
	return false;
}


/**
 * Trims the leading and trailing white spaces of a string.
 * @throws Error if the string is undefined or null.
 */
function trim(string) {
	assertDefined(string, "String to trim is undefiend");
	string = leftTrim(string);
	return rightTrim(string);
}


/**
 * Trims all the leading white spaces of a string.
 */
function leftTrim(string) {
	while (string.substring(0,1) == ' ') {
		string = string.substring(1, string.length);
	}
	return string;
}


/**
 * Trims all the trailing white spaces of a string.
 */
function rightTrim(string) {
	while (string.substring(string.length-1, string.length) == ' ') {
		string = string.substring(0,string.length-1);
	}
	return string;
}


/**
 * Returns true if the variabel is defined, false otherwise.
 * @param variable The variable to check for.
 * @param Optional parameter The object in which to check for the variabel.
 * If this parameter is not specified the window object is searched for the variabel.
 */
function isDefined(variable) {
	if (arguments.length == 2) {
		if (arguments[1] == undefined) {
			return false;
		}
		return (typeof(eval(arguments[1])[variable]) != 'undefined');
	}
    else {
    	return (typeof(variable) == "undefined")?  false: true;
    }
    return false;
}


/**
 * Returns true if a variable is null.
 */
function isNull(variable) {
    return typeof variable == 'object' && !variable;
}