

/**
 * The size of the border of the dragHelper element. This is used to move the drag helper around 
 * the element instead of just above. 
 */
var DRAG_HELPER_BORDER_SIZE = 2;


/**
 * If this drag mode is set the element is dragged on the x and y axis.
 */
var DRAG_MODE_Y_X = 0;


/**
 * If this drag mode is set the element is only dragged on the y axis.
 */
var DRAG_MODE_Y = 1;


/**
 * If this drag mode is set the element is only dragged on the x axis.
 */ 
var DRAG_MODE_X = 2;


// construction


/**
 * Used to drag a element.
 * @param element The element to drag. 
 * @param event The event that caused the drag.
 * @param useDragHelper Optional argument if set to true if the drag helper is to be used. 
 * @throws Error if element is undefined or null.
 */
function beginDrag(element, event, useDragHelper) {
	assertExists(element, "The element to drag is undefined or null");
	
	var dragger = new DragHelper(element);
	if (exists(useDragHelper) && useDragHelper == true) {
		dragger.setHelperUsed(true);
	} 
	dragger.beginDrag(event);
}


/**
 * Constructs a dragger.
 * @param dragElement The element to drag. 
 * @throws Error if dragElement is undefined or null.
 */
function DragHelper(dragElement) {
	assertExists(dragElement, "The drag element to add is undefined or null");
	
	
	/**
	 * The element to be dragged. 
	 */
	this.dragElement = dragElement;
	
	
	/**
	 * True if the drag helper is to be used during the drag.
	 */
	this.useDragHelper = false;
	
	
	/**
	 * True when the drag helper has been initialised and prepared for dragging a element.
	 */
	this.dragHelperInited = false;
	
	
	/**
	 * If the drag helper is being used this is the original element.
	 */
	this.originalElement;
	
	
	/**
	 * Due to performance reasons we drag a empty frame instead of the actual element.
	 * This is the frame moved. At the end of the drag the the original element is moved to 
	 * the position of the drag helper.
	 */
	this.dragHelper;
	
	
	/**
	 * If the target of the drag event is one of the html elements in this array the 
	 * drag operation is aborted. This is usefull for example dragging a window on a mouse move
	 * event on the window header. But the window is not be moved when a button in the header is 
	 * clicked. 
	 */
	this.exceptions = new Array();
	
		
	/**
	 * The element moved will get its y position set smaller than this value.
	 */
	this.yMin;
	
	
	/**
	 * The element moved will get its y position set larger than this value. 
	 */
	this.yMax;
	
	
	/**
	 * The element moved will get its x position set smaller than this value.
	 */
	this.xMin;
	
	
	/**
	 * The element moved will get its x position set larger than this value. 
	 */
	this.xMax;
	
	
	/**
	 * Indicating whether the element that received the onmousedown has actually ben moved.
	 * True if yes false if not.
	 */
	this.moved;
	
	/**
	 * If this is set to false events are not propagated, other event handlers are stopped from seeing
	 * this event. If this is set to true other event handlers can see this event. The default is not 
	 * to propagate events.  
	 */
	this.propagateEvents = false;
	 
	 
	/**
	 * The x distance of the left corner of the drag element and the mouse click. This value is
	 * used by the move handler. 
	 */
	this.deltaX;
	 
	 
	/**
	 * The y distance of the upper corner of the drag element and the mouse click. This value
	 * is used by the move handler. 
	 */
	this.deltaY;
	 
	/**
	 * The start x and y values. 
	 */
	this.startX = 0;
	this.startY = 0;
	
	
	/**
	 * The function used to listen to mouse moved events. 
	 */
	this.mouseMoveListener = this.createMouseMoveListener();
	
	
	/**
	 * The function used to listen to mouse up events. 
	 */
	this.mouseUpListener = this.createMouseUpListener();
	
	
	/**
	 * The drag mode decides whether the element is dragged on the e or y axis or both.
	 */
	this.dragMode = DRAG_MODE_Y_X;
	
	
	/**
	 * This function is called everytime the element to drag is moved. It is called for each move event
	 * not only at the end of the whole move operation.
	 */
	this.moveFunction;
	
	
	/**
	 * The object to call the move function on.
	 */
	this.moveObject;
}


// moving


/**
 * Called from a mousedown event handler. 
 * @param event The event.
 * @param optional argument. If this is set to false events are not propagated. The default
 * is to stop propagation of events.
 */
DragHelper.prototype.beginDrag = function(event) {
	this.moved = false;
	this.event = exists(event) ? event : window.event;
	if (this.isException(this.event)) {
		// the event is for one of the drag exceptions set, do not drag the element		
		return true;
	}

	// Compute the distance between the upper left corner of the 
	// element and the mouse click. The move handler function uses these values.
	if (this.useDragHelper) {
		this.deltaX = this.event.clientX - parseInt(this.dragElement.style.left) + DRAG_HELPER_BORDER_SIZE;
		this.deltaY = this.event.clientY - parseInt(this.dragElement.style.top) + DRAG_HELPER_BORDER_SIZE;
	}
	else {
		this.deltaX = this.event.clientX - parseInt(this.dragElement.style.left);
		this.deltaY = this.event.clientY - parseInt(this.dragElement.style.top);
	}
	// register the event handler that will respond to the mousemove events
	var thisObject = this;
	registerEventHandler("mousemove", this.mouseMoveListener);
	registerEventHandler("mouseup", this.mouseUpListener);
	
	this.startX = this.event.clientX;
	this.startY = this.event.clientY;
	
	// stop the event being processed by the browser	
	stopDefaultAction(this.event);
}


/**
 * The handler capturing mouse movement events when a element is been 
 * dragged. It is responsibel for moving the element.
 */
DragHelper.prototype.moveHandler = function(event) {
	event = exists(event) ? event : window.event;
	if (this.useDragHelper && (!this.dragHelperInited)) {
		this.initDragHelper();
	}
	this.moved = true;
	this.moveElement(event);
	
	// stop anybody else from seeing this event. 
	if (!this.propagateEvent) {
		stopPropagation(event);
	}
	// stop the event being processed by the browser	
	stopDefaultAction(event);
}
	

/**
 * Move the element according to a mouse move event.
 * @param e The mouse move event.
 */
DragHelper.prototype.moveElement = function(event) {
	// calculate new position of the element
	var xValue = this.calculateXValue(event);
	var yValue = this.calculateYValue(event);
	if (!this.isValidMove(xValue, yValue)) {
		return;
	}

	// set the new position
	if (this.dragMode == DRAG_MODE_Y_X) {
		this.dragElement.style.left = xValue + "px";
		this.dragElement.style.top = yValue + "px";
	}
	else if (this.dragMode == DRAG_MODE_Y) {
		this.dragElement.style.top = yValue + "px";
	}
	else {
		this.dragElement.style.left = xValue + "px";
	}
	
	this.notifyMoved();
}
	

/**
 * Notify the element has been moved, calls the move function if set.  
 */
DragHelper.prototype.notifyMoved = function() {
	if (exists(this.moveFunction) && exists(this.moveObject)) {
		this.moveObject[this.moveFunction]();
	}
	else if (exists(this.moveFunction)) {
		this.moveFunction();
	}
}


/**
 * This is the handler that captures the final mouse up event that
 * at the end of a drag.
 */
DragHelper.prototype.upHandler = function(event) {
	event = exists(event) ? event : window.event;
	// Unregister event handlers 
	removeEventHandler("mouseup", this.mouseUpListener);
	removeEventHandler("mousemove", this.mouseMoveListener);


	if (this.useDragHelper) {
		// move the element to the position of the dragHelper
		this.moveElementToHelper();
		this.hideDragHelper();
		// set back the original element as the drag element if it exists, it might not exist
		// if the drag helper is used and the begindrag function is called but the element
		// is not actually dragged. The original element is set when the  draghelper is inited and 
		// the drag helper is only inited when the element actually has been moved
		this.dragElement = exists(this.originalElement) ? this.originalElement : this.dragElement;
	}

	// stop anybody else from seing this event.
	if (!this.propagateEvent) {
		stopPropagation(event);
	}
	// stop the event being processed by the browser
	stopDefaultAction(event);
}


/**
 * Returns true if a move operations is valid. A move operation is valid if 
 * the new x and y value are not larger or less than the min and max values if they are set. 
 * @param xValue The x value to check.
 * @param yValue The y value to check.
 * @returns True if the move is valid, false otherwise. 
 */
DragHelper.prototype.isValidMove = function(xValue, yValue) {
	if (exists(this.yMin) && isNumber(this.yMin) && yValue < this.yMin) {
		return false;
	}
	if (exists(this.xMin) && isNumber(this.xMin) && xValue < this.xMin) {
		return false;
	}
	if (exists(this.yMax) && isNumber(this.yMax) && yValue > this.yMax) {
		return false;
	}
	if (exists(this.xMax) && isNumber(this.xMax) && xValue > this.xMax) {
		return false;
	}
	return true;
}


/**
 * Calculates the x value for the element.
 * @param event The event that happened.
 * @return The x vlaue to set for the element.
 */
DragHelper.prototype.calculateXValue = function(e) {
	var xValue = e.clientX - this.deltaX;
	return xValue;
}


/**
 * Calculates the y value for the element.
 * @param event The event that happened.
 * @return The y vlaue to set for the element.
 */
DragHelper.prototype.calculateYValue = function(e) {
	var yValue = e.clientY - this.deltaY;
	return yValue;
}

	 
// services


/**
 * Set the function to call when the element has been moved. This function will be called each time the 
 * element to drag has been moved not at the end of the move operation.
 * @param functionToCall The function to call.
 * @param object The object to call the function on, if this is set functionToCall must be the name of the function.
 * @throws Error if functionToCall is undefined or null.
 * @throws Error if object is set and functionToCall is empty.
 */
DragHelper.prototype.setMoveFunction = function(functionToCall, object) {
	assertExists(functionToCall, "The function to call is undefined or empty");
	if (exists(object) && isEmpty(functionToCall)) {
		throw new Error("Fucntion to call is empty");
	}
	this.moveFunction = functionToCall;
	this.moveObject = object;
}


/**
 * Sets the drag mode. The mode set must be one of DRAG_MODE_Y_X, DRAG_MODE_Y, DRAG_MODE_X
 * @param dragMode The drag mode to set. 
 * @throws Error if drag mode is undefined, null or not one of the valid drag modes. 
 */
DragHelper.prototype.setDragMode = function(dragMode) {
	assertExists(dragMode, "The drag mode to set is undefined or null");
	if (dragMode != DRAG_MODE_Y_X && dragMode != DRAG_MODE_Y && dragMode != DRAG_MODE_X) {
		throw new Error("Drag mode to set is invalid: " + dragMode);	
	}
	this.dragMode = dragMode;
}	
 
 
/**
 * Toggles whether events are to be propagated or not. 
 * @param propagate Set this to true to propagate events, false to stop other listeners to see the events.
 * @throws Error if propagate is undefined or null.   
 */
DragHelper.prototype.setPropagateEvents = function(propagate) {
	assertExists(propagate, "Propagate is undefined or null");
	this.propagateEvents = propagate;
}


/**
 * Instead of moving the actual element a drag helper element can be used to be moved instead. 
 * This can improve performance of the move. Toggles whether the helper is to be used or not. 
 * @param useHelper True if the helper element is to be used, false if not.
 * @throws Error if useHelper is undefined or null. 
 */
DragHelper.prototype.setHelperUsed = function(useHelper) {
	assertExists(useHelper, "UseHelper is undefined or null");
	this.useDragHelper = useHelper;
}


/**
 * Get the target for a event.
 * @param event The event.
 * @return The target of the event.
 */
DragHelper.prototype.getEventTarget = function(event) {
	var target = event.target; 
	if (!exists(target)) {
		// IE
		target = event.srcElement;
	}
	return target;
}


/**
 * Set the min y value, no elements moved will get there y value set
 * to a smaller value than the given min y value.
 * @param yValue The value to set as min y.
 * @throws Error if yValue is not a number.
 */
DragHelper.prototype.setYMin = function(yValue) {
	assertIsNumber(yValue, "Min y value to set is not a number: " + yValue);
	this.yMin = yValue;
}


/**
 * Set the max y value, no elements moved will get there y value set
 * to a larger value than the given max y value. 
 * @param yValue The value to set as max y.
 * @throws Error if yValue is not a number. 
 */
DragHelper.prototype.setYMax = function(yValue) {
	assertIsNumber(yValue, "Max y value to set is not a number: " + yValue);
	if (this.useDragHelper == true) {
		// also calculate in the border size of the draghelper
		this.yMax = yValue - (DRAG_HELPER_BORDER_SIZE * 2);
		return;
	}
	this.yMax = yValue;
}


/**
 * Set the min x value, no elements moved will get there x value set
 * to a smaller value than the given min x value.
 * @param xValue The value to set as min x.
 * @throws Error if xValue is not a number.
 */
DragHelper.prototype.setXMin = function(xValue) {
	assertIsNumber(xValue, "Min x value to set is not a number: " + xValue);
	this.xMin = xValue;
}


/**
 * Set the max x value, no elements moved will get there x value set
 * to a larger value than the given x value. 
 * @param xValue The value to set as max x.
 * @throws Error if xValue is not a number. 
 */
DragHelper.prototype.setXMax = function(xValue) {
	assertIsNumber(xValue, "Max x value to set is not a number: " + xValue);
	this.xMax = xValue;
}
 
 
/**
 * Add a exception to the dragging, if the target of the drag event is
 * one of the exception the dragging is aborted.
 * @param element The html element to add. 
 * @throws Error if element is undefined.
 */
DragHelper.prototype.addDragException = function(element) {
	assertDefined(element, "Element is undefined");
	
	this.exceptions[this.exceptions.length] = element;
}


// implementation


/**
 * Creates the function to use to listen to mouse up events. 
 * @return The function to set as mouse up listener. 
 */
DragHelper.prototype.createMouseUpListener = function() {
	var thisObject = this;
	return function(e) { thisObject.upHandler(e)};
}


/**
 * Creates the function to use to listen for mouse move events. 
 * @return The function to set as mouse move listener. 
 */
DragHelper.prototype.createMouseMoveListener = function() {
	var thisObject = this;
	return function(e) { thisObject.moveHandler(e)};
}


/**
 * Returns true if the target of this events is one of the exception elements.
 * @param event The event to check.
 */
DragHelper.prototype.isException = function(event) {
	var target = this.getEventTarget(event);
	for (var i = 0; i < this.exceptions.length; i++) {
		if (target == this.exceptions[i]) {
			return true;
		}
	}
	return false;
}


/**
 * Initialises the drag helper and prepares it to for dragging.
 */
DragHelper.prototype.initDragHelper = function() {
	if (!isDefined(this.dragHelper)) {
		this.createDragHelper();
	}
	
	this.fitDragHelper();
	this.originalElement = this.dragElement;
	this.dragElement = this.dragHelper;
	this.setDragHelperZIndex();
	this.showDragHelper()
	this.useDragHelper = true;
	this.dragHelperInited = true;
}
 
 
/**
 * Create the drag helper element.
 */
DragHelper.prototype.createDragHelper = function() {
	this.dragHelper = document.createElement("div");
	setClass(this.dragHelper, "dragHelper");
	document.body.appendChild(this.dragHelper);
}


/**
 * Make the drag helper fit a element that is to be dragged. The position
 * of the drag helper is set to be the position of the element, and the 
 * size is set to be the size of the element.
 */
DragHelper.prototype.fitDragHelper = function() {
	this.dragHelper.style.height = this.dragElement.offsetHeight + "px";
	this.dragHelper.style.width = this.dragElement.offsetWidth + "px";

	this.dragHelper.style.left = (parseInt(this.dragElement.style.left) - DRAG_HELPER_BORDER_SIZE) + "px";
	this.dragHelper.style.top = (parseInt(this.dragElement.style.top) - DRAG_HELPER_BORDER_SIZE) + "px";
}


/**
 * Set the z value of the drag helper in a way that it appears 
 * in front of the element to move.
 * @param element The element to make the drag helper appear above.
 */
DragHelper.prototype.setDragHelperZIndex = function() {
	if (this.originalElement.style.zIndex) {
		var zIndex = parseInt(this.originalElement.style.zIndex) + 1;
		}
	else {
		zIndex = 1;
	}

	this.dragHelper.style.zIndex = zIndex;
}


/**
 * Moves a element to the position of the drag helper.
 * @param element The element to move.
 */
DragHelper.prototype.moveElementToHelper = function(element) {
	if (isDefined(this.dragHelper)) {
		this.originalElement.style.left = (parseInt(this.dragHelper.style.left) + DRAG_HELPER_BORDER_SIZE) + "px";
		this.originalElement.style.top = (parseInt(this.dragHelper.style.top) + DRAG_HELPER_BORDER_SIZE) + "px";
	}
}


/**
 * Hides the dragHelper
 */
DragHelper.prototype.hideDragHelper = function() {
	if (isDefined(this.dragHelper)) {
		this.dragHelper.style.display = "none";
		this.dragHelper.style.zIndex = 0;
		this.dragHelperInited = false;
	}
}


/**
 * Shows the drag helper.
 */
DragHelper.prototype.showDragHelper = function() {
	this.dragHelper.style.display = "inline";
	setClass(this.dragHelper, "dragHelper");
}


/**
 * Returns true if a object is a window.
 */
function isWindow(objectToCheck) {
	try {
		if (objectToCheck instanceof Window) {
			return true;
		}		
	}
	catch (Error) {
		return false;
	}
	return false;
}


/**
 * Returns true if a object is a ProbeWindow.
 */
function isProbeWindow(objectToCheck) {
	try {
		if (objectToCheck instanceof ProbeWindow) {
			return true;
		}		
	}
	catch (Error) {
		return false;
	}
	return false;
}