/**
 * This class shows a dropdown menu shown on response to a onclick of a given html element. 
 */
 
 
/**
 * The class name set for the drop down element when it is shown.
 */
DropDown.DROPDOWN_CLASS = "dropDownMenu";


/**
 * The class name set for the drop down element when it is hidden. 
 */
DropDown.HIDDEN_DROPDOWN_CLASS = "inactive";


/**
 * The event fired when a dropdown menu is shown.
 */
DropDown.SHOWN_EVENT = "dropDownShown";


/**
 * The event fired when a dropdown menu is hidden. 
 */
DropDown.HIDDEN_EVENT = "dropDownHidden";


// constuction


/**
 * Constructs a instance of the dropdown menu.
 * @param triggerElement The element on whose onclick will show this dropdown. The onclick event is set by this class. 
 */
function DropDown(triggerElement) {
	this.triggerElement = triggerElement;
	this.setOnClick();
	
	// the html code of this dropdown
	this.htmlCode;
	
	// the dropdown sections in this menu
	this.sections = new Array();
	
	// the parent node to use for this dropdown menu
	this.parentNode;
	
	// the current parent node,when the parent node of the menu is changed this differs from
	// this.parentNode until the menu is repainted
	this.currentParentNode;
	
	// the current highlighted element
	this.highlightedElement;
	
	// the keydown event handler function used
	this.keyDownHandlerFunction;
	
	// the onclick handler used to select elements
	this.onClickHandlerFunction;
	
	// listener functions called when a element has been selected, a element can be selected by
	// a mouse click or a enter pressed event
	this.selectedListeners = new Array();
}


// services


/**
 * Shows this dropdown menu.
 */
DropDown.prototype.show = function() {
	if (!exists(this.htmlCode)) {
		this.repaint();
	}
	setClass(this.htmlCode, DropDown.DROPDOWN_CLASS);
	this.startKeyListening();
	this.setSelectOnClick();
	this.fireShownEvent();
}


/**
 * Hides this dropdown menu.
 */
DropDown.prototype.hide = function() {
	if (!exists(this.htmlCode)) {
		return;
	}
	setClass(this.htmlCode, DropDown.HIDDEN_DROPDOWN_CLASS);
	this.stopKeyListening();
	this.stopSelectMouseClickListening();
	this.fireHiddenEvent();
}


/**
 * Checks if this dropdown is currently shown. 
 * @returns True if the dropdown is shown, false otherwise.  
 */
DropDown.prototype.isShown = function() {
	return exists(this.htmlCode) ? getClassName(this.htmlCode) == DropDown.DROPDOWN_CLASS : false;
}


/**
 * Set a dropdown section to be shown in this dropdown menu. 
 * @param section The section to add. 
 * @throws Error if section is undefined, null or not a dropdown section. 
 */
DropDown.prototype.addSection = function(section) {
	assertExists(section, "The section to add to the dropdown menu does not exist");
	if (!(section instanceof DropDownSection)) {
		throw new Error("Section to add is not a dropdown section");
	}
	this.sections[this.sections.length] = section;
}


/** 
 * Gets all the sections in this dropdown menu.
 * @return A array containing the sections, empty array if none set. 
 */
DropDown.prototype.getSections = function(){
	return this.sections; 
}


/**
 * Gets the html code of this dropdown menu.
 * @return The html code for this dropdown menu, null the menu has not been painted yet. 
 */
DropDown.prototype.getHtmlCode = function() {
	return exists(this.htmlCode) ? this.htmlCode : null;
}


/**
 * Paints this dropdown menu. 
 */
DropDown.prototype.paint = function() {
	this.createHtmlCode();
	this.currentParentNode = this.getParentNode();
	this.getParentNode().appendChild(this.htmlCode);
}


/**
 * Repaints this dropdown menu.  
 */
DropDown.prototype.repaint = function() {
	var shown = this.isShown();
	if (exists(this.htmlCode)) {
		this.currentParentNode.removeChild(this.htmlCode);
	}
	this.paint();
	// repainting removes the highlight from the html code, redo the highlight
	var highlightedElement = this.highlightedElement; 
	this.highlightedElement = null;
	if (this.containsElement(highlightedElement)) {
		this.highlightElement(highlightedElement);
	}
	
	if (shown) {
		this.show();
	}
}


/**
 * Sets a parent node for this dropdown menu. The menu is appended to this node. 
 * @param parentNode The parentNode to attach the menu to, null or undefined to clear the parent node. 
 */
DropDown.prototype.setParentNode = function(parentNode) {
	this.parentNode = parentNode;
	this.repaint();	
}


/**
 * Highlight a given element. 
 * @param element The element to highlight. 
 * @throws Error if element is undefined or null. 
 */
DropDown.prototype.highlightElement = function(element) {
	assertExists(element, "The element to highlight does not exist");
	if (!this.containsElement(element)) {
		return;
	}
	this.updateHighlight(element);
}


/**
 * Highlights a element at a given index. 
 * @param index The index of the element to highlight starting at 0.
 * @throws Error if index is negative or larger than the element count in this dropdown menu. 
 */
DropDown.prototype.highlightElementAt = function(index) {
	assertNotNegative(index, "Index of the element to highlight is negative");
	if (index > this.getElementCount() - 1) {
		throw new Error("Index larger than element count");
	}
	
	var counter = 0;
	for (var i = 0; i < this.sections.length; i++) {
		if (index - counter <= this.sections[i].getElementCount() - 1) {
			this.highlightElement(this.sections[i].getElementAt(index - counter));
			return;
		}
		counter = this.sections[i].getElementCount();
	}
}



/**
 * Gets the selected element.
 * @return The selected element, null if no element selected. 
 */
DropDown.prototype.getHighlightedElement = function() {
	return exists(this.highlightedElement) ? this.highlightedElement : null;
}


/**
 * Gets the index of the highlighted element. 
 * @return The index of hte highlighted element, -1 if no elemenet is highlighted. 
 */
DropDown.prototype.getHighlightedIndex = function() {
	var index = 0;
	for (var i = 0; i < this.sections.length; i++) {
		if (this.sections[i].getElementIndex(this.highlightedElement) > -1) {
			index += this.sections[i].getElementIndex(this.highlightedElement);
			break;
		}
		index += this.sections[i].getElementCount();
	}
	return exists(this.highlightedElement) ? index : -1;
}


/**
 * Checks whether the dropdown menu contains a given element.
 * @param element The element to check.
 * @return True if the element is in the dropdown menu, false otherwise.  
 */
DropDown.prototype.containsElement = function(element) {
	for (var i = 0; i < this.sections.length; i++) {
		var index = this.sections[i].getElementIndex(element);
		if (index > -1) {
			return true;
		}
	}
	return false;
}


/**
 * Moves the highlighted element one up. 
 */
DropDown.prototype.moveHighlightUp = function() {
	if (!exists(this.highlightedElement) || !exists(this.getHtmlCode())) {
		// no selected element
		return;
	}
	
	var section = this.getSection(this.highlightedElement);
	if (this.getPreviousElement(this.highlightedElement)) {
		// highlight the previous element in this section
		this.updateHighlight(this.getPreviousElement(this.highlightedElement));
	}
	else if (this.getPreviousSection(section)) {
		// highlight the last element in the section before the current
		section = this.getPreviousSection(section);
		this.updateHighlight(section.getElementAt(section.getElementCount() - 1));	
	}
}


/**
 * Moves the selection one element down. 
 */ 
DropDown.prototype.moveHighlightDown = function() {
	if (!exists(this.getHtmlCode()) || !exists(this.getHtmlCode())) {
		// no selected element
		return;
	}
	if (!exists(this.highlightedElement)) {
		// no element yet highlighted, highlight the first element
		this.updateHighlight(this.sections[0].getElementAt(0));
		return;
	}
	
	var section = this.getSection(this.highlightedElement);
	if (exists(this.getNextElement(this.highlightedElement))) {
		// highlight the next element in this section 
		this.updateHighlight(this.getNextElement(this.highlightedElement));
	}
	else if (exists(this.getNextSection(section))) {
		// highlight the first element in the next section
		this.updateHighlight(this.getNextSection(section).getElementAt(0));
	}
	
}


/**
 * Sets a selected listener. These listener functions are called when a element has been selected. 
 * A element can be selected by pressing the enter on a selected element, or with a mouse click.
 * The selected element is provided as a parameter to these funtions. 
 * @param selectedFunction The selected function to set. 
 * @throws Error if selectedFunction is undefined, null or not a function. 
 */
DropDown.prototype.setSelectedListener = function(listener) {
	assertExists(listener, "The listener to set is undefined or null");
	if (!(typeof listener == "function")) {
		throw new Error("Listener to set is not a function");
	} 
	this.selectedListeners.push(listener);
}


/**
 * Clears this dropdown menu. 
 */
DropDown.prototype.clear = function() {
	this.sections = new Array();
	if (exists(this.htmlCode)) {
		this.getParentNode().removeChild(this.htmlCode);
	}
	this.htmlCode = null;
	this.highlightedElement = null;
	this.stopKeyListening();
	this.stopSelectMouseClickListening();
}


/**
 * Gets the element count in the dropdown menu. 
 * @return The number of elements in this dropdown. 
 */ 
DropDown.prototype.getElementCount = function(){
	var counter = 0;
	for (var i = 0; i < this.sections.length; i++) {
		counter += this.sections[i].getElementCount();
	}
	return counter;
}

 
/**
 * Gets all the elements in this section. 
 * @return A array containing all the elements in this dropdown, empty array if none.  
 */
DropDown.prototype.getElements = function() {
	var elements = new Array();
	for (var i = 0; i < this.sections.length; i++) {
		elements = elements.concat(this.sections[i].getElements());
	}
	return elements;
}


// implementation


/**
 * Gets the previous section. 
 * @param section The section to get the previous of. 
 * @return The section before section, null if section is the first section.
 */
DropDown.prototype.getPreviousSection = function(section) {
	var sectionIndex = this.getSectionIndex(section);
	if (sectionIndex > 0) {
		return  this.sections[sectionIndex - 1];
	}
	return null;
}


/**
 * Gets the next section.
 * @param section The section to get the next of. 
 * @return The section after section, null if section is the last section. 
 */
DropDown.prototype.getNextSection = function(section) {
	var sectionIndex = this.getSectionIndex(section);
	if (sectionIndex <= this.sections.length - 2) {
		return this.sections[sectionIndex + 1];
	}
	return null;
		
}


/**
 * Gets the previous element in the section element is in. 
 * @param element The element to get the previous of.
 * @return The element before element in the section element is in, null if element is the first element in the section. 
 */
DropDown.prototype.getPreviousElement = function(element) {
	var section = this.getSection(element);
	var index = exists(section) ? section.getElementIndex(element) : -1; 
	if (index > 0) {
		return section.getElementAt(index - 1);
	}
	return null;
}


/**
 * Gets the next element in the section element is in. 
 * @param element The element to get the next of. 
 * @return The element after element in the section element i in, null if element is the last element in the section. 
 */
DropDown.prototype.getNextElement = function(element) {
	var section = this.getSection(element);
	var index = section.getElementIndex(element);
	if (index <= section.getElementCount() - 2) {
		return section.getElementAt(index + 1);
	}
	return null;
}


/**
 * Gets the index of a given section.
 * @param section The section to get the index for. 
 * @return The index of the section, -1 if not exist. 
 */
DropDown.prototype.getSectionIndex = function(section) {
	for (var i = 0; i < this.sections.length; i++) {
		if (this.sections[i] == section) {
			return i;
		}
	}
	return -1;
}


/**
 * Updates the highlighted element with a given element. 
 * @param element The element to highlight. 
 */
DropDown.prototype.updateHighlight = function(element) {
	if  (element == this.highlightedElement) {
		return;
	}
	if (exists(this.highlightedElement)) {
		this.getSection(this.highlightedElement).deHighlightElement(this.highlightedElement);	 
	}
	
	this.getSection(element).highlightElement(element);
	this.highlightedElement = element;
}


/**
 * Gets the section a element is in. 
 * @param element The element to check for. 
 * @return The section the element is in, null if element is in no section. 
 */
DropDown.prototype.getSection = function(element) {
	for (var i = 0; i < this.sections.length; i++) {
		if (this.sections[i].getElementIndex(element) > -1) {
			return this.sections[i];
		}
	}
	return null;
}


/**
 * Gets the node to use as the parent node of the dropdown menu in the html file. 
 * @return The node to use as the parent node for this dropdown menu. 
 */
DropDown.prototype.getParentNode = function() {
	return exists(this.parentNode) ? this.parentNode : document.body;
}


/**
 * Adds the onclick event handlers used in this dropdown component. 
 */
DropDown.prototype.setOnClick = function() {
	if (exists(this.triggerElement)) {
		this.setTriggerElementOnClick();
	}
	this.setSelectOnClick();
}


/**
 * Sets the onclick handler used to listen to onclick events to the elements. 
 */
DropDown.prototype.setSelectOnClick = function() {
	this.onClickHandlerFunction = this.getOnClickListenerFunction();
	registerEventHandler("click", this.onClickHandlerFunction);
}


/**
 * Sets the onclick event handlers used. When the trigger element is clicked the dropdown is shown.  
 */
DropDown.prototype.setTriggerElementOnClick = function() {
	var thisObj = this;
	registerEventHandler("click", function() {
		if (!thisObj.isShown()) {
			thisObj.repaint();
			thisObj.show();
		}
		else {
			thisObj.hide();
		}
	}, this.triggerElement);
}


/**
 * Stops the listening to mouse clicks to select elements. This does not stop the 
 * listening to mouse clicks on the trigger element. 
 */
DropDown.prototype.stopSelectMouseClickListening = function() {
	if (exists(this.onClickHandlerFunction)) {
		removeEventHandler("click", this.onClickHandlerFunction);
	}
}


/**
 * Creates the html code for this dropdown menu. 
 */
DropDown.prototype.createHtmlCode = function() {
	this.htmlCode = document.createElement("div");
	for (var i = 0; i < this.sections.length; i++) {
		this.sections[i].repaint();
		this.htmlCode.appendChild(this.sections[i].getHtmlCode());
	}
}


/**
 * Handles key down events. 
 * @param event The keydown event to process. 
 */
DropDown.prototype.keyDownHandler = function(event) {
	event = exists(event) ? event : window.event;
	
	if (this.upArrowPressed(event)){
		this.moveHighlightUp();	
	}
	else if (this.downArrowPressed(event)) {
		this.moveHighlightDown();
	}
	else if (this.escapePressed(event)) {
		this.hide();
	}
	else if (this.enterPressed(event)) {
		this.hide();
		this.callListenerFunctions();
	}
}


/**
 * Handles mouse click events. 
 * @param event The mouse click vent to process. 
 */
DropDown.prototype.mouseClickHandler = function(event) {
	event = exists(event) ? event : window.event;
	var src = exists(event.srcElement) ? event.srcElement : event.target;
	if (this.isTriggerElement(event)) {
		// the trigger element has a separate listener set
		return;
	}
	
	if (this.isClickOnElement(event)) {
		this.callListenerFunctions();	
	}
	this.hide();
}


/**
 * Starts listening to key events. 
 */
DropDown.prototype.startKeyListening = function() {
	if (exists(this.keyDownHandlerFunction)) {
		// already listening to key down events
		return;
	}
	// use keydown for better usability
	this.keyDownHandlerFunction = this.getKeyListenerFunction();
	registerEventHandler("keydown", this.keyDownHandlerFunction);
}


/**
 * Stops the key listening. 
 */
DropDown.prototype.stopKeyListening = function() {
	if (exists(this.keyDownHandlerFunction)) {
		removeEventHandler("keydown", this.keyDownHandlerFunction);
		this.keyDownHandlerFunction = null;
	}
}


/**
 * Gets the function used to listen to select element mouse clicks. 
 * @return The function to set as a listener to mouse clicks to select elements. 
 */
DropDown.prototype.getOnClickListenerFunction = function() {
	var thisObj = this;
	return function(event) {
		thisObj.mouseClickHandler(event);
	}; 	
}

 
/**
 * Gets the function used to respond to key events.
 * @return The function to set as the listener to key events. 
 */
DropDown.prototype.getKeyListenerFunction = function() {
	var thisObj = this;
	return function(event) {
		thisObj.keyDownHandler(event);
	};
}


/**
 * Returns true if a keyevent is for the down arrow. 
 * @param event The event to check. 
 * @return True if the event is a key event for the down arrow, false otherwise. 
 */
DropDown.prototype.downArrowPressed = function(event) {
	if (40 == event.keyCode) {
		return true;
	}
	return false;
}


/**
 * Returns true if a key event is for the up arrow. 
 * @param event The event to check.
 * @return True if the event is a key event for the up arrow, false otherwise. 
 */
DropDown.prototype.upArrowPressed = function(event) {
	if (38 == event.keyCode) {
		return true;
	}
	return false;
}


/**
 * Returns true if a key event is for the escape button.
 * @param event The event to check. 
 * @return True if the event is a key event for the escape key, false otherwise.
 */
DropDown.prototype.escapePressed = function(event) {
	if (27 == event.keyCode) {
		return true;
	}
}	


/**
 * Returns true if a key event is for the enter button.
 * @param event The event to check.
 * @return True if the event is a key event for the enter key, false otherwise. 
 */
DropDown.prototype.enterPressed = function(event) {
	if (13 == event.keyCode) {
		return true;
	}
	return false;
}


/**
 * Calls the listener functions. 
 */
DropDown.prototype.callListenerFunctions = function() {
	if (!exists(this.getHighlightedElement())) {
		// no element is selected
		return;
	}
	
	for (var i = 0; i < this.selectedListeners.length; i++){
		this.selectedListeners[i](this.getHighlightedElement());
	}
}


/**
 * Returns true if a mouse click event has one of the elements in this dropdown 
 * menu as a traget false otherwise. 
 * @param event The event to test. 
 */
DropDown.prototype.isClickOnElement = function(event) {
	var src = exists(event.srcElement) ? event.srcElement : event.target;
	// on click events can happen on any of the elements child nodes, iterate up the
	// tree until we find a element node
	if (this.findElementCode(src)) {
		return true;
	}
	return false;
}


/**
 * The target element of mouse clicks can be any of the elemtns child nodes. Traverse up 
 * the tree to find a element node. 
 * @param node The node from where to start traversing up the tree. 
 * @return The html code of the element found, null if node is not a element or a child node of a element. 
 */
DropDown.prototype.findElementCode = function(node) {
	while(true) {
		if (!exists(node.parentNode)) {
			break;
		}
		
		if (getClassName(node) == DropDownSection.ELEMENT_CLASS || getClassName(node) == DropDownSection.HIGHLIGHTED_ELEMENT_CLASS) {
			return node;
		}
		node = node.parentNode;
	}
	return null;
}


/**
 * Checks if a mouse click event is for the trigger element orone of its childs.
 * @param event The event to check.
 * @return True if the mousclick event is for the trigger element or for one of the
 * trigger elements child nodes, false otherwise.  
 */
DropDown.prototype.isTriggerElement = function(event) {
	var src = exists(event.srcElement) ? event.srcElement : event.target;
	if (this.triggerElement == src) {
		return true;
	} 
	
	while(true) {
		if (!exists(src.parentNode)) {
			break;
		}
		
		if (this.triggerElement == src) {
			return true;
		}
		src = src.parentNode;
	}
	return false;
}


/**
 * Firees a dropdown menu shown event. 
 */
DropDown.prototype.fireShownEvent = function() {
	if (this.eventsAvailable()) {
		getEventServer().notifyListeners(new Event(DropDown.SHOWN_EVENT, this));
	}
}


/**
 * Fires a dropdown menu hidden event. 
 */
DropDown.prototype.fireHiddenEvent = function() {
	if (this.eventsAvailable()) {
		getEventServer().notifyListeners(new Event(DropDown.HIDDEN_EVENT, this));
	}
}


/**
 * Returns true if a EventServer is available.
 */
DropDown.prototype.eventsAvailable = function() {
	try {
		if (isDefined(getEventServer)) {
			return true;
		}
	}
	catch(Error) {
		return false;
	}
	try {
		if (isDefined(Event)) {
			return true;
		}
	}
	catch(Error) {
		return false;
	}
	return true;
}