/**
 * This class is one section that can be added to a dropdown menu. Each section can contain
 * multiple elements and one header. 
 */
 
 
/**
 * The class set for the dropdown section.
 */
DropDownSection.SECTION_CLASS = "dropDownSection";


/**
 * The class to set for the section elements. 
 */
DropDownSection.ELEMENT_CLASS = "dropDownElement";


/**
 * The class to set for the highlighted section elements. 
 */
DropDownSection.HIGHLIGHTED_ELEMENT_CLASS = "highlightedDropDownElement";


/**
 * The class to set for the section header. 
 */
DropDownSection.HEADER_CLASS = "dropDownSectionHeader";


/**
 * Constructs a instance of this class. 
 * @param elements A array containing the html code of the elements to be in this section (no text nodes). 
 * @param dropDown The dropdown menu this section is part of. 
 * @throws Error if elements is undefined or null.
 * @throws Error if elements is not array or an array of length 0. 
 * @throws Error if one of the elemnts is not html node. 
 * @throws Error if dropDown is undefined or null. 
 */
function DropDownSection(elements, dropDown) {
	assertExists(dropDown, "The drop down section to set is undefined null or empty");
	// the elements in this section
	this.elements = new Array();
	this.setElements(elements);
	
	this.dropDown = dropDown;
	
	// the header to use for this section
	this.header;
	
	// the html code for this section
	this.htmlCode;
	
	// array containing the html code for each element
	this.elementHtmlCode = new Array();
}


// services 


/**
 * Sets the header to use in this section. 
 * @param header The header to use, null or undefined if no is to be used. 
 */
DropDownSection.prototype.setHeader = function(header) {
	this.header = header;
}


/**
 * Get the header set for this section.
 * @return The header set. 
 */
DropDownSection.prototype.getHeader = function() {
	return this.header; 
}


/**
 * Set the dropdown elements. 
 * @param elements A array containing the html code of the elements to be in this section (no text nodes). 
 * @throws Error if elements is undefined or null.
 * @throws Error if elements is not array or an array of length 0. 
 * @throws Error if one of the elemnts is not html node. 
 */
DropDownSection.prototype.setElements = function(elements) {
	assertExists(elements, "The elements to set are undefined or null");
	if (!(elements instanceof Array) || elements.length == 0) {
		throw new Error("Elements to set is not a array or a empty array");
	} 
	
	this.elements = elements;
}


/**
 * Adds a element to this section.
 * @param element The html code of the element to add (no text nodes). 
 * @throws Error if element is undefined or null. 
 * @throws Error if the element is not html node or a string. 
 */
DropDownSection.prototype.addElement = function(element) {
	assertExists(element, "The drop down element to add does not exist");
	if (element instanceof String && isEmpty(element)) {
		throw new Error("The element to add to the dropdown section is empty");
	}
		
	this.elements[this.elements.length] = element;
}


/**
 * Gets the elements in this section.
 * @return A array containing the elements set in this section. 
 */
DropDownSection.prototype.getElements = function() {
	return this.elements;
}


/**
 * Returns the number of elements in this section. 
 */
DropDownSection.prototype.getElementCount = function() {
	return this.elements.length;
}


/**
 * Gets the html code for this section. 
 * @return The html code for this section, null if not painted yet or no elements added. 
 */
DropDownSection.prototype.getHtmlCode = function() {
	return exists(this.htmlCode) ? this.htmlCode : null;	
}


/**
 * Gets the html code used for a given element. 
 * @param element The element to get the html code for. 
 * @return The html code used for the given element, null if the section has not been painted 
 * yet, or element is not an element in this section 
 * @throws Error if element is undefined, null. 
 */
DropDownSection.prototype.getElementHtmlCode = function(element) {
	assertExists(element, "The element get the html code for is undefined or null");
	var index = this.getElementIndex(element);
	if (index > -1 && index <= this.elementHtmlCode.length - 1) {
		return this.elementHtmlCode[index];
	}	
	return null;
}


/**
 * Gets the index of a given element. 
 * @param element The element to check.
 * @return The index of element in this section, false if undefined.  
 */
DropDownSection.prototype.getElementIndex = function(element) {
	for (var i = 0; i < this.elements.length; i++){
		if (this.elements[i] == element) {
			return i;
		}	
	}
	return -1;
}


/**
 * Gets a element with a given index. 
 * @param index the index to get the element for. 
 * @return The element at the given index. 
 * @throws Error if index is negative, or larger than the element count.
 */
DropDownSection.prototype.getElementAt = function(index) {
	assertNotNegative(index, "The index to get is negative");
	if (index > this.elements.length - 1) {
		throw new Error("Index larger than element count: " + index);
	}
	return this.elements[index];
}


/**
 * Highlights a given element in this section.
 * @param element The element to highlight. 
 * @throws Error if element is undefined or null.
 */
DropDownSection.prototype.highlightElement = function(element) {
	assertExists(element, "The element to highlight is undefined null or empty");
	if (this.getElementIndex(element) == -1 || !exists(this.elementHtmlCode[this.getElementIndex(element)])) {
		return;
	}
	var index = this.getElementIndex(element);
	if (index < this.elementHtmlCode.length && exists(this.elementHtmlCode[index])) {
		// the section might not be painted yet
		setClass(this.elementHtmlCode[index], DropDownSection.HIGHLIGHTED_ELEMENT_CLASS);
	}
}


/**
 * Removes the highlight from a given element in this section.
 * @param element The element to remove the highlight from. 
 * @throws Error if element is undefind or null.
 */
DropDownSection.prototype.deHighlightElement = function(element) {
	assertExists(element, "The element to dehighlight is undefined null or empty");
	if (this.getElementIndex(element) == -1) {
		return;
	}
	var index = this.getElementIndex(element);
	if (index < this.elementHtmlCode.length && exists(this.elementHtmlCode[index])) {
		// the section might not be painted yet
		setClass(this.elementHtmlCode[index], DropDownSection.ELEMENT_CLASS);		
	}
}


/**
 * Paints the section. 
 */
DropDownSection.prototype.paint = function() {
	if (this.elements.length == 0) {
		return;
	}
	this.createHtmlCode();
}


/**
 * Repaints the section. 
 */
DropDownSection.prototype.repaint = function() {
	this.htmlCode = null;
	this.paint();
}


// implementation


/**
 * Returns true if a header is set. 
 * @returns True if a header is set for this section, false otherwise. 
 */
DropDownSection.prototype.isHeaderSet = function() {
	return exists(this.header);
}


/**
 * Creates the html code for this section. 
 * @return The html code for this section. 
 */
DropDownSection.prototype.createHtmlCode = function() {
	this.htmlCode = document.createElement("div"); 
	setClass(this.htmlCode, DropDownSection.SECTION_CLASS);
	if (this.isHeaderSet()) {
		this.htmlCode.appendChild(this.createHeaderCode());
	}
	this.createElementsHtmlCode(this.htmlCode);
}


/**
 * Creates the html code for this section. 
 * @param container The html node to append the elements to. 
 */
DropDownSection.prototype.createElementsHtmlCode = function(container) {
	for (var i = 0; i < this.elements.length; i++) {
		var elementCode = this.createElementCode(this.elements[i]);
		setClass(elementCode, DropDownSection.ELEMENT_CLASS);
		this.elementHtmlCode[i] = elementCode;
		container.appendChild(elementCode);
	}
}


/**
 * Creates the html code for the header. 
 * @return The html code for the header of this section, null if no header set.
 */
DropDownSection.prototype.createHeaderCode = function() {
	if (!this.isHeaderSet()) {
		return null;
	}
	var headerCode = this.createElementCode(this.header);
	setClass(headerCode, DropDownSection.HEADER_CLASS);
	return headerCode;
}


/**
 * Create the html code for a element. 
 * @param element The element to create the html code for. 
 * @return The html code to create the element for. 
 */ 
DropDownSection.prototype.createElementCode = function(element) {
	var container = document.createElement("div");
	if (exists(element.nodeType) && element.nodeType == 3) {
		// text nodes need to be in a container so we can add classes to it
		container.appendChild(element);
	}
	else if (typeof element == "string" || element instanceof String) {
		// detects both string literals ("a string") and string objects (new String("a string"))
		container.appendChild(document.createTextNode(element));
	}
	else if (!exists(element.nodeType)) {
		// element is a javascript object like a Date or a custom object, use the objects
		// toString method to convert it to a string
		container.appendChild(document.createTextNode(element.toString()));
	}
	else {
		// element is already a html node
		container = element;
	}
	
	this.setOnMouseOver(element, container)
	return container;
}


/**
 * Sets the on mouse over handler for a given element. 
 * @param element The element to set the onmouseover handler for. 
 * @param code The html code for the element.
 */
DropDownSection.prototype.setOnMouseOver = function(element, code) {
	var thisObj = this;
	code.onmouseover = function() {
		thisObj.dropDown.highlightElement(element);
	}
}