/**
 * Returns the client's horizontal screen resolution.
 */
function getScreenWidth() {
	return screen.width;
}


/**
 * Returns the client's vertical screen resolution.
 */
function getScreenHeight() {
	return screen.height;
}


/**
 * Get the height of a window.
 * @param window The window to get the height for.
 */
function getWindowHeight(window) {
	var height = window.innerHeight;

	if (!isDefined(height)) {
		// IE
		height = window.document.documentElement.clientHeight;
	}
	return height;
}


/**
 * Get the widht of a window.
 * @param window The window to get the width for.
 */
function getWindowWidth(window) {
	var width = window.innerWidth;
	if(!isDefined(width)) {
		// IE
		width = window.document.documentElement.clientWidth;
	}
	return width;
}


/**
 * Returns the first html input element with a given name.
 * @param code The html code to search.
 * @param name The name of the input field to get. 
 * @return The first html input tag in code with the given name, null if not found.
 * @throws Error if code is undefined or null.
 * @throws Error if name is undefined, null or empty.
 */
function getInput(code, name) {
	assertExists(code, "Code to search is undefined or null");
	assertNotEmpty(name, "The name of the input to get is undefined or null");
	return getElementWithAttribute(code, name, "name", "input");
}


/**
 * Returns the first html element with a value for a attribute.
 * @param code The html code to search.
 * @param value The value of the attribute.
 * @param attribute The attribute containing the value.
 * @param elementType The type of the element to search for, i.e. td, table, img...
 * @return The element with the value for the attribute, null if not found.
 * @throws Error if code is undefined or null.
 * @throws Error if value is undefined or empty.
 * @throws Error if attribute is undefined or empty.
 * @throws Error if elementType is undefined or empty. 
 */
function getElementWithAttribute(code, value, attribute, elementType) {
	assertExists(code, "Code to search is undefined or null");
	assertNotEmpty(value, "The value is undefined or empty");
	assertNotEmpty(attribute, "The attribute is undefined or empty");
	assertNotEmpty(elementType, "The element type is undefined or empty");
	
	var elements = code.getElementsByTagName(elementType);
	var len = elements.length;
	for (var i = 0; i < len; i++) {
		var elementValue = elements[i].getAttribute(attribute);
		if (attribute.indexOf("class") > -1) {
			elementValue = getClassName(elements[i]);
		}
		if (value == elementValue) {
			return elements[i];
		}
	}
	return null;
}


/**
 * Returns all html element with a value for a attribute.
 * @param code The html code to search.
 * @param value The value of the attribute.
 * @param attribute The attribute containing the value.
 * @param elementType The type of the element to search for, i.e. td, table, img...
 * @return A array containin the elements with the value for the attribute.
 * @throws Error if code is not defined.
 * @throws Error if value is undefined or empty.
 * @throws Error if attribute is undefined or empty.
 * @throws Error if elementType is undefined or empty. 
 */
function getElementsWithAttribute(code, value, attribute, elementType) {
	assertDefined(code, "Code to search in is not defined");
	assertNotEmpty(value, "The value is undefined or empty");
	assertNotEmpty(attribute, "The attribute is undefined or empty");
	assertNotEmpty(elementType, "The element type is undefined or empty");
	
	var elements = code.getElementsByTagName(elementType);
	var foundElements = new Array();
	var len = elements.length;
	for (var i = 0; i < len; i++) {
		if (value == elements[i].getAttribute(attribute)) {
			foundElements[foundElements.length] = elements[i];
		}
	}
	return foundElements;
}


/**
 * Get all elements with a given class name.
 * @param elementType The type of html elements with class windowClass to transform into windows.
 * @param className The name of the class.
 * @return A array containing all the html elements with className as class. 
 * @throws Error if elementType is undefined, null or empty.
 * @throws Error if className is undefined, null or empty.
 */
function getElementsWithClass(elementType, className) {
	assertNotNull(elementType, "The element type to get is undefined, null or empty: " + elementType);
	assertNotNull(className, "The className to get elements with is unefined, null or empty: " + className);
	
	var elements = getElementsWithAttribute(document.body, className, "class", elementType);
	if (elements.length == 0) {
		// IE
		elements = getElementsWithAttribute(document.body, className, "className", elementType);
	}

	return elements;
}


/**
 * Sets the value of the message field. The message field must have the name or id "messageField". 
 * @param htmlCode The htmlCode for where to search for the "messageField" attribute.
 * @param attribute The attribute containing the "messageField" value.
 * @param elementType The type of the element to search for, i.e. td, table, img...
 * @param errorMessage The error message.
 * Optional parameter specifying the success message, if no error message was found.
 */
function setMessageField(htmlCode, attribute, elementType, errorMessage, successMessage) {
	// clear the previous message field first, if found
	clearPreviousMessage();
			
	var messageField = getElementWithAttribute(htmlCode, "messageField", attribute, elementType);				
	if (errorMessage != null && errorMessage.length > 0) {
		setClass(messageField, "error");
		messageField.innerHTML = errorMessage;
	}
	else if (successMessage != null && successMessage.length > 0) {
		setClass(messageField, "ok");
		messageField.innerHTML = successMessage;
	}
	previousMessage = messageField;
}


/**
 * Clears the previous message field, if it exists.
 */
function clearPreviousMessage() {
	if (isDefined(previousMessage)) {
		previousMessage.innerHTML = "";
	}
}


/**
 * Set the class of html tag.
 * @param tag The tag to set the code for.
 * @param class The class to set.
 * @throws Error if tag is undefined or null.
 * @throws Error if class is undefined or null. 
  */
function setClass(tag, className) {
	assertExists(tag, "tag to set class for is not defined or null");
	assertExists(className, "The class to set is undefined or null");
	
	tag.setAttribute("class", className);
	tag.setAttribute("className", className);
}


/**
 * Gets the class of a html tag.
 * @param tag The tag to get the code for.
 * @return The class of the tag.
 */
function getClassName(tag) {
	var className = tag.getAttribute("class"); 
	if (!isDefined(className) || isNull(className)) {
		// IE
		className = tag.getAttribute("className");
	}
	return className;
}


/**
 * Calculates the top offset in pixels of a element from the top of the screen.
 * The offset can be calculate for all types of elements regarding how they
 * are positioned (absolute or relative).
 * @param element The element to calculate the offset for.
 * @param parentElement If defined the top offset is calculated until this parent element. 
 * @returns The top offset.
 * @throws Error if the element is undefined or null.
 */
function calculateTopOffset(element, parentElement) {
	assertDefined(element, "Element to calculate top ofset for is undefined or null");
	
	var offsetTop = element.offsetTop;
	var parent = element.offsetParent;
	while (true) {
		if (isNull(parent) || !isDefined(parent)) {
			break;
		}
		if (exists(parentElement) && parentElement == parent) {
			break;
		}
		
		var offset = parent.offsetTop;
		if (isDefined(offset) && !isNull(offset)) {
			offsetTop += offset;
		}
		parent = parent.offsetParent;
	}
	
	return offsetTop;
}


/**
 * Calculates the left offset in pixels of a element from the left of the screen.
 * The offset can be calculate for all types of elements regarding how they
 * are positioned (absolute or relative).
 * @param element The element to calculate the ofset for.
 * @param parentElement If defined the top offset is calculated until this parent element.
 * @returns The left offset.
 * @throws Error if the element is undefined or null.
 */
function calculateLeftOffset(element, parentElement) {
	assertDefined(element, "Element to calculate left ofset for is undefined or null");
	
	var offsetLeft = element.offsetLeft;
	var parent = element.offsetParent;
	while (true) {
		if (isNull(parent) || !isDefined(parent)) {
			break;
		}
		if (exists(parentElement) && parentElement == parent) {
			break;
		}

		var offset = parent.offsetLeft;
		if (isDefined(offset) && !isNull(offset)) {
			offsetLeft += offset;
		}
		parent = parent.offsetParent;
	}
	
	return offsetLeft;
}


/**
 * Appends all childs of a given source node to a destination node.
 * @param source The source node.
 * @param destination All child nodes of source are appended to this node.
 * @throws Error if source is undefined or null.
 * @throws Error if destination is undefined or null.
 */
function appendAll(source, destination) {
	assertDefined(source, "Source node is undefined or null: " + source);
	assertDefined(destination, "Destination is undefined or null: " + destination);
	
	if (!isDefined(source.childNodes) || isNull(source.childNodes)) {
		// the element has no child nodes
		return;
	}
	
	for (var i = source.childNodes.length - 1; i >= 0; i--) {
		var node = source.childNodes[i];
		source.removeChild(node);
		destination.appendChild(node);
	}
}


/**
 * Returns the tekst value of all the tekst nodes in a DOM node.
 * @param node The node to get the tekst content of.
 * @param optional argument, If this argument is given the tekst inside nodes with this name
 * are not included.
 */
function getTekstValue(node, name) {
	var tekst = "";
	if (!isDefined(node) || isNull(node)) {
		return tekst;
	}
	if (!node.hasChildNodes()) {
		return "";
	}
	if (isTekstNode(node)) {
		return node.nodeValue;
	}
	
	for (var i = 0; i < node.childNodes.length; i++) {
		var childNode = node.childNodes[i];
		if (isTekstNode(childNode) && !isEmpty(childNode.nodeValue)) {
			tekst += childNode.nodeValue;
		}
		else if (!isTekstNode(childNode) && isDefined(name) && childNode.getAttribute("name") == name) {
			continue;
		}
		else if (!isTekstNode(childNode)) {
			tekst += getTekstValue(childNode, name);
		}
	}
	return tekst;
}


/**
 * Returns true if a DOM node is a tekst node.
 * @param node The node to check.
 */
function isTekstNode(node) {
	if (node.nodeType == 3) {
		return true;
	}
	return false;
}


/**  
 * Finds the first td node going up the html node tree from a given source node. 
 * @param srcElement The source element to start the search from. 
 * @return The first td node found going up the node tree from srcElement, null if none found. 
 */
function findTdNode(srcElement) {
	try {
		if (!exists(srcElement)) {
			return null;
		}
		if (srcElement.tagName == "TD") {
			return srcElement;
		}
		var parentNode = srcElement.parentNode;
		var maxChildren = 10;
		var i = 0;
		// loop through all parent nodes until we find a table having the id attribute set
		while (true && i < maxChildren) {
			if (!exists(parentNode)) {
				break;
			}
			if (parentNode.tagName == "TD") {
				return parentNode;
			}
			parentNode = parentNode.parentNode;
			i++;
		}
	}
	catch (e) {
		// FF sometimes throws a Permission denied to get property tagname exception
		return null;
	}
	return null;
}