/**
 * This file can repeatedly call a function. Between each repetition there can be a given amount of time.
 */
 

/**
 * Constructs a repeater object. 
 * @param functionToCall The function to call repeatedly.
 * @param object The object to call the function on. 
 * @param time The time between each repetition.
 * @throws Error if functionToCall is undefined or null.
 * @throws Error if time is negative. 
 */
function Repeater(functionToCall, object, time) {
	assertDefined(functionToCall, "The function to call repeatedly is undefined or null");
	assertNotNegative(time, "The time between each repetition is negative: " + time);
	
	this.functionToCall = functionToCall;
	this.object = object;
	this.time = time;
	
	// if this is set to true the repeater is currently running
	this.repeating = false;
	
	// if this is set to true the repeater is currently delaying a call to a function
	this.delaying = false;
}


// services


/**
 * Set the time parameter for the repeater. 
 * @param time The time to set. 
 */
Repeater.prototype.setTime = function(time) {
	assertNotNegative(time, "Time to set is negative: " + time);
	
	this.time = time;
}


/**
 * Returns the time parameter for this repeater. 
 */
Repeater.prototype.getTime = function() {
	return this.time;	
}


/**
 * Delays the call to the provided function by the given amount of seconds. 
 */
Repeater.prototype.delay = function() {
	this.delaying = true;
	var thisObj = this;
    this.timeOutId = window.setTimeout(function(){ thisObj.delaying = false; thisObj.callFunction() }, (this.time * 1000));
}


/**
 * Starts the repeater. 
 */
Repeater.prototype.start = function() {
	if (!this.repeating) {
		this.repeating = true;
		this.repeat();
	}
}


/**
 * Stops any ongoing repeating or delaying.
 */
Repeater.prototype.stop = function() {
	this.repeating = false;
	this.delaying = false;
	if (isDefined(this.timeOutId) && !isNull(this.timeOutId)) {
		window.clearTimeout(this.timeOutId);
	}
}


/**
 * Returns true if the repeater is has been started and is running, false if
 * the repeater has been stopped or never has been started. 
 */
Repeater.prototype.isRepeating = function() {
	return this.repeating;
}


/**
 * Returns true if the repeaters is currently delaying a call to a function, false otherwise.
 */
Repeater.prototype.isDelaying = function() {
	return this.delaying;
}


/**
 * This method call functionToCall repeatedly with a given time
 * between each repitition.
 */
Repeater.prototype.repeat = function() {
	this.repeating = true;
	this.callFunction();
	var thisObj = this;
    this.timeOutId = window.setTimeout(function(){ thisObj.repeat() }, (this.time * 1000));
}


/**
 * Calls the function provided. The function is called as a method of the provided
 * object if a object is given. 
 */
Repeater.prototype.callFunction = function(functionToCall, object) {
	if (isDefined(this.object) && !isNull(this.object)) {
		this.functionToCall.call(this.object);
	}
	else {
		this.functionToCall();
	}	
}