// Handling the resizing of elements.


/**
 * The size of the border of the resizeHelper element. This is used to move the resize helper around 
 * the element instead of just above. 
 */
var RESIZE_HELPER_BORDER_SIZE = 2;


// construction


/**
 * Used to resize a element. 
 * @param elementToResize The element to resize.
 * @param event The event.
 * @param edge At which edge the element is been resized (bottom, top, left right, 
 * topLeft, topRight, bottomLeft, bottomRight);
 * @param useResizeHelper Optional argument. True if the resize helper is to be used, false otherwise.
 * @param minHeight Optional argument If this is set the elements height is not made smaller than minHeight.
 * @param minWidth Option argument If this is set the elements width is not made smaller than minWidth.
 * @throws Error if elementToResize is undefined or null.
 * @throws Error if edge is undefined or null.
 */
function beginResize(elementToResize, event, edge, useResizeHelper, minHeight, minWidth) {
	assertExists(elementToResize, "The element to resize does not exist");
	assertNotEmpty(edge, "The edge to resize the element on is undefined, null or empty");
	
	var resizer = new ResizeHelper();
	if (exists(useResizeHelper)) {
		resizer.setHelperUsed(useResizeHelper);
	}	
	if (isNumber(minHeight)) {
		resize.setMinHeight(minHeight);
	}
	if (isNumber(minWidth)) {
		resize.setMinWidth(minWidth);	
	}
}


/**
 * Called from a mousedown event handler. 
 * @param elementToResize The element to resize.
 * @throws Error if elementToResize is undefined or null.
 */
function ResizeHelper(elementToResize) {
	assertExists(elementToResize, "The element to resize is undefined or null");
		
		
	/**
	 * The element to resize.
	 */
	this.elementToResize = elementToResize;


	
	/**
	 * If the resize helper is being used this is the original element.
	 */
	this.originalResizeElement;
	
	
	/**
	 * For performance reasons we resize a empty frame instead of the actual element.
	 * At the end of the resize the upHandler resizes tha actual element.
	 */
	this.resizeHelper;

		
	/**
	 * True if we are to use the resize helper in the current resize operation.
	 */
	this.useResizeHelper = false;

	
	/**
	 * True if the resize helper has been inited for this resize operation.
	 */
	this.resizeHelperInited = false;


	/**
	 * The start size and position of the element to resize.
	 */
	this.startLeft = 0;
	this.startTop = 0;
	this.startHeight = 0;
	this.startWidth = 0;
	
	/**
	 * The resize handlers.
	 */
	this.handlers = new Array();
	this.handlers['bottom'] = "resizeHandlerbottom";
	this.handlers['top'] = "resizeHandlerTop";
	this.handlers['right'] = "resizeHandlerRight";
	this.handlers['left'] = "resizeHandlerLeft";
	this.handlers['bottomright'] = "resizeHandlerBottomRight";
	this.handlers['bottomleft'] = "resizeHandlerBottomLeft";
	this.handlers['topright'] = "resizeHandlerTopRight";
	this.handlers['topleft'] = "resizeHandlerTopLeft";
	
	
	/**
	 * If this is set the element to be resizeds width is not made smaller than the width set. 
	 */
	this.minWidth;	
	
	
	/**
	 * If this is set the element to be resizeds height is not made smaller than the height set. 
	 */
	this.minHeight;
	
	
	/**
	 * The function used to listen to mouse up events. 
	 */
	this.mouseUpListener = this.createMouseUpListener();
	
	
	/**
	 * The function used to listen to mouse move events. 
	 */
	this.mouseMoveListener;
	
	
	/**
	 * The mouse start x position of each resize event. 
	 */	
	this.startMouseX;
	
	
	/**
	 * The mouse start y position of each resize event
	 */
	this.startMouseY;
	
	
	/**
	 * This is set to true if the element has actually been resized.
	 */
	this.resized = false;
	
	
	/**
	 * These variabels are used to track how much the element has been resized.
	 */
	this.resizedY = 0;
	this.resizedX = 0;
	
	
	/**
	 * The edge the element is being resized on.
	 */
	this.edge;
}


// services


/**
 * Begins the resizing operation.
 * @param event The event.
 * @param edge At which edge the element is been resized (bottom, top, left right, 
 * topLeft, topRight, bottomLeft, bottomRight);
 * @throws Error if edge is undefined, null or empty.
 */
ResizeHelper.prototype.beginResize = function(event, edge) {
	assertNotEmpty(edge, "The edge the element is being resized on is undefined null or empty");
	event = exists(event) ? event : window.event;
	
	this.edge = edge;
	if (this.useResizeHelper) {
		this.setStart(this.elementToResize);
	}

	// register the event handlers responding to the mouse events
	this.mouseMoveListener = this.createMouseMoveListener(edge);
	registerEventHandler("mousemove", this.mouseMoveListener);
	registerEventHandler("mouseup", this.mouseUpListener);

	// capture the mouse position
	this.startMouseX = event.clientX + RESIZE_HELPER_BORDER_SIZE;
	this.startMouseY = event.clientY + RESIZE_HELPER_BORDER_SIZE;
	this.resized = false;
	
	stopDefaultAction(event);
}


/**
 * Instead of resizing the actual element dummy resize helper element can be used to be moved
 * instead. This can improve performance of the move operation. This toggles whether the helper
 * is to be used or not. 
 * @param useHelper True if the helper element is to be used, false if not.
 * @throws Error if useHelper is undefined or null.
 */
ResizeHelper.prototype.setHelperUsed = function(useHelper) {
	assertExists(useHelper, "UseHelper is undefined or null");
	this.useResizeHelper = useHelper;
}


/**
 * Set the min width. If this is set the element to be resized width is not made smaller than the width given.
 * @param minWidth The min width to set. 
 * @throws Error if minWidth is undefined, null, not a number or a negative number. 
 */
ResizeHelper.prototype.setMinWidth = function(minWidth) {
	assertNotNegative(minWidth, "Minwidth is undefined, null, not a number or a negative number: " + minWidth);
	this.minWidth = minWidth;
}


/**
 * Set the min height. If this is set the element to be resizeds height is not made smaller than the 
 * height given.
 * @param minHeight The height to be set. 
 * @throws Error if minHeight is undefined, null, not a number or negative.
 */
ResizeHelper.prototype.setMinHeight = function(minHeight) {
	assertNotNegative(minHeight, "The min height to set is undefined, null, nor a number or negative: " + minHeight);
	this.minHeight = minHeight;
}


// event handlers


/**
 * The handler capturing mouse movement events when a element is been 
 * resized at the bottom.
 */
ResizeHelper.prototype.resizeHandlerbottom = function(event) {
	event = exists(event) ? event : window.event;
	
	// init the reisze helper if it is to be used
	if (this.useResizeHelper && (!this.resizeHelperInited)) {
		this.initResizeHelper(this.elementToResize);
	}

	// capture the position of the mouse
	var mouseY = event.clientY;
	var deltaY = mouseY - this.startMouseY;
	if (this.isValidResize(this.elementToResize, deltaY, 0)) {
		this.startMouseY = mouseY;
		this.resized = true;
		this.resizedY += deltaY;
		resizeHtmlElement(this.elementToResize, deltaY, 0);			
	}	
	
	// stop the any further action to this event
	stopPropagation(event);
	stopDefaultAction(event);
}


/**
 * The handler capturing mouse movement events when a element is been 
 * resized at the top.
 */
ResizeHelper.prototype.resizeHandlerTop = function(event) {
	event = exists(event) ? event : window.event;
	
	// init the reisze helper if it is to be used
	if (this.useResizeHelper && (!this.resizeHelperInited)) {
		this.initResizeHelper(this.elementToResize);
	}
	
	// capture the position of the mouse
	var mouseY = event.clientY;
	var deltaY = mouseY - this.startMouseY;
	
	if (this.isValidResize(this.elementToResize, -deltaY, 0)) {
		this.startMouseY = mouseY;
		// move the element up so it looks like it grows up and not down
		var top = parseInt(this.elementToResize.style.top);
		top = top + deltaY;
		this.elementToResize.style.top = top + "px";
		
		this.resized = true;
		this.resizedY += -deltaY;
		resizeHtmlElement(this.elementToResize, -deltaY, 0);
	}	
	
	// stop the any further action to this event
	stopPropagation(event);
	stopDefaultAction(event);
}


/**
 * The handler capturing mouse movement events when a element is been 
 * resized at the right. 
 */
ResizeHelper.prototype.resizeHandlerRight = function(event) {
	event = exists(event) ? event : window.event;
	
	// init the reisze helper if it is to be used
	if (this.useResizeHelper && (!this.resizeHelperInited)) {
		this.initResizeHelper(this.elementToResize);
	}

	// capture the position of the mouse		
	var mouseX = event.clientX;
	var deltaX = mouseX - this.startMouseX;
	
	if (this.isValidResize(this.elementToResize, 0, deltaX)) {
		this.startMouseX = mouseX;			
		this.resized = true;
		this.resizedX += deltaX;
		resizeHtmlElement(this.elementToResize, 0, deltaX);
	}	
	
	// stop the any further action to this event
	stopPropagation(event);
	stopDefaultAction(event);
}


/**
 * The handler capturing mouse movement events when a element is been 
 * resized at the left.
 */
ResizeHelper.prototype.resizeHandlerLeft = function(event) {
	event = exists(event) ? event : window.event;
	
	// init the reisze helper if it is to be used
	if (this.useResizeHelper && (!this.resizeHelperInited)) {
		this.initResizeHelper(this.elementToResize);
	}
	
	// capture the position of the mouse
	var mouseX = event.clientX;
	var deltaX = mouseX - this.startMouseX;
	
	if (this.isValidResize(this.elementToResize, 0, -deltaX)) {
		this.startMouseX = mouseX;
		// move the element to the left so it looks like the element grows left and not right
		var left = parseInt(this.elementToResize.style.left);
		left = left + deltaX;
		this.elementToResize.style.left = left + "px";
		
		this.resized = true;
		this.resizedX += -deltaX;
		resizeHtmlElement(this.elementToResize, 0, -deltaX);
	}	
	
	// stop the any further action to this event
	stopPropagation(event);
	stopDefaultAction(event);
}


/**
 * The handler capturing mouse movement events when a element is been 
 * resized at the bottom right.
 */
ResizeHelper.prototype.resizeHandlerBottomRight = function(event) {
	event = exists(event) ? event : window.event;
	
	// init the reisze helper if it is to be used
	if (this.useResizeHelper && (!this.resizeHelperInited)) {
		this.initResizeHelper(this.elementToResize);
	}
	
	// capture the position of the mouse
	var mouseX = event.clientX;
	var mouseY = event.clientY;
	
	var deltaX = mouseX - this.startMouseX;
	var deltaY = mouseY - this.startMouseY;
			
	if (this.isValidResize(this.elementToResize, deltaY, deltaX)) {
		this.startMouseX = mouseX;
		this.startMouseY = mouseY;		
		this.resized = true;
		this.resizedX += deltaX;
		this.resizedY += deltaY;
		resizeHtmlElement(this.elementToResize, deltaY, deltaX);
	}			
			
	// stop the any further action to this event
	stopPropagation(event);
	stopDefaultAction(event);
}


/**
 * The handler capturing mouse movement events when a element is been 
 * resized at the bottom left.
 */
ResizeHelper.prototype.resizeHandlerBottomLeft = function(event) {
	event = exists(event) ? event : window.event;
	
	// init the reisze helper if it is to be used
	if (this.useResizeHelper && (!this.resizeHelperInited)) {
		this.initResizeHelper(this.elementToResize);
	}
	
	// capture the position of the mouse
	var mouseX = event.clientX;
	var mouseY = event.clientY;
	
	var deltaX = mouseX - this.startMouseX;
	var deltaY = mouseY - this.startMouseY;
		
	if (this.isValidResize(this.elementToResize, deltaY, -deltaX)) {
		this.startMouseX = mouseX;
		this.startMouseY = mouseY;
		// move the element to the left so it looks like
		// the element grows down and to the left
		this.moveLeft(this.elementToResize, deltaX);

		this.resized = true;
		this.resizedX += -deltaX;
		this.resizedY += deltaY;
		resizeHtmlElement(this.elementToResize, deltaY, -deltaX);
	}	
	
	// stop the any further action to this event
	stopPropagation(event);
	stopDefaultAction(event);
}


/**
 * The handler capturing mouse movement events when a element is been 
 * resized at the top right.
 */
ResizeHelper.prototype.resizeHandlerTopRight = function(event) {
	event = exists(event) ? event : window.event;
	
	// init the reisze helper if it is to be used
	if (this.useResizeHelper && (!this.resizeHelperInited)) {
		this.initResizeHelper(this.elementToResize);
	}
	
	// capture the position of the mouse
	var mouseX = event.clientX;
	var mouseY = event.clientY;
	
	var deltaX = mouseX - this.startMouseX;
	var deltaY = mouseY - this.startMouseY;
					
	if (this.isValidResize(this.elementToResize, -deltaY, deltaX)) {
		this.startMouseX = mouseX;
		this.startMouseY = mouseY;
		// move the element to up so it looks like
		// the element grows up and to the right
		this.moveUp(this.elementToResize, deltaY);
		
		this.resized = true;
		this.resizedX += deltaX;
		this.resizedY += -deltaY;
		resizeHtmlElement(this.elementToResize, -deltaY, deltaX);
	}
	
	// stop the any further action to this event
	stopPropagation(event);
	stopDefaultAction(event);
}


/**
 * The handler capturing mouse movement events when a element is been 
 * resized at the top left.
 */
ResizeHelper.prototype.resizeHandlerTopLeft = function(event) {
	event = exists(event) ? event : window.event;
	
	// init the reisze helper if it is to be used
	if (this.useResizeHelper && (!this.resizeHelperInited)) {
		this.initResizeHelper(this.elementToResize);
	}
	
	// capture the position of the mouse
	var mouseX = event.clientX;
	var mouseY = event.clientY;
	
	var deltaX = mouseX - this.startMouseX;
	var deltaY = mouseY - this.startMouseY;
			
	if (this.isValidResize(this.elementToResize, -deltaY, -deltaX)) {
		this.startMouseX = mouseX;
		this.startMouseY = mouseY;		
		// move the element up and to the left so it looks like
		// the element grows up and left
		this.moveLeft(this.elementToResize, deltaX);
		this.moveUp(this.elementToResize, deltaY);
		
		this.resized = true;
		this.resizedX += -deltaX;
		this.resizedY += -deltaY;
		resizeHtmlElement(this.elementToResize, -deltaY, -deltaX);
	}		
	
	// stop the any further action to this event
	stopPropagation(event);
	stopDefaultAction(event);
}



/**
 * This is the handler that captures the final mouse up event
 * at the end of a resize.
 */
ResizeHelper.prototype.upHandler = function(event) {
	event = exists(event) ? event : window.event;

	// Unregister event handlers 
	removeEventHandler("mouseup", this.mouseUpListener);
	removeEventHandler("mousemove", this.mouseMoveListener);
	
	if (this.useResizeHelper) {
		// make the element the size of the resize helper
		this.resizeElementToHelper(this.originalResizeElement, this.resized);
		this.hideResizeHelper();
	}
	
	// stop anybody else from seing this event.
	stopPropagation(event);

	// stop the event being processed by the browser
	stopDefaultAction(event);
	
	if (this.useResizeHelper) {
		this.checkPosition(this.edge)
		this.resizeHelperInited = false;
		this.elementToResize = exists(this.originalResizeElement) ? this.originalResizeElement : this.elementToResize;
	}
	
	this.resizedY = 0;
	this.resizedX = 0;
}


// implementation


/**
 * Creates the function to use to listen to mouse up events.
 * @param edge At which edge the element is been resized (bottom, top, left right, 
 * topLeft, topRight, bottomLeft, bottomRight); 
 */
ResizeHelper.prototype.createMouseMoveListener = function(edge) {
	var functionToCall = this.handlers[edge];
	var thisObj = this;
	return function(e) { thisObj[functionToCall](e) };
}


/**
 * Creates the function to use to listen to mouse up events. 
 * @return The function to set as mouse up listener. 
 */
ResizeHelper.prototype.createMouseUpListener = function() {
	var thisObject = this;
	return function(e) { thisObject.upHandler(e)};
}


/**
 * Checks if a resize is valid for a element. If the resize would make the 
 * size of the element negative the resize is not valid.
 * @param element The element to check.
 * @param deltaHeight The height the element is about to be resized with.
 * @param deltaWdith The width the element is about to be resized with.
 * @return True if the resize is valid, false otherwise.
 */
ResizeHelper.prototype.isValidResize = function(element, deltaHeight, deltaWidth) {
	var height = parseInt(element.style.height);
	if ((height + deltaHeight) < 0) {
		return false;
	}
	if (isDefined(this.minHeight) && (height + deltaHeight) < this.minHeight) {
		return false;
	}
	
	var width = parseInt(element.style.width);
	if ((width + deltaWidth) < 0) {
		return false;
	}
	if (isDefined(this.minWidth) && (width + deltaWidth) < this.minWidth) {
		return false;
	}
	
	return true;
}


/**
 * Moves a element to the left.
 * @param element The element to move. 
 * @param pixels The amount of pixels to move the element.
 */
ResizeHelper.prototype.moveLeft = function(element, pixels) {
	var left = parseInt(element.style.left);
	left = left + pixels;
	element.style.left = left + "px";
}


/**
 * Moves a element up.
 * @param element The element to move. 
 * @param pixels The amount of pixels to move the element.
 */
ResizeHelper.prototype.moveUp = function(element, pixels) {
	var top = parseInt(element.style.top);
	top = top + pixels;
	element.style.top = top + "px";	
}


/**
 * Resizes a element to the size of the resize helper.
 * @param element The element to resize.
 * @param resized True if the element actually has been resized, false otherwise.
 */
ResizeHelper.prototype.resizeElementToHelper = function(element, resized) {
	if (resized) {
		element.style.height = this.resizeHelper.style.height;
		element.style.width = this.resizeHelper.style.width;
		
		element.style.top = (parseInt(this.elementToResize.style.top) + RESIZE_HELPER_BORDER_SIZE) + "px";
		element.style.left = (parseInt(this.elementToResize.style.left) + RESIZE_HELPER_BORDER_SIZE) + "px";
		
		// this is hack to avoid a problem in Opera, in Opera when the height has
		// changed but the width has not the elements size may not be changed.
		// This is a problem experienced only in Opera, we temporarily set
		// the elements display to be none to force Opera to render the element and
		/// change the size
		var display = element.style.display;
		element.style.display = "none";
		element.style.display = display;
	}
}


/**
 * Initialises the resize helper and prepares it to for resizing.
 * @param element The element that is to be resized.
 */
ResizeHelper.prototype.initResizeHelper = function(element) {
	if (!isDefined(this.resizeHelper)) {
		this.createResizeHelper();
	}

	this.fitResizeHelper(element);
	this.setResizeHelperZIndex(element);
	this.originalResizeElement = element;
	this.elementToResize = this.resizeHelper;
	this.showResizeHelper()
	this.useResizeHelper = true;
	this.resizeHelperInited = true;
}



/**
 * Make the resize helper fit a element that is to be resized. The position
 * of the resize helper is set to be the position of the element, and the 
 * size is set to be the size of the element.
 * @param element The element to fit the resizeHelper to.
 */
ResizeHelper.prototype.fitResizeHelper = function(element) {
	this.resizeHelper.style.height = element.style.height;
	this.resizeHelper.style.width = element.style.width;
	
	// make the resizeHelper be around the element not on top
	this.resizeHelper.style.left = (parseInt(element.style.left) - RESIZE_HELPER_BORDER_SIZE) + "px";
	this.resizeHelper.style.top = (parseInt(element.style.top) - RESIZE_HELPER_BORDER_SIZE) + "px";
}


/**
 * Create the resize helper element.
 */
ResizeHelper.prototype.createResizeHelper = function() {
	var resizer = document.createElement("div");
	setClass(resizer, "resizeHelper");
	document.body.appendChild(resizer);
	this.resizeHelper = resizer;
}


/**
 * Hides the resize helper
 */
ResizeHelper.prototype.hideResizeHelper = function() {
	if (isDefined(this.resizeHelper)) {
		this.resizeHelper.style.display = "none";
		this.resizeHelper.style.zIndex = 0;
		this.resizeHelperInited = false;
	}
}


/**
 * Shows the resize helper.
 */
ResizeHelper.prototype.showResizeHelper = function() {
	if (isDefined(this.resizeHelper)) {
		this.resizeHelper.style.display = "inline";
		setClass(this.resizeHelper, "resizeHelper");
	}
}



/**
 * Set the z value of the resize helper in a way that it appears 
 * in front of the element to resize.
 * @param element The element to make the resize helper appear above.
 */
ResizeHelper.prototype.setResizeHelperZIndex = function(element) {
	if (element.style.zIndex) {
		var zIndex = parseInt(element.style.zIndex) + 1;
		}
	else {
		zIndex = 1;
	}

	this.resizeHelper.style.zIndex = zIndex;
}


/**
 * Resizes some given html elements.
 * @param elements The elements to resize.
 * @param deltaHeight The amount of pixels to increase the height.
 * @param deltaWidth The amount of pixels to increase the widht.
 */
function resizeHtmlElements(elements, deltaHeight, deltaWidht) {
	var len = elements.length;
	for (var i = 0; i < len; i++) {
		resizeHtmlElement(elements[i], deltaHeight, deltaWidht);
	}	
}


/**
 * Changes a html element size.
 * @param element The element resize.
 * @param deltaHeight The amount of pixels to increase the height.
 * @param deltaWidth The amount of pixels to increase the widht.
 */
function resizeHtmlElement(element, deltaHeight, deltaWidth) {
	resizeHtmlElementHeight(element, deltaHeight);
	resizeHtmlElementWidth(element, deltaWidth);
}


/**
 * Resizes the width of a html element. 
 * @param element The element resize.
 * @param deltaWidth The amount of pixels to increase the widht.
 */
function resizeHtmlElementWidth(element, deltaWidth) {
	if (deltaWidth == 0) {
		return;
	}
	var width = parseInt(element.style.width);
	width = isNaN(width) ? element.offsetWidth : width;
	if (!isNaN(width)) {
		var newWidth = (width + deltaWidth);
		if (newWidth < 0) {
			newWidth = 0; 
		}
		element.style.width = newWidth + "px";
	}
}

 
/**
 * Changes the height of a html element. 
 * @param element The element resize.
 * @param deltaHeight The amount of pixels to increase the height.
 */
function resizeHtmlElementHeight(element, deltaHeight) {
	if (deltaHeight == 0) {
		return;
	}
	var height = parseInt(element.style.height);
	height = isNaN(height) ? element.offsetHeight : height;
	if (!isNaN(height)) {
		var newHeight = (height + deltaHeight);
		if (newHeight < 0) {
			newHeight = 0;
		}
		element.style.height = newHeight + "px";
	}
}


/**
 * Sets the start positon and size of the element to resize.
 * @param elementToResize The element to resize.
 */
ResizeHelper.prototype.setStart = function(elementToResize) {
	this.startLeft = parseInt(elementToResize.style.left);
	this.startTop = parseInt(elementToResize.style.top);
	this.startHeight = parseInt(elementToResize.style.height);
	this.startWidth = parseInt(elementToResize.style.width);
}


/**
 * Resizing on the left, top, bottomleft, topright or topleft corner of a element
 * move the element, when the element is made smaller than it can be rendered by the 
 * browser and the resize helper is used it can be moved to much, correct this.
 */
ResizeHelper.prototype.checkPosition = function(edge) {
	if ("left" == edge) {
		this.checkXPosition();
	}
	else if ("top" == edge) {
		this.checkYPosition()
	}
	else if ("bottomleft" == edge) {
		this.checkXPosition();
	}
	else if ("topright" == edge) {
		this.checkYPosition()
	}
	else if ("topleft" == edge) {
		this.checkXPosition();
		this.checkYPosition()
	}
}


/**
 * Check that the y position of the element is correct.
 */
ResizeHelper.prototype.checkYPosition = function() {
	if (isDefined(this.originalResizeElement)) {
		var startBottom = this.startTop + this.startHeight;
		var bottomNow = parseInt(this.originalResizeElement.style.top) + parseInt(this.originalResizeElement.style.height);
		if (startBottom != bottomNow) {
			var diff = bottomNow - startBottom;
			this.originalResizeElement.style.top = (parseInt(this.originalResizeElement.style.top) - diff) + "px";
		}
	}
}


/**
 * Check that the x position of the element is correct.
 */
ResizeHelper.prototype.checkXPosition = function() {
	if (isDefined(this.originalResizeElement)) {
		var startRight = this.startLeft + this.startWidth;
		var rightNow = parseInt(this.originalResizeElement.style.left) + parseInt(this.originalResizeElement.style.width);
		if (startRight != rightNow) {
			var diff = rightNow - startRight;
			this.originalResizeElement.style.left = (parseInt(this.originalResizeElement.style.left) - diff) + "px";
		}
	}
}