/**
 * The min allowed port number.
 */
var MIN_PORT_NUMBER = 0;


/**
 * The max allowed port number.
 */
var MAX_PORT_NUMBER = 65535;


/**
 * The min value for one of the numbers in a ip address.
 */
var MIN_IP_ADDRESS = 0;


/**
 * The max value for one of the numbers in a ip address. 
 */
var MAX_IP_ADDRESS = 255;


/**
 * Returns true if the browser used is supported.
 */
function isBrowserSupported() {
	var nonSupportedBrowsers = new Array();
	nonSupportedBrowsers[0] = "Konqueror";
	for (var i = 0; i < nonSupportedBrowsers.length; i++) {
		if (navigator["appName"] == nonSupportedBrowsers[i]) {
			return false;
		}
	}
	return true;
}


/**
 * Removes the null entries in a table.
 * @param table The table to remove the null entries from.
 * @return The table wihtout null entries.
 */
function removeNullEntries(table) {
	assertDefined(table, "The table to remove null entries from is not defined");
	
	var newTable = new Array();
	var count = 0; 
	var len = table.length;
	for (var i = 0; i < len; i++) {
		if (isDefined(table[i]) && !isNull(table[i])) {
			newTable[count] = table[i];
			count++;
		}
	}
	return newTable;
}


/**
 * Returns a unique number each time. The value is increased by 1 each time.
 */
function uniqueInteger() {
	if (!isDefined(uniqueInteger.counter)) {
		uniqueInteger.counter = 0;
	}	
	return uniqueInteger.counter++;
}


/**
 * Returns a random integer from a given range. 
 * @param min The min for the number generated. The number generated will be equal to this or larger. 
 * @param max The max for the number generated. The number generated will be smaller or equal to this.  
 * @throws Error if min is not a number or negative. 
 * @throws Erorr if max is not a number or positive. 
 * @throws Error if min is equal to or larger than max. 
 */
function randomInteger(min, max) {
	if (isNaN(parseInt(min)) || parseInt(min) < 0) {
		throw new IllegalArgumentException ("Invalid min value");
	}
	if (isNaN(parseInt(max)) || parseInt(max) < 0) {
		throw new IllegalArgumentException ("Invalid max value");
	}
	min = parseInt(min);
	max = parseInt(max);
	if (min >= max) {
		throw new IllegalArgumentException ("Min is equal to or larger than max");
	}
	
	return Math.floor((max - min - 1)*Math.random()) + min;
}


/**
 * Returns true if a associative array contains a key, false otherwise.
 * @param array The array to check.
 * @param key The key to search for.
 * @throws Error if array is undefined or null.
 * @throws Error if element is undefined or null.
 */
function containsElement(array, key) {
	assertDefined(array, "Array is undefined or null");
	assertDefined(key, "Key is undefined or null");

	for (arrayElement in array) {
		if (key == arrayElement) {
			return true;
		}
	}
	
	return false;
}


/**
 * Returns true if a string is a float, false otherwise.
 * Numbers of type 10,5 and 10.5 are recognized as floats.
 * With a numbers like 10 false is returned.
 * @param string The string value to check.
 */
function isFloat(string) {
	var numberValue = parseFloat(string);
	if (isNaN(numberValue)) {
		return false;
	}
	
	// we still not if the number is a float, could just be a integer
	var index = string.indexOf(",");
	if (index >= 0) {
		return true;
	}
	index = string.indexOf(".");
	if (index >= 0) {
		return true;
	}
	
	return false;
}


/**
 * Rounds a given number to a given number of decimals.
 * @param number The number to round.
 * @param decimalCount The number of decimalCount to round to.
 * @return The rounded number.
 */
function roundNumber(number, decimalCount) {
	var result = Math.round(number * Math.pow(10, decimalCount)) / Math.pow(10, decimalCount);
	return result;
}


/**
 * URL decodes the specified string. It also replaces any occurences of "+" with a space.
 */
function URLDecode(str) {
	str = decodeURIComponent(str).replace(/\+/g, " ");
	return str;
}


/**
 * Checks if a specified port id string is valid. 
 * @returns True if the port is valid, false otherwise. 
 * @throws Error if port is undefined, null or empty. 
 */
function isValidPortIdString(port) {
	assertNotEmpty(port, "Port to check is undefined, null or empty: " + port);
	var startIndex = 0;
	var endIndex = 0;
	while(true) {
		if (startIndex == -1) {
			break;
		}
		var endIndex = port.indexOf(".", startIndex + ".".length);
		if (endIndex == -1) {
			endIndex = port.length;
		}
		var portNumber = parseInt(port.substring(startIndex, endIndex));
		if (!isValidPort(portNumber)) {
			return false;
		}
		if (endIndex == port.length) {
			break;
		}
		startIndex = endIndex + ".".length;
	}
	return true;
}


/**
 * Checks if the specified port is valid.
 */
function isValidPort(port) {
	return port >= MIN_PORT_NUMBER && port <= MAX_PORT_NUMBER;
}




/**
 * Checks if any of the numbers in a port id string (1.2048.6.80) are invalid. 
 * @param port The port id string (1.2048.6.80) to check.
 * @return True if the port id string set is valid, false otherwise. 
 */
function validPortNumbers(port) {
	if (isEmpty(port)) {
		return false;
	}
	var tokenizer = new StringTokenizer(port, ".");
	while (true) {
		if (!tokenizer.hasMoreTokens()) {
			break;
		}	
		var numberString = tokenizer.nextToken();
		if (isEmpty(numberString) && !tokenizer.hasMoreTokens()) {
			// the last token might be empty
			return true;
		}
		else if (isEmpty(numberString)) {
			// any of the other tokens might not be empty
			return false;
		}
		if (!isValidPort(numberString)) {
			return false;
		}
	}
	return true;
}


/**
 * Checks if a specified string contains a valid ip address (i.e. 10.1.1.1).
 * @param address The ip address to check.
 * @returns True if the ip address is valid, false otherwise.
 * @throws Error if address is undefined, null or empty.
 */
function isValidIpAddress(address) {
	assertNotEmpty(address, "Ip address to check is undefined, null or empty: " + address);
	
	var startIndex = 0;
	var endIndex = 0;
	var counter = 0;
	while (true) {
		if (startIndex == -1) {
			break;
		}
		counter++;
		
		var endIndex = address.indexOf(".", startIndex + ".".length);
		if (endIndex == -1) {
			endIndex = address.length;
		}
		
		if (!isValidIpAddressNumber(address.substring(startIndex, endIndex))) {
			return false;
		}
		
		if (endIndex == address.length) {
			break;
		}
		startIndex = endIndex + ".".length;
	}
	
	if (counter != 4) {
		return false;
	}
	return true;
}


/**
 * Checks if a given string is a valid part of a ip address. 
 * @param numberString The number to check as a string. 
 * @return True if numberString is a valid part of ip address, false otherwise.
 */
function isValidIpAddressNumber(numberString) {
	if (isEmpty(numberString) || !isNumberString(numberString)) {
		return false;
	}
	
	var addressNumber = parseInt(numberString);
	if (addressNumber < MIN_IP_ADDRESS || addressNumber > MAX_IP_ADDRESS) {
		return false;
	}
	return true;
}


/**
 * Checks if any of the numbers in a ip address are invalid. 
 * @param ip The ip address to check.
 * @return True if the ip address set is valid, false otherwise. 
 */
function validIpNumbers(ip) {
	if (isEmpty(ip)) {
		return false;
	}
	var tokenizer = new StringTokenizer(ip, ".");
	while (true) {
		if (!tokenizer.hasMoreTokens()) {
			break;
		}	
		var numberString = tokenizer.nextToken();
		if (isEmpty(numberString) && !tokenizer.hasMoreTokens()) {
			// the last token might be empty
			return true;
		}
		else if (isEmpty(numberString)) {
			// any of the other tokens might not be empty
			return false;
		}
		if (!isValidIpAddressNumber(numberString)) {
			return false;
		}
	}
	return true;
}


/**
 * Returns true if the ethernet address is valid.
 */
function isValidMac(address) {
	// smallest is 11 characters, largest is 17
	if (address == null || address.length < 11 || address.length > 17) {
		return false;
	}
	
	// valid if 5 colons
	if (countChar(address, ':') != 5) {
		return false;
	}
	
	// can not start or end with :
	if (address.charAt(0) == ':' || address.charAt(address.length-1) == ':') {
		return false;
	}
	
	// valid if characters :, 0-9, a-f
	for (i = 0; i < address.length; i++) {
		c = address.charAt(i);
		if (c != ':' && !isHexDigit(c)) {
			return false;
		}
	}
	
	// valid if 1 or 2 digits in a row and 1 colon
	digits = 0;
	colons = 0;
	for (i = 0; i < address.length; i++) {
		c = address.charAt(i);
		if (isHexDigit(c)) {
			digits++;
			colons = 0;
		}
		else if (c == ':') {
			digits = 0;
			colons++;
		}
		if (digits > 2 || colons > 1) {
			return false;
		}
	}
	return true;
}


/**
 * Checks if any of the numbers in a mac address are invalid. The last token 
 * can be empty: 00:001:
 * @param mac The mac address to check.
 * @return True if the mac address set is valid, false otherwise. 
 */
function validMacNumbers(mac) {
	if (isEmpty(mac)) {
		return false;
	}
	var tokenizer = new StringTokenizer(mac, ":");
	while (true) {
		if (!tokenizer.hasMoreTokens()) {
			break;
		}	
		var numberString = tokenizer.nextToken();
		if (isEmpty(numberString) && !tokenizer.hasMoreTokens()) {
			// the last token might be empty
			return true;
		}
		else if (isEmpty(numberString)) {
			// any of the other tokens might not be empty
			return false;
		}
		if (!isValidMacNumber(numberString)) {
			return false;
		}
	}
	return true;
}


/**
 * Returns true if a given address is a valid mac address number. 
 * @param mac The mac address number to check.
 * @return True if mac is valid (characters :, 0-9, a-f), false otherwise. 
 */
function isValidMacNumber(mac) {
	if (isEmpty(mac)) {
		return false;
	}
	for (i = 0; i < mac.length; i++) {
		c = mac.charAt(i);
		if (c != ':' && !isHexDigit(c)) {
			return false;
		}
	}
	return true;
}


/**
 * Counts the number of a specified character in a string.
 */
function countChar(text, c) {
	count = 0;
	for (i = 0; i < text.length; i++) {
		count += (text.charAt(i) == c ? 1 : 0);
	}
	return count;
}


/**
 * Returns true if the specified character is a hex digit.
 */
function isHexDigit(c) {
	return (c >= '0' && c <= '9') || (c >= 'A' && c <= 'F') || (c >= 'a' && c <= 'f');
}


/**
 * Selects a option with a given value in a selector.
 * @param selector The selector to select a option in.
 * @param value The value of the option to select.
 * @throws Error if selector is undefined or null.
 * @throws Error if value is undefined or null.
 */
function selectInSelector(selector, value) {
	assertDefined(selector, "Selector is undefined or null");
	assertExists(value, "Value to select in selector is undefined or null");
	
	var index = getIndexInSelector(selector, value);	
	if (index > -1) {
		selector.selectedIndex = index;
	}
}


/**
 * Gets the index of a option in a select with a given text. 
 * @param text The text of the option to get the index for. 
 * @param select The select to search in. 
 * @return The option with text as text, -1 if not found. 
 * @throws Error if text is undefined, null or empty.
 * @throws Error if select is undefined or null.
 */
function getOptionIndex(text, select) {
	assertNotEmpty(text, "Text to search for is undefined null or empty");
	assertExists(select, "The select tag to search in is undefined null or empty");
	for (var i = 0; i < select.options.length; i++) {
		if (select.options[i].text == text) {
			return i;
		}
	}
	return -1;
}


/**
 * Deletes all the options in a select.
 * @param select The select tag to delete options in.
 */
function deleteOptions(select) {
	var options = select.options;
	for (var i = options.length - 1; i >= 0; i--) {
		select.remove(options[i]);
		if (isSafari()) {
			// in safari remove does not work
			select.options[i] = undefined;
		}
	}
}


/**
 * Gets the index of a option with a given value in a selector. 
 * @param selector The selector to find the index in.
 * @param value The value of the option to get the indox for.
 * @return The index found, -1 if not found.
 * @throws Error if selector is undefined or null.
 * @throws Error if value is undefined, empty or null.
 */
function getIndexInSelector(selector, value) {
	var len = selector.options.length;
	for (var i = 0; i < len; i++) {
		if (value == selector.options[i].getAttribute("value")) {
			return i;
		}
	}
	return -1;
}


/**
 * Removes a option with a given value from the select tag.
 * @param value The of the option to remove.
 * @param selector The selector tag to remove the option from.
 * @throws Error if value is undefined, null or empty.
 * @throws Error if select is undefined or null.
 */ 
function removeOption(value, selector) {
	assertExists(selector, "Selector is undefined or null");
	assertNotEmpty(value, "Value is undefined or empty");
	
	var index = getIndexInSelector(selector, value);	
	if (index > -1) {
		selector.remove(index);
	}
}


/**
 * Replaces a option in a select tag. 
 * @param newOption The new option to add.
 * @param index The index into the select tags options to replace. 
 * @param select The select tag where to replace the option. 
 * @throws Error if newOption is undefined or null.
 * @throws Error if index is negative or larger than the option count in select.
 * @throws Error if the select tag is undefined or null.
 */
function replaceOption(newOption, index, select) {
	assertExists(newOption, "The new option is null or undefined");
	assertNotNegative(index, "The index is negative");
	assertExists(select, "The select tag to replace the option in is undefined or null");
	if (index < select.options - 1) {
		throw new Error("The index is larger than the option count");
	}
	
	select.removeChild(select.options[index]);
	select.insertBefore(newOption, select.options[index]);	
}


/**
 * Returns true if a xml block contains a error message, false otherwise.
 * @param xml The xml data to check.
 * @throws Error if xml is undefined or null.
 */
function containsErrorMessage(xml) {
	assertExists(xml, "Xml data is undefined, null or empty");
	var error = xml.getElementsByTagName("error")[0];
	if (isDefined(error) && !isNull(error)) {
		return true;
	}
	return false;
}


/**
 * Returns true if a xml block contains a success message, false otherwise.
 * @param xml The xml data to check. 
 * @throws Error if xml is undefined or null.
 */
function containsSuccessMessage(xml) {
	assertExists(xml, "Xml data is undefined, null or empty");
	var success = xml.getElementsByTagName("success")[0];
	if (isDefined(success) && !isNull(success)) {
		return true;
	}
	return false;
}	


/**
 * Returns true if a xml block contains a noaccess message, false otherwise.
 * @param xml The xml data to check.
 * @throws Error if xml is undefined or null.
 */
function containsNoAccessMessage(xml) {
	assertExists(xml, "Xml data is undefined, null or empty");
	var noAccess = xml.getElementsByTagName("noaccess")[0];
	if (exists(noAccess)) {
		return true;	
	}
	return false;
}


/**
 * Gets a error message from a block of xml.
 * @param xml the xml to get the error message from.
 * @return The error message in xml.
 * @throws Error if xml is undefined or null.
 */
function getErrorMessage(xml) {
	assertExists(xml, "Xml data is undefined null or empty");
	var message = xml.getElementsByTagName("message")[0];
	var error = exists(message) ? message.getElementsByTagName("error")[0] : null;
	if (!exists(error) || !isDefined(error.firstChild) || error.firstChild.nodeType != 3) {
		// the node is not a text node
		return "";
	}

	return error.firstChild.nodeValue;
}


/**
 * Gets a success message from a block of xml.
 * @param xml the xml to get the success message from.
 * @return The success message in xml.
 * @throws Error if xml is undefined or null.
 */
function getSuccessMessage(xml) {
	var message = xml.getElementsByTagName("message")[0];
	var success = exists(message) ? message.getElementsByTagName("success")[0] : null;
	if (!exists(success) || !isDefined(success.firstChild) || success.firstChild.nodeType != 3) {
		// the node is not a text node
		return;
	}

	return success.firstChild.nodeValue;
}


/**
 * Copies all the entries from a source array into a destination array. The 
 * entries are appended to the end of the destination array.
 * @parma source The source array.
 * @param destination The destination array to copy to.
 * @throws Error if source is undefined or null.
 * @throws Error if destination is undefine or null.
 */
function arrayCopy(source, destination) {
	assertExists(source, "Source array to copoy from does not exist");
	assertExists(destination, "The destinaion array to copy to does not exist");
	
	for (var i = 0; i < source.length; i++) {
		destination[destination.length] = source[i];
	}
}


/**
 * Converts a string to a number.
 * @param numberString The number as a string to convert to a number.
 * @return The string converted to a number.
 * @throws Error if numberString is undefined, null or empty.
 */
function toNumber(numberString) {
	assertNotEmpty(numberString, "The string to convert to a number is undefined, null or empty, " + numberString);
	var index = numberString.indexOf(",");
	if (index > -1) {
		numberString = numberString.replace(",", ".");	
	}
	return Number(numberString);
}