/**
 * Creates a comma separated list of values for a given tag in each xml entry of a given name in a xml document. 
 * @param xmlDoc The xml document from which to create the list from.
 * @param entryName The name of each entry to get data from. 
 * @param valueName The value of this the tag in each xml entry of name entryName 
 * in xmlDoc is included in the list.
 * @param valueName2 If this is defined the value of this tag in each xml entry 
 * of name entryName is included in the list. 
 * @return Comma separated list of the of valueName and valueName2 tags in each xml entry of name entryName in xmlDoc.
 */
function createValueList(xmlDoc, entryName, valueName, valueName2) {
	var list = "";
	var entries = xmlDoc.getElementsByTagName(entryName);	
	var len = entries.length;
	for (var i = 0; i < len; i++) {
		if (valueName == entries[i].getElementsByTagName(valueName)[0].tagName) {
			list += entries[i].getElementsByTagName(valueName)[0].firstChild.data;
		}
		if (isDefined(valueName2) && valueName2 == entries[i].getElementsByTagName(valueName2)[0].tagName) {
			list += " - " + entries[i].getElementsByTagName(valueName2)[0].firstChild.data;
		}
	 	if (i < entries.length - 1) {
	 		list += ",";
	 	}
	}
	return list;
}


/**
 * Returns the max value for a tag in a xml entry of a given name in a xmlDoc.
 * @param xmlDoc The xml document from which to get the max value.
 * @param entryName The name of the entries to get the max value from. 
 * @param valueName The name of the value for each entry to get the max value from.
 * @return The max value of the valueName tags in each entry of name entryName in xmlDoc.
 */
function maxValue(xmlDoc, entryName, valueName) {
	var max = 0;
	var entries = xmlDoc.getElementsByTagName(entryName);	
	var len = entries.length;
	for (var i = 0; i < len; i++) {
		if (valueName != entries[i].getElementsByTagName(valueName)[0].tagName) {
			continue;			
		}
		var value = parseInt(entries[i].getElementsByTagName(valueName)[0].firstChild.data);
		if (value > max) {
			max = value;
		}
	}
	return max;
}


/**
 * Gets the first childs node of a xml tag in a given xml document. 
 * @param name of the xml tag to get the nodevalue for. 
 * @param xmlDocument The xml document the tag is in. 
 * @return The tags first childs nodevalue, a empty string of the tag did not exist inside xmlDocument,
 * or if the tag has no childs. 
 * @throws Error if name is undefined, null or empty. 
 * @throws Error if xmlDocument is undefined or null. 
 */
function getTagNodeValue(name, xmlDocument) {
	assertNotEmpty(name, "Tag name is undefined, null or empty");
	assertDefined(xmlDocument, "XmlDocument to get the tag from is undefined or null");
	
	var tags = xmlDocument.getElementsByTagName(name);
	if (tags.length == 0) {
		return "";
	}
	
	var tag = tags[0];
	if (!isDefined(tag) || isNull(tag)) {
		return "";
	}
	
	if (!isDefined(tag.firstChild) || isNull(tag.firstChild)) {
		return "";
	}
	
	if (!isDefined(tag.firstChild.nodeValue) || isNull(tag.firstChild.nodeValue)) {
		return "";
	}
	
	return tag.firstChild.nodeValue;
}


/**
 * Gets the nodevalues of all tags in a given xml document. 
 * @param name of the xml tag to get the nodevalue for. 
 * @param xmlDocument The xml document to get the tags from.
 * @return A array containing the nodevalues of the tags with name in xmlDocument, null if xmlDocument 
 * contains no tags with name. 
 * @throws Error if name is undefined, null or empty. 
 * @throws Error if xmlDocument is undefined or null. 
 */
function getTagNodeValues(name, xmlDocument) {
	assertNotEmpty(name, "Tag name is undefined, null or empty");
	assertDefined(xmlDocument, "XmlDocument to get the tag from is undefined or null");
	
	var tags = xmlDocument.getElementsByTagName(name);
	if (tags.length == 0) {
		return null;
	}
	
	var values = new Array();
	for (var i = 0; exists(tags) && i < tags.length; i++) {
		var tag = tags[i];
		if (!isDefined(tag) || isNull(tag)) {
			continue;
		}
		if (!isDefined(tag.firstChild) || isNull(tag.firstChild)) {
			continue;
		}
		if (!isDefined(tag.firstChild.nodeValue) || isNull(tag.firstChild.nodeValue)) {
			continue;
		}
		values[i] = tag.firstChild.nodeValue;
	}
	return values.length == 0 ? null : values;
}


/**
 * Returns true if a block of xml data contains a tag with a given name.
 * @param name The name of the tag to search for.
 * @param xmlDocument The xml document to search in.
 * @throws Error if name is undefined, null or empty. 
 * @throws Error if xmlDocument is undefined or null. 
 */
function containsXmlTag(name, xmlDocument) {
	assertNotEmpty(name, "Tag name is undefined, null or empty");
	assertDefined(xmlDocument, "XmlDocument to get the tag from is undefined or null");
	
	var tags = xmlDocument.getElementsByTagName(name);
	if (tags.length == 0) {
		return false;
	}
	return true;
}


/**
 * Returns a block of xml as a text string.
 * @param xml The block of xml to convert into a string.
 * @return Xml as a string.
 * @throws Error if xml is undefined or null.
 */
function xmlToString(xmlToConvert) {
	assertExists(xmlToConvert, "Xml to convert into a string is undefined or null");
	
	try {
		var xmlSerializer = new XMLSerializer();
		return xmlSerializer.serializeToString(xmlToConvert);
	}
	catch (Error) {
		// IE
		return xmlToConvert.xml;
	}
}