/**
 * The size of the resize area of the window.
 */
Window.RESIZE_AREA_SIZE = 2;


/**
 * The width of the border.
 */
Window.BORDER_WIDTH = 1;


/**
 * The height of the window header. 
 */
Window.WINDOW_HEADER_HEIGHT = 25;


/**
 * Default window y position.
 */
Window.DEFAULT_Y_POSITION = "30px";


/**
 * Default window x position.
 */
Window.DEFAULT_X_POSITION = "10px";


/**
 * Opens a new browser window with specified url, width and height.
 */
function winOpen (url, width, height) {
	time = new Date();
	theName = Math.ceil(time);
	var options='';
	if (width != null) {
		options += 'width='+width;
	}
	if (height != null) {
		options += ',height='+height;
	}
	options += ',resizable=yes, scrollbars=false';
	w = window.open (url, theName, options);
}


/**
 * Constructs a window.
 * @param id The id of the window.
 */
function Window(id) {
	// init the properties of this class 
	this.id = id;	
	this.htmlCode = GetWindowCode(id);
	assertDefined(this.htmlCode, id + " window does not exist");
	this.width = parseInt(this.htmlCode.style.width);
	this.height = parseInt(this.htmlCode.style.height);
	this.header = this.getWindowHeader();
	
	// set some default values
	this.minWidth = 0;
	this.minHeight = 0;
	// true if the content section of this window is colapsed
	this.colapsedContent = false;
	WindowManager.instance().registerNewWindow(this);
	this.resizableElements = new Array();
	this.moveElements = new Array();
	// the select tags to replace with its selected text when the window is deactivated
	this.selects = new Array();
	// the fields in which the selected text for the selects in this.selects is shown while they are hidden
	this.replaceFields = new Array();
	// the original classes of the select tags
	this.originalSelectClasses = new Array();
	// classes to set for the select replace tags
	this.selectReplaceClasses = new Array();
	// set this to true if the window should be positioned according to the active window the next
	// time the window is shown. 
	this.position = false;
	// indicating whether this window is resizable
	this.resizable = false;
	
	// true if this window has the current layout set, false otherwise
	this.currentLayoutSet = false;
	
	// set to true if the window is minimized
	this.minimized = false;
	
	if (isIE()) {
		this.initSelects();
	}
	
	/**
	 * The drag helper used for dragging the window. 
	 */
	this.dragHelper = createWindowDragHelper(this);
	this.dragHelper.setHelperUsed(true);
	this.dragHelper.setPropagateEvents(false);
	this.dragHelper.setYMin(0);
	this.setDragExceptions();
	
	
	/**
	 * The resize helper used to resize this window
	 */
	this.resizeHelper = createWindowResizeHelper(this);
	this.resizeHelper.setHelperUsed(true);
	
	/**
	 * Whether this window is modal or not. 
	 */
	this.modal = false;
}


// services


/**
 * Returns true if a element is a window.
 * @param element The element to check.
 * @throws Error if element is undefined or null.
 */
function isWindow(element) {
	assertDefined(element, "Element is undefined or null");
	if ("window" == element.getAttribute("class") || "window" == element.getAttribute("className")) {
		return true;
	}
	return false;
}


/**
 * Returns the id of the window.
 */
Window.prototype.getId = function() {
	return this.id;
}


/**
 * Returns the html code of this window.
 */
Window.prototype.getHtmlCode = function() {
	return this.htmlCode;
}
 

/**
 * Get the height of this windows content.
 * @return The height of the content of this window.
 */
Window.prototype.getContentHeight = function() {
	var contentCell = getElementWithAttribute(this.getHtmlCode(), "windowContentCell", "id", "td");
	return contentCell.offsetHeight;
}


/**
 * Get the width of this windows content.
 * @return The width of the content of this window.
 */
Window.prototype.getContentWidth = function() {
	var contentCell = getElementWithAttribute(this.getHtmlCode(), "windowContentCell", "id", "td");
	return contentCell.offsetWidth;
} 


// layout


/**
 * Returns this window as xml.
 */
Window.prototype.toXml = function() {
	var xmlString = 
		"<window>" +
			"<windowId>" + this.id + "</windowId>" + 
			"<xPosition>" + this.getXPosition() + "</xPosition>" +
			"<yPosition>" + this.getYPosition() + "</yPosition>" +
			"<width>" + this.getWidth() + "</width>" +
			"<height>" + this.getHeight() + "</height>";
	xmlString += this.statusToXml();
	xmlString += this.colapsedContentToXml();
	xmlString += "</window>";
	return xmlString;
}


/**
 * Returns the windows colapsed status as a xml string.
 */
Window.prototype.colapsedContentToXml = function() {
	return "<contentColapsed>" +  this.isContentColapsed() + "</contentColapsed>";
}


/**
 * Returns the windows status as a xml string. 
 */
Window.prototype.statusToXml = function() {
	var status = "";
	if (this.isActive()) {
		status += "<status>active</status>";
	}
	else if (this.isShown()) {
		status += "<status>shown</status>";
	}
	else if (this.isMinimized()) {
		status += "<status>minimized</status>";
	}	
	else {
		status += "<status>hidden</status>";
	}
	return status;
}


/**
 * Set this to true if the window has the current layout set. 
 * @param currentLayoutSet True if the window has the current layout set, false otherwise.
 * @throws Error if currentLayoutSet is undefined or null.
 */
Window.prototype.setCurrentLayoutSet = function(currentLayoutSet) {
	assertExists(currentLayoutSet, "CurrentLayoutSet is null or undefined");
	this.currentLayoutSet = currentLayoutSet;
}


/**
 * Returns true if the window has the current layout set, false otherwise.
 * The current layout is not for the window if it is hidden. 
 */
Window.prototype.isCurrentLayoutSet = function() {
	return this.currentLayoutSet;
}


/**
 * Sets the layout for the window based on a block of xml.
 * @param xmlLayout The block of xml containing the layout to set. 
 */
Window.prototype.setXmlLayout = function(xmlLayout) {
	this.setXmlContentColapsedStatus(xmlLayout);
	this.setXmlPosition(xmlLayout);
	this.setXmlSize(xmlLayout);
	this.setXmlStatus(xmlLayout);
}
 

/**
 * Set the content colapsed status according to a given xml layout.
 * @param layout The layout containing the given content colapsed status to set. 
 */
Window.prototype.setXmlContentColapsedStatus = function(layout) {
	var status = getTagNodeValue("contentColapsed", layout);
	if  (status == "true" && !this.isContentColapsed()) {
		this.colapseContent();
	}
	else if (status == "false" && this.isContentColapsed()) {
		this.expandContent();
	}
}


/**
 * Sets the status of this window according to a given xml layout. 
 * @param layout The layout containing the status to set. 
 */
Window.prototype.setXmlStatus = function(layout) {
	var status = getTagNodeValue("status", layout);
	if (isEmpty(status)) {
		return;
	}
	if ("shown" == status && this.isActive()) {
		WindowManager.instance().showWindow(this.getId());
		WindowManager.instance().deactivateWindow(this.getId());
	}
	else if ("shown" == status) {
		WindowManager.instance().showWindow(this.getId());
	}
	else if ("active" == status) {
		WindowManager.instance().activateWindow(this.getId());
	}
	else if ("hidden" == status) {
		WindowManager.instance().hideWindow(this.getId());
	}
	else if ("minimized" == status) {
		WindowManager.instance().minimizeWindow(this.getId());
	}
}


/**
 * Sets the size of the window according to a given layout. 
 * @param layout The layout containing the size to set. 
 */
Window.prototype.setXmlSize = function(layout) {
	if (!this.isShown()) {
		// the window needs to be visible to set the size of the window, move the window out of the screen
		var originalClass = getClassName(this.getHtmlCode());
		WindowManager.instance().showWindow(this.getId());
		var xPos = this.getXPosition();
		var yPos = this.getYPosition();
		var width = getTagNodeValue("width", layout);
		var height = getTagNodeValue("height", layout);
		this.setXPosition(0 - width);
		this.setYPosition(0 - height);		
	}
	
	this.resize(this.calculateXmlDeltaHeight(layout), this.calculateXmlDeltaWidth(layout));
	
	if (originalClass) {
		// move the window back to its original position
		this.setYPosition(yPos);
		this.setXPosition(xPos);
		setClass(this.getHtmlCode(), originalClass);
	}
}


/**
 * Calculates the amount of pixels the windows width needs to be resized with according
 * to a given xml layout.
 * @param layout The layout to calculate the deltaWidth according to.
 * @return The amount of pixels the windows width needs to be resized with according to layout.
 */
Window.prototype.calculateXmlDeltaWidth = function(layout) {
	var width = getTagNodeValue("width", layout);
	var deltaWidth = 0;
	if (!isEmpty(width) && isNumberString(width) && !isNegative(width)) {
		deltaWidth = parseInt(width) - this.getWidth();
	}
	return deltaWidth;
}


/**
 * Calculates the amount of pixels the windows height needs to be resized with according 
 * to a given xml layout.
 * @param layout The layout to calculate the deltaHeight according to.
 * @return The amount of pixels the windows height need to be resized with according to layout.
 */
Window.prototype.calculateXmlDeltaHeight = function(layout) {
	var height = getTagNodeValue("height", layout);
	var deltaHeight = 0;
	if (!isEmpty(height) && isNumberString(height) && !isNegative(height)) {
		deltaHeight = parseInt(height) - this.getHeight();
	}
	return deltaHeight;
}


/**
 * Sets the position of the window according to a layout. 
 * @param layout The layout containing the position to set. 
 */
Window.prototype.setXmlPosition = function(layout) {
	var xPosition = getTagNodeValue("xPosition", layout);
	if (!isEmpty(xPosition) && isNumberString(xPosition)) {
		this.setXPosition(parseInt(xPosition));
	}
	
	var yPosition = getTagNodeValue("yPosition", layout);
	if (!isEmpty(yPosition) && isNumberString(yPosition)) {
		this.setYPosition(parseInt(yPosition));
	}
}

 
// handling select tags


/**
 * Finds all the selects in the windows html code.
 */
Window.prototype.initSelects = function() {
	this.selects = this.htmlCode.getElementsByTagName("select")
	for (var i = 0; i < this.selects.length; i++) {
		this.originalSelectClasses[i] = getClassName(this.selects[i]);
	}
}


/**
 * Checks if all the select tags in this window are hidden.
 * @return True if they are hidden, false otherwise. 
 */
Window.prototype.selectsHidden = function() {
	var len = this.selects.length;
	for (var i = 0; i < len; i++) {	
		if ("inactive" != getClassName(this.selects[i])) {
			return false;
		}
	}
	return true;
}


/**
 * Shows the select tags of this window instead of the selected text. 
 */
Window.prototype.showSelects = function() {
	var len = this.selects.length;
	for (var i = 0; i < len; i++) {	
		if (isDefined(this.replaceFields[i])) {
			setClass(this.replaceFields[i], "inactive");
		}

		var selectClassToSet = "active";
		if (isDefined(this.originalSelectClasses[i]) && !isNull(this.originalSelectClasses[i]) && !isEmpty(this.originalSelectClasses[i])) {
			selectClassToSet = this.originalSelectClasses[i];
		}
		setClass(this.selects[i], selectClassToSet);
	}
}


/**
 * Replaces the select tags of this window with the selected text. 
 */
Window.prototype.hideSelects = function() {
	var len = this.selects.length;
	for (var i = 0; i < len; i++) {
		if ("inactive" == getClassName(this.selects[i])) {
			// the select is already hidden
			continue;
		}
		if (!isDefined(this.replaceFields[i])) {
			this.replaceFields[i] = this.createReplaceField(this.selects[i]);
		}
		this.originalSelectClasses[i] = getClassName(this.selects[i]);
		this.replaceSelect(this.selects[i], this.replaceFields[i]);
	}
}


/**
 * Gets a replace field for a given select. 
 * @param select The select tag to get the replace field for.
 * @return The replace tag, undefined if no replace tag has been created for the select.
 * @throws Error if select is undefined. 
 */
Window.prototype.getReplaceField = function(select) {
	assertDefined(select, "Select is undefined");
	
	var len = this.selects.length;
	var replaceTag;
	for (var i = 0; i < len; i++) {
		if (select == this.selects[i]) {
			replaceTag = this.replaceFields[i];
		}
	}
	return replaceTag;
}


/**
 * Get the index of a select into this windows select table.
 * @param select The select to get the ofset for.
 * @return The index into this windows table of selects, -1 if this window does not contain the select tag.
 * @throws Error if select is undefinde.
 */
Window.prototype.getSelectIndex = function(select) {
	assertDefined(select, "The select tag is undefined");
	
	var len = this.selects.length;
	var index = -1;
	for (var i = 0; i < len; i++) {
		if (select == this.selects[i]) {
			index = i;
		}
	}
	return index;
}


/**
 * Show the selected text of a select tag in a replace field.
 * @param selectTag The selectTag to replace with text.
 * @param replaceField The field to show the selected text in.
 */
Window.prototype.replaceSelect = function(selectTag, replaceField) {
	// add the selected text to the replaceField
	var selectedOption = selectTag.selectedIndex;
	if (selectedOption == -1) {
		// no option is selected in the select tag
		setClass(selectTag, "inactive");
		return;
	}
	var text = selectTag.options[selectedOption].firstChild.nodeValue;
	var nobr = document.createElement("nobr");
	nobr.appendChild(document.createTextNode(text));
	if (replaceField.hasChildNodes()) {
		replaceField.replaceChild(nobr, replaceField.firstChild);
	}
	else {
		replaceField.appendChild(nobr);
	}
	
	if (isDefined(this.selectReplaceClasses[this.getSelectIndex(selectTag)])) {
		setClass(replaceField, this.selectReplaceClasses[this.getSelectIndex(selectTag)]);
	}
	else if ("inactive" != getClassName(selectTag)) {
		replaceField.style.width = selectTag.offsetWidth + "px";
		replaceField.style.height = selectTag.offsetHeight + "px";
		replaceField.style.lineHeight = selectTag.offsetHeight + "px";
		replaceField.style.textAlign = "left";
		replaceField.style.overflow = "hidden";
		replaceField.setAttribute("id", "selectReplaceField");
		setClass(replaceField, "active");
	}
	setClass(selectTag, "inactive");
}	


/**
 * Creates a div tag that can hold the text for a select tag.
 * @param selectTag The select tag to create the replace field for. 
 */
Window.prototype.createReplaceField = function(selectTag) {
	var replaceDiv = document.createElement("div");
	setClass(replaceDiv, "inactive");
	selectTag.parentNode.insertBefore(replaceDiv, selectTag);
	selectTag.style.whiteSpace = "nowrap";
	return replaceDiv;
}

 
/**
 * Add a array of select tags to replace with its selected text if the window is deactivated.
 * Select tags appear always on top in IE. They need to be replaced with just the selected text 
 * when a window containing select tags is inactive. 
 * @param selects The select tags to add. 
 * @throws Error if selects is undefined.
 */
Window.prototype.addSelects = function(selects) {
	assertDefined(selects, "The selects to add is undefined");
	
	var len = selects.length;
	for (var i = 0; i < len; i++) {
		this.addSelect(selects[i]);
	}
}


/**
 * Add a select tag to replace with its selected text if the window is deactivated.
 * Select tags appear always on top in IE. They need to be replaced with just the selected text 
 * when a window containing select tags is inactive. 
 * @param select The select tag to add. 
 * @throws Error if select is undefined.
 */
Window.prototype.addSelect = function(select) {
	assertDefined(select, "The select to add is undefined");
	
	this.selects[this.selects.length] = select;
}


/**
 * Removes a select tag from the list of tags to replace with text when the window is deactivated.
 * @param select The select tag to remove. 
 * @throws Error if select is undefined. 
 */
Window.prototype.removeSelect = function(select) {
	assertDefined(select, "The select tag to remove is undefined");
	var len = this.selects.length;
	
	if (len == 1) {
		this.selects = new Array();
	}
	
	for (var i = 0; i < len; i++) {
		if (select == this.selects[i]) {
			this.selects[i] = null;
			this.selects = removeNullEntries(this.selects);
		}
	}
}


// position


/**
 * Returns the x position of the window. 
 */
Window.prototype.getXPosition = function() {
	return parseInt(this.getHtmlCode().style.left);
}


/**
 * Returns the y position of the window.
 */
Window.prototype.getYPosition = function() {
	return parseInt(this.getHtmlCode().style.top);
}


/**
 * Toggles the automatic positioning of the window.
 * @param position If this is set to true the window will be positioned according to the active 
 * window the next time the window is shown, if false the windows position is not changed.
 * @throws Error if position is undefined or null.
 */
Window.prototype.toggleAutomaticPositioning = function(position) {
	assertDefined(position, "Position is undefined or null");
	
	this.position = position;
}


/**
 * Sets a position for the window. This does not assure that thw whole window is inside the visiual
 * part of the screen after the position has been set.
 * @param yPos The y position to set for the window.
 * @param xPos The xPosition to set for the window.
 * @throws Error if yPos is not a number.
 * @throws Error if xPos is not a number.
 */
Window.prototype.setPosition = function(yPos, xPos) {
	assertIsNumber(yPos, "The y position to set for the window is not a number");
	assertIsNumber(xPos, "The x position to set for the window is not a number");
	
	this.setYPosition(yPos);
	this.setXPosition(xPos);
}


/**
 * Sets the xPosition for the window. This does not assure that the whoel window is inside the 
 * visiual part of the screen after the position has been set. 
 * @throws Error if xPos is not a number.
 */
Window.prototype.setXPosition = function(xPos) {
	assertIsNumber(xPos, "The x position to set for the window is undefined");
	
	this.htmlCode.style.left = parseInt(xPos)+ "px";
}


/**
 * Sets the yPosition for the window. This does not assure that thw whole window is inside the visiual
 * part of the screen after the position has been set.
 * @param yPos The y position to set. 
 * @throws Error if yPos is not a number.
 */
Window.prototype.setYPosition = function(yPos) {
	assertIsNumber(yPos, "The y position to set for the window is undefined");
	
	this.htmlCode.style.top = parseInt(yPos) + "px";
}


/**
 * Returns true if the window has a given position.
 * @param yPos The y position to check for.
 * @param xPos The x position to check for.
 * @return True if the windwos position is like the given, false otherwise.
 * @throws Error if yPos is undefined.
 * @throws Error if xPos is undefined.
 */
Window.prototype.hasPosition = function(yPos, xPos) {
	assertDefined(yPos, "The y position to check for the window is undefined");
	assertDefined(xPos, "The x position to check for the window is undefined");
	
	if (parseInt(this.htmlCode.style.top) != parseInt(yPos)) {
		return false;
	}
	if (parseInt(this.htmlCode.style.left) != parseInt(xPos)) {
		return false;
	}
	
	return true;
}


// visibility


/**
 * Returns true if the content section of this window has been colapsed, false otherwise. 
 */
Window.prototype.isContentColapsed = function() {
	return this.colapsedContent;
}


/**
 * Colapse the content of the window.
 */
Window.prototype.colapseContent = function() {
	if (this.isContentColapsed()) {
		return;
	}
	var contentRow = getElementWithAttribute(this.htmlCode, "windowContent", "id", "tr");
	var content = getElementWithAttribute(this.htmlCode, "windowContentCell", "id", "td");
	var contentHeight = content.offsetHeight;
	// In IE we need to set the class on the row or else one pixel of the row will still be visible,
	// while in safari setting the class on a row has no effet, we need to set on the cell
	setClass(content, "inactive");
	setClass(contentRow, "inactive");
	
	// update the size of the window
	this.expandedContentHeight = contentHeight;
	var windowHeight = parseInt(this.htmlCode.style.height);
	this.htmlCode.style.height = windowHeight - contentHeight + "px";
	this.height = parseInt(this.htmlCode.style.height);
	
	// show the expand link
	var expandImage = getElementWithAttribute(this.htmlCode, "expandContentButton", "id", "div");
	setClass(expandImage, "expandButton WindowButton");
	var colapseImage = getElementWithAttribute(this.htmlCode, "colapseContentButton", "id", "div");
	setClass(colapseImage, "inactive");
	
	this.colapsedContent = true;
	this.makeNotResizable();
	this.fireColapsedEvent();
}


/**
 * Expands the content of the window.
 */
Window.prototype.expandContent = function() {
	if (!this.isContentColapsed()) {
		return;
	}
	var contentRow = getElementWithAttribute(this.htmlCode, "windowContent", "id", "tr");
	var content = getElementWithAttribute(this.htmlCode, "windowContentCell", "id", "td");
	// In IE we need to set the class on the row or else one pixel of the row will still be visible,
	// while in safari setting the class on a row has no effet, we need to set on the cell
	setClass(content, "active");
	setClass(contentRow, "active");
	
	// update the size of the window
	var windowHeight = parseInt(this.htmlCode.style.height);
	this.htmlCode.style.height = windowHeight + this.expandedContentHeight + "px";
	this.height = parseInt(this.htmlCode.style.height);
	
	// show the colapse link
	var expandImage = getElementWithAttribute(this.htmlCode, "expandContentButton", "id", "div");
	setClass(expandImage, "inactive");
	var colapseImage = getElementWithAttribute(this.htmlCode, "colapseContentButton", "id", "div");
	setClass(colapseImage, "colapseButton WindowButton");
	
	if (this.resizable) {
		this.makeResizable();
	}
	this.colapsedContent = false;
	this.fireExpandedEvent();
}


/**
 * Hides a window.
 */
Window.prototype.hide = function() {
	if (!this.isHidden()) {
		setClass(this.htmlCode, "inactive");
		this.fireHiddenEvent();
	}
}


/**
 * Shows a window.
 */
Window.prototype.show = function() {
	if (!this.isCurrentLayoutSet() && exists(getWindowLayout(this.id))) {
		setWindowLayout(getWindowLayout(this.id));
	}
	
	if (this.isShown() || this.isActive()) {
		return;
	}
	setClass(this.htmlCode, "inactiveBorderColor");
	this.fireShownEvent();
}


/**
 * Returns true if the window is hidden, false if it is shown.
 */
Window.prototype.isHidden = function() {
	if ("inactive" == getClassName(this.htmlCode)) {
		return true;
	}
	return false;
}


/**
 * Returns true if a window is closable, false otherwise.
 */
Window.prototype.isClosable = function() {
	var closeLink = getElementWithAttribute(this.htmlCode, "closeButton", "id", "div");
	
	if (!isDefined(closeLink)) {
		return false;
	}
	if ("inactive" == closeLink.getAttribute("class") || "inactive" == closeLink.getAttribute("className")) {
		return false;
	}
	
	return true;
}


// activation/deactivation


/**
 * Returns true if the window is active.
 */
Window.prototype.isActive = function() {
	var header = getElementWithAttribute(this.htmlCode, "windowHeader", "id", "tr");
	if ("activeBorderColor".equalsIgnoreCase(getClassName(this.htmlCode)) && "activeWindowHeader".equalsIgnoreCase(getClassName(header))) {
		return true;
	}
	return false;
}


/**
 * Activates a window.
 */ 
Window.prototype.activateWindow = function() {
	if (!this.isShown()) {
		WindowManager.instance().showWindow(this.getId());
	}
	if (this.isMinimized()) {
		this.minimized = false;
	}	
	if (this.isActive()) {
		return;
	}

	setClass(this.htmlCode, "activeBorderColor");
	this.activateWindowHeader();
	// show the hidden selects again, selects are replaced with the selected text
	// to prevent them from being always ontop in IE
	if (isIE()) {
		this.showSelects();
	}
	this.fireActivatedEvent();
}


/**
 * Activate the header of a window.
 */
Window.prototype.activateWindowHeader = function() {
	var header = getElementWithAttribute(this.htmlCode, "windowHeader", "id", "tr");
	setClass(header, "activeWindowHeader");
}


/**
 * Deactivates a window.
 */
Window.prototype.deactivateWindow = function() {
	if (this.isDeactivated()) {
		// the window is already deactivated
		return;
	}
	
	// deactivate the window header
	this.deactivateWindowHeader();
	setClass(this.htmlCode, "inactiveBorderColor");
	// hide this windows select tags
	if (isIE()) {	
		this.hideSelects();
	}
	this.fireDeactivatedEvent();
}


/**
 * Deactivates the header of a window.
 */
Window.prototype.deactivateWindowHeader = function() {
	var header = getElementWithAttribute(this.htmlCode, "windowHeader", "id", "tr");
	setClass(header, "inactiveWindowHeader");
}


/**
 * Returns true if the window is deactivated, false otherwise.
 */
Window.prototype.isDeactivated = function() {
	var header = getElementWithAttribute(this.htmlCode, "windowHeader", "id", "tr");
	if ("inactiveBorderColor".equalsIgnoreCase(getClassName(this.htmlCode)) && "inactiveWindowHeader".equalsIgnoreCase(getClassName(header))) {
		return true;
	}
	return false;
}


/**
 * Sets a windows z index.
 * @param index The z index to set.
 */
Window.prototype.setZIndex = function(index) {
	this.htmlCode.style.zIndex = index;
}	


/**
 * Checks if a window is shown or not.
 */
Window.prototype.isShown = function() {
	if (this.htmlCode.getAttribute("class") == "inactive" || this.htmlCode.getAttribute("className") == "inactive") {
		return false;
	}
	return true;
}


/**
 * Minimize the window.
 */
Window.prototype.minimize = function() {
	setClass(this.htmlCode, "inactive");
	this.minimized = true;
	this.fireMinimizedEvent();
}


/**
 * Maximizes the window.
 */
Window.prototype.maximize = function() {
	setClass(this.htmlCode, "inactiveBorderColor");
	this.minimized = false;
	this.fireMaximizedEvent();
}


/**
 * Checks if the window is minimized.
 * @return True if the window is minimized, false otherwise. 
 */
Window.prototype.isMinimized = function() {
	return this.minimized;
}

 
/**
 * Sets whether this window is modal or not. 
 * @param modal True if this window should be moda, false for normal. 
 * @throws Error if modal does not exists or is not a boolean. 
 */
Window.prototype.setModal = function(modal) {
	assertExists(modal, "The modal value to set does not exist");
	if (typeof modal != "boolean") {
		throw new Error("The modal value to set is not a boolean");
	}
	this.modal = modal;
}


/**
 * Returns true if this window is modal, false otherwise.
 */
Window.prototype.isModal = function() {
	return this.modal;
}


// moving of windows


/**
 * Add a element that should be moved along with the window.
 * @param element A reference to the html code of the element.
 * @throws Error if element is null or undefined. 
 */
Window.prototype.addMoveElement = function(element) {
	assertDefined(element, "Element is null or undefined");
	this.moveElements[this.moveElements.length] = element;
} 
 
 
/**
 * Remove one of the elements that should be moved along with the window.
 * @param element The element to remove.
 * @throws Error if element is null or undefined.
 */
Window.prototype.removeMoveElement = function(element) {
	assertDefined(element, "Element is null or undefined");

	for (var i = 0; i < this.moveElements.length; i++) {
		if (element == this.moveElements[i]) {
			this.moveElements[i] = null;
			this.moveElements = removeNullEntries(this.moveElements);
			break;
		}
	}
} 
 

/**
 * Checks if a given element is in this windows move elements. A windows move elements
 * are moved along with the window if the window is moved. 
 * @param element The element to check.
 * @return true if the element is among this windows move elements, false otherwise. 
 * @throws Error if element is undefined or null.
 */
Window.prototype.containsMoveElement = function(element) {
	assertDefined(element, "Element is null or undefined");
	
	for (var i = 0; i < this.moveElements.length; i++) {
		if (element == this.moveElements[i]) {
			return true;
		}
	}
	return false;
}


/**
 * Move the elements that should be moved along with the window.
 * @param movedX The amount of pixels to move the elements on the x-axis
 * @param movedY The amount of pixels to move the elements on the y-axis
 */ 
Window.prototype.moved = function(movedX, movedY) {
	for (var i = 0; i < this.moveElements.length; i++) {
		var xPos = parseInt(this.moveElements[i].style.left);
		var yPos = parseInt(this.moveElements[i].style.top);
		xPos += movedX;
		yPos += movedY;
		this.moveElements[i].style.left = xPos + "px";
		this.moveElements[i].style.top = yPos + "px";
	}
}


/**
 * Begins dragging a window.
 * @param event The mouse event that caused the dragging. 
 */
Window.prototype.beginDrag = function(event) {
	this.dragHelper.beginDrag(event);
}


/**
 * Needs to be called when dragging of a window is being started, hides
 * the select tags while dragging.
 */
Window.prototype.dragStart = function() {
	if (isIE()) {
		this.hideSelects();
	}
}


/**
 * Needs to be called when dragging of a window is being stopped, 
 * shows the select tags on drag stop.
 */
Window.prototype.dragStop = function() {
	if (isIE()) {
		this.showSelects();
	}
}


/**
 * Make this window draggable.
 */
Window.prototype.makeDragable = function() {
	var id = this.id;
	this.header.onmousedown = function(event) {
		WindowManager.instance().getWindow(id).beginDrag(event);
	}
}


/**
 * Make this window undraggable.
 */
Window.prototype.makeUnDragable = function() {
	this.header.onmousedown = "";
}


/**
 * The window is not to be moved on any of its buttons, add them to the 
 * drag exceptions.
 */
Window.prototype.setDragExceptions = function() {
	this.addDragException("closeButton", "id", "div");
	this.addDragException("expandContentButton", "id", "div");
	this.addDragException("colapseContentButton", "id", "div");
	this.addDragException("setupButton", "id", "div");
	this.addDragException("minimizeContentButton", "id", "div");
}


/**
 * Adds a element inside this windows html code to the drag exceptions.
 * @param value The value of the attribute.
 * @param attribute The attribute containing the value (id, name...).
 * @param elementType The type of the element (div, a, td...)
 */
Window.prototype.addDragException = function(value, attribute, elementType) {
	var element = getElementWithAttribute(this.htmlCode, value, attribute, elementType);
	if (isDefined(element) && !isNull(element)) {
		this.dragHelper.addDragException(element)
	}
}


// resizing of windows


/**
 * Resize a window.
 * @param edge At which edge the element is been resized (bottom, top, left right, 
 * topLeft, topRight, bottomLeft, bottomRight);
 * @param event The event.
 */
Window.prototype.beginResize = function(edge, event) {
	this.resizeHelper.setMinWidth(this.getMinWidth());
	this.resizeHelper.setMinHeight(this.getMinHeight());
	this.resizeHelper.beginResize(event, edge);
}


/**
 * Adjusts the size of the window to fit the content. The window needs to be 
 * visible when doing this. 
 */
Window.prototype.adjustSizeToContent = function() {
	var borderSize = (Window.BORDER_WIDTH * 2) + (Window.RESIZE_AREA_SIZE * 2);
	var contentWindowWidth = this.calculateContentWidth() + borderSize; 
	var contentWindowHeight = this.calculateContentHeight() + Window.WINDOW_HEADER_HEIGHT + borderSize
	var deltaWidth = contentWindowWidth - this.getWidth();
	var deltaHeight = contentWindowHeight - this.getHeight();
	this.resize(deltaHeight, deltaWidth);
	this.resized();
}


/**
 * Calculates the width of the content. 
 * @return The calculated with of the content. 
 */
Window.prototype.calculateContentWidth = function() {
	var contentCell = getElementWithAttribute(this.htmlCode, "windowContentCell", "id", "td");
	
	var width = 0;
	for (var i = 0; i < contentCell.childNodes.length; i++) {
		if (contentCell.childNodes[i].offsetWidth > width) {
			width = contentCell.childNodes[i].offsetWidth;
		}
	}
	return width;
}


/**
 * Calculates the height of the content. 
 * @return The calculated content of the content. 
 */
Window.prototype.calculateContentHeight = function() {
	var contentCell = getElementWithAttribute(this.htmlCode, "windowContentCell", "id", "td");
	
	var height = 0;
	for (var i = 0; i < contentCell.childNodes.length; i++) {
		height += contentCell.childNodes[i].offsetHeight;
	}
	return height;
}


/**
 * Returns the width of this window.
 */
Window.prototype.getWidth = function() {
	return parseInt(this.htmlCode.style.width);
}


/**
 * Returns the height of this window.
 */
Window.prototype.getHeight = function() {
	return parseInt(this.htmlCode.style.height);
}


/**
 * Returns True if this window can be resized with the given pixels.
 * making this window to small.
 * @param deltaHeight The amount of pixels to increase the height with.
 * @param deltaWidth The amount of pixels to increase the width with.
 */
Window.prototype.isValidResize = function(deltaHeight, deltaWidth) {
	var resizableElements = this.resizableElements;
	for (i = 0; i < resizableElements.length; i++) {
		// check that no negative sizes are set
		if (resizableElements[i].offsetHeight + deltaHeight <= 0) {
			return false;
		}
	}

	if (this.height + deltaHeight <= this.minHeight) {
		return false;
	}

	if (this.width + deltaWidth <= this.minWidth) {
		return false;
	}	
	
	return true;
}


/**
 * Sets this window to be resizable.
 * @param resizable If this is true the window is made resizable, if false
 * the winow is made not resizable.
 * @throws Error if resizable is undefined.
 */
Window.prototype.setResizable = function(resizable) {
	assertDefined(resizable, "Resizable is undefined");
	
	if (resizable) {
		this.resizable = true;
		this.makeResizable();
	}
	else {
		this.resizable = false;
		this.makeNotResizable();
	}
}


/**
 * Makes this window resizable.
 */
Window.prototype.makeResizable = function() {
	var thisWindow = this;
	var id = this.id;
	
	var topLeftResizing = getElementWithAttribute(this.htmlCode, "topleftresizing", "id", "td");
	topLeftResizing.style.cursor = 'se-resize';
	topLeftResizing.onmousedown = function(event) {
		thisWindow.beginResize('topleft', event)
	}
	
	var topResizing = getElementWithAttribute(this.htmlCode, "topresizing", "id", "td");
	topResizing.style.cursor = "n-resize";
	topResizing.onmousedown = function(event) {
		thisWindow.beginResize('top', event)
	}
	
	var topRightResizing = getElementWithAttribute(this.htmlCode, "toprightresizing", "id", "td");
	topRightResizing.style.cursor = "ne-resize";
	topRightResizing.onmousedown = function(event) {
		thisWindow.beginResize('topright', event)
	}
	
	var leftResizing = getElementWithAttribute(this.htmlCode, "leftresizing", "id", "td");
	leftResizing.style.cursor = "w-resize";
	leftResizing.onmousedown = function(event) {
		thisWindow.beginResize('left', event)
	}
	
	var rightResizing = getElementWithAttribute(this.htmlCode, "rightresizing", "id", "td");
	rightResizing.style.cursor = "w-resize";
	rightResizing.onmousedown = function(event) {
		thisWindow.beginResize('right', event)
	}
	
	var bottomLeftResizing = getElementWithAttribute(this.htmlCode, "bottomleftresizing", "id", "td");
	bottomLeftResizing.style.cursor = "sw-resize";
	bottomLeftResizing.onmousedown = function(event) {
		thisWindow.beginResize('bottomleft', event)
	}
	
	var bottomResizing = getElementWithAttribute(this.htmlCode, "bottomresizing", "id", "td");
	bottomResizing.style.cursor = "s-resize";
	bottomResizing.onmousedown = function(event) {
		thisWindow.beginResize('bottom', event)
	}
	
	var bottomRightResizing = getElementWithAttribute(this.htmlCode, "bottomrightresizing", "id", "td");
	bottomRightResizing.style.cursor = "nw-resize";
	bottomRightResizing.onmousedown = function(event) {
		thisWindow.beginResize('bottomright', event)
	}
}


/**
 * Makes this window not resizable.
 */
Window.prototype.makeNotResizable = function() {
	var topLeftResizing = getElementWithAttribute(this.htmlCode, "topleftresizing", "id", "td");
	topLeftResizing.style.cursor = "";
	topLeftResizing.onmousedown = function() {};
	
	var topResizing = getElementWithAttribute(this.htmlCode, "topresizing", "id", "td");
	topResizing.style.cursor = "";
	topResizing.onmousedown = function() {};
	
	var topRightResizing = getElementWithAttribute(this.htmlCode, "toprightresizing", "id", "td");
	topRightResizing.style.cursor = "";
	topRightResizing.onmousedown = function() {};
	
	var leftResizing = getElementWithAttribute(this.htmlCode, "leftresizing", "id", "td");
	leftResizing.style.cursor = "";
	leftResizing.onmousedown = function() {};
	
	var rightResizing = getElementWithAttribute(this.htmlCode, "rightresizing", "id", "td");
	rightResizing.style.cursor = "";
	rightResizing.onmousedown = function() {};
	
	var bottomLeftResizing = getElementWithAttribute(this.htmlCode, "bottomleftresizing", "id", "td");
	bottomLeftResizing.style.cursor = "";
	bottomLeftResizing.onmousedown = function() {};
	
	var bottomResizing = getElementWithAttribute(this.htmlCode, "bottomresizing", "id", "td");
	bottomResizing.style.cursor = "";
	bottomResizing.onmousedown = function() {};
	
	var bottomRightResizing = getElementWithAttribute(this.htmlCode, "bottomrightresizing", "id", "td");
	bottomRightResizing.style.cursor = "";
	bottomRightResizing.onmousedown = function() {};
}


/**
 * Sets a given size for the window. 
 * @param height The height to set. 
 * @param width The width to set. 
 * @throws Error if height is negative or not a number.
 * @throws Error if width is negative or not a number. 
 */
Window.prototype.setSize = function(height, width) {
	assertNotNegative(height, "The height to set is not a number or negative");
	assertNotNegative(width, "The width to set is not a number or negative");
	
	var deltaHeight = height - this.getHeight();
	var deltaWidth = width - this.getWidth();
	this.resize(deltaHeight, deltaWidth);
}


/**
 * Used to resize the window.
 * @param deltaHeight The amount of pixels to increase the height with.
 * @param deltaWidth The amount of pixels to increase the width with.
 * @throws Error if deltaHeight is not a number. 
 * @throws Error if deltaWidth is not a number. 
 * @param returns True if the resize could be performed, false if the resize is illegal
 */
Window.prototype.resize = function(deltaHeight, deltaWidth) {
	assertIsNumber(deltaHeight, "DeltaHeight is not a number: " + deltaHeight);
	assertIsNumber(deltaWidth, "DeltaWidth is not a number: " + deltaWidth);

	var resizedHeight = this.resizeWindowHeight(deltaHeight);
	var resizedWidth = this.resizeWindowWidth(deltaWidth);
	this.resizeElements(resizedHeight, resizedWidth);
	this.resized();
}


/**
 * Called when a window has been resized.
 */
Window.prototype.resized = function() {
	if (!this.isShown()) {
		// the window is not shown we can not use offsetHeight and offsetWidth to
		// determine the window size
		return;
	}
	// during the resize the resize area can have been made invisible, expand
	// the size of the window to show it if neccessary
	this.makeResizeAreaVisible();
	// make sure the size of the window is correct 
	this.checkWindowSize();
}



/**
 * Returns the offsetWidth of the window.
 */
Window.prototype.getOffsetWidth = function() {
	var width;
	if (isOpera()) {
		// in Opera the border is drawn on the outside of the window
		width = this.htmlCode.offsetWidth - (Window.BORDER_WIDTH * 2);
	}
	else {
		// in other browsers (IE & FF) the border is drawn on the inside of the window
		width = this.htmlCode.offsetWidth;
	}
	if (width < 0) {
		return 0;
	}
	return width;
}


/**
 * Returns the offsetHeight of the window.
 */
Window.prototype.getOffsetHeight = function() {
	return this.htmlCode.offsetHeight;
}


/**
 * Make sure the window is of correct size. The style.height can differ from 
 * the actual height of the window shown if the window was made smaller 
 * than the content can be made.
 */
Window.prototype.checkWindowSize = function() {
	if (this.getOffsetHeight() != parseInt(this.htmlCode.style.height)) {
		this.height = this.getOffsetHeight();		
		this.htmlCode.style.height = this.height + "px";
	}

	if (this.getOffsetWidth() != parseInt(this.htmlCode.style.width)) {
		this.width = this.getOffsetWidth();
		this.htmlCode.style.width = this.width + "px";
	}
}


/**
 * Resizes the elements that are to be resized along with the window.
 * @param deltaHeight The amount of pixels to increase the height with.
 * @param deltaWidth The amount of pixels to increase the width with.
 */
Window.prototype.resizeElements = function(deltaHeight, deltaWidth) {
	if (this.resizableElements.length > 0) {
		resizeHtmlElements(this.resizableElements, deltaHeight, deltaWidth);
	}
}


/**
 * Resizes the window by some given amount of pixels.
 * @param deltaHeight The amount of pixels to change the windows height.
 * @param deltaWidth The amount of pixels to change the windows width.
 * @return A associative array containing the height and width the window was actually resized with.
 */
Window.prototype.resizeWindow = function(deltaHeight, deltaWidth) {
	var resized = new Array();

	resized["height"] = this.resizeWindowHeight(deltaHeight);
	resized["width"] = this.resizeWindowWidth(deltaWidth);
	return resized;
}


/**
 * Resizes the windows width.
 * @param deltaWidth The amount of pixels to increase the width with.
 * @param returns The amount of pixels the windows width was actually resized with.
 */
Window.prototype.resizeWindowWidth = function (deltaWidth) {
	var newWidth = this.getWidth() + deltaWidth;
	var diff = deltaWidth;
	if (newWidth < 0) {
		newWidth = 0;
		diff = this.getHeight();
	}
	
	if (isDefined(this.getMinWidth()) && newWidth < this.getMinWidth()) {
		newWidth = this.getMinWidth();
		diff = this.getMinWidth() - this.getWidth();
	}
	
	this.htmlCode.style.width = newWidth + "px";
	return diff;
}


/**
 * Resizes the windows height.
 * @param deltaHeight The amount of pixels to increase the height with.
 * @param returns The amount of pixels the windows height was actually resized with.
 */
Window.prototype.resizeWindowHeight = function (deltaHeight) {
	var newHeight = this.getHeight() + deltaHeight;
	var diff = deltaHeight;
	if (newHeight < 0) {
		newHeight = 0;
		diff = this.getHeight();
	}

	if (isDefined(this.getMinHeight()) && newHeight < this.getMinHeight()) {
		newHeight = this.getMinHeight();
		diff = this.getHeight() - this.getMinHeight();
	}
	this.htmlCode.style.height = newHeight + "px";
	return diff;
}


/**
 * Set a element that needs to be resized when the window is been resized.
 * @param element A reference to the elements html code.
 * @throws Error if element is udnefined or null. 
 */
Window.prototype.setResizableElements = function(element) {
	assertDefined(element, "Element is null or undefined");
	if (!this.isResizableElement(element)) {
		this.resizableElements[this.resizableElements.length] = element;
	}	
}


/**
 * Gets the resizable elements that are resized along with this table. 
 * @return A array containing the elements to be reisze along with the table. 
 */
Window.prototype.getResizableElements = function() {
	return this.resizableElements;
}


/**
 * Removes one of the elements that are resized when the window is resized.
 * @param element A reference to the elements html code.
 * @throws Error if element is udnefined or null.
 */
Window.prototype.removeResizableElement = function (element) {
	assertExists(element, "Element is null or undefined");
	for (var i = 0; i < this.resizableElements.length; i++) {
		if (element == this.resizableElements[i]) {
			this.resizableElements[i] = null;
		}
	}
	
	this.resizableElements = removeNullEntries(this.resizableElements);
}


/**
 * Checks if a given element is among the resizable elements. 
 * @param element The element to check.
 * @return True if element is among the resizable elements, false otherwise.
 * @throws Error if element is undefined or null.
 */
Window.prototype.isResizableElement = function(element) {
	assertExists(element, "Element to check is undefined or null");
	for (var i = 0; i < this.resizableElements.length; i++) {
		if (element == this.resizableElements[i]) {
			return true;
		}
	}
	return false;
}

 
/**
 * Set the min size of the window.
 * @param minHeight The min height.
 * @param minWidth The min width.
 * @throws Error if minHeight is negative.
 * @throws Error if minWidth is negative.
 */
Window.prototype.setMinSize = function(minHeight, minWidth) {
	this.setMinWidth(minWidth);
	this.setMinHeight(minHeight);
}


/**
 * Set min width.
 * @param minWidth The min width.
 * @throws Error if minWidth is negative.
 */
Window.prototype.setMinWidth = function(minWidth) {
	assertNotNegative(minWidth, "MinWidth is negative");
	
	this.minWidth = minWidth;
}


/**
 * Set the min size of the window.
 * @param minHeight The min height.
 * @throws Error if minHeight is negative.
 */
Window.prototype.setMinHeight = function(minHeight) {
	assertNotNegative(minHeight, "MinHeight is negative");
	
	this.minHeight = minHeight;
}


/**
 * Get the min height set for this window, undefined if no min height has not been set. 
 */
Window.prototype.getMinHeight = function() {
	return this.minHeight;
}


/**
 * Get the min width set for this window, undefined if no min width has not been set. 
 */
Window.prototype.getMinWidth = function() {
	return this.minWidth;
}


/**
 * Returns true if the windows resize area is visible with the correct width and heihgt, false otherwise.
 * If the content is to large for the resize area can be be invisible.
 */
Window.prototype.isResizaAreaVisible = function() {
	var topLeftResizing = getElementWithAttribute(this.htmlCode, "topleftresizing", "id", "td");
	if (topLeftResizing.offsetWidth < Window.RESIZE_AREA_SIZE) {
		return false;
	}
	
	var topResizing = getElementWithAttribute(this.htmlCode, "topresizing", "id", "td");
	if (topResizing.offsetHeight < Window.RESIZE_AREA_SIZE) {
		return false;
	}
	
	var topRightResizing = getElementWithAttribute(this.htmlCode, "toprightresizing", "id", "td");
	if (topRightResizing.offsetWidth < Window.RESIZE_AREA_SIZE) {
		return false;
	}
	
	var leftResizing = getElementWithAttribute(this.htmlCode, "leftresizing", "id", "td");
	if (leftResizing.offsetWidth < Window.RESIZE_AREA_SIZE) {
		return false;
	}
	
	var rightResizing = getElementWithAttribute(this.htmlCode, "rightresizing", "id", "td");
	if (rightResizing.offsetWidth < Window.RESIZE_AREA_SIZE) {
		return false;
	}
	
	var bottomLeftResizing = getElementWithAttribute(this.htmlCode, "bottomleftresizing", "id", "td");
	if (bottomLeftResizing.offsetWidth < Window.RESIZE_AREA_SIZE) {
		return false;
	}
	
	var bottomResizing = getElementWithAttribute(this.htmlCode, "bottomresizing", "id", "td");
	if (bottomResizing.offsetHeight < Window.RESIZE_AREA_SIZE) {
		return false;
	}
	
	var bottomRightResizing = getElementWithAttribute(this.htmlCode, "bottomrightresizing", "id", "td");
	if (bottomRightResizing.offsetWidth < Window.RESIZE_AREA_SIZE) {
		return false;
	}
	
	return true;
}


/**
 * Increase the size of the window to make the resizeArea visible if neccessary.
 */
Window.prototype.makeResizeAreaVisible = function() {
	var contentCell = getElementWithAttribute(this.htmlCode, "windowContentCell", "id", "td");
	var contentWidth = contentCell.offsetWidth;
	var windowWidth = parseInt(this.htmlCode.style.width);
	var windowHeight = parseInt(this.htmlCode.style.height);
	var resized = false;

	if (contentWidth > windowWidth) {
		// the content is larger than the window
		var widthDiff = contentWidth - windowWidth;
		this.htmlCode.style.width = (parseInt(this.htmlCode.style.width) + widthDiff) + "px";
		windowWidth = contentWidth;
		resized = true;
	}
	
	var topLeftResizing = getElementWithAttribute(this.htmlCode, "topleftresizing", "id", "td");
	if (topLeftResizing.offsetWidth < Window.RESIZE_AREA_SIZE) {
		var widthDiff = Window.RESIZE_AREA_SIZE - topLeftResizing.offsetWidth;
		this.htmlCode.style.width = (parseInt(this.htmlCode.style.width) + widthDiff) + "px";
		windowWidth += widthDiff;
		resized = true;
	}
	
	var topResizing = getElementWithAttribute(this.htmlCode, "topresizing", "id", "td");
	if (topResizing.offsetHeight < Window.RESIZE_AREA_SIZE) {
		var heightDiff = Window.RESIZE_AREA_SIZE - topResizing.offsetHeight;
		this.htmlCode.style.height = (parseInt(this.htmlCode.style.height) + heightDiff) + "px";
		windowHeight += heightDiff;
		resized = true;
	}
	
	var topRightResizing = getElementWithAttribute(this.htmlCode, "toprightresizing", "id", "td");
	if (topRightResizing.offsetWidth < Window.RESIZE_AREA_SIZE) {
		var widthDiff = Window.RESIZE_AREA_SIZE - topRightResizing.offsetWidth;
		this.htmlCode.style.width = (parseInt(this.htmlCode.style.width) + widthDiff) + "px";
		windowWidth += widthDiff;
		resized = true;
	}
	
	var leftResizing = getElementWithAttribute(this.htmlCode, "leftresizing", "id", "td");
	if (leftResizing.offsetWidth < Window.RESIZE_AREA_SIZE) {
		var widthDiff = Window.RESIZE_AREA_SIZE - leftResizing.offsetWidth;
		this.htmlCode.style.width = (parseInt(this.htmlCode.style.width) + widthDiff) + "px";
		windowWidth += widthDiff;
		resized = true;	
	}
	
	var rightResizing = getElementWithAttribute(this.htmlCode, "rightresizing", "id", "td");
	if (rightResizing.offsetWidth < Window.RESIZE_AREA_SIZE) {
		var widthDiff = Window.RESIZE_AREA_SIZE - rightResizing.offsetWidth;
		this.htmlCode.style.width = (parseInt(this.htmlCode.style.width) + widthDiff) + "px";
		windowWidth += widthDiff;
		resized = true;		
	}
	
	var bottomLeftResizing = getElementWithAttribute(this.htmlCode, "bottomleftresizing", "id", "td");
	if (bottomLeftResizing.offsetWidth < Window.RESIZE_AREA_SIZE) {
		var widthDiff = Window.RESIZE_AREA_SIZE - bottomLeftResizing.offsetWidth;
		this.htmlCode.style.width = (parseInt(this.htmlCode.style.width) + widthDiff) + "px";
		windowWidth += widthDiff;
		resized = true;		
	}
	
	var bottomResizing = getElementWithAttribute(this.htmlCode, "bottomresizing", "id", "td");
	if (bottomResizing.offsetHeight < Window.RESIZE_AREA_SIZE) {
		var heightDiff = Window.RESIZE_AREA_SIZE - bottomResizing.offsetHeight;
		this.htmlCode.style.height = (parseInt(this.htmlCode.style.height) + heightDiff) + "px";
		windowHeight += heightDiff;
		resized = true;
	}
	
	var bottomRightResizing = getElementWithAttribute(this.htmlCode, "bottomrightresizing", "id", "td");
	if (bottomRightResizing.offsetWidth < Window.RESIZE_AREA_SIZE) {
		var widthDiff = Window.RESIZE_AREA_SIZE - bottomRightResizing.offsetWidth;
		this.htmlCode.style.width = (parseInt(this.htmlCode.style.width) + widthDiff) + "px";
		windowWidth += widthDiff;
		resized = true;
	}

	this.width = windowWidth;
	this.height = windowHeight;
	return resized;
}


// window content


/**
 * Adds content to the window.
 * @param content The html code to add to the window.
 * @throws Error if content is null or undefined.
 */
Window.prototype.addContent = function(content) {
	assertDefined(content, "Content is undefined or null");
	
	var contentCell = getElementWithAttribute(this.htmlCode, "windowContentCell", "id", "td");
	if (isNull(contentCell)) {
		// the cell containing the content was not found
		return;
	}
	
	contentCell.appendChild(content);
}


// util functions


/**
 * Gets the windows title.
 * @return The title of the window.
 */
Window.prototype.getWindowTitle = function () {
	var titleNode = getElementWithAttribute(this.getHtmlCode(), "windowTitle", "name", "td");
	return getTekstValue(titleNode);
}


/**
 * Sets the windows title.
 * @param title The title to set.
 * @return The title of the window.
 * @throws Error if title is undefined, null or empty.
 */
Window.prototype.setWindowTitle = function (title) {
	assertNotEmpty(title, "The title to set for the window is undefined, null or empty: " + title);
	var titleNode = getElementWithAttribute(this.getHtmlCode(), "windowTitle", "name", "td");
	var nobrNode = titleNode.firstChild;
	nobrNode.firstChild.nodeValue = title;
}
 

/**
 * Closes the window. The html code of the window is removed.
 * @param Optional parent window parameter. If specified, the parent window is activated.
 */
Window.prototype.closeWindow = function() {
	WindowManager.instance().getWindowRootElement().removeChild(this.htmlCode);
	WindowManager.instance().removeWindow(this);
	
	// set parent window active, if specified
	if (arguments.length == 1) {
		WindowManager.instance().activateWindow(arguments[0].getId());
	}
	
	this.fireClosedEvent();
}


// html


/**
 * Gets a reference to the html code of a window with a id.
 * @param id The id of the window.
 * @return The window. 
 * @throws Error if the id is undefined, null or empty.
 */
function GetWindowCode(id) {
	assertNotEmpty(id, "Id is undefined, null or empty");
	return getElementWithAttribute(document.body, id, "id", "table");
}


/**
 * Get a reference to the window header.
 * @return The window header.
 */
Window.prototype.getWindowHeader = function() {
	return getElementWithAttribute(this.htmlCode, "windowHeader", "id", "tr");
}


// events


/**
 * Fires a colapsed event. 
 */
Window.prototype.fireColapsedEvent = function() {
	if (!eventsAvailable()) {
		return;
	}
	getEventServer().notifyListeners(new Event(WINDOW_COLLAPSED_EVENT, this.id));
}


/**
 * Fires a expanded event. 
 */
Window.prototype.fireExpandedEvent = function() {
	if (!eventsAvailable()) {
		return;
	}
	getEventServer().notifyListeners(new Event(WINDOW_EXPANDED_EVENT, this.id));
}


/**
 * Fires a show event. 
 */
Window.prototype.fireShownEvent = function() {
	if (!eventsAvailable()) {
		return;
	}
	getEventServer().notifyListeners(new Event(WINDOW_SHOWN_EVENT, this.id));
}


/**
 * Fires a hidden event. 
 */
Window.prototype.fireHiddenEvent = function() {
	if (!eventsAvailable()) {
		return;
	}
	getEventServer().notifyListeners(new Event(WINDOW_HIDDEN_EVENT, this.id));
}


/**
 * Fires a activated event. 
 */
Window.prototype.fireActivatedEvent = function() {
	if (!eventsAvailable()) {
		return;
	}
	getEventServer().notifyListeners(new Event(WINDOW_ACTIVATED_EVENT, this.id));
}


/**
 * Fires a window deactivated event. 
 */
Window.prototype.fireDeactivatedEvent = function() {
	if (!eventsAvailable()) {
		return;
	}
	getEventServer().notifyListeners(new Event(WINDOW_DEACTIVATED_EVENT, this.id));
}


/**
 * Fires a window closed event. 
 */
Window.prototype.fireClosedEvent = function() {
	if (!eventsAvailable()) {
		return;
	}
	getEventServer().notifyListeners(new Event(WINDOW_CLOSED, this));
}


/**
 * Fires a window minimized event.
 */
Window.prototype.fireMinimizedEvent = function() {
	if (!eventsAvailable()) {
		return;
	}
	getEventServer().notifyListeners(new Event(WINDOW_MINIMIZED, this.id));
}


/**
 * Fires a window maximized event.
 */
Window.prototype.fireMaximizedEvent = function() {
	if (!eventsAvailable()) {
		return;
	}
	getEventServer().notifyListeners(new Event(WINDOW_MAXIMIZED, this.id));
}


/**
 * Returns true if a EventServer is available.
 */
function eventsAvailable() {
	try {
		if (isDefined(getEventServer)) {
			return true;
		}
	}
	catch(Error) {
		return false;
	}
	try {
		if (isDefined(Event)) {
			return true;
		}
	}
	catch(Error) {
		return false;
	}
	return true;
}