/**
 * This class subclassed the draghelper to drag windows. 
 */
 

// construction


/**
 * Factory method to construct a window drag helper.
 * @param window The window to drag. 
 * @throws Error if window is undefined or null.
 */
function createWindowDragHelper(window) {
	assertExists(window, "Window is undefined or null");
	
	var superInstance = new DragHelper(window.getHtmlCode()); 
	var subInstance = new WindowDragHelper(window);
	subInstance = createSubClass(superInstance, subInstance);
	return subInstance;	
}



/**
 * Constructs a window drag helper.  
 * @param window The window to be dragged.
 * @throws Error if window is undefined or null.
 */
function WindowDragHelper(window) {
	assertExists(window, "Window is undefined or null");
	
	this.window = window;
	
	/**
	 * Used to track the total amount of pixels the element has been moved during a movement operation
	 */
	this.totalMovedX = 0;
	this.totalMovedY = 0;
	
	
	/**
	 * The function used to listen to mouse moved events. 
	 */
	this.mouseMoveListener = this.createMouseMoveListener();
	
	
	/**
	 * The function used to listen to mouse up events. 
	 */
	this.mouseUpListener = this.createMouseUpListener();
}


// moving


/**
 * Called from a mousedown event handler. This is overridden to reset the movement tracking for the window.  
 * @param event The event.
 * @param optional argument. If this is set to false events are not propagated. The default
 * is to stop propagation of events.
 */
WindowDragHelper.prototype.beginDrag = function(event) {
	var argumentsToSuper = new Array();
	argumentsToSuper[0] = event;
	this.callSuper("beginDrag", argumentsToSuper);
	this.totalMovedX = 0;
	this.totalMovedY = 0;
}


/**
 * The handler capturing mouse movement events when a element is been 
 * dragged. It is responsibel for moving the element. This is overridden to notify the
 * window that it is being dragged. 
 */
WindowDragHelper.prototype.moveHandler = function(event) {
	this.window.dragStart();
	var argumentsToSuper = new Array();
	argumentsToSuper[0] = event;
	this.callSuper("moveHandler", argumentsToSuper);
}



/**
 * Move the element according to a mouse move event. This is overridden to let
 * the window know it has been moved. 
 * @param e The mouse move event.
 */
WindowDragHelper.prototype.moveElement = function(event) {
	// calculate new position of the element
	var xValue = this.calculateXValue(event);
	var yValue = this.calculateYValue(event);
	
	if (!this.isValidMove(xValue, yValue)) {
		return;
	}

	var argumentsToSuper = new Array();
	argumentsToSuper[0] = event;
	this.callSuper("moveElement", argumentsToSuper);
		
	if (!this.useDragHelper) {
		// let the window know it has been moved
		var movedX = event.clientX - startX;
		startX = event.clientX;
		var movedY = event.clientY - startY;
		startY = event.clientY;
		this.moved(movedX, movedY);
	}
	else {
		this.trackMovement(event);
	}
}


/**
 * This is the handler that captures the final mouse up event that
 * at the end of a drag. This is overridden to let the window know the dragging is over. 
 */
WindowDragHelper.prototype.upHandler = function(event) {
	event = exists(event) ? event : window.event;

	if (this.useDragHelper) {
		this.window.moved(this.totalMovedX, this.totalMovedY);
	}
	
	if(this.window.dragStop) {
		this.window.dragStop();
	}

	var argumentsToSuper = new Array();
	argumentsToSuper[0] = event;
	this.callSuper("upHandler", argumentsToSuper);
}


/**
 * Tracks the amount of pixels the element has been moved throughout 
 * the movement.
 */
WindowDragHelper.prototype.trackMovement = function(e) {
 	this.totalMovedX = e.clientX - this.startX;
 	this.totalMovedY = e.clientY - this.startY;
}


// implementation


/**
 * Creates the function to use to listen to mouse up events. This is overridden to use this object.
 * @return The function to set as mouse up listener. 
 */
WindowDragHelper.prototype.createMouseUpListener = function() {
	var thisObject = this;
	return function(e) { thisObject.upHandler(e)};
}


/**
 * Creates the function to use to listen for mouse move events. This is overridden to use this object.
 * @return The function to set as mouse move listener. 
 */
WindowDragHelper.prototype.createMouseMoveListener = function() {
	var thisObject = this;
	return function(e) { thisObject.moveHandler(e)};
}


/**
 * Create the drag helper element.
 */
WindowDragHelper.prototype.createDragHelper = function() {
	this.dragHelper = document.createElement("div");
	setClass(this.dragHelper, "dragHelper");
	WindowManager.instance().getWindowRootElement().appendChild(this.dragHelper);
}