/**
 * This file is responsible for managing windows.
 */


// constants


/**
 * The y offset for the position of a window that is placed relative to another window.
 */
WindowManager.Y_OFFSET = 20;


/**
 * The x offset for the position of a window that is placed relative to another window.
 */
WindowManager.X_OFFSET = 20;


/**
 * Modal windows are always ontop of all the other windows. However the user still can 
 * interact with the other windows. This is the  z-index to use for modal windows so they
 *  are above all other windows. Ideally this should be set to Number.MAX_VALUE however 
 * firefox does not accept such high z-index values. Just set a very high zIndex.
 */
WindowManager.MODAL_WINDOW_ZINDEX = 999999;


// construction


/**
 * The instance of this class.
 */
WindowManager.instanceObject = new WindowManager();


/**
 * Constructs the a instance of this class.
 */
function WindowManager() {
	WindowManager.windows = new Array();
	this.eventProcessor = new Listener(this);
	this.eventProcessor.addEvent(STATUSBAR_BUTTON_MINIMIZED_EVENT, processButtonMinimizedWindowManager);
	this.eventProcessor.addEvent(STATUSBAR_BUTTON_ACTIVATED_EVENT, processButtonActivatedWindowManager);
	this.eventProcessor.addEvent(STATUSBAR_BUTTON_REMOVED_EVENT, processButtonRemovedWindowManager);
	
	/**
	 * Maps the windows is to its button in the statusbar. 
	 */
	this.windowStatusbarButtons = new Array();
	
	
	/**
	 * The current active window.
	 */
	this.activeWindow;
	
	
	/**
	 * The previous active window.
	 */
	this.previousActiveWindow;
	
	
	/**
	 * The current active z index. The window with the highest z index is at on top of the other windows. 
	 * When windows have the same z-index the window that is defined last in the html code is shown ontop.
	 * To avoid this we increase the z-index each time a window is activated. 
	 */
	this.activeZIndex = 0;
	
	
	/**
	 * The status bar in use. If this is set the windows are not allowed to be dragged over the statusbar. 
	 * There are also added buttons for the windows in the status bar. 
	 */
	this.statusbar;
	
	
	/**
	 * The desktop in use. New windows are appended to the desktop.
	 */
	this.desktop;
}


/**
 * Returns the instance of this class. 
 */
WindowManager.instance = function() {
	return this.instanceObject;
}


// services


/**
 * Activates a window.
 * @param id The id of the window to activate. 
 * @throws Error if id is undefined, null or empty.
 */
WindowManager.prototype.activateWindow = function(id) {
	assertNotEmpty(id, "The id of the window is undefined null or empty");
	var window = this.getWindow(id);
	if(!exists(window) || (exists(this.activeWindow) && window.getId() == this.activeWindow.getId())) {
		// the window is already active, or does not exist
		return;
	}
	if (!isNull(this.activeWindow)) {
		this.previousActiveWindow = this.activeWindow;
	}
	this.deactivateActiveWindow();
	window.activateWindow()
	if (!window.isModal()) {
		window.setZIndex(this.getZIndex());
	}
	this.activeWindow = window;
	
	if (statusbarAvailable()) {
		this.statusbar.activateButton(this.createStatusbarButton(this.getWindow(id)));
	}
	
}


/**
 * Deactivates a window.
 * @param id The id of the window to deactivate. 
 * @throws Error if is is undefined, null or empty.
 */
WindowManager.prototype.deactivateWindow = function(id) {
	assertNotEmpty(id, "The id of the window is undefined null or empty");
	var window = this.getWindow(id);
	if (!exists(window) || window.isDeactivated()) {
		// the window is already deactivated
		return;
	}
	if (this.getActiveWindow() == window) {
		this.activeWindow = null;
	}
	window.deactivateWindow();
	
	if (statusbarAvailable()) {
		this.statusbar.deactivateButton(this.createStatusbarButton(this.getWindow(id)));
	}
	
}


/**
 * Deactivates the active window.
 */
WindowManager.prototype.deactivateActiveWindow = function() {
	if (exists(this.activeWindow)) {
		this.deactivateWindow(this.activeWindow.getId());
	 }
}


/**
 * Hides a window.
 * @param id The id of the window to hide.
 * @throws Error if id is undefined, null or empty.
 */
WindowManager.prototype.hideWindow = function(id) {
	assertNotEmpty(id, "The id of the window is undefined null or empty");
	if (!this.getWindow(id).isHidden()) {
		this.deactivateWindow(id);
		this.getWindow(id).hide();
	}
	if (exists(this.activeWindow) && id == this.activeWindow.getId()) {
		this.activeWindow = null;
	}
	this.removeStatusbarButton(id);
}


/**
 * Shows a window. 
 * @param id The id of the window to show.
 * @throws Error if id is undefined, null or empty.
 */
WindowManager.prototype.showWindow = function(id) {
	assertNotEmpty(id, "The id of the window is undefined null or empty");
	var window = this.getWindow(id);
	window.show();
	if (window.position) {
		this.positionWindow(window, this.previousActiveWindow);
		window.position = false;
	}
	if (statusbarAvailable()) {
		this.createStatusbarButton(this.getWindow(id));
	}
}


/**
 * Minimizes a window to the taskbar. 
 * @param id The id of the window to minimize.
 * @throws Error if id is undefined, null or empty.
 */
WindowManager.prototype.minimizeWindow = function(id) {
	assertNotEmpty(id, "The id of the window is undefined null or empty");
	var window = this.getWindow(id);
	if (window.isMinimized()) {
		return;
	}
	window.minimize();
	if (this.getActiveWindow() == window) {
		this.activeWindow = null;
	}
	this.statusbar.minimizeButton(this.createStatusbarButton(window));
}


/**
 * Maximizes a window. 
 * @param id The id of the window to maximize. 
 * @throws Error if id is undefined, null or empty.
 */
WindowManager.prototype.maximizeWindow = function(id) {
	assertNotEmpty(id, "The id of the window is undefined null or empty");
	this.getWindow(id).maximize();
}


/**
 * Get the current active window. 
 * @return The current active window, null if no window is active. 
 */
WindowManager.prototype.getActiveWindow = function() {
	if (!exists(this.activeWindow)) {
		return null;
	}
	return this.activeWindow;
}


/**
 * Set the active window.
 * @param window The window to set as the active.
 * @throws Error if window is undefined or null.
 */
WindowManager.prototype.setActiveWindow = function(window) {
	assertExists(window, "The window to set as the active window is undefined or null");
	this.activeWindow = window;
}


/**
 * Gets a window.
 * @param id The id of the window to get.
 * @return The window, null if a windwo with this name not exists.
 * @throws Error if the id is null, not defined or empty.
 */
WindowManager.prototype.getWindow = function(id) {
	assertNotEmpty(id, "Id is null, undefined or empty");

	for (i = 0; i < WindowManager.windows.length; i++) {
		if (id == WindowManager.windows[i].id) {
			return WindowManager.windows[i];
		}
	}
	return null;
}


/**
 * Register a new window.
 * @param window The window object to register.
 * @throws Error if the window is not defined or null.
 */
WindowManager.prototype.registerNewWindow = function(window) {
	assertDefined(window, "Window is null or not defined");
	
	WindowManager.windows[WindowManager.windows.length] = window;
	if (exists(window.isShown) && window.isShown()) {
		this.createStatusbarButton(window);
	}
}


/**
 * Returns true if a windwos exists.
 * @param id The id of the window to check.
 * @return True if a window with id exists, false otherwise.
 * @throws Error if id is undefined, null or empty.
 */
WindowManager.prototype.existsWindow = function(id) {
	assertNotEmpty(id, "Id is undefined, null or empty");
	
	for (var i = 0; i < WindowManager.windows.length; i++) {
		if (WindowManager.windows[i].getId() == id) {
			return true;
		}
	}	
	return false;
}


/**
 * Removes a window.
 * @param window The window object to remove.
 */
WindowManager.prototype.removeWindow = function(window) {
	if (!isDefined(WindowManager.windows)) {
		return;
	}
	
	var len = WindowManager.windows.length;
	for (var i = 0; i < len; i++) {
		if (window == WindowManager.windows[i]) {
			WindowManager.windows[i] = null;
			WindowManager.windows = removeNullEntries(WindowManager.windows);
		}
	}
	this.removeStatusbarButton(window.getId());
}


/**
 * Returns the active window, undefined if no active window is defined.
 */
WindowManager.prototype.getActiveWindow = function() {
	return this.activeWindow;
}


/**
 * Hides all the windows. 
 */
WindowManager.prototype.hideAllWindows = function() {
	var len = WindowManager.windows.length;
	for (var i = 0; i < len; i++) {
		if (WindowManager.windows[i].isShown()) {
			this.hideWindow(WindowManager.windows[i].getId());
		}
	}
}


/**
 * Returns a array containing all the existing windows.
 */
WindowManager.prototype.getWindows = function() {
	return WindowManager.windows;
}


/**
 * Gets a new z index to use for a window. The z index is increased by one for each time this is called. 
 * This is to avoid the problem with windows having the same z index. When windows have the same z index
 * the one defined last in the html code is shown ontop. 
 * @return The z index to set for the active window.
 */
WindowManager.prototype.getZIndex = function() {
	this.activeZIndex++;
	return this.activeZIndex;
}


/**
 * Sets the status bar used. This is used to not drag windows under the taskbar. There
 * are also added buttons in the status bar for the windows. 
 * @param statusbar The status bar used.
 * @throws Error if statusbar is undefined or null.  
 */
WindowManager.prototype.setStatusbar = function(statusbar) {
	assertExists(statusbar, "The statusbar is undefined or null");
	this.statusbar = statusbar;
}

 
/**
 * Sets the desktop used.
 * @param desktop The desktop instance to use. New window are appended to the desktop.
 * @throws Error if dekstop is null or undefined.
 */
WindowManager.prototype.setDesktop = function(desktop) {
	assertExists(desktop, "The desktop to set is undefined or null");
	this.desktop = desktop;	
}


/**
 * Gets the desktop set. 
 * @return The desktop object set, undefined if not set. 
 */
WindowManager.prototype.getDesktop = function() {
	return this.desktop;
}


/**
 * Gets the html code of the desktop.
 * @return The html code of the desktop, undefined if no desktop set. 
 */
WindowManager.prototype.getDesktopHtmlCode = function() {
	if (exists(this.desktop)) {
		return this.desktop.getDesktopCode();
	}
	return undefined;
}


/**
 * Positions a window relative to a given window. 
 * @param window The window to position.
 * @param referenceWindow Window is position with this window as a reference.
 * @throws Error if window is undefined null or empty. 
 */
WindowManager.prototype.positionWindow = function(window, referenceWindow) {
	assertExists(window, "The window to position is undefined or null");
	if (!exists(referenceWindow) || window == referenceWindow) {
		return;
	}
	
	var yPos = parseInt(referenceWindow.htmlCode.style.top) + WindowManager.Y_OFFSET;
	yPos = this.isYpositionInScreen(yPos + window.getHeight()) ? yPos : this.calculateMaxYPos() - window.getHeight();
	var xPos = parseInt(referenceWindow.htmlCode.style.left) + WindowManager.X_OFFSET;
	xPos = this.isXPositionInScreen(xPos + window.getWidth()) ? xPos : this.calculateMaxXPos() - window.getWidth();
	window.setPosition(yPos, xPos);
}


/**
 * Sets one window to be modal. A modal window will always be ontop of all the other
 * windows, however the user still can interact with other windows. 
 * @param window The window to set as modal.
 * @throws Error if window is undefined or null. 
 */
WindowManager.prototype.setModalWindow = function(window) {
	assertExists(window, "The window to set as modal does not exist");
	window.setModal(true);
	window.setZIndex(WindowManager.MODAL_WINDOW_ZINDEX);
}


// event processing


/** 
 * Processes a status bar button minimized event. When a status bar button is minimized the according window is hidden.  
 * @param event The event to process. 
 */
function processButtonMinimizedWindowManager(event) {
	if (!exists(this.getWindowStatusbarButton(event.getData()))) {
		return;
	}
	this.minimizeWindow(this.getWindowStatusbarButton(event.getData()).getId());
}


/**
 * Processes a status bar button activated event. When a status bar button is activated the according window is activated. 
 * @param event The event to process. 
 */
function processButtonActivatedWindowManager(event) {
	if (!exists(this.getWindowStatusbarButton(event.getData()))) {
		return;
	}
	this.activateWindow(this.getWindowStatusbarButton(event.getData()).getId());
}


/**
 * Processes a status bar button removed event. When a status bar button is removed the according window is hidden.
 * @param event The event to process. 
 */
function processButtonRemovedWindowManager(event) {
	var window = this.getWindowStatusbarButton(event.getData());
	this.windowStatusbarButtons[event.getData()] = undefined;
	if (!exists(window)) {
		return;
	}
	// simulate a click on the close button of the window
	var closeButton = getElementWithAttribute(window.getHtmlCode(), "closeButton", "id", "div");
	if (exists(closeButton)) {
		closeButton.onclick();
	}
}


/**
 * Processes a window closed event. When a window is closed its statusbar button is removed.
 */
function processWindowClosedEventWindowManager(event) {
	if (!statusbarAvailable()) {
		return;
	}
	var button = this.getStatusbarButton(event.getData().id);
	if (exists(button)) {
		this.statusbar.removeButton(button);
	}
	this.windowStatusbarButtons[event.getData().id] = undefined;
}


/**
 * A window button has been clicked in the statusbar, activate the window.
 * @param button The button clicked.  
 */
function windowStatusbarButtonClicked(button) {
	var window = this.getWindowStatusbarButton(button);
	if (exists(window)) {
		this.activateWindow(window.getId());
	}
}


// implementation


/**
 * Removes a status bar button for a window. 
 * @param id The id of the window to remove the button for. 
 */
WindowManager.prototype.removeStatusbarButton = function(id) {
	if (!statusbarAvailable()) {
		return;
	}
	var button = this.getStatusbarButton(id);
	if (exists(button)) {
		this.statusbar.removeButton(button);
	}
	this.windowStatusbarButtons[id] = undefined;
}


/**
 * Creates a statusbar button for a given window if it does not already exists.
 * @param window The window to create the button for. 
 * @return The button created. 
 */
WindowManager.prototype.createStatusbarButton = function(window)  {
	var button = this.getStatusbarButton(window.getId());
	if (!exists(button)) {
		button = new StatusbarButton(window.getWindowTitle(), null, null);
		this.statusbar.addButton(button);
		this.windowStatusbarButtons[window.getId()] = button.getId();
	}
	return button;
}


/**
 * Checks if the statusbar is available.
 * @return true if a statusbar is available, false otherwise.
 */
function statusbarAvailable() {
	if (exists(WindowManager.instance().statusbar)) {
		return true;
	}
}	


/**
 * Gets the windows statusbar button.
 * @param id The id of the window to get the button from. 
 * @return The windows statusbar button, null if the window does not have a button yet.
 * @thrwos Error if window is null or undefined.  
 */
WindowManager.prototype.getStatusbarButton = function(id) {
	for (var windowId in this.windowStatusbarButtons) {
		if (id == windowId && exists(this.windowStatusbarButtons[windowId])) {
			return this.statusbar.getButton(this.windowStatusbarButtons[windowId]);
		} 
	}
	return null;
}


/**
 * Gets the window a given statusbar button is for. 
 * @param button The button to get the window for. 
 * @return The window object, null if no window exists for the button.
 */
WindowManager.prototype.getWindowStatusbarButton = function(button) {
	for (var windowId in this.windowStatusbarButtons) {
		if (button.getId() == this.windowStatusbarButtons[windowId]) {
			return WindowManager.instance().getWindow(windowId);
		}
	}
	return null;
}


/**
 * Gets the element to append new windows to, if a desktop is defined the windows are appended to
 * the desktop, otherwise they are appended to the document. 
 */
WindowManager.prototype.getWindowRootElement = function() {
	if (exists(this.desktop)) {
		return this.desktop.getDesktopCode();
	}
	return document.getElementsByTagName("body").item(0);
}


/**
 * Checks if a given yPosition is in the screen.
 * @param yPos The y position to check.
 * @return True if yPos is in the screen, false otherwise.
 */
WindowManager.prototype.isYpositionInScreen = function(yPos) {
	var screenHeight = exists(this.desktop) ? this.desktop.getDesktopCode().offsetHeight : getWindowHeight(window);
	var scrollHeight = exists(this.desktop) ? this.desktop.getDesktopCode().scrollHeight : window.document.body.scrollHeight;
	if (scrollHeight > screenHeight) {
		return scrollHeight > yPos;
	}
	else {
		return screenHeight > yPos;
	}
}


/**
 * Checks if a given xPosition is in the screen.
 * @param xPos The x position to check.
 * @return True if xPos is in the screen, false otherwise. 
 */
WindowManager.prototype.isXPositionInScreen = function(xPos) {
	var screenWidth = exists(this.desktop) ? this.desktop.getDesktopCode().offsetWidth : getWindowWidth(window);
	var scrollWidth = exists(this.desktop) ? this.desktop.getDesktopCode().scrollWidth : window.document.body.scrollWidth;
	if (scrollWidth > screenWidth) {
		return scrollWidth > xPos;
	}
	else {
		return screenWidth > xPos;
	}
}


/**
 * Calculates the max y position that can be set for a window so that the window is not placed outside
 * the current scrolling. 
 * @return The max y value that can be set. 
 */
WindowManager.prototype.calculateMaxYPos = function() {
	var screenHeight = exists(this.desktop) ? this.desktop.getDesktopCode().offsetHeight : getWindowHeight(window);
	var scrollHeight = exists(this.desktop) ? this.desktop.getDesktopCode().scrollHeight : window.document.body.scrollHeight;
	if (scrollHeight > screenHeight) {
		return scrollHeight; 
	}
	return screenHeight;
}


/**
 * Calculates the max x position that can be set for a window so that the window is not placed outside
 * the current scrolling.
 * @return The max x value that can be set. 
 */
WindowManager.prototype.calculateMaxXPos = function() {
	var screenWidth = exists(this.desktop) ? this.desktop.getDesktopCode().offsetWidth : getWindowWidth(window);
	var scrollWidth = exists(this.desktop) ? this.desktop.getDesktopCode().scrollWidth : window.document.body.scrollWidth;
	if (scrollWidth > screenWidth) {
		return scrollWidth;
	}
	return screenWidth;
}


// html


/**
 * Parses the html code and each element with class set to a given value is transformed into a window.
 * The created windows are all deactivated, hidden and have no position set.
 * @param elementType The type of html elements with class windowClass to transform into windows.
 * @param windowClass The class of the elements to transform into windows.
 * @throws Error if elementType is undefined, null or empty.
 * @throws Error if className is undefined, null or empty.
 */
function createWindows(elementType, windowClass) {
	assertNotNull(elementType, "The element type to get is undefined, null or empty: " + elementType);
	assertNotNull(windowClass, "The windowClass is unefined, null or empty: " + windowClass);
	
	var elements = getElementsWithClass(elementType, windowClass);
	var len = elements.length;
	for (var i = 0; i < len; i++) {
		var newWindow = createWindow(elements[i], elements[i].getAttribute("id"), elements[i].getAttribute("name"));
		WindowManager.instance().hideWindow(newWindow.getId());
	}
}


/**
 * Creates a new window. The created window is not made active.
 * @param content The content to create a window for.
 * @param id The id to set for the window.
 * @param title The title to set for the window.
 * @return The created window object.
 */
function createWindow(content, id, title) {
	var windowCode = createWindowCode(content, id, title);
	var newWindow = new Window(id);
	newWindow.makeDragable();
	newWindow.setResizable(true);
	
	// set the onmousedown to activate the window when it is clicked
	var windowId = id;
	newWindow.htmlCode.onmousedown = function(event) {
		WindowManager.instance().activateWindow(windowId);
	}
	
	return newWindow;
}


/**
 * Creates html code for a window. The size fo the window is set according to the sie of the content.
 * The position of the window is set to default values. A call to makeDraggable() is needed to make 
 * the window draggable. A call to makeResizable() is needed to make the window resizable.
 * @param content The content to create a window for.
 * @param id The id to set for the window.
 * @param title The title to set for the window.
 */
function createWindowCode(content, id, title) {
	var resizeTable = document.createElement("table");
	resizeTable.style.position = "absolute";
	resizeTable.style.left = Window.DEFAULT_X_POSITION;
	resizeTable.style.top = Window.DEFAULT_Y_POSITION;
	resizeTable.style.zIndex = WindowManager.instance().getZIndex();
	resizeTable.setAttribute("id", id);
	resizeTable.border = 0;
	resizeTable.cellPadding = 0;
	resizeTable.cellSpacing = 0;
	setClass(resizeTable, "inactiveBorderColor");
	var resizeBody = document.createElement("TBODY");
	resizeTable.appendChild(resizeBody);
	
	resizeBody.appendChild(createTopResizingRow());
	resizeBody.appendChild(createMiddleRow(content, id, title));
	resizeBody.appendChild(createBottomResizingRow());
	
	WindowManager.instance().getWindowRootElement().appendChild(resizeTable);
	
	// set the size of the window
	if (isOpera()) {
		// In Opera the border of the windows is drawn on the outside of
		// the window, while in FF and IE the border is drawn on the inside of the table. 
		resizeTable.style.height = (resizeTable.offsetHeight - (Window.BORDER_WIDTH * 2)) + "px";
		resizeTable.style.width = (resizeTable.offsetWidth - (Window.BORDER_WIDTH * 2)) + "px";
	}
	else {
		resizeTable.style.height = resizeTable.offsetHeight + "px";
		resizeTable.style.width = resizeTable.offsetWidth + "px";
	}
		
	return resizeTable;
}


/**
 * Create the middle row of the window. 
 * @param id The id of the window.
 * @param title The title to set for the window.
 * @return The middle row created.
 */
function createMiddleRow(content, id, title) {
	var middleRow = document.createElement("tr");
	middleRow.appendChild(createLeftResizing());
	
	var windowCell = document.createElement("td");
	setClass(windowCell, "windowBorder");
	windowCell.appendChild(createWindowTable(content, id, title));
	middleRow.appendChild(windowCell);
	middleRow.appendChild(createRightResizingCell());
	
	return middleRow;
}

 
/**
 * Creates the table representing a window.
 * @param content The content to show in the window.
 * @param id The id of the window.
 * @param title The title to set for the window.
 * @return The created window.
 */
function createWindowTable(content, id, title) {
	var windowTable = document.createElement("table");
	windowTable.border = 0;
	windowTable.cellPadding = 0;
	windowTable.cellSpacing = 0;
	windowTable.setAttribute("class", "window");
	windowTable.setAttribute("className", "window");
	windowTable.setAttribute("height" , "100%");
	windowTable.setAttribute("width" , "100%");
	var windowBody = document.createElement("TBODY");
	windowTable.appendChild(windowBody);
	
	// create the window header
	windowBody.appendChild(createWindowHeader(id, title));
	
	// create the content 
	var windowContent = document.createElement("TR");
	windowContent.setAttribute("id", "windowContent");
	windowBody.appendChild(windowContent);
	var windowContentCell = document.createElement("TD");
	windowContentCell.setAttribute("align", "left");
	windowContentCell.colSpan = "2";
	windowContentCell.setAttribute("id", "windowContentCell");
	windowContentCell.appendChild(content);
	windowContent.appendChild(windowContentCell);
	return windowTable;
}
 
 
/**
 * Creates the top resizing row of a window.
 * @return The html code for the top resizing row.
 */
function createTopResizingRow() {
	var topResizeRow = document.createElement("tr");
	topResizeRow.setAttribute("height", Window.RESIZE_AREA_SIZE);
	setClass(topResizeRow, "windowTopBorder");
	
	var topLeftCell = document.createElement("td");
	var topMiddleCell = document.createElement("td");
	var topRightCell = document.createElement("td");
	
	topLeftCell.setAttribute("id", "topleftresizing");
	topLeftCell.setAttribute("width", Window.RESIZE_AREA_SIZE);
	topMiddleCell.setAttribute("id", "topresizing");
	topRightCell.setAttribute("id", "toprightresizing");
	topRightCell.setAttribute("width", Window.RESIZE_AREA_SIZE);
	
	topResizeRow.appendChild(topLeftCell);
	topResizeRow.appendChild(topMiddleCell);
	topResizeRow.appendChild(topRightCell);
	
	// In Safari the resize area needs to be filled
	// with something for example a div tag, or else safari reports the hight
	// of the resize cells to be 0. 
	topLeftCell.appendChild(createFillDiv());
	topMiddleCell.appendChild(createFillDiv());
	topRightCell.appendChild(createFillDiv());
	
	return topResizeRow;
}


/**
 * Creates a fill div tag. In Safari the resize area needs to be filled
 * with something for example a div tag, or else safari reports the hight
 * of the resize cells to be 0. The created div tag will fill the whole
 * element it is appended to.
 * @return The create div tag. 
 */
function createFillDiv() {
	var div = document.createElement("div");
	div.style.height = "100%";
	div.style.width = "100%";
	// IE has a bug with the size of div tags, in IE the div size depends
	// among others of the font size
	div.style.fontSize = "1px";
	return div;
}


/**
 * Creates the bottom resizing row of a window.
 * @return The created html code for the bottom resizing area.
 */
function createBottomResizingRow() {
	var bottomResizeRow = document.createElement("TR");
	setClass(bottomResizeRow, "windowBorder");
	bottomResizeRow.setAttribute("height", Window.RESIZE_AREA_SIZE);
	
	var leftBottomResizing = document.createElement("TD");
	leftBottomResizing.setAttribute("height", Window.RESIZE_AREA_SIZE);
	leftBottomResizing.setAttribute("id", "bottomleftresizing");
	leftBottomResizing.setAttribute("width", Window.RESIZE_AREA_SIZE);
	bottomResizeRow.appendChild(leftBottomResizing);
	
	var bottomResizing = document.createElement("TD");
	bottomResizing.setAttribute("height", Window.RESIZE_AREA_SIZE);
	bottomResizing.setAttribute("id", "bottomresizing");
	bottomResizeRow.appendChild(bottomResizing);
	
	var rightBottomResizing = document.createElement("TD");
	rightBottomResizing.setAttribute("height", Window.RESIZE_AREA_SIZE);
	rightBottomResizing.setAttribute("id", "bottomrightresizing");
	rightBottomResizing.setAttribute("width", Window.RESIZE_AREA_SIZE);
	bottomResizeRow.appendChild(rightBottomResizing);
	
	// In Safari the resize area needs to be filled
	// with something for example a div tag, or else safari reports the hight
	// of the resize cells to be 0. 
	leftBottomResizing.appendChild(createFillDiv());
	bottomResizing.appendChild(createFillDiv());
	rightBottomResizing.appendChild(createFillDiv());
	
	return bottomResizeRow;
}


/**
 * Create the left resizing cell.
 * @return The left resizing cell.
 */
function createLeftResizing() {
	var leftResizing = document.createElement("TD");
	setClass(leftResizing, "windowBorderBg");
	leftResizing.setAttribute("id", "leftresizing");
	leftResizing.setAttribute("width", Window.RESIZE_AREA_SIZE);
	return leftResizing;
}


/**
 * Create the right resizing cell.
 * @return The right resizing cell created.
 */
function createRightResizingCell() {
	var rightResizing = document.createElement("TD");
	setClass(rightResizing, "windowBorderBg");
	rightResizing.setAttribute("id", "rightresizing");
	rightResizing.setAttribute("width",Window.RESIZE_AREA_SIZE);
	return rightResizing;
}


/**
 * Create the window header.
 * @param id The id of the window.
 * @param title The title to set for the window.
 * @return The created html code for the window header.
 */ 
function createWindowHeader(id, title) {
	var header = document.createElement("tr");
	header.setAttribute("id", "windowHeader");
	header.setAttribute("className", "inactiveWindowHeader");
	header.setAttribute("class", "inactiveWindowHeader");
	header.setAttribute("height", "25px");
	var windowTitle = document.createElement("TD");
	windowTitle.setAttribute("name", "windowTitle");
	windowTitle.setAttribute("class", "windowTitle");
	windowTitle.setAttribute("className", "windowTitle");
	windowTitle.setAttribute("width", "75%");
	var textNode = document.createTextNode(title);
	var nobrTitle = document.createElement("nobr");
	nobrTitle.appendChild(textNode);
	windowTitle.appendChild(nobrTitle);
	header.appendChild(windowTitle);
	
	var headerButtons = document.createElement("TD");
	headerButtons.setAttribute("name", "windowButtons");
	headerButtons.setAttribute("class", "windowHeaderButtons");
	headerButtons.setAttribute("className", "windowHeaderButtons");
	headerButtons.setAttribute("width", "24%");
	headerButtons.setAttribute("align", "right");
	
	// buttons
	headerButtons.appendChild(createCloseButton(id));
	headerButtons.appendChild(createExpandButton(id));
	headerButtons.appendChild(createColapseButton(id));
	headerButtons.appendChild(createMinimizeButton(id));
	
	header.appendChild(headerButtons);
	return header;
}


/**
 * Creates the minimizeButton.
 * @param id The id of the window.
 * @return The minimize button.
 */
function createMinimizeButton(id) {
	var minimizeButton = document.createElement("div");
	minimizeButton.onclick = function() {
		WindowManager.instance().minimizeWindow(id);	
	}
	minimizeButton.onmouseover = function() {
		setClass(minimizeButton, "minimizeButtonMouseover WindowButton");
	}
	minimizeButton.onmouseout = function() {
		setClass(minimizeButton, "minimizeButton WindowButton");	
	}
	minimizeButton.setAttribute("id", "minimizeContentButton");
	minimizeButton.setAttribute("title", "Minimize");
	setClass(minimizeButton, "minimizeButton WindowButton");
	return minimizeButton;
}


/**
 * Create the close button.
 * @param id The id of the window.
 * @return The close button.
 */
function createCloseButton(id) {
	var closeButton = document.createElement("div");
	closeButton.onclick = function() {
		var windowToHide = WindowManager.instance().getWindow(id);
		WindowManager.instance().hideWindow(windowToHide.getId());
	}
	closeButton.setAttribute("id", "closeButton");
	closeButton.setAttribute("title", "Close");
	setClass(closeButton, "closeButton WindowButton");
	closeButton.onmouseover = function() {
		setClass(closeButton, "closeButtonMouseover WindowButton");
	}
	closeButton.onmouseout = function() {
		setClass(closeButton, "closeButton WindowButton");	
	}
	return closeButton;
}


/**
 * Create the expand button.
 * @param id The id of the window.
 * @return The expand button.
 */
function createExpandButton(id) {
	var expandButton = document.createElement("div");
	expandButton.onclick = function() {
		var windowToExpand = WindowManager.instance().getWindow(id);
		windowToExpand.expandContent();
	}
	expandButton.setAttribute("id", "expandContentButton");
	expandButton.setAttribute("title", "Expand");
	setClass(expandButton, "expandButton WindowButton inactive");
	expandButton.onmouseover = function() {
		setClass(expandButton, "expandButtonMouseover WindowButton");
	}
	expandButton.onmouseout = function() {
		var windowToExpand = WindowManager.instance().getWindow(id);			
		setClass(expandButton, "expandButton WindowButton");
		if (windowToExpand.colapsedContent == false) {
			setClass(expandButton, "expandButton WindowButton inactive");
		}
		else {
			setClass(expandButton, "expandButton WindowButton");
		}
	}
	return expandButton;
}


/**
 * Create the colapse button.
 * @param id The id of the window.
 * @return The colapse button.
 */
function createColapseButton(id) {
	var colapseButton = document.createElement("div");
	colapseButton.onclick = function() {
		var windowToColapse = WindowManager.instance().getWindow(id);
		windowToColapse.colapseContent();
	}
	colapseButton.setAttribute("id", "colapseContentButton");
	colapseButton.setAttribute("collapse", "Expand");
	setClass(colapseButton, "colapseButton WindowButton");
	colapseButton.onmouseover = function() {
		setClass(colapseButton, "colapseButtonMouseover WindowButton");
	}
	colapseButton.onmouseout = function() {
		var windowToColapse = WindowManager.instance().getWindow(id);
		setClass(colapseButton, "colapseButton WindowButton");
		if (windowToColapse.colapsedContent == true) {
			setClass(colapseButton, "colapseButton WindowButton inactive");
		}
		else {
			setClass(colapseButton, "colapseButton WindowButton");
		}
	}
	return colapseButton;
}